@@ -40,11 +40,6 @@ BASE_FEATURE(kRemoveMobileViewportDoubleTap,
              "RemoveMobileViewportDoubleTap",
              base::FEATURE_ENABLED_BY_DEFAULT);
 
-// Design doc: bit.ly/scrollunification
-BASE_FEATURE(kScrollUnification,
-             "ScrollUnification",
-             base::FEATURE_ENABLED_BY_DEFAULT);
-
 BASE_FEATURE(kScrollSnapCoveringAvoidNestedSnapAreas,
              "ScrollSnapCoveringAvoidNestedSnapAreas",
              base::FEATURE_ENABLED_BY_DEFAULT);
@@ -57,17 +52,25 @@ BASE_FEATURE(kScrollSnapPreferCloserCovering,
              "ScrollSnapPreferCloserCovering",
              base::FEATURE_ENABLED_BY_DEFAULT);
 
+BASE_FEATURE(kMainRepaintScrollPrefersNewContent,
+             "MainRepaintScrollPrefersNewContent",
+             base::FEATURE_ENABLED_BY_DEFAULT);
+
 BASE_FEATURE(kHudDisplayForPerformanceMetrics,
              "HudDisplayForPerformanceMetrics",
              base::FEATURE_DISABLED_BY_DEFAULT);
 
+BASE_FEATURE(kRenderSurfaceCommonAncestorClip,
+             "RenderSurfaceCommonAncestorClip",
+             base::FEATURE_ENABLED_BY_DEFAULT);
+
 BASE_FEATURE(kDurationEstimatesInCompositorTimingHistory,
              "DurationEstimatesInCompositorTimingHistory",
              base::FEATURE_DISABLED_BY_DEFAULT);
 
 BASE_FEATURE(kNonBlockingCommit,
              "NonBlockingCommit",
-             base::FEATURE_DISABLED_BY_DEFAULT);
+             base::FEATURE_ENABLED_BY_DEFAULT);
 
 BASE_FEATURE(kNoPreserveLastMutation,
              "NoPreserveLastMutation",
@@ -97,7 +100,7 @@ BASE_FEATURE(kUseDMSAAForTiles,
 // above flag UseDMSAAForTiles controls the launch on Vulkan backend.
 BASE_FEATURE(kUseDMSAAForTilesAndroidGL,
              "UseDMSAAForTilesAndroidGL",
-             base::FEATURE_DISABLED_BY_DEFAULT);
+             base::FEATURE_ENABLED_BY_DEFAULT);
 #endif
 
 BASE_FEATURE(kUpdateBrowserControlsWithoutProxy,
@@ -136,16 +139,24 @@ BASE_FEATURE(kReclaimPrepaintTilesWhenIdle,
              "ReclaimPrepaintTilesWhenIdle",
              base::FEATURE_DISABLED_BY_DEFAULT);
 
+// This saves memory on all platforms, but while on Android savings are
+// significant (~10MiB or more of foreground memory), on desktop they were
+// small, so only enable on Android.
 BASE_FEATURE(kSmallerInterestArea,
              "SmallerInterestArea",
-             base::FEATURE_DISABLED_BY_DEFAULT);
+#if BUILDFLAG(IS_ANDROID)
+             base::FEATURE_ENABLED_BY_DEFAULT
+#else
+             base::FEATURE_DISABLED_BY_DEFAULT
+#endif
+);
 
 const base::FeatureParam<int> kInterestAreaSizeInPixels{
     &kSmallerInterestArea, "size_in_pixels", kDefaultInterestAreaSizeInPixels};
 
 BASE_FEATURE(kImageCacheNoCache,
              "ImageCacheNoCache",
-             base::FEATURE_DISABLED_BY_DEFAULT);
+             base::FEATURE_ENABLED_BY_DEFAULT);
 
 BASE_FEATURE(kReclaimOldPrepaintTiles,
              "ReclaimOldPrepaintTiles",
@@ -158,4 +169,16 @@ BASE_FEATURE(kUseMapRectForPixelMovement,
              "UseMapRectForPixelMovement",
              base::FEATURE_DISABLED_BY_DEFAULT);
 
+BASE_FEATURE(kEvictionThrottlesDraw,
+             "EvictionThrottlesDraw",
+             base::FEATURE_DISABLED_BY_DEFAULT);
+
+BASE_FEATURE(kResetTimerWhenNoActiveTreeLikely,
+             "ResetTimerWhenNoActiveTreeLikely",
+             base::FEATURE_ENABLED_BY_DEFAULT);
+
+BASE_FEATURE(kAdjustFastMainThreadThreshold,
+             "AdjustFastMainThreadThreshold",
+             base::FEATURE_DISABLED_BY_DEFAULT);
+
 }  // namespace features
