@@ -8,6 +8,7 @@
 #include <utility>
 
 #include "base/command_line.h"
+#include "base/feature_list.h"
 #include "base/functional/bind.h"
 #include "base/functional/callback_helpers.h"
 #include "base/memory/read_only_shared_memory_region.h"
@@ -16,39 +17,25 @@
 #include "base/run_loop.h"
 #include "base/test/test_switches.h"
 #include "build/build_config.h"
-#include "cc/raster/raster_buffer_provider.h"
 #include "cc/test/fake_output_surface_client.h"
 #include "cc/test/pixel_test_output_surface.h"
 #include "cc/test/pixel_test_utils.h"
 #include "components/viz/client/client_resource_provider.h"
-#include "components/viz/common/features.h"
-#include "components/viz/common/frame_sinks/begin_frame_source.h"
 #include "components/viz/common/frame_sinks/copy_output_request.h"
 #include "components/viz/common/frame_sinks/copy_output_result.h"
-#include "components/viz/common/quads/compositor_frame_metadata.h"
 #include "components/viz/common/resources/bitmap_allocation.h"
 #include "components/viz/common/resources/shared_bitmap.h"
 #include "components/viz/service/display/display_resource_provider_skia.h"
 #include "components/viz/service/display/display_resource_provider_software.h"
-#include "components/viz/service/display/output_surface_client.h"
 #include "components/viz/service/display/software_output_device.h"
 #include "components/viz/service/display/software_renderer.h"
 #include "components/viz/service/display_embedder/skia_output_surface_dependency_impl.h"
 #include "components/viz/service/display_embedder/skia_output_surface_impl.h"
-#include "components/viz/service/gl/gpu_service_impl.h"
 #include "components/viz/test/paths.h"
 #include "components/viz/test/test_in_process_context_provider.h"
 #include "components/viz/test/test_shared_bitmap_manager.h"
-#include "gpu/command_buffer/client/gles2_interface.h"
-#include "gpu/command_buffer/client/shared_memory_limits.h"
 #include "gpu/command_buffer/service/gpu_switches.h"
-#include "gpu/command_buffer/service/service_utils.h"
-#include "gpu/config/gpu_feature_type.h"
 #include "gpu/config/gpu_finch_features.h"
-#include "gpu/config/gpu_info.h"
-#include "gpu/ipc/gpu_in_process_thread_service.h"
-#include "gpu/ipc/service/gpu_memory_buffer_factory.h"
-#include "services/viz/privileged/mojom/gl/gpu_host.mojom.h"
 #include "skia/buildflags.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
@@ -62,7 +49,8 @@ namespace cc {
 PixelTest::PixelTest(GraphicsBackend backend)
     : device_viewport_size_(gfx::Size(200, 200)),
       disable_picture_quad_image_filtering_(false),
-      output_surface_client_(std::make_unique<FakeOutputSurfaceClient>()) {
+      output_surface_client_(std::make_unique<FakeOutputSurfaceClient>()),
+      graphics_backend_(backend) {
   // Keep texture sizes exactly matching the bounds of the RenderPass to avoid
   // floating point badness in texcoords.
   renderer_settings_.dont_round_texture_sizes_for_pixel_tests = true;
@@ -73,18 +61,29 @@ PixelTest::PixelTest(GraphicsBackend backend)
   if (backend == kSkiaVulkan) {
     scoped_feature_list_.InitAndEnableFeature(features::kVulkan);
     init_vulkan = true;
-  } else if (backend == kSkiaGraphite) {
+  } else if (backend == kSkiaGraphiteDawn) {
     scoped_feature_list_.InitAndEnableFeature(features::kSkiaGraphite);
-
-    // Force the use of Graphite even if disallowed for other reasons e.g. ANGLE
-    // Metal is not enabled on Mac.
     auto* command_line = base::CommandLine::ForCurrentProcess();
-    command_line->AppendSwitch(::switches::kSkiaGraphiteBackend);
-
+    bool use_gpu = command_line->HasSwitch(::switches::kUseGpuInTests);
+    // Force the use of Graphite even if disallowed for other reasons e.g.
+    // ANGLE Metal is not enabled on Mac. Use dawn-swiftshader backend if
+    // kUseGpuInTests is not set.
+    command_line->AppendSwitch(::switches::kEnableSkiaGraphite);
+    command_line->AppendSwitchASCII(
+        ::switches::kSkiaGraphiteBackend,
+        use_gpu ? ::switches::kSkiaGraphiteBackendDawn
+                : ::switches::kSkiaGraphiteBackendDawnSwiftshader);
     init_dawn = true;
 #if BUILDFLAG(IS_LINUX) || BUILDFLAG(IS_CHROMEOS)
     init_vulkan = true;
 #endif
+  } else if (backend == kSkiaGraphiteMetal) {
+    scoped_feature_list_.InitAndEnableFeature(features::kSkiaGraphite);
+    auto* command_line = base::CommandLine::ForCurrentProcess();
+    // Force the use of Graphite even if disallowed for other reasons.
+    command_line->AppendSwitch(::switches::kEnableSkiaGraphite);
+    command_line->AppendSwitchASCII(::switches::kSkiaGraphiteBackend,
+                                    ::switches::kSkiaGraphiteBackendMetal);
   } else {
     // Ensure that we don't accidentally have vulkan or graphite enabled.
     scoped_feature_list_.InitWithFeatures(
@@ -110,6 +109,41 @@ PixelTest::PixelTest(GraphicsBackend backend)
 
 PixelTest::~PixelTest() = default;
 
+void PixelTest::RenderReadbackTargetAndAreaToResultBitmap(
+    viz::AggregatedRenderPassList* pass_list,
+    viz::AggregatedRenderPass* target,
+    const gfx::Rect* copy_rect) {
+  base::RunLoop run_loop;
+
+  std::unique_ptr<viz::CopyOutputRequest> request =
+      std::make_unique<viz::CopyOutputRequest>(
+          viz::CopyOutputRequest::ResultFormat::RGBA,
+          viz::CopyOutputRequest::ResultDestination::kSystemMemory,
+          base::BindOnce(&PixelTest::ReadbackResult, base::Unretained(this),
+                         run_loop.QuitClosure()));
+  if (copy_rect) {
+    request->set_area(*copy_rect);
+  }
+  target->copy_requests.push_back(std::move(request));
+
+  if (software_renderer_) {
+    software_renderer_->SetDisablePictureQuadImageFiltering(
+        disable_picture_quad_image_filtering_);
+  }
+
+  float device_scale_factor = 1.f;
+  renderer_->DrawFrame(pass_list, device_scale_factor, device_viewport_size_,
+                       display_color_spaces_,
+                       std::move(surface_damage_rect_list_));
+
+  // Call SwapBuffersSkipped(), so the renderer can have a chance to release
+  // resources.
+  renderer_->SwapBuffersSkipped();
+
+  // Wait for the readback to complete.
+  run_loop.Run();
+}
+
 bool PixelTest::RunPixelTest(viz::AggregatedRenderPassList* pass_list,
                              const base::FilePath& ref_file,
                              const PixelComparator& comparator) {
@@ -122,8 +156,8 @@ bool PixelTest::RunPixelTestWithReadbackTarget(
     viz::AggregatedRenderPass* target,
     const base::FilePath& ref_file,
     const PixelComparator& comparator) {
-  return RunPixelTestWithReadbackTargetAndArea(
-      pass_list, target, ref_file, comparator, nullptr);
+  return RunPixelTestWithReadbackTargetAndArea(pass_list, target, ref_file,
+                                               comparator, nullptr);
 }
 
 bool PixelTest::RunPixelTestWithReadbackTargetAndArea(
@@ -132,68 +166,32 @@ bool PixelTest::RunPixelTestWithReadbackTargetAndArea(
     const base::FilePath& ref_file,
     const PixelComparator& comparator,
     const gfx::Rect* copy_rect) {
-  base::RunLoop run_loop;
+  RenderReadbackTargetAndAreaToResultBitmap(pass_list, target, copy_rect);
 
-  std::unique_ptr<viz::CopyOutputRequest> request =
-      std::make_unique<viz::CopyOutputRequest>(
-          viz::CopyOutputRequest::ResultFormat::RGBA,
-          viz::CopyOutputRequest::ResultDestination::kSystemMemory,
-          base::BindOnce(&PixelTest::ReadbackResult, base::Unretained(this),
-                         run_loop.QuitClosure()));
-  if (copy_rect)
-    request->set_area(*copy_rect);
-  target->copy_requests.push_back(std::move(request));
-
-  if (software_renderer_) {
-    software_renderer_->SetDisablePictureQuadImageFiltering(
-        disable_picture_quad_image_filtering_);
+  base::FilePath test_data_dir;
+  if (!base::PathService::Get(viz::Paths::DIR_TEST_DATA, &test_data_dir)) {
+    return false;
   }
 
-  float device_scale_factor = 1.f;
-  renderer_->DrawFrame(pass_list, device_scale_factor, device_viewport_size_,
-                       display_color_spaces_,
-                       std::move(surface_damage_rect_list_));
+  // If this is false, we didn't set up a readback on a render pass.
+  if (!result_bitmap_) {
+    return false;
+  }
 
-  // Call SwapBuffersSkipped(), so the renderer can have a chance to release
-  // resources.
-  renderer_->SwapBuffersSkipped();
+  base::CommandLine* cmd = base::CommandLine::ForCurrentProcess();
+  if (cmd->HasSwitch(switches::kRebaselinePixelTests)) {
+    return WritePNGFile(*result_bitmap_, test_data_dir.Append(ref_file), true);
+  }
 
-  // Wait for the readback to complete.
-  run_loop.Run();
-
-  return PixelsMatchReference(ref_file, comparator);
+  return MatchesPNGFile(*result_bitmap_, test_data_dir.Append(ref_file),
+                        comparator);
 }
 
 bool PixelTest::RunPixelTest(viz::AggregatedRenderPassList* pass_list,
                              std::vector<SkColor>* ref_pixels,
                              const PixelComparator& comparator) {
-  base::RunLoop run_loop;
-  auto* target = pass_list->back().get();
-
-  std::unique_ptr<viz::CopyOutputRequest> request =
-      std::make_unique<viz::CopyOutputRequest>(
-          viz::CopyOutputRequest::ResultFormat::RGBA,
-          viz::CopyOutputRequest::ResultDestination::kSystemMemory,
-          base::BindOnce(&PixelTest::ReadbackResult, base::Unretained(this),
-                         run_loop.QuitClosure()));
-  target->copy_requests.push_back(std::move(request));
-
-  if (software_renderer_) {
-    software_renderer_->SetDisablePictureQuadImageFiltering(
-        disable_picture_quad_image_filtering_);
-  }
-
-  float device_scale_factor = 1.f;
-  renderer_->DrawFrame(pass_list, device_scale_factor, device_viewport_size_,
-                       display_color_spaces_,
-                       std::move(surface_damage_rect_list_));
-
-  // Call SwapBuffersSkipped(), so the renderer can have a chance to release
-  // resources.
-  renderer_->SwapBuffersSkipped();
-
-  // Wait for the readback to complete.
-  run_loop.Run();
+  RenderReadbackTargetAndAreaToResultBitmap(pass_list, pass_list->back().get(),
+                                            nullptr);
 
   // Need to wrap |ref_pixels| in a SkBitmap.
   DCHECK_EQ(ref_pixels->size(), static_cast<size_t>(result_bitmap_->width() *
@@ -227,24 +225,6 @@ void PixelTest::ReadbackResult(base::OnceClosure quit_run_loop,
   std::move(quit_run_loop).Run();
 }
 
-bool PixelTest::PixelsMatchReference(const base::FilePath& ref_file,
-                                     const PixelComparator& comparator) {
-  base::FilePath test_data_dir;
-  if (!base::PathService::Get(viz::Paths::DIR_TEST_DATA, &test_data_dir))
-    return false;
-
-  // If this is false, we didn't set up a readback on a render pass.
-  if (!result_bitmap_)
-    return false;
-
-  base::CommandLine* cmd = base::CommandLine::ForCurrentProcess();
-  if (cmd->HasSwitch(switches::kRebaselinePixelTests))
-    return WritePNGFile(*result_bitmap_, test_data_dir.Append(ref_file), true);
-
-  return MatchesPNGFile(
-      *result_bitmap_, test_data_dir.Append(ref_file), comparator);
-}
-
 base::WritableSharedMemoryMapping PixelTest::AllocateSharedBitmapMemory(
     const viz::SharedBitmapId& id,
     const gfx::Size& size) {
@@ -267,7 +247,8 @@ viz::ResourceId PixelTest::AllocateAndFillSoftwareResource(
 
   return child_resource_provider_->ImportResource(
       viz::TransferableResource::MakeSoftware(
-          shared_bitmap_id, size, viz::SinglePlaneFormat::kRGBA_8888),
+          shared_bitmap_id, gpu::SyncToken(), size,
+          viz::SinglePlaneFormat::kRGBA_8888),
       base::DoNothing());
 }
 
@@ -298,7 +279,7 @@ void PixelTest::SetUpSkiaRenderer(gfx::SurfaceOrigin output_surface_origin) {
   // Set up the client side context provider, etc
   child_context_provider_ =
       base::MakeRefCounted<viz::TestInProcessContextProvider>(
-          viz::TestContextType::kGLES2WithRaster, /*support_locking=*/false);
+          viz::TestContextType::kSoftwareRaster, /*support_locking=*/false);
   gpu::ContextResult result = child_context_provider_->BindToCurrentSequence();
   DCHECK_EQ(result, gpu::ContextResult::kSuccess);
   child_resource_provider_ = std::make_unique<viz::ClientResourceProvider>();
@@ -322,9 +303,13 @@ void PixelTest::SetUpSoftwareRenderer() {
       std::make_unique<viz::SoftwareOutputDevice>());
   output_surface_->BindToClient(output_surface_client_.get());
   shared_bitmap_manager_ = std::make_unique<viz::TestSharedBitmapManager>();
+  shared_image_manager_ = std::make_unique<gpu::SharedImageManager>();
+  sync_point_manager_ = std::make_unique<gpu::SyncPointManager>();
+
   auto resource_provider =
       std::make_unique<viz::DisplayResourceProviderSoftware>(
-          shared_bitmap_manager_.get());
+          shared_bitmap_manager_.get(), shared_image_manager_.get(),
+          sync_point_manager_.get());
   child_resource_provider_ = std::make_unique<viz::ClientResourceProvider>();
 
   auto renderer = std::make_unique<viz::SoftwareRenderer>(
