@@ -114,7 +114,9 @@ viz::SolidColorDrawQuad* AddClippedQuad(viz::AggregatedRenderPass* pass,
                                         SkColor4f color) {
   viz::SharedQuadState* shared_state = pass->CreateAndAppendSharedQuadState();
   shared_state->SetAll(gfx::Transform(), rect, rect, gfx::MaskFilterInfo(),
-                       rect, false, 1, SkBlendMode::kSrcOver, 0);
+                       rect, /*contents_opaque=*/false, /*opacity_f=*/1,
+                       SkBlendMode::kSrcOver, /*sorting_context=*/0,
+                       /*layer_id=*/0u, /*fast_rounded_corner=*/false);
   auto* quad = pass->CreateAndAppendDrawQuad<viz::SolidColorDrawQuad>();
   quad->SetNew(shared_state, rect, rect, color, false);
   return quad;
@@ -126,9 +128,10 @@ viz::SolidColorDrawQuad* AddTransformedQuad(viz::AggregatedRenderPass* pass,
                                             const gfx::Transform& transform) {
   viz::SharedQuadState* shared_state = pass->CreateAndAppendSharedQuadState();
   shared_state->SetAll(transform, rect, rect, gfx::MaskFilterInfo(),
-                       absl::nullopt, false, 1,
-
-                       SkBlendMode::kSrcOver, 0);
+                       /*clip=*/std::nullopt, /*contents_opaque=*/false,
+                       /*opacity_f=*/1, SkBlendMode::kSrcOver,
+                       /*sorting_context=*/0, /*layer_id=*/0u,
+                       /*fast_rounded_corner=*/false);
   auto* quad = pass->CreateAndAppendDrawQuad<viz::SolidColorDrawQuad>();
   quad->SetNew(shared_state, rect, rect, color, false);
   return quad;
@@ -141,8 +144,10 @@ QuadType* AddRenderPassQuadInternal(RenderPassType* to_pass,
   viz::SharedQuadState* shared_state =
       to_pass->CreateAndAppendSharedQuadState();
   shared_state->SetAll(gfx::Transform(), output_rect, output_rect,
-                       gfx::MaskFilterInfo(), absl::nullopt, false, 1,
-                       SkBlendMode::kSrcOver, 0);
+                       gfx::MaskFilterInfo(), /*clip=*/std::nullopt,
+                       /*contents_opaque=*/false, /*opacity_f=*/1,
+                       SkBlendMode::kSrcOver, /*sorting_context=*/0,
+                       /*layer_id=*/0u, /*fast_rounded_corner=*/false);
   auto* quad = to_pass->template CreateAndAppendDrawQuad<QuadType>();
   quad->SetNew(shared_state, output_rect, output_rect, contributing_pass->id,
                viz::kInvalidResourceId, gfx::RectF(), gfx::Size(),
@@ -172,9 +177,10 @@ void AddRenderPassQuad(viz::AggregatedRenderPass* to_pass,
   gfx::Rect output_rect = contributing_pass->output_rect;
   viz::SharedQuadState* shared_state =
       to_pass->CreateAndAppendSharedQuadState();
-  shared_state->SetAll(transform, output_rect, output_rect,
-                       gfx::MaskFilterInfo(), absl::nullopt, false, 1,
-                       blend_mode, 0);
+  shared_state->SetAll(
+      transform, output_rect, output_rect, gfx::MaskFilterInfo(),
+      /*clip=*/std::nullopt, /*contents_opaque=*/false, 1, blend_mode,
+      /*sorting_context=*/0, /*layer_id=*/0u, /*fast_rounded_corner=*/false);
   auto* quad =
       to_pass->CreateAndAppendDrawQuad<viz::AggregatedRenderPassDrawQuad>();
   gfx::Size arbitrary_nonzero_size(1, 1);
@@ -191,7 +197,6 @@ std::vector<viz::ResourceId> AddOneOfEveryQuadType(
   gfx::Rect rect(0, 0, 100, 100);
   gfx::Rect visible_rect(0, 0, 100, 100);
   bool needs_blending = true;
-  const float vertex_opacity[] = {1.0f, 1.0f, 1.0f, 1.0f};
 
   static const gpu::SyncToken kSyncToken(
       gpu::CommandBufferNamespace::GPU_IO,
@@ -221,7 +226,10 @@ std::vector<viz::ResourceId> AddOneOfEveryQuadType(
   viz::SharedQuadState* shared_state =
       to_pass->CreateAndAppendSharedQuadState();
   shared_state->SetAll(gfx::Transform(), rect, rect, gfx::MaskFilterInfo(),
-                       absl::nullopt, false, 1, SkBlendMode::kSrcOver, 0);
+                       /*clip=*/std::nullopt, /*contents_opaque=*/false,
+                       /*opacity_f=*/1, SkBlendMode::kSrcOver,
+                       /*sorting_context=*/0, /*layer_id=*/0u,
+                       /*fast_rounded_corner=*/false);
 
   auto* debug_border_quad =
       to_pass->CreateAndAppendDrawQuad<viz::DebugBorderDrawQuad>();
@@ -246,25 +254,24 @@ std::vector<viz::ResourceId> AddOneOfEveryQuadType(
   // code paths.
   auto* stream_video_quad =
       to_pass->CreateAndAppendDrawQuad<viz::TextureDrawQuad>();
-  stream_video_quad->SetNew(shared_state, rect, visible_rect, needs_blending,
-                            resource6, false, gfx::PointF(0.f, 0.f),
-                            gfx::PointF(1.f, 1.f), SkColors::kTransparent,
-                            vertex_opacity, false, false, false,
-                            gfx::ProtectedVideoType::kHardwareProtected);
+  stream_video_quad->SetNew(
+      shared_state, rect, visible_rect, needs_blending, resource6, false,
+      gfx::PointF(0.f, 0.f), gfx::PointF(1.f, 1.f), SkColors::kTransparent,
+      false, false, false, gfx::ProtectedVideoType::kHardwareProtected);
   stream_video_quad->is_stream_video = true;
 
   auto* texture_quad = to_pass->CreateAndAppendDrawQuad<viz::TextureDrawQuad>();
-  texture_quad->SetNew(
-      shared_state, rect, visible_rect, needs_blending, resource1, false,
-      gfx::PointF(0.f, 0.f), gfx::PointF(1.f, 1.f), SkColors::kTransparent,
-      vertex_opacity, false, false, false, gfx::ProtectedVideoType::kClear);
+  texture_quad->SetNew(shared_state, rect, visible_rect, needs_blending,
+                       resource1, false, gfx::PointF(0.f, 0.f),
+                       gfx::PointF(1.f, 1.f), SkColors::kTransparent, false,
+                       false, false, gfx::ProtectedVideoType::kClear);
 
   auto* external_resource_texture_quad =
       to_pass->CreateAndAppendDrawQuad<viz::TextureDrawQuad>();
   external_resource_texture_quad->SetNew(
       shared_state, rect, visible_rect, needs_blending, resource8, false,
       gfx::PointF(0.f, 0.f), gfx::PointF(1.f, 1.f), SkColors::kTransparent,
-      vertex_opacity, false, false, false, gfx::ProtectedVideoType::kClear);
+      false, false, false, gfx::ProtectedVideoType::kClear);
 
   auto* scaled_tile_quad =
       to_pass->CreateAndAppendDrawQuad<viz::TileDrawQuad>();
@@ -288,7 +295,10 @@ std::vector<viz::ResourceId> AddOneOfEveryQuadType(
   viz::SharedQuadState* shared_state2 =
       to_pass->CreateAndAppendSharedQuadState();
   shared_state->SetAll(gfx::Transform(), rect, rect, gfx::MaskFilterInfo(),
-                       absl::nullopt, false, 1, SkBlendMode::kSrcOver, 0);
+                       /*clip=*/std::nullopt, /*contents_opaque=*/false,
+                       /*opacity_f=*/1, SkBlendMode::kSrcOver,
+                       /*sorting_context=*/0, /*layer_id=*/0u,
+                       /*fast_rounded_corner=*/false);
 
   auto* tile_quad = to_pass->CreateAndAppendDrawQuad<viz::TileDrawQuad>();
   tile_quad->SetNew(shared_state2, rect, visible_rect, needs_blending,
@@ -301,7 +311,7 @@ std::vector<viz::ResourceId> AddOneOfEveryQuadType(
                    gfx::Size(2, 2), plane_resources[0], plane_resources[1],
                    plane_resources[2], plane_resources[3],
                    gfx::ColorSpace::CreateREC601(), 0.0, 1.0, 8,
-                   gfx::ProtectedVideoType::kClear, absl::nullopt);
+                   gfx::ProtectedVideoType::kClear, std::nullopt);
 
   return {resource1,          resource2,          resource3,
           resource4,          resource5,          resource6,
@@ -322,8 +332,6 @@ void AddOneOfEveryQuadTypeInDisplayResourceProvider(
   gfx::Rect rect(0, 0, 100, 100);
   gfx::Rect visible_rect(0, 0, 100, 100);
   bool needs_blending = true;
-  const float vertex_opacity[] = {1.0f, 1.0f, 1.0f, 1.0f};
-
   static const gpu::SyncToken kDefaultSyncToken(
       gpu::CommandBufferNamespace::GPU_IO,
       gpu::CommandBufferId::FromUnsafeValue(0x111), 42);
@@ -404,7 +412,10 @@ void AddOneOfEveryQuadTypeInDisplayResourceProvider(
   viz::SharedQuadState* shared_state =
       to_pass->CreateAndAppendSharedQuadState();
   shared_state->SetAll(gfx::Transform(), rect, rect, gfx::MaskFilterInfo(),
-                       absl::nullopt, false, 1, SkBlendMode::kSrcOver, 0);
+                       /*clip=*/std::nullopt, /*contents_opaque=*/false,
+                       /*opacity_f=*/1, SkBlendMode::kSrcOver,
+                       /*sorting_context=*/0, /*layer_id=*/0u,
+                       /*fast_rounded_corner=*/false);
 
   viz::DebugBorderDrawQuad* debug_border_quad =
       to_pass->CreateAndAppendDrawQuad<viz::DebugBorderDrawQuad>();
@@ -426,26 +437,25 @@ void AddOneOfEveryQuadTypeInDisplayResourceProvider(
 
   viz::TextureDrawQuad* stream_video_quad =
       to_pass->CreateAndAppendDrawQuad<viz::TextureDrawQuad>();
-  stream_video_quad->SetNew(shared_state, rect, visible_rect, needs_blending,
-                            mapped_resource6, false, gfx::PointF(0.f, 0.f),
-                            gfx::PointF(1.f, 1.f), SkColors::kTransparent,
-                            vertex_opacity, false, false, false,
-                            gfx::ProtectedVideoType::kHardwareProtected);
+  stream_video_quad->SetNew(
+      shared_state, rect, visible_rect, needs_blending, mapped_resource6, false,
+      gfx::PointF(0.f, 0.f), gfx::PointF(1.f, 1.f), SkColors::kTransparent,
+      false, false, false, gfx::ProtectedVideoType::kHardwareProtected);
   stream_video_quad->is_stream_video = true;
 
   viz::TextureDrawQuad* texture_quad =
       to_pass->CreateAndAppendDrawQuad<viz::TextureDrawQuad>();
-  texture_quad->SetNew(
-      shared_state, rect, visible_rect, needs_blending, mapped_resource1, false,
-      gfx::PointF(0.f, 0.f), gfx::PointF(1.f, 1.f), SkColors::kTransparent,
-      vertex_opacity, false, false, false, gfx::ProtectedVideoType::kClear);
+  texture_quad->SetNew(shared_state, rect, visible_rect, needs_blending,
+                       mapped_resource1, false, gfx::PointF(0.f, 0.f),
+                       gfx::PointF(1.f, 1.f), SkColors::kTransparent, false,
+                       false, false, gfx::ProtectedVideoType::kClear);
 
   viz::TextureDrawQuad* external_resource_texture_quad =
       to_pass->CreateAndAppendDrawQuad<viz::TextureDrawQuad>();
   external_resource_texture_quad->SetNew(
       shared_state, rect, visible_rect, needs_blending, mapped_resource8, false,
       gfx::PointF(0.f, 0.f), gfx::PointF(1.f, 1.f), SkColors::kTransparent,
-      vertex_opacity, false, false, false, gfx::ProtectedVideoType::kClear);
+      false, false, false, gfx::ProtectedVideoType::kClear);
 
   viz::TileDrawQuad* scaled_tile_quad =
       to_pass->CreateAndAppendDrawQuad<viz::TileDrawQuad>();
@@ -469,7 +479,10 @@ void AddOneOfEveryQuadTypeInDisplayResourceProvider(
   viz::SharedQuadState* shared_state2 =
       to_pass->CreateAndAppendSharedQuadState();
   shared_state2->SetAll(gfx::Transform(), rect, rect, gfx::MaskFilterInfo(),
-                        absl::nullopt, false, 1, SkBlendMode::kSrcOver, 0);
+                        /*clip=*/std::nullopt, /*contents_opaque=*/false,
+                        /*opacity_f=*/1, SkBlendMode::kSrcOver,
+                        /*sorting_context=*/0, /*layer_id=*/0u,
+                        /*fast_rounded_corner=*/false);
 
   viz::TileDrawQuad* tile_quad =
       to_pass->CreateAndAppendDrawQuad<viz::TileDrawQuad>();
@@ -484,7 +497,29 @@ void AddOneOfEveryQuadTypeInDisplayResourceProvider(
                    gfx::Size(2, 1), mapped_plane_resources[0],
                    mapped_plane_resources[1], mapped_plane_resources[2],
                    mapped_plane_resources[3], gfx::ColorSpace::CreateREC601(),
-                   0.0, 1.0, 8, gfx::ProtectedVideoType::kClear, absl::nullopt);
+                   0.0, 1.0, 8, gfx::ProtectedVideoType::kClear, std::nullopt);
+}
+
+std::unique_ptr<viz::AggregatedRenderPass> CopyToAggregatedRenderPass(
+    viz::CompositorRenderPass* from_pass,
+    viz::AggregatedRenderPassId to_id,
+    gfx::ContentColorUsage content_usage) {
+  auto copy_pass = std::make_unique<viz::AggregatedRenderPass>(
+      from_pass->shared_quad_state_list.size(), from_pass->quad_list.size());
+  copy_pass->SetAll(to_id, from_pass->output_rect, from_pass->damage_rect,
+                    from_pass->transform_to_root_target, from_pass->filters,
+                    from_pass->backdrop_filters,
+                    from_pass->backdrop_filter_bounds, content_usage,
+                    from_pass->has_transparent_background,
+                    from_pass->cache_render_pass,
+                    from_pass->has_damage_from_contributing_content,
+                    from_pass->generate_mipmap);
+
+  copy_pass->shared_quad_state_list =
+      std::move(from_pass->shared_quad_state_list);
+  copy_pass->quad_list = std::move(from_pass->quad_list);
+
+  return copy_pass;
 }
 
 }  // namespace cc
