@@ -6,8 +6,7 @@
 #define GOOGLE_APIS_COMMON_TIME_UTIL_H_
 
 #include <string>
-
-#include "base/strings/string_piece.h"
+#include <string_view>
 
 namespace base {
 class Time;
@@ -19,12 +18,12 @@ namespace util {
 // Parses an RFC 3339 date/time into a base::Time, returning true on success.
 // The time string must be in the format "yyyy-mm-ddThh:mm:ss.dddTZ" (TZ is
 // either '+hh:mm', '-hh:mm', 'Z' (representing UTC), or an empty string).
-bool GetTimeFromString(base::StringPiece raw_value, base::Time* time);
+bool GetTimeFromString(std::string_view raw_value, base::Time* time);
 
 // Parses a date only string of format "yyyy-mm-dd" into a `base::Time`
 // returning true on success. The time part of `base::Time` is set to 00:00
 // (midnight) UTC.
-bool GetDateOnlyFromString(base::StringPiece raw_value, base::Time* time);
+bool GetDateOnlyFromString(std::string_view raw_value, base::Time* time);
 
 // Formats a base::Time as an RFC 3339 date/time (in UTC).
 // If |time| is null, returns "null".
