@@ -4,6 +4,7 @@
 
 #include "google_apis/gaia/fake_gaia.h"
 
+#include <string_view>
 #include <utility>
 #include <vector>
 
@@ -21,7 +22,6 @@
 #include "base/ranges/algorithm.h"
 #include "base/strings/strcat.h"
 #include "base/strings/string_number_conversions.h"
-#include "base/strings/string_piece.h"
 #include "base/strings/string_split.h"
 #include "base/strings/string_util.h"
 #include "base/strings/stringprintf.h"
@@ -51,14 +51,10 @@ using net::test_server::HttpRequest;
 namespace {
 
 const char kTestAuthCode[] = "fake-auth-code";
-const char kTestGaiaUberToken[] = "fake-uber-token";
 const char kTestAuthLoginAccessToken[] = "fake-access-token";
 const char kTestRefreshToken[] = "fake-refresh-token";
 const char kTestSessionSIDCookie[] = "fake-session-SID-cookie";
 const char kTestSessionLSIDCookie[] = "fake-session-LSID-cookie";
-const char kTestOAuthLoginSID[] = "fake-oauth-SID-cookie";
-const char kTestOAuthLoginLSID[] = "fake-oauth-LSID-cookie";
-const char kTestOAuthLoginAuthCode[] = "fake-oauth-auth-code";
 const char kTestReauthProofToken[] = "fake-reauth-proof-token";
 // Add SameSite=None and Secure because these cookies are needed in a
 // cross-site context.
@@ -174,28 +170,28 @@ FakeGaia::AccessTokenInfo::AccessTokenInfo(const AccessTokenInfo& other) =
 
 FakeGaia::AccessTokenInfo::~AccessTokenInfo() = default;
 
-FakeGaia::MergeSessionParams::MergeSessionParams() = default;
+FakeGaia::Configuration::Configuration() = default;
 
-FakeGaia::MergeSessionParams::~MergeSessionParams() = default;
+FakeGaia::Configuration::~Configuration() = default;
 
-void FakeGaia::MergeSessionParams::Update(const MergeSessionParams& update) {
+void FakeGaia::Configuration::Update(const Configuration& update) {
   // This lambda uses a pointer to data member to merge attributes.
-  auto maybe_update_field =
-      [this, &update](std::string MergeSessionParams::*field_ptr) {
-        if (!(update.*field_ptr).empty())
-          this->*field_ptr = update.*field_ptr;
-      };
+  auto maybe_update_field = [this,
+                             &update](std::string Configuration::*field_ptr) {
+    if (!(update.*field_ptr).empty()) {
+      this->*field_ptr = update.*field_ptr;
+    }
+  };
 
-  maybe_update_field(&MergeSessionParams::auth_sid_cookie);
-  maybe_update_field(&MergeSessionParams::auth_lsid_cookie);
-  maybe_update_field(&MergeSessionParams::auth_code);
-  maybe_update_field(&MergeSessionParams::refresh_token);
-  maybe_update_field(&MergeSessionParams::access_token);
-  maybe_update_field(&MergeSessionParams::id_token);
-  maybe_update_field(&MergeSessionParams::gaia_uber_token);
-  maybe_update_field(&MergeSessionParams::session_sid_cookie);
-  maybe_update_field(&MergeSessionParams::session_lsid_cookie);
-  maybe_update_field(&MergeSessionParams::email);
+  maybe_update_field(&Configuration::auth_sid_cookie);
+  maybe_update_field(&Configuration::auth_lsid_cookie);
+  maybe_update_field(&Configuration::auth_code);
+  maybe_update_field(&Configuration::refresh_token);
+  maybe_update_field(&Configuration::access_token);
+  maybe_update_field(&Configuration::id_token);
+  maybe_update_field(&Configuration::session_sid_cookie);
+  maybe_update_field(&Configuration::session_lsid_cookie);
+  maybe_update_field(&Configuration::email);
 
   if (!update.signed_out_gaia_ids.empty())
     signed_out_gaia_ids = update.signed_out_gaia_ids;
@@ -207,7 +203,7 @@ FakeGaia::SyncTrustedVaultKeys::~SyncTrustedVaultKeys() = default;
 
 FakeGaia::FakeGaia() : issue_oauth_code_cookie_(false) {
   base::FilePath source_root_dir;
-  base::PathService::Get(base::DIR_SOURCE_ROOT, &source_root_dir);
+  base::PathService::Get(base::DIR_SRC_TEST_DATA_ROOT, &source_root_dir);
   CHECK(base::ReadFileToString(
       source_root_dir.Append(base::FilePath(kEmbeddedSetupChromeos)),
       &embedded_setup_chromeos_response_));
@@ -215,28 +211,27 @@ FakeGaia::FakeGaia() : issue_oauth_code_cookie_(false) {
 
 FakeGaia::~FakeGaia() = default;
 
-void FakeGaia::SetFakeMergeSessionParams(const std::string& email,
-                                         const std::string& auth_sid_cookie,
-                                         const std::string& auth_lsid_cookie) {
-  FakeGaia::MergeSessionParams params;
+void FakeGaia::SetConfigurationHelper(const std::string& email,
+                                      const std::string& auth_sid_cookie,
+                                      const std::string& auth_lsid_cookie) {
+  FakeGaia::Configuration params;
   params.auth_sid_cookie = auth_sid_cookie;
   params.auth_lsid_cookie = auth_lsid_cookie;
   params.auth_code = kTestAuthCode;
   params.refresh_token = kTestRefreshToken;
   params.access_token = kTestAuthLoginAccessToken;
-  params.gaia_uber_token = kTestGaiaUberToken;
   params.session_sid_cookie = kTestSessionSIDCookie;
   params.session_lsid_cookie = kTestSessionLSIDCookie;
   params.email = email;
-  SetMergeSessionParams(params);
+  SetConfiguration(params);
 }
 
-void FakeGaia::SetMergeSessionParams(const MergeSessionParams& params) {
-  merge_session_params_ = params;
+void FakeGaia::SetConfiguration(const Configuration& params) {
+  configuration_ = params;
 }
 
-void FakeGaia::UpdateMergeSessionParams(const MergeSessionParams& params) {
-  merge_session_params_.Update(params);
+void FakeGaia::UpdateConfiguration(const Configuration& params) {
+  configuration_.Update(params);
 }
 
 void FakeGaia::MapEmailToGaiaId(const std::string& email,
@@ -279,9 +274,9 @@ void FakeGaia::AddGoogleAccountsSigninHeader(BasicHttpResponse* http_response,
 void FakeGaia::SetOAuthCodeCookie(BasicHttpResponse* http_response) const {
   DCHECK(http_response);
   http_response->AddCustomHeader(
-      "Set-Cookie", base::StringPrintf("oauth_code=%s%s",
-                                       merge_session_params_.auth_code.c_str(),
-                                       kTestCookieAttributes));
+      "Set-Cookie",
+      base::StringPrintf("oauth_code=%s%s", configuration_.auth_code.c_str(),
+                         kTestCookieAttributes));
 }
 
 void FakeGaia::AddSyncTrustedKeysHeader(BasicHttpResponse* http_response,
@@ -297,9 +292,6 @@ void FakeGaia::AddSyncTrustedKeysHeader(BasicHttpResponse* http_response,
 
 void FakeGaia::Initialize() {
   GaiaUrls* gaia_urls = GaiaUrls::GetInstance();
-  // Handles /MergeSession GAIA call.
-  REGISTER_RESPONSE_HANDLER(gaia_urls->merge_session_url(), HandleMergeSession);
-
   // Handles /oauth/multilogin GAIA call.
   REGISTER_RESPONSE_HANDLER(gaia_urls->oauth_multilogin_url(),
                             HandleMultilogin);
@@ -320,9 +312,6 @@ void FakeGaia::Initialize() {
   REGISTER_RESPONSE_HANDLER(gaia_urls->embedded_reauth_chromeos_url(),
                             HandleEmbeddedReauthChromeos);
 
-  // Handles /OAuthLogin GAIA call.
-  REGISTER_RESPONSE_HANDLER(gaia_urls->oauth1_login_url(), HandleOAuthLogin);
-
   // Handles /_/embedded/lookup/accountlookup for /embedded/setup/chromeos
   // authentication request.
   REGISTER_PATH_RESPONSE_HANDLER("/_/embedded/lookup/accountlookup",
@@ -337,7 +326,8 @@ void FakeGaia::Initialize() {
   REGISTER_PATH_RESPONSE_HANDLER("/SSO", HandleSSO);
 
   // Handles the /samlredirect requests for tests.
-  REGISTER_PATH_RESPONSE_HANDLER("/samlredirect", HandleSAMLRedirect);
+  REGISTER_RESPONSE_HANDLER(gaia_urls->saml_redirect_chromeos_url(),
+                            HandleSAMLRedirect);
 
   REGISTER_RESPONSE_HANDLER(
       gaia_urls->gaia_url().Resolve(kFakeSAMLContinuePath),
@@ -485,44 +475,6 @@ void FakeGaia::SetRefreshTokenToDeviceIdMap(
   refresh_token_to_device_id_map_ = refresh_token_to_device_id_map;
 }
 
-void FakeGaia::HandleMergeSession(const HttpRequest& request,
-                                  BasicHttpResponse* http_response) {
-  http_response->set_code(net::HTTP_UNAUTHORIZED);
-  if (merge_session_params_.session_sid_cookie.empty() ||
-      merge_session_params_.session_lsid_cookie.empty()) {
-    http_response->set_code(net::HTTP_BAD_REQUEST);
-    return;
-  }
-
-  GURL request_url = GURL("http://localhost").Resolve(request.relative_url);
-  std::string request_query = request_url.query();
-
-  std::string uber_token;
-  if (!GetQueryParameter(request_query, "uberauth", &uber_token) ||
-      uber_token != merge_session_params_.gaia_uber_token) {
-    LOG(ERROR) << "Missing or invalid 'uberauth' param in /MergeSession call";
-    return;
-  }
-
-  std::string continue_url;
-  if (!GetQueryParameter(request_query, "continue", &continue_url)) {
-    LOG(ERROR) << "Missing or invalid 'continue' param in /MergeSession call";
-    return;
-  }
-
-  std::string source;
-  if (!GetQueryParameter(request_query, "source", &source)) {
-    LOG(ERROR) << "Missing or invalid 'source' param in /MergeSession call";
-    return;
-  }
-
-  SetCookies(http_response, merge_session_params_.session_sid_cookie,
-             merge_session_params_.session_lsid_cookie);
-  // TODO(zelidrag): Not used now.
-  http_response->set_content("OK");
-  http_response->set_code(net::HTTP_OK);
-}
-
 void FakeGaia::FormatOkJSONResponse(const base::ValueView& value,
                                     BasicHttpResponse* http_response) {
   FormatJSONResponse(value, net::HTTP_OK, http_response);
@@ -613,46 +565,6 @@ void FakeGaia::HandleEmbeddedReauthChromeos(const HttpRequest& request,
   http_response->set_content_type("text/html");
 }
 
-void FakeGaia::HandleOAuthLogin(const HttpRequest& request,
-                                BasicHttpResponse* http_response) {
-  http_response->set_code(net::HTTP_UNAUTHORIZED);
-  if (merge_session_params_.gaia_uber_token.empty()) {
-    http_response->set_code(net::HTTP_FORBIDDEN);
-    http_response->set_content("Error=BadAuthentication");
-    return;
-  }
-
-  std::string access_token;
-  if (!GetAccessToken(request, kAuthHeaderBearer, &access_token) &&
-      !GetAccessToken(request, kAuthHeaderOAuth, &access_token)) {
-    LOG(ERROR) << "/OAuthLogin missing access token in the header";
-    return;
-  }
-
-  GURL request_url = GURL("http://localhost").Resolve(request.relative_url);
-  std::string request_query = request_url.query();
-
-  std::string source;
-  if (!GetQueryParameter(request_query, "source", &source) &&
-      !GetQueryParameter(request.content, "source", &source)) {
-    LOG(ERROR) << "Missing 'source' param in /OAuthLogin call";
-    return;
-  }
-
-  std::string issue_uberauth;
-  if (GetQueryParameter(request_query, "issueuberauth", &issue_uberauth) &&
-      issue_uberauth == "1") {
-    http_response->set_content(merge_session_params_.gaia_uber_token);
-    http_response->set_code(net::HTTP_OK);
-    // Issue GAIA uber token.
-  } else {
-    http_response->set_content(
-        base::StringPrintf("SID=%s\nLSID=%s\nAuth=%s", kTestOAuthLoginSID,
-                           kTestOAuthLoginLSID, kTestOAuthLoginAuthCode));
-    http_response->set_code(net::HTTP_OK);
-  }
-}
-
 void FakeGaia::HandleEmbeddedLookupAccountLookup(
     const HttpRequest& request,
     BasicHttpResponse* http_response) {
@@ -689,10 +601,10 @@ void FakeGaia::HandleEmbeddedSigninChallenge(const HttpRequest& request,
                                          kTestCookieAttributes));
   }
 
-  if (!merge_session_params_.auth_sid_cookie.empty() &&
-      !merge_session_params_.auth_lsid_cookie.empty()) {
-    SetCookies(http_response, merge_session_params_.auth_sid_cookie,
-               merge_session_params_.auth_lsid_cookie);
+  if (!configuration_.auth_sid_cookie.empty() &&
+      !configuration_.auth_lsid_cookie.empty()) {
+    SetCookies(http_response, configuration_.auth_sid_cookie,
+               configuration_.auth_lsid_cookie);
   }
 
   AddGoogleAccountsSigninHeader(http_response, email);
@@ -707,10 +619,10 @@ void FakeGaia::HandleEmbeddedSigninChallenge(const HttpRequest& request,
 
 void FakeGaia::HandleSSO(const HttpRequest& request,
                          BasicHttpResponse* http_response) {
-  if (!merge_session_params_.auth_sid_cookie.empty() &&
-      !merge_session_params_.auth_lsid_cookie.empty()) {
-    SetCookies(http_response, merge_session_params_.auth_sid_cookie,
-               merge_session_params_.auth_lsid_cookie);
+  if (!configuration_.auth_sid_cookie.empty() &&
+      !configuration_.auth_lsid_cookie.empty()) {
+    SetCookies(http_response, configuration_.auth_sid_cookie,
+               configuration_.auth_lsid_cookie);
   }
   std::string relay_state;
   GetQueryParameter(request.content, "RelayState", &relay_state);
@@ -719,7 +631,7 @@ void FakeGaia::HandleSSO(const HttpRequest& request,
   http_response->AddCustomHeader("Location", redirect_url);
   http_response->AddCustomHeader("Google-Accounts-SAML", "End");
 
-  AddGoogleAccountsSigninHeader(http_response, merge_session_params_.email);
+  AddGoogleAccountsSigninHeader(http_response, configuration_.email);
 
   if (issue_oauth_code_cookie_)
     SetOAuthCodeCookie(http_response);
@@ -743,7 +655,7 @@ void FakeGaia::HandleAuthToken(const HttpRequest& request,
   if (grant_type == "authorization_code") {
     std::string auth_code;
     if (!GetQueryParameter(request.content, "code", &auth_code) ||
-        auth_code != merge_session_params_.auth_code) {
+        auth_code != configuration_.auth_code) {
       http_response->set_code(net::HTTP_BAD_REQUEST);
       LOG(ERROR) << "No 'code' param in /oauth2/v4/token";
       return;
@@ -765,17 +677,15 @@ void FakeGaia::HandleAuthToken(const HttpRequest& request,
     }
 
     if (!device_id.empty()) {
-      refresh_token_to_device_id_map_[merge_session_params_.refresh_token] =
-          device_id;
+      refresh_token_to_device_id_map_[configuration_.refresh_token] = device_id;
     }
 
-    auto response_dict =
-        base::Value::Dict()
-            .Set("refresh_token", merge_session_params_.refresh_token)
-            .Set("access_token", merge_session_params_.access_token)
-            .Set("expires_in", 3600);
-    if (!merge_session_params_.id_token.empty()) {
-      response_dict.Set("id_token", merge_session_params_.id_token);
+    auto response_dict = base::Value::Dict()
+                             .Set("refresh_token", configuration_.refresh_token)
+                             .Set("access_token", configuration_.access_token)
+                             .Set("expires_in", 3600);
+    if (!configuration_.id_token.empty()) {
+      response_dict.Set("id_token", configuration_.id_token);
     }
     FormatOkJSONResponse(response_dict, http_response);
     return;
@@ -819,7 +729,7 @@ void FakeGaia::HandleTokenInfo(const HttpRequest& request,
             .Set("issued_to", token_info->issued_to)
             .Set("audience", token_info->audience)
             .Set("user_id", token_info->user_id)
-            .Set("scope", base::JoinString(std::vector<base::StringPiece>(
+            .Set("scope", base::JoinString(std::vector<std::string_view>(
                                                token_info->scopes.begin(),
                                                token_info->scopes.end()),
                                            " "))
@@ -864,10 +774,10 @@ void FakeGaia::HandleListAccounts(const HttpRequest& request,
 
   std::vector<std::string> listed_accounts;
   listed_accounts.push_back(base::StringPrintf(
-      kIndividualListedAccountResponseFormat,
-      merge_session_params_.email.c_str(), kDefaultGaiaId, kAccountIsSignedIn));
+      kIndividualListedAccountResponseFormat, configuration_.email.c_str(),
+      kDefaultGaiaId, kAccountIsSignedIn));
 
-  for (const std::string& gaia_id : merge_session_params_.signed_out_gaia_ids) {
+  for (const std::string& gaia_id : configuration_.signed_out_gaia_ids) {
     DCHECK_NE(kDefaultGaiaId, gaia_id);
 
     const std::string email = GetEmailOfGaiaId(gaia_id);
@@ -907,7 +817,7 @@ void FakeGaia::HandleSAMLRedirect(const HttpRequest& request,
                                   BasicHttpResponse* http_response) {
   GURL request_url = GURL("http://localhost").Resolve(request.relative_url);
 
-  absl::optional<GURL> redirect_url = GetSamlRedirectUrl(request_url);
+  std::optional<GURL> redirect_url = GetSamlRedirectUrl(request_url);
   if (!redirect_url) {
     http_response->set_code(net::HTTP_BAD_REQUEST);
     return;
@@ -974,7 +884,6 @@ void FakeGaia::HandleGetReAuthProofToken(const HttpRequest& request,
     default:
       LOG(FATAL) << "Unsupported ReAuthProofTokenStatus: "
                  << static_cast<int>(next_reauth_status_);
-      break;
   }
 }
 
@@ -982,8 +891,8 @@ void FakeGaia::HandleMultilogin(const HttpRequest& request,
                                 BasicHttpResponse* http_response) {
   http_response->set_code(net::HTTP_UNAUTHORIZED);
 
-  if (merge_session_params_.session_sid_cookie.empty() ||
-      merge_session_params_.session_lsid_cookie.empty()) {
+  if (configuration_.session_sid_cookie.empty() ||
+      configuration_.session_lsid_cookie.empty()) {
     http_response->set_code(net::HTTP_BAD_REQUEST);
     return;
   }
@@ -999,11 +908,9 @@ void FakeGaia::HandleMultilogin(const HttpRequest& request,
 
   http_response->set_content(
       ")]}'\n{\"status\":\"OK\",\"cookies\":[" +
-      FormatCookieForMultilogin("SID",
-                                merge_session_params_.session_sid_cookie) +
+      FormatCookieForMultilogin("SID", configuration_.session_sid_cookie) +
       "," +
-      FormatCookieForMultilogin("LSID",
-                                merge_session_params_.session_lsid_cookie) +
+      FormatCookieForMultilogin("LSID", configuration_.session_lsid_cookie) +
       "]}");
   http_response->set_code(net::HTTP_OK);
 }
@@ -1016,7 +923,7 @@ void FakeGaia::HandleFakeRemoveLocalAccount(
   std::string gaia_id;
   GetQueryParameter(request.GetURL().query(), "gaia_id", &gaia_id);
 
-  if (!base::Erase(merge_session_params_.signed_out_gaia_ids, gaia_id)) {
+  if (!base::Erase(configuration_.signed_out_gaia_ids, gaia_id)) {
     http_response->set_code(net::HTTP_BAD_REQUEST);
     return;
   }
@@ -1043,7 +950,7 @@ std::string FakeGaia::GetEmbeddedSetupChromeosResponseContent() const {
   return response_with_iframe;
 }
 
-absl::optional<GURL> FakeGaia::GetSamlRedirectUrl(
+std::optional<GURL> FakeGaia::GetSamlRedirectUrl(
     const GURL& request_url) const {
   // When deciding on saml redirection, gaia is expected to prioritize sso
   // profile over the domain.
@@ -1064,5 +971,5 @@ absl::optional<GURL> FakeGaia::GetSamlRedirectUrl(
     return itr_domain->second;
   }
 
-  return absl::nullopt;
+  return std::nullopt;
 }
