@@ -8,6 +8,7 @@
 #include <utility>
 #include <vector>
 
+#include <optional>
 #include "base/functional/bind.h"
 #include "base/json/json_reader.h"
 #include "base/json/json_writer.h"
@@ -37,7 +38,6 @@
 #include "services/network/public/cpp/shared_url_loader_factory.h"
 #include "services/network/public/cpp/simple_url_loader.h"
 #include "services/network/public/mojom/url_response_head.mojom.h"
-#include "third_party/abseil-cpp/absl/types/optional.h"
 
 namespace {
 
@@ -52,14 +52,14 @@ constexpr char kJsonContentType[] = "application/json;charset=UTF-8";
 
 std::unique_ptr<const GaiaAuthConsumer::ClientOAuthResult>
 ExtractOAuth2TokenPairResponse(const std::string& data) {
-  absl::optional<base::Value> value = base::JSONReader::Read(data);
+  std::optional<base::Value> value = base::JSONReader::Read(data);
   if (!value || !value->is_dict())
     return nullptr;
   base::Value::Dict& dict = value->GetDict();
 
   std::string* refresh_token = dict.FindString("refresh_token");
   std::string* access_token = dict.FindString("access_token");
-  absl::optional<int> expires_in_secs = dict.FindInt("expires_in");
+  std::optional<int> expires_in_secs = dict.FindInt("expires_in");
   if (!refresh_token || !access_token || !expires_in_secs.has_value())
     return nullptr;
 
@@ -95,7 +95,7 @@ GetTokenRevocationStatusFromResponseData(const std::string& data,
   if (response_code == net::HTTP_INTERNAL_SERVER_ERROR)
     return GaiaAuthConsumer::TokenRevocationStatus::kServerError;
 
-  absl::optional<base::Value> value = base::JSONReader::Read(data);
+  std::optional<base::Value> value = base::JSONReader::Read(data);
   if (!value || !value->is_dict())
     return GaiaAuthConsumer::TokenRevocationStatus::kUnknownError;
   base::Value::Dict& dict = value->GetDict();
@@ -114,7 +114,7 @@ GetTokenRevocationStatusFromResponseData(const std::string& data,
 
 base::Value::Dict ParseJSONDict(const std::string& data) {
   base::Value::Dict response_dict;
-  absl::optional<base::Value> message_value = base::JSONReader::Read(data);
+  std::optional<base::Value> message_value = base::JSONReader::Read(data);
   if (message_value && message_value->is_dict()) {
     response_dict.Merge(std::move(message_value->GetDict()));
   }
@@ -202,15 +202,6 @@ const char
         "bound_token_registration_jwt=%s";
 // static
 const char GaiaAuthFetcher::kOAuth2RevokeTokenBodyFormat[] = "token=%s";
-// static
-const char GaiaAuthFetcher::kMergeSessionFormat[] =
-    "?uberauth=%s&"
-    "continue=%s&"
-    "source=%s";
-// static
-const char GaiaAuthFetcher::kUberAuthTokenURLFormat[] =
-    "?source=%s&"
-    "issueuberauth=1";
 
 // static
 const char GaiaAuthFetcher::kErrorParam[] = "Error";
@@ -232,9 +223,6 @@ GaiaAuthFetcher::GaiaAuthFetcher(
       source_(source.ToString()),
       oauth2_token_gurl_(GaiaUrls::GetInstance()->oauth2_token_url()),
       oauth2_revoke_gurl_(GaiaUrls::GetInstance()->oauth2_revoke_url()),
-      merge_session_gurl_(GaiaUrls::GetInstance()->merge_session_url()),
-      uberauth_token_gurl_(GaiaUrls::GetInstance()->oauth1_login_url().Resolve(
-          base::StringPrintf(kUberAuthTokenURLFormat, source_.c_str()))),
       oauth_multilogin_gurl_(GaiaUrls::GetInstance()->oauth_multilogin_url()),
       list_accounts_gurl_(
           GaiaUrls::GetInstance()->ListAccountsURLWithSource(source_)),
@@ -263,6 +251,10 @@ bool GaiaAuthFetcher::IsReAuthApiUrl(const GURL& url) {
                           base::CompareCase::SENSITIVE);
 }
 
+bool GaiaAuthFetcher::IsListAccountsUrl(const GURL& url) {
+  return url == list_accounts_gurl_;
+}
+
 void GaiaAuthFetcher::CreateAndStartGaiaFetcher(
     const std::string& body,
     const std::string& body_content_type,
@@ -362,28 +354,6 @@ std::string GaiaAuthFetcher::MakeRevokeTokenBody(
   return base::StringPrintf(kOAuth2RevokeTokenBodyFormat, auth_token.c_str());
 }
 
-// static
-std::string GaiaAuthFetcher::MakeMergeSessionQuery(
-    const std::string& auth_token,
-    const std::string& external_cc_result,
-    const std::string& continue_url,
-    const std::string& source) {
-  std::string encoded_auth_token = base::EscapeUrlEncodedData(auth_token, true);
-  std::string encoded_continue_url =
-      base::EscapeUrlEncodedData(continue_url, true);
-  std::string encoded_source = base::EscapeUrlEncodedData(source, true);
-  std::string result =
-      base::StringPrintf(kMergeSessionFormat, encoded_auth_token.c_str(),
-                         encoded_continue_url.c_str(), encoded_source.c_str());
-  if (!external_cc_result.empty()) {
-    base::StringAppendF(
-        &result, "&externalCcResult=%s",
-        base::EscapeUrlEncodedData(external_cc_result, true).c_str());
-  }
-
-  return result;
-}
-
 // static
 void GaiaAuthFetcher::ParseFailureResponse(const std::string& data,
                                            std::string* error,
@@ -484,97 +454,6 @@ void GaiaAuthFetcher::StartAuthCodeForOAuth2TokenExchangeWithDeviceId(
       google_apis::GetOmitCredentialsModeForGaiaRequests(), traffic_annotation);
 }
 
-void GaiaAuthFetcher::StartMergeSession(const std::string& uber_token,
-                                        const std::string& external_cc_result) {
-  DCHECK(!fetch_pending_) << "Tried to fetch two things at once!";
-
-  VLOG(1) << "Starting MergeSession with uber_token=" << uber_token;
-
-  // The continue URL is a required parameter of the MergeSession API, but in
-  // this case we don't actually need or want to navigate to it.  Setting it to
-  // an arbitrary Google URL.
-  //
-  // In order for the new session to be merged correctly, the server needs to
-  // know what sessions already exist in the browser.  The fetcher needs to be
-  // created such that it sends the cookies with the request, which is
-  // different from all other requests the fetcher can make.
-  std::string continue_url("http://www.google.com");
-  std::string query = MakeMergeSessionQuery(uber_token, external_cc_result,
-                                            continue_url, source_);
-  net::NetworkTrafficAnnotationTag traffic_annotation =
-      net::DefineNetworkTrafficAnnotation("gaia_auth_merge_sessions", R"(
-        semantics {
-          sender: "Chrome - Google authentication API"
-          description:
-            "This request adds an account to the Google authentication cookies."
-          trigger:
-            "This request is part of Gaia Auth API, and is triggered whenever "
-            "a new Google account is added to the browser."
-          data:
-            "This request includes the user-auth token and sometimes a string "
-            "containing the result of connection checks for various Google web "
-            "properties."
-          destination: GOOGLE_OWNED_SERVICE
-        }
-        policy {
-          cookies_allowed: YES
-          cookies_store: "user"
-          setting:
-            "This feature cannot be disabled in settings, but if the user "
-            "signs out of Chrome, this request would not be made."
-          chrome_policy {
-            SigninAllowed {
-              policy_options {mode: MANDATORY}
-              SigninAllowed: false
-            }
-          }
-        })");
-  CreateAndStartGaiaFetcher(std::string(), std::string(), std::string(),
-                            merge_session_gurl_.Resolve(query),
-                            network::mojom::CredentialsMode::kInclude,
-                            traffic_annotation);
-}
-
-void GaiaAuthFetcher::StartTokenFetchForUberAuthExchange(
-    const std::string& access_token) {
-  DCHECK(!fetch_pending_) << "Tried to fetch two things at once!";
-
-  VLOG(1) << "Starting StartTokenFetchForUberAuthExchange with access_token="
-          << access_token;
-  std::string authentication_header =
-      base::StringPrintf(kOAuthHeaderFormat, access_token.c_str());
-  net::NetworkTrafficAnnotationTag traffic_annotation =
-      net::DefineNetworkTrafficAnnotation("gaia_auth_fetch_for_uber", R"(
-        semantics {
-          sender: "Chrome - Google authentication API"
-          description:
-            "This request exchanges an Oauth2 access token for an uber-auth "
-            "token. This token may be used to add an account to the Google "
-            "authentication cookies."
-          trigger:
-            "This request is part of Gaia Auth API, and is triggered whenever "
-            "a new Google account is added to the browser."
-          data: "This request contains an OAuth 2.0 access token. "
-          destination: GOOGLE_OWNED_SERVICE
-        }
-        policy {
-          cookies_allowed: YES
-          cookies_store: "user"
-          setting:
-            "This feature cannot be disabled in settings, but if the user "
-            "signs out of Chrome, this request would not be made."
-          chrome_policy {
-            SigninAllowed {
-              policy_options {mode: MANDATORY}
-              SigninAllowed: false
-            }
-          }
-        })");
-  CreateAndStartGaiaFetcher(
-      std::string(), std::string(), authentication_header, uberauth_token_gurl_,
-      google_apis::GetOmitCredentialsModeForGaiaRequests(), traffic_annotation);
-}
-
 void GaiaAuthFetcher::StartListAccounts() {
   DCHECK(!fetch_pending_) << "Tried to fetch two things at once!";
 
@@ -943,26 +822,6 @@ void GaiaAuthFetcher::OnReAuthApiInfoFetched(const std::string& data,
   }
 }
 
-void GaiaAuthFetcher::OnMergeSessionFetched(const std::string& data,
-                                            net::Error net_error,
-                                            int response_code) {
-  if (net_error == net::OK && response_code == net::HTTP_OK) {
-    consumer_->OnMergeSessionSuccess(data);
-  } else {
-    consumer_->OnMergeSessionFailure(GenerateAuthError(data, net_error));
-  }
-}
-
-void GaiaAuthFetcher::OnUberAuthTokenFetch(const std::string& data,
-                                           net::Error net_error,
-                                           int response_code) {
-  if (net_error == net::OK && response_code == net::HTTP_OK) {
-    consumer_->OnUberAuthTokenSuccess(data);
-  } else {
-    consumer_->OnUberAuthTokenFailure(GenerateAuthError(data, net_error));
-  }
-}
-
 void GaiaAuthFetcher::OnGetCheckConnectionInfoFetched(const std::string& data,
                                                       net::Error net_error,
                                                       int response_code) {
@@ -1016,16 +875,11 @@ void GaiaAuthFetcher::DispatchFetchedRequest(const GURL& url,
                                              int response_code) {
   if (url == oauth2_token_gurl_) {
     OnOAuth2TokenPairFetched(data, net_error, response_code);
-  } else if (base::StartsWith(url.spec(), merge_session_gurl_.spec(),
-                              base::CompareCase::SENSITIVE)) {
-    OnMergeSessionFetched(data, net_error, response_code);
-  } else if (url == uberauth_token_gurl_) {
-    OnUberAuthTokenFetch(data, net_error, response_code);
   } else if (IsMultiloginUrl(url)) {
     OnOAuthMultiloginFetched(data, net_error, response_code);
   } else if (url == oauth2_revoke_gurl_) {
     OnOAuth2RevokeTokenFetched(data, net_error, response_code);
-  } else if (url == list_accounts_gurl_) {
+  } else if (IsListAccountsUrl(url)) {
     OnListAccountsFetched(data, net_error, response_code);
   } else if (url == logout_gurl_) {
     OnLogOutFetched(data, net_error, response_code);
