@@ -38,7 +38,8 @@ class FakeOAuth2AccessTokenManagerDelegate
   std::unique_ptr<OAuth2AccessTokenFetcher> CreateAccessTokenFetcher(
       const CoreAccountId& account_id,
       scoped_refptr<network::SharedURLLoaderFactory> url_loader_factory,
-      OAuth2AccessTokenConsumer* consumer) override {
+      OAuth2AccessTokenConsumer* consumer,
+      const std::string& token_binding_challenge) override {
     EXPECT_TRUE(base::Contains(account_ids_to_refresh_tokens_, account_id));
     return GaiaAccessTokenFetcher::
         CreateExchangeRefreshTokenForAccessTokenInstance(
