@@ -6,8 +6,10 @@
 
 #include <stddef.h>
 
+#include <optional>
 #include <set>
 #include <string>
+#include <string_view>
 #include <vector>
 
 #include "base/command_line.h"
@@ -28,7 +30,6 @@
 #include "net/base/net_errors.h"
 #include "net/cookies/cookie_constants.h"
 #include "services/network/public/mojom/url_response_head.mojom.h"
-#include "third_party/abseil-cpp/absl/types/optional.h"
 
 namespace {
 
@@ -80,7 +81,7 @@ static GoogleServiceAuthError CreateAuthError(
   if (body)
     response_body = std::move(*body);
 
-  absl::optional<base::Value> value = base::JSONReader::Read(response_body);
+  std::optional<base::Value> value = base::JSONReader::Read(response_body);
   if (!value || !value->is_dict()) {
     int http_response_code = -1;
     if (head && head->headers)
@@ -152,16 +153,16 @@ OAuth2MintTokenFlow::Parameters::Parameters() : mode(MODE_ISSUE_ADVICE) {}
 // static
 OAuth2MintTokenFlow::Parameters
 OAuth2MintTokenFlow::Parameters::CreateForExtensionFlow(
-    base::StringPiece extension_id,
-    base::StringPiece client_id,
-    base::span<const base::StringPiece> scopes,
+    std::string_view extension_id,
+    std::string_view client_id,
+    base::span<const std::string_view> scopes,
     Mode mode,
     bool enable_granular_permissions,
-    base::StringPiece version,
-    base::StringPiece channel,
-    base::StringPiece device_id,
-    base::StringPiece selected_user_id,
-    base::StringPiece consent_result) {
+    std::string_view version,
+    std::string_view channel,
+    std::string_view device_id,
+    std::string_view selected_user_id,
+    std::string_view consent_result) {
   Parameters parameters;
   parameters.extension_id = extension_id;
   parameters.client_id = client_id;
@@ -179,11 +180,11 @@ OAuth2MintTokenFlow::Parameters::CreateForExtensionFlow(
 // static
 OAuth2MintTokenFlow::Parameters
 OAuth2MintTokenFlow::Parameters::CreateForClientFlow(
-    base::StringPiece client_id,
-    base::span<const base::StringPiece> scopes,
-    base::StringPiece version,
-    base::StringPiece channel,
-    base::StringPiece device_id) {
+    std::string_view client_id,
+    base::span<const std::string_view> scopes,
+    std::string_view version,
+    std::string_view channel,
+    std::string_view device_id) {
   Parameters parameters;
   parameters.client_id = client_id;
   parameters.scopes = std::vector<std::string>(scopes.begin(), scopes.end());
@@ -296,7 +297,7 @@ void OAuth2MintTokenFlow::ProcessApiCallSuccess(
   if (body)
     response_body = std::move(*body);
 
-  absl::optional<base::Value> value = base::JSONReader::Read(response_body);
+  std::optional<base::Value> value = base::JSONReader::Read(response_body);
   if (!value || !value->is_dict()) {
     RecordApiCallResult(OAuth2MintTokenApiCallResult::kParseJsonFailure);
     ReportFailure(GoogleServiceAuthError::FromUnexpectedServiceResponse(
@@ -452,8 +453,8 @@ bool OAuth2MintTokenFlow::ParseRemoteConsentResponse(
       const std::string* path = cookie_dict->FindString("path");
       const std::string* max_age_seconds =
           cookie_dict->FindString("maxAgeSeconds");
-      absl::optional<bool> is_secure = cookie_dict->FindBool("isSecure");
-      absl::optional<bool> is_http_only = cookie_dict->FindBool("isHttpOnly");
+      std::optional<bool> is_secure = cookie_dict->FindBool("isSecure");
+      std::optional<bool> is_http_only = cookie_dict->FindBool("isHttpOnly");
       const std::string* same_site = cookie_dict->FindString("sameSite");
 
       int64_t max_age = -1;
@@ -473,8 +474,8 @@ bool OAuth2MintTokenFlow::ParseRemoteConsentResponse(
               is_secure ? *is_secure : false,
               is_http_only ? *is_http_only : false,
               net::StringToCookieSameSite(same_site ? *same_site : ""),
-              net::COOKIE_PRIORITY_DEFAULT, /* same_party */ false,
-              /* partition_key */ absl::nullopt);
+              net::COOKIE_PRIORITY_DEFAULT,
+              /* partition_key */ std::nullopt);
       cookies.push_back(*cookie);
     }
   }
