@@ -7,6 +7,7 @@
 
 #include <set>
 #include <string>
+#include <string_view>
 #include <vector>
 
 #include "base/component_export.h"
@@ -91,23 +92,23 @@ class COMPONENT_EXPORT(GOOGLE_APIS) OAuth2MintTokenFlow
     Parameters();
 
     static Parameters CreateForExtensionFlow(
-        base::StringPiece extension_id,
-        base::StringPiece client_id,
-        base::span<const base::StringPiece> scopes,
+        std::string_view extension_id,
+        std::string_view client_id,
+        base::span<const std::string_view> scopes,
         Mode mode,
         bool enable_granular_permissions,
-        base::StringPiece version,
-        base::StringPiece channel,
-        base::StringPiece device_id = {},
-        base::StringPiece selected_user_id = {},
-        base::StringPiece consent_result = {});
+        std::string_view version,
+        std::string_view channel,
+        std::string_view device_id = {},
+        std::string_view selected_user_id = {},
+        std::string_view consent_result = {});
 
     static Parameters CreateForClientFlow(
-        base::StringPiece client_id,
-        base::span<const base::StringPiece> scopes,
-        base::StringPiece version,
-        base::StringPiece channel,
-        base::StringPiece device_id = {});
+        std::string_view client_id,
+        base::span<const std::string_view> scopes,
+        std::string_view version,
+        std::string_view channel,
+        std::string_view device_id = {});
 
     Parameters(Parameters&& other) noexcept;
     Parameters& operator=(Parameters&& other) noexcept;
