@@ -7,12 +7,13 @@
 #include "google_apis/gaia/oauth2_mint_token_flow.h"
 
 #include <memory>
+#include <optional>
 #include <set>
 #include <string>
+#include <string_view>
 #include <vector>
 
 #include "base/json/json_reader.h"
-#include "base/strings/string_piece.h"
 #include "base/strings/utf_string_conversions.h"
 #include "base/test/metrics/histogram_tester.h"
 #include "base/test/values_test_util.h"
@@ -25,7 +26,6 @@
 #include "services/network/test/test_utils.h"
 #include "testing/gmock/include/gmock/gmock.h"
 #include "testing/gtest/include/gtest/gtest.h"
-#include "third_party/abseil-cpp/absl/types/optional.h"
 
 using testing::_;
 using testing::ByRef;
@@ -106,10 +106,10 @@ const char kTokenBindingChallengeResponse[] = R"(
     }
   )";
 
-constexpr base::StringPiece kVersion = "test_version";
-constexpr base::StringPiece kChannel = "test_channel";
-constexpr base::StringPiece kScopes[] = {"http://scope1", "http://scope2"};
-constexpr base::StringPiece kClientId = "client1";
+constexpr std::string_view kVersion = "test_version";
+constexpr std::string_view kChannel = "test_channel";
+constexpr std::string_view kScopes[] = {"http://scope1", "http://scope2"};
+constexpr std::string_view kClientId = "client1";
 
 static RemoteConsentResolutionData CreateRemoteConsentResolutionData() {
   RemoteConsentResolutionData resolution_data;
@@ -118,14 +118,14 @@ static RemoteConsentResolutionData CreateRemoteConsentResolutionData() {
       *net::CanonicalCookie::CreateSanitizedCookie(
           resolution_data.url, "test_name", "test_value", "test.com", "/",
           base::Time(), base::Time(), base::Time(), false, true,
-          net::CookieSameSite::LAX_MODE, net::COOKIE_PRIORITY_DEFAULT, false,
-          absl::nullopt));
+          net::CookieSameSite::LAX_MODE, net::COOKIE_PRIORITY_DEFAULT,
+          std::nullopt));
   resolution_data.cookies.push_back(
       *net::CanonicalCookie::CreateSanitizedCookie(
           resolution_data.url, "test_name2", "test_value2", "test.com", "/",
           base::Time(), base::Time(), base::Time(), false, false,
-          net::CookieSameSite::UNSPECIFIED, net::COOKIE_PRIORITY_DEFAULT, false,
-          absl::nullopt));
+          net::CookieSameSite::UNSPECIFIED, net::COOKIE_PRIORITY_DEFAULT,
+          std::nullopt));
   return resolution_data;
 }
 
@@ -202,7 +202,7 @@ class OAuth2MintTokenFlowTest : public testing::Test {
                   const std::string& device_id,
                   const std::string& selected_user_id,
                   const std::string& consent_result) {
-    const base::StringPiece kExtensionId = "ext1";
+    const std::string_view kExtensionId = "ext1";
     flow_ = std::make_unique<MockMintTokenFlow>(
         delegate,
         OAuth2MintTokenFlow::Parameters::CreateForExtensionFlow(
@@ -211,7 +211,7 @@ class OAuth2MintTokenFlowTest : public testing::Test {
   }
 
   void CreateClientFlow() {
-    const base::StringPiece kDeviceId = "test_device_id";
+    const std::string_view kDeviceId = "test_device_id";
     flow_ = std::make_unique<MockMintTokenFlow>(
         &delegate_, OAuth2MintTokenFlow::Parameters::CreateForClientFlow(
                         kClientId, kScopes, kVersion, kChannel, kDeviceId));
