@@ -11,5 +11,9 @@ BASE_FEATURE(kGCMDeleteIncomingMessagesWithoutTTL,
              "GCMDeleteIncomingMessagesWithoutTTL",
              base::FEATURE_DISABLED_BY_DEFAULT);
 
+BASE_FEATURE(kGCMAvoidConnectionWhenNetworkUnavailable,
+             "GCMAvoidConnectionWhenNetworkUnavailable",
+             base::FEATURE_DISABLED_BY_DEFAULT);
+
 }  // namespace features
 }  // namespace gcm
