@@ -4,6 +4,7 @@
 
 #include "google_apis/gcm/engine/gcm_store_impl.h"
 
+#include <string_view>
 #include <utility>
 
 #include "base/files/file_path.h"
@@ -16,7 +17,6 @@
 #include "base/metrics/histogram_macros.h"
 #include "base/strings/strcat.h"
 #include "base/strings/string_number_conversions.h"
-#include "base/strings/string_piece.h"
 #include "base/strings/string_tokenizer.h"
 #include "base/strings/string_util.h"
 #include "base/task/sequenced_task_runner.h"
@@ -188,7 +188,7 @@ std::string ParseInstanceIDKey(const std::string& key) {
 // Note: leveldb::Slice keeps a pointer to the data in |s|, which must therefore
 // outlive the slice.
 // For example: MakeSlice(MakeOutgoingKey(x)) is invalid.
-leveldb::Slice MakeSlice(const base::StringPiece& s) {
+leveldb::Slice MakeSlice(std::string_view s) {
   return leveldb::Slice(s.begin(), s.size());
 }
 
@@ -306,6 +306,13 @@ LoadStatus GCMStoreImpl::Backend::OpenStoreAndLoadData(StoreOpenMode open_mode,
   leveldb_env::Options options;
   options.create_if_missing = open_mode == CREATE_IF_MISSING;
   options.paranoid_checks = true;
+
+  // GCMStore does not typically handle large amounts of data, nor a
+  // high rate of store operations, so limit the write log size to something
+  // more appropriate (impact both in-memory and on-disk size before
+  // LevelDB compaction will be triggered).
+  options.write_buffer_size = 128 * 1024;
+
   leveldb::Status status =
       leveldb_env::OpenDB(options, path_.AsUTF8Unsafe(), &db_);
   UMA_HISTOGRAM_ENUMERATION("GCM.Database.Open",
@@ -1020,8 +1027,6 @@ bool GCMStoreImpl::Backend::LoadIncomingMessages(
   if (!expired_incoming_messages.empty()) {
     DVLOG(1) << "Removing " << expired_incoming_messages.size()
              << " expired incoming messages.";
-    UMA_HISTOGRAM_COUNTS_1M("GCM.ExpiredIncomingMessages",
-                            expired_incoming_messages.size());
     RemoveIncomingMessages(expired_incoming_messages, base::DoNothing());
   }
   return true;
