@@ -5,24 +5,24 @@
 #include "google_apis/tasks/tasks_api_response_types.h"
 
 #include <memory>
+#include <optional>
 #include <string>
+#include <string_view>
 
-#include "base/containers/fixed_flat_map.h"
 #include "base/json/json_value_converter.h"
 #include "base/logging.h"
-#include "base/notreached.h"
-#include "base/strings/string_piece.h"
 #include "base/time/time.h"
 #include "base/values.h"
 #include "google_apis/common/parser_util.h"
 #include "google_apis/common/time_util.h"
-#include "third_party/abseil-cpp/absl/types/optional.h"
+#include "google_apis/tasks/tasks_api_task_status.h"
 
 namespace google_apis::tasks {
 namespace {
 
 using ::base::JSONValueConverter;
 
+constexpr char kTaskKind[] = "tasks#task";
 constexpr char kTaskListsKind[] = "tasks#taskLists";
 constexpr char kTasksKind[] = "tasks#tasks";
 
@@ -38,21 +38,13 @@ constexpr char kApiResponseUpdatedKey[] = "updated";
 
 constexpr char kLinkTypeEmail[] = "email";
 
-constexpr char kTaskStatusCompleted[] = "completed";
-constexpr char kTaskStatusNeedsAction[] = "needsAction";
-constexpr auto kTaskStatuses =
-    base::MakeFixedFlatMap<base::StringPiece, Task::Status>(
-        {{kTaskStatusNeedsAction, Task::Status::kNeedsAction},
-         {kTaskStatusCompleted, Task::Status::kCompleted}});
-
-bool ConvertTaskStatus(base::StringPiece input, Task::Status* output) {
-  *output = kTaskStatuses.contains(input) ? kTaskStatuses.at(input)
-                                          : Task::Status::kUnknown;
+bool ConvertTaskStatus(std::string_view input, TaskStatus* output) {
+  *output = TaskStatusFromString(input);
   return true;
 }
 
-bool ConvertTaskDueDate(base::StringPiece input,
-                        absl::optional<base::Time>* output) {
+bool ConvertTaskDueDate(std::string_view input,
+                        std::optional<base::Time>* output) {
   base::Time due;
   if (!util::GetTimeFromString(input, &due)) {
     return false;
@@ -61,7 +53,7 @@ bool ConvertTaskDueDate(base::StringPiece input,
   return true;
 }
 
-bool ConvertTaskLinkType(base::StringPiece input, TaskLink::Type* output) {
+bool ConvertTaskLinkType(std::string_view input, TaskLink::Type* output) {
   *output = input == kLinkTypeEmail ? TaskLink::Type::kEmail
                                     : TaskLink::Type::kUnknown;
   return true;
@@ -125,27 +117,29 @@ Task::~Task() = default;
 void Task::RegisterJSONConverter(JSONValueConverter<Task>* converter) {
   converter->RegisterStringField(kApiResponseIdKey, &Task::id_);
   converter->RegisterStringField(kApiResponseTitleKey, &Task::title_);
-  converter->RegisterCustomField<Status>(kApiResponseStatusKey, &Task::status_,
-                                         &ConvertTaskStatus);
+  converter->RegisterCustomField<TaskStatus>(
+      kApiResponseStatusKey, &Task::status_, &ConvertTaskStatus);
   converter->RegisterStringField(kApiResponseParentKey, &Task::parent_id_);
   converter->RegisterStringField(kApiResponsePositionKey, &Task::position_);
-  converter->RegisterCustomField<absl::optional<base::Time>>(
+  converter->RegisterCustomField<std::optional<base::Time>>(
       kApiResponseDueKey, &Task::due_, &ConvertTaskDueDate);
   converter->RegisterRepeatedMessage<TaskLink>(kApiResponseLinksKey,
                                                &Task::links_);
   converter->RegisterStringField(kApiResponseNotesKey, &Task::notes_);
+  converter->RegisterCustomField<base::Time>(
+      kApiResponseUpdatedKey, &Task::updated_, &util::GetTimeFromString);
 }
 
 // static
-std::string Task::StatusToString(Status status) {
-  switch (status) {
-    case Status::kCompleted:
-      return kTaskStatusCompleted;
-    case Status::kNeedsAction:
-      return kTaskStatusNeedsAction;
-    default:
-      NOTREACHED_NORETURN();
+std::unique_ptr<Task> Task::CreateFrom(const base::Value& value) {
+  auto task = std::make_unique<Task>();
+  JSONValueConverter<Task> converter;
+  if (!IsResourceKindExpected(value, kTaskKind) ||
+      !converter.Convert(value, task.get())) {
+    DVLOG(1) << "Unable to construct a `Task` from parsed json.";
+    return nullptr;
   }
+  return task;
 }
 
 // ----- Tasks -----
