@@ -9,8 +9,9 @@
 #include <string>
 #include <vector>
 
+#include <optional>
 #include "base/time/time.h"
-#include "third_party/abseil-cpp/absl/types/optional.h"
+#include "google_apis/tasks/tasks_api_task_status.h"
 
 namespace base {
 template <class StructType>
@@ -105,13 +106,6 @@ class TaskLink {
 // https://developers.google.com/tasks/reference/rest/v1/tasks
 class Task {
  public:
-  // Status of the task.
-  enum class Status {
-    kUnknown,
-    kNeedsAction,
-    kCompleted,
-  };
-
   Task();
   Task(const Task&) = delete;
   Task& operator=(const Task&) = delete;
@@ -121,17 +115,18 @@ class Task {
   // class.
   static void RegisterJSONConverter(base::JSONValueConverter<Task>* converter);
 
-  // Stringifies `Status` enum value.
-  static std::string StatusToString(Status);
+  // Creates a `Task` from parsed JSON.
+  static std::unique_ptr<Task> CreateFrom(const base::Value& value);
 
   const std::string& id() const { return id_; }
   const std::string& title() const { return title_; }
-  Status status() const { return status_; }
+  TaskStatus status() const { return status_; }
   const std::string& parent_id() const { return parent_id_; }
   const std::string& position() const { return position_; }
-  const absl::optional<base::Time>& due() const { return due_; }
+  const std::optional<base::Time>& due() const { return due_; }
   const std::vector<std::unique_ptr<TaskLink>>& links() const { return links_; }
   const std::string& notes() const { return notes_; }
+  const base::Time& updated() const { return updated_; }
 
  private:
   // Task identifier.
@@ -141,7 +136,7 @@ class Task {
   std::string title_;
 
   // Status of the task.
-  Status status_ = Status::kUnknown;
+  TaskStatus status_ = TaskStatus::kUnknown;
 
   // Parent task identifier.
   std::string parent_id_;
@@ -152,13 +147,16 @@ class Task {
   // Due date of the task (comes as a RFC 3339 timestamp and converted to
   // `base::Time`). The due date only records date information. Not all tasks
   // have a due date.
-  absl::optional<base::Time> due_ = absl::nullopt;
+  std::optional<base::Time> due_ = std::nullopt;
 
   // Collection of links related to this task.
   std::vector<std::unique_ptr<TaskLink>> links_;
 
   // Notes describing the task.
   std::string notes_;
+
+  // When the task was last updated.
+  base::Time updated_;
 };
 
 // Container for multiple `Task`s.
