@@ -8,6 +8,7 @@
 
 #include "base/json/json_reader.h"
 #include "google_apis/common/time_util.h"
+#include "google_apis/tasks/tasks_api_task_status.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
 namespace google_apis::tasks {
@@ -87,7 +88,8 @@ TEST(TasksApiResponseTypesTest, CreatesTasksFromResponse) {
             "title": "Completed child task",
             "parent": "asd",
             "position": "00000000000000000000",
-            "status": "completed"
+            "status": "completed",
+            "updated": "2023-01-30T22:19:22.812Z"
           },
           {
             "id": "asd",
@@ -95,7 +97,8 @@ TEST(TasksApiResponseTypesTest, CreatesTasksFromResponse) {
             "position": "00000000000000000001",
             "status": "needsAction",
             "due": "2023-04-19T00:00:00.000Z",
-            "notes": "Lorem ipsum dolor sit amet"
+            "notes": "Lorem ipsum dolor sit amet",
+            "updated": "2022-12-21T23:38:22.590Z"
           }
         ]
       })");
@@ -108,20 +111,24 @@ TEST(TasksApiResponseTypesTest, CreatesTasksFromResponse) {
 
   EXPECT_EQ(tasks->items()[0]->id(), "qwe");
   EXPECT_EQ(tasks->items()[0]->title(), "Completed child task");
-  EXPECT_EQ(tasks->items()[0]->status(), Task::Status::kCompleted);
+  EXPECT_EQ(tasks->items()[0]->status(), TaskStatus::kCompleted);
   EXPECT_EQ(tasks->items()[0]->parent_id(), "asd");
   EXPECT_EQ(tasks->items()[0]->position(), "00000000000000000000");
   EXPECT_FALSE(tasks->items()[0]->due());
   EXPECT_TRUE(tasks->items()[0]->notes().empty());
+  EXPECT_EQ(util::FormatTimeAsString(tasks->items()[0]->updated()),
+            "2023-01-30T22:19:22.812Z");
 
   EXPECT_EQ(tasks->items()[1]->id(), "asd");
   EXPECT_EQ(tasks->items()[1]->title(), "Parent task");
-  EXPECT_EQ(tasks->items()[1]->status(), Task::Status::kNeedsAction);
+  EXPECT_EQ(tasks->items()[1]->status(), TaskStatus::kNeedsAction);
   EXPECT_TRUE(tasks->items()[1]->parent_id().empty());
   EXPECT_EQ(tasks->items()[1]->position(), "00000000000000000001");
   EXPECT_EQ(util::FormatTimeAsString(tasks->items()[1]->due().value()),
             "2023-04-19T00:00:00.000Z");
   EXPECT_EQ(tasks->items()[1]->notes(), "Lorem ipsum dolor sit amet");
+  EXPECT_EQ(util::FormatTimeAsString(tasks->items()[1]->updated()),
+            "2022-12-21T23:38:22.590Z");
 }
 
 TEST(TasksApiResponseTypesTest, CreatesTasksWithNextPageTokenFromResponse) {
@@ -138,11 +145,6 @@ TEST(TasksApiResponseTypesTest, CreatesTasksWithNextPageTokenFromResponse) {
   EXPECT_EQ(tasks->next_page_token(), "qwerty");
 }
 
-TEST(TasksApiResponseTypesTest, ConvertsTaskStatusToString) {
-  EXPECT_EQ(Task::StatusToString(Task::Status::kCompleted), "completed");
-  EXPECT_EQ(Task::StatusToString(Task::Status::kNeedsAction), "needsAction");
-}
-
 TEST(TasksApiResponseTypesTest, ConvertsTaskLinks) {
   const auto raw_tasks = JSONReader::Read(R"(
       {
@@ -183,4 +185,42 @@ TEST(TasksApiResponseTypesTest, FailsToCreateTasksFromInvalidResponse) {
   ASSERT_FALSE(tasks);
 }
 
+TEST(TasksApiResponseTypesTest, CreatesTaskFromResponse) {
+  const auto raw_task = JSONReader::Read(R"(
+      {
+        "kind": "tasks#task",
+        "id": "asd",
+        "title": "Parent task",
+        "position": "00000000000000000001",
+        "status": "needsAction",
+        "due": "2023-04-19T00:00:00.000Z",
+        "notes": "Lorem ipsum dolor sit amet"
+      })");
+  ASSERT_TRUE(raw_task);
+
+  const auto task = Task::CreateFrom(*raw_task);
+  ASSERT_TRUE(task);
+
+  EXPECT_EQ(task->id(), "asd");
+  EXPECT_EQ(task->title(), "Parent task");
+  EXPECT_EQ(task->status(), TaskStatus::kNeedsAction);
+  EXPECT_TRUE(task->parent_id().empty());
+  EXPECT_EQ(task->position(), "00000000000000000001");
+  EXPECT_EQ(util::FormatTimeAsString(task->due().value()),
+            "2023-04-19T00:00:00.000Z");
+  EXPECT_EQ(task->notes(), "Lorem ipsum dolor sit amet");
+}
+
+TEST(TasksApiResponseTypesTest, FailsToCreateTaskFromInvalidResponse) {
+  const auto raw_task = JSONReader::Read(R"(
+      {
+        "kind": "invalid_kind",
+        "id": true
+      })");
+  ASSERT_TRUE(raw_task);
+
+  const auto task = Task::CreateFrom(*raw_task);
+  ASSERT_FALSE(task);
+}
+
 }  // namespace google_apis::tasks
