@@ -5,6 +5,8 @@
 #include <stddef.h>
 #include <stdint.h>
 
+#include <string_view>
+
 #include "base/memory/scoped_refptr.h"
 #include "net/cert/ct_serialization.h"
 #include "net/cert/signed_certificate_timestamp.h"
@@ -14,7 +16,7 @@ using net::ct::SignedCertificateTimestamp;
 
 extern "C" int LLVMFuzzerTestOneInput(const uint8_t* data, size_t size) {
   scoped_refptr<SignedCertificateTimestamp> sct;
-  base::StringPiece buffer(reinterpret_cast<const char*>(data), size);
+  std::string_view buffer(reinterpret_cast<const char*>(data), size);
   DecodeSignedCertificateTimestamp(&buffer, &sct);
   return 0;
 }
