@@ -2,12 +2,15 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-#include "net/cert/pki/cert_issuer_source_sync_unittest.h"
+#include "net/cert/internal/cert_issuer_source_sync_unittest.h"
 
 namespace net {
 
 // This suite is only instantiated when NSS is used.
 GTEST_ALLOW_UNINSTANTIATED_PARAMETERIZED_TEST(
     CertIssuerSourceSyncNotNormalizedTest);
+GTEST_ALLOW_UNINSTANTIATED_PARAMETERIZED_TEST(
+    CertIssuerSourceSyncNormalizationTest);
+GTEST_ALLOW_UNINSTANTIATED_PARAMETERIZED_TEST(CertIssuerSourceSyncTest);
 
-}  // namespace net
\ No newline at end of file
+}  // namespace net
