@@ -4,16 +4,18 @@
 
 #include "net/cert/internal/revocation_checker.h"
 
+#include <string_view>
+
 #include "base/time/time.h"
 #include "net/cert/mock_cert_net_fetcher.h"
-#include "net/cert/pki/cert_errors.h"
-#include "net/cert/pki/common_cert_errors.h"
-#include "net/cert/pki/parse_certificate.h"
-#include "net/cert/pki/parsed_certificate.h"
 #include "net/test/cert_builder.h"
 #include "net/test/revocation_builder.h"
 #include "testing/gmock/include/gmock/gmock.h"
 #include "testing/gtest/include/gtest/gtest.h"
+#include "third_party/boringssl/src/pki/cert_errors.h"
+#include "third_party/boringssl/src/pki/common_cert_errors.h"
+#include "third_party/boringssl/src/pki/parse_certificate.h"
+#include "third_party/boringssl/src/pki/parsed_certificate.h"
 #include "url/gurl.h"
 
 namespace net {
@@ -27,9 +29,9 @@ using ::testing::Return;
 using ::testing::StrictMock;
 
 bool AddCertsToList(std::vector<CertBuilder*> builders,
-                    ParsedCertificateList* out_certs) {
+                    bssl::ParsedCertificateList* out_certs) {
   for (auto* builder : builders) {
-    if (!ParsedCertificate::CreateAndAddToVector(
+    if (!bssl::ParsedCertificate::CreateAndAddToVector(
             builder->DupCertBuffer(), {}, out_certs, /*errors=*/nullptr)) {
       return false;
     }
@@ -40,7 +42,7 @@ bool AddCertsToList(std::vector<CertBuilder*> builders,
 TEST(RevocationChecker, NoRevocationMechanism) {
   auto [leaf, root] = CertBuilder::CreateSimpleChain2();
 
-  ParsedCertificateList chain;
+  bssl::ParsedCertificateList chain;
   ASSERT_TRUE(AddCertsToList({leaf.get(), root.get()}, &chain));
 
   RevocationPolicy policy;
@@ -56,14 +58,15 @@ TEST(RevocationChecker, NoRevocationMechanism) {
     // No methods on |mock_fetcher| should be called.
     auto mock_fetcher = base::MakeRefCounted<StrictMock<MockCertNetFetcher>>();
 
-    CertPathErrors errors;
+    bssl::CertPathErrors errors;
     CheckValidatedChainRevocation(
         chain, policy, /*deadline=*/base::TimeTicks(),
-        /*stapled_leaf_ocsp_response=*/base::StringPiece(), mock_fetcher.get(),
+        /*stapled_leaf_ocsp_response=*/std::string_view(), mock_fetcher.get(),
         &errors, /*stapled_ocsp_verify_result=*/nullptr);
 
     EXPECT_TRUE(errors.ContainsHighSeverityErrors());
-    EXPECT_TRUE(errors.ContainsError(cert_errors::kNoRevocationMechanism));
+    EXPECT_TRUE(
+        errors.ContainsError(bssl::cert_errors::kNoRevocationMechanism));
   }
 
   {
@@ -73,10 +76,10 @@ TEST(RevocationChecker, NoRevocationMechanism) {
     // No methods on |mock_fetcher| should be called.
     auto mock_fetcher = base::MakeRefCounted<StrictMock<MockCertNetFetcher>>();
 
-    CertPathErrors errors;
+    bssl::CertPathErrors errors;
     CheckValidatedChainRevocation(
         chain, policy, /*deadline=*/base::TimeTicks(),
-        /*stapled_leaf_ocsp_response=*/base::StringPiece(), mock_fetcher.get(),
+        /*stapled_leaf_ocsp_response=*/std::string_view(), mock_fetcher.get(),
         &errors, /*stapled_ocsp_verify_result=*/nullptr);
 
     EXPECT_FALSE(errors.ContainsHighSeverityErrors());
@@ -92,10 +95,10 @@ TEST(RevocationChecker, NoRevocationMechanism) {
     // No methods on |mock_fetcher| should be called.
     auto mock_fetcher = base::MakeRefCounted<StrictMock<MockCertNetFetcher>>();
 
-    CertPathErrors errors;
+    bssl::CertPathErrors errors;
     CheckValidatedChainRevocation(
         chain, policy, /*deadline=*/base::TimeTicks(),
-        /*stapled_leaf_ocsp_response=*/base::StringPiece(), mock_fetcher.get(),
+        /*stapled_leaf_ocsp_response=*/std::string_view(), mock_fetcher.get(),
         &errors, /*stapled_ocsp_verify_result=*/nullptr);
 
     EXPECT_FALSE(errors.ContainsHighSeverityErrors());
@@ -108,7 +111,7 @@ TEST(RevocationChecker, ValidCRL) {
   const GURL kTestCrlUrl("http://example.com/crl1");
   leaf->SetCrlDistributionPointUrl(kTestCrlUrl);
 
-  ParsedCertificateList chain;
+  bssl::ParsedCertificateList chain;
   ASSERT_TRUE(AddCertsToList({leaf.get(), root.get()}, &chain));
 
   RevocationPolicy policy;
@@ -130,10 +133,10 @@ TEST(RevocationChecker, ValidCRL) {
     EXPECT_CALL(*mock_fetcher, FetchCrl(kTestCrlUrl, _, _))
         .WillOnce(Return(ByMove(MockCertNetFetcherRequest::Create(crl_data))));
 
-    CertPathErrors errors;
+    bssl::CertPathErrors errors;
     CheckValidatedChainRevocation(
         chain, policy, /*deadline=*/base::TimeTicks(),
-        /*stapled_leaf_ocsp_response=*/base::StringPiece(), mock_fetcher.get(),
+        /*stapled_leaf_ocsp_response=*/std::string_view(), mock_fetcher.get(),
         &errors, /*stapled_ocsp_verify_result=*/nullptr);
 
     EXPECT_FALSE(errors.ContainsHighSeverityErrors());
@@ -146,14 +149,15 @@ TEST(RevocationChecker, ValidCRL) {
     // No methods on |mock_fetcher| should be called.
     auto mock_fetcher = base::MakeRefCounted<StrictMock<MockCertNetFetcher>>();
 
-    CertPathErrors errors;
+    bssl::CertPathErrors errors;
     CheckValidatedChainRevocation(
         chain, policy, /*deadline=*/base::TimeTicks(),
-        /*stapled_leaf_ocsp_response=*/base::StringPiece(), mock_fetcher.get(),
+        /*stapled_leaf_ocsp_response=*/std::string_view(), mock_fetcher.get(),
         &errors, /*stapled_ocsp_verify_result=*/nullptr);
 
     EXPECT_TRUE(errors.ContainsHighSeverityErrors());
-    EXPECT_TRUE(errors.ContainsError(cert_errors::kUnableToCheckRevocation));
+    EXPECT_TRUE(
+        errors.ContainsError(bssl::cert_errors::kUnableToCheckRevocation));
   }
 
   {
@@ -163,16 +167,17 @@ TEST(RevocationChecker, ValidCRL) {
     // No methods on |mock_fetcher| should be called.
     auto mock_fetcher = base::MakeRefCounted<StrictMock<MockCertNetFetcher>>();
 
-    CertPathErrors errors;
+    bssl::CertPathErrors errors;
     CheckValidatedChainRevocation(
         chain, policy, /*deadline=*/base::TimeTicks(),
-        /*stapled_leaf_ocsp_response=*/base::StringPiece(), mock_fetcher.get(),
+        /*stapled_leaf_ocsp_response=*/std::string_view(), mock_fetcher.get(),
         &errors, /*stapled_ocsp_verify_result=*/nullptr);
 
     EXPECT_TRUE(errors.ContainsHighSeverityErrors());
     // Since CRLs were not considered, the error should be "no revocation
     // mechanism".
-    EXPECT_TRUE(errors.ContainsError(cert_errors::kNoRevocationMechanism));
+    EXPECT_TRUE(
+        errors.ContainsError(bssl::cert_errors::kNoRevocationMechanism));
   }
 }
 
@@ -182,7 +187,7 @@ TEST(RevocationChecker, RevokedCRL) {
   const GURL kTestCrlUrl("http://example.com/crl1");
   leaf->SetCrlDistributionPointUrl(kTestCrlUrl);
 
-  ParsedCertificateList chain;
+  bssl::ParsedCertificateList chain;
   ASSERT_TRUE(AddCertsToList({leaf.get(), root.get()}, &chain));
 
   RevocationPolicy policy;
@@ -205,14 +210,14 @@ TEST(RevocationChecker, RevokedCRL) {
     EXPECT_CALL(*mock_fetcher, FetchCrl(kTestCrlUrl, _, _))
         .WillOnce(Return(ByMove(MockCertNetFetcherRequest::Create(crl_data))));
 
-    CertPathErrors errors;
+    bssl::CertPathErrors errors;
     CheckValidatedChainRevocation(
         chain, policy, /*deadline=*/base::TimeTicks(),
-        /*stapled_leaf_ocsp_response=*/base::StringPiece(), mock_fetcher.get(),
+        /*stapled_leaf_ocsp_response=*/std::string_view(), mock_fetcher.get(),
         &errors, /*stapled_ocsp_verify_result=*/nullptr);
 
     EXPECT_TRUE(errors.ContainsHighSeverityErrors());
-    EXPECT_TRUE(errors.ContainsError(cert_errors::kCertificateRevoked));
+    EXPECT_TRUE(errors.ContainsError(bssl::cert_errors::kCertificateRevoked));
   }
 
   {
@@ -224,14 +229,14 @@ TEST(RevocationChecker, RevokedCRL) {
     EXPECT_CALL(*mock_fetcher, FetchCrl(kTestCrlUrl, _, _))
         .WillOnce(Return(ByMove(MockCertNetFetcherRequest::Create(crl_data))));
 
-    CertPathErrors errors;
+    bssl::CertPathErrors errors;
     CheckValidatedChainRevocation(
         chain, policy, /*deadline=*/base::TimeTicks(),
-        /*stapled_leaf_ocsp_response=*/base::StringPiece(), mock_fetcher.get(),
+        /*stapled_leaf_ocsp_response=*/std::string_view(), mock_fetcher.get(),
         &errors, /*stapled_ocsp_verify_result=*/nullptr);
 
     EXPECT_TRUE(errors.ContainsHighSeverityErrors());
-    EXPECT_TRUE(errors.ContainsError(cert_errors::kCertificateRevoked));
+    EXPECT_TRUE(errors.ContainsError(bssl::cert_errors::kCertificateRevoked));
   }
 }
 
@@ -241,7 +246,7 @@ TEST(RevocationChecker, CRLRequestFails) {
   const GURL kTestCrlUrl("http://example.com/crl1");
   leaf->SetCrlDistributionPointUrl(kTestCrlUrl);
 
-  ParsedCertificateList chain;
+  bssl::ParsedCertificateList chain;
   ASSERT_TRUE(AddCertsToList({leaf.get(), root.get()}, &chain));
 
   RevocationPolicy policy;
@@ -258,14 +263,15 @@ TEST(RevocationChecker, CRLRequestFails) {
         .WillOnce(Return(
             ByMove(MockCertNetFetcherRequest::Create(ERR_CONNECTION_FAILED))));
 
-    CertPathErrors errors;
+    bssl::CertPathErrors errors;
     CheckValidatedChainRevocation(
         chain, policy, /*deadline=*/base::TimeTicks(),
-        /*stapled_leaf_ocsp_response=*/base::StringPiece(), mock_fetcher.get(),
+        /*stapled_leaf_ocsp_response=*/std::string_view(), mock_fetcher.get(),
         &errors, /*stapled_ocsp_verify_result=*/nullptr);
 
     EXPECT_TRUE(errors.ContainsHighSeverityErrors());
-    EXPECT_TRUE(errors.ContainsError(cert_errors::kUnableToCheckRevocation));
+    EXPECT_TRUE(
+        errors.ContainsError(bssl::cert_errors::kUnableToCheckRevocation));
   }
 
   {
@@ -277,14 +283,15 @@ TEST(RevocationChecker, CRLRequestFails) {
         .WillOnce(Return(
             ByMove(MockCertNetFetcherRequest::Create(ERR_CONNECTION_FAILED))));
 
-    CertPathErrors errors;
+    bssl::CertPathErrors errors;
     CheckValidatedChainRevocation(
         chain, policy, /*deadline=*/base::TimeTicks(),
-        /*stapled_leaf_ocsp_response=*/base::StringPiece(), mock_fetcher.get(),
+        /*stapled_leaf_ocsp_response=*/std::string_view(), mock_fetcher.get(),
         &errors, /*stapled_ocsp_verify_result=*/nullptr);
 
     EXPECT_TRUE(errors.ContainsHighSeverityErrors());
-    EXPECT_TRUE(errors.ContainsError(cert_errors::kUnableToCheckRevocation));
+    EXPECT_TRUE(
+        errors.ContainsError(bssl::cert_errors::kUnableToCheckRevocation));
   }
 
   {
@@ -296,10 +303,10 @@ TEST(RevocationChecker, CRLRequestFails) {
         .WillOnce(Return(
             ByMove(MockCertNetFetcherRequest::Create(ERR_CONNECTION_FAILED))));
 
-    CertPathErrors errors;
+    bssl::CertPathErrors errors;
     CheckValidatedChainRevocation(
         chain, policy, /*deadline=*/base::TimeTicks(),
-        /*stapled_leaf_ocsp_response=*/base::StringPiece(), mock_fetcher.get(),
+        /*stapled_leaf_ocsp_response=*/std::string_view(), mock_fetcher.get(),
         &errors, /*stapled_ocsp_verify_result=*/nullptr);
 
     EXPECT_FALSE(errors.ContainsHighSeverityErrors());
@@ -312,7 +319,7 @@ TEST(RevocationChecker, CRLNonHttpUrl) {
   const GURL kTestCrlUrl("https://example.com/crl1");
   leaf->SetCrlDistributionPointUrl(kTestCrlUrl);
 
-  ParsedCertificateList chain;
+  bssl::ParsedCertificateList chain;
   ASSERT_TRUE(AddCertsToList({leaf.get(), root.get()}, &chain));
 
   RevocationPolicy policy;
@@ -325,14 +332,14 @@ TEST(RevocationChecker, CRLNonHttpUrl) {
   // HTTPS CRL URLs should not be fetched.
   auto mock_fetcher = base::MakeRefCounted<StrictMock<MockCertNetFetcher>>();
 
-  CertPathErrors errors;
+  bssl::CertPathErrors errors;
   CheckValidatedChainRevocation(
       chain, policy, /*deadline=*/base::TimeTicks(),
-      /*stapled_leaf_ocsp_response=*/base::StringPiece(), mock_fetcher.get(),
+      /*stapled_leaf_ocsp_response=*/std::string_view(), mock_fetcher.get(),
       &errors, /*stapled_ocsp_verify_result=*/nullptr);
 
   EXPECT_TRUE(errors.ContainsHighSeverityErrors());
-  EXPECT_TRUE(errors.ContainsError(cert_errors::kNoRevocationMechanism));
+  EXPECT_TRUE(errors.ContainsError(bssl::cert_errors::kNoRevocationMechanism));
 }
 
 TEST(RevocationChecker, SkipEntireInvalidCRLDistributionPoints) {
@@ -347,7 +354,7 @@ TEST(RevocationChecker, SkipEntireInvalidCRLDistributionPoints) {
   //   SEQUENCE {
   //     [0] {
   //       [0] {
-  //         # [9] is not a valid tag in GeneralNames
+  //         # [9] is not a valid tag in bssl::GeneralNames
   //         [9 PRIMITIVE] { "foo" }
   //       }
   //     }
@@ -369,10 +376,10 @@ TEST(RevocationChecker, SkipEntireInvalidCRLDistributionPoints) {
                            0x70, 0x6c, 0x65, 0x2e, 0x63, 0x6f, 0x6d, 0x2f, 0x62,
                            0x61, 0x72, 0x2e, 0x63, 0x72, 0x6c};
   leaf->SetExtension(
-      der::Input(kCrlDistributionPointsOid),
+      bssl::der::Input(bssl::kCrlDistributionPointsOid),
       std::string(reinterpret_cast<const char*>(crldp), std::size(crldp)));
 
-  ParsedCertificateList chain;
+  bssl::ParsedCertificateList chain;
   ASSERT_TRUE(AddCertsToList({leaf.get(), root.get()}, &chain));
 
   RevocationPolicy policy;
@@ -391,16 +398,16 @@ TEST(RevocationChecker, SkipEntireInvalidCRLDistributionPoints) {
   // No methods on |mock_fetcher| should be called.
   auto mock_fetcher = base::MakeRefCounted<StrictMock<MockCertNetFetcher>>();
 
-  CertPathErrors errors;
+  bssl::CertPathErrors errors;
   CheckValidatedChainRevocation(
       chain, policy, /*deadline=*/base::TimeTicks(),
-      /*stapled_leaf_ocsp_response=*/base::StringPiece(), mock_fetcher.get(),
+      /*stapled_leaf_ocsp_response=*/std::string_view(), mock_fetcher.get(),
       &errors, /*stapled_ocsp_verify_result=*/nullptr);
 
   // Should fail since the entire cRLDistributionPoints extension was skipped
   // and no other revocation method is present.
   EXPECT_TRUE(errors.ContainsHighSeverityErrors());
-  EXPECT_TRUE(errors.ContainsError(cert_errors::kNoRevocationMechanism));
+  EXPECT_TRUE(errors.ContainsError(bssl::cert_errors::kNoRevocationMechanism));
 }
 
 TEST(RevocationChecker, SkipUnsupportedCRLDistPointWithNonUriFullname) {
@@ -454,10 +461,10 @@ TEST(RevocationChecker, SkipUnsupportedCRLDistPointWithNonUriFullname) {
       0x77, 0x77, 0x2e, 0x65, 0x78, 0x61, 0x6d, 0x70, 0x6c, 0x65, 0x2e,
       0x63, 0x6f, 0x6d, 0x2f, 0x62, 0x61, 0x72, 0x2e, 0x63, 0x72, 0x6c};
   leaf->SetExtension(
-      der::Input(kCrlDistributionPointsOid),
+      bssl::der::Input(bssl::kCrlDistributionPointsOid),
       std::string(reinterpret_cast<const char*>(crldp), std::size(crldp)));
 
-  ParsedCertificateList chain;
+  bssl::ParsedCertificateList chain;
   ASSERT_TRUE(AddCertsToList({leaf.get(), root.get()}, &chain));
 
   RevocationPolicy policy;
@@ -478,10 +485,10 @@ TEST(RevocationChecker, SkipUnsupportedCRLDistPointWithNonUriFullname) {
   EXPECT_CALL(*mock_fetcher, FetchCrl(kSecondCrlUrl, _, _))
       .WillOnce(Return(ByMove(MockCertNetFetcherRequest::Create(crl_data))));
 
-  CertPathErrors errors;
+  bssl::CertPathErrors errors;
   CheckValidatedChainRevocation(
       chain, policy, /*deadline=*/base::TimeTicks(),
-      /*stapled_leaf_ocsp_response=*/base::StringPiece(), mock_fetcher.get(),
+      /*stapled_leaf_ocsp_response=*/std::string_view(), mock_fetcher.get(),
       &errors, /*stapled_ocsp_verify_result=*/nullptr);
 
   EXPECT_FALSE(errors.ContainsHighSeverityErrors());
@@ -523,10 +530,10 @@ TEST(RevocationChecker, SkipUnsupportedCRLDistPointWithReasons) {
       0x77, 0x77, 0x2e, 0x65, 0x78, 0x61, 0x6d, 0x70, 0x6c, 0x65, 0x2e, 0x63,
       0x6f, 0x6d, 0x2f, 0x62, 0x61, 0x72, 0x2e, 0x63, 0x72, 0x6c};
   leaf->SetExtension(
-      der::Input(kCrlDistributionPointsOid),
+      bssl::der::Input(bssl::kCrlDistributionPointsOid),
       std::string(reinterpret_cast<const char*>(crldp), std::size(crldp)));
 
-  ParsedCertificateList chain;
+  bssl::ParsedCertificateList chain;
   ASSERT_TRUE(AddCertsToList({leaf.get(), root.get()}, &chain));
 
   RevocationPolicy policy;
@@ -547,10 +554,10 @@ TEST(RevocationChecker, SkipUnsupportedCRLDistPointWithReasons) {
   EXPECT_CALL(*mock_fetcher, FetchCrl(kSecondCrlUrl, _, _))
       .WillOnce(Return(ByMove(MockCertNetFetcherRequest::Create(crl_data))));
 
-  CertPathErrors errors;
+  bssl::CertPathErrors errors;
   CheckValidatedChainRevocation(
       chain, policy, /*deadline=*/base::TimeTicks(),
-      /*stapled_leaf_ocsp_response=*/base::StringPiece(), mock_fetcher.get(),
+      /*stapled_leaf_ocsp_response=*/std::string_view(), mock_fetcher.get(),
       &errors, /*stapled_ocsp_verify_result=*/nullptr);
 
   EXPECT_FALSE(errors.ContainsHighSeverityErrors());
@@ -624,10 +631,10 @@ TEST(RevocationChecker, SkipUnsupportedCRLDistPointWithCrlIssuer) {
       0x77, 0x77, 0x77, 0x2e, 0x65, 0x78, 0x61, 0x6d, 0x70, 0x6c, 0x65, 0x2e,
       0x63, 0x6f, 0x6d, 0x2f, 0x62, 0x61, 0x72, 0x2e, 0x63, 0x72, 0x6c};
   leaf->SetExtension(
-      der::Input(kCrlDistributionPointsOid),
+      bssl::der::Input(bssl::kCrlDistributionPointsOid),
       std::string(reinterpret_cast<const char*>(crldp), std::size(crldp)));
 
-  ParsedCertificateList chain;
+  bssl::ParsedCertificateList chain;
   ASSERT_TRUE(AddCertsToList({leaf.get(), root.get()}, &chain));
 
   RevocationPolicy policy;
@@ -648,10 +655,10 @@ TEST(RevocationChecker, SkipUnsupportedCRLDistPointWithCrlIssuer) {
   EXPECT_CALL(*mock_fetcher, FetchCrl(kSecondCrlUrl, _, _))
       .WillOnce(Return(ByMove(MockCertNetFetcherRequest::Create(crl_data))));
 
-  CertPathErrors errors;
+  bssl::CertPathErrors errors;
   CheckValidatedChainRevocation(
       chain, policy, /*deadline=*/base::TimeTicks(),
-      /*stapled_leaf_ocsp_response=*/base::StringPiece(), mock_fetcher.get(),
+      /*stapled_leaf_ocsp_response=*/std::string_view(), mock_fetcher.get(),
       &errors, /*stapled_ocsp_verify_result=*/nullptr);
 
   EXPECT_FALSE(errors.ContainsHighSeverityErrors());
