@@ -12,10 +12,7 @@
 #include "net/cert/internal/system_trust_store_nss.h"
 #endif  // BUILDFLAG(USE_NSS_CERTS)
 
-#if BUILDFLAG(USE_NSS_CERTS)
-#include <cert.h>
-#include <pk11pub.h>
-#elif BUILDFLAG(IS_MAC)
+#if BUILDFLAG(IS_MAC)
 #include <Security/Security.h>
 #endif
 
@@ -28,18 +25,15 @@
 #include "base/task/task_traits.h"
 #include "base/task/thread_pool.h"
 #include "build/build_config.h"
-#include "net/cert/pki/cert_errors.h"
-#include "net/cert/pki/parsed_certificate.h"
-#include "net/cert/pki/trust_store_collection.h"
-#include "net/cert/pki/trust_store_in_memory.h"
 #include "net/cert/x509_certificate.h"
 #include "net/cert/x509_util.h"
+#include "third_party/boringssl/src/pki/cert_errors.h"
+#include "third_party/boringssl/src/pki/parsed_certificate.h"
+#include "third_party/boringssl/src/pki/trust_store_collection.h"
+#include "third_party/boringssl/src/pki/trust_store_in_memory.h"
 
 #if BUILDFLAG(USE_NSS_CERTS)
-#include "crypto/nss_util.h"
 #include "net/cert/internal/trust_store_nss.h"
-#include "net/cert/known_roots_nss.h"
-#include "net/cert/scoped_nss_types.h"
 #elif BUILDFLAG(IS_MAC)
 #include "net/base/features.h"
 #include "net/cert/internal/trust_store_mac.h"
@@ -58,29 +52,38 @@
 
 namespace net {
 
-namespace {
-
-class DummySystemTrustStore : public SystemTrustStore {
+#if BUILDFLAG(CHROME_ROOT_STORE_SUPPORTED)
+class SystemTrustStoreChromeOnly : public SystemTrustStore {
  public:
-  TrustStore* GetTrustStore() override { return &trust_store_; }
+  // Creates a SystemTrustStore that gets publicly trusted roots from
+  // |trust_store_chrome|.
+  explicit SystemTrustStoreChromeOnly(
+      std::unique_ptr<TrustStoreChrome> trust_store_chrome)
+      : trust_store_chrome_(std::move(trust_store_chrome)) {}
 
-  bool UsesSystemTrustStore() const override { return false; }
-
-  bool IsKnownRoot(const ParsedCertificate* trust_anchor) const override {
-    return false;
+  bssl::TrustStore* GetTrustStore() override {
+    return trust_store_chrome_.get();
   }
 
-#if BUILDFLAG(CHROME_ROOT_STORE_SUPPORTED)
-  int64_t chrome_root_store_version() override { return 0; }
-#endif
+  // IsKnownRoot returns true if the given trust anchor is a standard one (as
+  // opposed to a user-installed root)
+  bool IsKnownRoot(const bssl::ParsedCertificate* trust_anchor) const override {
+    return trust_store_chrome_->Contains(trust_anchor);
+  }
+
+  int64_t chrome_root_store_version() const override {
+    return trust_store_chrome_->version();
+  }
 
  private:
-  TrustStoreCollection trust_store_;
+  std::unique_ptr<TrustStoreChrome> trust_store_chrome_;
 };
 
-}  // namespace
+std::unique_ptr<SystemTrustStore> CreateChromeOnlySystemTrustStore(
+    std::unique_ptr<TrustStoreChrome> chrome_root) {
+  return std::make_unique<SystemTrustStoreChromeOnly>(std::move(chrome_root));
+}
 
-#if BUILDFLAG(CHROME_ROOT_STORE_SUPPORTED)
 class SystemTrustStoreChromeWithUnOwnedSystemStore : public SystemTrustStore {
  public:
   // Creates a SystemTrustStore that gets publicly trusted roots from
@@ -89,29 +92,29 @@ class SystemTrustStoreChromeWithUnOwnedSystemStore : public SystemTrustStore {
   // object.
   explicit SystemTrustStoreChromeWithUnOwnedSystemStore(
       std::unique_ptr<TrustStoreChrome> trust_store_chrome,
-      TrustStore* trust_store_system)
+      bssl::TrustStore* trust_store_system)
       : trust_store_chrome_(std::move(trust_store_chrome)) {
     trust_store_collection_.AddTrustStore(trust_store_system);
     trust_store_collection_.AddTrustStore(trust_store_chrome_.get());
   }
 
-  TrustStore* GetTrustStore() override { return &trust_store_collection_; }
-
-  bool UsesSystemTrustStore() const override { return true; }
+  bssl::TrustStore* GetTrustStore() override {
+    return &trust_store_collection_;
+  }
 
   // IsKnownRoot returns true if the given trust anchor is a standard one (as
   // opposed to a user-installed root)
-  bool IsKnownRoot(const ParsedCertificate* trust_anchor) const override {
+  bool IsKnownRoot(const bssl::ParsedCertificate* trust_anchor) const override {
     return trust_store_chrome_->Contains(trust_anchor);
   }
 
-  int64_t chrome_root_store_version() override {
+  int64_t chrome_root_store_version() const override {
     return trust_store_chrome_->version();
   }
 
  private:
   std::unique_ptr<TrustStoreChrome> trust_store_chrome_;
-  TrustStoreCollection trust_store_collection_;
+  bssl::TrustStoreCollection trust_store_collection_;
 };
 
 class SystemTrustStoreChrome
@@ -121,80 +124,30 @@ class SystemTrustStoreChrome
   // |trust_store_chrome| and local trust settings from |trust_store_system|.
   explicit SystemTrustStoreChrome(
       std::unique_ptr<TrustStoreChrome> trust_store_chrome,
-      std::unique_ptr<TrustStore> trust_store_system)
+      std::unique_ptr<bssl::TrustStore> trust_store_system)
       : SystemTrustStoreChromeWithUnOwnedSystemStore(
             std::move(trust_store_chrome),
             trust_store_system.get()),
         trust_store_system_(std::move(trust_store_system)) {}
 
  private:
-  std::unique_ptr<TrustStore> trust_store_system_;
+  std::unique_ptr<bssl::TrustStore> trust_store_system_;
 };
 
 std::unique_ptr<SystemTrustStore> CreateSystemTrustStoreChromeForTesting(
     std::unique_ptr<TrustStoreChrome> trust_store_chrome,
-    std::unique_ptr<TrustStore> trust_store_system) {
+    std::unique_ptr<bssl::TrustStore> trust_store_system) {
   return std::make_unique<SystemTrustStoreChrome>(
       std::move(trust_store_chrome), std::move(trust_store_system));
 }
 #endif  // CHROME_ROOT_STORE_SUPPORTED
 
 #if BUILDFLAG(USE_NSS_CERTS)
-namespace {
 
-class SystemTrustStoreNSS : public SystemTrustStore {
- public:
-  explicit SystemTrustStoreNSS(std::unique_ptr<TrustStoreNSS> trust_store_nss)
-      : trust_store_nss_(std::move(trust_store_nss)) {}
-
-  TrustStore* GetTrustStore() override { return trust_store_nss_.get(); }
-
-  bool UsesSystemTrustStore() const override { return true; }
-
-  // IsKnownRoot returns true if the given trust anchor is a standard one (as
-  // opposed to a user-installed root)
-  bool IsKnownRoot(const ParsedCertificate* trust_anchor) const override {
-    // TODO(eroman): The overall approach of IsKnownRoot() is inefficient -- it
-    // requires searching for the trust anchor by DER in NSS, however path
-    // building already had a handle to it.
-    SECItem der_cert;
-    der_cert.data = const_cast<uint8_t*>(trust_anchor->der_cert().UnsafeData());
-    der_cert.len = trust_anchor->der_cert().Length();
-    der_cert.type = siDERCertBuffer;
-    ScopedCERTCertificate nss_cert(
-        CERT_FindCertByDERCert(CERT_GetDefaultCertDB(), &der_cert));
-    if (!nss_cert)
-      return false;
-
-    if (!net::IsKnownRoot(nss_cert.get()))
-      return false;
-
-    return trust_anchor->der_cert() ==
-           der::Input(nss_cert->derCert.data, nss_cert->derCert.len);
-  }
-
-#if BUILDFLAG(CHROME_ROOT_STORE_SUPPORTED)
-  int64_t chrome_root_store_version() override { return 0; }
-#endif
-
- private:
-  std::unique_ptr<TrustStoreNSS> trust_store_nss_;
-};
-
-}  // namespace
-
-std::unique_ptr<SystemTrustStore> CreateSslSystemTrustStore() {
-  return std::make_unique<SystemTrustStoreNSS>(std::make_unique<TrustStoreNSS>(
-      TrustStoreNSS::kUseSystemTrust,
-      TrustStoreNSS::UseTrustFromAllUserSlots()));
-}
-
-#if BUILDFLAG(CHROME_ROOT_STORE_SUPPORTED)
 std::unique_ptr<SystemTrustStore> CreateSslSystemTrustStoreChromeRoot(
     std::unique_ptr<TrustStoreChrome> chrome_root) {
   return std::make_unique<SystemTrustStoreChrome>(
       std::move(chrome_root), std::make_unique<TrustStoreNSS>(
-                                  TrustStoreNSS::kIgnoreSystemTrust,
                                   TrustStoreNSS::UseTrustFromAllUserSlots()));
 }
 
@@ -204,27 +157,11 @@ CreateSslSystemTrustStoreChromeRootWithUserSlotRestriction(
     crypto::ScopedPK11Slot user_slot_restriction) {
   return std::make_unique<SystemTrustStoreChrome>(
       std::move(chrome_root),
-      std::make_unique<TrustStoreNSS>(TrustStoreNSS::kIgnoreSystemTrust,
-                                      std::move(user_slot_restriction)));
-}
-
-#endif  // CHROME_ROOT_STORE_SUPPORTED
-
-std::unique_ptr<SystemTrustStore>
-CreateSslSystemTrustStoreNSSWithUserSlotRestriction(
-    crypto::ScopedPK11Slot user_slot_restriction) {
-  return std::make_unique<SystemTrustStoreNSS>(std::make_unique<TrustStoreNSS>(
-      TrustStoreNSS::kUseSystemTrust, std::move(user_slot_restriction)));
+      std::make_unique<TrustStoreNSS>(std::move(user_slot_restriction)));
 }
 
 #elif BUILDFLAG(IS_MAC)
 
-// Using the Builtin Verifier w/o the Chrome Root Store is unsupported on
-// Mac.
-std::unique_ptr<SystemTrustStore> CreateSslSystemTrustStore() {
-  return std::make_unique<DummySystemTrustStore>();
-}
-
 namespace {
 
 TrustStoreMac* GetGlobalTrustStoreMacForCRS() {
@@ -271,12 +208,11 @@ class FuchsiaSystemCerts {
     }
 
     CertificateList certs = X509Certificate::CreateCertificateListFromBytes(
-        base::as_bytes(base::make_span(certs_file)),
-        X509Certificate::FORMAT_AUTO);
+        base::as_byte_span(certs_file), X509Certificate::FORMAT_AUTO);
 
     for (const auto& cert : certs) {
-      CertErrors errors;
-      auto parsed = ParsedCertificate::Create(
+      bssl::CertErrors errors;
+      auto parsed = bssl::ParsedCertificate::Create(
           bssl::UpRef(cert->cert_buffer()),
           x509_util::DefaultParseCertificateOptions(), &errors);
       CHECK(parsed) << errors.ToDebugString();
@@ -284,10 +220,12 @@ class FuchsiaSystemCerts {
     }
   }
 
-  TrustStoreInMemory* system_trust_store() { return &system_trust_store_; }
+  bssl::TrustStoreInMemory* system_trust_store() {
+    return &system_trust_store_;
+  }
 
  private:
-  TrustStoreInMemory system_trust_store_;
+  bssl::TrustStoreInMemory system_trust_store_;
 };
 
 base::LazyInstance<FuchsiaSystemCerts>::Leaky g_root_certs_fuchsia =
@@ -299,13 +237,11 @@ class SystemTrustStoreFuchsia : public SystemTrustStore {
  public:
   SystemTrustStoreFuchsia() = default;
 
-  TrustStore* GetTrustStore() override {
+  bssl::TrustStore* GetTrustStore() override {
     return g_root_certs_fuchsia.Get().system_trust_store();
   }
 
-  bool UsesSystemTrustStore() const override { return true; }
-
-  bool IsKnownRoot(const ParsedCertificate* trust_anchor) const override {
+  bool IsKnownRoot(const bssl::ParsedCertificate* trust_anchor) const override {
     return g_root_certs_fuchsia.Get().system_trust_store()->Contains(
         trust_anchor);
   }
@@ -317,12 +253,6 @@ std::unique_ptr<SystemTrustStore> CreateSslSystemTrustStore() {
 
 #elif BUILDFLAG(IS_WIN)
 
-// Using the Builtin Verifier w/o the Chrome Root Store is unsupported on
-// Windows.
-std::unique_ptr<SystemTrustStore> CreateSslSystemTrustStore() {
-  return std::make_unique<DummySystemTrustStore>();
-}
-
 namespace {
 TrustStoreWin* GetGlobalTrustStoreWinForCRS() {
   static base::NoDestructor<TrustStoreWin> static_trust_store_win;
@@ -351,12 +281,6 @@ void InitializeTrustStoreWinSystem() {
 
 #elif BUILDFLAG(IS_ANDROID)
 
-// Using the Builtin Verifier w/o the Chrome Root Store is unsupported on
-// Android.
-std::unique_ptr<SystemTrustStore> CreateSslSystemTrustStore() {
-  return std::make_unique<DummySystemTrustStore>();
-}
-
 #if BUILDFLAG(CHROME_ROOT_STORE_SUPPORTED)
 
 namespace {
@@ -396,16 +320,6 @@ void InitializeTrustStoreAndroid() {}
 
 #endif  // CHROME_ROOT_STORE_SUPPORTED
 
-#else
-
-std::unique_ptr<SystemTrustStore> CreateSslSystemTrustStore() {
-  return std::make_unique<DummySystemTrustStore>();
-}
-
 #endif
 
-std::unique_ptr<SystemTrustStore> CreateEmptySystemTrustStore() {
-  return std::make_unique<DummySystemTrustStore>();
-}
-
 }  // namespace net
