@@ -11,14 +11,6 @@
 
 namespace net {
 
-// Create a SystemTrustStore that will accept trust for:
-// (*) built-in certificates
-// (*) certificates stored on the |user_slot_restriction|, if non-null.
-NET_EXPORT std::unique_ptr<SystemTrustStore>
-CreateSslSystemTrustStoreNSSWithUserSlotRestriction(
-    crypto::ScopedPK11Slot user_slot_restriction);
-
-#if BUILDFLAG(CHROME_ROOT_STORE_SUPPORTED)
 // Create a SystemTrustStore that will accept trust for:
 // (*) Chrome Root Store certificates
 // (*) certificates stored on the |user_slot_restriction|, if non-null.
@@ -26,7 +18,6 @@ NET_EXPORT std::unique_ptr<SystemTrustStore>
 CreateSslSystemTrustStoreChromeRootWithUserSlotRestriction(
     std::unique_ptr<TrustStoreChrome> chrome_root,
     crypto::ScopedPK11Slot user_slot_restriction);
-#endif
 
 }  // namespace net
 
