@@ -25,16 +25,16 @@
 #include "net/base/hash_value.h"
 #include "net/base/network_notification_thread_mac.h"
 #include "net/cert/internal/trust_store_features.h"
-#include "net/cert/pki/cert_errors.h"
-#include "net/cert/pki/cert_issuer_source_static.h"
-#include "net/cert/pki/extended_key_usage.h"
-#include "net/cert/pki/parse_name.h"
-#include "net/cert/pki/parsed_certificate.h"
-#include "net/cert/pki/trust_store.h"
 #include "net/cert/test_keychain_search_list_mac.h"
 #include "net/cert/x509_util.h"
 #include "net/cert/x509_util_apple.h"
 #include "third_party/boringssl/src/include/openssl/sha.h"
+#include "third_party/boringssl/src/pki/cert_errors.h"
+#include "third_party/boringssl/src/pki/cert_issuer_source_static.h"
+#include "third_party/boringssl/src/pki/extended_key_usage.h"
+#include "third_party/boringssl/src/pki/parse_name.h"
+#include "third_party/boringssl/src/pki/parsed_certificate.h"
+#include "third_party/boringssl/src/pki/trust_store.h"
 
 namespace net {
 
@@ -56,51 +56,17 @@ enum class TrustStatus {
   DISTRUSTED
 };
 
-const void* kResultDebugDataKey = &kResultDebugDataKey;
-
 // Returns trust status of usage constraints dictionary |trust_dict| for a
 // certificate that |is_self_issued|.
 TrustStatus IsTrustDictionaryTrustedForPolicy(
     CFDictionaryRef trust_dict,
     bool is_self_issued,
-    const CFStringRef target_policy_oid,
-    int* debug_info) {
+    const CFStringRef target_policy_oid) {
   crypto::GetMacSecurityServicesLock().AssertAcquired();
 
   // An empty trust dict should be interpreted as
   // kSecTrustSettingsResultTrustRoot. This is handled by falling through all
   // the conditions below with the default value of |trust_settings_result|.
-  CFIndex dict_size = CFDictionaryGetCount(trust_dict);
-  if (dict_size == 0)
-    *debug_info |= TrustStoreMac::TRUST_SETTINGS_DICT_EMPTY;
-
-  CFIndex known_elements = 0;
-  if (CFDictionaryContainsKey(trust_dict, kSecTrustSettingsPolicy)) {
-    *debug_info |= TrustStoreMac::TRUST_SETTINGS_DICT_CONTAINS_POLICY;
-    known_elements++;
-  }
-  if (CFDictionaryContainsKey(trust_dict, kSecTrustSettingsApplication)) {
-    *debug_info |= TrustStoreMac::TRUST_SETTINGS_DICT_CONTAINS_APPLICATION;
-    known_elements++;
-  }
-  if (CFDictionaryContainsKey(trust_dict, kSecTrustSettingsPolicyString)) {
-    *debug_info |= TrustStoreMac::TRUST_SETTINGS_DICT_CONTAINS_POLICY_STRING;
-    known_elements++;
-  }
-  if (CFDictionaryContainsKey(trust_dict, kSecTrustSettingsKeyUsage)) {
-    *debug_info |= TrustStoreMac::TRUST_SETTINGS_DICT_CONTAINS_KEY_USAGE;
-    known_elements++;
-  }
-  if (CFDictionaryContainsKey(trust_dict, kSecTrustSettingsResult)) {
-    *debug_info |= TrustStoreMac::TRUST_SETTINGS_DICT_CONTAINS_RESULT;
-    known_elements++;
-  }
-  if (CFDictionaryContainsKey(trust_dict, kSecTrustSettingsAllowedError)) {
-    *debug_info |= TrustStoreMac::TRUST_SETTINGS_DICT_CONTAINS_ALLOWED_ERROR;
-    known_elements++;
-  }
-  if (known_elements != dict_size)
-    *debug_info |= TrustStoreMac::TRUST_SETTINGS_DICT_UNKNOWN_KEY;
 
   // Trust settings may be scoped to a single application, by checking that the
   // code signing identity of the current application matches the serialized
@@ -129,7 +95,6 @@ TrustStatus IsTrustDictionaryTrustedForPolicy(
     SecPolicyRef policy_ref = base::apple::GetValueFromDictionary<SecPolicyRef>(
         trust_dict, kSecTrustSettingsPolicy);
     if (!policy_ref) {
-      *debug_info |= TrustStoreMac::TRUST_SETTINGS_DICT_INVALID_POLICY_TYPE;
       return TrustStatus::UNSPECIFIED;
     }
     base::apple::ScopedCFTypeRef<CFDictionaryRef> policy_dict(
@@ -137,7 +102,7 @@ TrustStatus IsTrustDictionaryTrustedForPolicy(
 
     // kSecPolicyOid is guaranteed to be present in the policy dictionary.
     CFStringRef policy_oid = base::apple::GetValueFromDictionary<CFStringRef>(
-        policy_dict, kSecPolicyOid);
+        policy_dict.get(), kSecPolicyOid);
 
     if (!CFEqual(policy_oid, target_policy_oid))
       return TrustStatus::UNSPECIFIED;
@@ -153,7 +118,6 @@ TrustStatus IsTrustDictionaryTrustedForPolicy(
     if (!trust_settings_result_ref ||
         !CFNumberGetValue(trust_settings_result_ref, kCFNumberIntType,
                           &trust_settings_result)) {
-      *debug_info |= TrustStoreMac::TRUST_SETTINGS_DICT_INVALID_RESULT_TYPE;
       return TrustStatus::UNSPECIFIED;
     }
   }
@@ -188,14 +152,12 @@ TrustStatus IsTrustDictionaryTrustedForPolicy(
 // that |is_self_issued| should be treated as a trust anchor.
 TrustStatus IsTrustSettingsTrustedForPolicy(CFArrayRef trust_settings,
                                             bool is_self_issued,
-                                            const CFStringRef policy_oid,
-                                            int* debug_info) {
+                                            const CFStringRef policy_oid) {
   // An empty trust settings array (that is, the trust_settings parameter
   // returns a valid but empty CFArray) means "always trust this certificate"
   // with an overall trust setting for the certificate of
   // kSecTrustSettingsResultTrustRoot.
   if (CFArrayGetCount(trust_settings) == 0) {
-    *debug_info |= TrustStoreMac::TRUST_SETTINGS_ARRAY_EMPTY;
     return is_self_issued ? TrustStatus::TRUSTED : TrustStatus::UNSPECIFIED;
   }
 
@@ -204,7 +166,7 @@ TrustStatus IsTrustSettingsTrustedForPolicy(CFArrayRef trust_settings,
     CFDictionaryRef trust_dict = reinterpret_cast<CFDictionaryRef>(
         const_cast<void*>(CFArrayGetValueAtIndex(trust_settings, i)));
     TrustStatus trust = IsTrustDictionaryTrustedForPolicy(
-        trust_dict, is_self_issued, policy_oid, debug_info);
+        trust_dict, is_self_issued, policy_oid);
     if (trust != TrustStatus::UNSPECIFIED)
       return trust;
   }
@@ -217,8 +179,7 @@ TrustStatus IsSecCertificateTrustedForPolicyInDomain(
     SecCertificateRef cert_handle,
     const bool is_self_issued,
     const CFStringRef policy_oid,
-    SecTrustSettingsDomain trust_domain,
-    int* debug_info) {
+    SecTrustSettingsDomain trust_domain) {
   crypto::GetMacSecurityServicesLock().AssertAcquired();
 
   base::apple::ScopedCFTypeRef<CFArrayRef> trust_settings;
@@ -231,22 +192,20 @@ TrustStatus IsSecCertificateTrustedForPolicyInDomain(
   }
   if (err) {
     OSSTATUS_LOG(ERROR, err) << "SecTrustSettingsCopyTrustSettings error";
-    *debug_info |= TrustStoreMac::COPY_TRUST_SETTINGS_ERROR;
     return TrustStatus::UNSPECIFIED;
   }
   TrustStatus trust = IsTrustSettingsTrustedForPolicy(
-      trust_settings, is_self_issued, policy_oid, debug_info);
+      trust_settings.get(), is_self_issued, policy_oid);
   return trust;
 }
 
 TrustStatus IsCertificateTrustedForPolicyInDomain(
-    const ParsedCertificate* cert,
+    const bssl::ParsedCertificate* cert,
     const CFStringRef policy_oid,
-    SecTrustSettingsDomain trust_domain,
-    int* debug_info) {
+    SecTrustSettingsDomain trust_domain) {
   // TODO(eroman): Inefficient -- path building will convert between
-  // SecCertificateRef and ParsedCertificate representations multiple times
-  // (when getting the issuers, and again here).
+  // SecCertificateRef and bssl::ParsedCertificate representations multiple
+  // times (when getting the issuers, and again here).
   //
   // This conversion will also be done for each domain the cert policy is
   // checked, but the TrustDomainCache ensures this function is only called on
@@ -264,13 +223,12 @@ TrustStatus IsCertificateTrustedForPolicyInDomain(
       cert->normalized_subject() == cert->normalized_issuer();
 
   return IsSecCertificateTrustedForPolicyInDomain(
-      cert_handle, is_self_issued, policy_oid, trust_domain, debug_info);
+      cert_handle.get(), is_self_issued, policy_oid, trust_domain);
 }
 
-TrustStatus IsCertificateTrustedForPolicy(const ParsedCertificate* cert,
+TrustStatus IsCertificateTrustedForPolicy(const bssl::ParsedCertificate* cert,
                                           SecCertificateRef cert_handle,
-                                          const CFStringRef policy_oid,
-                                          int* debug_info) {
+                                          const CFStringRef policy_oid) {
   crypto::GetMacSecurityServicesLock().AssertAcquired();
 
   const bool is_self_issued =
@@ -290,11 +248,10 @@ TrustStatus IsCertificateTrustedForPolicy(const ParsedCertificate* cert,
         continue;
       }
       OSSTATUS_LOG(ERROR, err) << "SecTrustSettingsCopyTrustSettings error";
-      *debug_info |= TrustStoreMac::COPY_TRUST_SETTINGS_ERROR;
       continue;
     }
     TrustStatus trust = IsTrustSettingsTrustedForPolicy(
-        trust_settings, is_self_issued, policy_oid, debug_info);
+        trust_settings.get(), is_self_issued, policy_oid);
     if (trust != TrustStatus::UNSPECIFIED)
       return trust;
   }
@@ -304,9 +261,8 @@ TrustStatus IsCertificateTrustedForPolicy(const ParsedCertificate* cert,
   return TrustStatus::UNSPECIFIED;
 }
 
-TrustStatus IsCertificateTrustedForPolicy(const ParsedCertificate* cert,
-                                          const CFStringRef policy_oid,
-                                          int* debug_info) {
+TrustStatus IsCertificateTrustedForPolicy(const bssl::ParsedCertificate* cert,
+                                          const CFStringRef policy_oid) {
   base::apple::ScopedCFTypeRef<SecCertificateRef> cert_handle =
       x509_util::CreateSecCertificateFromBytes(cert->der_cert().UnsafeData(),
                                                cert->der_cert().Length());
@@ -314,18 +270,7 @@ TrustStatus IsCertificateTrustedForPolicy(const ParsedCertificate* cert,
   if (!cert_handle)
     return TrustStatus::UNSPECIFIED;
 
-  return IsCertificateTrustedForPolicy(cert, cert_handle, policy_oid,
-                                       debug_info);
-}
-
-void UpdateUserData(int debug_info,
-                    base::SupportsUserData* user_data,
-                    TrustStoreMac::TrustImplType impl_type) {
-  if (!user_data)
-    return;
-  TrustStoreMac::ResultDebugData* result_debug_data =
-      TrustStoreMac::ResultDebugData::GetOrCreate(user_data);
-  result_debug_data->UpdateTrustDebugInfo(debug_info, impl_type);
+  return IsCertificateTrustedForPolicy(cert, cert_handle.get(), policy_oid);
 }
 
 // Returns true if |cert| would never be a valid intermediate. (A return
@@ -334,7 +279,7 @@ void UpdateUserData(int debug_info,
 // chain. It's not intended to exhaustively test everything that
 // VerifyCertificateChain does, just to filter out some of the most obviously
 // unusable certs.
-bool IsNotAcceptableIntermediate(const ParsedCertificate* cert,
+bool IsNotAcceptableIntermediate(const bssl::ParsedCertificate* cert,
                                  const CFStringRef policy_oid) {
   if (!cert->has_basic_constraints() || !cert->basic_constraints().is_ca) {
     return true;
@@ -344,8 +289,10 @@ bool IsNotAcceptableIntermediate(const ParsedCertificate* cert,
   // actually care about.
   if (cert->has_extended_key_usage() &&
       CFEqual(policy_oid, kSecPolicyAppleSSL) &&
-      !base::Contains(cert->extended_key_usage(), der::Input(kAnyEKU)) &&
-      !base::Contains(cert->extended_key_usage(), der::Input(kServerAuth))) {
+      !base::Contains(cert->extended_key_usage(),
+                      bssl::der::Input(bssl::kAnyEKU)) &&
+      !base::Contains(cert->extended_key_usage(),
+                      bssl::der::Input(bssl::kServerAuth))) {
     return true;
   }
 
@@ -359,7 +306,6 @@ class TrustDomainCacheFullCerts {
  public:
   struct TrustStatusDetails {
     TrustStatus trust_status = TrustStatus::UNKNOWN;
-    int debug_info = 0;
   };
 
   TrustDomainCacheFullCerts(SecTrustSettingsDomain domain,
@@ -393,9 +339,10 @@ class TrustDomainCacheFullCerts {
     }
     std::vector<std::pair<SHA256HashValue, TrustStatusDetails>>
         trust_status_vector;
-    for (CFIndex i = 0, size = CFArrayGetCount(cert_array); i < size; ++i) {
+    for (CFIndex i = 0, size = CFArrayGetCount(cert_array.get()); i < size;
+         ++i) {
       SecCertificateRef cert = reinterpret_cast<SecCertificateRef>(
-          const_cast<void*>(CFArrayGetValueAtIndex(cert_array, i)));
+          const_cast<void*>(CFArrayGetValueAtIndex(cert_array.get(), i)));
       base::apple::ScopedCFTypeRef<CFDataRef> der_data(
           SecCertificateCopyData(cert));
       if (!der_data) {
@@ -405,11 +352,11 @@ class TrustDomainCacheFullCerts {
       auto buffer = x509_util::CreateCryptoBuffer(base::make_span(
           CFDataGetBytePtr(der_data.get()),
           base::checked_cast<size_t>(CFDataGetLength(der_data.get()))));
-      CertErrors errors;
-      ParseCertificateOptions options;
+      bssl::CertErrors errors;
+      bssl::ParseCertificateOptions options;
       options.allow_invalid_serial_numbers = true;
-      std::shared_ptr<const ParsedCertificate> parsed_cert =
-          ParsedCertificate::Create(std::move(buffer), options, &errors);
+      std::shared_ptr<const bssl::ParsedCertificate> parsed_cert =
+          bssl::ParsedCertificate::Create(std::move(buffer), options, &errors);
       if (!parsed_cert) {
         LOG(ERROR) << "Error parsing certificate:\n" << errors.ToDebugString();
         continue;
@@ -424,22 +371,17 @@ class TrustDomainCacheFullCerts {
   }
 
   // Returns the trust status for |cert| in |domain_|.
-  TrustStatus IsCertTrusted(const ParsedCertificate* cert,
-                            const SHA256HashValue& cert_hash,
-                            base::SupportsUserData* debug_data) {
+  TrustStatus IsCertTrusted(const bssl::ParsedCertificate* cert,
+                            const SHA256HashValue& cert_hash) {
     auto cache_iter = trust_status_cache_.find(cert_hash);
     if (cache_iter == trust_status_cache_.end()) {
       // Cert does not have trust settings in this domain, return UNSPECIFIED.
-      UpdateUserData(0, debug_data,
-                     TrustStoreMac::TrustImplType::kDomainCacheFullCerts);
       return TrustStatus::UNSPECIFIED;
     }
 
     if (cache_iter->second.trust_status != TrustStatus::UNKNOWN) {
       // Cert has trust settings and trust has already been calculated, return
       // the cached value.
-      UpdateUserData(cache_iter->second.debug_info, debug_data,
-                     TrustStoreMac::TrustImplType::kDomainCacheFullCerts);
       return cache_iter->second.trust_status;
     }
 
@@ -447,11 +389,9 @@ class TrustDomainCacheFullCerts {
 
     // Cert has trust settings but trust has not been calculated yet.
     // Calculate it now, insert into cache, and return.
-    TrustStatus cert_trust = IsCertificateTrustedForPolicyInDomain(
-        cert, policy_oid_, domain_, &cache_iter->second.debug_info);
+    TrustStatus cert_trust =
+        IsCertificateTrustedForPolicyInDomain(cert, policy_oid_, domain_);
     cache_iter->second.trust_status = cert_trust;
-    UpdateUserData(cache_iter->second.debug_info, debug_data,
-                   TrustStoreMac::TrustImplType::kDomainCacheFullCerts);
     return cert_trust;
   }
 
@@ -460,9 +400,9 @@ class TrustDomainCacheFullCerts {
     return trust_status_cache_.find(cert_hash) != trust_status_cache_.end();
   }
 
-  // Returns a CertIssuerSource containing all the certificates that are
+  // Returns a bssl::CertIssuerSource containing all the certificates that are
   // present in |domain_|.
-  CertIssuerSource& cert_issuer_source() { return cert_issuer_source_; }
+  bssl::CertIssuerSource& cert_issuer_source() { return cert_issuer_source_; }
 
  private:
   void HistogramTrustDomainCertCount(size_t count) const {
@@ -487,10 +427,10 @@ class TrustDomainCacheFullCerts {
   const SecTrustSettingsDomain domain_;
   const CFStringRef policy_oid_;
   base::flat_map<SHA256HashValue, TrustStatusDetails> trust_status_cache_;
-  CertIssuerSourceStatic cert_issuer_source_;
+  bssl::CertIssuerSourceStatic cert_issuer_source_;
 };
 
-SHA256HashValue CalculateFingerprint256(const der::Input& buffer) {
+SHA256HashValue CalculateFingerprint256(const bssl::der::Input& buffer) {
   SHA256HashValue sha256;
   SHA256(buffer.UnsafeData(), buffer.Length(), sha256.data);
   return sha256;
@@ -619,38 +559,6 @@ using KeychainTrustOrCertsObserver =
 
 }  // namespace
 
-// static
-const TrustStoreMac::ResultDebugData* TrustStoreMac::ResultDebugData::Get(
-    const base::SupportsUserData* debug_data) {
-  return static_cast<ResultDebugData*>(
-      debug_data->GetUserData(kResultDebugDataKey));
-}
-
-// static
-TrustStoreMac::ResultDebugData* TrustStoreMac::ResultDebugData::GetOrCreate(
-    base::SupportsUserData* debug_data) {
-  ResultDebugData* data = static_cast<ResultDebugData*>(
-      debug_data->GetUserData(kResultDebugDataKey));
-  if (!data) {
-    std::unique_ptr<ResultDebugData> new_data =
-        std::make_unique<ResultDebugData>();
-    data = new_data.get();
-    debug_data->SetUserData(kResultDebugDataKey, std::move(new_data));
-  }
-  return data;
-}
-
-void TrustStoreMac::ResultDebugData::UpdateTrustDebugInfo(
-    int trust_debug_info,
-    TrustImplType impl_type) {
-  combined_trust_debug_info_ |= trust_debug_info;
-  trust_impl_ = impl_type;
-}
-
-std::unique_ptr<base::SupportsUserData::Data>
-TrustStoreMac::ResultDebugData::Clone() {
-  return std::make_unique<ResultDebugData>(*this);
-}
 
 // Interface for different implementations of getting trust settings from the
 // Mac APIs. This abstraction can be removed once a single implementation has
@@ -659,11 +567,10 @@ class TrustStoreMac::TrustImpl {
  public:
   virtual ~TrustImpl() = default;
 
-  virtual TrustStatus IsCertTrusted(const ParsedCertificate* cert,
-                                    base::SupportsUserData* debug_data) = 0;
+  virtual TrustStatus IsCertTrusted(const bssl::ParsedCertificate* cert) = 0;
   virtual bool ImplementsSyncGetIssuersOf() const { return false; }
-  virtual void SyncGetIssuersOf(const ParsedCertificate* cert,
-                                ParsedCertificateList* issuers) {}
+  virtual void SyncGetIssuersOf(const bssl::ParsedCertificate* cert,
+                                bssl::ParsedCertificateList* issuers) {}
   virtual void InitializeTrustCache() = 0;
 };
 
@@ -693,8 +600,7 @@ class TrustStoreMac::TrustImplDomainCacheFullCerts
       const TrustImplDomainCacheFullCerts&) = delete;
 
   // Returns the trust status for |cert|.
-  TrustStatus IsCertTrusted(const ParsedCertificate* cert,
-                            base::SupportsUserData* debug_data) override {
+  TrustStatus IsCertTrusted(const bssl::ParsedCertificate* cert) override {
     SHA256HashValue cert_hash = CalculateFingerprint256(cert->der_cert());
 
     base::AutoLock lock(cache_lock_);
@@ -704,8 +610,7 @@ class TrustStoreMac::TrustImplDomainCacheFullCerts
     // admin (and both override the system domain, but we don't check that).
     for (TrustDomainCacheFullCerts* trust_domain_cache :
          {&user_domain_cache_, &admin_domain_cache_}) {
-      TrustStatus ts =
-          trust_domain_cache->IsCertTrusted(cert, cert_hash, debug_data);
+      TrustStatus ts = trust_domain_cache->IsCertTrusted(cert, cert_hash);
       if (ts != TrustStatus::UNSPECIFIED)
         return ts;
     }
@@ -716,8 +621,8 @@ class TrustStoreMac::TrustImplDomainCacheFullCerts
 
   bool ImplementsSyncGetIssuersOf() const override { return true; }
 
-  void SyncGetIssuersOf(const ParsedCertificate* cert,
-                        ParsedCertificateList* issuers) override {
+  void SyncGetIssuersOf(const bssl::ParsedCertificate* cert,
+                        bssl::ParsedCertificateList* issuers) override {
     base::AutoLock lock(cache_lock_);
     MaybeInitializeCache();
     user_domain_cache_.cert_issuer_source().SyncGetIssuersOf(cert, issuers);
@@ -779,9 +684,9 @@ class TrustStoreMac::TrustImplDomainCacheFullCerts
         CFDictionaryCreateMutable(nullptr, 0, &kCFTypeDictionaryKeyCallBacks,
                                   &kCFTypeDictionaryValueCallBacks));
 
-    CFDictionarySetValue(query, kSecClass, kSecClassCertificate);
-    CFDictionarySetValue(query, kSecReturnRef, kCFBooleanTrue);
-    CFDictionarySetValue(query, kSecMatchLimit, kSecMatchLimitAll);
+    CFDictionarySetValue(query.get(), kSecClass, kSecClassCertificate);
+    CFDictionarySetValue(query.get(), kSecReturnRef, kCFBooleanTrue);
+    CFDictionarySetValue(query.get(), kSecMatchLimit, kSecMatchLimitAll);
 
     base::AutoLock lock(crypto::GetMacSecurityServicesLock());
 
@@ -795,12 +700,13 @@ class TrustStoreMac::TrustImplDomainCacheFullCerts
             << "TestKeychainSearchList::CopySearchList error";
         return;
       }
-      CFDictionarySetValue(query, kSecMatchSearchList,
+      CFDictionarySetValue(query.get(), kSecMatchSearchList,
                            scoped_alternate_keychain_search_list.get());
     }
 
     base::apple::ScopedCFTypeRef<CFTypeRef> matching_items;
-    OSStatus err = SecItemCopyMatching(query, matching_items.InitializeInto());
+    OSStatus err =
+        SecItemCopyMatching(query.get(), matching_items.InitializeInto());
     if (err == errSecItemNotFound) {
       RecordCachedIntermediatesHistograms(0, timer.Elapsed());
       // No matches found.
@@ -812,7 +718,7 @@ class TrustStoreMac::TrustImplDomainCacheFullCerts
       return;
     }
     CFArrayRef matching_items_array =
-        base::apple::CFCastStrict<CFArrayRef>(matching_items);
+        base::apple::CFCastStrict<CFArrayRef>(matching_items.get());
     for (CFIndex i = 0, item_count = CFArrayGetCount(matching_items_array);
          i < item_count; ++i) {
       SecCertificateRef match_cert_handle =
@@ -837,16 +743,16 @@ class TrustStoreMac::TrustImplDomainCacheFullCerts
       auto buffer = x509_util::CreateCryptoBuffer(base::make_span(
           CFDataGetBytePtr(der_data.get()),
           base::checked_cast<size_t>(CFDataGetLength(der_data.get()))));
-      CertErrors errors;
-      ParseCertificateOptions options;
+      bssl::CertErrors errors;
+      bssl::ParseCertificateOptions options;
       options.allow_invalid_serial_numbers = true;
-      std::shared_ptr<const ParsedCertificate> parsed_cert =
-          ParsedCertificate::Create(std::move(buffer), options, &errors);
+      std::shared_ptr<const bssl::ParsedCertificate> parsed_cert =
+          bssl::ParsedCertificate::Create(std::move(buffer), options, &errors);
       if (!parsed_cert) {
         LOG(ERROR) << "Error parsing certificate:\n" << errors.ToDebugString();
         continue;
       }
-      if (IsNotAcceptableIntermediate(parsed_cert.get(), policy_oid_)) {
+      if (IsNotAcceptableIntermediate(parsed_cert.get(), policy_oid_.get())) {
         continue;
       }
       intermediates_cert_issuer_source_.AddCert(std::move(parsed_cert));
@@ -883,7 +789,7 @@ class TrustStoreMac::TrustImplDomainCacheFullCerts
   TrustDomainCacheFullCerts admin_domain_cache_ GUARDED_BY(cache_lock_);
   TrustDomainCacheFullCerts user_domain_cache_ GUARDED_BY(cache_lock_);
 
-  CertIssuerSourceStatic intermediates_cert_issuer_source_
+  bssl::CertIssuerSourceStatic intermediates_cert_issuer_source_
       GUARDED_BY(cache_lock_);
 };
 
@@ -907,16 +813,9 @@ class TrustStoreMac::TrustImplKeychainCacheFullCerts
   TrustImplKeychainCacheFullCerts& operator=(
       const TrustImplKeychainCacheFullCerts&) = delete;
 
-  TrustStatus IsCertTrusted(const ParsedCertificate* cert,
-                            base::SupportsUserData* debug_data) override {
+  TrustStatus IsCertTrusted(const bssl::ParsedCertificate* cert) override {
     SHA256HashValue cert_hash = CalculateFingerprint256(cert->der_cert());
 
-    // This impl doesn't bother to set the debug_info field since we're not
-    // using that anymore, but the related code hasn't been cleaned up yet.
-    // TODO(https://crbug.com/1379461): delete the debug user data code.
-    UpdateUserData(0, debug_data,
-                   TrustStoreMac::TrustImplType::kKeychainCacheFullCerts);
-
     base::AutoLock lock(cache_lock_);
     MaybeInitializeCache();
 
@@ -928,8 +827,8 @@ class TrustStoreMac::TrustImplKeychainCacheFullCerts
 
   bool ImplementsSyncGetIssuersOf() const override { return true; }
 
-  void SyncGetIssuersOf(const ParsedCertificate* cert,
-                        ParsedCertificateList* issuers) override {
+  void SyncGetIssuersOf(const bssl::ParsedCertificate* cert,
+                        bssl::ParsedCertificateList* issuers) override {
     base::AutoLock lock(cache_lock_);
     MaybeInitializeCache();
     cert_issuer_source_.SyncGetIssuersOf(cert, issuers);
@@ -962,9 +861,9 @@ class TrustStoreMac::TrustImplKeychainCacheFullCerts
         CFDictionaryCreateMutable(nullptr, 0, &kCFTypeDictionaryKeyCallBacks,
                                   &kCFTypeDictionaryValueCallBacks));
 
-    CFDictionarySetValue(query, kSecClass, kSecClassCertificate);
-    CFDictionarySetValue(query, kSecReturnRef, kCFBooleanTrue);
-    CFDictionarySetValue(query, kSecMatchLimit, kSecMatchLimitAll);
+    CFDictionarySetValue(query.get(), kSecClass, kSecClassCertificate);
+    CFDictionarySetValue(query.get(), kSecReturnRef, kCFBooleanTrue);
+    CFDictionarySetValue(query.get(), kSecMatchLimit, kSecMatchLimitAll);
 
     base::AutoLock lock(crypto::GetMacSecurityServicesLock());
 
@@ -978,12 +877,13 @@ class TrustStoreMac::TrustImplKeychainCacheFullCerts
             << "TestKeychainSearchList::CopySearchList error";
         return;
       }
-      CFDictionarySetValue(query, kSecMatchSearchList,
+      CFDictionarySetValue(query.get(), kSecMatchSearchList,
                            scoped_alternate_keychain_search_list.get());
     }
 
     base::apple::ScopedCFTypeRef<CFTypeRef> matching_items;
-    OSStatus err = SecItemCopyMatching(query, matching_items.InitializeInto());
+    OSStatus err =
+        SecItemCopyMatching(query.get(), matching_items.InitializeInto());
     if (err == errSecItemNotFound) {
       RecordHistograms(0, timer.Elapsed());
       // No matches found.
@@ -995,7 +895,7 @@ class TrustStoreMac::TrustImplKeychainCacheFullCerts
       return;
     }
     CFArrayRef matching_items_array =
-        base::apple::CFCastStrict<CFArrayRef>(matching_items);
+        base::apple::CFCastStrict<CFArrayRef>(matching_items.get());
     std::vector<std::pair<SHA256HashValue, TrustStatus>> trust_status_vector;
     for (CFIndex i = 0, item_count = CFArrayGetCount(matching_items_array);
          i < item_count; ++i) {
@@ -1011,19 +911,18 @@ class TrustStoreMac::TrustImplKeychainCacheFullCerts
       auto buffer = x509_util::CreateCryptoBuffer(base::make_span(
           CFDataGetBytePtr(der_data.get()),
           base::checked_cast<size_t>(CFDataGetLength(der_data.get()))));
-      CertErrors errors;
-      ParseCertificateOptions options;
+      bssl::CertErrors errors;
+      bssl::ParseCertificateOptions options;
       options.allow_invalid_serial_numbers = true;
-      std::shared_ptr<const ParsedCertificate> parsed_cert =
-          ParsedCertificate::Create(std::move(buffer), options, &errors);
+      std::shared_ptr<const bssl::ParsedCertificate> parsed_cert =
+          bssl::ParsedCertificate::Create(std::move(buffer), options, &errors);
       if (!parsed_cert) {
         LOG(ERROR) << "Error parsing certificate:\n" << errors.ToDebugString();
         continue;
       }
 
-      int debug_info = 0;
       TrustStatus trust_status = IsCertificateTrustedForPolicy(
-          parsed_cert.get(), sec_cert, policy_oid_, &debug_info);
+          parsed_cert.get(), sec_cert, policy_oid_.get());
 
       if (trust_status == TrustStatus::TRUSTED ||
           trust_status == TrustStatus::DISTRUSTED) {
@@ -1035,7 +934,7 @@ class TrustStoreMac::TrustImplKeychainCacheFullCerts
         continue;
       }
 
-      if (IsNotAcceptableIntermediate(parsed_cert.get(), policy_oid_)) {
+      if (IsNotAcceptableIntermediate(parsed_cert.get(), policy_oid_.get())) {
         continue;
       }
       cert_issuer_source_.AddCert(std::move(parsed_cert));
@@ -1069,7 +968,7 @@ class TrustStoreMac::TrustImplKeychainCacheFullCerts
   int64_t keychain_iteration_ GUARDED_BY(cache_lock_) = -1;
   base::flat_map<SHA256HashValue, TrustStatus> trust_status_cache_
       GUARDED_BY(cache_lock_);
-  CertIssuerSourceStatic cert_issuer_source_ GUARDED_BY(cache_lock_);
+  bssl::CertIssuerSourceStatic cert_issuer_source_ GUARDED_BY(cache_lock_);
 };
 
 // TrustImplNoCache is the simplest approach which calls
@@ -1084,14 +983,9 @@ class TrustStoreMac::TrustImplNoCache : public TrustStoreMac::TrustImpl {
   ~TrustImplNoCache() override = default;
 
   // Returns the trust status for |cert|.
-  TrustStatus IsCertTrusted(const ParsedCertificate* cert,
-                            base::SupportsUserData* debug_data) override {
-    int debug_info = 0;
+  TrustStatus IsCertTrusted(const bssl::ParsedCertificate* cert) override {
     base::AutoLock lock(crypto::GetMacSecurityServicesLock());
-    TrustStatus result =
-        IsCertificateTrustedForPolicy(cert, policy_oid_, &debug_info);
-    UpdateUserData(debug_info, debug_data,
-                   TrustStoreMac::TrustImplType::kSimple);
+    TrustStatus result = IsCertificateTrustedForPolicy(cert, policy_oid_);
     return result;
   }
 
@@ -1128,8 +1022,8 @@ void TrustStoreMac::InitializeTrustCache() const {
   trust_cache_->InitializeTrustCache();
 }
 
-void TrustStoreMac::SyncGetIssuersOf(const ParsedCertificate* cert,
-                                     ParsedCertificateList* issuers) {
+void TrustStoreMac::SyncGetIssuersOf(const bssl::ParsedCertificate* cert,
+                                     bssl::ParsedCertificateList* issuers) {
   if (trust_cache_->ImplementsSyncGetIssuersOf()) {
     trust_cache_->SyncGetIssuersOf(cert, issuers);
     return;
@@ -1141,15 +1035,15 @@ void TrustStoreMac::SyncGetIssuersOf(const ParsedCertificate* cert,
     return;
 
   std::vector<bssl::UniquePtr<CRYPTO_BUFFER>> matching_cert_buffers =
-      FindMatchingCertificatesForMacNormalizedSubject(name_data);
+      FindMatchingCertificatesForMacNormalizedSubject(name_data.get());
 
-  // Convert to ParsedCertificate.
+  // Convert to bssl::ParsedCertificate.
   for (auto& buffer : matching_cert_buffers) {
-    CertErrors errors;
-    ParseCertificateOptions options;
+    bssl::CertErrors errors;
+    bssl::ParseCertificateOptions options;
     options.allow_invalid_serial_numbers = true;
-    std::shared_ptr<const ParsedCertificate> anchor_cert =
-        ParsedCertificate::Create(std::move(buffer), options, &errors);
+    std::shared_ptr<const bssl::ParsedCertificate> anchor_cert =
+        bssl::ParsedCertificate::Create(std::move(buffer), options, &errors);
     if (!anchor_cert) {
       // TODO(crbug.com/634443): return errors better.
       LOG(ERROR) << "Error parsing issuer certificate:\n"
@@ -1161,21 +1055,22 @@ void TrustStoreMac::SyncGetIssuersOf(const ParsedCertificate* cert,
   }
 }
 
-CertificateTrust TrustStoreMac::GetTrust(const ParsedCertificate* cert,
-                                         base::SupportsUserData* debug_data) {
-  TrustStatus trust_status = trust_cache_->IsCertTrusted(cert, debug_data);
+bssl::CertificateTrust TrustStoreMac::GetTrust(
+    const bssl::ParsedCertificate* cert) {
+  TrustStatus trust_status = trust_cache_->IsCertTrusted(cert);
   switch (trust_status) {
     case TrustStatus::TRUSTED: {
-      CertificateTrust trust;
+      bssl::CertificateTrust trust;
       if (base::FeatureList::IsEnabled(
               features::kTrustStoreTrustedLeafSupport)) {
         // Mac trust settings don't distinguish between trusted anchors and
         // trusted leafs, return a trust record valid for both, which will
         // depend on the context the certificate is encountered in.
-        trust =
-            CertificateTrust::ForTrustAnchorOrLeaf().WithEnforceAnchorExpiry();
+        trust = bssl::CertificateTrust::ForTrustAnchorOrLeaf()
+                    .WithEnforceAnchorExpiry();
       } else {
-        trust = CertificateTrust::ForTrustAnchor().WithEnforceAnchorExpiry();
+        trust =
+            bssl::CertificateTrust::ForTrustAnchor().WithEnforceAnchorExpiry();
       }
       if (IsLocalAnchorConstraintsEnforcementEnabled()) {
         trust = trust.WithEnforceAnchorConstraints()
@@ -1184,9 +1079,9 @@ CertificateTrust TrustStoreMac::GetTrust(const ParsedCertificate* cert,
       return trust;
     }
     case TrustStatus::DISTRUSTED:
-      return CertificateTrust::ForDistrusted();
+      return bssl::CertificateTrust::ForDistrusted();
     case TrustStatus::UNSPECIFIED:
-      return CertificateTrust::ForUnspecified();
+      return bssl::CertificateTrust::ForUnspecified();
     case TrustStatus::UNKNOWN:
       // UNKNOWN is an implementation detail of TrustImpl and should never be
       // returned.
@@ -1194,7 +1089,7 @@ CertificateTrust TrustStoreMac::GetTrust(const ParsedCertificate* cert,
       break;
   }
 
-  return CertificateTrust::ForUnspecified();
+  return bssl::CertificateTrust::ForUnspecified();
 }
 
 // static
@@ -1206,10 +1101,10 @@ TrustStoreMac::FindMatchingCertificatesForMacNormalizedSubject(
       CFDictionaryCreateMutable(nullptr, 0, &kCFTypeDictionaryKeyCallBacks,
                                 &kCFTypeDictionaryValueCallBacks));
 
-  CFDictionarySetValue(query, kSecClass, kSecClassCertificate);
-  CFDictionarySetValue(query, kSecReturnRef, kCFBooleanTrue);
-  CFDictionarySetValue(query, kSecMatchLimit, kSecMatchLimitAll);
-  CFDictionarySetValue(query, kSecAttrSubject, name_data);
+  CFDictionarySetValue(query.get(), kSecClass, kSecClassCertificate);
+  CFDictionarySetValue(query.get(), kSecReturnRef, kCFBooleanTrue);
+  CFDictionarySetValue(query.get(), kSecMatchLimit, kSecMatchLimitAll);
+  CFDictionarySetValue(query.get(), kSecAttrSubject, name_data);
 
   base::AutoLock lock(crypto::GetMacSecurityServicesLock());
 
@@ -1226,13 +1121,14 @@ TrustStoreMac::FindMatchingCertificatesForMacNormalizedSubject(
   }
 
   if (scoped_alternate_keychain_search_list) {
-    CFDictionarySetValue(query, kSecMatchSearchList,
+    CFDictionarySetValue(query.get(), kSecMatchSearchList,
                          scoped_alternate_keychain_search_list.get());
   }
 
   base::apple::ScopedCFTypeRef<CFArrayRef> matching_items;
   OSStatus err = SecItemCopyMatching(
-      query, reinterpret_cast<CFTypeRef*>(matching_items.InitializeInto()));
+      query.get(),
+      reinterpret_cast<CFTypeRef*>(matching_items.InitializeInto()));
   if (err == errSecItemNotFound) {
     // No matches found.
     return matching_cert_buffers;
@@ -1242,10 +1138,10 @@ TrustStoreMac::FindMatchingCertificatesForMacNormalizedSubject(
     return matching_cert_buffers;
   }
 
-  for (CFIndex i = 0, item_count = CFArrayGetCount(matching_items);
+  for (CFIndex i = 0, item_count = CFArrayGetCount(matching_items.get());
        i < item_count; ++i) {
     SecCertificateRef match_cert_handle = reinterpret_cast<SecCertificateRef>(
-        const_cast<void*>(CFArrayGetValueAtIndex(matching_items, i)));
+        const_cast<void*>(CFArrayGetValueAtIndex(matching_items.get(), i)));
 
     base::apple::ScopedCFTypeRef<CFDataRef> der_data(
         SecCertificateCopyData(match_cert_handle));
@@ -1263,7 +1159,7 @@ TrustStoreMac::FindMatchingCertificatesForMacNormalizedSubject(
 
 // static
 base::apple::ScopedCFTypeRef<CFDataRef> TrustStoreMac::GetMacNormalizedIssuer(
-    const ParsedCertificate* cert) {
+    const bssl::ParsedCertificate* cert) {
   base::apple::ScopedCFTypeRef<CFDataRef> name_data;
   base::AutoLock lock(crypto::GetMacSecurityServicesLock());
   // There does not appear to be any public API to get the normalized version
@@ -1275,7 +1171,8 @@ base::apple::ScopedCFTypeRef<CFDataRef> TrustStoreMac::GetMacNormalizedIssuer(
     LOG(ERROR) << "CreateCertBufferFromBytes";
     return name_data;
   }
-  name_data.reset(SecCertificateCopyNormalizedIssuerSequence(cert_handle));
+  name_data.reset(
+      SecCertificateCopyNormalizedIssuerSequence(cert_handle.get()));
   if (!name_data)
     LOG(ERROR) << "SecCertificateCopyNormalizedIssuerContent";
   return name_data;
