@@ -22,12 +22,8 @@
 #include "crypto/mac_security_services_lock.h"
 #include "crypto/sha2.h"
 #include "net/base/features.h"
+#include "net/cert/internal/test_helpers.h"
 #include "net/cert/internal/trust_store_features.h"
-#include "net/cert/pem.h"
-#include "net/cert/pki/cert_errors.h"
-#include "net/cert/pki/parsed_certificate.h"
-#include "net/cert/pki/test_helpers.h"
-#include "net/cert/pki/trust_store.h"
 #include "net/cert/test_keychain_search_list_mac.h"
 #include "net/cert/x509_certificate.h"
 #include "net/cert/x509_util.h"
@@ -35,6 +31,10 @@
 #include "net/test/test_data_directory.h"
 #include "testing/gmock/include/gmock/gmock.h"
 #include "testing/gtest/include/gtest/gtest.h"
+#include "third_party/boringssl/src/pki/cert_errors.h"
+#include "third_party/boringssl/src/pki/parsed_certificate.h"
+#include "third_party/boringssl/src/pki/pem.h"
+#include "third_party/boringssl/src/pki/trust_store.h"
 
 using ::testing::UnorderedElementsAreArray;
 
@@ -48,7 +48,7 @@ const char kCertificateHeader[] = "CERTIFICATE";
 // Parses a PEM encoded certificate from |file_name| and stores in |result|.
 ::testing::AssertionResult ReadTestCert(
     const std::string& file_name,
-    std::shared_ptr<const ParsedCertificate>* result) {
+    std::shared_ptr<const bssl::ParsedCertificate>* result) {
   std::string der;
   const PemBlockMapping mappings[] = {
       {kCertificateHeader, &der},
@@ -59,12 +59,12 @@ const char kCertificateHeader[] = "CERTIFICATE";
   if (!r)
     return r;
 
-  CertErrors errors;
-  *result = ParsedCertificate::Create(x509_util::CreateCryptoBuffer(der), {},
-                                      &errors);
+  bssl::CertErrors errors;
+  *result = bssl::ParsedCertificate::Create(x509_util::CreateCryptoBuffer(der),
+                                            {}, &errors);
   if (!*result) {
     return ::testing::AssertionFailure()
-           << "ParseCertificate::Create() failed:\n"
+           << "bssl::ParseCertificate::Create() failed:\n"
            << errors.ToDebugString();
   }
   return ::testing::AssertionSuccess();
@@ -72,7 +72,7 @@ const char kCertificateHeader[] = "CERTIFICATE";
 
 // Returns the DER encodings of the ParsedCertificates in |list|.
 std::vector<std::string> ParsedCertificateListAsDER(
-    ParsedCertificateList list) {
+    bssl::ParsedCertificateList list) {
   std::vector<std::string> result;
   for (const auto& it : list)
     result.push_back(it->der_cert().AsString());
@@ -89,7 +89,7 @@ std::set<std::string> ParseFindCertificateOutputToDerCerts(std::string output) {
         hash_and_pem_partial + "\n-----END CERTIFICATE-----\n";
 
     // Parse the PEM encoded text to DER bytes.
-    PEMTokenizer pem_tokenizer(hash_and_pem, {kCertificateHeader});
+    bssl::PEMTokenizer pem_tokenizer(hash_and_pem, {kCertificateHeader});
     if (!pem_tokenizer.GetNext()) {
       ADD_FAILURE() << "!pem_tokenizer.GetNext()";
       continue;
@@ -101,11 +101,6 @@ std::set<std::string> ParseFindCertificateOutputToDerCerts(std::string output) {
   return certs;
 }
 
-class DebugData : public base::SupportsUserData {
- public:
-  ~DebugData() override = default;
-};
-
 const char* TrustImplTypeToString(TrustStoreMac::TrustImplType t) {
   switch (t) {
     case TrustStoreMac::TrustImplType::kSimple:
@@ -149,14 +144,15 @@ class TrustStoreMacImplTest
     return std::get<2>(GetParam());
   }
 
-  CertificateTrust ExpectedTrustForAnchor() const {
-    CertificateTrust trust;
+  bssl::CertificateTrust ExpectedTrustForAnchor() const {
+    bssl::CertificateTrust trust;
 
     if (ExpectedTrustedLeafSupportEnabled()) {
-      trust =
-          CertificateTrust::ForTrustAnchorOrLeaf().WithEnforceAnchorExpiry();
+      trust = bssl::CertificateTrust::ForTrustAnchorOrLeaf()
+                  .WithEnforceAnchorExpiry();
     } else {
-      trust = CertificateTrust::ForTrustAnchor().WithEnforceAnchorExpiry();
+      trust =
+          bssl::CertificateTrust::ForTrustAnchor().WithEnforceAnchorExpiry();
     }
 
     if (ExpectedEnforceLocalAnchorConstraintsEnabled()) {
@@ -196,14 +192,14 @@ TEST_P(TrustStoreMacImplTest, MultiRootNotTrusted) {
                                     keychain.InitializeInto());
   ASSERT_EQ(errSecSuccess, status);
   ASSERT_TRUE(keychain);
-  test_keychain_search_list->AddKeychain(keychain);
+  test_keychain_search_list->AddKeychain(keychain.get());
 
 #pragma clang diagnostic pop
 
   const TrustStoreMac::TrustImplType trust_impl = GetImplParam();
   TrustStoreMac trust_store(kSecPolicyAppleSSL, trust_impl);
 
-  std::shared_ptr<const ParsedCertificate> a_by_b, b_by_c, b_by_f, c_by_d,
+  std::shared_ptr<const bssl::ParsedCertificate> a_by_b, b_by_c, b_by_f, c_by_d,
       c_by_e, f_by_e, d_by_d, e_by_e;
   ASSERT_TRUE(ReadTestCert("multi-root-A-by-B.pem", &a_by_b));
   ASSERT_TRUE(ReadTestCert("multi-root-B-by-C.pem", &b_by_c));
@@ -217,7 +213,7 @@ TEST_P(TrustStoreMacImplTest, MultiRootNotTrusted) {
   // Test that the untrusted keychain certs would be found during issuer
   // searching.
   {
-    ParsedCertificateList found_issuers;
+    bssl::ParsedCertificateList found_issuers;
     trust_store.SyncGetIssuersOf(a_by_b.get(), &found_issuers);
     EXPECT_THAT(ParsedCertificateListAsDER(found_issuers),
                 UnorderedElementsAreArray(
@@ -225,7 +221,7 @@ TEST_P(TrustStoreMacImplTest, MultiRootNotTrusted) {
   }
 
   {
-    ParsedCertificateList found_issuers;
+    bssl::ParsedCertificateList found_issuers;
     trust_store.SyncGetIssuersOf(b_by_c.get(), &found_issuers);
     EXPECT_THAT(ParsedCertificateListAsDER(found_issuers),
                 UnorderedElementsAreArray(
@@ -233,7 +229,7 @@ TEST_P(TrustStoreMacImplTest, MultiRootNotTrusted) {
   }
 
   {
-    ParsedCertificateList found_issuers;
+    bssl::ParsedCertificateList found_issuers;
     trust_store.SyncGetIssuersOf(b_by_f.get(), &found_issuers);
     EXPECT_THAT(
         ParsedCertificateListAsDER(found_issuers),
@@ -241,7 +237,7 @@ TEST_P(TrustStoreMacImplTest, MultiRootNotTrusted) {
   }
 
   {
-    ParsedCertificateList found_issuers;
+    bssl::ParsedCertificateList found_issuers;
     trust_store.SyncGetIssuersOf(c_by_d.get(), &found_issuers);
     EXPECT_THAT(
         ParsedCertificateListAsDER(found_issuers),
@@ -249,7 +245,7 @@ TEST_P(TrustStoreMacImplTest, MultiRootNotTrusted) {
   }
 
   {
-    ParsedCertificateList found_issuers;
+    bssl::ParsedCertificateList found_issuers;
     trust_store.SyncGetIssuersOf(f_by_e.get(), &found_issuers);
     EXPECT_THAT(
         ParsedCertificateListAsDER(found_issuers),
@@ -261,17 +257,9 @@ TEST_P(TrustStoreMacImplTest, MultiRootNotTrusted) {
   // added and trusted the test certs on the machine the test is being run on).
   for (const auto& cert :
        {a_by_b, b_by_c, b_by_f, c_by_d, c_by_e, f_by_e, d_by_d, e_by_e}) {
-    DebugData debug_data;
-    CertificateTrust trust = trust_store.GetTrust(cert.get(), &debug_data);
-    EXPECT_EQ(CertificateTrust::ForUnspecified().ToDebugString(),
+    bssl::CertificateTrust trust = trust_store.GetTrust(cert.get());
+    EXPECT_EQ(bssl::CertificateTrust::ForUnspecified().ToDebugString(),
               trust.ToDebugString());
-    // The combined_trust_debug_info should be 0 since no trust records
-    // should exist for these test certs.
-    const TrustStoreMac::ResultDebugData* trust_debug_data =
-        TrustStoreMac::ResultDebugData::Get(&debug_data);
-    ASSERT_TRUE(trust_debug_data);
-    EXPECT_EQ(0, trust_debug_data->combined_trust_debug_info());
-    EXPECT_EQ(trust_impl, trust_debug_data->trust_impl());
   }
 }
 
@@ -323,19 +311,21 @@ TEST_P(TrustStoreMacImplTest, SystemCerts) {
     std::string hash_text = base::HexEncode(hash.data(), hash.size());
     SCOPED_TRACE(hash_text);
 
-    CertErrors errors;
-    // Note: don't actually need to make a ParsedCertificate here, just need
-    // the DER bytes. But parsing it here ensures the test can skip any certs
-    // that won't be returned due to parsing failures inside TrustStoreMac.
-    // The parsing options set here need to match the ones used in
-    // trust_store_mac.cc.
-    ParseCertificateOptions options;
+    bssl::CertErrors errors;
+    // Note: don't actually need to make a bssl::ParsedCertificate here, just
+    // need the DER bytes. But parsing it here ensures the test can skip any
+    // certs that won't be returned due to parsing failures inside
+    // TrustStoreMac. The parsing options set here need to match the ones used
+    // in trust_store_mac.cc.
+    bssl::ParseCertificateOptions options;
     // For https://crt.sh/?q=D3EEFBCBBCF49867838626E23BB59CA01E305DB7:
     options.allow_invalid_serial_numbers = true;
-    std::shared_ptr<const ParsedCertificate> cert = ParsedCertificate::Create(
-        x509_util::CreateCryptoBuffer(cert_der), options, &errors);
+    std::shared_ptr<const bssl::ParsedCertificate> cert =
+        bssl::ParsedCertificate::Create(x509_util::CreateCryptoBuffer(cert_der),
+                                        options, &errors);
     if (!cert) {
-      LOG(WARNING) << "ParseCertificate::Create " << hash_text << " failed:\n"
+      LOG(WARNING) << "bssl::ParseCertificate::Create " << hash_text
+                   << " failed:\n"
                    << errors.ToDebugString();
       continue;
     }
@@ -349,8 +339,7 @@ TEST_P(TrustStoreMacImplTest, SystemCerts) {
     }
 
     // Check if this cert is considered a trust anchor by TrustStoreMac.
-    DebugData debug_data;
-    CertificateTrust cert_trust = trust_store.GetTrust(cert.get(), &debug_data);
+    bssl::CertificateTrust cert_trust = trust_store.GetTrust(cert.get());
     bool is_trusted = cert_trust.IsTrustAnchor() || cert_trust.IsTrustLeaf();
     if (is_trusted) {
       EXPECT_EQ(ExpectedTrustForAnchor().ToDebugString(),
@@ -361,11 +350,11 @@ TEST_P(TrustStoreMacImplTest, SystemCerts) {
     base::apple::ScopedCFTypeRef<SecTrustRef> trust;
     {
       base::AutoLock lock(crypto::GetMacSecurityServicesLock());
-      ASSERT_EQ(noErr,
-                SecTrustCreateWithCertificates(cert_handle, sec_policy,
-                                               trust.InitializeInto()));
-      ASSERT_EQ(noErr,
-                SecTrustSetOptions(trust, kSecTrustOptionLeafIsCA |
+      ASSERT_EQ(noErr, SecTrustCreateWithCertificates(cert_handle.get(),
+                                                      sec_policy.get(),
+                                                      trust.InitializeInto()));
+      ASSERT_EQ(noErr, SecTrustSetOptions(trust.get(),
+                                          kSecTrustOptionLeafIsCA |
                                               kSecTrustOptionAllowExpired |
                                               kSecTrustOptionAllowExpiredRoot));
 
@@ -380,37 +369,17 @@ TEST_P(TrustStoreMacImplTest, SystemCerts) {
         // Cert is only in the system domain. It should be untrusted.
         EXPECT_FALSE(is_trusted);
       } else {
-        bool trusted = SecTrustEvaluateWithError(trust, nullptr);
+        bool trusted = SecTrustEvaluateWithError(trust.get(), nullptr);
         bool expected_trust_anchor =
-            trusted && (SecTrustGetCertificateCount(trust) == 1);
+            trusted && (SecTrustGetCertificateCount(trust.get()) == 1);
         EXPECT_EQ(expected_trust_anchor, is_trusted);
       }
-      auto* trust_debug_data = TrustStoreMac::ResultDebugData::Get(&debug_data);
-      ASSERT_TRUE(trust_debug_data);
-      if (is_trusted &&
-          trust_impl != TrustStoreMac::TrustImplType::kKeychainCacheFullCerts) {
-        // Since this test queries the real trust store, can't know exactly
-        // what bits should be set in the trust debug info, but if it's trusted
-        // it should at least have something set.
-        EXPECT_NE(0, trust_debug_data->combined_trust_debug_info());
-      }
-      // The impl that was used should be specified in the debug data.
-      EXPECT_EQ(trust_impl, trust_debug_data->trust_impl());
     }
 
     // Call GetTrust again on the same cert. This should exercise the code
     // that checks the trust value for a cert which has already been cached.
-    DebugData debug_data2;
-    CertificateTrust cert_trust2 =
-        trust_store.GetTrust(cert.get(), &debug_data2);
+    bssl::CertificateTrust cert_trust2 = trust_store.GetTrust(cert.get());
     EXPECT_EQ(cert_trust.ToDebugString(), cert_trust2.ToDebugString());
-    auto* trust_debug_data = TrustStoreMac::ResultDebugData::Get(&debug_data);
-    ASSERT_TRUE(trust_debug_data);
-    auto* trust_debug_data2 = TrustStoreMac::ResultDebugData::Get(&debug_data2);
-    ASSERT_TRUE(trust_debug_data2);
-    EXPECT_EQ(trust_debug_data->combined_trust_debug_info(),
-              trust_debug_data2->combined_trust_debug_info());
-    EXPECT_EQ(trust_debug_data->trust_impl(), trust_debug_data2->trust_impl());
   }
 
   // Since this is testing the actual platform certs and trust settings, we
