@@ -5,6 +5,7 @@
 #include "net/cert/internal/trust_store_win.h"
 
 #include <memory>
+#include <string_view>
 
 #include "base/logging.h"
 #include "base/memory/ptr_util.h"
@@ -15,11 +16,8 @@
 #include "crypto/scoped_capi_types.h"
 #include "net/base/features.h"
 #include "net/cert/cert_net_fetcher.h"
+#include "net/cert/internal/test_helpers.h"
 #include "net/cert/internal/trust_store_features.h"
-#include "net/cert/pki/cert_errors.h"
-#include "net/cert/pki/parsed_certificate.h"
-#include "net/cert/pki/test_helpers.h"
-#include "net/cert/pki/trust_store.h"
 #include "net/cert/x509_certificate.h"
 #include "net/cert/x509_util.h"
 #include "net/cert/x509_util_win.h"
@@ -28,26 +26,31 @@
 #include "testing/gmock/include/gmock/gmock.h"
 #include "testing/gtest/include/gtest/gtest.h"
 #include "third_party/boringssl/src/include/openssl/pool.h"
+#include "third_party/boringssl/src/pki/cert_errors.h"
+#include "third_party/boringssl/src/pki/parsed_certificate.h"
+#include "third_party/boringssl/src/pki/trust_store.h"
 
 namespace net {
 
 namespace {
 
 ::testing::AssertionResult ParseCertFromFile(
-    base::StringPiece file_name,
-    std::shared_ptr<const ParsedCertificate>* out_cert) {
+    std::string_view file_name,
+    std::shared_ptr<const bssl::ParsedCertificate>* out_cert) {
   const scoped_refptr<X509Certificate> cert =
       ImportCertFromFile(net::GetTestCertsDirectory(), file_name);
   if (!cert) {
     return ::testing::AssertionFailure() << "ImportCertFromFile failed";
   }
-  CertErrors errors;
-  std::shared_ptr<const ParsedCertificate> parsed = ParsedCertificate::Create(
-      bssl::UpRef(cert->cert_buffer()),
-      x509_util::DefaultParseCertificateOptions(), &errors);
+  bssl::CertErrors errors;
+  std::shared_ptr<const bssl::ParsedCertificate> parsed =
+      bssl::ParsedCertificate::Create(
+          bssl::UpRef(cert->cert_buffer()),
+          x509_util::DefaultParseCertificateOptions(), &errors);
   if (!parsed) {
-    return ::testing::AssertionFailure() << "ParseCertificate::Create failed:\n"
-                                         << errors.ToDebugString();
+    return ::testing::AssertionFailure()
+           << "bssl::ParseCertificate::Create failed:\n"
+           << errors.ToDebugString();
   }
   *out_cert = parsed;
   return ::testing::AssertionSuccess();
@@ -87,32 +90,33 @@ class TrustStoreWinTest
     return std::get<1>(GetParam());
   }
 
-  CertificateTrust ExpectedTrustForAnchor() const {
+  bssl::CertificateTrust ExpectedTrustForAnchor() const {
     if (ExpectedTrustedLeafSupportEnabled()) {
-      return CertificateTrust::ForTrustAnchorOrLeaf()
+      return bssl::CertificateTrust::ForTrustAnchorOrLeaf()
           .WithEnforceAnchorExpiry()
           .WithEnforceAnchorConstraints(
               ExpectedEnforceLocalAnchorConstraintsEnabled())
           .WithRequireLeafSelfSigned();
     } else {
-      return CertificateTrust::ForTrustAnchor()
+      return bssl::CertificateTrust::ForTrustAnchor()
           .WithEnforceAnchorExpiry()
           .WithEnforceAnchorConstraints(
               ExpectedEnforceLocalAnchorConstraintsEnabled());
     }
   }
 
-  CertificateTrust ExpectedTrustForPeer() const {
+  bssl::CertificateTrust ExpectedTrustForPeer() const {
     if (ExpectedTrustedLeafSupportEnabled()) {
-      return CertificateTrust::ForTrustedLeaf().WithRequireLeafSelfSigned();
+      return bssl::CertificateTrust::ForTrustedLeaf()
+          .WithRequireLeafSelfSigned();
     } else {
-      return CertificateTrust::ForUnspecified();
+      return bssl::CertificateTrust::ForUnspecified();
     }
   }
 
   // Returns true if |cert| successfully added to store, false otherwise.
   bool AddToStore(HCERTSTORE store,
-                  std::shared_ptr<const ParsedCertificate> cert) {
+                  std::shared_ptr<const bssl::ParsedCertificate> cert) {
     crypto::ScopedPCCERT_CONTEXT os_cert(CertCreateCertificateContext(
         X509_ASN_ENCODING, CRYPTO_BUFFER_data(cert->cert_buffer()),
         CRYPTO_BUFFER_len(cert->cert_buffer())));
@@ -124,7 +128,7 @@ class TrustStoreWinTest
   // restricted usage, false otherwise.
   bool AddToStoreWithEKURestriction(
       HCERTSTORE store,
-      std::shared_ptr<const ParsedCertificate> cert,
+      std::shared_ptr<const bssl::ParsedCertificate> cert,
       LPCSTR usage_identifier) {
     crypto::ScopedPCCERT_CONTEXT os_cert(CertCreateCertificateContext(
         X509_ASN_ENCODING, CRYPTO_BUFFER_data(cert->cert_buffer()),
@@ -153,8 +157,8 @@ class TrustStoreWinTest
   TrustStoreWin::CertStores stores_ =
       TrustStoreWin::CertStores::CreateInMemoryStoresForTesting();
 
-  std::shared_ptr<const ParsedCertificate> a_by_b_, b_by_c_, b_by_f_, c_by_d_,
-      c_by_e_, d_by_d_, e_by_e_, f_by_e_;
+  std::shared_ptr<const bssl::ParsedCertificate> a_by_b_, b_by_c_, b_by_f_,
+      c_by_d_, c_by_e_, d_by_d_, e_by_e_, f_by_e_;
 
  private:
   base::test::ScopedFeatureList feature_list_;
@@ -168,8 +172,8 @@ TEST_P(TrustStoreWinTest, GetTrustInitializationError) {
       TrustStoreWin::CreateForTesting(
           TrustStoreWin::CertStores::CreateNullStoresForTesting());
   ASSERT_TRUE(trust_store_win);
-  CertificateTrust trust = trust_store_win->GetTrust(d_by_d_.get(), nullptr);
-  EXPECT_EQ(CertificateTrust::ForUnspecified().ToDebugString(),
+  bssl::CertificateTrust trust = trust_store_win->GetTrust(d_by_d_.get());
+  EXPECT_EQ(bssl::CertificateTrust::ForUnspecified().ToDebugString(),
             trust.ToDebugString());
 }
 
@@ -183,23 +187,23 @@ TEST_P(TrustStoreWinTest, GetTrust) {
 
   // Explicitly trusted root should be trusted.
   EXPECT_EQ(ExpectedTrustForAnchor().ToDebugString(),
-            trust_store_win->GetTrust(d_by_d_.get(), nullptr).ToDebugString());
+            trust_store_win->GetTrust(d_by_d_.get()).ToDebugString());
 
   // Explicitly trusted peer should be trusted.
   // (Although it wouldn't actually verify since it's not self-signed but has
   // require_leaf_selfsigned set. That doesn't matter for the purposes of these
   // tests.)
   EXPECT_EQ(ExpectedTrustForPeer().ToDebugString(),
-            trust_store_win->GetTrust(a_by_b_.get(), nullptr).ToDebugString());
+            trust_store_win->GetTrust(a_by_b_.get()).ToDebugString());
 
   // Intermediate for path building should not be trusted.
-  EXPECT_EQ(CertificateTrust::ForUnspecified().ToDebugString(),
-            trust_store_win->GetTrust(c_by_d_.get(), nullptr).ToDebugString());
+  EXPECT_EQ(bssl::CertificateTrust::ForUnspecified().ToDebugString(),
+            trust_store_win->GetTrust(c_by_d_.get()).ToDebugString());
 
   // Unknown roots should not be trusted (e.g. just because they're
   // self-signed doesn't make them a root)
-  EXPECT_EQ(CertificateTrust::ForUnspecified().ToDebugString(),
-            trust_store_win->GetTrust(e_by_e_.get(), nullptr).ToDebugString());
+  EXPECT_EQ(bssl::CertificateTrust::ForUnspecified().ToDebugString(),
+            trust_store_win->GetTrust(e_by_e_.get()).ToDebugString());
 }
 
 // This test has a special TrustStoreWin setup with restricted EKU usages.
@@ -226,25 +230,25 @@ TEST_P(TrustStoreWinTest, GetTrustRestrictedEKU) {
   // Root cert with EKU szOID_PKIX_KP_SERVER_AUTH usage set should be
   // trusted.
   EXPECT_EQ(ExpectedTrustForAnchor().ToDebugString(),
-            trust_store_win->GetTrust(d_by_d_.get(), nullptr).ToDebugString());
+            trust_store_win->GetTrust(d_by_d_.get()).ToDebugString());
 
   // Root cert with EKU szOID_ANY_ENHANCED_KEY_USAGE usage set should be
   // trusted.
   EXPECT_EQ(ExpectedTrustForAnchor().ToDebugString(),
-            trust_store_win->GetTrust(c_by_e_.get(), nullptr).ToDebugString());
+            trust_store_win->GetTrust(c_by_e_.get()).ToDebugString());
 
   // Root cert with EKU szOID_PKIX_KP_CLIENT_AUTH does not allow usage of
   // cert for server auth, return UNSPECIFIED.
-  EXPECT_EQ(CertificateTrust::ForUnspecified().ToDebugString(),
-            trust_store_win->GetTrust(e_by_e_.get(), nullptr).ToDebugString());
+  EXPECT_EQ(bssl::CertificateTrust::ForUnspecified().ToDebugString(),
+            trust_store_win->GetTrust(e_by_e_.get()).ToDebugString());
 
   // Root cert with no EKU usages, return UNSPECIFIED.
-  EXPECT_EQ(CertificateTrust::ForUnspecified().ToDebugString(),
-            trust_store_win->GetTrust(c_by_d_.get(), nullptr).ToDebugString());
+  EXPECT_EQ(bssl::CertificateTrust::ForUnspecified().ToDebugString(),
+            trust_store_win->GetTrust(c_by_d_.get()).ToDebugString());
 
   // Unknown cert has unspecified trust.
-  EXPECT_EQ(CertificateTrust::ForUnspecified().ToDebugString(),
-            trust_store_win->GetTrust(f_by_e_.get(), nullptr).ToDebugString());
+  EXPECT_EQ(bssl::CertificateTrust::ForUnspecified().ToDebugString(),
+            trust_store_win->GetTrust(f_by_e_.get()).ToDebugString());
 }
 
 // Same as GetTrustRestrictedEKU but for the Trusted People store.
@@ -264,25 +268,25 @@ TEST_P(TrustStoreWinTest, GetTrustTrustedPeopleRestrictedEKU) {
   // TrustedPeople cert with EKU szOID_PKIX_KP_SERVER_AUTH usage set should be
   // trusted.
   EXPECT_EQ(ExpectedTrustForPeer().ToDebugString(),
-            trust_store_win->GetTrust(d_by_d_.get(), nullptr).ToDebugString());
+            trust_store_win->GetTrust(d_by_d_.get()).ToDebugString());
 
   // TrustedPeople cert with EKU szOID_ANY_ENHANCED_KEY_USAGE usage set should
   // be trusted.
   EXPECT_EQ(ExpectedTrustForPeer().ToDebugString(),
-            trust_store_win->GetTrust(c_by_e_.get(), nullptr).ToDebugString());
+            trust_store_win->GetTrust(c_by_e_.get()).ToDebugString());
 
   // TrustedPeople cert with EKU szOID_PKIX_KP_CLIENT_AUTH does not allow usage
   // of cert for server auth, return UNSPECIFIED.
-  EXPECT_EQ(CertificateTrust::ForUnspecified().ToDebugString(),
-            trust_store_win->GetTrust(e_by_e_.get(), nullptr).ToDebugString());
+  EXPECT_EQ(bssl::CertificateTrust::ForUnspecified().ToDebugString(),
+            trust_store_win->GetTrust(e_by_e_.get()).ToDebugString());
 
   // TrustedPeople cert with no EKU usages, return UNSPECIFIED.
-  EXPECT_EQ(CertificateTrust::ForUnspecified().ToDebugString(),
-            trust_store_win->GetTrust(c_by_d_.get(), nullptr).ToDebugString());
+  EXPECT_EQ(bssl::CertificateTrust::ForUnspecified().ToDebugString(),
+            trust_store_win->GetTrust(c_by_d_.get()).ToDebugString());
 
   // Unknown cert has unspecified trust.
-  EXPECT_EQ(CertificateTrust::ForUnspecified().ToDebugString(),
-            trust_store_win->GetTrust(f_by_e_.get(), nullptr).ToDebugString());
+  EXPECT_EQ(bssl::CertificateTrust::ForUnspecified().ToDebugString(),
+            trust_store_win->GetTrust(f_by_e_.get()).ToDebugString());
 }
 
 // If duplicate certs are added to the root store with different EKU usages,
@@ -305,7 +309,7 @@ TEST_P(TrustStoreWinTest, GetTrustRestrictedEKUDuplicateCerts) {
 
   // One copy of the Root cert is trusted for TLS Server Auth.
   EXPECT_EQ(ExpectedTrustForAnchor().ToDebugString(),
-            trust_store_win->GetTrust(d_by_d_.get(), nullptr).ToDebugString());
+            trust_store_win->GetTrust(d_by_d_.get()).ToDebugString());
 }
 
 // Test that disallowed certs will be distrusted regardless of EKU settings.
@@ -323,19 +327,19 @@ TEST_P(TrustStoreWinTest, GetTrustDisallowedCerts) {
   ASSERT_TRUE(trust_store_win);
 
   // E-by-E is in both root and distrusted store. Distrust takes precedence.
-  EXPECT_EQ(CertificateTrust::ForDistrusted().ToDebugString(),
-            trust_store_win->GetTrust(e_by_e_.get(), nullptr).ToDebugString());
+  EXPECT_EQ(bssl::CertificateTrust::ForDistrusted().ToDebugString(),
+            trust_store_win->GetTrust(e_by_e_.get()).ToDebugString());
 
   // F-by-E is in both trusted people and distrusted store. Distrust takes
   // precedence.
-  EXPECT_EQ(CertificateTrust::ForDistrusted().ToDebugString(),
-            trust_store_win->GetTrust(f_by_e_.get(), nullptr).ToDebugString());
+  EXPECT_EQ(bssl::CertificateTrust::ForDistrusted().ToDebugString(),
+            trust_store_win->GetTrust(f_by_e_.get()).ToDebugString());
 
   // D-by-D is in root and in distrusted but without szOID_PKIX_KP_SERVER_AUTH
   // set. It should still be distrusted since the EKU settings aren't checked
   // on distrust.
-  EXPECT_EQ(CertificateTrust::ForDistrusted().ToDebugString(),
-            trust_store_win->GetTrust(d_by_d_.get(), nullptr).ToDebugString());
+  EXPECT_EQ(bssl::CertificateTrust::ForDistrusted().ToDebugString(),
+            trust_store_win->GetTrust(d_by_d_.get()).ToDebugString());
 }
 
 MATCHER_P(ParsedCertEq, expected_cert, "") {
@@ -350,7 +354,7 @@ TEST_P(TrustStoreWinTest, GetIssuersInitializationError) {
       TrustStoreWin::CreateForTesting(
           TrustStoreWin::CertStores::CreateNullStoresForTesting());
   ASSERT_TRUE(trust_store_win);
-  ParsedCertificateList issuers;
+  bssl::ParsedCertificateList issuers;
   trust_store_win->SyncGetIssuersOf(b_by_f_.get(), &issuers);
   ASSERT_EQ(0U, issuers.size());
 }
@@ -370,14 +374,14 @@ TEST_P(TrustStoreWinTest, GetIssuers) {
 
   // No matching issuer (Trusted People and Disallowed are not consulted).
   {
-    ParsedCertificateList issuers;
+    bssl::ParsedCertificateList issuers;
     trust_store_win->SyncGetIssuersOf(a_by_b_.get(), &issuers);
     ASSERT_EQ(0U, issuers.size());
   }
 
   // Single matching issuer found in intermediates.
   {
-    ParsedCertificateList issuers;
+    bssl::ParsedCertificateList issuers;
     trust_store_win->SyncGetIssuersOf(b_by_f_.get(), &issuers);
     ASSERT_EQ(1U, issuers.size());
     EXPECT_THAT(issuers, testing::UnorderedElementsAre(ParsedCertEq(f_by_e_)));
@@ -385,7 +389,7 @@ TEST_P(TrustStoreWinTest, GetIssuers) {
 
   // Single matching issuer found in roots.
   {
-    ParsedCertificateList issuers;
+    bssl::ParsedCertificateList issuers;
     trust_store_win->SyncGetIssuersOf(d_by_d_.get(), &issuers);
     ASSERT_EQ(1U, issuers.size());
     EXPECT_THAT(issuers, testing::UnorderedElementsAre(ParsedCertEq(d_by_d_)));
@@ -393,7 +397,7 @@ TEST_P(TrustStoreWinTest, GetIssuers) {
 
   // Multiple issuers found.
   {
-    ParsedCertificateList issuers;
+    bssl::ParsedCertificateList issuers;
     trust_store_win->SyncGetIssuersOf(b_by_c_.get(), &issuers);
     ASSERT_EQ(2U, issuers.size());
     EXPECT_THAT(issuers, testing::UnorderedElementsAre(ParsedCertEq(c_by_d_),
