@@ -7,6 +7,7 @@
 #include <stdint.h>
 
 #include <memory>
+#include <string_view>
 
 #include "base/files/file_path.h"
 #include "base/files/file_util.h"
@@ -20,14 +21,14 @@
 #include "crypto/rsa_private_key.h"
 #include "net/base/net_errors.h"
 #include "net/cert/asn1_util.h"
-#include "net/cert/pem.h"
-#include "net/cert/pki/parse_certificate.h"
 #include "net/cert/x509_util.h"
 #include "net/test/cert_builder.h"
 #include "net/test/cert_test_util.h"
 #include "net/test/test_certificate_data.h"
 #include "net/test/test_data_directory.h"
 #include "testing/gtest/include/gtest/gtest.h"
+#include "third_party/boringssl/src/pki/parse_certificate.h"
+#include "third_party/boringssl/src/pki/pem.h"
 
 using base::HexEncode;
 using base::Time;
@@ -99,10 +100,10 @@ void CheckGoogleCert(const scoped_refptr<X509Certificate>& google_cert,
 
   // Use DoubleT because its epoch is the same on all platforms
   const Time& valid_start = google_cert->valid_start();
-  EXPECT_EQ(valid_from, valid_start.ToDoubleT());
+  EXPECT_EQ(valid_from, valid_start.InSecondsFSinceUnixEpoch());
 
   const Time& valid_expiry = google_cert->valid_expiry();
-  EXPECT_EQ(valid_to, valid_expiry.ToDoubleT());
+  EXPECT_EQ(valid_to, valid_expiry.InSecondsFSinceUnixEpoch());
 
   EXPECT_EQ(expected_fingerprint, X509Certificate::CalculateFingerprint256(
                                       google_cert->cert_buffer()));
@@ -158,10 +159,14 @@ TEST(X509CertificateTest, WebkitCertParsing) {
 
   // Use DoubleT because its epoch is the same on all platforms
   const Time& valid_start = webkit_cert->valid_start();
-  EXPECT_EQ(1205883319, valid_start.ToDoubleT());  // Mar 18 23:35:19 2008 GMT
+  EXPECT_EQ(
+      1205883319,
+      valid_start.InSecondsFSinceUnixEpoch());  // Mar 18 23:35:19 2008 GMT
 
   const Time& valid_expiry = webkit_cert->valid_expiry();
-  EXPECT_EQ(1300491319, valid_expiry.ToDoubleT());  // Mar 18 23:35:19 2011 GMT
+  EXPECT_EQ(
+      1300491319,
+      valid_expiry.InSecondsFSinceUnixEpoch());  // Mar 18 23:35:19 2011 GMT
 
   std::vector<std::string> dns_names;
   EXPECT_TRUE(webkit_cert->GetSubjectAltName(&dns_names, nullptr));
@@ -205,14 +210,18 @@ TEST(X509CertificateTest, ThawteCertParsing) {
 
   // Use DoubleT because its epoch is the same on all platforms
   const Time& valid_start = thawte_cert->valid_start();
-  EXPECT_EQ(1227052800, valid_start.ToDoubleT());  // Nov 19 00:00:00 2008 GMT
+  EXPECT_EQ(
+      1227052800,
+      valid_start.InSecondsFSinceUnixEpoch());  // Nov 19 00:00:00 2008 GMT
 
   const Time& valid_expiry = thawte_cert->valid_expiry();
-  EXPECT_EQ(1263772799, valid_expiry.ToDoubleT());  // Jan 17 23:59:59 2010 GMT
+  EXPECT_EQ(
+      1263772799,
+      valid_expiry.InSecondsFSinceUnixEpoch());  // Jan 17 23:59:59 2010 GMT
 }
 
 // Test that all desired AttributeAndValue pairs can be extracted when only
-// a single RelativeDistinguishedName is present. "Normally" there is only
+// a single bssl::RelativeDistinguishedName is present. "Normally" there is only
 // one AVA per RDN, but some CAs place all AVAs within a single RDN.
 // This is a regression test for http://crbug.com/101009
 TEST(X509CertificateTest, MultivalueRDN) {
@@ -265,7 +274,7 @@ TEST(X509CertificateTest, InvalidPrintableStringIsUtf8) {
           "subject_printable_string_containing_utf8_client_cert.pem"),
       &file_data));
 
-  net::PEMTokenizer pem_tokenizer(file_data, {"CERTIFICATE"});
+  bssl::PEMTokenizer pem_tokenizer(file_data, {"CERTIFICATE"});
   ASSERT_TRUE(pem_tokenizer.GetNext());
   std::string cert_der(pem_tokenizer.data());
   ASSERT_FALSE(pem_tokenizer.GetNext());
@@ -566,7 +575,7 @@ TEST(X509CertificateTest, ExtractSPKIFromDERCert) {
       ImportCertFromFile(certs_dir, "nist.der");
   ASSERT_NE(static_cast<X509Certificate*>(nullptr), cert.get());
 
-  base::StringPiece spkiBytes;
+  std::string_view spkiBytes;
   EXPECT_TRUE(asn1::ExtractSPKIFromDERCert(
       x509_util::CryptoBufferAsStringPiece(cert->cert_buffer()), &spkiBytes));
 
@@ -614,20 +623,20 @@ TEST(X509CertificateTest, ExtractExtension) {
   ASSERT_TRUE(cert);
 
   bool present, critical;
-  base::StringPiece contents;
+  std::string_view contents;
   ASSERT_TRUE(asn1::ExtractExtensionFromDERCert(
       x509_util::CryptoBufferAsStringPiece(cert->cert_buffer()),
-      der::Input(kBasicConstraintsOid).AsStringView(), &present, &critical,
-      &contents));
+      bssl::der::Input(bssl::kBasicConstraintsOid).AsStringView(), &present,
+      &critical, &contents));
   EXPECT_TRUE(present);
   EXPECT_TRUE(critical);
-  ASSERT_EQ(base::StringPiece("\x30\x00", 2), contents);
+  ASSERT_EQ(std::string_view("\x30\x00", 2), contents);
 
   static constexpr uint8_t kNonsenseOID[] = {0x56, 0x1d, 0x13};
   ASSERT_TRUE(asn1::ExtractExtensionFromDERCert(
       x509_util::CryptoBufferAsStringPiece(cert->cert_buffer()),
-      base::StringPiece(reinterpret_cast<const char*>(kNonsenseOID),
-                        sizeof(kNonsenseOID)),
+      std::string_view(reinterpret_cast<const char*>(kNonsenseOID),
+                       sizeof(kNonsenseOID)),
       &present, &critical, &contents));
   ASSERT_FALSE(present);
 
@@ -636,11 +645,11 @@ TEST(X509CertificateTest, ExtractExtension) {
   ASSERT_TRUE(uid_cert);
   ASSERT_TRUE(asn1::ExtractExtensionFromDERCert(
       x509_util::CryptoBufferAsStringPiece(uid_cert->cert_buffer()),
-      der::Input(kBasicConstraintsOid).AsStringView(), &present, &critical,
-      &contents));
+      bssl::der::Input(bssl::kBasicConstraintsOid).AsStringView(), &present,
+      &critical, &contents));
   EXPECT_TRUE(present);
   EXPECT_FALSE(critical);
-  ASSERT_EQ(base::StringPiece("\x30\x00", 2), contents);
+  ASSERT_EQ(std::string_view("\x30\x00", 2), contents);
 }
 
 // Tests CRYPTO_BUFFER deduping via X509Certificate::CreateFromBuffer.  We
@@ -992,7 +1001,7 @@ TEST(X509CertificateTest, IsSelfSigned) {
 
   constexpr char invalid_cert_data[] = "this is not a certificate";
   bssl::UniquePtr<CRYPTO_BUFFER> invalid_cert_handle =
-      x509_util::CreateCryptoBuffer(base::StringPiece(invalid_cert_data));
+      x509_util::CreateCryptoBuffer(std::string_view(invalid_cert_data));
   ASSERT_TRUE(invalid_cert_handle);
   EXPECT_FALSE(X509Certificate::IsSelfSigned(invalid_cert_handle.get()));
 }
