@@ -5,8 +5,10 @@
 #include "net/cert/x509_util.h"
 
 #include <string.h>
+
 #include <map>
 #include <memory>
+#include <string_view>
 
 #include "base/lazy_instance.h"
 #include "base/logging.h"
@@ -21,15 +23,8 @@
 #include "crypto/sha2.h"
 #include "net/base/hash_value.h"
 #include "net/cert/asn1_util.h"
-#include "net/cert/pki/cert_errors.h"
-#include "net/cert/pki/name_constraints.h"
-#include "net/cert/pki/parse_certificate.h"
-#include "net/cert/pki/parse_name.h"
-#include "net/cert/pki/signature_algorithm.h"
 #include "net/cert/time_conversions.h"
 #include "net/cert/x509_certificate.h"
-#include "net/der/input.h"
-#include "net/der/parse_values.h"
 #include "third_party/boringssl/src/include/openssl/bytestring.h"
 #include "third_party/boringssl/src/include/openssl/digest.h"
 #include "third_party/boringssl/src/include/openssl/evp.h"
@@ -37,6 +32,13 @@
 #include "third_party/boringssl/src/include/openssl/pkcs7.h"
 #include "third_party/boringssl/src/include/openssl/pool.h"
 #include "third_party/boringssl/src/include/openssl/stack.h"
+#include "third_party/boringssl/src/pki/cert_errors.h"
+#include "third_party/boringssl/src/pki/input.h"
+#include "third_party/boringssl/src/pki/name_constraints.h"
+#include "third_party/boringssl/src/pki/parse_certificate.h"
+#include "third_party/boringssl/src/pki/parse_name.h"
+#include "third_party/boringssl/src/pki/parse_values.h"
+#include "third_party/boringssl/src/pki/signature_algorithm.h"
 
 namespace net::x509_util {
 
@@ -99,7 +101,7 @@ base::LazyInstance<BufferPoolSingleton>::Leaky g_buffer_pool_singleton =
 }  // namespace
 
 // Adds an X.509 Name with the specified distinguished name to |cbb|.
-bool AddName(CBB* cbb, base::StringPiece name) {
+bool AddName(CBB* cbb, std::string_view name) {
   // See RFC 4519.
   static const uint8_t kCommonName[] = {0x55, 0x04, 0x03};
   static const uint8_t kCountryName[] = {0x55, 0x04, 0x06};
@@ -167,8 +169,37 @@ bool AddName(CBB* cbb, base::StringPiece name) {
   return true;
 }
 
+NET_EXPORT net::CertificateList ConvertToX509CertificatesIgnoreErrors(
+    const std::vector<std::vector<uint8_t>>& certs_bytes) {
+  net::CertificateList x509_certs;
+  for (const auto& cert_uint8 : certs_bytes) {
+    scoped_refptr<net::X509Certificate> x509_cert =
+        net::X509Certificate::CreateFromBytes(base::as_byte_span(cert_uint8));
+    if (x509_cert) {
+      x509_certs.push_back(std::move(x509_cert));
+    }
+  }
+  return x509_certs;
+}
+
+bssl::ParsedCertificateList ParseAllValidCerts(
+    const CertificateList& x509_certs) {
+  bssl::ParsedCertificateList parsed_certs;
+  for (const auto& x509_cert : x509_certs) {
+    std::shared_ptr<const bssl::ParsedCertificate> cert =
+        bssl::ParsedCertificate::Create(
+            bssl::UpRef(x509_cert->cert_buffer()),
+            net::x509_util::DefaultParseCertificateOptions(), NULL);
+    if (cert) {
+      parsed_certs.push_back(std::move(cert));
+    }
+  }
+
+  return parsed_certs;
+}
+
 bool CBBAddTime(CBB* cbb, base::Time time) {
-  der::GeneralizedTime generalized_time;
+  bssl::der::GeneralizedTime generalized_time;
   if (!EncodeTimeAsGeneralizedTime(time, &generalized_time)) {
     return false;
   }
@@ -179,62 +210,63 @@ bool CBBAddTime(CBB* cbb, base::Time time) {
   uint8_t* out;
   if (generalized_time.InUTCTimeRange()) {
     return CBB_add_asn1(cbb, &child, CBS_ASN1_UTCTIME) &&
-           CBB_add_space(&child, &out, der::kUTCTimeLength) &&
-           der::EncodeUTCTime(generalized_time, out) && CBB_flush(cbb);
+           CBB_add_space(&child, &out, bssl::der::kUTCTimeLength) &&
+           bssl::der::EncodeUTCTime(generalized_time, out) && CBB_flush(cbb);
   }
 
   return CBB_add_asn1(cbb, &child, CBS_ASN1_GENERALIZEDTIME) &&
-         CBB_add_space(&child, &out, der::kGeneralizedTimeLength) &&
-         der::EncodeGeneralizedTime(generalized_time, out) && CBB_flush(cbb);
+         CBB_add_space(&child, &out, bssl::der::kGeneralizedTimeLength) &&
+         bssl::der::EncodeGeneralizedTime(generalized_time, out) &&
+         CBB_flush(cbb);
 }
 
 bool GetTLSServerEndPointChannelBinding(const X509Certificate& certificate,
                                         std::string* token) {
   static const char kChannelBindingPrefix[] = "tls-server-end-point:";
 
-  base::StringPiece der_encoded_certificate =
+  std::string_view der_encoded_certificate =
       x509_util::CryptoBufferAsStringPiece(certificate.cert_buffer());
 
-  der::Input tbs_certificate_tlv;
-  der::Input signature_algorithm_tlv;
-  der::BitString signature_value;
-  if (!ParseCertificate(der::Input(der_encoded_certificate),
-                        &tbs_certificate_tlv, &signature_algorithm_tlv,
-                        &signature_value, nullptr)) {
+  bssl::der::Input tbs_certificate_tlv;
+  bssl::der::Input signature_algorithm_tlv;
+  bssl::der::BitString signature_value;
+  if (!bssl::ParseCertificate(bssl::der::Input(der_encoded_certificate),
+                              &tbs_certificate_tlv, &signature_algorithm_tlv,
+                              &signature_value, nullptr)) {
     return false;
   }
-  absl::optional<SignatureAlgorithm> signature_algorithm =
-      ParseSignatureAlgorithm(signature_algorithm_tlv);
+  std::optional<bssl::SignatureAlgorithm> signature_algorithm =
+      bssl::ParseSignatureAlgorithm(signature_algorithm_tlv);
   if (!signature_algorithm) {
     return false;
   }
 
-  absl::optional<net::DigestAlgorithm> binding_digest =
-      GetTlsServerEndpointDigestAlgorithm(*signature_algorithm);
+  std::optional<bssl::DigestAlgorithm> binding_digest =
+      bssl::GetTlsServerEndpointDigestAlgorithm(*signature_algorithm);
   if (!binding_digest) {
     return false;
   }
   const EVP_MD* digest_evp_md = nullptr;
   switch (binding_digest.value()) {
-    case net::DigestAlgorithm::Md2:
-    case net::DigestAlgorithm::Md4:
-    case net::DigestAlgorithm::Md5:
-    case net::DigestAlgorithm::Sha1:
+    case bssl::DigestAlgorithm::Md2:
+    case bssl::DigestAlgorithm::Md4:
+    case bssl::DigestAlgorithm::Md5:
+    case bssl::DigestAlgorithm::Sha1:
       // Legacy digests are not supported, and
       // `GetTlsServerEndpointDigestAlgorithm` internally maps MD5 and SHA-1 to
       // SHA-256.
       NOTREACHED();
       break;
 
-    case net::DigestAlgorithm::Sha256:
+    case bssl::DigestAlgorithm::Sha256:
       digest_evp_md = EVP_sha256();
       break;
 
-    case net::DigestAlgorithm::Sha384:
+    case bssl::DigestAlgorithm::Sha384:
       digest_evp_md = EVP_sha384();
       break;
 
-    case net::DigestAlgorithm::Sha512:
+    case bssl::DigestAlgorithm::Sha512:
       digest_evp_md = EVP_sha512();
       break;
   }
@@ -391,7 +423,7 @@ bssl::UniquePtr<CRYPTO_BUFFER> CreateCryptoBuffer(
       CRYPTO_BUFFER_new(data.data(), data.size(), GetBufferPool()));
 }
 
-bssl::UniquePtr<CRYPTO_BUFFER> CreateCryptoBuffer(base::StringPiece data) {
+bssl::UniquePtr<CRYPTO_BUFFER> CreateCryptoBuffer(std::string_view data) {
   return bssl::UniquePtr<CRYPTO_BUFFER>(
       CRYPTO_BUFFER_new(reinterpret_cast<const uint8_t*>(data.data()),
                         data.size(), GetBufferPool()));
@@ -413,8 +445,8 @@ bool CryptoBufferEqual(const CRYPTO_BUFFER* a, const CRYPTO_BUFFER* b) {
                 CRYPTO_BUFFER_len(a)) == 0;
 }
 
-base::StringPiece CryptoBufferAsStringPiece(const CRYPTO_BUFFER* buffer) {
-  return base::StringPiece(
+std::string_view CryptoBufferAsStringPiece(const CRYPTO_BUFFER* buffer) {
+  return std::string_view(
       reinterpret_cast<const char*>(CRYPTO_BUFFER_data(buffer)),
       CRYPTO_BUFFER_len(buffer));
 }
@@ -464,14 +496,14 @@ bool CreateCertBuffersFromPKCS7Bytes(
   return success;
 }
 
-ParseCertificateOptions DefaultParseCertificateOptions() {
-  ParseCertificateOptions options;
+bssl::ParseCertificateOptions DefaultParseCertificateOptions() {
+  bssl::ParseCertificateOptions options;
   options.allow_invalid_serial_numbers = true;
   return options;
 }
 
 bool CalculateSha256SpkiHash(const CRYPTO_BUFFER* buffer, HashValue* hash) {
-  base::StringPiece spki;
+  std::string_view spki;
   if (!asn1::ExtractSPKIFromDERCert(CryptoBufferAsStringPiece(buffer), &spki)) {
     return false;
   }
@@ -485,15 +517,15 @@ bool SignatureVerifierInitWithCertificate(
     crypto::SignatureVerifier::SignatureAlgorithm signature_algorithm,
     base::span<const uint8_t> signature,
     const CRYPTO_BUFFER* certificate) {
-  base::StringPiece cert_der =
-      x509_util::CryptoBufferAsStringPiece(certificate);
+  std::string_view cert_der = x509_util::CryptoBufferAsStringPiece(certificate);
 
-  der::Input tbs_certificate_tlv;
-  der::Input signature_algorithm_tlv;
-  der::BitString signature_value;
-  ParsedTbsCertificate tbs;
-  if (!ParseCertificate(der::Input(cert_der), &tbs_certificate_tlv,
-                        &signature_algorithm_tlv, &signature_value, nullptr) ||
+  bssl::der::Input tbs_certificate_tlv;
+  bssl::der::Input signature_algorithm_tlv;
+  bssl::der::BitString signature_value;
+  bssl::ParsedTbsCertificate tbs;
+  if (!bssl::ParseCertificate(bssl::der::Input(cert_der), &tbs_certificate_tlv,
+                              &signature_algorithm_tlv, &signature_value,
+                              nullptr) ||
       !ParseTbsCertificate(tbs_certificate_tlv,
                            DefaultParseCertificateOptions(), &tbs, nullptr)) {
     return false;
@@ -501,16 +533,16 @@ bool SignatureVerifierInitWithCertificate(
 
   // The key usage extension, if present, must assert the digitalSignature bit.
   if (tbs.extensions_tlv) {
-    std::map<der::Input, ParsedExtension> extensions;
+    std::map<bssl::der::Input, bssl::ParsedExtension> extensions;
     if (!ParseExtensions(tbs.extensions_tlv.value(), &extensions)) {
       return false;
     }
-    ParsedExtension key_usage_ext;
-    if (ConsumeExtension(der::Input(kKeyUsageOid), &extensions,
+    bssl::ParsedExtension key_usage_ext;
+    if (ConsumeExtension(bssl::der::Input(bssl::kKeyUsageOid), &extensions,
                          &key_usage_ext)) {
-      der::BitString key_usage;
-      if (!ParseKeyUsage(key_usage_ext.value, &key_usage) ||
-          !key_usage.AssertsBit(KEY_USAGE_BIT_DIGITAL_SIGNATURE)) {
+      bssl::der::BitString key_usage;
+      if (!bssl::ParseKeyUsage(key_usage_ext.value, &key_usage) ||
+          !key_usage.AssertsBit(bssl::KEY_USAGE_BIT_DIGITAL_SIGNATURE)) {
         return false;
       }
     }
@@ -522,21 +554,21 @@ bool SignatureVerifierInitWithCertificate(
 }
 
 bool HasRsaPkcs1Sha1Signature(const CRYPTO_BUFFER* cert_buffer) {
-  der::Input tbs_certificate_tlv;
-  der::Input signature_algorithm_tlv;
-  der::BitString signature_value;
-  if (!ParseCertificate(der::Input(CRYPTO_BUFFER_data(cert_buffer),
-                                   CRYPTO_BUFFER_len(cert_buffer)),
-                        &tbs_certificate_tlv, &signature_algorithm_tlv,
-                        &signature_value, /*out_errors=*/nullptr)) {
+  bssl::der::Input tbs_certificate_tlv;
+  bssl::der::Input signature_algorithm_tlv;
+  bssl::der::BitString signature_value;
+  if (!bssl::ParseCertificate(bssl::der::Input(CRYPTO_BUFFER_data(cert_buffer),
+                                               CRYPTO_BUFFER_len(cert_buffer)),
+                              &tbs_certificate_tlv, &signature_algorithm_tlv,
+                              &signature_value, /*out_errors=*/nullptr)) {
     return false;
   }
 
-  absl::optional<SignatureAlgorithm> signature_algorithm =
-      ParseSignatureAlgorithm(signature_algorithm_tlv);
+  std::optional<bssl::SignatureAlgorithm> signature_algorithm =
+      bssl::ParseSignatureAlgorithm(signature_algorithm_tlv);
 
   return signature_algorithm &&
-         *signature_algorithm == SignatureAlgorithm::kRsaPkcs1Sha1;
+         *signature_algorithm == bssl::SignatureAlgorithm::kRsaPkcs1Sha1;
 }
 
 }  // namespace net::x509_util
