@@ -46,6 +46,18 @@ bool CookieInclusionStatus::operator!=(
   return !operator==(other);
 }
 
+bool CookieInclusionStatus::operator<(
+    const CookieInclusionStatus& other) const {
+  static_assert(NUM_EXCLUSION_REASONS <= sizeof(unsigned long) * CHAR_BIT,
+                "use .ullong() instead");
+  static_assert(NUM_WARNING_REASONS <= sizeof(unsigned long) * CHAR_BIT,
+                "use .ullong() instead");
+  return std::make_pair(exclusion_reasons_.to_ulong(),
+                        warning_reasons_.to_ulong()) <
+         std::make_pair(other.exclusion_reasons_.to_ulong(),
+                        other.warning_reasons_.to_ulong());
+}
+
 bool CookieInclusionStatus::IsInclude() const {
   return exclusion_reasons_.none();
 }
@@ -64,6 +76,9 @@ void CookieInclusionStatus::AddExclusionReason(ExclusionReason reason) {
   // If the cookie would be excluded for reasons other than the new SameSite
   // rules, don't bother warning about it.
   MaybeClearSameSiteWarning();
+  // If the cookie would be excluded for reasons unrelated to 3pcd, don't bother
+  // warning about 3pcd.
+  MaybeClearThirdPartyPhaseoutReason();
 }
 
 void CookieInclusionStatus::RemoveExclusionReason(ExclusionReason reason) {
@@ -106,6 +121,18 @@ void CookieInclusionStatus::MaybeClearSameSiteWarning() {
   }
 }
 
+void CookieInclusionStatus::MaybeClearThirdPartyPhaseoutReason() {
+  if (!IsInclude()) {
+    RemoveWarningReason(WARN_THIRD_PARTY_PHASEOUT);
+  }
+  if (ExclusionReasonsWithout(
+          {EXCLUDE_THIRD_PARTY_PHASEOUT,
+           EXCLUDE_THIRD_PARTY_BLOCKED_WITHIN_FIRST_PARTY_SET}) != 0u) {
+    RemoveExclusionReason(EXCLUDE_THIRD_PARTY_PHASEOUT);
+    RemoveExclusionReason(EXCLUDE_THIRD_PARTY_BLOCKED_WITHIN_FIRST_PARTY_SET);
+  }
+}
+
 bool CookieInclusionStatus::ShouldRecordDowngradeMetrics() const {
   return ExclusionReasonsWithout({
              EXCLUDE_SAMESITE_STRICT,
@@ -231,9 +258,7 @@ std::string CookieInclusionStatus::GetDebugString() const {
       {EXCLUDE_SHADOWING_DOMAIN, "EXCLUDE_SHADOWING_DOMAIN"},
       {EXCLUDE_DISALLOWED_CHARACTER, "EXCLUDE_DISALLOWED_CHARACTER"},
       {EXCLUDE_THIRD_PARTY_PHASEOUT, "EXCLUDE_THIRD_PARTY_PHASEOUT"},
-      {EXCLUDE_SAMEPARTY_CROSS_PARTY_CONTEXT,
-       "EXCLUDE_SAMEPARTY_CROSS_PARTY_CONTEXT"},
-      {EXCLUDE_INVALID_SAMEPARTY, "EXCLUDE_INVALID_SAMEPARTY"},
+      {EXCLUDE_NO_COOKIE_CONTENT, "EXCLUDE_NO_COOKIE_CONTENT"},
   };
   static_assert(
       std::size(exclusion_reasons) == ExclusionReason::NUM_EXCLUSION_REASONS,
@@ -283,11 +308,6 @@ std::string CookieInclusionStatus::GetDebugString() const {
        "WARN_TENTATIVELY_ALLOWING_SECURE_SOURCE_SCHEME"},
       {WARN_SHADOWING_DOMAIN, "WARN_SHADOWING_DOMAIN"},
       {WARN_THIRD_PARTY_PHASEOUT, "WARN_THIRD_PARTY_PHASEOUT"},
-      {WARN_TREATED_AS_SAMEPARTY, "WARN_TREATED_AS_SAMEPARTY"},
-      {WARN_SAMEPARTY_EXCLUSION_OVERRULED_SAMESITE,
-       "WARN_SAMEPARTY_EXCLUSION_OVERRULED_SAMESITE"},
-      {WARN_SAMEPARTY_INCLUSION_OVERRULED_SAMESITE,
-       "WARN_SAMEPARTY_INCLUSION_OVERRULED_SAMESITE"},
   };
   static_assert(
       std::size(warning_reasons) == WarningReason::NUM_WARNING_REASONS,
@@ -346,10 +366,13 @@ CookieInclusionStatus CookieInclusionStatus::MakeFromReasonsForTesting(
 }
 
 bool CookieInclusionStatus::ExcludedByUserPreferences() const {
-  if (HasOnlyExclusionReason(ExclusionReason::EXCLUDE_USER_PREFERENCES))
+  if (HasOnlyExclusionReason(ExclusionReason::EXCLUDE_USER_PREFERENCES) ||
+      HasOnlyExclusionReason(ExclusionReason::EXCLUDE_THIRD_PARTY_PHASEOUT)) {
     return true;
+  }
   return exclusion_reasons_.count() == 2 &&
-         exclusion_reasons_[ExclusionReason::EXCLUDE_USER_PREFERENCES] &&
+         (exclusion_reasons_[ExclusionReason::EXCLUDE_USER_PREFERENCES] ||
+          exclusion_reasons_[ExclusionReason::EXCLUDE_THIRD_PARTY_PHASEOUT]) &&
          exclusion_reasons_
              [ExclusionReason::
                   EXCLUDE_THIRD_PARTY_BLOCKED_WITHIN_FIRST_PARTY_SET];
