@@ -78,8 +78,7 @@ class NET_EXPORT CookieInclusionStatus {
     // Cookie was set with an invalid __Host- or __Secure- prefix.
     EXCLUDE_INVALID_PREFIX = 15,
     /// Cookie was set with an invalid Partitioned attribute, which is only
-    // valid if the cookie has a __Host- prefix and does not have the SameParty
-    // attribute.
+    // valid if the cookie has a __Host- prefix.
     EXCLUDE_INVALID_PARTITIONED = 16,
     // Cookie exceeded the name/value pair size limit.
     EXCLUDE_NAME_VALUE_PAIR_EXCEEDS_MAX_SIZE = 17,
@@ -107,12 +106,8 @@ class NET_EXPORT CookieInclusionStatus {
     EXCLUDE_DISALLOWED_CHARACTER = 24,
     // Cookie is blocked for third-party cookie phaseout.
     EXCLUDE_THIRD_PARTY_PHASEOUT = 25,
-    // The cookie specified SameParty, but was used in a cross-party context.
-    EXCLUDE_SAMEPARTY_CROSS_PARTY_CONTEXT = 26,
-    // Cookie was set with an invalid SameParty attribute in combination with
-    // other attributes. (SameParty is invalid if Secure is not present, or if
-    // SameSite=Strict is present.)
-    EXCLUDE_INVALID_SAMEPARTY = 27,
+    // Cookie contains no content or only whitespace.
+    EXCLUDE_NO_COOKIE_CONTENT = 26,
 
     // This should be kept last.
     NUM_EXCLUSION_REASONS
@@ -232,24 +227,6 @@ class NET_EXPORT CookieInclusionStatus {
     // This cookie will be blocked for third-party cookie phaseout.
     WARN_THIRD_PARTY_PHASEOUT = 16,
 
-    // The cookie was treated as SameParty. This is different from looking at
-    // whether the cookie has the SameParty attribute, since we may choose to
-    // ignore that attribute for one reason or another. E.g., we ignore the
-    // SameParty attribute if the site is not a member of a nontrivial
-    // First-Party Set.
-    WARN_TREATED_AS_SAMEPARTY = 17,
-
-    // The cookie was excluded solely for SameParty reasons (i.e. it was in
-    // cross-party context), but would have been included by SameSite. (This can
-    // only occur in cross-party, cross-site contexts, for cookies that are
-    // 'SameParty; SameSite=None'.)
-    WARN_SAMEPARTY_EXCLUSION_OVERRULED_SAMESITE = 18,
-
-    // The cookie was included due to SameParty, even though it would have been
-    // excluded by SameSite. (This can only occur in same-party, cross-site
-    // contexts, for cookies that are 'SameParty; SameSite=Lax'.)
-    WARN_SAMEPARTY_INCLUSION_OVERRULED_SAMESITE = 19,
-
     // This should be kept last.
     NUM_WARNING_REASONS
   };
@@ -309,6 +286,7 @@ class NET_EXPORT CookieInclusionStatus {
 
   bool operator==(const CookieInclusionStatus& other) const;
   bool operator!=(const CookieInclusionStatus& other) const;
+  bool operator<(const CookieInclusionStatus& other) const;
 
   // Whether the status is to include the cookie, and has no other reasons for
   // exclusion.
@@ -400,9 +378,7 @@ class NET_EXPORT CookieInclusionStatus {
 
   // Returns true if the cookie was excluded because of user preferences.
   // HasOnlyExclusionReason(EXCLUDE_USER_PREFERENCES) will not return true for
-  // third-party cookies blocked in sites in the same First-Party Set (note:
-  // this is not the same as the cookie being blocked in a same-party context,
-  // which takes the entire ancestor chain into account). See
+  // third-party cookies blocked in sites in the same First-Party Set. See
   // https://crbug.com/1366868.
   bool ExcludedByUserPreferences() const;
 
@@ -416,6 +392,11 @@ class NET_EXPORT CookieInclusionStatus {
   ExclusionReasonBitset ExclusionReasonsWithout(
       const std::vector<ExclusionReason>& reasons) const;
 
+  // If the cookie would have been excluded by reasons that are not
+  // Third-party cookie phaseout related, clear the Third-party cookie phaseout
+  // warning/exclusion reason in this case.
+  void MaybeClearThirdPartyPhaseoutReason();
+
   // A bit vector of the applicable exclusion reasons.
   ExclusionReasonBitset exclusion_reasons_;
 
