@@ -37,7 +37,6 @@ CookieMonsterChangeDispatcher::Subscription::Subscription(
     std::string name_key,
     GURL url,
     CookiePartitionKeyCollection cookie_partition_key_collection,
-    bool same_party_attribute_enabled,
     net::CookieChangeCallback callback)
     : change_dispatcher_(std::move(change_dispatcher)),
       domain_key_(std::move(domain_key)),
@@ -46,7 +45,6 @@ CookieMonsterChangeDispatcher::Subscription::Subscription(
       cookie_partition_key_collection_(
           std::move(cookie_partition_key_collection)),
       callback_(std::move(callback)),
-      same_party_attribute_enabled_(same_party_attribute_enabled),
       task_runner_(base::SingleThreadTaskRunner::GetCurrentDefault()) {
   DCHECK(url_.is_valid() || url_.is_empty());
   DCHECK_EQ(url_.is_empty(), domain_key_ == kGlobalDomainKey);
@@ -75,14 +73,11 @@ void CookieMonsterChangeDispatcher::Subscription::DispatchChange(
         cookie_access_delegate &&
         cookie_access_delegate->ShouldTreatUrlAsTrustworthy(url_);
     CookieOptions options = CookieOptions::MakeAllInclusive();
-    CookieSamePartyStatus same_party_status = cookie_util::GetSamePartyStatus(
-        cookie, options, same_party_attribute_enabled_);
     if (!cookie
              .IncludeForRequestURL(
                  url_, options,
                  CookieAccessParams{change.access_result.access_semantics,
-                                    delegate_treats_url_as_trustworthy,
-                                    same_party_status})
+                                    delegate_treats_url_as_trustworthy})
              .status.IsInclude()) {
       return;
     }
@@ -116,10 +111,8 @@ void CookieMonsterChangeDispatcher::Subscription::DoDispatchChange(
 }
 
 CookieMonsterChangeDispatcher::CookieMonsterChangeDispatcher(
-    const CookieMonster* cookie_monster,
-    bool same_party_attribute_enabled)
-    : cookie_monster_(cookie_monster),
-      same_party_attribute_enabled_(same_party_attribute_enabled) {}
+    const CookieMonster* cookie_monster)
+    : cookie_monster_(cookie_monster) {}
 
 CookieMonsterChangeDispatcher::~CookieMonsterChangeDispatcher() {
   DCHECK_CALLED_ON_VALID_THREAD(thread_checker_);
@@ -161,7 +154,7 @@ CookieMonsterChangeDispatcher::AddCallbackForCookie(
   std::unique_ptr<Subscription> subscription = std::make_unique<Subscription>(
       weak_ptr_factory_.GetWeakPtr(), DomainKey(url), NameKey(name), url,
       CookiePartitionKeyCollection::FromOptional(cookie_partition_key),
-      same_party_attribute_enabled_, std::move(callback));
+      std::move(callback));
 
   LinkSubscription(subscription.get());
   return subscription;
@@ -178,7 +171,7 @@ CookieMonsterChangeDispatcher::AddCallbackForUrl(
       weak_ptr_factory_.GetWeakPtr(), DomainKey(url),
       std::string(kGlobalNameKey), url,
       CookiePartitionKeyCollection::FromOptional(cookie_partition_key),
-      same_party_attribute_enabled_, std::move(callback));
+      std::move(callback));
 
   LinkSubscription(subscription.get());
   return subscription;
@@ -192,8 +185,7 @@ CookieMonsterChangeDispatcher::AddCallbackForAllChanges(
   std::unique_ptr<Subscription> subscription = std::make_unique<Subscription>(
       weak_ptr_factory_.GetWeakPtr(), std::string(kGlobalDomainKey),
       std::string(kGlobalNameKey), GURL(""),
-      CookiePartitionKeyCollection::ContainsAll(),
-      same_party_attribute_enabled_, std::move(callback));
+      CookiePartitionKeyCollection::ContainsAll(), std::move(callback));
 
   LinkSubscription(subscription.get());
   return subscription;
