@@ -16,7 +16,6 @@
 #include "net/base/features.h"
 #include "net/base/net_errors.h"
 #include "net/base/network_anonymization_key.h"
-#include "net/cert/cert_and_ct_verifier.h"
 #include "net/cert/cert_status_flags.h"
 #include "net/cert/cert_verifier.h"
 #include "net/cert/ct_log_verifier.h"
@@ -26,13 +25,13 @@
 #include "net/cert/mock_cert_verifier.h"
 #include "net/cert/multi_log_ct_verifier.h"
 #include "net/cert/sct_auditing_delegate.h"
+#include "net/cert/sct_status_flags.h"
 #include "net/cert/x509_util.h"
 #include "net/http/transport_security_state.h"
 #include "net/http/transport_security_state_test_util.h"
 #include "net/quic/crypto/proof_source_chromium.h"
 #include "net/quic/quic_context.h"
 #include "net/test/cert_test_util.h"
-#include "net/test/ct_test_util.h"
 #include "net/test/test_data_directory.h"
 #include "net/third_party/quiche/src/quiche/quic/core/crypto/proof_verifier.h"
 #include "net/third_party/quiche/src/quiche/quic/core/quic_error_codes.h"
@@ -68,15 +67,6 @@ class FailsTestCertVerifier : public CertVerifier {
   void RemoveObserver(Observer* observer) override {}
 };
 
-// A mock CTPolicyEnforcer that returns a custom verification result.
-class MockCTPolicyEnforcer : public CTPolicyEnforcer {
- public:
-  MOCK_METHOD3(CheckCompliance,
-               ct::CTPolicyCompliance(X509Certificate* cert,
-                                      const ct::SCTList&,
-                                      const NetLogWithSource&));
-};
-
 class MockRequireCTDelegate : public TransportSecurityState::RequireCTDelegate {
  public:
   MOCK_METHOD3(IsCTRequiredForHost,
@@ -132,8 +122,6 @@ const char kTestEmptyOCSPResponse[] = "";
 const char kTestEmptySCT[] = "";
 const char kTestEmptySignature[] = "";
 
-const char kLogDescription[] = "somelog";
-
 // This test exercises code that does not depend on the QUIC version in use
 // but that still requires a version so we just use the first one.
 const quic::QuicTransportVersion kTestTransportVersion =
@@ -141,35 +129,6 @@ const quic::QuicTransportVersion kTestTransportVersion =
 
 }  // namespace
 
-// A mock ReportSenderInterface that just remembers the latest report
-// URI and its NetworkAnonymizationKey.
-class MockCertificateReportSender
-    : public TransportSecurityState::ReportSenderInterface {
- public:
-  MockCertificateReportSender() = default;
-  ~MockCertificateReportSender() override = default;
-
-  void Send(
-      const GURL& report_uri,
-      base::StringPiece content_type,
-      base::StringPiece report,
-      const NetworkAnonymizationKey& network_anonymization_key,
-      base::OnceCallback<void()> success_callback,
-      base::OnceCallback<void(const GURL&, int, int)> error_callback) override {
-    latest_report_uri_ = report_uri;
-    latest_network_anonymization_key_ = network_anonymization_key;
-  }
-
-  const GURL& latest_report_uri() { return latest_report_uri_; }
-  const NetworkAnonymizationKey& latest_network_anonymization_key() {
-    return latest_network_anonymization_key_;
-  }
-
- private:
-  GURL latest_report_uri_;
-  NetworkAnonymizationKey latest_network_anonymization_key_;
-};
-
 class ProofVerifierChromiumTest : public ::testing::Test {
  public:
   ProofVerifierChromiumTest()
@@ -178,10 +137,6 @@ class ProofVerifierChromiumTest : public ::testing::Test {
             NetLogWithSource())) {}
 
   void SetUp() override {
-    EXPECT_CALL(ct_policy_enforcer_, CheckCompliance(_, _, _))
-        .WillRepeatedly(
-            Return(ct::CTPolicyCompliance::CT_POLICY_NOT_ENOUGH_SCTS));
-
     static const char kTestCert[] = "quic-chain.pem";
     test_cert_ = ImportCertFromFile(GetTestCertsDirectory(), kTestCert);
     ASSERT_TRUE(test_cert_);
@@ -191,6 +146,8 @@ class ProofVerifierChromiumTest : public ::testing::Test {
 
     dummy_result_.verified_cert = test_cert_;
     dummy_result_.is_issued_by_known_root = true;
+    dummy_result_.policy_compliance =
+        ct::CTPolicyCompliance::CT_POLICY_NOT_ENOUGH_SCTS;
   }
 
   std::string GetTestSignature() {
@@ -206,37 +163,8 @@ class ProofVerifierChromiumTest : public ::testing::Test {
     return signature;
   }
 
-  void GetSCTTestCertificates(std::vector<std::string>* certs) {
-    std::string der_test_cert(ct::GetDerEncodedX509Cert());
-    scoped_refptr<X509Certificate> test_cert = X509Certificate::CreateFromBytes(
-        base::as_bytes(base::make_span(der_test_cert)));
-    ASSERT_TRUE(test_cert.get());
-
-    certs->clear();
-    certs->emplace_back(
-        x509_util::CryptoBufferAsStringPiece(test_cert->cert_buffer()));
-  }
-
-  void CheckSCT(bool sct_expected_ok) {
-    ProofVerifyDetailsChromium* proof_details =
-        reinterpret_cast<ProofVerifyDetailsChromium*>(details_.get());
-    const CertVerifyResult& cert_verify_result =
-        proof_details->cert_verify_result;
-    if (sct_expected_ok) {
-      EXPECT_TRUE(ct::CheckForSingleVerifiedSCTInResult(cert_verify_result.scts,
-                                                        kLogDescription));
-      EXPECT_TRUE(ct::CheckForSCTOrigin(
-          cert_verify_result.scts,
-          ct::SignedCertificateTimestamp::SCT_FROM_TLS_EXTENSION));
-    } else {
-      ASSERT_EQ(1U, cert_verify_result.scts.size());
-      EXPECT_EQ(ct::SCT_STATUS_LOG_UNKNOWN, cert_verify_result.scts[0].status);
-    }
-  }
-
  protected:
   TransportSecurityState transport_security_state_;
-  MockCTPolicyEnforcer ct_policy_enforcer_;
 
   std::unique_ptr<quic::ProofVerifyContext> verify_context_;
   std::unique_ptr<quic::ProofVerifyDetails> details_;
@@ -251,7 +179,7 @@ TEST_F(ProofVerifierChromiumTest, VerifyProof) {
   MockCertVerifier dummy_verifier;
   dummy_verifier.AddResultForCert(test_cert_.get(), dummy_result_, OK);
 
-  ProofVerifierChromium proof_verifier(&dummy_verifier, &ct_policy_enforcer_,
+  ProofVerifierChromium proof_verifier(&dummy_verifier,
                                        &transport_security_state_, nullptr, {},
                                        NetworkAnonymizationKey());
 
@@ -285,7 +213,7 @@ TEST_F(ProofVerifierChromiumTest, VerifyProof) {
 // verification fails.
 TEST_F(ProofVerifierChromiumTest, FailsIfCertFails) {
   MockCertVerifier dummy_verifier;
-  ProofVerifierChromium proof_verifier(&dummy_verifier, &ct_policy_enforcer_,
+  ProofVerifierChromium proof_verifier(&dummy_verifier,
                                        &transport_security_state_, nullptr, {},
                                        NetworkAnonymizationKey());
 
@@ -304,101 +232,42 @@ TEST_F(ProofVerifierChromiumTest, FailsIfCertFails) {
   ASSERT_EQ(quic::QUIC_FAILURE, status);
 }
 
-class DoNothingLogNotifier : public MultiLogCTVerifier::CTLogProvider {
- public:
-  DoNothingLogNotifier() = default;
-  ~DoNothingLogNotifier() = default;
-};
-
-// Valid SCT and cert
-TEST_F(ProofVerifierChromiumTest, ValidSCTList) {
-  // Use different certificates for SCT tests.
-  ASSERT_NO_FATAL_FAILURE(GetSCTTestCertificates(&certs_));
-
-  std::string der_test_cert(ct::GetDerEncodedX509Cert());
-  scoped_refptr<X509Certificate> test_cert = X509Certificate::CreateFromBytes(
-      base::as_bytes(base::make_span(der_test_cert)));
-  ASSERT_TRUE(test_cert);
+// Confirms that the parameters get passed through to the
+// CertVerifier::RequestParams as expected.
+TEST_F(ProofVerifierChromiumTest, PassesCertVerifierRequestParams) {
   CertVerifyResult dummy_result;
-  dummy_result.verified_cert = test_cert;
+  dummy_result.verified_cert = test_cert_;
   dummy_result.is_issued_by_known_root = true;
-  auto dummy_verifier = std::make_unique<MockCertVerifier>();
-  dummy_verifier->AddResultForCert(test_cert.get(), dummy_result, OK);
 
-  // Combine the mocked cert verify result with the results of the
-  // MultiLogCTVerifier.
-  std::vector<scoped_refptr<const CTLogVerifier>> log_verifiers;
-  scoped_refptr<const CTLogVerifier> log(
-      CTLogVerifier::Create(ct::GetTestPublicKey(), kLogDescription));
-  ASSERT_TRUE(log);
-  log_verifiers.push_back(log);
-  DoNothingLogNotifier notifier;
-  auto ct_verifier = std::make_unique<MultiLogCTVerifier>(&notifier);
-  ct_verifier->SetLogs(log_verifiers);
+  ParamRecordingMockCertVerifier dummy_verifier;
+  dummy_verifier.AddResultForCert(test_cert_.get(), dummy_result, OK);
 
-  CertAndCTVerifier cert_verifier(std::move(dummy_verifier),
-                                  std::move(ct_verifier));
-
-  ProofVerifierChromium proof_verifier(&cert_verifier, &ct_policy_enforcer_,
+  ProofVerifierChromium proof_verifier(&dummy_verifier,
                                        &transport_security_state_, nullptr, {},
                                        NetworkAnonymizationKey());
 
-  auto callback = std::make_unique<DummyProofVerifierCallback>();
-  quic::QuicAsyncStatus status = proof_verifier.VerifyCertChain(
-      kTestHostname, kTestPort, certs_, kTestEmptyOCSPResponse,
-      ct::GetSCTListForTesting(), verify_context_.get(), &error_details_,
-      &details_, &tls_alert_, std::move(callback));
-  ASSERT_EQ(quic::QUIC_SUCCESS, status);
-  CheckSCT(/*sct_expected_ok=*/true);
-}
-
-// Invalid SCT, but valid cert
-TEST_F(ProofVerifierChromiumTest, InvalidSCTList) {
-  // Use different certificates for SCT tests.
-  ASSERT_NO_FATAL_FAILURE(GetSCTTestCertificates(&certs_));
-
-  std::string der_test_cert(ct::GetDerEncodedX509Cert());
-  scoped_refptr<X509Certificate> test_cert = X509Certificate::CreateFromBytes(
-      base::as_bytes(base::make_span(der_test_cert)));
-  ASSERT_TRUE(test_cert);
-  CertVerifyResult dummy_result;
-  dummy_result.verified_cert = test_cert;
-  dummy_result.is_issued_by_known_root = true;
-  auto dummy_verifier = std::make_unique<MockCertVerifier>();
-  dummy_verifier->AddResultForCert(test_cert.get(), dummy_result, OK);
-
-  // Combine the mocked cert verify result with the results of the
-  // MultiLogCTVerifier.
-  std::vector<scoped_refptr<const CTLogVerifier>> log_verifiers;
-  scoped_refptr<const CTLogVerifier> log(
-      CTLogVerifier::Create(ct::GetTestPublicKey(), kLogDescription));
-  ASSERT_TRUE(log);
-  log_verifiers.push_back(log);
-  DoNothingLogNotifier notifier;
-  auto ct_verifier = std::make_unique<MultiLogCTVerifier>(&notifier);
-  ct_verifier->SetLogs(log_verifiers);
-
-  CertAndCTVerifier cert_verifier(std::move(dummy_verifier),
-                                  std::move(ct_verifier));
-
-  ProofVerifierChromium proof_verifier(&cert_verifier, &ct_policy_enforcer_,
-                                       &transport_security_state_, nullptr, {},
-                                       NetworkAnonymizationKey());
+  const std::string kTestOcspResponse = "ocsp";
+  const std::string kTestSctList = "sct list";
 
   auto callback = std::make_unique<DummyProofVerifierCallback>();
   quic::QuicAsyncStatus status = proof_verifier.VerifyCertChain(
-      kTestHostname, kTestPort, certs_, kTestEmptyOCSPResponse,
-      ct::GetSCTListWithInvalidSCT(), verify_context_.get(), &error_details_,
-      &details_, &tls_alert_, std::move(callback));
+      kTestHostname, kTestPort, certs_, kTestOcspResponse, kTestSctList,
+      verify_context_.get(), &error_details_, &details_, &tls_alert_,
+      std::move(callback));
   ASSERT_EQ(quic::QUIC_SUCCESS, status);
-  CheckSCT(/*sct_expected_ok=*/false);
+  ASSERT_EQ(dummy_verifier.GetVerifyParams().size(), 1u);
+  const auto& params = dummy_verifier.GetVerifyParams().front();
+  EXPECT_TRUE(params.certificate()->EqualsIncludingChain(test_cert_.get()));
+  EXPECT_EQ(params.hostname(), kTestHostname);
+  EXPECT_EQ(params.ocsp_response(), kTestOcspResponse);
+  EXPECT_EQ(params.sct_list(), kTestSctList);
 }
 
 // Tests that the quic::ProofVerifier doesn't verify certificates if the config
 // signature fails.
 TEST_F(ProofVerifierChromiumTest, FailsIfSignatureFails) {
   FailsTestCertVerifier cert_verifier;
-  ProofVerifierChromium proof_verifier(&cert_verifier, &ct_policy_enforcer_,
+  ProofVerifierChromium proof_verifier(&cert_verifier,
                                        &transport_security_state_, nullptr, {},
                                        NetworkAnonymizationKey());
 
@@ -410,93 +279,6 @@ TEST_F(ProofVerifierChromiumTest, FailsIfSignatureFails) {
   ASSERT_EQ(quic::QUIC_FAILURE, status);
 }
 
-// Tests that the certificate policy enforcer is consulted for EV
-// and the certificate is allowed to be EV.
-TEST_F(ProofVerifierChromiumTest, PreservesEVIfAllowed) {
-  dummy_result_.cert_status = CERT_STATUS_IS_EV;
-
-  MockCertVerifier dummy_verifier;
-  dummy_verifier.AddResultForCert(test_cert_.get(), dummy_result_, OK);
-
-  EXPECT_CALL(ct_policy_enforcer_, CheckCompliance(_, _, _))
-      .WillRepeatedly(
-          Return(ct::CTPolicyCompliance::CT_POLICY_COMPLIES_VIA_SCTS));
-
-  ProofVerifierChromium proof_verifier(&dummy_verifier, &ct_policy_enforcer_,
-                                       &transport_security_state_, nullptr, {},
-                                       NetworkAnonymizationKey());
-
-  auto callback = std::make_unique<DummyProofVerifierCallback>();
-  quic::QuicAsyncStatus status = proof_verifier.VerifyProof(
-      kTestHostname, kTestPort, kTestConfig, kTestTransportVersion,
-      kTestChloHash, certs_, kTestEmptySCT, GetTestSignature(),
-      verify_context_.get(), &error_details_, &details_, std::move(callback));
-  ASSERT_EQ(quic::QUIC_SUCCESS, status);
-
-  ASSERT_TRUE(details_.get());
-  ProofVerifyDetailsChromium* verify_details =
-      static_cast<ProofVerifyDetailsChromium*>(details_.get());
-  EXPECT_EQ(dummy_result_.cert_status,
-            verify_details->cert_verify_result.cert_status);
-
-  // Repeat the test with VerifyCertChain.
-  callback = std::make_unique<DummyProofVerifierCallback>();
-  status = proof_verifier.VerifyCertChain(
-      kTestHostname, kTestPort, certs_, kTestEmptyOCSPResponse, kTestEmptySCT,
-      verify_context_.get(), &error_details_, &details_, &tls_alert_,
-      std::move(callback));
-  ASSERT_EQ(quic::QUIC_SUCCESS, status);
-
-  ASSERT_TRUE(details_.get());
-  verify_details = static_cast<ProofVerifyDetailsChromium*>(details_.get());
-  EXPECT_EQ(dummy_result_.cert_status,
-            verify_details->cert_verify_result.cert_status);
-}
-
-// Tests that the certificate policy enforcer is consulted for EV
-// and the certificate is not allowed to be EV.
-TEST_F(ProofVerifierChromiumTest, StripsEVIfNotAllowed) {
-  dummy_result_.cert_status = CERT_STATUS_IS_EV;
-
-  MockCertVerifier dummy_verifier;
-  dummy_verifier.AddResultForCert(test_cert_.get(), dummy_result_, OK);
-
-  EXPECT_CALL(ct_policy_enforcer_, CheckCompliance(_, _, _))
-      .WillRepeatedly(
-          Return(ct::CTPolicyCompliance::CT_POLICY_NOT_ENOUGH_SCTS));
-
-  ProofVerifierChromium proof_verifier(&dummy_verifier, &ct_policy_enforcer_,
-                                       &transport_security_state_, nullptr, {},
-                                       NetworkAnonymizationKey());
-
-  auto callback = std::make_unique<DummyProofVerifierCallback>();
-  quic::QuicAsyncStatus status = proof_verifier.VerifyProof(
-      kTestHostname, kTestPort, kTestConfig, kTestTransportVersion,
-      kTestChloHash, certs_, kTestEmptySCT, GetTestSignature(),
-      verify_context_.get(), &error_details_, &details_, std::move(callback));
-  ASSERT_EQ(quic::QUIC_SUCCESS, status);
-
-  ASSERT_TRUE(details_.get());
-  ProofVerifyDetailsChromium* verify_details =
-      static_cast<ProofVerifyDetailsChromium*>(details_.get());
-  EXPECT_EQ(CERT_STATUS_CT_COMPLIANCE_FAILED,
-            verify_details->cert_verify_result.cert_status &
-                (CERT_STATUS_CT_COMPLIANCE_FAILED | CERT_STATUS_IS_EV));
-
-  callback = std::make_unique<DummyProofVerifierCallback>();
-  status = proof_verifier.VerifyCertChain(
-      kTestHostname, kTestPort, certs_, kTestEmptyOCSPResponse, kTestEmptySCT,
-      verify_context_.get(), &error_details_, &details_, &tls_alert_,
-      std::move(callback));
-  ASSERT_EQ(quic::QUIC_SUCCESS, status);
-
-  ASSERT_TRUE(details_.get());
-  verify_details = static_cast<ProofVerifyDetailsChromium*>(details_.get());
-  EXPECT_EQ(CERT_STATUS_CT_COMPLIANCE_FAILED,
-            verify_details->cert_verify_result.cert_status &
-                (CERT_STATUS_CT_COMPLIANCE_FAILED | CERT_STATUS_IS_EV));
-}
-
 HashValueVector MakeHashValueVector(uint8_t tag) {
   HashValue hash(HASH_VALUE_SHA256);
   memset(hash.data(), tag, hash.size());
@@ -512,7 +294,7 @@ TEST_F(ProofVerifierChromiumTest, IsFatalErrorNotSetForNonFatalError) {
   dummy_verifier.AddResultForCert(test_cert_.get(), dummy_result_,
                                   ERR_CERT_DATE_INVALID);
 
-  ProofVerifierChromium proof_verifier(&dummy_verifier, &ct_policy_enforcer_,
+  ProofVerifierChromium proof_verifier(&dummy_verifier,
                                        &transport_security_state_, nullptr, {},
                                        NetworkAnonymizationKey());
 
@@ -548,7 +330,7 @@ TEST_F(ProofVerifierChromiumTest, IsFatalErrorSetForFatalError) {
   const base::Time expiry = base::Time::Now() + base::Seconds(1000);
   transport_security_state_.AddHSTS(kTestHostname, expiry, true);
 
-  ProofVerifierChromium proof_verifier(&dummy_verifier, &ct_policy_enforcer_,
+  ProofVerifierChromium proof_verifier(&dummy_verifier,
                                        &transport_security_state_, nullptr, {},
                                        NetworkAnonymizationKey());
 
@@ -587,7 +369,7 @@ TEST_F(ProofVerifierChromiumTest, PKPEnforced) {
   transport_security_state_.SetPinningListAlwaysTimelyForTesting(true);
   ScopedTransportSecurityStateSource scoped_security_state_source;
 
-  ProofVerifierChromium proof_verifier(&dummy_verifier, &ct_policy_enforcer_,
+  ProofVerifierChromium proof_verifier(&dummy_verifier,
                                        &transport_security_state_, nullptr, {},
                                        NetworkAnonymizationKey());
 
@@ -604,7 +386,6 @@ TEST_F(ProofVerifierChromiumTest, PKPEnforced) {
   EXPECT_TRUE(verify_details->cert_verify_result.cert_status &
               CERT_STATUS_PINNED_KEY_MISSING);
   EXPECT_FALSE(verify_details->pkp_bypassed);
-  EXPECT_NE("", verify_details->pinning_failure_log);
 
   callback = std::make_unique<DummyProofVerifierCallback>();
   status = proof_verifier.VerifyCertChain(
@@ -618,7 +399,6 @@ TEST_F(ProofVerifierChromiumTest, PKPEnforced) {
   EXPECT_TRUE(verify_details->cert_verify_result.cert_status &
               CERT_STATUS_PINNED_KEY_MISSING);
   EXPECT_FALSE(verify_details->pkp_bypassed);
-  EXPECT_NE("", verify_details->pinning_failure_log);
 }
 
 // Test |pkp_bypassed| is set when PKP is bypassed due to a local
@@ -638,8 +418,8 @@ TEST_F(ProofVerifierChromiumTest, PKPBypassFlagSet) {
   ScopedTransportSecurityStateSource scoped_security_state_source;
 
   ProofVerifierChromium proof_verifier(
-      &dummy_verifier, &ct_policy_enforcer_, &transport_security_state_,
-      nullptr, {kCTAndPKPHost}, NetworkAnonymizationKey());
+      &dummy_verifier, &transport_security_state_, nullptr, {kCTAndPKPHost},
+      NetworkAnonymizationKey());
 
   auto callback = std::make_unique<DummyProofVerifierCallback>();
   quic::QuicAsyncStatus status = proof_verifier.VerifyProof(
@@ -665,76 +445,13 @@ TEST_F(ProofVerifierChromiumTest, PKPBypassFlagSet) {
   EXPECT_TRUE(verify_details->pkp_bypassed);
 }
 
-// Test that PKP errors result in sending reports.
-TEST_F(ProofVerifierChromiumTest, PKPReport) {
-  NetworkAnonymizationKey network_anonymization_key =
-      NetworkAnonymizationKey::CreateTransient();
-
-  MockCertificateReportSender report_sender;
-  transport_security_state_.SetReportSender(&report_sender);
-
-  HashValueVector spki_hashes;
-  HashValue hash(HASH_VALUE_SHA256);
-  memset(hash.data(), 0, hash.size());
-  spki_hashes.push_back(hash);
-
-  GURL report_uri("https://foo.test/");
-  transport_security_state_.AddHPKP(
-      kCTAndPKPHost, base::Time::Now() + base::Days(1),
-      false /* include_subdomains */, spki_hashes, report_uri);
-  ScopedTransportSecurityStateSource scoped_security_state_source;
-
-  dummy_result_.is_issued_by_known_root = true;
-  dummy_result_.public_key_hashes = MakeHashValueVector(0x01);
-
-  MockCertVerifier dummy_verifier;
-  dummy_verifier.AddResultForCert(test_cert_.get(), dummy_result_, OK);
-
-  ProofVerifierChromium proof_verifier(&dummy_verifier, &ct_policy_enforcer_,
-                                       &transport_security_state_, nullptr, {},
-                                       network_anonymization_key);
-
-  auto callback = std::make_unique<DummyProofVerifierCallback>();
-  quic::QuicAsyncStatus status = proof_verifier.VerifyProof(
-      kCTAndPKPHost, kTestPort, kTestConfig, kTestTransportVersion,
-      kTestChloHash, certs_, kTestEmptySCT, GetTestSignature(),
-      verify_context_.get(), &error_details_, &details_, std::move(callback));
-  ASSERT_EQ(quic::QUIC_FAILURE, status);
-
-  ASSERT_TRUE(details_.get());
-  ProofVerifyDetailsChromium* verify_details =
-      static_cast<ProofVerifyDetailsChromium*>(details_.get());
-  EXPECT_TRUE(verify_details->cert_verify_result.cert_status &
-              CERT_STATUS_PINNED_KEY_MISSING);
-  EXPECT_FALSE(verify_details->pkp_bypassed);
-  EXPECT_NE("", verify_details->pinning_failure_log);
-
-  callback = std::make_unique<DummyProofVerifierCallback>();
-  status = proof_verifier.VerifyCertChain(
-      kCTAndPKPHost, kTestPort, certs_, kTestEmptyOCSPResponse, kTestEmptySCT,
-      verify_context_.get(), &error_details_, &details_, &tls_alert_,
-      std::move(callback));
-  ASSERT_EQ(quic::QUIC_FAILURE, status);
-
-  ASSERT_TRUE(details_.get());
-  verify_details = static_cast<ProofVerifyDetailsChromium*>(details_.get());
-  EXPECT_TRUE(verify_details->cert_verify_result.cert_status &
-              CERT_STATUS_PINNED_KEY_MISSING);
-  EXPECT_FALSE(verify_details->pkp_bypassed);
-  EXPECT_NE("", verify_details->pinning_failure_log);
-
-  EXPECT_EQ(report_uri, report_sender.latest_report_uri());
-  EXPECT_EQ(network_anonymization_key,
-            report_sender.latest_network_anonymization_key());
-
-  transport_security_state_.SetReportSender(nullptr);
-}
-
 // Test that when CT is required (in this case, by the delegate), the
 // absence of CT information is a socket error.
 TEST_F(ProofVerifierChromiumTest, CTIsRequired) {
   dummy_result_.is_issued_by_known_root = true;
   dummy_result_.public_key_hashes = MakeHashValueVector(0x01);
+  dummy_result_.policy_compliance =
+      ct::CTPolicyCompliance::CT_POLICY_NOT_ENOUGH_SCTS;
 
   MockCertVerifier dummy_verifier;
   dummy_verifier.AddResultForCert(test_cert_.get(), dummy_result_, OK);
@@ -748,11 +465,8 @@ TEST_F(ProofVerifierChromiumTest, CTIsRequired) {
   EXPECT_CALL(require_ct_delegate, IsCTRequiredForHost(kTestHostname, _, _))
       .WillRepeatedly(Return(TransportSecurityState::RequireCTDelegate::
                                  CTRequirementLevel::REQUIRED));
-  EXPECT_CALL(ct_policy_enforcer_, CheckCompliance(_, _, _))
-      .WillRepeatedly(
-          Return(ct::CTPolicyCompliance::CT_POLICY_NOT_ENOUGH_SCTS));
 
-  ProofVerifierChromium proof_verifier(&dummy_verifier, &ct_policy_enforcer_,
+  ProofVerifierChromium proof_verifier(&dummy_verifier,
                                        &transport_security_state_, nullptr, {},
                                        NetworkAnonymizationKey());
 
@@ -791,6 +505,8 @@ TEST_F(ProofVerifierChromiumTest, PKPAndCTBothTested) {
       net::features::kStaticKeyPinningEnforcement);
   dummy_result_.is_issued_by_known_root = true;
   dummy_result_.public_key_hashes = MakeHashValueVector(0x01);
+  dummy_result_.policy_compliance =
+      ct::CTPolicyCompliance::CT_POLICY_NOT_ENOUGH_SCTS;
 
   MockCertVerifier dummy_verifier;
   dummy_verifier.AddResultForCert(test_cert_.get(), dummy_result_, OK);
@@ -809,11 +525,8 @@ TEST_F(ProofVerifierChromiumTest, PKPAndCTBothTested) {
   EXPECT_CALL(require_ct_delegate, IsCTRequiredForHost(kCTAndPKPHost, _, _))
       .WillRepeatedly(Return(TransportSecurityState::RequireCTDelegate::
                                  CTRequirementLevel::REQUIRED));
-  EXPECT_CALL(ct_policy_enforcer_, CheckCompliance(_, _, _))
-      .WillRepeatedly(
-          Return(ct::CTPolicyCompliance::CT_POLICY_NOT_ENOUGH_SCTS));
 
-  ProofVerifierChromium proof_verifier(&dummy_verifier, &ct_policy_enforcer_,
+  ProofVerifierChromium proof_verifier(&dummy_verifier,
                                        &transport_security_state_, nullptr, {},
                                        NetworkAnonymizationKey());
 
@@ -855,7 +568,7 @@ TEST_F(ProofVerifierChromiumTest, UnknownRootRejected) {
   MockCertVerifier dummy_verifier;
   dummy_verifier.AddResultForCert(test_cert_.get(), dummy_result_, OK);
 
-  ProofVerifierChromium proof_verifier(&dummy_verifier, &ct_policy_enforcer_,
+  ProofVerifierChromium proof_verifier(&dummy_verifier,
                                        &transport_security_state_, nullptr, {},
                                        NetworkAnonymizationKey());
 
@@ -887,8 +600,8 @@ TEST_F(ProofVerifierChromiumTest, UnknownRootAcceptedWithOverride) {
   dummy_verifier.AddResultForCert(test_cert_.get(), dummy_result_, OK);
 
   ProofVerifierChromium proof_verifier(
-      &dummy_verifier, &ct_policy_enforcer_, &transport_security_state_,
-      nullptr, {kTestHostname}, NetworkAnonymizationKey());
+      &dummy_verifier, &transport_security_state_, nullptr, {kTestHostname},
+      NetworkAnonymizationKey());
 
   auto callback = std::make_unique<DummyProofVerifierCallback>();
   quic::QuicAsyncStatus status = proof_verifier.VerifyProof(
@@ -922,7 +635,7 @@ TEST_F(ProofVerifierChromiumTest, UnknownRootAcceptedWithWildcardOverride) {
   MockCertVerifier dummy_verifier;
   dummy_verifier.AddResultForCert(test_cert_.get(), dummy_result_, OK);
 
-  ProofVerifierChromium proof_verifier(&dummy_verifier, &ct_policy_enforcer_,
+  ProofVerifierChromium proof_verifier(&dummy_verifier,
                                        &transport_security_state_, nullptr,
                                        {""}, NetworkAnonymizationKey());
 
@@ -955,13 +668,11 @@ TEST_F(ProofVerifierChromiumTest, UnknownRootAcceptedWithWildcardOverride) {
 // Tests that the SCTAuditingDelegate is called to enqueue SCT reports when
 // verifying a good proof and cert.
 TEST_F(ProofVerifierChromiumTest, SCTAuditingReportCollected) {
+  dummy_result_.policy_compliance =
+      ct::CTPolicyCompliance::CT_POLICY_COMPLIES_VIA_SCTS;
   MockCertVerifier cert_verifier;
   cert_verifier.AddResultForCert(test_cert_.get(), dummy_result_, OK);
 
-  EXPECT_CALL(ct_policy_enforcer_, CheckCompliance(_, _, _))
-      .WillRepeatedly(
-          Return(ct::CTPolicyCompliance::CT_POLICY_COMPLIES_VIA_SCTS));
-
   MockSCTAuditingDelegate sct_auditing_delegate;
   EXPECT_CALL(sct_auditing_delegate, IsSCTAuditingEnabled())
       .WillRepeatedly(Return(true));
@@ -972,8 +683,8 @@ TEST_F(ProofVerifierChromiumTest, SCTAuditingReportCollected) {
       .Times(2);
 
   ProofVerifierChromium proof_verifier(
-      &cert_verifier, &ct_policy_enforcer_, &transport_security_state_,
-      &sct_auditing_delegate, {}, NetworkAnonymizationKey());
+      &cert_verifier, &transport_security_state_, &sct_auditing_delegate, {},
+      NetworkAnonymizationKey());
 
   auto callback = std::make_unique<DummyProofVerifierCallback>();
   quic::QuicAsyncStatus status = proof_verifier.VerifyProof(
