@@ -39,8 +39,7 @@ namespace {
 
 class MockQuicClientSessionBase : public quic::QuicSpdyClientSessionBase {
  public:
-  explicit MockQuicClientSessionBase(quic::QuicConnection* connection,
-                                     quic::QuicClientPushPromiseIndex* index);
+  explicit MockQuicClientSessionBase(quic::QuicConnection* connection);
 
   MockQuicClientSessionBase(const MockQuicClientSessionBase&) = delete;
   MockQuicClientSessionBase& operator=(const MockQuicClientSessionBase&) =
@@ -83,7 +82,7 @@ class MockQuicClientSessionBase : public quic::QuicSpdyClientSessionBase {
 
   MOCK_METHOD2(OnStreamHeaders,
                void(quic::QuicStreamId stream_id,
-                    absl::string_view headers_data));
+                    std::string_view headers_data));
   MOCK_METHOD2(OnStreamHeadersPriority,
                void(quic::QuicStreamId stream_id,
                     const spdy::SpdyStreamPrecedence& precedence));
@@ -126,7 +125,6 @@ class MockQuicClientSessionBase : public quic::QuicSpdyClientSessionBase {
       const quic::QuicCryptoClientConfig::CachedState& cached) override {}
   void OnProofVerifyDetailsAvailable(
       const quic::ProofVerifyDetails& verify_details) override {}
-  bool IsAuthorized(const std::string& hostname) override { return true; }
 
  protected:
   MOCK_METHOD1(ShouldCreateIncomingStream, bool(quic::QuicStreamId id));
@@ -138,11 +136,9 @@ class MockQuicClientSessionBase : public quic::QuicSpdyClientSessionBase {
 };
 
 MockQuicClientSessionBase::MockQuicClientSessionBase(
-    quic::QuicConnection* connection,
-    quic::QuicClientPushPromiseIndex* push_promise_index)
+    quic::QuicConnection* connection)
     : quic::QuicSpdyClientSessionBase(connection,
                                       /*visitor=*/nullptr,
-                                      push_promise_index,
                                       quic::test::DefaultQuicConfig(),
                                       connection->supported_versions()) {
   crypto_stream_ = std::make_unique<quic::test::MockQuicCryptoStream>(this);
@@ -162,11 +158,10 @@ class QuicChromiumClientStreamTest
         crypto_config_(
             quic::test::crypto_test_utils::ProofVerifierForTesting()),
         session_(new quic::test::MockQuicConnection(
-                     &helper_,
-                     &alarm_factory_,
-                     quic::Perspective::IS_CLIENT,
-                     quic::test::SupportedVersions(version_)),
-                 &push_promise_index_) {
+            &helper_,
+            &alarm_factory_,
+            quic::Perspective::IS_CLIENT,
+            quic::test::SupportedVersions(version_))) {
     quic::test::QuicConfigPeer::SetReceivedInitialSessionFlowControlWindow(
         session_.config(), quic::kMinimumFlowControlSendWindow);
     quic::test::QuicConfigPeer::
@@ -226,13 +221,13 @@ class QuicChromiumClientStreamTest
     return headers;
   }
 
-  void ReadData(absl::string_view expected_data) {
-    scoped_refptr<IOBuffer> buffer =
-        base::MakeRefCounted<IOBuffer>(expected_data.length() + 1);
+  void ReadData(std::string_view expected_data) {
+    auto buffer =
+        base::MakeRefCounted<IOBufferWithSize>(expected_data.length() + 1);
     EXPECT_EQ(static_cast<int>(expected_data.length()),
               stream_->Read(buffer.get(), expected_data.length() + 1));
     EXPECT_EQ(expected_data,
-              absl::string_view(buffer->data(), expected_data.length()));
+              std::string_view(buffer->data(), expected_data.length()));
   }
 
   quic::QuicHeaderList ProcessHeaders(const spdy::Http2HeaderBlock& headers) {
@@ -288,7 +283,6 @@ class QuicChromiumClientStreamTest
   raw_ptr<QuicChromiumClientStream> stream_;
   spdy::Http2HeaderBlock headers_;
   spdy::Http2HeaderBlock trailers_;
-  quic::QuicClientPushPromiseIndex push_promise_index_;
 };
 
 INSTANTIATE_TEST_SUITE_P(Version,
@@ -318,7 +312,7 @@ TEST_P(QuicChromiumClientStreamTest, Handle) {
   quic::QuicStreamFrame frame2(
       quic::test::GetNthClientInitiatedBidirectionalStreamId(
           version_.transport_version, 0),
-      true, offset, absl::string_view());
+      true, offset, std::string_view());
   stream_->OnStreamFrame(frame2);
   EXPECT_TRUE(handle_->fin_received());
   handle_->OnFinRead();
@@ -357,7 +351,7 @@ TEST_P(QuicChromiumClientStreamTest, Handle) {
                                      callback.callback()));
 
   std::vector<scoped_refptr<IOBuffer>> buffers = {
-      base::MakeRefCounted<IOBuffer>(10)};
+      base::MakeRefCounted<IOBufferWithSize>(10)};
   std::vector<int> lengths = {10};
   EXPECT_EQ(
       ERR_CONNECTION_CLOSED,
@@ -406,7 +400,7 @@ TEST_P(QuicChromiumClientStreamTest, OnFinRead) {
   quic::QuicStreamFrame frame2(
       quic::test::GetNthClientInitiatedBidirectionalStreamId(
           version_.transport_version, 0),
-      true, offset, absl::string_view());
+      true, offset, std::string_view());
   stream_->OnStreamFrame(frame2);
 }
 
@@ -432,11 +426,10 @@ TEST_P(QuicChromiumClientStreamTest, OnDataAvailable) {
 
   // Read the body and verify that it arrives correctly.
   TestCompletionCallback callback;
-  scoped_refptr<IOBuffer> buffer = base::MakeRefCounted<IOBuffer>(2 * data_len);
+  auto buffer = base::MakeRefCounted<IOBufferWithSize>(2 * data_len);
   EXPECT_EQ(data_len,
             handle_->ReadBody(buffer.get(), 2 * data_len, callback.callback()));
-  EXPECT_EQ(absl::string_view(data),
-            absl::string_view(buffer->data(), data_len));
+  EXPECT_EQ(std::string_view(data), std::string_view(buffer->data(), data_len));
 }
 
 TEST_P(QuicChromiumClientStreamTest, OnDataAvailableAfterReadBody) {
@@ -448,7 +441,7 @@ TEST_P(QuicChromiumClientStreamTest, OnDataAvailableAfterReadBody) {
 
   // Start to read the body.
   TestCompletionCallback callback;
-  scoped_refptr<IOBuffer> buffer = base::MakeRefCounted<IOBuffer>(2 * data_len);
+  auto buffer = base::MakeRefCounted<IOBufferWithSize>(2 * data_len);
   EXPECT_EQ(ERR_IO_PENDING,
             handle_->ReadBody(buffer.get(), 2 * data_len, callback.callback()));
 
@@ -468,8 +461,7 @@ TEST_P(QuicChromiumClientStreamTest, OnDataAvailableAfterReadBody) {
       /*offset=*/offset, data));
 
   EXPECT_EQ(data_len, callback.WaitForResult());
-  EXPECT_EQ(absl::string_view(data),
-            absl::string_view(buffer->data(), data_len));
+  EXPECT_EQ(std::string_view(data), std::string_view(buffer->data(), data_len));
   base::RunLoop().RunUntilIdle();
 }
 
@@ -500,7 +492,7 @@ TEST_P(QuicChromiumClientStreamTest, OnDataAvailableWithError) {
 
   // Start to read the body.
   TestCompletionCallback callback;
-  scoped_refptr<IOBuffer> buffer = base::MakeRefCounted<IOBuffer>(2 * data_len);
+  auto buffer = base::MakeRefCounted<IOBufferWithSize>(2 * data_len);
   EXPECT_EQ(
       ERR_IO_PENDING,
       handle_->ReadBody(
@@ -561,11 +553,10 @@ TEST_P(QuicChromiumClientStreamTest, OnTrailers) {
 
   // Read the body and verify that it arrives correctly.
   TestCompletionCallback callback;
-  scoped_refptr<IOBuffer> buffer = base::MakeRefCounted<IOBuffer>(2 * data_len);
+  auto buffer = base::MakeRefCounted<IOBufferWithSize>(2 * data_len);
   EXPECT_EQ(data_len,
             handle_->ReadBody(buffer.get(), 2 * data_len, callback.callback()));
-  EXPECT_EQ(absl::string_view(data),
-            absl::string_view(buffer->data(), data_len));
+  EXPECT_EQ(std::string_view(data), std::string_view(buffer->data(), data_len));
 
   spdy::Http2HeaderBlock trailers;
   trailers["bar"] = "foo";
@@ -609,11 +600,10 @@ TEST_P(QuicChromiumClientStreamTest, MarkTrailersConsumedWhenNotifyDelegate) {
 
   // Read the body and verify that it arrives correctly.
   TestCompletionCallback callback;
-  scoped_refptr<IOBuffer> buffer = base::MakeRefCounted<IOBuffer>(2 * data_len);
+  auto buffer = base::MakeRefCounted<IOBufferWithSize>(2 * data_len);
   EXPECT_EQ(data_len,
             handle_->ReadBody(buffer.get(), 2 * data_len, callback.callback()));
-  EXPECT_EQ(absl::string_view(data),
-            absl::string_view(buffer->data(), data_len));
+  EXPECT_EQ(std::string_view(data), std::string_view(buffer->data(), data_len));
 
   // Read again, and it will be pending.
   EXPECT_THAT(
@@ -664,11 +654,10 @@ TEST_P(QuicChromiumClientStreamTest, ReadAfterTrailersReceivedButNotDelivered) {
 
   // Read the body and verify that it arrives correctly.
   TestCompletionCallback callback;
-  scoped_refptr<IOBuffer> buffer = base::MakeRefCounted<IOBuffer>(2 * data_len);
+  auto buffer = base::MakeRefCounted<IOBufferWithSize>(2 * data_len);
   EXPECT_EQ(data_len,
             handle_->ReadBody(buffer.get(), 2 * data_len, callback.callback()));
-  EXPECT_EQ(absl::string_view(data),
-            absl::string_view(buffer->data(), data_len));
+  EXPECT_EQ(std::string_view(data), std::string_view(buffer->data(), data_len));
 
   // Deliver trailers. Delegate notification is posted asynchronously.
   spdy::Http2HeaderBlock trailers;
@@ -880,10 +869,9 @@ TEST_P(QuicChromiumClientStreamTest, HeadersAndDataBeforeHandle) {
 
   // Now explicitly read the data.
   int data_len = std::size(data) - 1;
-  scoped_refptr<IOBuffer> buffer = base::MakeRefCounted<IOBuffer>(data_len + 1);
+  auto buffer = base::MakeRefCounted<IOBufferWithSize>(data_len + 1);
   ASSERT_EQ(data_len, stream2->Read(buffer.get(), data_len + 1));
-  EXPECT_EQ(absl::string_view(data),
-            absl::string_view(buffer->data(), data_len));
+  EXPECT_EQ(std::string_view(data), std::string_view(buffer->data(), data_len));
 }
 
 // Regression test for https://crbug.com/1043531.
