@@ -345,7 +345,7 @@ OnHostResolutionCallbackResult SpdySessionPool::OnHostResolutionComplete(
         if (!compare_result.is_socket_tag_match) {
           SpdySessionKey old_key = available_session->spdy_session_key();
           SpdySessionKey new_key(old_key.host_port_pair(),
-                                 old_key.proxy_server(), old_key.privacy_mode(),
+                                 old_key.proxy_chain(), old_key.privacy_mode(),
                                  old_key.is_proxy_session(), key.socket_tag(),
                                  old_key.network_anonymization_key(),
                                  old_key.secure_dns_policy());
@@ -393,7 +393,7 @@ OnHostResolutionCallbackResult SpdySessionPool::OnHostResolutionComplete(
                 GetDnsAliasesForSessionKey(*it);
             UnmapKey(*it);
             SpdySessionKey new_pool_alias_key = SpdySessionKey(
-                it->host_port_pair(), it->proxy_server(), it->privacy_mode(),
+                it->host_port_pair(), it->proxy_chain(), it->privacy_mode(),
                 it->is_proxy_session(), key.socket_tag(),
                 it->network_anonymization_key(), it->secure_dns_policy());
             MapKeyToAvailableSession(new_pool_alias_key, available_session,
@@ -541,14 +541,27 @@ void SpdySessionPool::OnSSLConfigForServersChanged(
     const base::flat_set<HostPortPair>& servers) {
   WeakSessionList current_sessions = GetCurrentSessions();
   for (base::WeakPtr<SpdySession>& session : current_sessions) {
+    bool session_matches = false;
     if (!session)
       continue;
 
-    const ProxyServer& proxy_server =
-        session->spdy_session_key().proxy_server();
-    if (servers.contains(session->host_port_pair()) ||
-        (proxy_server.is_http_like() && !proxy_server.is_http() &&
-         servers.contains(proxy_server.host_port_pair()))) {
+    // If the destination for this session is invalidated, or any of the proxy
+    // hops along the way, make the session go away.
+    if (servers.contains(session->host_port_pair())) {
+      session_matches = true;
+    } else {
+      const ProxyChain& proxy_chain = session->spdy_session_key().proxy_chain();
+
+      for (const ProxyServer& proxy_server : proxy_chain.proxy_servers()) {
+        if (proxy_server.is_http_like() && !proxy_server.is_http() &&
+            servers.contains(proxy_server.host_port_pair())) {
+          session_matches = true;
+          break;
+        }
+      }
+    }
+
+    if (session_matches) {
       session->MakeUnavailable();
       // Note this call preserves active streams but fails any streams that are
       // waiting on a stream ID.
@@ -724,7 +737,7 @@ base::WeakPtr<SpdySession> SpdySessionPool::InsertSession(
   // potentially be pooled with this one. Because GetPeerAddress()
   // reports the proxy's address instead of the origin server, check
   // to see if this is a direct connection.
-  if (key.proxy_server().is_direct()) {
+  if (key.proxy_chain().is_direct()) {
     IPEndPoint address;
     if (available_session->GetPeerAddress(&address) == OK)
       aliases_.insert(AliasMap::value_type(address, key));
