@@ -4,8 +4,10 @@
 
 #include "src/api/api-natives.h"
 
+#include "src/api/api-inl.h"
 #include "src/common/message-template.h"
 #include "src/execution/isolate-inl.h"
+#include "src/execution/protectors-inl.h"
 #include "src/heap/heap-inl.h"
 #include "src/logging/runtime-call-stats-scope.h"
 #include "src/objects/api-callbacks.h"
@@ -22,7 +24,7 @@ class V8_NODISCARD InvokeScope {
   explicit InvokeScope(Isolate* isolate)
       : isolate_(isolate), save_context_(isolate) {}
   ~InvokeScope() {
-    bool has_exception = isolate_->has_pending_exception();
+    bool has_exception = isolate_->has_exception();
     if (has_exception) {
       isolate_->ReportPendingMessages();
     } else {
@@ -55,10 +57,10 @@ MaybeHandle<JSFunction> InstantiateFunction(
 MaybeHandle<Object> Instantiate(
     Isolate* isolate, Handle<Object> data,
     MaybeHandle<Name> maybe_name = MaybeHandle<Name>()) {
-  if (data->IsFunctionTemplateInfo()) {
+  if (IsFunctionTemplateInfo(*data)) {
     return InstantiateFunction(
         isolate, Handle<FunctionTemplateInfo>::cast(data), maybe_name);
-  } else if (data->IsObjectTemplateInfo()) {
+  } else if (IsObjectTemplateInfo(*data)) {
     return InstantiateObject(isolate, Handle<ObjectTemplateInfo>::cast(data),
                              Handle<JSReceiver>(), false);
   } else {
@@ -72,12 +74,12 @@ MaybeHandle<Object> DefineAccessorProperty(Isolate* isolate,
                                            Handle<Object> getter,
                                            Handle<Object> setter,
                                            PropertyAttributes attributes) {
-  DCHECK(!getter->IsFunctionTemplateInfo() ||
-         FunctionTemplateInfo::cast(*getter).should_cache());
-  DCHECK(!setter->IsFunctionTemplateInfo() ||
-         FunctionTemplateInfo::cast(*setter).should_cache());
-  if (getter->IsFunctionTemplateInfo() &&
-      FunctionTemplateInfo::cast(*getter).BreakAtEntry()) {
+  DCHECK(!IsFunctionTemplateInfo(*getter) ||
+         FunctionTemplateInfo::cast(*getter)->should_cache());
+  DCHECK(!IsFunctionTemplateInfo(*setter) ||
+         FunctionTemplateInfo::cast(*setter)->should_cache());
+  if (IsFunctionTemplateInfo(*getter) &&
+      FunctionTemplateInfo::cast(*getter)->BreakAtEntry(isolate)) {
     ASSIGN_RETURN_ON_EXCEPTION(
         isolate, getter,
         InstantiateFunction(isolate,
@@ -86,8 +88,8 @@ MaybeHandle<Object> DefineAccessorProperty(Isolate* isolate,
     Handle<Code> trampoline = BUILTIN_CODE(isolate, DebugBreakTrampoline);
     Handle<JSFunction>::cast(getter)->set_code(*trampoline);
   }
-  if (setter->IsFunctionTemplateInfo() &&
-      FunctionTemplateInfo::cast(*setter).BreakAtEntry()) {
+  if (IsFunctionTemplateInfo(*setter) &&
+      FunctionTemplateInfo::cast(*setter)->BreakAtEntry(isolate)) {
     ASSIGN_RETURN_ON_EXCEPTION(
         isolate, setter,
         InstantiateFunction(isolate,
@@ -145,7 +147,7 @@ void EnableAccessChecks(Isolate* isolate, Handle<JSObject> object) {
   // Copy map so it won't interfere constructor's initial map.
   Handle<Map> new_map = Map::Copy(isolate, old_map, "EnableAccessChecks");
   new_map->set_is_access_check_needed(true);
-  new_map->set_may_have_interesting_symbols(true);
+  new_map->set_may_have_interesting_properties(true);
   JSObject::MigrateToMap(isolate, object, new_map);
 }
 
@@ -153,7 +155,7 @@ class V8_NODISCARD AccessCheckDisableScope {
  public:
   AccessCheckDisableScope(Isolate* isolate, Handle<JSObject> obj)
       : isolate_(isolate),
-        disabled_(obj->map().is_access_check_needed()),
+        disabled_(obj->map()->is_access_check_needed()),
         obj_(obj) {
     if (disabled_) {
       DisableAccessChecks(isolate_, obj_);
@@ -171,7 +173,7 @@ class V8_NODISCARD AccessCheckDisableScope {
   Handle<JSObject> obj_;
 };
 
-Object GetIntrinsic(Isolate* isolate, v8::Intrinsic intrinsic) {
+Tagged<Object> GetIntrinsic(Isolate* isolate, v8::Intrinsic intrinsic) {
   Handle<Context> native_context = isolate->native_context();
   DCHECK(!native_context.is_null());
   switch (intrinsic) {
@@ -181,7 +183,7 @@ Object GetIntrinsic(Isolate* isolate, v8::Intrinsic intrinsic) {
     V8_INTRINSICS_LIST(GET_INTRINSIC_VALUE)
 #undef GET_INTRINSIC_VALUE
   }
-  return Object();
+  return Tagged<Object>();
 }
 
 template <typename TemplateInfoT>
@@ -194,13 +196,13 @@ MaybeHandle<JSObject> ConfigureInstance(Isolate* isolate, Handle<JSObject> obj,
 
   // Walk the inheritance chain and copy all accessors to current object.
   int max_number_of_properties = 0;
-  TemplateInfoT info = *data;
+  Tagged<TemplateInfoT> info = *data;
   while (!info.is_null()) {
-    Object props = info.property_accessors();
-    if (!props.IsUndefined(isolate)) {
-      max_number_of_properties += TemplateList::cast(props).length();
+    Tagged<Object> props = info->property_accessors();
+    if (!IsUndefined(props, isolate)) {
+      max_number_of_properties += ArrayList::cast(props)->length();
     }
-    info = info.GetParent(isolate);
+    info = info->GetParent(isolate);
   }
 
   if (max_number_of_properties > 0) {
@@ -209,11 +211,13 @@ MaybeHandle<JSObject> ConfigureInstance(Isolate* isolate, Handle<JSObject> obj,
     Handle<FixedArray> array =
         isolate->factory()->NewFixedArray(max_number_of_properties);
 
-    for (Handle<TemplateInfoT> temp(*data, isolate); !temp->is_null();
+    // TODO(leszeks): Avoid creating unnecessary handles for cases where we
+    // don't need to append anything.
+    for (Handle<TemplateInfoT> temp(*data, isolate); !(*temp).is_null();
          temp = handle(temp->GetParent(isolate), isolate)) {
       // Accumulate accessors.
-      Object maybe_properties = temp->property_accessors();
-      if (!maybe_properties.IsUndefined(isolate)) {
+      Tagged<Object> maybe_properties = temp->property_accessors();
+      if (!IsUndefined(maybe_properties, isolate)) {
         valid_descriptors = AccessorInfo::AppendUnique(
             isolate, handle(maybe_properties, isolate), array,
             valid_descriptors);
@@ -230,17 +234,16 @@ MaybeHandle<JSObject> ConfigureInstance(Isolate* isolate, Handle<JSObject> obj,
     }
   }
 
-  Object maybe_property_list = data->property_list();
-  if (maybe_property_list.IsUndefined(isolate)) return obj;
-  Handle<TemplateList> properties(TemplateList::cast(maybe_property_list),
-                                  isolate);
+  Tagged<Object> maybe_property_list = data->property_list();
+  if (IsUndefined(maybe_property_list, isolate)) return obj;
+  Handle<ArrayList> properties(ArrayList::cast(maybe_property_list), isolate);
   if (properties->length() == 0) return obj;
 
   int i = 0;
   for (int c = 0; c < data->number_of_properties(); c++) {
     auto name = handle(Name::cast(properties->get(i++)), isolate);
-    Object bit = properties->get(i++);
-    if (bit.IsSmi()) {
+    Tagged<Object> bit = properties->get(i++);
+    if (IsSmi(bit)) {
       PropertyDetails details(Smi::cast(bit));
       PropertyAttributes attributes = details.attributes();
       PropertyKind kind = details.kind();
@@ -296,19 +299,19 @@ MaybeHandle<JSObject> ProbeInstantiationsCache(
   }
 
   if (serial_number < TemplateInfo::kFastTemplateInstantiationsCacheSize) {
-    FixedArray fast_cache =
+    Tagged<FixedArray> fast_cache =
         native_context->fast_template_instantiations_cache();
-    Handle<Object> object{fast_cache.get(serial_number), isolate};
-    if (object->IsTheHole(isolate)) return {};
+    Handle<Object> object{fast_cache->get(serial_number), isolate};
+    if (IsTheHole(*object, isolate)) return {};
     return Handle<JSObject>::cast(object);
   }
   if (caching_mode == CachingMode::kUnlimited ||
       (serial_number < TemplateInfo::kSlowTemplateInstantiationsCacheSize)) {
-    SimpleNumberDictionary slow_cache =
+    Tagged<SimpleNumberDictionary> slow_cache =
         native_context->slow_template_instantiations_cache();
-    InternalIndex entry = slow_cache.FindEntry(isolate, serial_number);
+    InternalIndex entry = slow_cache->FindEntry(isolate, serial_number);
     if (entry.is_found()) {
-      return handle(JSObject::cast(slow_cache.ValueAt(entry)), isolate);
+      return handle(JSObject::cast(slow_cache->ValueAt(entry)), isolate);
     }
   }
   return {};
@@ -360,10 +363,11 @@ void UncacheTemplateInstantiation(Isolate* isolate,
   if (serial_number < 0) return;
 
   if (serial_number < TemplateInfo::kFastTemplateInstantiationsCacheSize) {
-    FixedArray fast_cache =
+    Tagged<FixedArray> fast_cache =
         native_context->fast_template_instantiations_cache();
-    DCHECK(!fast_cache.get(serial_number).IsUndefined(isolate));
-    fast_cache.set_undefined(serial_number);
+    DCHECK(!IsUndefined(fast_cache->get(serial_number), isolate));
+    fast_cache->set(serial_number, ReadOnlyRoots{isolate}.undefined_value(),
+                    SKIP_WRITE_BARRIER);
     data->set_serial_number(TemplateInfo::kUncached);
   } else if (caching_mode == CachingMode::kUnlimited ||
              (serial_number <
@@ -378,16 +382,16 @@ void UncacheTemplateInstantiation(Isolate* isolate,
   }
 }
 
-bool IsSimpleInstantiation(Isolate* isolate, ObjectTemplateInfo info,
-                           JSReceiver new_target) {
+bool IsSimpleInstantiation(Isolate* isolate, Tagged<ObjectTemplateInfo> info,
+                           Tagged<JSReceiver> new_target) {
   DisallowGarbageCollection no_gc;
 
-  if (!new_target.IsJSFunction()) return false;
-  JSFunction fun = JSFunction::cast(new_target);
-  if (fun.shared().function_data(kAcquireLoad) != info.constructor())
-    return false;
-  if (info.immutable_proto()) return false;
-  return fun.native_context() == isolate->raw_native_context();
+  if (!IsJSFunction(new_target)) return false;
+  Tagged<JSFunction> fun = JSFunction::cast(new_target);
+  if (!fun->shared()->IsApiFunction()) return false;
+  if (fun->shared()->api_func_data() != info->constructor()) return false;
+  if (info->immutable_proto()) return false;
+  return fun->native_context() == isolate->raw_native_context();
 }
 
 MaybeHandle<JSObject> InstantiateObject(Isolate* isolate,
@@ -416,8 +420,8 @@ MaybeHandle<JSObject> InstantiateObject(Isolate* isolate,
   }
 
   if (constructor.is_null()) {
-    Object maybe_constructor_info = info->constructor();
-    if (maybe_constructor_info.IsUndefined(isolate)) {
+    Tagged<Object> maybe_constructor_info = info->constructor();
+    if (IsUndefined(maybe_constructor_info, isolate)) {
       constructor = isolate->object_function();
     } else {
       // Enter a new scope.  Recursion could otherwise create a lot of handles.
@@ -500,10 +504,10 @@ MaybeHandle<JSFunction> InstantiateFunction(
   Handle<Object> prototype;
   if (!data->remove_prototype()) {
     Handle<Object> prototype_templ(data->GetPrototypeTemplate(), isolate);
-    if (prototype_templ->IsUndefined(isolate)) {
+    if (IsUndefined(*prototype_templ, isolate)) {
       Handle<Object> protoype_provider_templ(
           data->GetPrototypeProviderTemplate(), isolate);
-      if (protoype_provider_templ->IsUndefined(isolate)) {
+      if (IsUndefined(*protoype_provider_templ, isolate)) {
         prototype = isolate->factory()->NewJSObject(isolate->object_function());
       } else {
         ASSIGN_RETURN_ON_EXCEPTION(
@@ -519,20 +523,20 @@ MaybeHandle<JSFunction> InstantiateFunction(
           JSFunction);
     }
     Handle<Object> parent(data->GetParentTemplate(), isolate);
-    if (!parent->IsUndefined(isolate)) {
+    if (!IsUndefined(*parent, isolate)) {
       Handle<Object> parent_prototype;
       ASSIGN_RETURN_ON_EXCEPTION(isolate, parent_prototype,
                                  GetInstancePrototype(isolate, parent),
                                  JSFunction);
-      CHECK(parent_prototype->IsHeapObject());
+      CHECK(IsHeapObject(*parent_prototype));
       JSObject::ForceSetPrototype(isolate, Handle<JSObject>::cast(prototype),
                                   Handle<HeapObject>::cast(parent_prototype));
     }
   }
   InstanceType function_type = JS_SPECIAL_API_OBJECT_TYPE;
   if (!data->needs_access_check() &&
-      data->GetNamedPropertyHandler().IsUndefined(isolate) &&
-      data->GetIndexedPropertyHandler().IsUndefined(isolate)) {
+      IsUndefined(data->GetNamedPropertyHandler(), isolate) &&
+      IsUndefined(data->GetIndexedPropertyHandler(), isolate)) {
     function_type = v8_flags.embedder_instance_types && data->HasInstanceType()
                         ? static_cast<InstanceType>(data->InstanceType())
                         : JS_API_OBJECT_TYPE;
@@ -558,12 +562,12 @@ MaybeHandle<JSFunction> InstantiateFunction(
 
 void AddPropertyToPropertyList(Isolate* isolate, Handle<TemplateInfo> templ,
                                int length, Handle<Object>* data) {
-  Object maybe_list = templ->property_list();
-  Handle<TemplateList> list;
-  if (maybe_list.IsUndefined(isolate)) {
-    list = TemplateList::New(isolate, length);
+  Tagged<Object> maybe_list = templ->property_list();
+  Handle<ArrayList> list;
+  if (IsUndefined(maybe_list, isolate)) {
+    list = ArrayList::New(isolate, length, AllocationType::kOld);
   } else {
-    list = handle(TemplateList::cast(maybe_list), isolate);
+    list = handle(ArrayList::cast(maybe_list), isolate);
   }
   templ->set_number_of_properties(templ->number_of_properties() + 1);
   for (int i = 0; i < length; i++) {
@@ -571,13 +575,26 @@ void AddPropertyToPropertyList(Isolate* isolate, Handle<TemplateInfo> templ,
         data[i].is_null()
             ? Handle<Object>::cast(isolate->factory()->undefined_value())
             : data[i];
-    list = TemplateList::Add(isolate, list, value);
+    list = ArrayList::Add(isolate, list, value);
   }
   templ->set_property_list(*list);
 }
 
 }  // namespace
 
+// static
+i::Handle<i::FunctionTemplateInfo>
+ApiNatives::CreateAccessorFunctionTemplateInfo(
+    i::Isolate* i_isolate, FunctionCallback callback, int length,
+    SideEffectType side_effect_type) {
+  // TODO(v8:5962): move FunctionTemplateNew() from api.cc here.
+  auto isolate = reinterpret_cast<v8::Isolate*>(i_isolate);
+  Local<FunctionTemplate> func_template = FunctionTemplate::New(
+      isolate, callback, v8::Local<Value>{}, v8::Local<v8::Signature>{}, length,
+      v8::ConstructorBehavior::kThrow, side_effect_type);
+  return Utils::OpenHandle(*func_template);
+}
+
 MaybeHandle<JSFunction> ApiNatives::InstantiateFunction(
     Isolate* isolate, Handle<NativeContext> native_context,
     Handle<FunctionTemplateInfo> data, MaybeHandle<Name> maybe_name) {
@@ -587,8 +604,8 @@ MaybeHandle<JSFunction> ApiNatives::InstantiateFunction(
 }
 
 MaybeHandle<JSFunction> ApiNatives::InstantiateFunction(
-    Handle<FunctionTemplateInfo> data, MaybeHandle<Name> maybe_name) {
-  Isolate* isolate = data->GetIsolate();
+    Isolate* isolate, Handle<FunctionTemplateInfo> data,
+    MaybeHandle<Name> maybe_name) {
   InvokeScope invoke_scope(isolate);
   return ::v8::internal::InstantiateFunction(isolate, data, maybe_name);
 }
@@ -607,14 +624,14 @@ MaybeHandle<JSObject> ApiNatives::InstantiateRemoteObject(
 
   Handle<FunctionTemplateInfo> constructor(
       FunctionTemplateInfo::cast(data->constructor()), isolate);
-  Handle<Map> object_map = isolate->factory()->NewMap(
+  Handle<Map> object_map = isolate->factory()->NewContextlessMap(
       JS_SPECIAL_API_OBJECT_TYPE,
       JSObject::kHeaderSize +
           data->embedder_field_count() * kEmbedderDataSlotSize,
       TERMINAL_FAST_ELEMENTS_KIND);
   object_map->SetConstructor(*constructor);
   object_map->set_is_access_check_needed(true);
-  object_map->set_may_have_interesting_symbols(true);
+  object_map->set_may_have_interesting_properties(true);
 
   Handle<JSObject> object = isolate->factory()->NewJSObjectFromMap(object_map);
   JSObject::ForceSetPrototype(isolate, object,
@@ -663,14 +680,14 @@ void ApiNatives::AddAccessorProperty(Isolate* isolate,
 void ApiNatives::AddNativeDataProperty(Isolate* isolate,
                                        Handle<TemplateInfo> info,
                                        Handle<AccessorInfo> property) {
-  Object maybe_list = info->property_accessors();
-  Handle<TemplateList> list;
-  if (maybe_list.IsUndefined(isolate)) {
-    list = TemplateList::New(isolate, 1);
+  Tagged<Object> maybe_list = info->property_accessors();
+  Handle<ArrayList> list;
+  if (IsUndefined(maybe_list, isolate)) {
+    list = ArrayList::New(isolate, 1, AllocationType::kOld);
   } else {
-    list = handle(TemplateList::cast(maybe_list), isolate);
+    list = handle(ArrayList::cast(maybe_list), isolate);
   }
-  list = TemplateList::Add(isolate, list, property);
+  list = ArrayList::Add(isolate, list, property);
   info->set_property_accessors(*list);
 }
 
@@ -690,8 +707,8 @@ Handle<JSFunction> ApiNatives::CreateApiFunction(
 
   if (obj->remove_prototype()) {
     DCHECK(prototype.is_null());
-    DCHECK(result->shared().IsApiFunction());
-    DCHECK(!result->IsConstructor());
+    DCHECK(result->shared()->IsApiFunction());
+    DCHECK(!IsConstructor(*result));
     DCHECK(!result->has_prototype_slot());
     return result;
   }
@@ -704,9 +721,9 @@ Handle<JSFunction> ApiNatives::CreateApiFunction(
     result->set_map(*isolate->sloppy_function_with_readonly_prototype_map());
   }
 
-  if (prototype->IsTheHole(isolate)) {
+  if (IsTheHole(*prototype, isolate)) {
     prototype = isolate->factory()->NewFunctionPrototype(result);
-  } else if (obj->GetPrototypeProviderTemplate().IsUndefined(isolate)) {
+  } else if (IsUndefined(obj->GetPrototypeProviderTemplate(), isolate)) {
     JSObject::AddProperty(isolate, Handle<JSObject>::cast(prototype),
                           isolate->factory()->constructor_string(), result,
                           DONT_ENUM);
@@ -714,7 +731,7 @@ Handle<JSFunction> ApiNatives::CreateApiFunction(
 
   int embedder_field_count = 0;
   bool immutable_proto = false;
-  if (!obj->GetInstanceTemplate().IsUndefined(isolate)) {
+  if (!IsUndefined(obj->GetInstanceTemplate(), isolate)) {
     Handle<ObjectTemplateInfo> GetInstanceTemplate = Handle<ObjectTemplateInfo>(
         ObjectTemplateInfo::cast(obj->GetInstanceTemplate()), isolate);
     embedder_field_count = GetInstanceTemplate->embedder_field_count();
@@ -726,8 +743,8 @@ Handle<JSFunction> ApiNatives::CreateApiFunction(
   int instance_size = JSObject::GetHeaderSize(type) +
                       kEmbedderDataSlotSize * embedder_field_count;
 
-  Handle<Map> map = isolate->factory()->NewMap(type, instance_size,
-                                               TERMINAL_FAST_ELEMENTS_KIND);
+  Handle<Map> map = isolate->factory()->NewContextfulMap(
+      native_context, type, instance_size, TERMINAL_FAST_ELEMENTS_KIND);
 
   // Mark as undetectable if needed.
   if (obj->undetectable()) {
@@ -736,27 +753,31 @@ Handle<JSFunction> ApiNatives::CreateApiFunction(
     // undetectable and callable. If we ever see the need to have an object
     // that is undetectable but not callable, we need to update the types.h
     // to allow encoding this.
-    CHECK(!obj->GetInstanceCallHandler().IsUndefined(isolate));
+    CHECK(!IsUndefined(obj->GetInstanceCallHandler(), isolate));
+
+    if (Protectors::IsNoUndetectableObjectsIntact(isolate)) {
+      Protectors::InvalidateNoUndetectableObjects(isolate);
+    }
     map->set_is_undetectable(true);
   }
 
   // Mark as needs_access_check if needed.
   if (obj->needs_access_check()) {
     map->set_is_access_check_needed(true);
-    map->set_may_have_interesting_symbols(true);
+    map->set_may_have_interesting_properties(true);
   }
 
   // Set interceptor information in the map.
-  if (!obj->GetNamedPropertyHandler().IsUndefined(isolate)) {
+  if (!IsUndefined(obj->GetNamedPropertyHandler(), isolate)) {
     map->set_has_named_interceptor(true);
-    map->set_may_have_interesting_symbols(true);
+    map->set_may_have_interesting_properties(true);
   }
-  if (!obj->GetIndexedPropertyHandler().IsUndefined(isolate)) {
+  if (!IsUndefined(obj->GetIndexedPropertyHandler(), isolate)) {
     map->set_has_indexed_interceptor(true);
   }
 
   // Mark instance as callable in the map.
-  if (!obj->GetInstanceCallHandler().IsUndefined(isolate)) {
+  if (!IsUndefined(obj->GetInstanceCallHandler(), isolate)) {
     map->set_is_callable(true);
     map->set_is_constructor(!obj->undetectable());
   }
