@@ -149,7 +149,7 @@ class V8_EXPORT_PRIVATE Scope : public NON_EXPORTED_BASE(ZoneObject) {
   template <typename IsolateT>
   EXPORT_TEMPLATE_DECLARE(V8_EXPORT_PRIVATE)
   static Scope* DeserializeScopeChain(IsolateT* isolate, Zone* zone,
-                                      ScopeInfo scope_info,
+                                      Tagged<ScopeInfo> scope_info,
                                       DeclarationScope* script_scope,
                                       AstValueFactory* ast_value_factory,
                                       DeserializationMode deserialization_mode);
@@ -227,9 +227,6 @@ class V8_EXPORT_PRIVATE Scope : public NON_EXPORTED_BASE(ZoneObject) {
   VariableProxy* NewUnresolved(AstNodeFactory* factory,
                                const AstRawString* name, int start_pos,
                                VariableKind kind = NORMAL_VARIABLE) {
-    // Note that we must not share the unresolved variables with
-    // the same name because they may be removed selectively via
-    // RemoveUnresolved().
     DCHECK(!already_resolved_);
     DCHECK_EQ(factory->zone(), zone());
     VariableProxy* proxy = factory->NewVariableProxy(name, kind, start_pos);
@@ -239,11 +236,6 @@ class V8_EXPORT_PRIVATE Scope : public NON_EXPORTED_BASE(ZoneObject) {
 
   void AddUnresolved(VariableProxy* proxy);
 
-  // Removes an unresolved variable from the list so it can be readded to
-  // another list. This is used to reparent parameter initializers that contain
-  // sloppy eval.
-  bool RemoveUnresolved(VariableProxy* var);
-
   // Deletes an unresolved variable. The variable proxy cannot be reused for
   // another list later. During parsing, an unresolved variable may have been
   // added optimistically, but then only the variable name was used (typically
@@ -328,6 +320,14 @@ class V8_EXPORT_PRIVATE Scope : public NON_EXPORTED_BASE(ZoneObject) {
   //     switch (tag) { cases }
   //   start position: start position of '{'
   //   end position: end position of '}'
+  // * For the scope of a class literal or declaration
+  //     class A extends B { body }
+  //   start position: start position of 'class'
+  //   end position: end position of '}'
+  // * For the scope of a class member initializer functions:
+  //     class A extends B { body }
+  //   start position: start position of '{'
+  //   end position: end position of '}'
   int start_position() const { return start_position_; }
   void set_start_position(int statement_pos) {
     start_position_ = statement_pos;
@@ -603,10 +603,6 @@ class V8_EXPORT_PRIVATE Scope : public NON_EXPORTED_BASE(ZoneObject) {
     needs_home_object_ = true;
   }
 
-  VariableProxy* NewHomeObjectVariableProxy(AstNodeFactory* factory,
-                                            const AstRawString* name,
-                                            int start_pos);
-
   bool RemoveInnerScope(Scope* inner_scope) {
     DCHECK_NOT_NULL(inner_scope);
     if (inner_scope == inner_scope_) {
@@ -865,7 +861,7 @@ class V8_EXPORT_PRIVATE DeclarationScope : public Scope {
   FunctionKind function_kind() const { return function_kind_; }
 
   // Inform the scope that the corresponding code uses "super".
-  Scope* RecordSuperPropertyUsage() {
+  void RecordSuperPropertyUsage() {
     DCHECK(IsConciseMethod(function_kind()) ||
            IsAccessorFunction(function_kind()) ||
            IsClassConstructor(function_kind()));
@@ -873,7 +869,6 @@ class V8_EXPORT_PRIVATE DeclarationScope : public Scope {
     Scope* home_object_scope = GetHomeObjectScope();
     DCHECK_NOT_NULL(home_object_scope);
     home_object_scope->set_needs_home_object();
-    return home_object_scope;
   }
 
   bool uses_super_property() const { return uses_super_property_; }
@@ -1491,7 +1486,7 @@ class V8_EXPORT_PRIVATE ClassScope : public Scope {
   // If the reparsed scope declares any variable that needs allocation
   // fixup using the scope info, needs_allocation_fixup is true.
   void FinalizeReparsedClassScope(Isolate* isolate,
-                                  MaybeHandle<ScopeInfo> outer_scope_info,
+                                  MaybeHandle<ScopeInfo> maybe_class_scope_info,
                                   AstValueFactory* ast_value_factory,
                                   bool needs_allocation_fixup);
 #ifdef DEBUG
