@@ -8,6 +8,7 @@
 #include <limits>
 #include <type_traits>
 
+#include "include/v8config.h"
 #include "src/base/compiler-specific.h"
 #include "src/base/logging.h"
 
@@ -46,68 +47,43 @@ template <typename T, size_t N>
 char (&ArraySizeHelper(const T (&array)[N]))[N];
 #endif
 
-// bit_cast<Dest,Source> is a template function that implements the
-// equivalent of "*reinterpret_cast<Dest*>(&source)".  We need this in
-// very low-level functions like the protobuf library and fast math
-// support.
+// This is an equivalent to C++20's std::bit_cast<>(), but with additional
+// warnings. It morally does what `*reinterpret_cast<Dest*>(&source)` does, but
+// the cast/deref pair is undefined behavior, while bit_cast<>() isn't.
 //
-//   float f = 3.14159265358979;
-//   int i = bit_cast<int32>(f);
-//   // i = 0x40490fdb
-//
-// The classical address-casting method is:
-//
-//   // WRONG
-//   float f = 3.14159265358979;            // WRONG
-//   int i = * reinterpret_cast<int*>(&f);  // WRONG
-//
-// The address-casting method actually produces undefined behavior
-// according to ISO C++ specification section 3.10 -15 -.  Roughly, this
-// section says: if an object in memory has one type, and a program
-// accesses it with a different type, then the result is undefined
-// behavior for most values of "different type".
-//
-// This is true for any cast syntax, either *(int*)&f or
-// *reinterpret_cast<int*>(&f).  And it is particularly true for
-// conversions between integral lvalues and floating-point lvalues.
-//
-// The purpose of 3.10 -15- is to allow optimizing compilers to assume
-// that expressions with different types refer to different memory.  gcc
-// 4.0.1 has an optimizer that takes advantage of this.  So a
-// non-conforming program quietly produces wildly incorrect output.
-//
-// The problem is not the use of reinterpret_cast.  The problem is type
-// punning: holding an object in memory of one type and reading its bits
-// back using a different type.
-//
-// The C++ standard is more subtle and complex than this, but that
-// is the basic idea.
-//
-// Anyways ...
-//
-// bit_cast<> calls memcpy() which is blessed by the standard,
-// especially by the example in section 3.9 .  Also, of course,
-// bit_cast<> wraps up the nasty logic in one place.
-//
-// Fortunately memcpy() is very fast.  In optimized mode, with a
-// constant size, gcc 2.95.3, gcc 4.0.1, and msvc 7.1 produce inline
-// code with the minimal amount of data movement.  On a 32-bit system,
-// memcpy(d,s,4) compiles to one load and one store, and memcpy(d,s,8)
-// compiles to two loads and two stores.
-//
-// I tested this code with gcc 2.95.3, gcc 4.0.1, icc 8.1, and msvc 7.1.
-//
-// WARNING: if Dest or Source is a non-POD type, the result of the memcpy
-// is likely to surprise you.
+// This is not a magic "get out of UB free" card. This must only be used on
+// values, not on references or pointers. For pointers, use
+// reinterpret_cast<>(), or static_cast<>() when casting between void* and other
+// pointers, and then look at https://eel.is/c++draft/basic.lval#11 as that's
+// probably UB also.
 namespace v8::base {
+
 template <class Dest, class Source>
 V8_INLINE Dest bit_cast(Source const& source) {
-  static_assert(sizeof(Dest) == sizeof(Source),
-                "source and dest must be same size");
+  static_assert(!std::is_pointer_v<Source>,
+                "bit_cast must not be used on pointer types");
+  static_assert(!std::is_pointer_v<Dest>,
+                "bit_cast must not be used on pointer types");
+  static_assert(!std::is_reference_v<Dest>,
+                "bit_cast must not be used on reference types");
+  static_assert(
+      sizeof(Dest) == sizeof(Source),
+      "bit_cast requires source and destination types to be the same size");
+  static_assert(std::is_trivially_copyable_v<Source>,
+                "bit_cast requires the source type to be trivially copyable");
+  static_assert(
+      std::is_trivially_copyable_v<Dest>,
+      "bit_cast requires the destination type to be trivially copyable");
+
+#if V8_HAS_BUILTIN_BIT_CAST
+  return __builtin_bit_cast(Dest, source);
+#else
   Dest dest;
   memcpy(&dest, &source, sizeof(dest));
   return dest;
+#endif
 }
+
 }  // namespace v8::base
 
 // Explicitly declare the assignment operator as deleted.
@@ -355,10 +331,13 @@ constexpr inline bool IsAligned(T value, U alignment) {
 }
 
 inline void* AlignedAddress(void* address, size_t alignment) {
-  // The alignment must be a power of two.
-  DCHECK_EQ(alignment & (alignment - 1), 0u);
-  return reinterpret_cast<void*>(reinterpret_cast<uintptr_t>(address) &
-                                 ~static_cast<uintptr_t>(alignment - 1));
+  return reinterpret_cast<void*>(
+      RoundDown(reinterpret_cast<uintptr_t>(address), alignment));
+}
+
+inline void* RoundUpAddress(void* address, size_t alignment) {
+  return reinterpret_cast<void*>(
+      RoundUp(reinterpret_cast<uintptr_t>(address), alignment));
 }
 
 // Bounds checks for float to integer conversions, which does truncation. Hence,
@@ -388,23 +367,19 @@ bool is_inbounds(float_t v) {
 
 // Setup for Windows shared library export.
 #define V8_EXPORT_ENUM
-#if defined(HIDE_PRIVATE_SYMBOLS)
-#define V8_EXPORT_PRIVATE
-#else //if !defined(HIDE_PRIVATE_SYMBOLS)
-#ifdef BUILDING_V8_SHARED
+#ifdef BUILDING_V8_SHARED_PRIVATE
 #define V8_EXPORT_PRIVATE __declspec(dllexport)
-#elif USING_V8_SHARED
+#elif USING_V8_SHARED_PRIVATE
 #define V8_EXPORT_PRIVATE __declspec(dllimport)
-#else //!(BUILDING_V8_SHARED || USING_V8_SHARED)
+#else
 #define V8_EXPORT_PRIVATE
-#endif
-#endif
+#endif  // BUILDING_V8_SHARED
 
 #else  // V8_OS_WIN
 
 // Setup for Linux shared library export.
 #if V8_HAS_ATTRIBUTE_VISIBILITY
-#ifdef BUILDING_V8_SHARED
+#ifdef BUILDING_V8_SHARED_PRIVATE
 #define V8_EXPORT_PRIVATE __attribute__((visibility("default")))
 #define V8_EXPORT_ENUM V8_EXPORT_PRIVATE
 #else
@@ -434,7 +409,16 @@ bool is_inbounds(float_t v) {
 #define IF_TSAN(V, ...) EXPAND(V(__VA_ARGS__))
 #else
 #define IF_TSAN(V, ...)
-#endif  // V8_ENABLE_WEBASSEMBLY
+#endif  // V8_IS_TSAN
+
+// Defines IF_INTL, to be used in macro lists for elements that should only be
+// there if INTL is enabled.
+#ifdef V8_INTL_SUPPORT
+// EXPAND is needed to work around MSVC's broken __VA_ARGS__ expansion.
+#define IF_INTL(V, ...) EXPAND(V(__VA_ARGS__))
+#else
+#define IF_INTL(V, ...)
+#endif  // V8_INTL_SUPPORT
 
 // Defines IF_TARGET_ARCH_64_BIT, to be used in macro lists for elements that
 // should only be there if the target architecture is a 64-bit one.
