@@ -9,6 +9,8 @@
 #include <errno.h>
 #include <limits.h>
 #include <pthread.h>
+
+#include "src/base/logging.h"
 #if defined(__DragonFly__) || defined(__FreeBSD__) || defined(__OpenBSD__)
 #include <pthread_np.h>  // for pthread_set_name_np
 #endif
@@ -34,10 +36,9 @@
 #include <cmath>
 #include <cstdlib>
 
-#include "src/base/platform/platform-posix.h"
-
 #include "src/base/lazy-instance.h"
 #include "src/base/macros.h"
+#include "src/base/platform/platform-posix.h"
 #include "src/base/platform/platform.h"
 #include "src/base/platform/time.h"
 #include "src/base/utils/random-number-generator.h"
@@ -99,8 +100,6 @@ namespace {
 // 0 is never a valid thread id.
 const pthread_t kNoThread = static_cast<pthread_t>(0);
 
-bool g_hard_abort = false;
-
 const char* g_gc_fake_mmap = nullptr;
 
 DEFINE_LAZY_LEAKY_OBJECT_GETTER(RandomNumberGenerator,
@@ -158,6 +157,12 @@ void* Allocate(void* hint, size_t size, OS::MemoryPermission access,
   int flags = GetFlagsForMemoryPermission(access, page_type);
   void* result = mmap(hint, size, prot, flags, kMmapFd, kMmapFdOffset);
   if (result == MAP_FAILED) return nullptr;
+
+#if V8_OS_LINUX && V8_ENABLE_PRIVATE_MAPPING_FORK_OPTIMIZATION
+  // This is advisory, so we ignore errors.
+  madvise(result, size, MADV_DONTFORK);
+#endif
+
 #if ENABLE_HUGEPAGE
   if (result != nullptr && size >= kHugePageSize) {
     const uintptr_t huge_start =
@@ -243,14 +248,15 @@ bool OS::ArmUsingHardFloat() {
 #endif  // def __arm__
 #endif
 
-void PosixInitializeCommon(bool hard_abort, const char* const gc_fake_mmap) {
-  g_hard_abort = hard_abort;
+void PosixInitializeCommon(AbortMode abort_mode,
+                           const char* const gc_fake_mmap) {
+  g_abort_mode = abort_mode;
   g_gc_fake_mmap = gc_fake_mmap;
 }
 
 #if !V8_OS_FUCHSIA
-void OS::Initialize(bool hard_abort, const char* const gc_fake_mmap) {
-  PosixInitializeCommon(hard_abort, gc_fake_mmap);
+void OS::Initialize(AbortMode abort_mode, const char* const gc_fake_mmap) {
+  PosixInitializeCommon(abort_mode, gc_fake_mmap);
 }
 #endif  // !V8_OS_FUCHSIA
 
@@ -362,9 +368,9 @@ void* OS::GetRandomMmapAddr() {
   // this address for RISC-V. https://github.com/v8-riscv/v8/issues/375
   raw_addr &= 0x3FFFF000;
 #elif V8_TARGET_ARCH_LOONG64
-  // 42 bits of virtual addressing. Truncate to 40 bits to allow kernel chance
-  // to fulfill request.
-  raw_addr &= uint64_t{0xFFFFFF0000};
+  // 40 or 47 bits of virtual addressing. Truncate to 38 bits to allow kernel
+  // chance to fulfill request.
+  raw_addr &= uint64_t{0x3FFFFF0000};
 #else
   raw_addr &= 0x3FFFF000;
 
@@ -479,6 +485,7 @@ bool OS::SetPermissions(void* address, size_t size, MemoryPermission access) {
   int prot = GetProtectionFromMemoryPermission(access);
   int ret = mprotect(address, size, prot);
 
+  // Setting permissions can fail if the limit of VMAs is exceeded.
   // Any failure that's not OOM likely indicates a bug in the caller (e.g.
   // using an invalid mapping) so attempt to catch that here to facilitate
   // debugging of these failures.
@@ -588,6 +595,7 @@ bool OS::DecommitPages(void* address, size_t size) {
   void* ret = mmap(address, size, PROT_NONE,
                    MAP_FIXED | MAP_ANONYMOUS | MAP_PRIVATE, -1, 0);
   if (V8_UNLIKELY(ret == MAP_FAILED)) {
+    // Decommitting pages can fail if the limit of VMAs is exceeded.
     CHECK_EQ(ENOMEM, errno);
     return false;
   }
@@ -684,8 +692,13 @@ void OS::Sleep(TimeDelta interval) {
 
 
 void OS::Abort() {
-  if (g_hard_abort) {
-    IMMEDIATE_CRASH();
+  switch (g_abort_mode) {
+    case AbortMode::kSoft:
+      _exit(-1);
+    case AbortMode::kHard:
+      IMMEDIATE_CRASH();
+    case AbortMode::kDefault:
+      break;
   }
   // Redirect to std abort to signal abnormal program termination.
   abort();
@@ -839,6 +852,25 @@ int OS::GetUserTime(uint32_t* secs, uint32_t* usecs) {
 }
 #endif
 
+int OS::GetPeakMemoryUsageKb() {
+#if defined(V8_OS_FUCHSIA)
+  // Fuchsia does not implement getrusage()
+  return -1;
+#else
+  struct rusage usage;
+  if (getrusage(RUSAGE_SELF, &usage) < 0) return -1;
+
+#if defined(V8_OS_MACOS) || defined(V8_OS_IOS)
+  constexpr int KB = 1024;
+  // MacOS and iOS ru_maxrss count bytes
+  return static_cast<int>(usage.ru_maxrss / KB);
+#else
+  // Most other cases (at least Linux, IOS, return kilobytes)
+  return static_cast<int>(usage.ru_maxrss);
+#endif  // defined(V8_OS_MACOS) || defined(V8_OS_IOS)
+#endif  // defined(V8_OS_FUCHSIA)
+}
+
 double OS::TimeCurrentMillis() {
   return Time::Now().ToJsTime();
 }
@@ -921,10 +953,12 @@ void OS::FPrint(FILE* out, const char* format, ...) {
 
 void OS::VFPrint(FILE* out, const char* format, va_list args) {
 #if defined(ANDROID) && !defined(V8_ANDROID_LOG_STDOUT)
-  __android_log_vprint(ANDROID_LOG_INFO, LOG_TAG, format, args);
-#else
-  vfprintf(out, format, args);
+  if (out == stdout) {
+    __android_log_vprint(ANDROID_LOG_INFO, LOG_TAG, format, args);
+    return;
+  }
 #endif
+  vfprintf(out, format, args);
 }
 
 
@@ -1077,13 +1111,13 @@ class Thread::PlatformData {
 Thread::Thread(const Options& options)
     : data_(new PlatformData),
       stack_size_(options.stack_size()),
+      priority_(options.priority()),
       start_semaphore_(nullptr) {
   const int min_stack_size = static_cast<int>(PTHREAD_STACK_MIN);
   if (stack_size_ > 0) stack_size_ = std::max(stack_size_, min_stack_size);
   set_name(options.name());
 }
 
-
 Thread::~Thread() {
   delete data_;
 }
@@ -1114,7 +1148,6 @@ static void SetThreadName(const char* name) {
 #endif
 }
 
-
 static void* ThreadEntry(void* arg) {
   Thread* thread = reinterpret_cast<Thread*>(arg);
   // We take the lock here to make sure that pthread_create finished first since
@@ -1122,6 +1155,35 @@ static void* ThreadEntry(void* arg) {
   // one).
   { MutexGuard lock_guard(&thread->data()->thread_creation_mutex_); }
   SetThreadName(thread->name());
+#if V8_OS_DARWIN
+  switch (thread->priority()) {
+    case Thread::Priority::kBestEffort:
+      pthread_set_qos_class_self_np(QOS_CLASS_BACKGROUND, 0);
+      break;
+    case Thread::Priority::kUserVisible:
+      pthread_set_qos_class_self_np(QOS_CLASS_USER_INITIATED, -1);
+      break;
+    case Thread::Priority::kUserBlocking:
+      pthread_set_qos_class_self_np(QOS_CLASS_USER_INITIATED, 0);
+      break;
+    case Thread::Priority::kDefault:
+      break;
+  }
+#elif V8_OS_LINUX
+  switch (thread->priority()) {
+    case Thread::Priority::kBestEffort:
+      setpriority(PRIO_PROCESS, 0, 10);
+      break;
+    case Thread::Priority::kUserVisible:
+      setpriority(PRIO_PROCESS, 0, 1);
+      break;
+    case Thread::Priority::kUserBlocking:
+      setpriority(PRIO_PROCESS, 0, 0);
+      break;
+    case Thread::Priority::kDefault:
+      break;
+  }
+#endif
   DCHECK_NE(thread->data()->thread_, kNoThread);
   thread->NotifyStartedAndRun();
   return nullptr;
@@ -1243,28 +1305,52 @@ void Thread::SetThreadLocal(LocalStorageKey key, void* value) {
 #if !defined(V8_OS_FREEBSD) && !defined(V8_OS_DARWIN) && !defined(_AIX) && \
     !defined(V8_OS_SOLARIS)
 
+namespace {
+#if DEBUG
+bool MainThreadIsCurrentThread() {
+  // This method assumes the first time is called is from the main thread.
+  // It returns true for subsequent calls only if they are called from the
+  // same thread.
+  static int main_thread_id = -1;
+  if (main_thread_id == -1) {
+    main_thread_id = OS::GetCurrentThreadId();
+  }
+  return main_thread_id == OS::GetCurrentThreadId();
+}
+#endif  // DEBUG
+}  // namespace
+
 // static
 Stack::StackSlot Stack::ObtainCurrentThreadStackStart() {
   pthread_attr_t attr;
   int error = pthread_getattr_np(pthread_self(), &attr);
-  if (!error) {
-    void* base;
-    size_t size;
-    error = pthread_attr_getstack(&attr, &base, &size);
-    CHECK(!error);
-    pthread_attr_destroy(&attr);
-    return reinterpret_cast<uint8_t*>(base) + size;
-  }
-
+  if (error) {
+    DCHECK(MainThreadIsCurrentThread());
 #if defined(V8_LIBC_GLIBC)
-  // pthread_getattr_np can fail for the main thread. In this case
-  // just like NaCl we rely on the __libc_stack_end to give us
-  // the start of the stack.
-  // See https://code.google.com/p/nativeclient/issues/detail?id=3431.
-  return __libc_stack_end;
+    // pthread_getattr_np can fail for the main thread.
+    // For the main thread we prefer using __libc_stack_end (if it exists) since
+    // it generally provides a tighter limit for CSS.
+    return __libc_stack_end;
 #else
-  return nullptr;
+    return nullptr;
 #endif  // !defined(V8_LIBC_GLIBC)
+  }
+  void* base;
+  size_t size;
+  error = pthread_attr_getstack(&attr, &base, &size);
+  CHECK(!error);
+  pthread_attr_destroy(&attr);
+  void* stack_start = reinterpret_cast<uint8_t*>(base) + size;
+#if defined(V8_LIBC_GLIBC)
+  // __libc_stack_end is process global and thus is only valid for
+  // the main thread. Check whether this is the main thread by checking
+  // __libc_stack_end is within the thread's stack.
+  if ((base <= __libc_stack_end) && (__libc_stack_end <= stack_start)) {
+    DCHECK(MainThreadIsCurrentThread());
+    return __libc_stack_end;
+  }
+#endif  // !defined(V8_LIBC_GLIBC)
+  return stack_start;
 }
 
 #endif  // !defined(V8_OS_FREEBSD) && !defined(V8_OS_DARWIN) &&
