@@ -9,7 +9,7 @@
 #include "src/builtins/builtins-typed-array-gen.h"
 #include "src/builtins/builtins-utils-gen.h"
 #include "src/builtins/builtins.h"
-#include "src/codegen/code-stub-assembler.h"
+#include "src/codegen/code-stub-assembler-inl.h"
 #include "src/codegen/interface-descriptors-inl.h"
 #include "src/codegen/tnode.h"
 #include "src/execution/frame-constants.h"
@@ -38,7 +38,19 @@ void ArrayBuiltinsAssembler::TypedArrayMapResultGenerator() {
       context(), method_name, original_array, len());
   // In the Spec and our current implementation, the length check is already
   // performed in TypedArraySpeciesCreate.
-  CSA_DCHECK(this, UintPtrLessThanOrEqual(len(), LoadJSTypedArrayLength(a)));
+#ifdef DEBUG
+  Label detached_or_out_of_bounds(this), done(this);
+  CSA_DCHECK(this, UintPtrLessThanOrEqual(
+                       len(), LoadJSTypedArrayLengthAndCheckDetached(
+                                  a, &detached_or_out_of_bounds)));
+  Goto(&done);
+  BIND(&detached_or_out_of_bounds);
+  Unreachable();
+  BIND(&done);
+#endif  // DEBUG
+
+  // TODO(v8:11111): Make storing fast when the elements kinds only differ
+  // because of their RAB/GSABness.
   fast_typed_array_target_ =
       Word32Equal(LoadElementsKind(original_array), LoadElementsKind(a));
   a_ = a;
@@ -65,19 +77,23 @@ TNode<Object> ArrayBuiltinsAssembler::TypedArrayMapProcessor(
   // numValue be ? ToBigInt(v).
   // 3. Otherwise, let numValue be ? ToNumber(value).
   TNode<Object> num_value;
-  if (source_elements_kind_ == BIGINT64_ELEMENTS ||
-      source_elements_kind_ == BIGUINT64_ELEMENTS) {
+  if (IsBigIntTypedArrayElementsKind(source_elements_kind_)) {
     num_value = ToBigInt(context(), mapped_value);
   } else {
     num_value = ToNumber_Inline(context(), mapped_value);
   }
 
-  // The only way how this can bailout is because of a detached buffer.
+  // The only way how this can bailout is because of a detached or out of bounds
+  // buffer.
   // TODO(v8:4153): Consider checking IsDetachedBuffer() and calling
   // TypedArrayBuiltinsAssembler::StoreJSTypedArrayElementFromNumeric() here
   // instead to avoid converting k_number back to UintPtrT.
+
+  // Using source_elements_kind_ (not "target elements kind") is correct here,
+  // because the fast branch is taken only when the source and the target
+  // elements kinds match.
   EmitElementStore(CAST(a()), k_number, num_value, source_elements_kind_,
-                   KeyedAccessStoreMode::STANDARD_STORE, &detached, context());
+                   KeyedAccessStoreMode::kInBounds, &detached, context());
   Goto(&done);
 
   BIND(&slow);
@@ -179,25 +195,14 @@ void ArrayBuiltinsAssembler::GenerateIteratingTypedArrayBuiltinBody(
   for (auto it = labels.begin(); it != labels.end(); ++i, ++it) {
     BIND(&*it);
     source_elements_kind_ = static_cast<ElementsKind>(elements_kinds[i]);
-    // TODO(v8:11111): Only RAB-backed TAs need special handling here since the
-    // backing store can shrink mid-iteration. This implementation has an
-    // overzealous check for GSAB-backed length-tracking TAs. Then again, the
-    // non-RAB/GSAB code also has an overzealous detached check for SABs.
-    bool is_rab_gsab = IsRabGsabTypedArrayElementsKind(source_elements_kind_);
-    if (is_rab_gsab) {
-      source_elements_kind_ =
-          GetCorrespondingNonRabGsabElementsKind(source_elements_kind_);
-    }
-    VisitAllTypedArrayElements(array_buffer, processor, direction, typed_array,
-                               is_rab_gsab);
+    VisitAllTypedArrayElements(array_buffer, processor, direction, typed_array);
     ReturnFromBuiltin(a_.value());
   }
 }
 
 void ArrayBuiltinsAssembler::VisitAllTypedArrayElements(
     TNode<JSArrayBuffer> array_buffer, const CallResultProcessor& processor,
-    ForEachDirection direction, TNode<JSTypedArray> typed_array,
-    bool can_shrink) {
+    ForEachDirection direction, TNode<JSTypedArray> typed_array) {
   VariableList list({&a_, &k_}, zone());
 
   TNode<UintPtrT> start = UintPtrConstant(0);
@@ -210,13 +215,24 @@ void ArrayBuiltinsAssembler::VisitAllTypedArrayElements(
     incr = -1;
   }
   k_ = start;
+
+  // TODO(v8:11111): Only RAB-backed TAs need special handling here since the
+  // backing store can shrink mid-iteration. This implementation has an
+  // overzealous check for GSAB-backed length-tracking TAs. Then again, the
+  // non-RAB/GSAB code also has an overzealous detached check for SABs.
+  ElementsKind effective_elements_kind = source_elements_kind_;
+  bool is_rab_gsab = IsRabGsabTypedArrayElementsKind(effective_elements_kind);
+  if (is_rab_gsab) {
+    effective_elements_kind =
+        GetCorrespondingNonRabGsabElementsKind(effective_elements_kind);
+  }
   BuildFastLoop<UintPtrT>(
       list, start, end,
       [&](TNode<UintPtrT> index) {
         TVARIABLE(Object, value);
         Label detached(this, Label::kDeferred);
         Label process(this);
-        if (can_shrink) {
+        if (is_rab_gsab) {
           // If `index` is out of bounds, Get returns undefined.
           CheckJSTypedArrayIndex(typed_array, index, &detached);
         } else {
@@ -225,7 +241,7 @@ void ArrayBuiltinsAssembler::VisitAllTypedArrayElements(
         {
           TNode<RawPtrT> data_ptr = LoadJSTypedArrayDataPtr(typed_array);
           value = LoadFixedTypedArrayElementAsTagged(data_ptr, index,
-                                                     source_elements_kind_);
+                                                     effective_elements_kind);
           Goto(&process);
         }
 
@@ -268,33 +284,34 @@ TF_BUILTIN(ArrayPrototypePop, CodeStubAssembler) {
   {
     TNode<JSArray> array_receiver = CAST(receiver);
     CSA_DCHECK(this, TaggedIsPositiveSmi(LoadJSArrayLength(array_receiver)));
-    TNode<IntPtrT> length =
-        LoadAndUntagObjectField(array_receiver, JSArray::kLengthOffset);
+    TNode<Int32T> length =
+        LoadAndUntagToWord32ObjectField(array_receiver, JSArray::kLengthOffset);
     Label return_undefined(this), fast_elements(this);
 
     // 2) Ensure that the length is writable.
     EnsureArrayLengthWritable(context, LoadMap(array_receiver), &runtime);
 
-    GotoIf(IntPtrEqual(length, IntPtrConstant(0)), &return_undefined);
+    GotoIf(Word32Equal(length, Int32Constant(0)), &return_undefined);
 
     // 3) Check that the elements backing store isn't copy-on-write.
     TNode<FixedArrayBase> elements = LoadElements(array_receiver);
     GotoIf(TaggedEqual(LoadMap(elements), FixedCOWArrayMapConstant()),
            &runtime);
 
-    TNode<IntPtrT> new_length = IntPtrSub(length, IntPtrConstant(1));
+    TNode<Int32T> new_length = Int32Sub(length, Int32Constant(1));
 
     // 4) Check that we're not supposed to shrink the backing store, as
     //    implemented in elements.cc:ElementsAccessorBase::SetLengthImpl.
-    TNode<IntPtrT> capacity = SmiUntag(LoadFixedArrayBaseLength(elements));
-    GotoIf(IntPtrLessThan(
-               IntPtrAdd(IntPtrAdd(new_length, new_length),
-                         IntPtrConstant(JSObject::kMinAddedElementsCapacity)),
+    TNode<Int32T> capacity = SmiToInt32(LoadFixedArrayBaseLength(elements));
+    GotoIf(Int32LessThan(
+               Int32Add(Int32Add(new_length, new_length),
+                        Int32Constant(JSObject::kMinAddedElementsCapacity)),
                capacity),
            &runtime);
 
+    TNode<IntPtrT> new_length_intptr = ChangePositiveInt32ToIntPtr(new_length);
     StoreObjectFieldNoWriteBarrier(array_receiver, JSArray::kLengthOffset,
-                                   SmiTag(new_length));
+                                   SmiTag(new_length_intptr));
 
     TNode<Int32T> elements_kind = LoadElementsKind(array_receiver);
     GotoIf(Int32LessThanOrEqual(elements_kind,
@@ -305,9 +322,9 @@ TF_BUILTIN(ArrayPrototypePop, CodeStubAssembler) {
       TNode<FixedDoubleArray> elements_known_double_array =
           ReinterpretCast<FixedDoubleArray>(elements);
       TNode<Float64T> value = LoadFixedDoubleArrayElement(
-          elements_known_double_array, new_length, &return_undefined);
+          elements_known_double_array, new_length_intptr, &return_undefined);
 
-      StoreFixedDoubleArrayHole(elements_known_double_array, new_length);
+      StoreFixedDoubleArrayHole(elements_known_double_array, new_length_intptr);
       args.PopAndReturn(AllocateHeapNumberWithValue(value));
     }
 
@@ -315,8 +332,8 @@ TF_BUILTIN(ArrayPrototypePop, CodeStubAssembler) {
     {
       TNode<FixedArray> elements_known_fixed_array = CAST(elements);
       TNode<Object> value =
-          LoadFixedArrayElement(elements_known_fixed_array, new_length);
-      StoreFixedArrayElement(elements_known_fixed_array, new_length,
+          LoadFixedArrayElement(elements_known_fixed_array, new_length_intptr);
+      StoreFixedArrayElement(elements_known_fixed_array, new_length_intptr,
                              TheHoleConstant());
       GotoIf(TaggedEqual(value, TheHoleConstant()), &return_undefined);
       args.PopAndReturn(value);
@@ -659,7 +676,7 @@ void ArrayIncludesIndexofAssembler::Generate(SearchVariant variant,
   // JSArray length is always a positive Smi for fast arrays.
   CSA_DCHECK(this, TaggedIsPositiveSmi(LoadJSArrayLength(array)));
   TNode<Smi> array_length = LoadFastJSArrayLength(array);
-  TNode<IntPtrT> array_length_untagged = SmiUntag(array_length);
+  TNode<IntPtrT> array_length_untagged = PositiveSmiUntag(array_length);
 
   {
     // Initialize fromIndex.
@@ -727,49 +744,44 @@ void ArrayIncludesIndexofAssembler::Generate(SearchVariant variant,
 
   BIND(&if_smi);
   {
-    Callable callable = Builtins::CallableFor(
-        isolate(), (variant == kIncludes) ? Builtin::kArrayIncludesSmi
-                                          : Builtin::kArrayIndexOfSmi);
-    TNode<Object> result = CallStub(callable, context, elements, search_element,
-                                    array_length, SmiTag(index_var.value()));
+    Builtin builtin = (variant == kIncludes) ? Builtin::kArrayIncludesSmi
+                                             : Builtin::kArrayIndexOfSmi;
+    TNode<Object> result =
+        CallBuiltin(builtin, context, elements, search_element, array_length,
+                    SmiTag(index_var.value()));
     args.PopAndReturn(result);
   }
 
   BIND(&if_smiorobjects);
   {
-    Callable callable = (variant == kIncludes)
-                            ? Builtins::CallableFor(
-                                  isolate(), Builtin::kArrayIncludesSmiOrObject)
-                            : Builtins::CallableFor(
-                                  isolate(), Builtin::kArrayIndexOfSmiOrObject);
-    TNode<Object> result = CallStub(callable, context, elements, search_element,
-                                    array_length, SmiTag(index_var.value()));
+    Builtin builtin = (variant == kIncludes)
+                          ? Builtin::kArrayIncludesSmiOrObject
+                          : Builtin::kArrayIndexOfSmiOrObject;
+    TNode<Object> result =
+        CallBuiltin(builtin, context, elements, search_element, array_length,
+                    SmiTag(index_var.value()));
     args.PopAndReturn(result);
   }
 
   BIND(&if_packed_doubles);
   {
-    Callable callable =
-        (variant == kIncludes)
-            ? Builtins::CallableFor(isolate(),
-                                    Builtin::kArrayIncludesPackedDoubles)
-            : Builtins::CallableFor(isolate(),
-                                    Builtin::kArrayIndexOfPackedDoubles);
-    TNode<Object> result = CallStub(callable, context, elements, search_element,
-                                    array_length, SmiTag(index_var.value()));
+    Builtin builtin = (variant == kIncludes)
+                          ? Builtin::kArrayIncludesPackedDoubles
+                          : Builtin::kArrayIndexOfPackedDoubles;
+    TNode<Object> result =
+        CallBuiltin(builtin, context, elements, search_element, array_length,
+                    SmiTag(index_var.value()));
     args.PopAndReturn(result);
   }
 
   BIND(&if_holey_doubles);
   {
-    Callable callable =
-        (variant == kIncludes)
-            ? Builtins::CallableFor(isolate(),
-                                    Builtin::kArrayIncludesHoleyDoubles)
-            : Builtins::CallableFor(isolate(),
-                                    Builtin::kArrayIndexOfHoleyDoubles);
-    TNode<Object> result = CallStub(callable, context, elements, search_element,
-                                    array_length, SmiTag(index_var.value()));
+    Builtin builtin = (variant == kIncludes)
+                          ? Builtin::kArrayIncludesHoleyDoubles
+                          : Builtin::kArrayIndexOfHoleyDoubles;
+    TNode<Object> result =
+        CallBuiltin(builtin, context, elements, search_element, array_length,
+                    SmiTag(index_var.value()));
     args.PopAndReturn(result);
   }
 
@@ -797,7 +809,7 @@ void ArrayIncludesIndexofAssembler::GenerateSmiOrObject(
     TNode<Smi> from_index, SimpleElementKind array_kind) {
   TVARIABLE(IntPtrT, index_var, SmiUntag(from_index));
   TVARIABLE(Float64T, search_num);
-  TNode<IntPtrT> array_length_untagged = SmiUntag(array_length);
+  TNode<IntPtrT> array_length_untagged = PositiveSmiUntag(array_length);
 
   Label ident_loop(this, &index_var), heap_num_loop(this, &search_num),
       string_loop(this), bigint_loop(this, &index_var),
@@ -1019,7 +1031,7 @@ void ArrayIncludesIndexofAssembler::GeneratePackedDoubles(
     TNode<Object> search_element, TNode<Smi> array_length,
     TNode<Smi> from_index) {
   TVARIABLE(IntPtrT, index_var, SmiUntag(from_index));
-  TNode<IntPtrT> array_length_untagged = SmiUntag(array_length);
+  TNode<IntPtrT> array_length_untagged = PositiveSmiUntag(array_length);
 
   Label nan_loop(this, &index_var), not_nan_case(this),
       not_nan_loop(this, &index_var), hole_loop(this, &index_var),
@@ -1111,7 +1123,7 @@ void ArrayIncludesIndexofAssembler::GenerateHoleyDoubles(
     TNode<Object> search_element, TNode<Smi> array_length,
     TNode<Smi> from_index) {
   TVARIABLE(IntPtrT, index_var, SmiUntag(from_index));
-  TNode<IntPtrT> array_length_untagged = SmiUntag(array_length);
+  TNode<IntPtrT> array_length_untagged = PositiveSmiUntag(array_length);
 
   Label nan_loop(this, &index_var), not_nan_case(this),
       not_nan_loop(this, &index_var), hole_loop(this, &index_var),
@@ -1357,7 +1369,7 @@ TF_BUILTIN(ArrayIteratorPrototypeNext, CodeStubAssembler) {
   auto context = Parameter<Context>(Descriptor::kContext);
   auto maybe_iterator = Parameter<Object>(Descriptor::kReceiver);
 
-  TVARIABLE(Oddball, var_done, TrueConstant());
+  TVARIABLE(Boolean, var_done, TrueConstant());
   TVARIABLE(Object, var_value, UndefinedConstant());
 
   Label allocate_entry_if_needed(this);
@@ -1541,261 +1553,6 @@ TF_BUILTIN(ArrayIteratorPrototypeNext, CodeStubAssembler) {
   }
 }
 
-class ArrayFlattenAssembler : public CodeStubAssembler {
- public:
-  explicit ArrayFlattenAssembler(compiler::CodeAssemblerState* state)
-      : CodeStubAssembler(state) {}
-
-  // https://tc39.github.io/proposal-flatMap/#sec-FlattenIntoArray
-  TNode<Number> FlattenIntoArray(
-      TNode<Context> context, TNode<JSReceiver> target,
-      TNode<JSReceiver> source, TNode<Number> source_length,
-      TNode<Number> start, TNode<Number> depth,
-      base::Optional<TNode<HeapObject>> mapper_function = base::nullopt,
-      base::Optional<TNode<Object>> this_arg = base::nullopt) {
-    CSA_DCHECK(this, IsNumberPositive(source_length));
-    CSA_DCHECK(this, IsNumberPositive(start));
-
-    // 1. Let targetIndex be start.
-    TVARIABLE(Number, var_target_index, start);
-
-    // 2. Let sourceIndex be 0.
-    TVARIABLE(Number, var_source_index, SmiConstant(0));
-
-    // 3. Repeat...
-    Label loop(this, {&var_target_index, &var_source_index}), done_loop(this);
-    Goto(&loop);
-    BIND(&loop);
-    {
-      TNode<Number> source_index = var_source_index.value();
-      TNode<Number> target_index = var_target_index.value();
-
-      // ...while sourceIndex < sourceLen
-      GotoIfNumberGreaterThanOrEqual(source_index, source_length, &done_loop);
-
-      // a. Let P be ! ToString(sourceIndex).
-      // b. Let exists be ? HasProperty(source, P).
-      CSA_DCHECK(this,
-                 SmiGreaterThanOrEqual(CAST(source_index), SmiConstant(0)));
-      const TNode<Oddball> exists =
-          HasProperty(context, source, source_index, kHasProperty);
-
-      // c. If exists is true, then
-      Label next(this);
-      GotoIfNot(IsTrue(exists), &next);
-      {
-        // i. Let element be ? Get(source, P).
-        TNode<Object> element_maybe_smi =
-            GetProperty(context, source, source_index);
-
-        // ii. If mapperFunction is present, then
-        if (mapper_function) {
-          CSA_DCHECK(this, Word32Or(IsUndefined(mapper_function.value()),
-                                    IsCallable(mapper_function.value())));
-          DCHECK(this_arg.has_value());
-
-          // 1. Set element to ? Call(mapperFunction, thisArg , « element,
-          //                          sourceIndex, source »).
-          element_maybe_smi =
-              Call(context, mapper_function.value(), this_arg.value(),
-                   element_maybe_smi, source_index, source);
-        }
-
-        // iii. Let shouldFlatten be false.
-        Label if_flatten_array(this), if_flatten_proxy(this, Label::kDeferred),
-            if_noflatten(this);
-        // iv. If depth > 0, then
-        GotoIfNumberGreaterThanOrEqual(SmiConstant(0), depth, &if_noflatten);
-        // 1. Set shouldFlatten to ? IsArray(element).
-        GotoIf(TaggedIsSmi(element_maybe_smi), &if_noflatten);
-        TNode<HeapObject> element = CAST(element_maybe_smi);
-        GotoIf(IsJSArray(element), &if_flatten_array);
-        GotoIfNot(IsJSProxy(element), &if_noflatten);
-        Branch(IsTrue(CallRuntime(Runtime::kArrayIsArray, context, element)),
-               &if_flatten_proxy, &if_noflatten);
-
-        BIND(&if_flatten_array);
-        {
-          CSA_DCHECK(this, IsJSArray(element));
-
-          // 1. Let elementLen be ? ToLength(? Get(element, "length")).
-          const TNode<Object> element_length =
-              LoadObjectField(element, JSArray::kLengthOffset);
-
-          // 2. Set targetIndex to ? FlattenIntoArray(target, element,
-          //                                          elementLen, targetIndex,
-          //                                          depth - 1).
-          var_target_index = CAST(
-              CallBuiltin(Builtin::kFlattenIntoArray, context, target, element,
-                          element_length, target_index, NumberDec(depth)));
-          Goto(&next);
-        }
-
-        BIND(&if_flatten_proxy);
-        {
-          CSA_DCHECK(this, IsJSProxy(element));
-
-          // 1. Let elementLen be ? ToLength(? Get(element, "length")).
-          const TNode<Number> element_length = ToLength_Inline(
-              context, GetProperty(context, element, LengthStringConstant()));
-
-          // 2. Set targetIndex to ? FlattenIntoArray(target, element,
-          //                                          elementLen, targetIndex,
-          //                                          depth - 1).
-          var_target_index = CAST(
-              CallBuiltin(Builtin::kFlattenIntoArray, context, target, element,
-                          element_length, target_index, NumberDec(depth)));
-          Goto(&next);
-        }
-
-        BIND(&if_noflatten);
-        {
-          // 1. If targetIndex >= 2^53-1, throw a TypeError exception.
-          Label throw_error(this, Label::kDeferred);
-          GotoIfNumberGreaterThanOrEqual(
-              target_index, NumberConstant(kMaxSafeInteger), &throw_error);
-
-          // 2. Perform ? CreateDataPropertyOrThrow(target,
-          //                                        ! ToString(targetIndex),
-          //                                        element).
-          CallRuntime(Runtime::kCreateDataProperty, context, target,
-                      target_index, element);
-
-          // 3. Increase targetIndex by 1.
-          var_target_index = NumberInc(target_index);
-          Goto(&next);
-
-          BIND(&throw_error);
-          ThrowTypeError(context, MessageTemplate::kFlattenPastSafeLength,
-                         source_length, target_index);
-        }
-      }
-      BIND(&next);
-
-      // d. Increase sourceIndex by 1.
-      var_source_index = NumberInc(source_index);
-      Goto(&loop);
-    }
-
-    BIND(&done_loop);
-    return var_target_index.value();
-  }
-};
-
-// https://tc39.github.io/proposal-flatMap/#sec-FlattenIntoArray
-TF_BUILTIN(FlattenIntoArray, ArrayFlattenAssembler) {
-  auto context = Parameter<Context>(Descriptor::kContext);
-  auto target = Parameter<JSReceiver>(Descriptor::kTarget);
-  auto source = Parameter<JSReceiver>(Descriptor::kSource);
-  auto source_length = Parameter<Number>(Descriptor::kSourceLength);
-  auto start = Parameter<Number>(Descriptor::kStart);
-  auto depth = Parameter<Number>(Descriptor::kDepth);
-
-  // FlattenIntoArray might get called recursively, check stack for overflow
-  // manually as it has stub linkage.
-  PerformStackCheck(context);
-
-  Return(
-      FlattenIntoArray(context, target, source, source_length, start, depth));
-}
-
-// https://tc39.github.io/proposal-flatMap/#sec-FlattenIntoArray
-TF_BUILTIN(FlatMapIntoArray, ArrayFlattenAssembler) {
-  auto context = Parameter<Context>(Descriptor::kContext);
-  auto target = Parameter<JSReceiver>(Descriptor::kTarget);
-  auto source = Parameter<JSReceiver>(Descriptor::kSource);
-  auto source_length = Parameter<Number>(Descriptor::kSourceLength);
-  auto start = Parameter<Number>(Descriptor::kStart);
-  auto depth = Parameter<Number>(Descriptor::kDepth);
-  auto mapper_function = Parameter<HeapObject>(Descriptor::kMapperFunction);
-  auto this_arg = Parameter<Object>(Descriptor::kThisArg);
-
-  Return(FlattenIntoArray(context, target, source, source_length, start, depth,
-                          mapper_function, this_arg));
-}
-
-// https://tc39.github.io/proposal-flatMap/#sec-Array.prototype.flat
-TF_BUILTIN(ArrayPrototypeFlat, CodeStubAssembler) {
-  const TNode<IntPtrT> argc = ChangeInt32ToIntPtr(
-      UncheckedParameter<Int32T>(Descriptor::kJSActualArgumentsCount));
-  CodeStubArguments args(this, argc);
-  const auto context = Parameter<Context>(Descriptor::kContext);
-  const TNode<Object> receiver = args.GetReceiver();
-  const TNode<Object> depth = args.GetOptionalArgumentValue(0);
-
-  // 1. Let O be ? ToObject(this value).
-  const TNode<JSReceiver> o = ToObject_Inline(context, receiver);
-
-  // 2. Let sourceLen be ? ToLength(? Get(O, "length")).
-  const TNode<Number> source_length =
-      ToLength_Inline(context, GetProperty(context, o, LengthStringConstant()));
-
-  // 3. Let depthNum be 1.
-  TVARIABLE(Number, var_depth_num, SmiConstant(1));
-
-  // 4. If depth is not undefined, then
-  Label done(this);
-  GotoIf(IsUndefined(depth), &done);
-  {
-    // a. Set depthNum to ? ToInteger(depth).
-    var_depth_num = ToInteger_Inline(context, depth);
-    Goto(&done);
-  }
-  BIND(&done);
-
-  // 5. Let A be ? ArraySpeciesCreate(O, 0).
-  const TNode<JSReceiver> constructor =
-      CAST(CallRuntime(Runtime::kArraySpeciesConstructor, context, o));
-  const TNode<JSReceiver> a = Construct(context, constructor, SmiConstant(0));
-
-  // 6. Perform ? FlattenIntoArray(A, O, sourceLen, 0, depthNum).
-  CallBuiltin(Builtin::kFlattenIntoArray, context, a, o, source_length,
-              SmiConstant(0), var_depth_num.value());
-
-  // 7. Return A.
-  args.PopAndReturn(a);
-}
-
-// https://tc39.github.io/proposal-flatMap/#sec-Array.prototype.flatMap
-TF_BUILTIN(ArrayPrototypeFlatMap, CodeStubAssembler) {
-  const TNode<IntPtrT> argc = ChangeInt32ToIntPtr(
-      UncheckedParameter<Int32T>(Descriptor::kJSActualArgumentsCount));
-  CodeStubArguments args(this, argc);
-  const auto context = Parameter<Context>(Descriptor::kContext);
-  const TNode<Object> receiver = args.GetReceiver();
-  const TNode<Object> mapper_function = args.GetOptionalArgumentValue(0);
-
-  // 1. Let O be ? ToObject(this value).
-  const TNode<JSReceiver> o = ToObject_Inline(context, receiver);
-
-  // 2. Let sourceLen be ? ToLength(? Get(O, "length")).
-  const TNode<Number> source_length =
-      ToLength_Inline(context, GetProperty(context, o, LengthStringConstant()));
-
-  // 3. If IsCallable(mapperFunction) is false, throw a TypeError exception.
-  Label if_not_callable(this, Label::kDeferred);
-  GotoIf(TaggedIsSmi(mapper_function), &if_not_callable);
-  GotoIfNot(IsCallable(CAST(mapper_function)), &if_not_callable);
-
-  // 4. If thisArg is present, let T be thisArg; else let T be undefined.
-  const TNode<Object> t = args.GetOptionalArgumentValue(1);
-
-  // 5. Let A be ? ArraySpeciesCreate(O, 0).
-  const TNode<JSReceiver> constructor =
-      CAST(CallRuntime(Runtime::kArraySpeciesConstructor, context, o));
-  const TNode<JSReceiver> a = Construct(context, constructor, SmiConstant(0));
-
-  // 6. Perform ? FlattenIntoArray(A, O, sourceLen, 0, 1, mapperFunction, T).
-  CallBuiltin(Builtin::kFlatMapIntoArray, context, a, o, source_length,
-              SmiConstant(0), SmiConstant(1), mapper_function, t);
-
-  // 7. Return A.
-  args.PopAndReturn(a);
-
-  BIND(&if_not_callable);
-  { ThrowTypeError(context, MessageTemplate::kMapperFunctionNonCallable); }
-}
-
 TF_BUILTIN(ArrayConstructor, ArrayBuiltinsAssembler) {
   // This is a trampoline to ArrayConstructorImpl which just adds
   // allocation_site parameter value and sets new_target if necessary.
@@ -1818,7 +1575,7 @@ TF_BUILTIN(ArrayConstructor, ArrayBuiltinsAssembler) {
 void ArrayBuiltinsAssembler::TailCallArrayConstructorStub(
     const Callable& callable, TNode<Context> context, TNode<JSFunction> target,
     TNode<HeapObject> allocation_site_or_undefined, TNode<Int32T> argc) {
-  TNode<Code> code = HeapConstant(callable.code());
+  TNode<Code> code = HeapConstantNoHole(callable.code());
 
   // We are going to call here ArrayNoArgumentsConstructor or
   // ArraySingleArgumentsConstructor which in addition to the register arguments
