@@ -8,8 +8,7 @@
 #include "src/builtins/builtins-utils-gen.h"
 #include "src/builtins/builtins.h"
 #include "src/builtins/growable-fixed-array-gen.h"
-#include "src/codegen/code-factory.h"
-#include "src/codegen/code-stub-assembler.h"
+#include "src/codegen/code-stub-assembler-inl.h"
 #include "src/codegen/macro-assembler.h"
 #include "src/common/globals.h"
 #include "src/execution/protectors.h"
@@ -61,7 +60,7 @@ TNode<JSRegExpResult> RegExpBuiltinsAssembler::AllocateRegExpResult(
   Label result_has_indices(this), allocated(this);
   const ElementsKind elements_kind = PACKED_ELEMENTS;
   base::Optional<TNode<AllocationSite>> no_gc_site = base::nullopt;
-  TNode<IntPtrT> length_intptr = SmiUntag(length);
+  TNode<IntPtrT> length_intptr = PositiveSmiUntag(length);
   // Note: The returned `var_elements` may be in young large object space, but
   // `var_array` is guaranteed to be in new space so we could skip write
   // barriers below.
@@ -75,7 +74,7 @@ TNode<JSRegExpResult> RegExpBuiltinsAssembler::AllocateRegExpResult(
     std::tie(var_array, var_elements) =
         AllocateUninitializedJSArrayWithElements(
             elements_kind, map, length, no_gc_site, length_intptr,
-            AllocationFlag::kAllowLargeObjectAllocation, JSRegExpResult::kSize);
+            AllocationFlag::kNone, JSRegExpResult::kSize);
     Goto(&allocated);
   }
 
@@ -87,8 +86,7 @@ TNode<JSRegExpResult> RegExpBuiltinsAssembler::AllocateRegExpResult(
     std::tie(var_array, var_elements) =
         AllocateUninitializedJSArrayWithElements(
             elements_kind, map, length, no_gc_site, length_intptr,
-            AllocationFlag::kAllowLargeObjectAllocation,
-            JSRegExpResultWithIndices::kSize);
+            AllocationFlag::kNone, JSRegExpResultWithIndices::kSize);
     Goto(&allocated);
   }
 
@@ -171,7 +169,8 @@ void RegExpBuiltinsAssembler::FastStoreLastIndex(TNode<JSRegExp> regexp,
 void RegExpBuiltinsAssembler::SlowStoreLastIndex(TNode<Context> context,
                                                  TNode<Object> regexp,
                                                  TNode<Object> value) {
-  TNode<String> name = HeapConstant(isolate()->factory()->lastIndex_string());
+  TNode<String> name =
+      HeapConstantNoHole(isolate()->factory()->lastIndex_string());
   SetPropertyStrict(context, regexp, name, value);
 }
 
@@ -181,13 +180,11 @@ TNode<JSRegExpResult> RegExpBuiltinsAssembler::ConstructNewResultFromMatchInfo(
     TNode<Number> last_index) {
   Label named_captures(this), maybe_build_indices(this), out(this);
 
-  TNode<IntPtrT> num_indices = SmiUntag(CAST(UnsafeLoadFixedArrayElement(
-      match_info, RegExpMatchInfo::kNumberOfCapturesIndex)));
+  TNode<IntPtrT> num_indices = PositiveSmiUntag(CAST(LoadObjectField(
+      match_info, RegExpMatchInfo::kNumberOfCaptureRegistersOffset)));
   TNode<Smi> num_results = SmiTag(WordShr(num_indices, 1));
-  TNode<Smi> start = CAST(UnsafeLoadFixedArrayElement(
-      match_info, RegExpMatchInfo::kFirstCaptureIndex));
-  TNode<Smi> end = CAST(UnsafeLoadFixedArrayElement(
-      match_info, RegExpMatchInfo::kFirstCaptureIndex + 1));
+  TNode<Smi> start = LoadArrayElement(match_info, IntPtrConstant(0));
+  TNode<Smi> end = LoadArrayElement(match_info, IntPtrConstant(1));
 
   // Calculate the substring of the first match before creating the result array
   // to avoid an unnecessary write barrier storing the first result.
@@ -210,11 +207,9 @@ TNode<JSRegExpResult> RegExpBuiltinsAssembler::ConstructNewResultFromMatchInfo(
   GotoIf(SmiEqual(num_results, SmiConstant(1)), &maybe_build_indices);
 
   // Store all remaining captures.
-  TNode<IntPtrT> limit = IntPtrAdd(
-      IntPtrConstant(RegExpMatchInfo::kFirstCaptureIndex), num_indices);
+  TNode<IntPtrT> limit = num_indices;
 
-  TVARIABLE(IntPtrT, var_from_cursor,
-            IntPtrConstant(RegExpMatchInfo::kFirstCaptureIndex + 2));
+  TVARIABLE(IntPtrT, var_from_cursor, IntPtrConstant(2));
   TVARIABLE(IntPtrT, var_to_cursor, IntPtrConstant(1));
 
   Label loop(this, {&var_from_cursor, &var_to_cursor});
@@ -224,16 +219,14 @@ TNode<JSRegExpResult> RegExpBuiltinsAssembler::ConstructNewResultFromMatchInfo(
   {
     TNode<IntPtrT> from_cursor = var_from_cursor.value();
     TNode<IntPtrT> to_cursor = var_to_cursor.value();
-    TNode<Smi> start_cursor =
-        CAST(UnsafeLoadFixedArrayElement(match_info, from_cursor));
+    TNode<Smi> start_cursor = LoadArrayElement(match_info, from_cursor);
 
     Label next_iter(this);
     GotoIf(SmiEqual(start_cursor, SmiConstant(-1)), &next_iter);
 
     TNode<IntPtrT> from_cursor_plus1 =
         IntPtrAdd(from_cursor, IntPtrConstant(1));
-    TNode<Smi> end_cursor =
-        CAST(UnsafeLoadFixedArrayElement(match_info, from_cursor_plus1));
+    TNode<Smi> end_cursor = LoadArrayElement(match_info, from_cursor_plus1);
 
     TNode<String> capture = CAST(CallBuiltin(Builtin::kSubString, context,
                                              string, start_cursor, end_cursor));
@@ -293,8 +286,7 @@ TNode<JSRegExpResult> RegExpBuiltinsAssembler::ConstructNewResultFromMatchInfo(
     if (V8_ENABLE_SWISS_NAME_DICTIONARY_BOOL) {
       properties = AllocateSwissNameDictionary(num_properties);
     } else {
-      properties = AllocateNameDictionary(
-          num_properties, AllocationFlag::kAllowLargeObjectAllocation);
+      properties = AllocateNameDictionary(num_properties);
     }
 
     TNode<JSObject> group_object = AllocateJSObjectFromMap(map, properties);
@@ -330,8 +322,8 @@ TNode<JSRegExpResult> RegExpBuiltinsAssembler::ConstructNewResultFromMatchInfo(
       // - Receiver is extensible
       // - Receiver has no interceptors
       Label add_dictionary_property_slow(this, Label::kDeferred);
-      Add<PropertyDictionary>(CAST(properties), name, capture,
-                              &add_dictionary_property_slow);
+      AddToDictionary<PropertyDictionary>(CAST(properties), name, capture,
+                                          &add_dictionary_property_slow);
 
       var_i = i_plus_2;
       Branch(IntPtrGreaterThanOrEqual(var_i.value(), names_length),
@@ -415,7 +407,7 @@ TNode<HeapObject> RegExpBuiltinsAssembler::RegExpExecInternal(
   GotoIf(TaggedIsNotSmi(last_index), &if_failure);
 
   TNode<IntPtrT> int_string_length = LoadStringLengthAsWord(string);
-  TNode<IntPtrT> int_last_index = SmiUntag(CAST(last_index));
+  TNode<IntPtrT> int_last_index = PositiveSmiUntag(CAST(last_index));
 
   GotoIf(UintPtrGreaterThan(int_last_index, int_string_length), &if_failure);
 
@@ -516,7 +508,9 @@ TNode<HeapObject> RegExpBuiltinsAssembler::RegExpExecInternal(
 #endif
 
   GotoIf(TaggedIsSmi(var_code.value()), &runtime);
-  TNode<Code> code = CAST(var_code.value());
+  TNode<CodeWrapper> code_wrapper = CAST(var_code.value());
+  TNode<Code> code =
+      LoadCodePointerFromObject(code_wrapper, CodeWrapper::kCodeOffset);
 
   Label if_success(this), if_exception(this, Label::kDeferred);
   {
@@ -580,6 +574,10 @@ TNode<HeapObject> RegExpBuiltinsAssembler::RegExpExecInternal(
     MachineType arg8_type = type_tagged;
     TNode<JSRegExp> arg8 = regexp;
 
+    // TODO(saelo): if we refactor RegExp objects to contain a code pointer
+    // instead of referencing the CodeWrapper object, we could directly load
+    // the entrypoint from that via LoadCodeEntrypointViaCodePointerField. This
+    // will save an indirection when the sandbox is enabled.
     TNode<RawPtrT> code_entry = LoadCodeInstructionStart(code);
 
     // AIX uses function descriptors on CFunction calls. code_entry in this case
@@ -627,12 +625,8 @@ TNode<HeapObject> RegExpBuiltinsAssembler::RegExpExecInternal(
       GotoIf(UintPtrGreaterThanOrEqual(value, int_string_length), &if_failure);
     }
 
-    // Check that the last match info has space for the capture registers and
-    // the additional information. Ensure no overflow in add.
-    static_assert(FixedArray::kMaxLength < kMaxInt - FixedArray::kLengthOffset);
-    TNode<Smi> available_slots =
-        SmiSub(LoadFixedArrayBaseLength(match_info),
-               SmiConstant(RegExpMatchInfo::kLastMatchOverhead));
+    // Check that the last match info has space for the capture registers.
+    TNode<Smi> available_slots = LoadArrayCapacity(match_info);
     TNode<Smi> capture_count = CAST(UnsafeLoadFixedArrayElement(
         data, JSRegExp::kIrregexpCaptureCountIndex));
     // Calculate number of register_count = (capture_count + 1) * 2.
@@ -641,21 +635,20 @@ TNode<HeapObject> RegExpBuiltinsAssembler::RegExpExecInternal(
     GotoIf(SmiGreaterThan(register_count, available_slots), &runtime);
 
     // Fill match_info.
-    UnsafeStoreFixedArrayElement(
-        match_info, RegExpMatchInfo::kNumberOfCapturesIndex, register_count);
-    UnsafeStoreFixedArrayElement(match_info, RegExpMatchInfo::kLastSubjectIndex,
-                                 string);
-    UnsafeStoreFixedArrayElement(match_info, RegExpMatchInfo::kLastInputIndex,
-                                 string);
+    StoreObjectField(match_info,
+                     RegExpMatchInfo::kNumberOfCaptureRegistersOffset,
+                     register_count);
+    StoreObjectField(match_info, RegExpMatchInfo::kLastSubjectOffset, string);
+    StoreObjectField(match_info, RegExpMatchInfo::kLastInputOffset, string);
 
     // Fill match and capture offsets in match_info.
     {
+      // The offset within static_offsets_vector.
       TNode<IntPtrT> limit_offset =
           ElementOffsetFromIndex(register_count, INT32_ELEMENTS, 0);
-
-      TNode<IntPtrT> to_offset = ElementOffsetFromIndex(
-          IntPtrConstant(RegExpMatchInfo::kFirstCaptureIndex), PACKED_ELEMENTS,
-          RegExpMatchInfo::kHeaderSize - kHeapObjectTag);
+      // The offset within RegExpMatchInfo.
+      TNode<IntPtrT> to_offset =
+          OffsetOfElementAt<RegExpMatchInfo>(SmiConstant(0));
       TVARIABLE(IntPtrT, var_to_offset, to_offset);
 
       VariableList vars({&var_to_offset}, zone());
@@ -686,11 +679,11 @@ TNode<HeapObject> RegExpBuiltinsAssembler::RegExpExecInternal(
   {
 // A stack overflow was detected in RegExp code.
 #ifdef DEBUG
-    TNode<ExternalReference> pending_exception_address =
+    TNode<ExternalReference> exception_address =
         ExternalConstant(ExternalReference::Create(
-            IsolateAddressId::kPendingExceptionAddress, isolate()));
-    TNode<Object> pending_exception = LoadFullTagged(pending_exception_address);
-    CSA_DCHECK(this, IsTheHole(pending_exception));
+            IsolateAddressId::kExceptionAddress, isolate()));
+    TNode<Object> exception = LoadFullTagged(exception_address);
+    CSA_DCHECK(this, IsTheHole(exception));
 #endif  // DEBUG
     CallRuntime(Runtime::kThrowStackOverflow, context);
     Unreachable();
@@ -889,7 +882,7 @@ TF_BUILTIN(RegExpExecAtom, RegExpBuiltinsAssembler) {
   auto regexp = Parameter<JSRegExp>(Descriptor::kRegExp);
   auto subject_string = Parameter<String>(Descriptor::kString);
   auto last_index = Parameter<Smi>(Descriptor::kLastIndex);
-  auto match_info = Parameter<FixedArray>(Descriptor::kMatchInfo);
+  auto match_info = Parameter<RegExpMatchInfo>(Descriptor::kMatchInfo);
   auto context = Parameter<Context>(Descriptor::kContext);
 
   CSA_DCHECK(this, TaggedIsPositiveSmi(last_index));
@@ -930,22 +923,21 @@ TF_BUILTIN(RegExpExecAtom, RegExpBuiltinsAssembler) {
                                      LoadStringLengthAsWord(subject_string)));
 
     const int kNumRegisters = 2;
-    static_assert(RegExpMatchInfo::kInitialCaptureIndices >= kNumRegisters);
+    static_assert(kNumRegisters <= RegExpMatchInfo::kMinCapacity);
 
     const TNode<Smi> match_to =
         SmiAdd(match_from, LoadStringLengthAsSmi(needle_string));
 
-    UnsafeStoreFixedArrayElement(match_info,
-                                 RegExpMatchInfo::kNumberOfCapturesIndex,
-                                 SmiConstant(kNumRegisters));
-    UnsafeStoreFixedArrayElement(match_info, RegExpMatchInfo::kLastSubjectIndex,
-                                 subject_string);
-    UnsafeStoreFixedArrayElement(match_info, RegExpMatchInfo::kLastInputIndex,
-                                 subject_string);
-    UnsafeStoreFixedArrayElement(
-        match_info, RegExpMatchInfo::kFirstCaptureIndex, match_from);
-    UnsafeStoreFixedArrayElement(
-        match_info, RegExpMatchInfo::kFirstCaptureIndex + 1, match_to);
+    StoreObjectField(match_info,
+                     RegExpMatchInfo::kNumberOfCaptureRegistersOffset,
+                     SmiConstant(kNumRegisters));
+    StoreObjectField(match_info, RegExpMatchInfo::kLastSubjectOffset,
+                     subject_string);
+    StoreObjectField(match_info, RegExpMatchInfo::kLastInputOffset,
+                     subject_string);
+    UnsafeStoreArrayElement(match_info, 0, match_from,
+                            UNSAFE_SKIP_WRITE_BARRIER);
+    UnsafeStoreArrayElement(match_info, 1, match_to, UNSAFE_SKIP_WRITE_BARRIER);
 
     Return(match_info);
   }
@@ -1066,10 +1058,12 @@ TNode<String> RegExpBuiltinsAssembler::FlagsGetter(TNode<Context> context,
   // corresponding char for each set flag.
 
   {
-    const TNode<String> string = AllocateSeqOneByteString(var_length.value());
+    const TNode<SeqOneByteString> string =
+        CAST(AllocateSeqOneByteString(var_length.value()));
 
     TVARIABLE(IntPtrT, var_offset,
-              IntPtrConstant(SeqOneByteString::kHeaderSize - kHeapObjectTag));
+              IntPtrSub(FieldSliceSeqOneByteStringChars(string).offset,
+                        IntPtrConstant(1)));
 
 #define CASE_FOR_FLAG(Lower, Camel, LowerCamel, Char, ...)              \
   do {                                                                  \
@@ -1575,11 +1569,9 @@ TNode<JSArray> RegExpBuiltinsAssembler::RegExpPrototypeSplitBody(
       BIND(&next);
     }
 
-    TNode<FixedArray> match_indices = CAST(match_indices_ho);
-    const TNode<Smi> match_from = CAST(UnsafeLoadFixedArrayElement(
-        match_indices, RegExpMatchInfo::kFirstCaptureIndex));
-    const TNode<Smi> match_to = CAST(UnsafeLoadFixedArrayElement(
-        match_indices, RegExpMatchInfo::kFirstCaptureIndex + 1));
+    TNode<RegExpMatchInfo> match_info = CAST(match_indices_ho);
+    TNode<Smi> match_from = LoadArrayElement(match_info, IntPtrConstant(0));
+    TNode<Smi> match_to = LoadArrayElement(match_info, IntPtrConstant(1));
     CSA_DCHECK(this, SmiNotEqual(match_from, string_length));
 
     // Advance index and continue if the match is empty.
@@ -1610,9 +1602,9 @@ TNode<JSArray> RegExpBuiltinsAssembler::RegExpPrototypeSplitBody(
 
     // Add all captures to the array.
     {
-      const TNode<Smi> num_registers = CAST(LoadFixedArrayElement(
-          match_indices, RegExpMatchInfo::kNumberOfCapturesIndex));
-      const TNode<IntPtrT> int_num_registers = SmiUntag(num_registers);
+      const TNode<Smi> num_registers = CAST(LoadObjectField(
+          match_info, RegExpMatchInfo::kNumberOfCaptureRegistersOffset));
+      const TNode<IntPtrT> int_num_registers = PositiveSmiUntag(num_registers);
 
       TVARIABLE(IntPtrT, var_reg, IntPtrConstant(2));
 
@@ -1624,13 +1616,9 @@ TNode<JSArray> RegExpBuiltinsAssembler::RegExpPrototypeSplitBody(
 
       BIND(&nested_loop);
       {
-        const TNode<IntPtrT> reg = var_reg.value();
-        const TNode<Object> from = LoadFixedArrayElement(
-            match_indices, reg,
-            RegExpMatchInfo::kFirstCaptureIndex * kTaggedSize);
-        const TNode<Smi> to = CAST(LoadFixedArrayElement(
-            match_indices, reg,
-            (RegExpMatchInfo::kFirstCaptureIndex + 1) * kTaggedSize));
+        TNode<IntPtrT> reg = var_reg.value();
+        TNode<Smi> from = LoadArrayElement(match_info, reg);
+        TNode<Smi> to = LoadArrayElement(match_info, reg, 1 * kTaggedSize);
 
         Label select_capture(this), select_undefined(this), store_value(this);
         TVARIABLE(Object, var_value);
