@@ -6,6 +6,7 @@
 
 #include "src/api/api-inl.h"
 #include "src/builtins/builtins-descriptors.h"
+#include "src/builtins/data-view-ops.h"
 #include "src/codegen/assembler-inl.h"
 #include "src/codegen/callable.h"
 #include "src/codegen/macro-assembler-inl.h"
@@ -114,59 +115,13 @@ const char* Builtins::Lookup(Address pc) {
   if (!initialized_) return nullptr;
   for (Builtin builtin_ix = Builtins::kFirst; builtin_ix <= Builtins::kLast;
        ++builtin_ix) {
-    if (code(builtin_ix).contains(isolate_, pc)) {
+    if (code(builtin_ix)->contains(isolate_, pc)) {
       return name(builtin_ix);
     }
   }
   return nullptr;
 }
 
-Handle<Code> Builtins::CallFunction(ConvertReceiverMode mode) {
-  switch (mode) {
-    case ConvertReceiverMode::kNullOrUndefined:
-      return code_handle(Builtin::kCallFunction_ReceiverIsNullOrUndefined);
-    case ConvertReceiverMode::kNotNullOrUndefined:
-      return code_handle(Builtin::kCallFunction_ReceiverIsNotNullOrUndefined);
-    case ConvertReceiverMode::kAny:
-      return code_handle(Builtin::kCallFunction_ReceiverIsAny);
-  }
-  UNREACHABLE();
-}
-
-Handle<Code> Builtins::Call(ConvertReceiverMode mode) {
-  switch (mode) {
-    case ConvertReceiverMode::kNullOrUndefined:
-      return code_handle(Builtin::kCall_ReceiverIsNullOrUndefined);
-    case ConvertReceiverMode::kNotNullOrUndefined:
-      return code_handle(Builtin::kCall_ReceiverIsNotNullOrUndefined);
-    case ConvertReceiverMode::kAny:
-      return code_handle(Builtin::kCall_ReceiverIsAny);
-  }
-  UNREACHABLE();
-}
-
-Handle<Code> Builtins::NonPrimitiveToPrimitive(ToPrimitiveHint hint) {
-  switch (hint) {
-    case ToPrimitiveHint::kDefault:
-      return code_handle(Builtin::kNonPrimitiveToPrimitive_Default);
-    case ToPrimitiveHint::kNumber:
-      return code_handle(Builtin::kNonPrimitiveToPrimitive_Number);
-    case ToPrimitiveHint::kString:
-      return code_handle(Builtin::kNonPrimitiveToPrimitive_String);
-  }
-  UNREACHABLE();
-}
-
-Handle<Code> Builtins::OrdinaryToPrimitive(OrdinaryToPrimitiveHint hint) {
-  switch (hint) {
-    case OrdinaryToPrimitiveHint::kNumber:
-      return code_handle(Builtin::kOrdinaryToPrimitive_Number);
-    case OrdinaryToPrimitiveHint::kString:
-      return code_handle(Builtin::kOrdinaryToPrimitive_String);
-  }
-  UNREACHABLE();
-}
-
 FullObjectSlot Builtins::builtin_slot(Builtin builtin) {
   Address* location = &isolate_->builtin_table()[Builtins::ToInt(builtin)];
   return FullObjectSlot(location);
@@ -179,16 +134,16 @@ FullObjectSlot Builtins::builtin_tier0_slot(Builtin builtin) {
   return FullObjectSlot(location);
 }
 
-void Builtins::set_code(Builtin builtin, Code code) {
-  DCHECK_EQ(builtin, code.builtin_id());
+void Builtins::set_code(Builtin builtin, Tagged<Code> code) {
+  DCHECK_EQ(builtin, code->builtin_id());
   DCHECK(Internals::HasHeapObjectTag(code.ptr()));
   // The given builtin may be uninitialized thus we cannot check its type here.
   isolate_->builtin_table()[Builtins::ToInt(builtin)] = code.ptr();
 }
 
-Code Builtins::code(Builtin builtin) {
+Tagged<Code> Builtins::code(Builtin builtin) {
   Address ptr = isolate_->builtin_table()[Builtins::ToInt(builtin)];
-  return Code::cast(Object(ptr));
+  return Code::cast(Tagged<Object>(ptr));
 }
 
 Handle<Code> Builtins::code_handle(Builtin builtin) {
@@ -246,6 +201,89 @@ const char* Builtins::name(Builtin builtin) {
   return builtin_metadata[index].name;
 }
 
+// static
+const char* Builtins::NameForStackTrace(Isolate* isolate, Builtin builtin) {
+#if V8_ENABLE_WEBASSEMBLY
+  // Most builtins are never shown in stack traces. Those that are exposed
+  // to JavaScript get their name from the object referring to them. Here
+  // we only support a few internal builtins that have special reasons for
+  // being shown on stack traces:
+  // - builtins that are allowlisted in {StubFrame::Summarize}.
+  // - builtins that throw the same error as one of those above, but would
+  //   lose information and e.g. print "indexOf" instead of "String.indexOf".
+  switch (builtin) {
+    case Builtin::kDataViewPrototypeGetBigInt64:
+      return "DataView.prototype.getBigInt64";
+    case Builtin::kDataViewPrototypeGetBigUint64:
+      return "DataView.prototype.getBigUint64";
+    case Builtin::kDataViewPrototypeGetFloat32:
+      return "DataView.prototype.getFloat32";
+    case Builtin::kDataViewPrototypeGetFloat64:
+      return "DataView.prototype.getFloat64";
+    case Builtin::kDataViewPrototypeGetInt8:
+      return "DataView.prototype.getInt8";
+    case Builtin::kDataViewPrototypeGetInt16:
+      return "DataView.prototype.getInt16";
+    case Builtin::kDataViewPrototypeGetInt32:
+      return "DataView.prototype.getInt32";
+    case Builtin::kDataViewPrototypeGetUint8:
+      return "DataView.prototype.getUint8";
+    case Builtin::kDataViewPrototypeGetUint16:
+      return "DataView.prototype.getUint16";
+    case Builtin::kDataViewPrototypeGetUint32:
+      return "DataView.prototype.getUint32";
+    case Builtin::kDataViewPrototypeSetBigInt64:
+      return "DataView.prototype.setBigInt64";
+    case Builtin::kDataViewPrototypeSetBigUint64:
+      return "DataView.prototype.setBigUint64";
+    case Builtin::kDataViewPrototypeSetFloat32:
+      return "DataView.prototype.setFloat32";
+    case Builtin::kDataViewPrototypeSetFloat64:
+      return "DataView.prototype.setFloat64";
+    case Builtin::kDataViewPrototypeSetInt8:
+      return "DataView.prototype.setInt8";
+    case Builtin::kDataViewPrototypeSetInt16:
+      return "DataView.prototype.setInt16";
+    case Builtin::kDataViewPrototypeSetInt32:
+      return "DataView.prototype.setInt32";
+    case Builtin::kDataViewPrototypeSetUint8:
+      return "DataView.prototype.setUint8";
+    case Builtin::kDataViewPrototypeSetUint16:
+      return "DataView.prototype.setUint16";
+    case Builtin::kDataViewPrototypeSetUint32:
+      return "DataView.prototype.setUint32";
+    case Builtin::kDataViewPrototypeGetByteLength:
+      return "get DataView.prototype.byteLength";
+    case Builtin::kThrowDataViewDetachedError:
+    case Builtin::kThrowDataViewOutOfBounds:
+    case Builtin::kThrowDataViewTypeError: {
+      DataViewOp op = static_cast<DataViewOp>(isolate->error_message_param());
+      return ToString(op);
+    }
+    case Builtin::kStringPrototypeToLocaleLowerCase:
+      return "String.toLocaleLowerCase";
+    case Builtin::kStringPrototypeIndexOf:
+    case Builtin::kThrowIndexOfCalledOnNull:
+      return "String.indexOf";
+#if V8_INTL_SUPPORT
+    case Builtin::kStringPrototypeToLowerCaseIntl:
+#endif
+    case Builtin::kThrowToLowerCaseCalledOnNull:
+      return "String.toLowerCase";
+    case Builtin::kWasmIntToString:
+      return "Number.toString";
+    default:
+      // Callers getting this might well crash, which might be desirable
+      // because it's similar to {UNREACHABLE()}, but contrary to that a
+      // careful caller can also check the value and use it as an "is a
+      // name available for this builtin?" check.
+      return nullptr;
+  }
+#else
+  return nullptr;
+#endif  // V8_ENABLE_WEBASSEMBLY
+}
+
 void Builtins::PrintBuiltinCode() {
   DCHECK(v8_flags.print_builtin_code);
 #ifdef ENABLE_DISASSEMBLER
@@ -256,8 +294,8 @@ void Builtins::PrintBuiltinCode() {
                      base::CStrVector(v8_flags.print_builtin_code_filter))) {
       CodeTracer::Scope trace_scope(isolate_->GetCodeTracer());
       OFStream os(trace_scope.file());
-      Code builtin_code = code(builtin);
-      builtin_code.Disassemble(builtin_name, os, isolate_);
+      Tagged<Code> builtin_code = code(builtin);
+      builtin_code->Disassemble(builtin_name, os, isolate_);
       os << "\n";
     }
   }
@@ -270,9 +308,9 @@ void Builtins::PrintBuiltinSize() {
        ++builtin) {
     const char* builtin_name = name(builtin);
     const char* kind = KindNameOf(builtin);
-    Code code = Builtins::code(builtin);
+    Tagged<Code> code = Builtins::code(builtin);
     PrintF(stdout, "%s Builtin, %s, %d\n", kind, builtin_name,
-           code.instruction_size());
+           code->instruction_size());
   }
 }
 
@@ -283,8 +321,8 @@ Address Builtins::CppEntryOf(Builtin builtin) {
 }
 
 // static
-bool Builtins::IsBuiltin(const Code code) {
-  return Builtins::IsBuiltinId(code.builtin_id());
+bool Builtins::IsBuiltin(const Tagged<Code> code) {
+  return Builtins::IsBuiltinId(code->builtin_id());
 }
 
 bool Builtins::IsBuiltinHandle(Handle<HeapObject> maybe_code,
@@ -299,8 +337,8 @@ bool Builtins::IsBuiltinHandle(Handle<HeapObject> maybe_code,
 }
 
 // static
-bool Builtins::IsIsolateIndependentBuiltin(Code code) {
-  Builtin builtin = code.builtin_id();
+bool Builtins::IsIsolateIndependentBuiltin(Tagged<Code> code) {
+  Builtin builtin = code->builtin_id();
   return Builtins::IsBuiltinId(builtin) &&
          Builtins::IsIsolateIndependent(builtin);
 }
@@ -312,8 +350,8 @@ void Builtins::InitializeIsolateDataTables(Isolate* isolate) {
 
   // The entry table.
   for (Builtin i = Builtins::kFirst; i <= Builtins::kLast; ++i) {
-    DCHECK(Builtins::IsBuiltinId(isolate->builtins()->code(i).builtin_id()));
-    DCHECK(!isolate->builtins()->code(i).has_instruction_stream());
+    DCHECK(Builtins::IsBuiltinId(isolate->builtins()->code(i)->builtin_id()));
+    DCHECK(!isolate->builtins()->code(i)->has_instruction_stream());
     isolate_data->builtin_entry_table()[ToInt(i)] =
         embedded_data.InstructionStartOf(i);
   }
@@ -362,27 +400,27 @@ Handle<Code> Builtins::CreateInterpreterEntryTrampolineForProfiling(
   DCHECK_NOT_NULL(isolate->embedded_blob_code());
   DCHECK_NE(0, isolate->embedded_blob_code_size());
 
-  Code code = isolate->builtins()->code(
+  Tagged<Code> code = isolate->builtins()->code(
       Builtin::kInterpreterEntryTrampolineForProfiling);
 
   CodeDesc desc;
-  desc.buffer = reinterpret_cast<byte*>(code.instruction_start());
+  desc.buffer = reinterpret_cast<uint8_t*>(code->instruction_start());
 
-  int instruction_size = code.instruction_size();
+  int instruction_size = code->instruction_size();
   desc.buffer_size = instruction_size;
   desc.instr_size = instruction_size;
 
   // Ensure the code doesn't require creation of metadata, otherwise respective
   // fields of CodeDesc should be initialized.
-  DCHECK_EQ(code.safepoint_table_size(), 0);
-  DCHECK_EQ(code.handler_table_size(), 0);
-  DCHECK_EQ(code.constant_pool_size(), 0);
+  DCHECK_EQ(code->safepoint_table_size(), 0);
+  DCHECK_EQ(code->handler_table_size(), 0);
+  DCHECK_EQ(code->constant_pool_size(), 0);
   // TODO(v8:11036): The following DCHECK currently fails if the mksnapshot is
   // run with enabled code comments, i.e. --interpreted_frames_native_stack is
   // incompatible with --code-comments at mksnapshot-time. If ever needed,
   // implement support.
-  DCHECK_EQ(code.code_comments_size(), 0);
-  DCHECK_EQ(code.unwinding_info_size(), 0);
+  DCHECK_EQ(code->code_comments_size(), 0);
+  DCHECK_EQ(code->unwinding_info_size(), 0);
 
   desc.safepoint_table_offset = instruction_size;
   desc.handler_table_offset = instruction_size;
@@ -429,7 +467,7 @@ bool Builtins::AllowDynamicFunction(Isolate* isolate, Handle<JSFunction> target,
                                     Handle<JSObject> target_global_proxy) {
   if (v8_flags.allow_unsafe_function_constructor) return true;
   HandleScopeImplementer* impl = isolate->handle_scope_implementer();
-  Handle<Context> responsible_context = impl->LastEnteredOrMicrotaskContext();
+  Handle<NativeContext> responsible_context = impl->LastEnteredContext();
   // TODO(verwaest): Remove this.
   if (responsible_context.is_null()) {
     return true;
