@@ -38,7 +38,6 @@
 #include "src/base/cpu.h"
 #include "src/codegen/assembler-inl.h"
 #include "src/codegen/safepoint-table.h"
-#include "src/codegen/string-constants.h"
 #include "src/deoptimizer/deoptimizer.h"
 #include "src/diagnostics/disasm.h"
 #include "src/diagnostics/disassembler.h"
@@ -59,6 +58,18 @@ static unsigned CpuFeaturesImpliedByCompiler() {
 #if (defined CAN_USE_RVV_INSTRUCTIONS)
   answer |= 1u << RISCV_SIMD;
 #endif  // def CAN_USE_RVV_INSTRUCTIONS
+
+#if (defined CAN_USE_ZBA_INSTRUCTIONS)
+  answer |= 1u << ZBA;
+#endif  // def CAN_USE_ZBA_INSTRUCTIONS
+
+#if (defined CAN_USE_ZBB_INSTRUCTIONS)
+  answer |= 1u << ZBB;
+#endif  // def CAN_USE_ZBA_INSTRUCTIONS
+
+#if (defined CAN_USE_ZBS_INSTRUCTIONS)
+  answer |= 1u << ZBS;
+#endif  // def CAN_USE_ZBA_INSTRUCTIONS
   return answer;
 }
 
@@ -72,6 +83,10 @@ void CpuFeatures::ProbeImpl(bool cross_compile) {
   base::CPU cpu;
   if (cpu.has_fpu()) supported_ |= 1u << FPU;
   if (cpu.has_rvv()) supported_ |= 1u << RISCV_SIMD;
+  if (cpu.riscv_mmu() == base::CPU::RV_MMU_MODE::kRiscvSV57) {
+    FATAL("SV57 is not supported");
+    UNIMPLEMENTED();
+  }
   // Set a static value on whether SIMD is supported.
   // This variable is only used for certain archs to query SupportWasmSimd128()
   // at runtime in builtins using an extern ref. Other callers should use
@@ -179,7 +194,7 @@ MemOperand::MemOperand(Register rm, int32_t unit, int32_t multiplier,
   offset_ = unit * multiplier + offset_addend;
 }
 
-void Assembler::AllocateAndInstallRequestedHeapNumbers(Isolate* isolate) {
+void Assembler::AllocateAndInstallRequestedHeapNumbers(LocalIsolate* isolate) {
   DCHECK_IMPLIES(isolate == nullptr, heap_number_requests_.empty());
   for (auto& request : heap_number_requests_) {
     Handle<HeapObject> object =
@@ -220,7 +235,10 @@ Assembler::Assembler(const AssemblerOptions& options,
 void Assembler::AbortedCodeGeneration() { constpool_.Clear(); }
 Assembler::~Assembler() { CHECK(constpool_.IsEmpty()); }
 
-void Assembler::GetCode(Isolate* isolate, CodeDesc* desc,
+void Assembler::GetCode(Isolate* isolate, CodeDesc* desc) {
+  GetCode(isolate->main_thread_local_isolate(), desc);
+}
+void Assembler::GetCode(LocalIsolate* isolate, CodeDesc* desc,
                         SafepointTableBuilder* safepoint_table_builder,
                         int handler_table_offset) {
   // As a crutch to avoid having to add manual Align calls wherever we use a
@@ -309,7 +327,7 @@ int Assembler::target_at(int pos, bool is_internal) {
   DEBUG_PRINTF("target_at: %p (%d)\n\t",
                reinterpret_cast<Instr*>(buffer_start_ + pos), pos);
   Instr instr = instruction->InstructionBits();
-  disassembleInstr(instruction->InstructionBits());
+  disassembleInstr(buffer_start_ + pos);
 
   switch (instruction->InstructionOpcodeType()) {
     case BRANCH: {
@@ -483,13 +501,13 @@ bool Assembler::MustUseReg(RelocInfo::Mode rmode) {
   return !RelocInfo::IsNoInfo(rmode);
 }
 
-void Assembler::disassembleInstr(Instr instr) {
+void Assembler::disassembleInstr(uint8_t* pc) {
   if (!v8_flags.riscv_debug) return;
   disasm::NameConverter converter;
   disasm::Disassembler disasm(converter);
   base::EmbeddedVector<char, 128> disasm_buffer;
 
-  disasm.InstructionDecode(disasm_buffer, reinterpret_cast<byte*>(&instr));
+  disasm.InstructionDecode(disasm_buffer, pc);
   DEBUG_PRINTF("%s\n", disasm_buffer.begin());
 }
 
@@ -569,7 +587,11 @@ void Assembler::target_at_put(int pos, int target_pos, bool is_internal,
           pos, target_pos + (InstructionStream::kHeaderSize - kHeapObjectTag));
     } break;
   }
-  disassembleInstr(instr);
+
+  disassembleInstr(buffer_start_ + pos);
+  if (instruction->InstructionOpcodeType() == AUIPC) {
+    disassembleInstr(buffer_start_ + pos + 4);
+  }
 }
 
 void Assembler::print(const Label* L) {
@@ -1061,21 +1083,26 @@ void Assembler::GeneralLi(Register rd, int64_t imm) {
 }
 
 void Assembler::li_ptr(Register rd, int64_t imm) {
-  // Initialize rd with an address
-  // Pointers are 48 bits
-  // 6 fixed instructions are generated
-  DCHECK_EQ((imm & 0xfff0000000000000ll), 0);
-  int64_t a6 = imm & 0x3f;                      // bits 0:5. 6 bits
-  int64_t b11 = (imm >> 6) & 0x7ff;             // bits 6:11. 11 bits
-  int64_t high_31 = (imm >> 17) & 0x7fffffff;   // 31 bits
-  int64_t high_20 = ((high_31 + 0x800) >> 12);  // 19 bits
-  int64_t low_12 = high_31 & 0xfff;             // 12 bits
-  lui(rd, (int32_t)high_20);
-  addi(rd, rd, low_12);  // 31 bits in rd.
-  slli(rd, rd, 11);      // Space for next 11 bis
-  ori(rd, rd, b11);      // 11 bits are put in. 42 bit in rd
-  slli(rd, rd, 6);       // Space for next 6 bits
-  ori(rd, rd, a6);       // 6 bits are put in. 48 bis in rd
+  base::CPU cpu;
+  if (cpu.riscv_mmu() != base::CPU::RV_MMU_MODE::kRiscvSV57) {
+    // Initialize rd with an address
+    // Pointers are 48 bits
+    // 6 fixed instructions are generated
+    DCHECK_EQ((imm & 0xfff0000000000000ll), 0);
+    int64_t a6 = imm & 0x3f;                      // bits 0:5. 6 bits
+    int64_t b11 = (imm >> 6) & 0x7ff;             // bits 6:11. 11 bits
+    int64_t high_31 = (imm >> 17) & 0x7fffffff;   // 31 bits
+    int64_t high_20 = ((high_31 + 0x800) >> 12);  // 19 bits
+    int64_t low_12 = high_31 & 0xfff;             // 12 bits
+    lui(rd, (int32_t)high_20);
+    addi(rd, rd, low_12);  // 31 bits in rd.
+    slli(rd, rd, 11);      // Space for next 11 bis
+    ori(rd, rd, b11);      // 11 bits are put in. 42 bit in rd
+    slli(rd, rd, 6);       // Space for next 6 bits
+    ori(rd, rd, a6);       // 6 bits are put in. 48 bis in rd
+  } else {
+    FATAL("SV57 is not supported");
+  }
 }
 
 void Assembler::li_constant(Register rd, int64_t imm) {
@@ -1278,7 +1305,7 @@ void Assembler::GrowBuffer() {
   // Set up new buffer.
   std::unique_ptr<AssemblerBuffer> new_buffer = buffer_->Grow(new_size);
   DCHECK_EQ(new_size, new_buffer->size());
-  byte* new_start = new_buffer->start();
+  uint8_t* new_start = new_buffer->start();
 
   // Copy the data.
   intptr_t pc_delta = new_start - buffer_start_;
@@ -1297,9 +1324,9 @@ void Assembler::GrowBuffer() {
                                reloc_info_writer.last_pc() + pc_delta);
 
   // Relocate runtime entries.
-  base::Vector<byte> instructions{buffer_start_,
-                                  static_cast<size_t>(pc_offset())};
-  base::Vector<const byte> reloc_info{reloc_info_writer.pos(), reloc_size};
+  base::Vector<uint8_t> instructions{buffer_start_,
+                                     static_cast<size_t>(pc_offset())};
+  base::Vector<const uint8_t> reloc_info{reloc_info_writer.pos(), reloc_size};
   for (RelocIterator it(instructions, reloc_info, 0); !it.done(); it.next()) {
     RelocInfo::Mode rmode = it.rinfo()->rmode();
     if (rmode == RelocInfo::INTERNAL_REFERENCE) {
@@ -1312,18 +1339,23 @@ void Assembler::GrowBuffer() {
 
 void Assembler::db(uint8_t data) {
   if (!is_buffer_growth_blocked()) CheckBuffer();
-  DEBUG_PRINTF("%p(%x): constant 0x%x\n", pc_, pc_offset(), data);
+  DEBUG_PRINTF("%p(%d): constant 0x%x\n", pc_, pc_offset(), data);
   EmitHelper(data);
 }
 
 void Assembler::dd(uint32_t data) {
   if (!is_buffer_growth_blocked()) CheckBuffer();
-  DEBUG_PRINTF("%p(%x): constant 0x%x\n", pc_, pc_offset(), data);
+  DEBUG_PRINTF("%p(%d): constant 0x%x\n", pc_, pc_offset(), data);
   EmitHelper(data);
 }
 
 void Assembler::dq(uint64_t data) {
   if (!is_buffer_growth_blocked()) CheckBuffer();
+#if V8_TARGET_ARCH_RISCV64
+  DEBUG_PRINTF("%p(%d): constant 0x%lx\n", pc_, pc_offset(), data);
+#elif V8_TARGET_ARCH_RISCV32
+  DEBUG_PRINTF("%p(%d): constant 0x%llx\n", pc_, pc_offset(), data);
+#endif
   EmitHelper(data);
 }
 
@@ -1604,24 +1636,6 @@ void Assembler::set_target_value_at(Address pc, uint32_t target,
 }
 #endif
 
-UseScratchRegisterScope::UseScratchRegisterScope(Assembler* assembler)
-    : available_(assembler->GetScratchRegisterList()),
-      old_available_(*available_) {}
-
-UseScratchRegisterScope::~UseScratchRegisterScope() {
-  *available_ = old_available_;
-}
-
-Register UseScratchRegisterScope::Acquire() {
-  DCHECK_NOT_NULL(available_);
-  DCHECK(!available_->is_empty());
-  int index =
-      static_cast<int>(base::bits::CountTrailingZeros32(available_->bits()));
-  *available_ &= RegList::FromBits(~(1U << index));
-
-  return Register::from_code(index);
-}
-
 bool UseScratchRegisterScope::hasAvailable() const {
   return !available_->is_empty();
 }
@@ -1682,9 +1696,9 @@ void Assembler::emit(Instr x) {
   if (!is_buffer_growth_blocked()) {
     CheckBuffer();
   }
-  DEBUG_PRINTF("%p(%x): ", pc_, pc_offset());
-  disassembleInstr(x);
+  DEBUG_PRINTF("%p(%d): ", pc_, pc_offset());
   EmitHelper(x);
+  disassembleInstr(pc_ - sizeof(x));
   CheckTrampolinePoolQuick();
 }
 
@@ -1692,13 +1706,14 @@ void Assembler::emit(ShortInstr x) {
   if (!is_buffer_growth_blocked()) {
     CheckBuffer();
   }
-  DEBUG_PRINTF("%p(%x): ", pc_, pc_offset());
-  disassembleInstr(x);
+  DEBUG_PRINTF("%p(%d): ", pc_, pc_offset());
   EmitHelper(x);
+  disassembleInstr(pc_ - sizeof(x));
   CheckTrampolinePoolQuick();
 }
 
 void Assembler::emit(uint64_t data) {
+  DEBUG_PRINTF("%p(%d): ", pc_, pc_offset());
   if (!is_buffer_growth_blocked()) CheckBuffer();
   EmitHelper(data);
 }
@@ -2121,6 +2136,5 @@ int Assembler::GeneralLiCount(int64_t imm, bool is_get_temp_reg) {
   return count;
 }
 #endif
-
 }  // namespace internal
 }  // namespace v8
