@@ -34,6 +34,8 @@ namespace internal {
   T(ArgumentsDisallowedInInitializerAndStaticBlock,                            \
     "'arguments' is not allowed in class field initializer or static "         \
     "initialization block")                                                    \
+  T(ArgumentIsNonObject, "% argument must be an object")                       \
+  T(ArgumentIsNonString, "% argument must be a string")                        \
   T(ArrayBufferDetachKeyDoesntMatch,                                           \
     "Provided key doesn't match [[ArrayBufferDetachKey]]")                     \
   T(ArrayBufferTooShort,                                                       \
@@ -51,7 +53,7 @@ namespace internal {
   T(AtomicsOperationNotAllowed, "% cannot be called in this context")          \
   T(BadRoundingType, "RoundingType is not fractionDigits")                     \
   T(BadSortComparisonFunction,                                                 \
-    "The comparison function must be either a function or undefined")          \
+    "The comparison function must be either a function or undefined: %")       \
   T(BigIntFromNumber,                                                          \
     "The number % cannot be converted to a BigInt because it is not an "       \
     "integer")                                                                 \
@@ -100,7 +102,7 @@ namespace internal {
   T(DeclarationMissingInitializer, "Missing initializer in % declaration")     \
   T(DefineDisallowed, "Cannot define property %, object is not extensible")    \
   T(DefineDisallowedFixedLayout,                                               \
-    "Cannot define property %, object is fixed layout")                        \
+    "Cannot define property %, object has fixed layout")                       \
   T(DetachedOperation, "Cannot perform % on a detached ArrayBuffer")           \
   T(DoNotUse, "Do not use %; %")                                               \
   T(DuplicateTemplateProperty, "Object template has duplicate property '%'")   \
@@ -129,6 +131,7 @@ namespace internal {
   T(InvalidRegExpExecResult,                                                   \
     "RegExp exec method returned something other than an Object or null")      \
   T(InvalidUnit, "Invalid unit argument for %() '%'")                          \
+  T(IsNotNumber, "Type of '%' must be 'number', found '%'")                    \
   T(IterableYieldedNonString, "Iterable yielded % which is not a string")      \
   T(IteratorReduceNoInitial,                                                   \
     "Reduce of a done iterator with no initial value")                         \
@@ -138,7 +141,11 @@ namespace internal {
   T(FirstArgumentIteratorSymbolNonCallable,                                    \
     "% requires that the property of the first argument, "                     \
     "items[Symbol.iterator], when exists, be a function")                      \
+  T(FirstArgumentAsyncIteratorSymbolNonCallable,                               \
+    "% requires that the property of the first argument, "                     \
+    "items[Symbol.asyncIterator], when exists, be a function")                 \
   T(IteratorValueNotAnObject, "Iterator value % is not an entry object")       \
+  T(KeysMethodInvalid, "Result of the keys method is not an object")           \
   T(LanguageID, "Language ID should be string or object.")                     \
   T(LocaleNotEmpty,                                                            \
     "First argument to Intl.Locale constructor can't be empty or missing")     \
@@ -191,6 +198,7 @@ namespace internal {
   T(NotInt32OrBigInt64TypedArray,                                              \
     "% is not an int32 or BigInt64 typed array.")                              \
   T(NotSharedTypedArray, "% is not a shared typed array.")                     \
+  T(ObjectFixedLayout, "Cannot add property %, object has fixed layout")       \
   T(ObjectGetterExpectingFunction,                                             \
     "Object.prototype.__defineGetter__: Expecting function")                   \
   T(ObjectGetterCallable, "Getter must be a function: %")                      \
@@ -318,11 +326,14 @@ namespace internal {
     "Incorrect RelativeDateTimeFormatter provided")                            \
   T(ResolverNotAFunction, "Promise resolver % is not a function")              \
   T(ReturnMethodNotCallable, "The iterator's 'return' method is not callable") \
+  T(SizeIsNaN, "The .size property is NaN")                                    \
   T(SharedArrayBufferTooShort,                                                 \
     "Derived SharedArrayBuffer constructor created a buffer which was too "    \
     "small")                                                                   \
   T(SharedArrayBufferSpeciesThis,                                              \
     "SharedArrayBuffer subclass returned this from species constructor")       \
+  T(SharedStructTypeRegistryMismatch,                                          \
+    "SharedStructType registered as '%' does not match")                       \
   T(StaticPrototype,                                                           \
     "Classes may not have a static property named 'prototype'")                \
   T(StrictDeleteProperty, "Cannot delete property '%' of %")                   \
@@ -400,8 +411,6 @@ namespace internal {
     "maximumFractionDigits not equal to minimumFractionDigits")                \
   T(NormalizationForm, "The normalization form should be one of %.")           \
   T(OutOfMemory, "%: Out of memory")                                           \
-  T(ParameterOfFunctionOutOfRange,                                             \
-    "Paramenter % of function %() is % and out of range")                      \
   T(ZeroDigitNumericSeparator,                                                 \
     "Numeric separator can not be used after leading 0.")                      \
   T(NumberFormatRange, "% argument must be between 0 and 100")                 \
@@ -414,8 +423,7 @@ namespace internal {
   T(ToPrecisionFormatRange,                                                    \
     "toPrecision() argument must be between 1 and 100")                        \
   T(ToRadixFormatRange, "toString() radix argument must be between 2 and 36")  \
-  T(SharedArraySizeOutOfRange,                                                 \
-    "SharedArray length out of range (maximum of 2**14-2 allowed)")            \
+  T(SharedArraySizeOutOfRange, "SharedArray length out of range")              \
   T(StructFieldCountOutOfRange,                                                \
     "Struct field count out of range (maximum of 999 allowed)")                \
   T(TypedArraySetOffsetOutOfBounds, "offset is out of bounds")                 \
@@ -503,34 +511,43 @@ namespace internal {
   T(InvalidUnusedPrivateStaticMethodAccessedByDebugger,                        \
     "Unused static private method '%' cannot be accessed at debug time")       \
   T(JsonParseUnexpectedEOS, "Unexpected end of JSON input")                    \
-  T(JsonParseUnexpectedTokenNumber, "Unexpected number in JSON at position %") \
-  T(JsonParseUnexpectedTokenString, "Unexpected string in JSON at position %") \
-  T(JsonParseUnterminatedString, "Unterminated string in JSON at position %")  \
+  T(JsonParseUnexpectedTokenNumber,                                            \
+    "Unexpected number in JSON at position % (line % column %)")               \
+  T(JsonParseUnexpectedTokenString,                                            \
+    "Unexpected string in JSON at position % (line % column %)")               \
+  T(JsonParseUnterminatedString,                                               \
+    "Unterminated string in JSON at position % (line % column %)")             \
   T(JsonParseExpectedPropNameOrRBrace,                                         \
-    "Expected property name or '}' in JSON at position %")                     \
+    "Expected property name or '}' in JSON at position % (line % column %)")   \
   T(JsonParseExpectedCommaOrRBrack,                                            \
-    "Expected ',' or ']' after array element in JSON at position %")           \
+    "Expected ',' or ']' after array element in JSON at position % (line % "   \
+    "column %)")                                                               \
   T(JsonParseExpectedCommaOrRBrace,                                            \
     "Expected ',' or '}' after property value in JSON at position "            \
-    "%")                                                                       \
+    "% (line % column %)")                                                     \
   T(JsonParseExpectedDoubleQuotedPropertyName,                                 \
-    "Expected double-quoted property name in JSON at position %")              \
+    "Expected double-quoted property name in JSON at position % (line % "      \
+    "column %)")                                                               \
   T(JsonParseExponentPartMissingNumber,                                        \
-    "Exponent part is missing a number in JSON at position %")                 \
+    "Exponent part is missing a number in JSON at position % (line % column "  \
+    "%)")                                                                      \
   T(JsonParseExpectedColonAfterPropertyName,                                   \
-    "Expected ':' after property name in JSON at position %")                  \
+    "Expected ':' after property name in JSON at position % (line % column "   \
+    "%)")                                                                      \
   T(JsonParseUnterminatedFractionalNumber,                                     \
-    "Unterminated fractional number in JSON at position %")                    \
+    "Unterminated fractional number in JSON at position % (line % column %)")  \
   T(JsonParseUnexpectedNonWhiteSpaceCharacter,                                 \
     "Unexpected non-whitespace character after JSON at position "              \
-    "%")                                                                       \
+    "% (line % column %)")                                                     \
   T(JsonParseBadEscapedCharacter,                                              \
-    "Bad escaped character in JSON at position %")                             \
+    "Bad escaped character in JSON at position % (line % column %)")           \
   T(JsonParseBadControlCharacter,                                              \
-    "Bad control character in string literal in JSON at position %")           \
-  T(JsonParseBadUnicodeEscape, "Bad Unicode escape in JSON at position %")     \
+    "Bad control character in string literal in JSON at position % (line % "   \
+    "column %)")                                                               \
+  T(JsonParseBadUnicodeEscape,                                                 \
+    "Bad Unicode escape in JSON at position % (line % column %)")              \
   T(JsonParseNoNumberAfterMinusSign,                                           \
-    "No number after minus sign in JSON at position %")                        \
+    "No number after minus sign in JSON at position % (line % column %)")      \
   T(JsonParseShortString, "\"%\" is not valid JSON")                           \
   T(JsonParseUnexpectedTokenShortString,                                       \
     "Unexpected token '%', \"%\" is not valid JSON")                           \
