@@ -98,7 +98,7 @@ void JSGenericLowering::ReplaceWithBuiltinCall(
   auto call_descriptor = Linkage::GetStubCallDescriptor(
       zone(), descriptor, descriptor.GetStackParameterCount(), flags,
       properties);
-  Node* stub_code = jsgraph()->HeapConstant(callable.code());
+  Node* stub_code = jsgraph()->HeapConstantNoHole(callable.code());
   node->InsertInput(zone(), 0, stub_code);
   NodeProperties::ChangeOp(node, common()->Call(call_descriptor));
 }
@@ -133,7 +133,7 @@ void JSGenericLowering::ReplaceUnaryOpWithBuiltinCall(
     auto call_descriptor = Linkage::GetStubCallDescriptor(
         zone(), descriptor, descriptor.GetStackParameterCount(), flags,
         node->op()->properties());
-    Node* stub_code = jsgraph()->HeapConstant(callable.code());
+    Node* stub_code = jsgraph()->HeapConstantNoHole(callable.code());
     static_assert(JSUnaryOpNode::ValueIndex() == 0);
     static_assert(JSUnaryOpNode::FeedbackVectorIndex() == 1);
     DCHECK_EQ(node->op()->ValueInputCount(), 2);
@@ -234,14 +234,14 @@ void JSGenericLowering::LowerJSStrictEqual(Node* node) {
 
 namespace {
 
-// The megamorphic load builtin can be used as a performance optimization in
-// some cases - unlike the full builtin, the megamorphic builtin does fewer
+// The megamorphic load/store builtin can be used as a performance optimization
+// in some cases - unlike the full builtin, the megamorphic builtin does fewer
 // checks and does not collect feedback.
-bool ShouldUseMegamorphicLoadBuiltin(FeedbackSource const& source,
-                                     OptionalNameRef name,
-                                     JSHeapBroker* broker) {
+bool ShouldUseMegamorphicAccessBuiltin(FeedbackSource const& source,
+                                       OptionalNameRef name, AccessMode mode,
+                                       JSHeapBroker* broker) {
   ProcessedFeedback const& feedback =
-      broker->GetFeedbackForPropertyAccess(source, AccessMode::kLoad, name);
+      broker->GetFeedbackForPropertyAccess(source, mode, name);
 
   if (feedback.kind() == ProcessedFeedback::kElementAccess) {
     return feedback.AsElementAccess().transition_groups().empty();
@@ -269,14 +269,6 @@ void JSGenericLowering::LowerJSHasProperty(Node* node) {
   }
 }
 
-bool HasStringType(Node* key) {
-  if (key->opcode() == IrOpcode::kLoadElement) {
-    ElementAccess const& access = ElementAccessOf(key->op());
-    return access.type.Is(Type::String());
-  }
-  return false;
-}
-
 void JSGenericLowering::LowerJSLoadProperty(Node* node) {
   JSLoadPropertyNode n(node);
   const PropertyAccess& p = n.Parameters();
@@ -288,19 +280,17 @@ void JSGenericLowering::LowerJSLoadProperty(Node* node) {
     n->InsertInput(zone(), 2,
                    jsgraph()->TaggedIndexConstant(p.feedback().index()));
     ReplaceWithBuiltinCall(
-        node, ShouldUseMegamorphicLoadBuiltin(p.feedback(), {}, broker())
-                  ? (HasStringType(n->InputAt(1))
-                         ? Builtin::kKeyedLoadICTrampoline_MegamorphicStringKey
-                         : Builtin::kKeyedLoadICTrampoline_Megamorphic)
+        node, ShouldUseMegamorphicAccessBuiltin(p.feedback(), {},
+                                                AccessMode::kLoad, broker())
+                  ? Builtin::kKeyedLoadICTrampoline_Megamorphic
                   : Builtin::kKeyedLoadICTrampoline);
   } else {
     n->InsertInput(zone(), 2,
                    jsgraph()->TaggedIndexConstant(p.feedback().index()));
     ReplaceWithBuiltinCall(
-        node, ShouldUseMegamorphicLoadBuiltin(p.feedback(), {}, broker())
-                  ? (HasStringType(n->InputAt(1))
-                         ? Builtin::kKeyedLoadIC_MegamorphicStringKey
-                         : Builtin::kKeyedLoadIC_Megamorphic)
+        node, ShouldUseMegamorphicAccessBuiltin(p.feedback(), {},
+                                                AccessMode::kLoad, broker())
+                  ? Builtin::kKeyedLoadIC_Megamorphic
                   : Builtin::kKeyedLoadIC);
   }
 }
@@ -313,23 +303,25 @@ void JSGenericLowering::LowerJSLoadNamed(Node* node) {
   static_assert(n.FeedbackVectorIndex() == 1);
   if (!p.feedback().IsValid()) {
     n->RemoveInput(n.FeedbackVectorIndex());
-    node->InsertInput(zone(), 1, jsgraph()->Constant(p.name(), broker()));
+    node->InsertInput(zone(), 1, jsgraph()->ConstantNoHole(p.name(), broker()));
     ReplaceWithBuiltinCall(node, Builtin::kGetProperty);
   } else if (outer_state->opcode() != IrOpcode::kFrameState) {
     n->RemoveInput(n.FeedbackVectorIndex());
-    node->InsertInput(zone(), 1, jsgraph()->Constant(p.name(), broker()));
+    node->InsertInput(zone(), 1, jsgraph()->ConstantNoHole(p.name(), broker()));
     node->InsertInput(zone(), 2,
                       jsgraph()->TaggedIndexConstant(p.feedback().index()));
     ReplaceWithBuiltinCall(
-        node, ShouldUseMegamorphicLoadBuiltin(p.feedback(), p.name(), broker())
+        node, ShouldUseMegamorphicAccessBuiltin(p.feedback(), p.name(),
+                                                AccessMode::kLoad, broker())
                   ? Builtin::kLoadICTrampoline_Megamorphic
                   : Builtin::kLoadICTrampoline);
   } else {
-    node->InsertInput(zone(), 1, jsgraph()->Constant(p.name(), broker()));
+    node->InsertInput(zone(), 1, jsgraph()->ConstantNoHole(p.name(), broker()));
     node->InsertInput(zone(), 2,
                       jsgraph()->TaggedIndexConstant(p.feedback().index()));
     ReplaceWithBuiltinCall(
-        node, ShouldUseMegamorphicLoadBuiltin(p.feedback(), p.name(), broker())
+        node, ShouldUseMegamorphicAccessBuiltin(p.feedback(), p.name(),
+                                                AccessMode::kLoad, broker())
                   ? Builtin::kLoadIC_Megamorphic
                   : Builtin::kLoadIC);
   }
@@ -356,7 +348,7 @@ void JSGenericLowering::LowerJSLoadNamedFromSuper(Node* node) {
   // be double-checked that the FeedbackVector parameter will be the
   // UndefinedConstant.
   DCHECK(p.feedback().IsValid());
-  node->InsertInput(zone(), 2, jsgraph()->Constant(p.name(), broker()));
+  node->InsertInput(zone(), 2, jsgraph()->ConstantNoHole(p.name(), broker()));
   node->InsertInput(zone(), 3,
                     jsgraph()->TaggedIndexConstant(p.feedback().index()));
   ReplaceWithBuiltinCall(node, Builtin::kLoadSuperIC);
@@ -371,13 +363,13 @@ void JSGenericLowering::LowerJSLoadGlobal(Node* node) {
   static_assert(n.FeedbackVectorIndex() == 0);
   if (outer_state->opcode() != IrOpcode::kFrameState) {
     n->RemoveInput(n.FeedbackVectorIndex());
-    node->InsertInput(zone(), 0, jsgraph()->Constant(p.name(), broker()));
+    node->InsertInput(zone(), 0, jsgraph()->ConstantNoHole(p.name(), broker()));
     node->InsertInput(zone(), 1,
                       jsgraph()->TaggedIndexConstant(p.feedback().index()));
     Callable callable = CodeFactory::LoadGlobalIC(isolate(), p.typeof_mode());
     ReplaceWithBuiltinCall(node, callable, flags);
   } else {
-    node->InsertInput(zone(), 0, jsgraph()->Constant(p.name(), broker()));
+    node->InsertInput(zone(), 0, jsgraph()->ConstantNoHole(p.name(), broker()));
     node->InsertInput(zone(), 1,
                       jsgraph()->TaggedIndexConstant(p.feedback().index()));
     Callable callable =
@@ -426,11 +418,19 @@ void JSGenericLowering::LowerJSSetKeyedProperty(Node* node) {
     // the paths are controlled by feedback.
     // TODO(v8:12548): refactor SetKeyedIC as a subclass of KeyedStoreIC, which
     // can be called here.
-    ReplaceWithBuiltinCall(node, Builtin::kKeyedStoreICTrampoline);
+    ReplaceWithBuiltinCall(
+        node, ShouldUseMegamorphicAccessBuiltin(p.feedback(), {},
+                                                AccessMode::kStore, broker())
+                  ? Builtin::kKeyedStoreICTrampoline_Megamorphic
+                  : Builtin::kKeyedStoreICTrampoline);
   } else {
     node->InsertInput(zone(), 3,
                       jsgraph()->TaggedIndexConstant(p.feedback().index()));
-    ReplaceWithBuiltinCall(node, Builtin::kKeyedStoreIC);
+    ReplaceWithBuiltinCall(
+        node, ShouldUseMegamorphicAccessBuiltin(p.feedback(), {},
+                                                AccessMode::kStore, broker())
+                  ? Builtin::kKeyedStoreIC_Megamorphic
+                  : Builtin::kKeyedStoreIC);
   }
 }
 
@@ -460,11 +460,11 @@ void JSGenericLowering::LowerJSSetNamedProperty(Node* node) {
   static_assert(n.FeedbackVectorIndex() == 2);
   if (!p.feedback().IsValid()) {
     n->RemoveInput(n.FeedbackVectorIndex());
-    node->InsertInput(zone(), 1, jsgraph()->Constant(p.name(), broker()));
+    node->InsertInput(zone(), 1, jsgraph()->ConstantNoHole(p.name(), broker()));
     ReplaceWithRuntimeCall(node, Runtime::kSetNamedProperty);
   } else if (outer_state->opcode() != IrOpcode::kFrameState) {
     n->RemoveInput(n.FeedbackVectorIndex());
-    node->InsertInput(zone(), 1, jsgraph()->Constant(p.name(), broker()));
+    node->InsertInput(zone(), 1, jsgraph()->ConstantNoHole(p.name(), broker()));
     node->InsertInput(zone(), 3,
                       jsgraph()->TaggedIndexConstant(p.feedback().index()));
     // StoreIC is currently a base class for multiple property store operations
@@ -474,7 +474,7 @@ void JSGenericLowering::LowerJSSetNamedProperty(Node* node) {
     // be called here.
     ReplaceWithBuiltinCall(node, Builtin::kStoreICTrampoline);
   } else {
-    node->InsertInput(zone(), 1, jsgraph()->Constant(p.name(), broker()));
+    node->InsertInput(zone(), 1, jsgraph()->ConstantNoHole(p.name(), broker()));
     node->InsertInput(zone(), 3,
                       jsgraph()->TaggedIndexConstant(p.feedback().index()));
     ReplaceWithBuiltinCall(node, Builtin::kStoreIC);
@@ -490,13 +490,13 @@ void JSGenericLowering::LowerJSDefineNamedOwnProperty(Node* node) {
   static_assert(n.FeedbackVectorIndex() == 2);
   if (outer_state->opcode() != IrOpcode::kFrameState) {
     n->RemoveInput(n.FeedbackVectorIndex());
-    node->InsertInput(zone(), 1, jsgraph()->Constant(p.name(), broker()));
+    node->InsertInput(zone(), 1, jsgraph()->ConstantNoHole(p.name(), broker()));
     node->InsertInput(zone(), 3,
                       jsgraph()->TaggedIndexConstant(p.feedback().index()));
     Callable callable = CodeFactory::DefineNamedOwnIC(isolate());
     ReplaceWithBuiltinCall(node, callable, flags);
   } else {
-    node->InsertInput(zone(), 1, jsgraph()->Constant(p.name(), broker()));
+    node->InsertInput(zone(), 1, jsgraph()->ConstantNoHole(p.name(), broker()));
     node->InsertInput(zone(), 3,
                       jsgraph()->TaggedIndexConstant(p.feedback().index()));
     Callable callable = CodeFactory::DefineNamedOwnICInOptimizedCode(isolate());
@@ -512,12 +512,12 @@ void JSGenericLowering::LowerJSStoreGlobal(Node* node) {
   static_assert(n.FeedbackVectorIndex() == 1);
   if (outer_state->opcode() != IrOpcode::kFrameState) {
     n->RemoveInput(n.FeedbackVectorIndex());
-    node->InsertInput(zone(), 0, jsgraph()->Constant(p.name(), broker()));
+    node->InsertInput(zone(), 0, jsgraph()->ConstantNoHole(p.name(), broker()));
     node->InsertInput(zone(), 2,
                       jsgraph()->TaggedIndexConstant(p.feedback().index()));
     ReplaceWithBuiltinCall(node, Builtin::kStoreGlobalICTrampoline);
   } else {
-    node->InsertInput(zone(), 0, jsgraph()->Constant(p.name(), broker()));
+    node->InsertInput(zone(), 0, jsgraph()->ConstantNoHole(p.name(), broker()));
     node->InsertInput(zone(), 2,
                       jsgraph()->TaggedIndexConstant(p.feedback().index()));
     ReplaceWithBuiltinCall(node, Builtin::kStoreGlobalIC);
@@ -629,7 +629,7 @@ void JSGenericLowering::LowerJSCreateArray(Node* node) {
   Node* stub_arity = jsgraph()->Int32Constant(JSParameterCount(arity));
   OptionalAllocationSiteRef const site = p.site();
   Node* type_info = site.has_value()
-                        ? jsgraph()->Constant(site.value(), broker())
+                        ? jsgraph()->ConstantNoHole(site.value(), broker())
                         : jsgraph()->UndefinedConstant();
   Node* receiver = jsgraph()->UndefinedConstant();
   node->InsertInput(zone(), 0, stub_code);
@@ -676,7 +676,8 @@ void JSGenericLowering::LowerJSCreateClosure(Node* node) {
   CreateClosureParameters const& p = n.Parameters();
   SharedFunctionInfoRef shared_info = p.shared_info();
   static_assert(n.FeedbackCellIndex() == 0);
-  node->InsertInput(zone(), 0, jsgraph()->Constant(shared_info, broker()));
+  node->InsertInput(zone(), 0,
+                    jsgraph()->ConstantNoHole(shared_info, broker()));
   node->RemoveInput(4);  // control
 
   // Use the FastNewClosure builtin only for functions allocated in new space.
@@ -698,11 +699,13 @@ void JSGenericLowering::LowerJSCreateFunctionContext(Node* node) {
   if (slot_count <= ConstructorBuiltins::MaximumFunctionContextSlots()) {
     Callable callable =
         CodeFactory::FastNewFunctionContext(isolate(), scope_type);
-    node->InsertInput(zone(), 0, jsgraph()->Constant(scope_info, broker()));
+    node->InsertInput(zone(), 0,
+                      jsgraph()->ConstantNoHole(scope_info, broker()));
     node->InsertInput(zone(), 1, jsgraph()->Int32Constant(slot_count));
     ReplaceWithBuiltinCall(node, callable, flags);
   } else {
-    node->InsertInput(zone(), 0, jsgraph()->Constant(scope_info, broker()));
+    node->InsertInput(zone(), 0,
+                      jsgraph()->ConstantNoHole(scope_info, broker()));
     ReplaceWithRuntimeCall(node, Runtime::kNewFunctionContext);
   }
 }
@@ -738,7 +741,8 @@ void JSGenericLowering::LowerJSCreateLiteralArray(Node* node) {
   static_assert(n.FeedbackVectorIndex() == 0);
   node->InsertInput(zone(), 1,
                     jsgraph()->TaggedIndexConstant(p.feedback().index()));
-  node->InsertInput(zone(), 2, jsgraph()->Constant(p.constant(), broker()));
+  node->InsertInput(zone(), 2,
+                    jsgraph()->ConstantNoHole(p.constant(), broker()));
   node->InsertInput(zone(), 3, jsgraph()->SmiConstant(p.flags()));
 
   // Use the CreateShallowArrayLiteral builtin only for shallow boilerplates
@@ -761,8 +765,9 @@ void JSGenericLowering::LowerJSGetTemplateObject(Node* node) {
   node->RemoveInput(NodeProperties::FirstControlIndex(node));
 
   static_assert(JSGetTemplateObjectNode::FeedbackVectorIndex() == 0);
-  node->InsertInput(zone(), 0, jsgraph()->Constant(shared, broker()));
-  node->InsertInput(zone(), 1, jsgraph()->Constant(description, broker()));
+  node->InsertInput(zone(), 0, jsgraph()->ConstantNoHole(shared, broker()));
+  node->InsertInput(zone(), 1,
+                    jsgraph()->ConstantNoHole(description, broker()));
   node->InsertInput(zone(), 2,
                     jsgraph()->UintPtrConstant(p.feedback().index()));
 
@@ -789,7 +794,8 @@ void JSGenericLowering::LowerJSCreateLiteralObject(Node* node) {
   static_assert(n.FeedbackVectorIndex() == 0);
   node->InsertInput(zone(), 1,
                     jsgraph()->TaggedIndexConstant(p.feedback().index()));
-  node->InsertInput(zone(), 2, jsgraph()->Constant(p.constant(), broker()));
+  node->InsertInput(zone(), 2,
+                    jsgraph()->ConstantNoHole(p.constant(), broker()));
   node->InsertInput(zone(), 3, jsgraph()->SmiConstant(p.flags()));
 
   // Use the CreateShallowObjectLiteratal builtin only for shallow boilerplates
@@ -823,7 +829,8 @@ void JSGenericLowering::LowerJSCreateLiteralRegExp(Node* node) {
   static_assert(n.FeedbackVectorIndex() == 0);
   node->InsertInput(zone(), 1,
                     jsgraph()->TaggedIndexConstant(p.feedback().index()));
-  node->InsertInput(zone(), 2, jsgraph()->Constant(p.constant(), broker()));
+  node->InsertInput(zone(), 2,
+                    jsgraph()->ConstantNoHole(p.constant(), broker()));
   node->InsertInput(zone(), 3, jsgraph()->SmiConstant(p.flags()));
   ReplaceWithBuiltinCall(node, Builtin::kCreateRegExpLiteral);
 }
@@ -831,19 +838,19 @@ void JSGenericLowering::LowerJSCreateLiteralRegExp(Node* node) {
 
 void JSGenericLowering::LowerJSCreateCatchContext(Node* node) {
   ScopeInfoRef scope_info = ScopeInfoOf(node->op());
-  node->InsertInput(zone(), 1, jsgraph()->Constant(scope_info, broker()));
+  node->InsertInput(zone(), 1, jsgraph()->ConstantNoHole(scope_info, broker()));
   ReplaceWithRuntimeCall(node, Runtime::kPushCatchContext);
 }
 
 void JSGenericLowering::LowerJSCreateWithContext(Node* node) {
   ScopeInfoRef scope_info = ScopeInfoOf(node->op());
-  node->InsertInput(zone(), 1, jsgraph()->Constant(scope_info, broker()));
+  node->InsertInput(zone(), 1, jsgraph()->ConstantNoHole(scope_info, broker()));
   ReplaceWithRuntimeCall(node, Runtime::kPushWithContext);
 }
 
 void JSGenericLowering::LowerJSCreateBlockContext(Node* node) {
   ScopeInfoRef scope_info = ScopeInfoOf(node->op());
-  node->InsertInput(zone(), 0, jsgraph()->Constant(scope_info, broker()));
+  node->InsertInput(zone(), 0, jsgraph()->ConstantNoHole(scope_info, broker()));
   ReplaceWithRuntimeCall(node, Runtime::kPushBlockContext);
 }
 
@@ -860,7 +867,7 @@ void JSGenericLowering::LowerJSConstructForwardVarargs(Node* node) {
   DCHECK_EQ(callable.descriptor().GetStackParameterCount(), 0);
   auto call_descriptor = Linkage::GetStubCallDescriptor(
       zone(), callable.descriptor(), arg_count + 1, flags);
-  Node* stub_code = jsgraph()->HeapConstant(callable.code());
+  Node* stub_code = jsgraph()->HeapConstantNoHole(callable.code());
   Node* stub_arity = jsgraph()->Int32Constant(JSParameterCount(arg_count));
   Node* start_index = jsgraph()->Uint32Constant(p.start_index());
   Node* receiver = jsgraph()->UndefinedConstant();
@@ -871,6 +878,33 @@ void JSGenericLowering::LowerJSConstructForwardVarargs(Node* node) {
   NodeProperties::ChangeOp(node, common()->Call(call_descriptor));
 }
 
+void JSGenericLowering::LowerJSConstructForwardAllArgs(Node* node) {
+  // Inlined JSConstructForwardAllArgs are reduced earlier in the pipeline in
+  // JSCallReducer.
+  DCHECK(FrameState{NodeProperties::GetFrameStateInput(node)}
+             .outer_frame_state()
+             ->opcode() != IrOpcode::kFrameState);
+
+  JSConstructForwardAllArgsNode n(node);
+
+  // Call a builtin for forwarding the arguments of non-inlined (i.e. outermost)
+  // frames.
+  Callable callable =
+      Builtins::CallableFor(isolate(), Builtin::kConstructForwardAllArgs);
+  DCHECK_EQ(callable.descriptor().GetStackParameterCount(), 0);
+  auto call_descriptor = Linkage::GetStubCallDescriptor(
+      zone(), callable.descriptor(), 0, CallDescriptor::kNeedsFrameState);
+
+  Node* stub_code = jsgraph()->HeapConstantNoHole(callable.code());
+
+  // Shuffling inputs.
+  // Before: {target, new target, feedback vector}
+  node->RemoveInput(n.FeedbackVectorIndex());
+  node->InsertInput(zone(), 0, stub_code);
+  // After: {code, target, new target}
+  NodeProperties::ChangeOp(node, common()->Call(call_descriptor));
+}
+
 void JSGenericLowering::LowerJSConstruct(Node* node) {
   JSConstructNode n(node);
   ConstructParameters const& p = n.Parameters();
@@ -883,7 +917,7 @@ void JSGenericLowering::LowerJSConstruct(Node* node) {
   Callable callable = Builtins::CallableFor(isolate(), Builtin::kConstruct);
   auto call_descriptor = Linkage::GetStubCallDescriptor(
       zone(), callable.descriptor(), stack_argument_count, flags);
-  Node* stub_code = jsgraph()->HeapConstant(callable.code());
+  Node* stub_code = jsgraph()->HeapConstantNoHole(callable.code());
   Node* stub_arity = jsgraph()->Int32Constant(JSParameterCount(arg_count));
   Node* receiver = jsgraph()->UndefinedConstant();
   node->RemoveInput(n.FeedbackVectorIndex());
@@ -915,7 +949,7 @@ void JSGenericLowering::LowerJSConstructWithArrayLike(Node* node) {
   DCHECK_EQ(callable.descriptor().GetStackParameterCount(), 0);
   auto call_descriptor = Linkage::GetStubCallDescriptor(
       zone(), callable.descriptor(), stack_argument_count, flags);
-  Node* stub_code = jsgraph()->HeapConstant(callable.code());
+  Node* stub_code = jsgraph()->HeapConstantNoHole(callable.code());
   Node* receiver = jsgraph()->UndefinedConstant();
   node->RemoveInput(n.FeedbackVectorIndex());
   node->InsertInput(zone(), 0, stub_code);
@@ -944,7 +978,7 @@ void JSGenericLowering::LowerJSConstructWithSpread(Node* node) {
   DCHECK_EQ(callable.descriptor().GetStackParameterCount(), 0);
   auto call_descriptor = Linkage::GetStubCallDescriptor(
       zone(), callable.descriptor(), stack_argument_count, flags);
-  Node* stub_code = jsgraph()->HeapConstant(callable.code());
+  Node* stub_code = jsgraph()->HeapConstantNoHole(callable.code());
 
   // We pass the spread in a register, not on the stack.
   Node* stub_arity =
@@ -971,7 +1005,7 @@ void JSGenericLowering::LowerJSCallForwardVarargs(Node* node) {
   Callable callable = CodeFactory::CallForwardVarargs(isolate());
   auto call_descriptor = Linkage::GetStubCallDescriptor(
       zone(), callable.descriptor(), arg_count + 1, flags);
-  Node* stub_code = jsgraph()->HeapConstant(callable.code());
+  Node* stub_code = jsgraph()->HeapConstantNoHole(callable.code());
   Node* stub_arity = jsgraph()->Int32Constant(JSParameterCount(arg_count));
   Node* start_index = jsgraph()->Uint32Constant(p.start_index());
   node->InsertInput(zone(), 0, stub_code);
@@ -992,7 +1026,7 @@ void JSGenericLowering::LowerJSCall(Node* node) {
   CallDescriptor::Flags flags = FrameStateFlagForCall(node);
   auto call_descriptor = Linkage::GetStubCallDescriptor(
       zone(), callable.descriptor(), arg_count + 1, flags);
-  Node* stub_code = jsgraph()->HeapConstant(callable.code());
+  Node* stub_code = jsgraph()->HeapConstantNoHole(callable.code());
   Node* stub_arity = jsgraph()->Int32Constant(JSParameterCount(arg_count));
   node->InsertInput(zone(), 0, stub_code);
   node->InsertInput(zone(), 2, stub_arity);
@@ -1013,7 +1047,7 @@ void JSGenericLowering::LowerJSCallWithArrayLike(Node* node) {
   Callable callable = CodeFactory::CallWithArrayLike(isolate());
   auto call_descriptor = Linkage::GetStubCallDescriptor(
       zone(), callable.descriptor(), stack_argument_count, flags);
-  Node* stub_code = jsgraph()->HeapConstant(callable.code());
+  Node* stub_code = jsgraph()->HeapConstantNoHole(callable.code());
   Node* receiver = n.receiver();
   Node* arguments_list = n.Argument(0);
 
@@ -1048,7 +1082,7 @@ void JSGenericLowering::LowerJSCallWithSpread(Node* node) {
   DCHECK_EQ(callable.descriptor().GetStackParameterCount(), 0);
   auto call_descriptor = Linkage::GetStubCallDescriptor(
       zone(), callable.descriptor(), stack_argument_count, flags);
-  Node* stub_code = jsgraph()->HeapConstant(callable.code());
+  Node* stub_code = jsgraph()->HeapConstantNoHole(callable.code());
 
   // We pass the spread in a register, not on the stack.
   Node* stub_arity =
@@ -1201,15 +1235,12 @@ void JSGenericLowering::LowerJSStackCheck(Node* node) {
   // function takes an offset argument which is subtracted from the stack
   // pointer prior to the stack check (i.e. the check is `sp - offset >=
   // limit`).
+  Runtime::FunctionId builtin = GetBuiltinForStackCheckKind(stack_check_kind);
   if (stack_check_kind == StackCheckKind::kJSFunctionEntry) {
     node->InsertInput(zone(), 0,
                       graph()->NewNode(machine()->LoadStackCheckOffset()));
-    ReplaceWithRuntimeCall(node, Runtime::kStackGuardWithGap);
-  } else if (stack_check_kind == StackCheckKind::kJSIterationBody) {
-    ReplaceWithRuntimeCall(node, Runtime::kHandleNoHeapWritesInterrupts);
-  } else {
-    ReplaceWithRuntimeCall(node, Runtime::kStackGuard);
   }
+  ReplaceWithRuntimeCall(node, builtin);
 }
 
 void JSGenericLowering::LowerJSDebugger(Node* node) {
