@@ -9,6 +9,7 @@
 #endif
 
 #include "src/codegen/optimized-compilation-info.h"
+#include "src/compiler/js-heap-broker-inl.h"
 #include "src/handles/handles-inl.h"
 #include "src/heap/heap-inl.h"
 #include "src/objects/allocation-site-inl.h"
@@ -120,16 +121,18 @@ void JSHeapBroker::CollectArrayAndObjectPrototypes() {
   CHECK_EQ(mode(), kSerializing);
   CHECK(array_and_object_prototypes_.empty());
 
-  Object maybe_context = isolate()->heap()->native_contexts_list();
-  while (!maybe_context.IsUndefined(isolate())) {
-    Context context = Context::cast(maybe_context);
-    Object array_prot = context.get(Context::INITIAL_ARRAY_PROTOTYPE_INDEX);
-    Object object_prot = context.get(Context::INITIAL_OBJECT_PROTOTYPE_INDEX);
+  Tagged<Object> maybe_context = isolate()->heap()->native_contexts_list();
+  while (!IsUndefined(maybe_context, isolate())) {
+    Tagged<Context> context = Context::cast(maybe_context);
+    Tagged<Object> array_prot =
+        context->get(Context::INITIAL_ARRAY_PROTOTYPE_INDEX);
+    Tagged<Object> object_prot =
+        context->get(Context::INITIAL_OBJECT_PROTOTYPE_INDEX);
     array_and_object_prototypes_.emplace(
         CanonicalPersistentHandle(JSObject::cast(array_prot)));
     array_and_object_prototypes_.emplace(
         CanonicalPersistentHandle(JSObject::cast(object_prot)));
-    maybe_context = context.next_context_link();
+    maybe_context = context->next_context_link();
   }
 
   CHECK(!array_and_object_prototypes_.empty());
@@ -165,7 +168,7 @@ bool JSHeapBroker::IsArrayOrObjectPrototype(Handle<JSObject> object) const {
          array_and_object_prototypes_.end();
 }
 
-ObjectData* JSHeapBroker::TryGetOrCreateData(Object object,
+ObjectData* JSHeapBroker::TryGetOrCreateData(Tagged<Object> object,
                                              GetOrCreateDataFlags flags) {
   return TryGetOrCreateData(CanonicalPersistentHandle(object), flags);
 }
@@ -177,7 +180,7 @@ ObjectData* JSHeapBroker::GetOrCreateData(Handle<Object> object,
   return return_value;
 }
 
-ObjectData* JSHeapBroker::GetOrCreateData(Object object,
+ObjectData* JSHeapBroker::GetOrCreateData(Tagged<Object> object,
                                           GetOrCreateDataFlags flags) {
   return GetOrCreateData(CanonicalPersistentHandle(object), flags);
 }
@@ -194,12 +197,12 @@ bool JSHeapBroker::ObjectMayBeUninitialized(Handle<Object> object) const {
   return ObjectMayBeUninitialized(*object);
 }
 
-bool JSHeapBroker::ObjectMayBeUninitialized(Object object) const {
-  if (!object.IsHeapObject()) return false;
+bool JSHeapBroker::ObjectMayBeUninitialized(Tagged<Object> object) const {
+  if (!IsHeapObject(object)) return false;
   return ObjectMayBeUninitialized(HeapObject::cast(object));
 }
 
-bool JSHeapBroker::ObjectMayBeUninitialized(HeapObject object) const {
+bool JSHeapBroker::ObjectMayBeUninitialized(Tagged<HeapObject> object) const {
   return !IsMainThread() && isolate()->heap()->IsPendingAllocation(object);
 }
 
@@ -487,7 +490,7 @@ ProcessedFeedback const& JSHeapBroker::ReadFeedbackForPropertyAccess(
       if (map.is_deprecated()) {
         // TODO(ishell): support fast map updating if we enable it.
         CHECK(!v8_flags.fast_map_update);
-        base::Optional<Map> maybe_map = MapUpdater::TryUpdateNoLock(
+        base::Optional<Tagged<Map>> maybe_map = MapUpdater::TryUpdateNoLock(
             isolate(), *map.object(), ConcurrencyMode::kConcurrent);
         if (maybe_map.has_value()) {
           map = MakeRefAssumeMemoryFence(this, maybe_map.value());
@@ -556,30 +559,29 @@ ProcessedFeedback const& JSHeapBroker::ReadFeedbackForGlobalAccess(
   }
 
   Handle<Object> feedback_value =
-      CanonicalPersistentHandle(nexus.GetFeedback()->GetHeapObjectOrSmi());
+      CanonicalPersistentHandle(nexus.GetFeedback().GetHeapObjectOrSmi());
 
-  if (feedback_value->IsSmi()) {
+  if (IsSmi(*feedback_value)) {
     // The wanted name belongs to a script-scope variable and the feedback
     // tells us where to find its value.
-    int const number = feedback_value->Number();
+    int const number = Object::Number(*feedback_value);
     int const script_context_index =
         FeedbackNexus::ContextIndexBits::decode(number);
     int const context_slot_index = FeedbackNexus::SlotIndexBits::decode(number);
     ContextRef context = MakeRefAssumeMemoryFence(
-        this, target_native_context()
-                  .script_context_table(broker)
-                  .object()
-                  ->get_context(script_context_index, kAcquireLoad));
+        this,
+        target_native_context().script_context_table(broker).object()->get(
+            script_context_index, kAcquireLoad));
 
     OptionalObjectRef contents = context.get(broker, context_slot_index);
-    if (contents.has_value()) CHECK(!contents->IsTheHole(broker));
+    if (contents.has_value()) CHECK(!contents->IsTheHole());
 
     return *zone()->New<GlobalAccessFeedback>(
         context, context_slot_index,
         FeedbackNexus::ImmutabilityBit::decode(number), nexus.kind());
   }
 
-  CHECK(feedback_value->IsPropertyCell());
+  CHECK(IsPropertyCell(*feedback_value));
   // The wanted name belongs (or did belong) to a property on the global
   // object and the feedback is the cell holding its value.
   return *zone()->New<GlobalAccessFeedback>(
@@ -636,8 +638,8 @@ ProcessedFeedback const& JSHeapBroker::ReadFeedbackForArrayOrObjectLiteral(
   FeedbackNexus nexus(source.vector, source.slot, feedback_nexus_config());
   if (nexus.IsUninitialized()) return NewInsufficientFeedback(nexus.kind());
 
-  HeapObject object;
-  if (!nexus.GetFeedback()->GetHeapObject(&object)) {
+  Tagged<HeapObject> object;
+  if (!nexus.GetFeedback().GetHeapObject(&object)) {
     return NewInsufficientFeedback(nexus.kind());
   }
 
@@ -651,8 +653,8 @@ ProcessedFeedback const& JSHeapBroker::ReadFeedbackForRegExpLiteral(
   FeedbackNexus nexus(source.vector, source.slot, feedback_nexus_config());
   if (nexus.IsUninitialized()) return NewInsufficientFeedback(nexus.kind());
 
-  HeapObject object;
-  if (!nexus.GetFeedback()->GetHeapObject(&object)) {
+  Tagged<HeapObject> object;
+  if (!nexus.GetFeedback().GetHeapObject(&object)) {
     return NewInsufficientFeedback(nexus.kind());
   }
 
@@ -666,8 +668,8 @@ ProcessedFeedback const& JSHeapBroker::ReadFeedbackForTemplateObject(
   FeedbackNexus nexus(source.vector, source.slot, feedback_nexus_config());
   if (nexus.IsUninitialized()) return NewInsufficientFeedback(nexus.kind());
 
-  HeapObject object;
-  if (!nexus.GetFeedback()->GetHeapObject(&object)) {
+  Tagged<HeapObject> object;
+  if (!nexus.GetFeedback().GetHeapObject(&object)) {
     return NewInsufficientFeedback(nexus.kind());
   }
 
@@ -683,8 +685,8 @@ ProcessedFeedback const& JSHeapBroker::ReadFeedbackForCall(
   OptionalHeapObjectRef target_ref;
   {
     MaybeObject maybe_target = nexus.GetFeedback();
-    HeapObject target_object;
-    if (maybe_target->GetHeapObject(&target_object)) {
+    Tagged<HeapObject> target_object;
+    if (maybe_target.GetHeapObject(&target_object)) {
       target_ref = TryMakeRef(this, target_object);
     }
   }
@@ -821,7 +823,7 @@ ElementAccessFeedback const& JSHeapBroker::ProcessFeedbackMapsForElementAccess(
 
   // Separate the actual receiver maps and the possible transition sources.
   for (MapRef map : maps) {
-    Map transition_target;
+    Tagged<Map> transition_target;
 
     // Don't generate elements kind transitions from stable maps.
     if (!map.is_stable()) {
@@ -875,7 +877,7 @@ void ElementAccessFeedback::AddGroup(TransitionGroup&& group) {
 }
 
 OptionalNameRef JSHeapBroker::GetNameFeedback(FeedbackNexus const& nexus) {
-  Name raw_name = nexus.GetName();
+  Tagged<Name> raw_name = nexus.GetName();
   if (raw_name.is_null()) return base::nullopt;
   return MakeRefAssumeMemoryFence(this, raw_name);
 }
