@@ -15,6 +15,7 @@
 #include "src/compiler/opcodes.h"
 #include "src/compiler/operator-properties.h"
 #include "src/objects/feedback-cell.h"
+#include "src/objects/oddball.h"
 #include "src/runtime/runtime.h"
 
 namespace v8 {
@@ -842,11 +843,13 @@ class JSWasmCallParameters {
   explicit JSWasmCallParameters(const wasm::WasmModule* module,
                                 const wasm::FunctionSig* signature,
                                 int function_index,
+                                SharedFunctionInfoRef shared_fct_info,
                                 wasm::NativeModule* native_module,
                                 FeedbackSource const& feedback)
       : module_(module),
         signature_(signature),
         function_index_(function_index),
+        shared_fct_info_(shared_fct_info),
         native_module_(native_module),
         feedback_(feedback) {
     DCHECK_NOT_NULL(module);
@@ -856,6 +859,7 @@ class JSWasmCallParameters {
   const wasm::WasmModule* module() const { return module_; }
   const wasm::FunctionSig* signature() const { return signature_; }
   int function_index() const { return function_index_; }
+  SharedFunctionInfoRef shared_fct_info() const { return shared_fct_info_; }
   wasm::NativeModule* native_module() const { return native_module_; }
   FeedbackSource const& feedback() const { return feedback_; }
   int input_count() const;
@@ -865,6 +869,7 @@ class JSWasmCallParameters {
   const wasm::WasmModule* const module_;
   const wasm::FunctionSig* const signature_;
   int function_index_;
+  SharedFunctionInfoRef shared_fct_info_;
   wasm::NativeModule* native_module_;
   const FeedbackSource feedback_;
 };
@@ -1001,7 +1006,8 @@ class V8_EXPORT_PRIVATE JSOperatorBuilder final
 #if V8_ENABLE_WEBASSEMBLY
   const Operator* CallWasm(const wasm::WasmModule* wasm_module,
                            const wasm::FunctionSig* wasm_signature,
-                           int function_index,
+                           int wasm_function_index,
+                           SharedFunctionInfoRef shared_fct_info,
                            wasm::NativeModule* native_module,
                            FeedbackSource const& feedback);
 #endif  // V8_ENABLE_WEBASSEMBLY
@@ -1015,6 +1021,9 @@ class V8_EXPORT_PRIVATE JSOperatorBuilder final
   const Operator* ConstructWithSpread(
       uint32_t arity, CallFrequency const& frequency = CallFrequency(),
       FeedbackSource const& feedback = FeedbackSource());
+  const Operator* ConstructForwardAllArgs(
+      CallFrequency const& frequency = CallFrequency(),
+      FeedbackSource const& feedback = FeedbackSource());
 
   const Operator* LoadProperty(FeedbackSource const& feedback);
   const Operator* LoadNamed(NameRef name, FeedbackSource const& feedback);
@@ -1402,9 +1411,8 @@ class JSCallOrConstructNode : public JSNodeWrapperBase {
     DCHECK_GT(ArgumentCount(), 0);
     return Argument(ArgumentCount() - 1);
   }
-  TNode<Object> ArgumentOr(int i, Node* default_value) const {
-    return i < ArgumentCount() ? Argument(i)
-                               : TNode<Object>::UncheckedCast(default_value);
+  TNode<Object> ArgumentOr(int i, TNode<Object> default_value) const {
+    return i < ArgumentCount() ? Argument(i) : default_value;
   }
   TNode<Object> ArgumentOrUndefined(int i, JSGraph* jsgraph) const {
     return ArgumentOr(i, js_node_wrapper_utils::UndefinedConstant(jsgraph));
@@ -1430,7 +1438,8 @@ class JSCallOrConstructNode : public JSNodeWrapperBase {
            node->opcode() == IrOpcode::kJSCallWithSpread ||
            node->opcode() == IrOpcode::kJSConstruct ||
            node->opcode() == IrOpcode::kJSConstructWithArrayLike ||
-           node->opcode() == IrOpcode::kJSConstructWithSpread
+           node->opcode() == IrOpcode::kJSConstructWithSpread ||
+           node->opcode() == IrOpcode::kJSConstructForwardAllArgs
 #if V8_ENABLE_WEBASSEMBLY
            || node->opcode() == IrOpcode::kJSWasmCall
 #endif     // V8_ENABLE_WEBASSEMBLY
@@ -1547,6 +1556,8 @@ using JSConstructWithSpreadNode =
     JSConstructNodeBase<IrOpcode::kJSConstructWithSpread>;
 using JSConstructWithArrayLikeNode =
     JSConstructNodeBase<IrOpcode::kJSConstructWithArrayLike>;
+using JSConstructForwardAllArgsNode =
+    JSConstructNodeBase<IrOpcode::kJSConstructForwardAllArgs>;
 
 class JSLoadNamedNode final : public JSNodeWrapperBase {
  public:
