@@ -44,16 +44,6 @@ class OptionalOperator final {
   const Operator* const op_;
 };
 
-enum class MemoryAccessKind {
-  kNormal,
-  kUnaligned,
-  kProtected,
-};
-
-size_t hash_value(MemoryAccessKind);
-
-V8_EXPORT_PRIVATE std::ostream& operator<<(std::ostream&, MemoryAccessKind);
-
 // A Load needs a MachineType.
 using LoadRepresentation = MachineType;
 
@@ -114,20 +104,36 @@ V8_EXPORT_PRIVATE AtomicOpParameters AtomicOpParametersOf(Operator const*)
     V8_WARN_UNUSED_RESULT;
 
 enum class LoadTransformation {
+  // 128-bit LoadSplats must be first.
   kS128Load8Splat,
   kS128Load16Splat,
   kS128Load32Splat,
   kS128Load64Splat,
+  kFirst128Splat = kS128Load8Splat,
+  kLast128Splat = kS128Load64Splat,
+  // 128-bit LoadExtend.
   kS128Load8x8S,
   kS128Load8x8U,
   kS128Load16x4S,
   kS128Load16x4U,
   kS128Load32x2S,
   kS128Load32x2U,
+  kFirst128Extend = kS128Load8x8S,
+  kLast128Extend = kS128Load32x2U,
   kS128Load32Zero,
   kS128Load64Zero,
+  // 256-bit transformations must be last.
+  kS256Load8Splat,
+  kS256Load16Splat,
   kS256Load32Splat,
   kS256Load64Splat,
+  kS256Load8x16S,
+  kS256Load8x16U,
+  kS256Load16x8S,
+  kS256Load16x8U,
+  kS256Load32x4S,
+  kS256Load32x4U,
+  kFirst256Transform = kS256Load8Splat
 };
 
 size_t hash_value(LoadTransformation);
@@ -180,6 +186,14 @@ class StoreRepresentation final {
   WriteBarrierKind write_barrier_kind_;
 };
 
+struct StorePairRepresentation final
+    : public std::pair<StoreRepresentation, StoreRepresentation> {
+  StorePairRepresentation(StoreRepresentation first, StoreRepresentation second)
+      : std::pair<StoreRepresentation, StoreRepresentation>(first, second) {}
+  friend std::ostream& operator<<(std::ostream& out,
+                                  const StorePairRepresentation rep);
+};
+
 V8_EXPORT_PRIVATE bool operator==(StoreRepresentation, StoreRepresentation);
 bool operator!=(StoreRepresentation, StoreRepresentation);
 
@@ -190,6 +204,9 @@ V8_EXPORT_PRIVATE std::ostream& operator<<(std::ostream&, StoreRepresentation);
 V8_EXPORT_PRIVATE StoreRepresentation const& StoreRepresentationOf(
     Operator const*) V8_WARN_UNUSED_RESULT;
 
+V8_EXPORT_PRIVATE StorePairRepresentation const& StorePairRepresentationOf(
+    Operator const*) V8_WARN_UNUSED_RESULT;
+
 // A Word(32|64)AtomicStore needs both a StoreRepresentation and a memory order.
 class AtomicStoreParameters final {
  public:
@@ -274,33 +291,60 @@ V8_EXPORT_PRIVATE StackSlotRepresentation const& StackSlotRepresentationOf(
 
 MachineType AtomicOpType(Operator const* op) V8_WARN_UNUSED_RESULT;
 
-class S128ImmediateParameter {
+template <const int simd_size = kSimd128Size,
+          typename = std::enable_if_t<simd_size == kSimd128Size ||
+                                      simd_size == kSimd256Size>>
+class SimdImmediateParameter {
  public:
-  explicit S128ImmediateParameter(const uint8_t immediate[16]) {
-    std::copy(immediate, immediate + 16, immediate_.begin());
+  explicit SimdImmediateParameter(const uint8_t immediate[simd_size]) {
+    std::copy(immediate, immediate + simd_size, immediate_.begin());
   }
-  S128ImmediateParameter() = default;
-  const std::array<uint8_t, 16>& immediate() const { return immediate_; }
+  SimdImmediateParameter() = default;
+  const std::array<uint8_t, simd_size>& immediate() const { return immediate_; }
   const uint8_t* data() const { return immediate_.data(); }
   uint8_t operator[](int x) const { return immediate_[x]; }
 
  private:
-  std::array<uint8_t, 16> immediate_;
+  std::array<uint8_t, simd_size> immediate_;
 };
 
-V8_EXPORT_PRIVATE bool operator==(S128ImmediateParameter const& lhs,
-                                  S128ImmediateParameter const& rhs);
-bool operator!=(S128ImmediateParameter const& lhs,
-                S128ImmediateParameter const& rhs);
+using S128ImmediateParameter = SimdImmediateParameter<kSimd128Size>;
+using S256ImmediateParameter = SimdImmediateParameter<kSimd256Size>;
 
-size_t hash_value(S128ImmediateParameter const& p);
+template <const int simd_size>
+V8_EXPORT_PRIVATE inline bool operator==(
+    SimdImmediateParameter<simd_size> const& lhs,
+    SimdImmediateParameter<simd_size> const& rhs) {
+  return (lhs.immediate() == rhs.immediate());
+}
 
-V8_EXPORT_PRIVATE std::ostream& operator<<(std::ostream&,
-                                           S128ImmediateParameter const&);
+template <const int simd_size>
+bool operator!=(SimdImmediateParameter<simd_size> const& lhs,
+                SimdImmediateParameter<simd_size> const& rhs) {
+  return !(lhs == rhs);
+}
+
+template <const int simd_size>
+size_t hash_value(SimdImmediateParameter<simd_size> const& p) {
+  return base::hash_range(p.immediate().begin(), p.immediate().end());
+}
+
+template <const int simd_size>
+V8_EXPORT_PRIVATE inline std::ostream& operator<<(
+    std::ostream& os, SimdImmediateParameter<simd_size> const& p) {
+  for (int i = 0; i < simd_size; i++) {
+    const char* separator = (i < simd_size - 1) ? "," : "";
+    os << static_cast<uint32_t>(p[i]) << separator;
+  }
+  return os;
+}
 
 V8_EXPORT_PRIVATE S128ImmediateParameter const& S128ImmediateParameterOf(
     Operator const* op) V8_WARN_UNUSED_RESULT;
 
+V8_EXPORT_PRIVATE S256ImmediateParameter const& S256ImmediateParameterOf(
+    Operator const* op) V8_WARN_UNUSED_RESULT;
+
 StackCheckKind StackCheckKindOf(Operator const* op) V8_WARN_UNUSED_RESULT;
 
 // ShiftKind::kShiftOutZeros means that it is guaranteed that the bits shifted
@@ -360,6 +404,7 @@ class V8_EXPORT_PRIVATE MachineOperatorBuilder final
     kSatConversionIsSafe = 1u << 26,
     kWord32Select = 1u << 27,
     kWord64Select = 1u << 28,
+    kLoadStorePairs = 1u << 29,
     kAllOptionalOps =
         kFloat32RoundDown | kFloat64RoundDown | kFloat32RoundUp |
         kFloat64RoundUp | kFloat32RoundTruncate | kFloat64RoundTruncate |
@@ -368,7 +413,8 @@ class V8_EXPORT_PRIVATE MachineOperatorBuilder final
         kWord64Popcnt | kWord32ReverseBits | kWord64ReverseBits |
         kInt32AbsWithOverflow | kInt64AbsWithOverflow | kWord32Rol |
         kWord64Rol | kWord64RolLowerable | kSatConversionIsSafe |
-        kFloat32Select | kFloat64Select | kWord32Select | kWord64Select
+        kFloat32Select | kFloat64Select | kWord32Select | kWord64Select |
+        kLoadStorePairs
   };
   using Flags = base::Flags<Flag, unsigned>;
 
@@ -477,6 +523,13 @@ class V8_EXPORT_PRIVATE MachineOperatorBuilder final
   // generate the correct value if a saturating conversion is requested.
   bool SatConversionIsSafe() const { return flags_ & kSatConversionIsSafe; }
 
+  // Return true if the target suppoerts performing a pair of loads/stores in
+  // a single operation.
+  bool SupportsLoadStorePairs() const {
+    return !v8_flags.enable_unconditional_write_barriers &&
+           (flags_ & kLoadStorePairs);
+  }
+
   const Operator* Word64And();
   const Operator* Word64Or();
   const Operator* Word64Xor();
@@ -730,6 +783,12 @@ class V8_EXPORT_PRIVATE MachineOperatorBuilder final
   // Identity for any input that is not signalling NaN.
   const Operator* Float64SilenceNaN();
 
+  // SIMD operators also used outside of Wasm (e.g. swisstable).
+  const Operator* I8x16Splat();
+  const Operator* I8x16Eq();
+  const Operator* I8x16BitMask();
+
+#if V8_ENABLE_WEBASSEMBLY
   // SIMD operators.
   const Operator* F64x2Splat();
   const Operator* F64x2Abs();
@@ -896,7 +955,6 @@ class V8_EXPORT_PRIVATE MachineOperatorBuilder final
   const Operator* I16x8ExtAddPairwiseI8x16S();
   const Operator* I16x8ExtAddPairwiseI8x16U();
 
-  const Operator* I8x16Splat();
   const Operator* I8x16ExtractLaneU(int32_t);
   const Operator* I8x16ExtractLaneS(int32_t);
   const Operator* I8x16ReplaceLane(int32_t);
@@ -910,7 +968,6 @@ class V8_EXPORT_PRIVATE MachineOperatorBuilder final
   const Operator* I8x16SubSatS();
   const Operator* I8x16MinS();
   const Operator* I8x16MaxS();
-  const Operator* I8x16Eq();
   const Operator* I8x16Ne();
   const Operator* I8x16GtS();
   const Operator* I8x16GeS();
@@ -926,7 +983,6 @@ class V8_EXPORT_PRIVATE MachineOperatorBuilder final
   const Operator* I8x16RoundingAverageU();
   const Operator* I8x16Popcnt();
   const Operator* I8x16Abs();
-  const Operator* I8x16BitMask();
 
   const Operator* S128Const(const uint8_t value[16]);
 
@@ -939,6 +995,8 @@ class V8_EXPORT_PRIVATE MachineOperatorBuilder final
   const Operator* S128AndNot();
 
   const Operator* I8x16Swizzle(bool relaxed = false);
+  // Helper for turboshaft/recreate-schedule.cc.
+  const Operator* I8x16RelaxedSwizzle() { return I8x16Swizzle(true); }
   const Operator* I8x16Shuffle(const uint8_t shuffle[16]);
 
   const Operator* V128AnyTrue();
@@ -964,10 +1022,14 @@ class V8_EXPORT_PRIVATE MachineOperatorBuilder final
   const Operator* I16x8DotI8x16I7x16S();
   const Operator* I32x4DotI8x16I7x16AddS();
 
-  const Operator* TraceInstruction(uint32_t markid);
-
   // SIMD256
+  const Operator* F64x4Min();
+  const Operator* F64x4Max();
   const Operator* F64x4Add();
+  const Operator* F64x4Sqrt();
+  const Operator* F32x8Abs();
+  const Operator* F32x8Neg();
+  const Operator* F32x8Sqrt();
   const Operator* F32x8Add();
   const Operator* I64x4Add();
   const Operator* I32x8Add();
@@ -999,17 +1061,108 @@ class V8_EXPORT_PRIVATE MachineOperatorBuilder final
   const Operator* F32x8Pmin();
   const Operator* F32x8Pmax();
   const Operator* F32x8Eq();
+  const Operator* F64x4Eq();
+  const Operator* I64x4Eq();
+  const Operator* I32x8Eq();
+  const Operator* I16x16Eq();
+  const Operator* I8x32Eq();
   const Operator* F32x8Ne();
+  const Operator* F64x4Ne();
+  const Operator* I64x4GtS();
+  const Operator* I32x8GtS();
+  const Operator* I16x16GtS();
+  const Operator* I8x32GtS();
+  const Operator* F64x4Lt();
   const Operator* F32x8Lt();
+  const Operator* F64x4Le();
   const Operator* F32x8Le();
-  const Operator* S256Select();
+  const Operator* I32x8MinS();
+  const Operator* I16x16MinS();
+  const Operator* I8x32MinS();
+  const Operator* I32x8MinU();
+  const Operator* I16x16MinU();
+  const Operator* I8x32MinU();
+  const Operator* I32x8MaxS();
+  const Operator* I16x16MaxS();
+  const Operator* I8x32MaxS();
+  const Operator* I32x8MaxU();
+  const Operator* I16x16MaxU();
+  const Operator* I8x32MaxU();
+  const Operator* I64x4Ne();
+  const Operator* I64x4GeS();
+  const Operator* I32x8Ne();
+  const Operator* I32x8GtU();
+  const Operator* I32x8GeS();
+  const Operator* I32x8GeU();
+  const Operator* I16x16Ne();
+  const Operator* I16x16GtU();
+  const Operator* I16x16GeS();
+  const Operator* I16x16GeU();
+  const Operator* I8x32Ne();
+  const Operator* I8x32GtU();
+  const Operator* I8x32GeS();
+  const Operator* I8x32GeU();
+  const Operator* I32x8UConvertF32x8();
+  const Operator* F64x4ConvertI32x4S();
+  const Operator* F32x8SConvertI32x8();
+  const Operator* F32x8UConvertI32x8();
+  const Operator* F32x4DemoteF64x4();
+  const Operator* I64x4SConvertI32x4();
+  const Operator* I64x4UConvertI32x4();
+  const Operator* I32x8SConvertI16x8();
+  const Operator* I32x8UConvertI16x8();
+  const Operator* I16x16SConvertI8x16();
+  const Operator* I16x16UConvertI8x16();
+  const Operator* I16x16SConvertI32x8();
+  const Operator* I16x16UConvertI32x8();
+  const Operator* I8x32SConvertI16x16();
+  const Operator* I8x32UConvertI16x16();
+  const Operator* I32x8Neg();
+  const Operator* I32x8Abs();
+  const Operator* I16x16Neg();
+  const Operator* I16x16Abs();
+  const Operator* I8x32Neg();
+  const Operator* I8x32Abs();
+  const Operator* I64x4Shl();
+  const Operator* I64x4ShrU();
+  const Operator* I32x8Shl();
+  const Operator* I32x8ShrS();
+  const Operator* I32x8ShrU();
+  const Operator* I16x16Shl();
+  const Operator* I16x16ShrS();
+  const Operator* I16x16ShrU();
+  const Operator* I32x8DotI16x16S();
+  const Operator* I16x16RoundingAverageU();
+  const Operator* I8x32RoundingAverageU();
+  const Operator* I64x4ExtMulI32x4S();
+  const Operator* I64x4ExtMulI32x4U();
+  const Operator* I32x8ExtMulI16x8S();
+  const Operator* I32x8ExtMulI16x8U();
+  const Operator* I16x16ExtMulI8x16S();
+  const Operator* I16x16ExtMulI8x16U();
+  const Operator* I32x8ExtAddPairwiseI16x16S();
+  const Operator* I32x8ExtAddPairwiseI16x16U();
+  const Operator* I16x16ExtAddPairwiseI8x32S();
+  const Operator* I16x16ExtAddPairwiseI8x32U();
   const Operator* ExtractF128(int32_t lane_index);
+  const Operator* I64x4Splat();
+  const Operator* I32x8Splat();
+  const Operator* I16x16Splat();
+  const Operator* I8x32Splat();
+  const Operator* F64x4Pmin();
+  const Operator* F64x4Pmax();
+  const Operator* F64x4Splat();
+  const Operator* F32x8Splat();
+  const Operator* I8x32Shuffle(const uint8_t shuffle[32]);
 
-  // load [base + index]
-  const Operator* Load(LoadRepresentation rep);
-  const Operator* LoadImmutable(LoadRepresentation rep);
-  const Operator* ProtectedLoad(LoadRepresentation rep);
-  const Operator* LoadTrapOnNull(LoadRepresentation rep);
+  const Operator* S256Const(const uint8_t value[32]);
+  const Operator* S256Zero();
+  const Operator* S256And();
+  const Operator* S256Or();
+  const Operator* S256Xor();
+  const Operator* S256Not();
+  const Operator* S256Select();
+  const Operator* S256AndNot();
 
   const Operator* LoadTransform(MemoryAccessKind kind,
                                 LoadTransformation transform);
@@ -1018,15 +1171,28 @@ class V8_EXPORT_PRIVATE MachineOperatorBuilder final
   const Operator* LoadLane(MemoryAccessKind kind, LoadRepresentation rep,
                            uint8_t laneidx);
 
-  // store [base + index], value
-  const Operator* Store(StoreRepresentation rep);
-  const Operator* ProtectedStore(MachineRepresentation rep);
-  const Operator* StoreTrapOnNull(StoreRepresentation rep);
-
   // SIMD store: store a specified lane of value into [base + index].
   const Operator* StoreLane(MemoryAccessKind kind, MachineRepresentation rep,
                             uint8_t laneidx);
 
+#endif  // V8_ENABLE_WEBASSEMBLY
+
+  const Operator* TraceInstruction(uint32_t markid);
+
+  // load [base + index]
+  const Operator* Load(LoadRepresentation rep);
+  const Operator* LoadImmutable(LoadRepresentation rep);
+  const Operator* ProtectedLoad(LoadRepresentation rep);
+  const Operator* LoadTrapOnNull(LoadRepresentation rep);
+
+  // store [base + index], value
+  const Operator* Store(StoreRepresentation rep);
+  base::Optional<const Operator*> TryStorePair(StoreRepresentation rep1,
+                                               StoreRepresentation rep2);
+  const Operator* StoreIndirectPointer(WriteBarrierKind write_barrier_kind);
+  const Operator* ProtectedStore(MachineRepresentation rep);
+  const Operator* StoreTrapOnNull(StoreRepresentation rep);
+
   // unaligned load [base + index]
   const Operator* UnalignedLoad(LoadRepresentation rep);
 
@@ -1045,6 +1211,10 @@ class V8_EXPORT_PRIVATE MachineOperatorBuilder final
   // Access to the machine stack.
   const Operator* LoadFramePointer();
   const Operator* LoadParentFramePointer();
+#if V8_ENABLE_WEBASSEMBLY
+  const Operator* LoadStackPointer();
+  const Operator* SetStackPointer(wasm::FPRelativeScope fp_scope);
+#endif
 
   // Compares: stack_pointer [- offset] > value. The offset is optionally
   // applied for kFunctionEntry stack checks.
