@@ -25,6 +25,7 @@
   V(Deoptimize)            \
   V(DeoptimizeIf)          \
   V(DeoptimizeUnless)      \
+  V(Assert)                \
   V(TrapIf)                \
   V(TrapUnless)            \
   V(Return)                \
@@ -213,6 +214,7 @@
 
 #define JS_CONSTRUCT_OP_LIST(V) \
   V(JSConstructForwardVarargs)  \
+  V(JSConstructForwardAllArgs)  \
   V(JSConstruct)                \
   V(JSConstructWithArrayLike)   \
   V(JSConstructWithSpread)
@@ -556,9 +558,11 @@
   V(Null)                          \
   V(RttCanon)                      \
   V(WasmTypeCast)                  \
+  V(WasmTypeCastAbstract)          \
   V(WasmTypeCheck)                 \
-  V(WasmExternInternalize)         \
-  V(WasmExternExternalize)         \
+  V(WasmTypeCheckAbstract)         \
+  V(WasmAnyConvertExtern)          \
+  V(WasmExternConvertAny)          \
   V(WasmStructGet)                 \
   V(WasmStructSet)                 \
   V(WasmArrayGet)                  \
@@ -757,6 +761,8 @@
   V(Load)                                \
   V(LoadImmutable)                       \
   V(Store)                               \
+  V(StorePair)                           \
+  V(StoreIndirectPointer)                \
   V(StackSlot)                           \
   V(Word32Popcnt)                        \
   V(Word64Popcnt)                        \
@@ -818,6 +824,8 @@
   V(Float64Select)                       \
   V(LoadStackCheckOffset)                \
   V(LoadFramePointer)                    \
+  IF_WASM(V, LoadStackPointer)           \
+  IF_WASM(V, SetStackPointer)            \
   V(LoadParentFramePointer)              \
   V(LoadRootRegister)                    \
   V(UnalignedLoad)                       \
@@ -841,246 +849,249 @@
   V(StackPointerGreaterThan)             \
   V(TraceInstruction)
 
-#define MACHINE_SIMD128_OP_LIST(V) \
-  V(F64x2Splat)                    \
-  V(F64x2ExtractLane)              \
-  V(F64x2ReplaceLane)              \
-  V(F64x2Abs)                      \
-  V(F64x2Neg)                      \
-  V(F64x2Sqrt)                     \
-  V(F64x2Add)                      \
-  V(F64x2Sub)                      \
-  V(F64x2Mul)                      \
-  V(F64x2Div)                      \
-  V(F64x2Min)                      \
-  V(F64x2Max)                      \
-  V(F64x2Eq)                       \
-  V(F64x2Ne)                       \
-  V(F64x2Lt)                       \
-  V(F64x2Le)                       \
-  V(F64x2Qfma)                     \
-  V(F64x2Qfms)                     \
-  V(F64x2Pmin)                     \
-  V(F64x2Pmax)                     \
-  V(F64x2Ceil)                     \
-  V(F64x2Floor)                    \
-  V(F64x2Trunc)                    \
-  V(F64x2NearestInt)               \
-  V(F64x2ConvertLowI32x4S)         \
-  V(F64x2ConvertLowI32x4U)         \
-  V(F64x2PromoteLowF32x4)          \
-  V(F32x4Splat)                    \
-  V(F32x4ExtractLane)              \
-  V(F32x4ReplaceLane)              \
-  V(F32x4SConvertI32x4)            \
-  V(F32x4UConvertI32x4)            \
-  V(F32x4Abs)                      \
-  V(F32x4Neg)                      \
-  V(F32x4Sqrt)                     \
-  V(F32x4Add)                      \
-  V(F32x4Sub)                      \
-  V(F32x4Mul)                      \
-  V(F32x4Div)                      \
-  V(F32x4Min)                      \
-  V(F32x4Max)                      \
-  V(F32x4Eq)                       \
-  V(F32x4Ne)                       \
-  V(F32x4Lt)                       \
-  V(F32x4Le)                       \
-  V(F32x4Gt)                       \
-  V(F32x4Ge)                       \
-  V(F32x4Qfma)                     \
-  V(F32x4Qfms)                     \
-  V(F32x4Pmin)                     \
-  V(F32x4Pmax)                     \
-  V(F32x4Ceil)                     \
-  V(F32x4Floor)                    \
-  V(F32x4Trunc)                    \
-  V(F32x4NearestInt)               \
-  V(F32x4DemoteF64x2Zero)          \
-  V(I64x2Splat)                    \
-  V(I64x2SplatI32Pair)             \
-  V(I64x2ExtractLane)              \
-  V(I64x2ReplaceLane)              \
-  V(I64x2ReplaceLaneI32Pair)       \
-  V(I64x2Abs)                      \
-  V(I64x2Neg)                      \
-  V(I64x2SConvertI32x4Low)         \
-  V(I64x2SConvertI32x4High)        \
-  V(I64x2UConvertI32x4Low)         \
-  V(I64x2UConvertI32x4High)        \
-  V(I64x2BitMask)                  \
-  V(I64x2Shl)                      \
-  V(I64x2ShrS)                     \
-  V(I64x2Add)                      \
-  V(I64x2Sub)                      \
-  V(I64x2Mul)                      \
-  V(I64x2Eq)                       \
-  V(I64x2Ne)                       \
-  V(I64x2GtS)                      \
-  V(I64x2GeS)                      \
-  V(I64x2ShrU)                     \
-  V(I64x2ExtMulLowI32x4S)          \
-  V(I64x2ExtMulHighI32x4S)         \
-  V(I64x2ExtMulLowI32x4U)          \
-  V(I64x2ExtMulHighI32x4U)         \
-  V(I32x4Splat)                    \
-  V(I32x4ExtractLane)              \
-  V(I32x4ReplaceLane)              \
-  V(I32x4SConvertF32x4)            \
-  V(I32x4SConvertI16x8Low)         \
-  V(I32x4SConvertI16x8High)        \
-  V(I32x4Neg)                      \
-  V(I32x4Shl)                      \
-  V(I32x4ShrS)                     \
-  V(I32x4Add)                      \
-  V(I32x4Sub)                      \
-  V(I32x4Mul)                      \
-  V(I32x4MinS)                     \
-  V(I32x4MaxS)                     \
-  V(I32x4Eq)                       \
-  V(I32x4Ne)                       \
-  V(I32x4LtS)                      \
-  V(I32x4LeS)                      \
-  V(I32x4GtS)                      \
-  V(I32x4GeS)                      \
-  V(I32x4UConvertF32x4)            \
-  V(I32x4UConvertI16x8Low)         \
-  V(I32x4UConvertI16x8High)        \
-  V(I32x4ShrU)                     \
-  V(I32x4MinU)                     \
-  V(I32x4MaxU)                     \
-  V(I32x4LtU)                      \
-  V(I32x4LeU)                      \
-  V(I32x4GtU)                      \
-  V(I32x4GeU)                      \
-  V(I32x4Abs)                      \
-  V(I32x4BitMask)                  \
-  V(I32x4DotI16x8S)                \
-  V(I32x4ExtMulLowI16x8S)          \
-  V(I32x4ExtMulHighI16x8S)         \
-  V(I32x4ExtMulLowI16x8U)          \
-  V(I32x4ExtMulHighI16x8U)         \
-  V(I32x4ExtAddPairwiseI16x8S)     \
-  V(I32x4ExtAddPairwiseI16x8U)     \
-  V(I32x4TruncSatF64x2SZero)       \
-  V(I32x4TruncSatF64x2UZero)       \
-  V(I16x8Splat)                    \
-  V(I16x8ExtractLaneU)             \
-  V(I16x8ExtractLaneS)             \
-  V(I16x8ReplaceLane)              \
-  V(I16x8SConvertI8x16Low)         \
-  V(I16x8SConvertI8x16High)        \
-  V(I16x8Neg)                      \
-  V(I16x8Shl)                      \
-  V(I16x8ShrS)                     \
-  V(I16x8SConvertI32x4)            \
-  V(I16x8Add)                      \
-  V(I16x8AddSatS)                  \
-  V(I16x8Sub)                      \
-  V(I16x8SubSatS)                  \
-  V(I16x8Mul)                      \
-  V(I16x8MinS)                     \
-  V(I16x8MaxS)                     \
-  V(I16x8Eq)                       \
-  V(I16x8Ne)                       \
-  V(I16x8LtS)                      \
-  V(I16x8LeS)                      \
-  V(I16x8GtS)                      \
-  V(I16x8GeS)                      \
-  V(I16x8UConvertI8x16Low)         \
-  V(I16x8UConvertI8x16High)        \
-  V(I16x8ShrU)                     \
-  V(I16x8UConvertI32x4)            \
-  V(I16x8AddSatU)                  \
-  V(I16x8SubSatU)                  \
-  V(I16x8MinU)                     \
-  V(I16x8MaxU)                     \
-  V(I16x8LtU)                      \
-  V(I16x8LeU)                      \
-  V(I16x8GtU)                      \
-  V(I16x8GeU)                      \
-  V(I16x8RoundingAverageU)         \
-  V(I16x8Q15MulRSatS)              \
-  V(I16x8Abs)                      \
-  V(I16x8BitMask)                  \
-  V(I16x8ExtMulLowI8x16S)          \
-  V(I16x8ExtMulHighI8x16S)         \
-  V(I16x8ExtMulLowI8x16U)          \
-  V(I16x8ExtMulHighI8x16U)         \
-  V(I16x8ExtAddPairwiseI8x16S)     \
-  V(I16x8ExtAddPairwiseI8x16U)     \
-  V(I8x16Splat)                    \
-  V(I8x16ExtractLaneU)             \
-  V(I8x16ExtractLaneS)             \
-  V(I8x16ReplaceLane)              \
-  V(I8x16SConvertI16x8)            \
-  V(I8x16Neg)                      \
-  V(I8x16Shl)                      \
-  V(I8x16ShrS)                     \
-  V(I8x16Add)                      \
-  V(I8x16AddSatS)                  \
-  V(I8x16Sub)                      \
-  V(I8x16SubSatS)                  \
-  V(I8x16MinS)                     \
-  V(I8x16MaxS)                     \
-  V(I8x16Eq)                       \
-  V(I8x16Ne)                       \
-  V(I8x16LtS)                      \
-  V(I8x16LeS)                      \
-  V(I8x16GtS)                      \
-  V(I8x16GeS)                      \
-  V(I8x16UConvertI16x8)            \
-  V(I8x16AddSatU)                  \
-  V(I8x16SubSatU)                  \
-  V(I8x16ShrU)                     \
-  V(I8x16MinU)                     \
-  V(I8x16MaxU)                     \
-  V(I8x16LtU)                      \
-  V(I8x16LeU)                      \
-  V(I8x16GtU)                      \
-  V(I8x16GeU)                      \
-  V(I8x16RoundingAverageU)         \
-  V(I8x16Popcnt)                   \
-  V(I8x16Abs)                      \
-  V(I8x16BitMask)                  \
-  V(S128Zero)                      \
-  V(S128Const)                     \
-  V(S128Not)                       \
-  V(S128And)                       \
-  V(S128Or)                        \
-  V(S128Xor)                       \
-  V(S128Select)                    \
-  V(S128AndNot)                    \
-  V(I8x16Swizzle)                  \
-  V(I8x16RelaxedLaneSelect)        \
-  V(I16x8RelaxedLaneSelect)        \
-  V(I32x4RelaxedLaneSelect)        \
-  V(I64x2RelaxedLaneSelect)        \
-  V(F32x4RelaxedMin)               \
-  V(F32x4RelaxedMax)               \
-  V(F64x2RelaxedMin)               \
-  V(F64x2RelaxedMax)               \
-  V(I32x4RelaxedTruncF32x4S)       \
-  V(I32x4RelaxedTruncF32x4U)       \
-  V(I32x4RelaxedTruncF64x2SZero)   \
-  V(I32x4RelaxedTruncF64x2UZero)   \
-  V(I16x8RelaxedQ15MulRS)          \
-  V(I16x8DotI8x16I7x16S)           \
-  V(I32x4DotI8x16I7x16AddS)        \
-  V(I8x16Shuffle)                  \
-  V(V128AnyTrue)                   \
-  V(I64x2AllTrue)                  \
-  V(I32x4AllTrue)                  \
-  V(I16x8AllTrue)                  \
-  V(I8x16AllTrue)                  \
-  V(LoadTransform)                 \
-  V(LoadLane)                      \
-  V(StoreLane)
+#define MACHINE_SIMD128_OP_LIST(V)        \
+  IF_WASM(V, F64x2Splat)                  \
+  IF_WASM(V, F64x2ExtractLane)            \
+  IF_WASM(V, F64x2ReplaceLane)            \
+  IF_WASM(V, F64x2Abs)                    \
+  IF_WASM(V, F64x2Neg)                    \
+  IF_WASM(V, F64x2Sqrt)                   \
+  IF_WASM(V, F64x2Add)                    \
+  IF_WASM(V, F64x2Sub)                    \
+  IF_WASM(V, F64x2Mul)                    \
+  IF_WASM(V, F64x2Div)                    \
+  IF_WASM(V, F64x2Min)                    \
+  IF_WASM(V, F64x2Max)                    \
+  IF_WASM(V, F64x2Eq)                     \
+  IF_WASM(V, F64x2Ne)                     \
+  IF_WASM(V, F64x2Lt)                     \
+  IF_WASM(V, F64x2Le)                     \
+  IF_WASM(V, F64x2Qfma)                   \
+  IF_WASM(V, F64x2Qfms)                   \
+  IF_WASM(V, F64x2Pmin)                   \
+  IF_WASM(V, F64x2Pmax)                   \
+  IF_WASM(V, F64x2Ceil)                   \
+  IF_WASM(V, F64x2Floor)                  \
+  IF_WASM(V, F64x2Trunc)                  \
+  IF_WASM(V, F64x2NearestInt)             \
+  IF_WASM(V, F64x2ConvertLowI32x4S)       \
+  IF_WASM(V, F64x2ConvertLowI32x4U)       \
+  IF_WASM(V, F64x2PromoteLowF32x4)        \
+  IF_WASM(V, F32x4Splat)                  \
+  IF_WASM(V, F32x4ExtractLane)            \
+  IF_WASM(V, F32x4ReplaceLane)            \
+  IF_WASM(V, F32x4SConvertI32x4)          \
+  IF_WASM(V, F32x4UConvertI32x4)          \
+  IF_WASM(V, F32x4Abs)                    \
+  IF_WASM(V, F32x4Neg)                    \
+  IF_WASM(V, F32x4Sqrt)                   \
+  IF_WASM(V, F32x4Add)                    \
+  IF_WASM(V, F32x4Sub)                    \
+  IF_WASM(V, F32x4Mul)                    \
+  IF_WASM(V, F32x4Div)                    \
+  IF_WASM(V, F32x4Min)                    \
+  IF_WASM(V, F32x4Max)                    \
+  IF_WASM(V, F32x4Eq)                     \
+  IF_WASM(V, F32x4Ne)                     \
+  IF_WASM(V, F32x4Lt)                     \
+  IF_WASM(V, F32x4Le)                     \
+  IF_WASM(V, F32x4Gt)                     \
+  IF_WASM(V, F32x4Ge)                     \
+  IF_WASM(V, F32x4Qfma)                   \
+  IF_WASM(V, F32x4Qfms)                   \
+  IF_WASM(V, F32x4Pmin)                   \
+  IF_WASM(V, F32x4Pmax)                   \
+  IF_WASM(V, F32x4Ceil)                   \
+  IF_WASM(V, F32x4Floor)                  \
+  IF_WASM(V, F32x4Trunc)                  \
+  IF_WASM(V, F32x4NearestInt)             \
+  IF_WASM(V, F32x4DemoteF64x2Zero)        \
+  IF_WASM(V, I64x2Splat)                  \
+  IF_WASM(V, I64x2SplatI32Pair)           \
+  IF_WASM(V, I64x2ExtractLane)            \
+  IF_WASM(V, I64x2ReplaceLane)            \
+  IF_WASM(V, I64x2ReplaceLaneI32Pair)     \
+  IF_WASM(V, I64x2Abs)                    \
+  IF_WASM(V, I64x2Neg)                    \
+  IF_WASM(V, I64x2SConvertI32x4Low)       \
+  IF_WASM(V, I64x2SConvertI32x4High)      \
+  IF_WASM(V, I64x2UConvertI32x4Low)       \
+  IF_WASM(V, I64x2UConvertI32x4High)      \
+  IF_WASM(V, I64x2BitMask)                \
+  IF_WASM(V, I64x2Shl)                    \
+  IF_WASM(V, I64x2ShrS)                   \
+  IF_WASM(V, I64x2Add)                    \
+  IF_WASM(V, I64x2Sub)                    \
+  IF_WASM(V, I64x2Mul)                    \
+  IF_WASM(V, I64x2Eq)                     \
+  IF_WASM(V, I64x2Ne)                     \
+  IF_WASM(V, I64x2GtS)                    \
+  IF_WASM(V, I64x2GeS)                    \
+  IF_WASM(V, I64x2ShrU)                   \
+  IF_WASM(V, I64x2ExtMulLowI32x4S)        \
+  IF_WASM(V, I64x2ExtMulHighI32x4S)       \
+  IF_WASM(V, I64x2ExtMulLowI32x4U)        \
+  IF_WASM(V, I64x2ExtMulHighI32x4U)       \
+  IF_WASM(V, I32x4Splat)                  \
+  IF_WASM(V, I32x4ExtractLane)            \
+  IF_WASM(V, I32x4ReplaceLane)            \
+  IF_WASM(V, I32x4SConvertF32x4)          \
+  IF_WASM(V, I32x4SConvertI16x8Low)       \
+  IF_WASM(V, I32x4SConvertI16x8High)      \
+  IF_WASM(V, I32x4Neg)                    \
+  IF_WASM(V, I32x4Shl)                    \
+  IF_WASM(V, I32x4ShrS)                   \
+  IF_WASM(V, I32x4Add)                    \
+  IF_WASM(V, I32x4Sub)                    \
+  IF_WASM(V, I32x4Mul)                    \
+  IF_WASM(V, I32x4MinS)                   \
+  IF_WASM(V, I32x4MaxS)                   \
+  IF_WASM(V, I32x4Eq)                     \
+  IF_WASM(V, I32x4Ne)                     \
+  IF_WASM(V, I32x4LtS)                    \
+  IF_WASM(V, I32x4LeS)                    \
+  IF_WASM(V, I32x4GtS)                    \
+  IF_WASM(V, I32x4GeS)                    \
+  IF_WASM(V, I32x4UConvertF32x4)          \
+  IF_WASM(V, I32x4UConvertI16x8Low)       \
+  IF_WASM(V, I32x4UConvertI16x8High)      \
+  IF_WASM(V, I32x4ShrU)                   \
+  IF_WASM(V, I32x4MinU)                   \
+  IF_WASM(V, I32x4MaxU)                   \
+  IF_WASM(V, I32x4LtU)                    \
+  IF_WASM(V, I32x4LeU)                    \
+  IF_WASM(V, I32x4GtU)                    \
+  IF_WASM(V, I32x4GeU)                    \
+  IF_WASM(V, I32x4Abs)                    \
+  IF_WASM(V, I32x4BitMask)                \
+  IF_WASM(V, I32x4DotI16x8S)              \
+  IF_WASM(V, I32x4ExtMulLowI16x8S)        \
+  IF_WASM(V, I32x4ExtMulHighI16x8S)       \
+  IF_WASM(V, I32x4ExtMulLowI16x8U)        \
+  IF_WASM(V, I32x4ExtMulHighI16x8U)       \
+  IF_WASM(V, I32x4ExtAddPairwiseI16x8S)   \
+  IF_WASM(V, I32x4ExtAddPairwiseI16x8U)   \
+  IF_WASM(V, I32x4TruncSatF64x2SZero)     \
+  IF_WASM(V, I32x4TruncSatF64x2UZero)     \
+  IF_WASM(V, I16x8Splat)                  \
+  IF_WASM(V, I16x8ExtractLaneU)           \
+  IF_WASM(V, I16x8ExtractLaneS)           \
+  IF_WASM(V, I16x8ReplaceLane)            \
+  IF_WASM(V, I16x8SConvertI8x16Low)       \
+  IF_WASM(V, I16x8SConvertI8x16High)      \
+  IF_WASM(V, I16x8Neg)                    \
+  IF_WASM(V, I16x8Shl)                    \
+  IF_WASM(V, I16x8ShrS)                   \
+  IF_WASM(V, I16x8SConvertI32x4)          \
+  IF_WASM(V, I16x8Add)                    \
+  IF_WASM(V, I16x8AddSatS)                \
+  IF_WASM(V, I16x8Sub)                    \
+  IF_WASM(V, I16x8SubSatS)                \
+  IF_WASM(V, I16x8Mul)                    \
+  IF_WASM(V, I16x8MinS)                   \
+  IF_WASM(V, I16x8MaxS)                   \
+  IF_WASM(V, I16x8Eq)                     \
+  IF_WASM(V, I16x8Ne)                     \
+  IF_WASM(V, I16x8LtS)                    \
+  IF_WASM(V, I16x8LeS)                    \
+  IF_WASM(V, I16x8GtS)                    \
+  IF_WASM(V, I16x8GeS)                    \
+  IF_WASM(V, I16x8UConvertI8x16Low)       \
+  IF_WASM(V, I16x8UConvertI8x16High)      \
+  IF_WASM(V, I16x8ShrU)                   \
+  IF_WASM(V, I16x8UConvertI32x4)          \
+  IF_WASM(V, I16x8AddSatU)                \
+  IF_WASM(V, I16x8SubSatU)                \
+  IF_WASM(V, I16x8MinU)                   \
+  IF_WASM(V, I16x8MaxU)                   \
+  IF_WASM(V, I16x8LtU)                    \
+  IF_WASM(V, I16x8LeU)                    \
+  IF_WASM(V, I16x8GtU)                    \
+  IF_WASM(V, I16x8GeU)                    \
+  IF_WASM(V, I16x8RoundingAverageU)       \
+  IF_WASM(V, I16x8Q15MulRSatS)            \
+  IF_WASM(V, I16x8Abs)                    \
+  IF_WASM(V, I16x8BitMask)                \
+  IF_WASM(V, I16x8ExtMulLowI8x16S)        \
+  IF_WASM(V, I16x8ExtMulHighI8x16S)       \
+  IF_WASM(V, I16x8ExtMulLowI8x16U)        \
+  IF_WASM(V, I16x8ExtMulHighI8x16U)       \
+  IF_WASM(V, I16x8ExtAddPairwiseI8x16S)   \
+  IF_WASM(V, I16x8ExtAddPairwiseI8x16U)   \
+  V(I8x16Splat)                           \
+  IF_WASM(V, I8x16ExtractLaneU)           \
+  IF_WASM(V, I8x16ExtractLaneS)           \
+  IF_WASM(V, I8x16ReplaceLane)            \
+  IF_WASM(V, I8x16SConvertI16x8)          \
+  IF_WASM(V, I8x16Neg)                    \
+  IF_WASM(V, I8x16Shl)                    \
+  IF_WASM(V, I8x16ShrS)                   \
+  IF_WASM(V, I8x16Add)                    \
+  IF_WASM(V, I8x16AddSatS)                \
+  IF_WASM(V, I8x16Sub)                    \
+  IF_WASM(V, I8x16SubSatS)                \
+  IF_WASM(V, I8x16MinS)                   \
+  IF_WASM(V, I8x16MaxS)                   \
+  V(I8x16Eq)                              \
+  IF_WASM(V, I8x16Ne)                     \
+  IF_WASM(V, I8x16LtS)                    \
+  IF_WASM(V, I8x16LeS)                    \
+  IF_WASM(V, I8x16GtS)                    \
+  IF_WASM(V, I8x16GeS)                    \
+  IF_WASM(V, I8x16UConvertI16x8)          \
+  IF_WASM(V, I8x16AddSatU)                \
+  IF_WASM(V, I8x16SubSatU)                \
+  IF_WASM(V, I8x16ShrU)                   \
+  IF_WASM(V, I8x16MinU)                   \
+  IF_WASM(V, I8x16MaxU)                   \
+  IF_WASM(V, I8x16LtU)                    \
+  IF_WASM(V, I8x16LeU)                    \
+  IF_WASM(V, I8x16GtU)                    \
+  IF_WASM(V, I8x16GeU)                    \
+  IF_WASM(V, I8x16RoundingAverageU)       \
+  IF_WASM(V, I8x16Popcnt)                 \
+  IF_WASM(V, I8x16Abs)                    \
+  V(I8x16BitMask)                         \
+  IF_WASM(V, S128Zero)                    \
+  IF_WASM(V, S128Const)                   \
+  IF_WASM(V, S128Not)                     \
+  IF_WASM(V, S128And)                     \
+  IF_WASM(V, S128Or)                      \
+  IF_WASM(V, S128Xor)                     \
+  IF_WASM(V, S128Select)                  \
+  IF_WASM(V, S128AndNot)                  \
+  IF_WASM(V, I8x16Swizzle)                \
+  IF_WASM(V, I8x16RelaxedLaneSelect)      \
+  IF_WASM(V, I16x8RelaxedLaneSelect)      \
+  IF_WASM(V, I32x4RelaxedLaneSelect)      \
+  IF_WASM(V, I64x2RelaxedLaneSelect)      \
+  IF_WASM(V, F32x4RelaxedMin)             \
+  IF_WASM(V, F32x4RelaxedMax)             \
+  IF_WASM(V, F64x2RelaxedMin)             \
+  IF_WASM(V, F64x2RelaxedMax)             \
+  IF_WASM(V, I32x4RelaxedTruncF32x4S)     \
+  IF_WASM(V, I32x4RelaxedTruncF32x4U)     \
+  IF_WASM(V, I32x4RelaxedTruncF64x2SZero) \
+  IF_WASM(V, I32x4RelaxedTruncF64x2UZero) \
+  IF_WASM(V, I16x8RelaxedQ15MulRS)        \
+  IF_WASM(V, I16x8DotI8x16I7x16S)         \
+  IF_WASM(V, I32x4DotI8x16I7x16AddS)      \
+  IF_WASM(V, I8x16Shuffle)                \
+  IF_WASM(V, V128AnyTrue)                 \
+  IF_WASM(V, I64x2AllTrue)                \
+  IF_WASM(V, I32x4AllTrue)                \
+  IF_WASM(V, I16x8AllTrue)                \
+  IF_WASM(V, I8x16AllTrue)                \
+  IF_WASM(V, LoadTransform)               \
+  IF_WASM(V, LoadLane)                    \
+  IF_WASM(V, StoreLane)
 
 // SIMD256 for AVX
 #define MACHINE_SIMD256_OP_LIST(V) \
+  V(F64x4Min)                      \
+  V(F64x4Max)                      \
   V(F64x4Add)                      \
+  V(F64x4Sqrt)                     \
   V(F32x8Add)                      \
   V(I64x4Add)                      \
   V(I32x8Add)                      \
@@ -1110,18 +1121,119 @@
   V(F32x8Pmin)                     \
   V(F32x8Pmax)                     \
   V(F32x8Eq)                       \
+  V(F64x4Eq)                       \
+  V(I64x4Eq)                       \
+  V(I32x8Eq)                       \
+  V(I16x16Eq)                      \
+  V(I8x32Eq)                       \
   V(F32x8Ne)                       \
+  V(F64x4Ne)                       \
+  V(I64x4GtS)                      \
+  V(I32x8GtS)                      \
+  V(I16x16GtS)                     \
+  V(I8x32GtS)                      \
+  V(F64x4Lt)                       \
   V(F32x8Lt)                       \
+  V(F64x4Le)                       \
   V(F32x8Le)                       \
+  V(I32x8MinS)                     \
+  V(I16x16MinS)                    \
+  V(I8x32MinS)                     \
+  V(I32x8MinU)                     \
+  V(I16x16MinU)                    \
+  V(I8x32MinU)                     \
+  V(I32x8MaxS)                     \
+  V(I16x16MaxS)                    \
+  V(I8x32MaxS)                     \
+  V(I32x8MaxU)                     \
+  V(I16x16MaxU)                    \
+  V(I8x32MaxU)                     \
+  V(F32x8Min)                      \
+  V(F32x8Max)                      \
+  V(I64x4Ne)                       \
+  V(I64x4GeS)                      \
+  V(I32x8Ne)                       \
+  V(I32x8GtU)                      \
+  V(I32x8GeS)                      \
+  V(I32x8GeU)                      \
+  V(I16x16Ne)                      \
+  V(I16x16GtU)                     \
+  V(I16x16GeS)                     \
+  V(I16x16GeU)                     \
+  V(I8x32Ne)                       \
+  V(I8x32GtU)                      \
+  V(I8x32GeS)                      \
+  V(I8x32GeU)                      \
+  V(I32x8UConvertF32x8)            \
+  V(F64x4ConvertI32x4S)            \
+  V(F32x8SConvertI32x8)            \
+  V(F32x8UConvertI32x8)            \
+  V(F32x4DemoteF64x4)              \
+  V(I64x4SConvertI32x4)            \
+  V(I64x4UConvertI32x4)            \
+  V(I32x8SConvertI16x8)            \
+  V(I32x8UConvertI16x8)            \
+  V(I16x16SConvertI8x16)           \
+  V(I16x16UConvertI8x16)           \
+  V(I16x16SConvertI32x8)           \
+  V(I16x16UConvertI32x8)           \
+  V(I8x32SConvertI16x16)           \
+  V(I8x32UConvertI16x16)           \
+  V(F32x8Abs)                      \
+  V(F32x8Neg)                      \
+  V(F32x8Sqrt)                     \
+  V(I32x8Abs)                      \
+  V(I32x8Neg)                      \
+  V(I16x16Abs)                     \
+  V(I16x16Neg)                     \
+  V(I8x32Abs)                      \
+  V(I8x32Neg)                      \
+  V(I64x4Shl)                      \
+  V(I64x4ShrU)                     \
+  V(I32x8Shl)                      \
+  V(I32x8ShrS)                     \
+  V(I32x8ShrU)                     \
+  V(I16x16Shl)                     \
+  V(I16x16ShrS)                    \
+  V(I16x16ShrU)                    \
+  V(I32x8DotI16x16S)               \
+  V(I16x16RoundingAverageU)        \
+  V(I8x32RoundingAverageU)         \
+  V(I64x4ExtMulI32x4S)             \
+  V(I64x4ExtMulI32x4U)             \
+  V(I32x8ExtMulI16x8S)             \
+  V(I32x8ExtMulI16x8U)             \
+  V(I16x16ExtMulI8x16S)            \
+  V(I16x16ExtMulI8x16U)            \
+  V(I32x8ExtAddPairwiseI16x16S)    \
+  V(I32x8ExtAddPairwiseI16x16U)    \
+  V(I16x16ExtAddPairwiseI8x32S)    \
+  V(I16x16ExtAddPairwiseI8x32U)    \
+  V(ExtractF128)                   \
+  V(S256Const)                     \
+  V(S256Zero)                      \
+  V(S256Not)                       \
+  V(S256And)                       \
+  V(S256Or)                        \
+  V(S256Xor)                       \
   V(S256Select)                    \
-  V(ExtractF128)
+  V(S256AndNot)                    \
+  V(I64x4Splat)                    \
+  V(I32x8Splat)                    \
+  V(I16x16Splat)                   \
+  V(I8x32Splat)                    \
+  V(F64x4Pmin)                     \
+  V(F64x4Pmax)                     \
+  V(F64x4Splat)                    \
+  V(F32x8Splat)                    \
+  V(I8x32Shuffle)
 
-#define VALUE_OP_LIST(V)     \
-  COMMON_OP_LIST(V)          \
-  SIMPLIFIED_OP_LIST(V)      \
-  MACHINE_OP_LIST(V)         \
-  MACHINE_SIMD128_OP_LIST(V) \
-  MACHINE_SIMD256_OP_LIST(V) \
+#define VALUE_OP_LIST(V)              \
+  COMMON_OP_LIST(V)                   \
+  SIMPLIFIED_OP_LIST(V)               \
+  MACHINE_OP_LIST(V)                  \
+  MACHINE_SIMD128_OP_LIST(V)          \
+  IF_WASM(MACHINE_SIMD256_OP_LIST, V) \
   JS_OP_LIST(V)
 
 // The combination of all operators at all levels and the common operators.
@@ -1308,6 +1420,7 @@ class V8_EXPORT_PRIVATE IrOpcode {
     UNREACHABLE();
   }
 
+#if V8_ENABLE_WEBASSEMBLY
   static bool IsSimd128Opcode(Value value) {
 #define CASE(Name, ...) case k##Name:
     switch (value) {
@@ -1319,6 +1432,7 @@ class V8_EXPORT_PRIVATE IrOpcode {
 #undef CASE
     UNREACHABLE();
   }
+#endif  // V8_ENABLE_WEBASSEMBLY
 };
 
 V8_EXPORT_PRIVATE std::ostream& operator<<(std::ostream&, IrOpcode::Value);
