@@ -1182,7 +1182,7 @@ struct SimplifiedOperatorGlobalCache final {
               IrOpcode::kConvertReceiver,  // opcode
               Operator::kEliminatable,     // flags
               "ConvertReceiver",           // name
-              2, 1, 1, 1, 1, 0,            // counts
+              3, 1, 1, 1, 1, 0,            // counts
               kMode) {}                    // param
   };
   ConvertReceiverOperator<ConvertReceiverMode::kAny>
@@ -1270,7 +1270,8 @@ struct SimplifiedOperatorGlobalCache final {
 
   struct StringAsWtf16Operator final : public Operator {
     StringAsWtf16Operator()
-        : Operator(IrOpcode::kStringAsWtf16, Operator::kEliminatable,
+        : Operator(IrOpcode::kStringAsWtf16,
+                   Operator::kEliminatable | Operator::kIdempotent,
                    "StringAsWtf16", 1, 1, 1, 1, 1, 1) {}
   };
   StringAsWtf16Operator kStringAsWtf16;
@@ -1485,6 +1486,14 @@ const Operator* SimplifiedOperatorBuilder::WasmTypeCheck(
       "WasmTypeCheck", 2, 1, 1, 1, 1, 1, config);
 }
 
+const Operator* SimplifiedOperatorBuilder::WasmTypeCheckAbstract(
+    WasmTypeCheckConfig config) {
+  return zone_->New<Operator1<WasmTypeCheckConfig>>(
+      IrOpcode::kWasmTypeCheckAbstract,
+      Operator::kEliminatable | Operator::kIdempotent, "WasmTypeCheckAbstract",
+      1, 1, 1, 1, 1, 1, config);
+}
+
 const Operator* SimplifiedOperatorBuilder::WasmTypeCast(
     WasmTypeCheckConfig config) {
   return zone_->New<Operator1<WasmTypeCheckConfig>>(
@@ -1493,6 +1502,14 @@ const Operator* SimplifiedOperatorBuilder::WasmTypeCast(
       "WasmTypeCast", 2, 1, 1, 1, 1, 1, config);
 }
 
+const Operator* SimplifiedOperatorBuilder::WasmTypeCastAbstract(
+    WasmTypeCheckConfig config) {
+  return zone_->New<Operator1<WasmTypeCheckConfig>>(
+      IrOpcode::kWasmTypeCastAbstract,
+      Operator::kNoWrite | Operator::kNoThrow | Operator::kIdempotent,
+      "WasmTypeCastAbstract", 1, 1, 1, 1, 1, 1, config);
+}
+
 const Operator* SimplifiedOperatorBuilder::RttCanon(int index) {
   return zone()->New<Operator1<int>>(IrOpcode::kRttCanon, Operator::kPure,
                                      "RttCanon", 1, 0, 0, 1, 0, 0, index);
@@ -1550,15 +1567,15 @@ const Operator* SimplifiedOperatorBuilder::StringPrepareForGetCodeunit() {
   return &cache_.kStringPrepareForGetCodeunit;
 }
 
-const Operator* SimplifiedOperatorBuilder::WasmExternInternalize() {
-  return zone()->New<Operator>(IrOpcode::kWasmExternInternalize,
-                               Operator::kEliminatable, "WasmExternInternalize",
+const Operator* SimplifiedOperatorBuilder::WasmAnyConvertExtern() {
+  return zone()->New<Operator>(IrOpcode::kWasmAnyConvertExtern,
+                               Operator::kEliminatable, "WasmAnyConvertExtern",
                                1, 1, 1, 1, 1, 1);
 }
 
-const Operator* SimplifiedOperatorBuilder::WasmExternExternalize() {
-  return zone()->New<Operator>(IrOpcode::kWasmExternExternalize,
-                               Operator::kEliminatable, "WasmExternExternalize",
+const Operator* SimplifiedOperatorBuilder::WasmExternConvertAny() {
+  return zone()->New<Operator>(IrOpcode::kWasmExternConvertAny,
+                               Operator::kEliminatable, "WasmExternConvertAny",
                                1, 1, 1, 1, 1, 1);
 }
 
@@ -2098,11 +2115,10 @@ const Operator* SimplifiedOperatorBuilder::Allocate(Type type,
 }
 
 const Operator* SimplifiedOperatorBuilder::AllocateRaw(
-    Type type, AllocationType allocation,
-    AllowLargeObjects allow_large_objects) {
+    Type type, AllocationType allocation) {
   return zone()->New<Operator1<AllocateParameters>>(
       IrOpcode::kAllocateRaw, Operator::kEliminatable, "AllocateRaw", 1, 1, 1,
-      1, 1, 1, AllocateParameters(type, allocation, allow_large_objects));
+      1, 1, 1, AllocateParameters(type, allocation));
 }
 
 #define SPECULATIVE_NUMBER_BINOP(Name)                                        \
