@@ -27,7 +27,7 @@ namespace v8::internal::compiler::turboshaft {
 //
 // OperationState reflects the liveness of operations. An operation is live if
 //
-//   1) The operation has the `observable_when_unused` property
+//   1) The operation has the `IsRequiredWhenUnused()` property.
 //   2) Any of its outputs is live (is used in a live operation).
 //
 // If the operation is not live, it is dead and can be eliminated.
@@ -178,13 +178,14 @@ class DeadCodeAnalysis {
  public:
   explicit DeadCodeAnalysis(Graph& graph, Zone* phase_zone)
       : graph_(graph),
-        liveness_(graph.op_id_count(), OperationState::kDead, phase_zone),
+        liveness_(graph.op_id_count(), OperationState::kDead, phase_zone,
+                  &graph),
         entry_control_state_(graph.block_count(), ControlState::Unreachable(),
                              phase_zone),
         rewritable_branch_targets_(phase_zone) {}
 
   template <bool trace_analysis>
-  std::pair<FixedSidetable<OperationState::Liveness>,
+  std::pair<FixedOpIndexSidetable<OperationState::Liveness>,
             ZoneMap<uint32_t, BlockIndex>>
   Run() {
     if constexpr (trace_analysis) {
@@ -255,7 +256,10 @@ class DeadCodeAnalysis {
       if constexpr (trace_analysis) std::cout << index << ":" << op << "\n";
       OperationState::Liveness op_state = liveness_[index];
 
-      if (op.Is<BranchOp>()) {
+      if (op.Is<CallOp>()) {
+        // The function contains a call, so it's not a leaf function.
+        is_leaf_function_ = false;
+      } else if (op.Is<BranchOp>()) {
         if (control_state != ControlState::NotEliminatable()) {
           // Branch is still dead.
           DCHECK_EQ(op_state, OperationState::kDead);
@@ -273,7 +277,7 @@ class DeadCodeAnalysis {
             rewritable_branch_targets_.erase(it);
           }
         }
-      } else if (op.saturated_use_count == 0) {
+      } else if (op.saturated_use_count.IsZero()) {
         // Operation is already recognized as dead by a previous analysis.
         DCHECK_EQ(op_state, OperationState::kDead);
       } else if (op.Is<GotoOp>()) {
@@ -281,7 +285,7 @@ class DeadCodeAnalysis {
         // state, so we skip them here.
         liveness_[index] = OperationState::kLive;
         continue;
-      } else if (op.Properties().observable_when_unused) {
+      } else if (op.IsRequiredWhenUnused()) {
         op_state = OperationState::kLive;
       } else if (op.Is<PhiOp>()) {
         has_live_phis = has_live_phis || (op_state == OperationState::kLive);
@@ -404,11 +408,17 @@ class DeadCodeAnalysis {
     entry_control_state_[block.index()] = control_state;
   }
 
+  bool is_leaf_function() const { return is_leaf_function_; }
+
  private:
   Graph& graph_;
-  FixedSidetable<OperationState::Liveness> liveness_;
+  FixedOpIndexSidetable<OperationState::Liveness> liveness_;
   FixedBlockSidetable<ControlState> entry_control_state_;
   ZoneMap<uint32_t, BlockIndex> rewritable_branch_targets_;
+  // The stack check at function entry of leaf functions can be eliminated, as
+  // it is guaranteed that another stack check will be hit eventually. This flag
+  // records if the current function is a leaf function.
+  bool is_leaf_function_ = true;
 };
 
 template <class Next>
@@ -419,12 +429,6 @@ class DeadCodeEliminationReducer
 
   using Adapter = UniformReducerAdapter<DeadCodeEliminationReducer, Next>;
 
-  template <class... Args>
-  explicit DeadCodeEliminationReducer(const std::tuple<Args...>& args)
-      : Adapter(args),
-        branch_rewrite_targets_(Asm().phase_zone()),
-        analyzer_(Asm().modifiable_input_graph(), Asm().phase_zone()) {}
-
   void Analyze() {
     // TODO(nicohartmann@): We might want to make this a flag.
     constexpr bool trace_analysis = false;
@@ -437,7 +441,7 @@ class DeadCodeEliminationReducer
     auto it = branch_rewrite_targets_.find(ig_index.id());
     if (it != branch_rewrite_targets_.end()) {
       BlockIndex goto_target = it->second;
-      Asm().Goto(Asm().input_graph().Get(goto_target).MapToNextGraph());
+      Asm().Goto(Asm().MapToNewGraph(&Asm().input_graph().Get(goto_target)));
       return OpIndex::Invalid();
     }
     return Next::ReduceInputGraphBranch(ig_index, branch);
@@ -451,15 +455,13 @@ class DeadCodeEliminationReducer
     return Continuation{this}.ReduceInputGraph(ig_index, op);
   }
 
-  template <Opcode opcode, typename Continuation, typename... Args>
-  OpIndex ReduceOperation(const Args&... args) {
-    return Continuation{this}.Reduce(args...);
-  }
+  bool IsLeafFunction() const { return analyzer_.is_leaf_function(); }
 
  private:
-  base::Optional<FixedSidetable<OperationState::Liveness>> liveness_;
-  ZoneMap<uint32_t, BlockIndex> branch_rewrite_targets_;
-  DeadCodeAnalysis analyzer_;
+  base::Optional<FixedOpIndexSidetable<OperationState::Liveness>> liveness_;
+  ZoneMap<uint32_t, BlockIndex> branch_rewrite_targets_{Asm().phase_zone()};
+  DeadCodeAnalysis analyzer_{Asm().modifiable_input_graph(),
+                             Asm().phase_zone()};
 };
 
 #include "src/compiler/turboshaft/undef-assembler-macros.inc"
