@@ -8,33 +8,21 @@
 #include "include/v8-fast-api-calls.h"
 #include "src/compiler/fast-api-calls.h"
 #include "src/compiler/turboshaft/assembler.h"
+#include "src/compiler/turboshaft/copying-phase.h"
 #include "src/compiler/turboshaft/index.h"
 #include "src/compiler/turboshaft/operations.h"
-#include "src/compiler/turboshaft/optimization-phase.h"
+#include "src/compiler/turboshaft/phase.h"
+#include "src/compiler/turboshaft/representations.h"
 
 namespace v8::internal::compiler::turboshaft {
 
 #include "src/compiler/turboshaft/define-assembler-macros.inc"
 
-struct FastApiCallReducerArgs {
-  Factory* factory;
-  Isolate* isolate;
-};
-
 template <typename Next>
 class FastApiCallReducer : public Next {
  public:
   TURBOSHAFT_REDUCER_BOILERPLATE()
 
-  using ArgT =
-      base::append_tuple_type<typename Next::ArgT, FastApiCallReducerArgs>;
-
-  template <typename... Args>
-  explicit FastApiCallReducer(const std::tuple<Args...>& args)
-      : Next(args),
-        factory_(std::get<FastApiCallReducerArgs>(args).factory),
-        isolate_(std::get<FastApiCallReducerArgs>(args).isolate) {}
-
   OpIndex REDUCE(FastApiCall)(OpIndex data_argument,
                               base::Vector<const OpIndex> arguments,
                               const FastApiCallParameters* parameters) {
@@ -102,7 +90,7 @@ class FastApiCallReducer : public Next {
       // {data_argument}.
       OpIndex data_stack_slot =
           __ StackSlot(sizeof(uintptr_t), alignof(uintptr_t));
-      __ StoreOffHeap(data_stack_slot, __ BitcastTaggedToWord(data_argument),
+      __ StoreOffHeap(data_stack_slot, __ BitcastTaggedToWordPtr(data_argument),
                       MemoryRepresentation::PointerSized());
       // data = data_stack_slot
       __ StoreOffHeap(stack_slot, data_stack_slot,
@@ -120,7 +108,7 @@ class FastApiCallReducer : public Next {
     // Build the actual call.
     const TSCallDescriptor* call_descriptor = TSCallDescriptor::Create(
         Linkage::GetSimplifiedCDescriptor(__ graph_zone(), builder.Build()),
-        __ graph_zone());
+        CanThrow::kNo, __ graph_zone());
     OpIndex c_call_result =
         WrapFastCall(call_descriptor, callee, base::VectorOf(args));
     V<Object> fast_call_result = ConvertReturnValue(c_signature, c_call_result);
@@ -136,9 +124,9 @@ class FastApiCallReducer : public Next {
     GOTO(done, FastApiCallOp::kSuccessValue, fast_call_result);
 
     if (BIND(handle_error)) {
-      // We pass Smi(0) as the value here, although this should never be visible
-      // when calling code reacts to `kFailureValue` properly.
-      GOTO(done, FastApiCallOp::kFailureValue, __ SmiTag(0));
+      // We pass Tagged<Smi>(0) as the value here, although this should never be
+      // visible when calling code reacts to `kFailureValue` properly.
+      GOTO(done, FastApiCallOp::kFailureValue, __ TagSmi(0));
     }
 
     BIND(done, state, value);
@@ -173,7 +161,7 @@ class FastApiCallReducer : public Next {
 
           OpIndex stack_slot =
               __ StackSlot(sizeof(uintptr_t), alignof(uintptr_t));
-          __ StoreOffHeap(stack_slot, __ BitcastTaggedToWord(argument),
+          __ StoreOffHeap(stack_slot, __ BitcastHeapObjectToWordPtr(argument),
                           MemoryRepresentation::PointerSized());
           OpIndex target_address = __ ExternalConstant(
               ExternalReference::Create(c_functions[func_index].address,
@@ -253,7 +241,7 @@ class FastApiCallReducer : public Next {
             case CTypeInfo::Type::kV8Value: {
               OpIndex stack_slot =
                   __ StackSlot(sizeof(uintptr_t), alignof(uintptr_t));
-              __ StoreOffHeap(stack_slot, __ BitcastTaggedToWord(argument),
+              __ StoreOffHeap(stack_slot, __ BitcastTaggedToWordPtr(argument),
                               MemoryRepresentation::PointerSized());
               return stack_slot;
             }
@@ -286,8 +274,9 @@ class FastApiCallReducer : public Next {
             case CTypeInfo::Type::kSeqOneByteString: {
               // Check that the value is a HeapObject.
               GOTO_IF(__ ObjectIsSmi(argument), handle_error);
+              V<HeapObject> argument_obj = V<HeapObject>::Cast(argument);
 
-              V<Map> map = __ LoadMapField(argument);
+              V<Map> map = __ LoadMapField(argument_obj);
               V<Word32> instance_type = __ LoadInstanceTypeField(map);
 
               V<Word32> encoding = __ Word32BitwiseAnd(
@@ -296,11 +285,9 @@ class FastApiCallReducer : public Next {
                           handle_error);
 
               V<WordPtr> length_in_bytes = __ template LoadField<WordPtr>(
-                  V<HeapObject>::Cast(argument),
-                  AccessBuilder::ForStringLength());
-              OpIndex data_ptr = __ WordPtrAdd(
-                  __ BitcastTaggedToWord(argument),
-                  (SeqOneByteString::kHeaderSize - kHeapObjectTag));
+                  argument_obj, AccessBuilder::ForStringLength());
+              V<WordPtr> data_ptr = __ GetElementStartPointer(
+                  argument_obj, AccessBuilder::ForSeqOneByteStringCharacter());
 
               constexpr int kAlign = alignof(FastOneByteString);
               constexpr int kSize = sizeof(FastOneByteString);
@@ -337,7 +324,7 @@ class FastApiCallReducer : public Next {
 
         OpIndex stack_slot =
             __ StackSlot(sizeof(uintptr_t), alignof(uintptr_t));
-        __ StoreOffHeap(stack_slot, __ BitcastTaggedToWord(argument),
+        __ StoreOffHeap(stack_slot, __ BitcastHeapObjectToWordPtr(argument),
                         MemoryRepresentation::PointerSized());
 
         return stack_slot;
@@ -453,14 +440,14 @@ class FastApiCallReducer : public Next {
     } else {
       V<Object> base_pointer = __ template LoadField<Object>(
           argument, AccessBuilder::ForJSTypedArrayBasePointer());
-      V<WordPtr> base = __ BitcastTaggedToWord(base_pointer);
+      V<WordPtr> base = __ BitcastTaggedToWordPtr(base_pointer);
       if (COMPRESS_POINTERS_BOOL) {
         // Zero-extend Tagged_t to UintPtr according to current compression
         // scheme so that the addition with |external_pointer| (which already
         // contains compensated offset value) will decompress the tagged value.
         // See JSTypedArray::ExternalPointerCompensationForOnHeapArray() for
         // details.
-        base = __ ChangeUint32ToUintPtr(base);
+        base = __ ChangeUint32ToUintPtr(__ TruncateWordPtrToWord32(base));
       }
       data_ptr = __ WordPtrAdd(base, external_pointer);
     }
@@ -506,9 +493,44 @@ class FastApiCallReducer : public Next {
         return __ ConvertInt32ToNumber(result);
       case CTypeInfo::Type::kUint32:
         return __ ConvertUint32ToNumber(result);
-      case CTypeInfo::Type::kInt64:
-      case CTypeInfo::Type::kUint64:
-        UNREACHABLE();
+      case CTypeInfo::Type::kInt64: {
+        CFunctionInfo::Int64Representation repr =
+            c_signature->GetInt64Representation();
+        if (repr == CFunctionInfo::Int64Representation::kBigInt) {
+          return __ ConvertUntaggedToJSPrimitive(
+              result, ConvertUntaggedToJSPrimitiveOp::JSPrimitiveKind::kBigInt,
+              RegisterRepresentation::Word64(),
+              ConvertUntaggedToJSPrimitiveOp::InputInterpretation::kSigned,
+              CheckForMinusZeroMode::kDontCheckForMinusZero);
+        } else if (repr == CFunctionInfo::Int64Representation::kNumber) {
+          return __ ConvertUntaggedToJSPrimitive(
+              result, ConvertUntaggedToJSPrimitiveOp::JSPrimitiveKind::kNumber,
+              RegisterRepresentation::Word64(),
+              ConvertUntaggedToJSPrimitiveOp::InputInterpretation::kSigned,
+              CheckForMinusZeroMode::kDontCheckForMinusZero);
+        } else {
+          UNREACHABLE();
+        }
+      }
+      case CTypeInfo::Type::kUint64: {
+        CFunctionInfo::Int64Representation repr =
+            c_signature->GetInt64Representation();
+        if (repr == CFunctionInfo::Int64Representation::kBigInt) {
+          return __ ConvertUntaggedToJSPrimitive(
+              result, ConvertUntaggedToJSPrimitiveOp::JSPrimitiveKind::kBigInt,
+              RegisterRepresentation::Word64(),
+              ConvertUntaggedToJSPrimitiveOp::InputInterpretation::kUnsigned,
+              CheckForMinusZeroMode::kDontCheckForMinusZero);
+        } else if (repr == CFunctionInfo::Int64Representation::kNumber) {
+          return __ ConvertUntaggedToJSPrimitive(
+              result, ConvertUntaggedToJSPrimitiveOp::JSPrimitiveKind::kNumber,
+              RegisterRepresentation::Word64(),
+              ConvertUntaggedToJSPrimitiveOp::InputInterpretation::kUnsigned,
+              CheckForMinusZeroMode::kDontCheckForMinusZero);
+        } else {
+          UNREACHABLE();
+        }
+      }
       case CTypeInfo::Type::kFloat32:
         return __ ConvertFloat64ToNumber(
             __ ChangeFloat32ToFloat64(result),
@@ -537,16 +559,16 @@ class FastApiCallReducer : public Next {
     GOTO_IF(__ WordPtrEqual(pointer, 0), done,
             __ HeapConstant(factory_->null_value()));
 
-    V<HeapObject> external =
+    Uninitialized<HeapObject> external =
         __ Allocate(JSExternalObject::kHeaderSize, AllocationType::kYoung);
-    __ StoreField(external, AccessBuilder::ForMap(),
-                  __ HeapConstant(factory_->external_map()));
+    __ InitializeField(external, AccessBuilder::ForMap(),
+                       __ HeapConstant(factory_->external_map()));
     V<FixedArray> empty_fixed_array =
         __ HeapConstant(factory_->empty_fixed_array());
-    __ StoreField(external, AccessBuilder::ForJSObjectPropertiesOrHash(),
-                  empty_fixed_array);
-    __ StoreField(external, AccessBuilder::ForJSObjectElements(),
-                  empty_fixed_array);
+    __ InitializeField(external, AccessBuilder::ForJSObjectPropertiesOrHash(),
+                       empty_fixed_array);
+    __ InitializeField(external, AccessBuilder::ForJSObjectElements(),
+                       empty_fixed_array);
 
 #ifdef V8_ENABLE_SANDBOX
     OpIndex isolate_ptr =
@@ -564,13 +586,15 @@ class FastApiCallReducer : public Next {
     OpIndex handle =
         __ Call(allocate_and_initialize_external_pointer_table_entry,
                 {isolate_ptr, pointer},
-                TSCallDescriptor::Create(call_descriptor, __ graph_zone()));
-    __ StoreField(external, AccessBuilder::ForJSExternalObjectPointerHandle(),
-                  handle);
+                TSCallDescriptor::Create(call_descriptor, CanThrow::kNo,
+                                         __ graph_zone()));
+    __ InitializeField(
+        external, AccessBuilder::ForJSExternalObjectPointerHandle(), handle);
 #else
-    __ StoreField(external, AccessBuilder::ForJSExternalObjectValue(), pointer);
+    __ InitializeField(external, AccessBuilder::ForJSExternalObjectValue(),
+                       pointer);
 #endif  // V8_ENABLE_SANDBOX
-    GOTO(done, external);
+    GOTO(done, __ FinishInitialization(std::move(external)));
 
     BIND(done, result);
     return result;
@@ -581,7 +605,7 @@ class FastApiCallReducer : public Next {
     // CPU profiler support.
     OpIndex target_address = __ ExternalConstant(
         ExternalReference::fast_api_call_target_address(isolate_));
-    __ StoreOffHeap(target_address, __ BitcastTaggedToWord(callee),
+    __ StoreOffHeap(target_address, __ BitcastHeapObjectToWordPtr(callee),
                     MemoryRepresentation::PointerSized());
 
     // Disable JS execution.
@@ -614,8 +638,8 @@ class FastApiCallReducer : public Next {
     return result;
   }
 
-  Factory* factory_;
-  Isolate* isolate_;
+  Isolate* isolate_ = PipelineData::Get().isolate();
+  Factory* factory_ = isolate_->factory();
 };
 
 #include "src/compiler/turboshaft/undef-assembler-macros.inc"
