@@ -72,9 +72,10 @@ class OperationBuffer {
 
   explicit OperationBuffer(Zone* zone, size_t initial_capacity) : zone_(zone) {
     DCHECK_NE(initial_capacity, 0);
-    begin_ = end_ = zone_->NewArray<OperationStorageSlot>(initial_capacity);
+    begin_ = end_ =
+        zone_->AllocateArray<OperationStorageSlot>(initial_capacity);
     operation_sizes_ =
-        zone_->NewArray<uint16_t>((initial_capacity + 1) / kSlotsPerId);
+        zone_->AllocateArray<uint16_t>((initial_capacity + 1) / kSlotsPerId);
     end_cap_ = begin_ + initial_capacity;
   }
 
@@ -163,11 +164,11 @@ class OperationBuffer {
                                sizeof(OperationStorageSlot));
 
     OperationStorageSlot* new_buffer =
-        zone_->NewArray<OperationStorageSlot>(new_capacity);
+        zone_->AllocateArray<OperationStorageSlot>(new_capacity);
     memcpy(new_buffer, begin_, size * sizeof(OperationStorageSlot));
 
     uint16_t* new_operation_sizes =
-        zone_->NewArray<uint16_t>(new_capacity / kSlotsPerId);
+        zone_->AllocateArray<uint16_t>(new_capacity / kSlotsPerId);
     memcpy(new_operation_sizes, operation_sizes_,
            size / kSlotsPerId * sizeof(uint16_t));
 
@@ -237,7 +238,7 @@ class RandomAccessStackDominatorNode
  public:
   void SetDominator(Derived* dominator);
   void SetAsDominatorRoot();
-  Derived* GetDominator() { return nxt_; }
+  Derived* GetDominator() const { return nxt_; }
 
   // Returns the lowest common dominator of {this} and {other}.
   Derived* GetCommonDominator(
@@ -267,17 +268,49 @@ class RandomAccessStackDominatorNode
   Derived* jmp_ = nullptr;
 };
 
+// A simple iterator to walk over the predecessors of a block. Note that the
+// iteration order is reversed.
+class PredecessorIterator {
+ public:
+  explicit PredecessorIterator(Block* block) : current_(block) {}
+
+  PredecessorIterator& operator++();
+  constexpr bool operator==(const PredecessorIterator& other) const {
+    return current_ == other.current_;
+  }
+  constexpr bool operator!=(const PredecessorIterator& other) const {
+    return !(*this == other);
+  }
+
+  Block* operator*() const { return current_; }
+
+ private:
+  Block* current_;
+};
+
+// An iterable wrapper for the predecessors of a block.
+class NeighboringPredecessorIterable {
+ public:
+  explicit NeighboringPredecessorIterable(Block* begin) : begin_(begin) {}
+
+  PredecessorIterator begin() const { return PredecessorIterator(begin_); }
+  PredecessorIterator end() const { return PredecessorIterator(nullptr); }
+
+ private:
+  Block* begin_;
+};
+
 // A basic block
 class Block : public RandomAccessStackDominatorNode<Block> {
  public:
   enum class Kind : uint8_t { kMerge, kLoopHeader, kBranchTarget };
 
+  explicit Block(Kind kind) : kind_(kind) {}
+
   bool IsLoopOrMerge() const { return IsLoop() || IsMerge(); }
   bool IsLoop() const { return kind_ == Kind::kLoopHeader; }
   bool IsMerge() const { return kind_ == Kind::kMerge; }
   bool IsBranchTarget() const { return kind_ == Kind::kBranchTarget; }
-  bool IsHandler() const { return false; }
-  bool IsSwitchCase() const { return false; }
 
   Kind kind() const { return kind_; }
   void SetKind(Kind kind) { kind_ = kind; }
@@ -300,17 +333,31 @@ class Block : public RandomAccessStackDominatorNode<Block> {
     return result;
   }
 
-  // TODO(dmercadier): we should store predecessor count in the Blocks directly
-  // (or in the Graph, or in the Assembler), to avoid this O(n) PredecessorCount
-  // method.
+  // Returns an iterable object (defining begin() and end()) to iterate over the
+  // block's predecessors.
+  NeighboringPredecessorIterable PredecessorsIterable() const {
+    return NeighboringPredecessorIterable(last_predecessor_);
+  }
+
   int PredecessorCount() const {
+#ifdef DEBUG
+    CheckPredecessorCount();
+#endif
+    return predecessor_count_;
+  }
+
+#ifdef DEBUG
+  // Checks that the {predecessor_count_} is equal to the number of predecessors
+  // reachable through {last_predecessor_}.
+  void CheckPredecessorCount() const {
     int count = 0;
     for (Block* pred = last_predecessor_; pred != nullptr;
          pred = pred->neighboring_predecessor_) {
       count++;
     }
-    return count;
+    DCHECK_EQ(count, predecessor_count_);
   }
+#endif
 
   // Returns the index of {target} in the predecessors of the current Block.
   // If {target} is not a direct predecessor, returns -1.
@@ -331,36 +378,20 @@ class Block : public RandomAccessStackDominatorNode<Block> {
     return pred_count - pred_reverse_index - 1;
   }
 
-  // HasExactlyNPredecessors(n) returns the same result as
-  // `PredecessorCount() == n`, but stops early and iterates at most the first
-  // {n} predecessors.
-  bool HasExactlyNPredecessors(unsigned int n) const {
-    Block* current_pred = last_predecessor_;
-    while (current_pred != nullptr && n != 0) {
-      current_pred = current_pred->neighboring_predecessor_;
-      n--;
-    }
-    return n == 0 && current_pred == nullptr;
-  }
-
   Block* LastPredecessor() const { return last_predecessor_; }
   Block* NeighboringPredecessor() const { return neighboring_predecessor_; }
-  bool HasPredecessors() const { return last_predecessor_ != nullptr; }
-  void ResetLastPredecessor() { last_predecessor_ = nullptr; }
+  bool HasPredecessors() const {
+    DCHECK_EQ(predecessor_count_ == 0, last_predecessor_ == nullptr);
+    return last_predecessor_ != nullptr;
+  }
+  void ResetLastPredecessor() {
+    last_predecessor_ = nullptr;
+    predecessor_count_ = 0;
+  }
 
-  void SetMappingToNextGraph(Block* next_graph_block) {
-    DCHECK_NULL(next_graph_mapping_);
-    DCHECK_NOT_NULL(next_graph_block);
-    next_graph_mapping_ = next_graph_block;
-    next_graph_block->SetOrigin(this);
-  }
-  Block* MapToNextGraph() const {
-    DCHECK_NOT_NULL(next_graph_mapping_);
-    return next_graph_mapping_;
-  }
   // The block from the previous graph which produced the current block. This
   // has to be updated to be the last block that contributed operations to the
-  // current block to ensure that phi nodes are created correctly.git cl
+  // current block to ensure that phi nodes are created correctly.
   void SetOrigin(const Block* origin) {
     DCHECK_IMPLIES(origin != nullptr,
                    origin->graph_generation_ + 1 == graph_generation_);
@@ -373,19 +404,8 @@ class Block : public RandomAccessStackDominatorNode<Block> {
     DCHECK(IsBound());
     return origin_;
   }
-  // The block from the input graph that corresponds to the current block as a
-  // branch destination. Such a block might not exist, and this function uses a
-  // trick to compute such a block in almost all cases, but might rarely fail
-  // and return `nullptr` instead.
-  const Block* OriginForBlockStart() const {
-    // Check that `origin_` is still valid as a block start and was not changed
-    // to a semantically different block when inlining blocks.
-    if (origin_ && origin_->MapToNextGraph() == this) {
-      return origin_;
-    }
-    return nullptr;
-  }
 
+  bool IsComplete() const { return end_.valid(); }
   OpIndex begin() const {
     DCHECK(begin_.valid());
     return begin_;
@@ -395,7 +415,6 @@ class Block : public RandomAccessStackDominatorNode<Block> {
     return end_;
   }
 
-  // Might return nullptr if the first operation is invalid.
   const Operation& FirstOperation(const Graph& graph) const;
   const Operation& LastOperation(const Graph& graph) const;
 
@@ -403,15 +422,39 @@ class Block : public RandomAccessStackDominatorNode<Block> {
     switch (LastOperation(graph).opcode) {
       case Opcode::kBranch:
       case Opcode::kSwitch:
-      case Opcode::kCallAndCatchException:
+      case Opcode::kCheckException:
         return true;
       default:
+        DCHECK_LE(SuccessorBlocks(*this, graph).size(), 1);
         return false;
     }
   }
 
   bool HasPhis(const Graph& graph) const;
 
+  bool HasBackedge(const Graph& graph) const {
+    if (const GotoOp* gto = LastOperation(graph).TryCast<GotoOp>()) {
+      return gto->destination->index().id() < index().id();
+    }
+    return false;
+  }
+
+#ifdef DEBUG
+  // {has_peeled_iteration_} is currently only updated for loops peeled in
+  // Turboshaft (it is true only for loop headers of loops that have had their
+  // first iteration peeled). So be aware that while Turbofan loop peeling is
+  // enabled, this is not a reliable way to check if a loop has a peeled
+  // iteration.
+  bool has_peeled_iteration() const {
+    DCHECK(IsLoop());
+    return has_peeled_iteration_;
+  }
+  void set_has_peeled_iteration() {
+    DCHECK(IsLoop());
+    has_peeled_iteration_ = true;
+  }
+#endif
+
   // Computes the dominators of the this block, assuming that the dominators of
   // its predecessors are already computed. Returns the depth of the current
   // block in the dominator tree.
@@ -421,10 +464,30 @@ class Block : public RandomAccessStackDominatorNode<Block> {
       std::vector<const char*> tree_symbols = std::vector<const char*>(),
       bool has_next = false) const;
 
-  explicit Block(Kind kind) : kind_(kind) {}
+  enum class CustomDataKind {
+    kUnset,  // No custom data has been set for this block.
+    kPhiInputIndex,
+    kDeferredInSchedule,
+  };
 
-  uint32_t& custom_data() { return custom_data_; }
-  const uint32_t& custom_data() const { return custom_data_; }
+  void set_custom_data(uint32_t data, CustomDataKind kind_for_debug_check) {
+    custom_data_ = data;
+#ifdef DEBUG
+    custom_data_kind_for_debug_check_ = kind_for_debug_check;
+#endif
+  }
+
+  uint32_t get_custom_data(CustomDataKind kind_for_debug_check) const {
+    DCHECK_EQ(custom_data_kind_for_debug_check_, kind_for_debug_check);
+    return custom_data_;
+  }
+
+  void clear_custom_data() {
+    custom_data_ = 0;
+#ifdef DEBUG
+    custom_data_kind_for_debug_check_ = CustomDataKind::kUnset;
+#endif
+  }
 
  private:
   // AddPredecessor should never be called directly except from Assembler's
@@ -436,6 +499,7 @@ class Block : public RandomAccessStackDominatorNode<Block> {
     DCHECK_EQ(predecessor->neighboring_predecessor_, nullptr);
     predecessor->neighboring_predecessor_ = last_predecessor_;
     last_predecessor_ = predecessor;
+    predecessor_count_++;
   }
 
   friend class Graph;
@@ -448,19 +512,31 @@ class Block : public RandomAccessStackDominatorNode<Block> {
   BlockIndex index_ = BlockIndex::Invalid();
   Block* last_predecessor_ = nullptr;
   Block* neighboring_predecessor_ = nullptr;
+  uint32_t predecessor_count_ = 0;
   const Block* origin_ = nullptr;
-  Block* next_graph_mapping_ = nullptr;
   // The {custom_data_} field can be used by algorithms to temporarily store
   // block-specific data. This field is not preserved when constructing a new
   // output graph and algorithms cannot rely on this field being properly reset
   // after previous uses.
   uint32_t custom_data_ = 0;
 #ifdef DEBUG
+  CustomDataKind custom_data_kind_for_debug_check_ = CustomDataKind::kUnset;
   size_t graph_generation_ = 0;
+  // True if this is a loop header of a loop with a peeled iteration.
+  bool has_peeled_iteration_ = false;
 #endif
+
+  template <class Assembler>
+  friend class GraphVisitor;
 };
 
-std::ostream& operator<<(std::ostream& os, const Block* b);
+V8_EXPORT_PRIVATE std::ostream& operator<<(std::ostream& os, const Block* b);
+
+inline PredecessorIterator& PredecessorIterator::operator++() {
+  DCHECK_NE(current_, nullptr);
+  current_ = current_->NeighboringPredecessor();
+  return *this;
+}
 
 class Graph {
  public:
@@ -469,11 +545,12 @@ class Graph {
   explicit Graph(Zone* graph_zone, size_t initial_capacity = 2048)
       : operations_(graph_zone, initial_capacity),
         bound_blocks_(graph_zone),
-        all_blocks_(graph_zone),
+        all_blocks_(),
+        block_permutation_(graph_zone),
         graph_zone_(graph_zone),
-        source_positions_(graph_zone),
-        operation_origins_(graph_zone),
-        operation_types_(graph_zone)
+        source_positions_(graph_zone, this),
+        operation_origins_(graph_zone, this),
+        operation_types_(graph_zone, this)
 #ifdef DEBUG
         ,
         block_type_refinement_(graph_zone)
@@ -485,10 +562,13 @@ class Graph {
   void Reset() {
     operations_.Reset();
     bound_blocks_.clear();
+    // No need to explicitly reset `all_blocks_`, since we will placement-new
+    // new blocks into it, reusing the already allocated backing storage.
+    next_block_ = 0;
+    block_permutation_.clear();
     source_positions_.Reset();
     operation_origins_.Reset();
     operation_types_.Reset();
-    next_block_ = 0;
     dominator_tree_depth_ = 0;
 #ifdef DEBUG
     block_type_refinement_.Reset();
@@ -496,6 +576,8 @@ class Graph {
   }
 
   V8_INLINE const Operation& Get(OpIndex i) const {
+    DCHECK(i.valid());
+    DCHECK(BelongsToThisGraph(i));
     // `Operation` contains const fields and can be overwritten with placement
     // new. Therefore, std::launder is necessary to avoid undefined behavior.
     const Operation* ptr =
@@ -505,6 +587,8 @@ class Graph {
     return *ptr;
   }
   V8_INLINE Operation& Get(OpIndex i) {
+    DCHECK(i.valid());
+    DCHECK(BelongsToThisGraph(i));
     // `Operation` contains const fields and can be overwritten with placement
     // new. Therefore, std::launder is necessary to avoid undefined behavior.
     Operation* ptr =
@@ -514,8 +598,9 @@ class Graph {
     return *ptr;
   }
 
-  void MarkAsUnused(OpIndex i) { Get(i).saturated_use_count = 0; }
+  void MarkAsUnused(OpIndex i) { Get(i).saturated_use_count.SetToZero(); }
 
+  Block& StartBlock() { return Get(BlockIndex(0)); }
   const Block& StartBlock() const { return Get(BlockIndex(0)); }
 
   Block& Get(BlockIndex i) {
@@ -527,19 +612,44 @@ class Graph {
     return *bound_blocks_[i.id()];
   }
 
-  OpIndex Index(const Operation& op) const { return operations_.Index(op); }
+  OpIndex Index(const Operation& op) const {
+    OpIndex result = operations_.Index(op);
+#ifdef DEBUG
+    result.set_generation_mod2(generation_mod2());
+#endif
+    return result;
+  }
   BlockIndex BlockOf(OpIndex index) const {
-    auto it = std::upper_bound(
-        bound_blocks_.begin(), bound_blocks_.end(), index,
-        [](OpIndex value, const Block* b) { return value < b->begin_; });
-    DCHECK_NE(it, bound_blocks_.begin());
-    --it;
+    ZoneVector<Block*>::const_iterator it;
+    if (block_permutation_.empty()) {
+      it = std::upper_bound(
+          bound_blocks_.begin(), bound_blocks_.end(), index,
+          [](OpIndex value, const Block* b) { return value < b->begin_; });
+      DCHECK_NE(it, bound_blocks_.begin());
+    } else {
+      it = std::upper_bound(
+          block_permutation_.begin(), block_permutation_.end(), index,
+          [](OpIndex value, const Block* b) { return value < b->begin_; });
+      DCHECK_NE(it, block_permutation_.begin());
+    }
+    it = std::prev(it);
+    DCHECK((*it)->Contains(index));
     return (*it)->index();
   }
 
-  OpIndex NextIndex(const OpIndex idx) const { return operations_.Next(idx); }
+  OpIndex NextIndex(const OpIndex idx) const {
+    OpIndex next = operations_.Next(idx);
+#ifdef DEBUG
+    next.set_generation_mod2(generation_mod2());
+#endif
+    return next;
+  }
   OpIndex PreviousIndex(const OpIndex idx) const {
-    return operations_.Previous(idx);
+    OpIndex prev = operations_.Previous(idx);
+#ifdef DEBUG
+    prev.set_generation_mod2(generation_mod2());
+#endif
+    return prev;
   }
 
   OperationStorageSlot* Allocate(size_t slot_count) {
@@ -549,6 +659,11 @@ class Graph {
   void RemoveLast() {
     DecrementInputUses(*AllOperations().rbegin());
     operations_.RemoveLast();
+#ifdef DEBUG
+    if (v8_flags.turboshaft_trace_emitted) {
+      std::cout << "/!\\ Removed last emitted operation /!\\\n";
+    }
+#endif
   }
 
   template <class Op, class... Args>
@@ -559,21 +674,28 @@ class Graph {
     Op& op = Op::New(this, args...);
     IncrementInputUses(op);
 
-    if (op.Properties().observable_when_unused) {
+    if (op.IsRequiredWhenUnused()) {
       // Once the graph is built, an operation with a `saturated_use_count` of 0
       // is guaranteed to be unused and can be removed. Thus, to avoid removing
       // operations that never have uses (such as Goto or Branch), we set the
-      // `saturated_use_count` of Operations that are `required_when_unused`
+      // `saturated_use_count` of Operations that are `IsRequiredWhenUnused()`
       // to 1.
-      op.saturated_use_count = 1;
+      op.saturated_use_count.SetToOne();
     }
 
     DCHECK_EQ(result, Index(op));
 #ifdef DEBUG
     for (OpIndex input : op.inputs()) {
       DCHECK_LT(input, result);
+      DCHECK(BelongsToThisGraph(input));
     }
+
+    if (v8_flags.turboshaft_trace_emitted) {
+      std::cout << "Emitted: " << result << " => " << op << "\n";
+    }
+
 #endif  // DEBUG
+
     return op;
   }
 
@@ -594,15 +716,24 @@ class Graph {
     IncrementInputUses(*new_op);
   }
 
-  V8_INLINE Block* NewLoopHeader() {
-    return NewBlock(Block::Kind::kLoopHeader);
+  V8_INLINE Block* NewLoopHeader(const Block* origin = nullptr) {
+    return NewBlock(Block::Kind::kLoopHeader, origin);
   }
-  V8_INLINE Block* NewBlock() { return NewBlock(Block::Kind::kMerge); }
-  V8_INLINE Block* NewMappedBlock(Block* origin) {
-    Block* new_block = NewBlock(origin->IsLoop() ? Block::Kind::kLoopHeader
-                                                 : Block::Kind::kMerge);
-    origin->SetMappingToNextGraph(new_block);
-    return new_block;
+  V8_INLINE Block* NewBlock(const Block* origin = nullptr) {
+    return NewBlock(Block::Kind::kMerge, origin);
+  }
+
+  V8_INLINE Block* NewBlock(Block::Kind kind, const Block* origin = nullptr) {
+    if (V8_UNLIKELY(next_block_ == all_blocks_.size())) {
+      AllocateNewBlocks();
+    }
+    Block* result = all_blocks_[next_block_++];
+    new (result) Block(kind);
+#ifdef DEBUG
+    result->graph_generation_ = generation_;
+#endif
+    result->SetOrigin(origin);
+    return result;
   }
 
   V8_INLINE bool Add(Block* block) {
@@ -617,6 +748,12 @@ class Graph {
     uint32_t depth = block->ComputeDominator();
     dominator_tree_depth_ = std::max<uint32_t>(dominator_tree_depth_, depth);
 
+#ifdef DEBUG
+    if (v8_flags.turboshaft_trace_emitted) {
+      std::cout << "\nBound: " << block->index() << "\n";
+    }
+#endif
+
     return true;
   }
 
@@ -638,7 +775,7 @@ class Graph {
     }
   }
 
-  OpIndex next_operation_index() const { return operations_.EndIndex(); }
+  OpIndex next_operation_index() const { return EndIndex(); }
   BlockIndex next_block_index() const {
     return BlockIndex(static_cast<uint32_t>(bound_blocks_.size()));
   }
@@ -650,24 +787,47 @@ class Graph {
   uint32_t op_id_count() const {
     return (operations_.size() + (kSlotsPerId - 1)) / kSlotsPerId;
   }
+  uint32_t number_of_operations() const {
+    uint32_t number_of_operations = 0;
+    for ([[maybe_unused]] auto& op : AllOperations()) {
+      ++number_of_operations;
+    }
+    return number_of_operations;
+  }
   uint32_t op_id_capacity() const {
     return operations_.capacity() / kSlotsPerId;
   }
 
+  OpIndex BeginIndex() const {
+    OpIndex begin = operations_.BeginIndex();
+#ifdef DEBUG
+    begin.set_generation_mod2(generation_mod2());
+#endif
+    return begin;
+  }
+  OpIndex EndIndex() const {
+    OpIndex end = operations_.EndIndex();
+#ifdef DEBUG
+    end.set_generation_mod2(generation_mod2());
+#endif
+    return end;
+  }
+
   class OpIndexIterator
-      : public base::iterator<std::bidirectional_iterator_tag, OpIndex> {
+      : public base::iterator<std::bidirectional_iterator_tag, OpIndex,
+                              std::ptrdiff_t, OpIndex*, OpIndex> {
    public:
     using value_type = OpIndex;
 
     explicit OpIndexIterator(OpIndex index, const Graph* graph)
         : index_(index), graph_(graph) {}
-    value_type& operator*() { return index_; }
+    value_type operator*() const { return index_; }
     OpIndexIterator& operator++() {
-      index_ = graph_->operations_.Next(index_);
+      index_ = graph_->NextIndex(index_);
       return *this;
     }
     OpIndexIterator& operator--() {
-      index_ = graph_->operations_.Previous(index_);
+      index_ = graph_->PreviousIndex(index_);
       return *this;
     }
     bool operator!=(OpIndexIterator other) const {
@@ -693,11 +853,11 @@ class Graph {
         : index_(index), graph_(graph) {}
     value_type& operator*() { return graph_->Get(index_); }
     OperationIterator& operator++() {
-      index_ = graph_->operations_.Next(index_);
+      index_ = graph_->NextIndex(index_);
       return *this;
     }
     OperationIterator& operator--() {
-      index_ = graph_->operations_.Previous(index_);
+      index_ = graph_->PreviousIndex(index_);
       return *this;
     }
     bool operator!=(OperationIterator other) const {
@@ -716,14 +876,14 @@ class Graph {
       OperationIterator<const Operation, const Graph>;
 
   base::iterator_range<MutableOperationIterator> AllOperations() {
-    return operations(operations_.BeginIndex(), operations_.EndIndex());
+    return operations(BeginIndex(), EndIndex());
   }
   base::iterator_range<ConstOperationIterator> AllOperations() const {
-    return operations(operations_.BeginIndex(), operations_.EndIndex());
+    return operations(BeginIndex(), EndIndex());
   }
 
   base::iterator_range<OpIndexIterator> AllOperationIndices() const {
-    return OperationIndices(operations_.BeginIndex(), operations_.EndIndex());
+    return OperationIndices(BeginIndex(), EndIndex());
   }
 
   base::iterator_range<MutableOperationIterator> operations(
@@ -772,6 +932,7 @@ class Graph {
             base::DerefPtrIterator<const Block>(bound_blocks_.data() +
                                                 bound_blocks_.size())};
   }
+  const ZoneVector<Block*>& blocks_vector() const { return bound_blocks_; }
 
   bool IsLoopBackedge(const GotoOp& op) const {
     DCHECK(op.destination->IsBound());
@@ -780,23 +941,26 @@ class Graph {
 
   bool IsValid(OpIndex i) const { return i < next_operation_index(); }
 
-  const GrowingSidetable<SourcePosition>& source_positions() const {
+  const GrowingOpIndexSidetable<SourcePosition>& source_positions() const {
     return source_positions_;
   }
-  GrowingSidetable<SourcePosition>& source_positions() {
+  GrowingOpIndexSidetable<SourcePosition>& source_positions() {
     return source_positions_;
   }
 
-  const GrowingSidetable<OpIndex>& operation_origins() const {
+  const GrowingOpIndexSidetable<OpIndex>& operation_origins() const {
+    return operation_origins_;
+  }
+  GrowingOpIndexSidetable<OpIndex>& operation_origins() {
     return operation_origins_;
   }
-  GrowingSidetable<OpIndex>& operation_origins() { return operation_origins_; }
 
   uint32_t DominatorTreeDepth() const { return dominator_tree_depth_; }
-  const GrowingSidetable<Type>& operation_types() const {
+
+  const GrowingOpIndexSidetable<Type>& operation_types() const {
     return operation_types_;
   }
-  GrowingSidetable<Type>& operation_types() { return operation_types_; }
+  GrowingOpIndexSidetable<Type>& operation_types() { return operation_types_; }
 #ifdef DEBUG
   // Store refined types per block here for --trace-turbo printing.
   // TODO(nicohartmann@): Remove this once we have a proper way to print
@@ -810,9 +974,22 @@ class Graph {
   }
 #endif  // DEBUG
 
+  void ReorderBlocks(base::Vector<uint32_t> permutation) {
+    DCHECK_EQ(permutation.size(), bound_blocks_.size());
+    block_permutation_.resize(bound_blocks_.size());
+    std::swap(block_permutation_, bound_blocks_);
+
+    for (size_t i = 0; i < permutation.size(); ++i) {
+      DCHECK_LE(0, permutation[i]);
+      DCHECK_LT(permutation[i], block_permutation_.size());
+      bound_blocks_[i] = block_permutation_[permutation[i]];
+      bound_blocks_[i]->index_ = BlockIndex(static_cast<uint32_t>(i));
+    }
+  }
+
   Graph& GetOrCreateCompanion() {
     if (!companion_) {
-      companion_ = std::make_unique<Graph>(graph_zone_, operations_.size());
+      companion_ = graph_zone_->New<Graph>(graph_zone_, operations_.size());
 #ifdef DEBUG
       companion_->generation_ = generation_ + 1;
 #endif  // DEBUG
@@ -828,10 +1005,11 @@ class Graph {
     std::swap(bound_blocks_, companion.bound_blocks_);
     std::swap(all_blocks_, companion.all_blocks_);
     std::swap(next_block_, companion.next_block_);
+    std::swap(block_permutation_, companion.block_permutation_);
     std::swap(graph_zone_, companion.graph_zone_);
-    std::swap(source_positions_, companion.source_positions_);
-    std::swap(operation_origins_, companion.operation_origins_);
-    std::swap(operation_types_, companion.operation_types_);
+    source_positions_.SwapData(companion.source_positions_);
+    operation_origins_.SwapData(companion.operation_origins_);
+    operation_types_.SwapData(companion.operation_types_);
 #ifdef DEBUG
     std::swap(block_type_refinement_, companion.block_type_refinement_);
     // Update generation index.
@@ -842,6 +1020,11 @@ class Graph {
 
 #ifdef DEBUG
   size_t generation() const { return generation_; }
+  int generation_mod2() const { return generation_ % 2; }
+
+  bool BelongsToThisGraph(OpIndex idx) const {
+    return idx.generation_mod2() == generation_mod2();
+  }
 #endif  // DEBUG
 
  private:
@@ -855,60 +1038,70 @@ class Graph {
   template <class Op>
   void IncrementInputUses(const Op& op) {
     for (OpIndex input : op.inputs()) {
-      Operation& input_op = Get(input);
-      auto uses = input_op.saturated_use_count;
-      if (V8_LIKELY(uses != Operation::kUnknownUseCount)) {
-        input_op.saturated_use_count = uses + 1;
-      }
+      Get(input).saturated_use_count.Incr();
     }
   }
 
   template <class Op>
   void DecrementInputUses(const Op& op) {
     for (OpIndex input : op.inputs()) {
-      Operation& input_op = Get(input);
-      auto uses = input_op.saturated_use_count;
-      DCHECK_GT(uses, 0);
-      // Do not decrement if we already reached the threshold. In this case, we
-      // don't know the exact number of uses anymore and shouldn't assume
-      // anything.
-      if (V8_LIKELY(uses != Operation::kUnknownUseCount)) {
-        input_op.saturated_use_count = uses - 1;
-      }
+      Get(input).saturated_use_count.Decr();
     }
   }
 
-  V8_INLINE Block* NewBlock(Block::Kind kind) {
-    if (V8_UNLIKELY(next_block_ == all_blocks_.size())) {
-      constexpr size_t new_block_count = 64;
-      base::Vector<Block> blocks =
-          graph_zone_->NewVector<Block>(new_block_count, Block(kind));
-      for (size_t i = 0; i < new_block_count; ++i) {
-        all_blocks_.push_back(&blocks[i]);
-      }
+  // Allocates pointer-stable storage for new blocks, and pushes the pointers
+  // to that storage to `bound_blocks_`. Initialization of the blocks is defered
+  // to when they are actually constructed in `NewBlocks`.
+  V8_NOINLINE V8_PRESERVE_MOST void AllocateNewBlocks() {
+    constexpr size_t kMinCapacity = 32;
+    size_t next_capacity = std::max(kMinCapacity, all_blocks_.size() * 2);
+    size_t new_block_count = next_capacity - all_blocks_.size();
+    DCHECK_GT(new_block_count, 0);
+    base::Vector<Block> block_storage =
+        graph_zone_->AllocateVector<Block>(new_block_count);
+    base::Vector<Block*> new_all_blocks =
+        graph_zone_->AllocateVector<Block*>(next_capacity);
+    DCHECK_EQ(new_all_blocks.size(), all_blocks_.size() + new_block_count);
+    std::copy(all_blocks_.begin(), all_blocks_.end(), new_all_blocks.begin());
+    Block** insert_begin = new_all_blocks.begin() + all_blocks_.size();
+    DCHECK_EQ(insert_begin + new_block_count, new_all_blocks.end());
+    for (size_t i = 0; i < new_block_count; ++i) {
+      insert_begin[i] = &block_storage[i];
     }
-    Block* result = all_blocks_[next_block_++];
-    *result = Block(kind);
-#ifdef DEBUG
-    result->graph_generation_ = generation_;
-#endif
-    return result;
+    base::Vector<Block*> old_all_blocks = all_blocks_;
+    all_blocks_ = new_all_blocks;
+    if (!old_all_blocks.empty()) {
+      graph_zone_->DeleteArray(old_all_blocks.data(), old_all_blocks.length());
+    }
+
+    // Eventually most new blocks will be bound anyway, so pre-allocate as well.
+    DCHECK_LE(bound_blocks_.size(), all_blocks_.size());
+    bound_blocks_.reserve(all_blocks_.size());
   }
 
   OperationBuffer operations_;
   ZoneVector<Block*> bound_blocks_;
-  ZoneVector<Block*> all_blocks_;
+  // The next two fields essentially form a `ZoneVector` but with pointer
+  // stability for the `Block` elements. That is, `all_blocks_` contains
+  // pointers to (potentially non-contiguous) Zone-allocated `Block`s.
+  // Each pointer in `all_blocks_` points to already allocated space, but they
+  // are only properly value-initialized up to index `next_block_`.
+  base::Vector<Block*> all_blocks_;
   size_t next_block_ = 0;
+  // When `ReorderBlocks` is called, `block_permutation_` contains the original
+  // order of blocks in order to provide a proper OpIndex->Block mapping for
+  // `BlockOf`. In non-reordered graphs, this vector is empty.
+  ZoneVector<Block*> block_permutation_;
   Zone* graph_zone_;
-  GrowingSidetable<SourcePosition> source_positions_;
-  GrowingSidetable<OpIndex> operation_origins_;
+  GrowingOpIndexSidetable<SourcePosition> source_positions_;
+  GrowingOpIndexSidetable<OpIndex> operation_origins_;
   uint32_t dominator_tree_depth_ = 0;
-  GrowingSidetable<Type> operation_types_;
+  GrowingOpIndexSidetable<Type> operation_types_;
 #ifdef DEBUG
   GrowingBlockSidetable<TypeRefinements> block_type_refinement_;
 #endif
 
-  std::unique_ptr<Graph> companion_ = {};
+  Graph* companion_ = nullptr;
 #ifdef DEBUG
   size_t generation_ = 1;
 #endif  // DEBUG
@@ -936,43 +1129,32 @@ V8_INLINE const Operation& Block::LastOperation(const Graph& graph) const {
 }
 
 V8_INLINE bool Block::HasPhis(const Graph& graph) const {
+  // TODO(dmercadier): consider re-introducing the invariant that Phis are
+  // always at the begining of a block to speed up such functions. Currently,
+  // in practice, Phis do not appear after the first non-FrameState non-Constant
+  // operation, but this is not enforced.
   DCHECK_EQ(graph_generation_, graph.generation());
-#ifdef DEBUG
-  // Verify that only Phis/FrameStates are found, then all other Phis/
-  // FrameStateOps in the block come consecutively.
-  bool starts_with_phi = false;
-  bool finished_phis = false;
   for (const auto& op : graph.operations(*this)) {
-    if (op.Is<PhiOp>()) {
-      DCHECK(!finished_phis);
-      starts_with_phi = true;
-    }
-    if (!op.Is<PhiOp>() && !op.Is<FrameStateOp>()) {
-      finished_phis = true;
-    }
-  }
-  return starts_with_phi;
-#else   // DEBUG
-  for (const auto& op : graph.operations(*this)) {
-    if (op.Is<PhiOp>()) {
-      return true;
-    } else if (op.Is<FrameStateOp>()) {
-      continue;
-    } else {
-      return false;
-    }
+    if (op.Is<PhiOp>()) return true;
   }
   return false;
-#endif  // DEBUG
 }
 
 struct PrintAsBlockHeader {
   const Block& block;
-  BlockIndex block_id = block.index();
+  BlockIndex block_id;
+
+  explicit PrintAsBlockHeader(const Block& block)
+      : block(block), block_id(block.index()) {}
+  PrintAsBlockHeader(const Block& block, BlockIndex block_id)
+      : block(block), block_id(block_id) {}
 };
-std::ostream& operator<<(std::ostream& os, PrintAsBlockHeader block);
-std::ostream& operator<<(std::ostream& os, const Graph& graph);
-std::ostream& operator<<(std::ostream& os, const Block::Kind& kind);
+V8_EXPORT_PRIVATE std::ostream& operator<<(std::ostream& os,
+                                           PrintAsBlockHeader block);
+V8_EXPORT_PRIVATE std::ostream& operator<<(std::ostream& os,
+                                           const Graph& graph);
+V8_EXPORT_PRIVATE std::ostream& operator<<(std::ostream& os,
+                                           const Block::Kind& kind);
 
 inline uint32_t Block::ComputeDominator() {
   if (V8_UNLIKELY(LastPredecessor() == nullptr)) {
@@ -1077,4 +1259,12 @@ inline Derived* RandomAccessStackDominatorNode<Derived>::GetCommonDominator(
 
 }  // namespace v8::internal::compiler::turboshaft
 
+// MSVC needs this definition to know how to deal with the PredecessorIterator.
+template <>
+class std::iterator_traits<
+    v8::internal::compiler::turboshaft::PredecessorIterator> {
+ public:
+  using iterator_category = std::forward_iterator_tag;
+};
+
 #endif  // V8_COMPILER_TURBOSHAFT_GRAPH_H_
