@@ -11,8 +11,8 @@
 #include "src/base/vector.h"
 #include "src/compiler/common-operator.h"
 #include "src/compiler/turboshaft/assembler.h"
+#include "src/compiler/turboshaft/copying-phase.h"
 #include "src/compiler/turboshaft/operations.h"
-#include "src/compiler/turboshaft/optimization-phase.h"
 #include "src/compiler/turboshaft/representations.h"
 #include "src/compiler/turboshaft/sidetable.h"
 #include "src/compiler/turboshaft/snapshot-table.h"
@@ -31,7 +31,8 @@ V8_INLINE bool CanBeTyped(const Op& operation) {
   return operation.outputs_rep().size() > 0;
 }
 
-struct TypeInferenceReducerArgs {
+struct TypeInferenceReducerArgs
+    : base::ContextualClass<TypeInferenceReducerArgs> {
   enum class InputGraphTyping {
     kNone,     // Do not compute types for the input graph.
     kPrecise,  // Run a complete fixpoint analysis on the input graph.
@@ -44,9 +45,13 @@ struct TypeInferenceReducerArgs {
                             // for new nodes and more precise types where
                             // possible.
   };
-  Isolate* isolate;
   InputGraphTyping input_graph_typing;
   OutputGraphTyping output_graph_typing;
+
+  TypeInferenceReducerArgs(InputGraphTyping input_graph_typing,
+                           OutputGraphTyping output_graph_typing)
+      : input_graph_typing(input_graph_typing),
+        output_graph_typing(output_graph_typing) {}
 };
 
 // TypeInferenceReducer is the central component to infer types for Turboshaft
@@ -78,20 +83,8 @@ class TypeInferenceReducer
 
   using Adapter = UniformReducerAdapter<TypeInferenceReducer, Next>;
   using Args = TypeInferenceReducerArgs;
-  using ArgT = base::append_tuple_type<typename Next::ArgT, Args>;
 
-  template <typename... Ts>
-  explicit TypeInferenceReducer(const std::tuple<Ts...>& args)
-      : Adapter(args),
-        args_(std::get<Args>(args)),
-        input_graph_types_(Asm().graph_zone()),
-        output_graph_types_(Asm().output_graph().operation_types()),
-        table_(Asm().phase_zone()),
-        op_to_key_mapping_(Asm().phase_zone()),
-        block_to_snapshot_mapping_(Asm().input_graph().block_count(),
-                                   base::nullopt, Asm().phase_zone()),
-        predecessors_(Asm().phase_zone()),
-        analyzer_(Asm().modifiable_input_graph(), Asm().phase_zone()) {
+  TypeInferenceReducer() {
     // It is not reasonable to try to reuse input graph types if there are none.
     DCHECK_IMPLIES(args_.output_graph_typing ==
                        Args::OutputGraphTyping::kPreserveFromInputGraph,
@@ -197,8 +190,7 @@ class TypeInferenceReducer
     // Collect the snapshots of all predecessors.
     {
       predecessors_.clear();
-      for (const Block* pred = new_block->LastPredecessor(); pred != nullptr;
-           pred = pred->NeighboringPredecessor()) {
+      for (const Block* pred : new_block->PredecessorsIterable()) {
         base::Optional<table_t::Snapshot> pred_snapshot =
             block_to_snapshot_mapping_[pred->index()];
         DCHECK(pred_snapshot.has_value());
@@ -228,7 +220,7 @@ class TypeInferenceReducer
     // types.
     if (args_.output_graph_typing ==
         Args::OutputGraphTyping::kRefineFromInputGraph) {
-      if (new_block->HasExactlyNPredecessors(1)) {
+      if (new_block->PredecessorCount() == 1) {
         Block* predecessor = new_block->LastPredecessor();
         const Operation& terminator =
             predecessor->LastOperation(Asm().output_graph());
@@ -289,9 +281,8 @@ class TypeInferenceReducer
     }
   }
 
-  OpIndex REDUCE(PendingLoopPhi)(OpIndex first, RegisterRepresentation rep,
-                                 PendingLoopPhiOp::Data data) {
-    OpIndex index = Next::ReducePendingLoopPhi(first, rep, data);
+  OpIndex REDUCE(PendingLoopPhi)(OpIndex first, RegisterRepresentation rep) {
+    OpIndex index = Next::ReducePendingLoopPhi(first, rep);
     if (!NeedsTyping(index)) return index;
 
     // There is not much we can do for pending loop phis, because we don't know
@@ -413,7 +404,7 @@ class TypeInferenceReducer
   }
 
   void RemoveLast(OpIndex index_of_last_operation) {
-    if (auto key_opt = op_to_key_mapping_[index_of_last_operation]) {
+    if (op_to_key_mapping_[index_of_last_operation]) {
       op_to_key_mapping_[index_of_last_operation] = base::nullopt;
       TURBOSHAFT_TRACE_TYPING_OK(
           "REM  %3d:%-40s %-40s\n", index_of_last_operation.id(),
@@ -444,7 +435,7 @@ class TypeInferenceReducer
         (og_type.IsInvalid() ? "invalid" : og_type.ToString().c_str()),
         ig_type.ToString().c_str());
 
-    SetType(index, ig_type);
+    RefineOperationType(Asm().current_block(), index, ig_type, 'I');
   }
 
   Type GetTypeOrInvalid(OpIndex index) {
@@ -452,11 +443,24 @@ class TypeInferenceReducer
     return Type::Invalid();
   }
 
+  Type GetTupleType(const TupleOp& tuple) {
+    base::SmallVector<Type, 4> tuple_types;
+    for (OpIndex input : tuple.inputs()) {
+      tuple_types.push_back(GetType(input));
+    }
+    return TupleType::Tuple(base::VectorOf(tuple_types), Asm().graph_zone());
+  }
+
   Type GetType(OpIndex index) {
     Type type = GetTypeOrInvalid(index);
     if (type.IsInvalid()) {
       const Operation& op = Asm().output_graph().Get(index);
-      return Typer::TypeForRepresentation(op.outputs_rep(), Asm().graph_zone());
+      if (op.Is<TupleOp>()) {
+        return GetTupleType(op.Cast<TupleOp>());
+      } else {
+        return Typer::TypeForRepresentation(op.outputs_rep(),
+                                            Asm().graph_zone());
+      }
     }
     return type;
   }
@@ -542,18 +546,23 @@ class TypeInferenceReducer
                                 Args::OutputGraphTyping::kRefineFromInputGraph;
   }
 
-  Args args_;
-  GrowingSidetable<Type> input_graph_types_;
-  GrowingSidetable<Type>& output_graph_types_;
-  table_t table_;
+  TypeInferenceReducerArgs args_{TypeInferenceReducerArgs::Get()};
+  GrowingOpIndexSidetable<Type> input_graph_types_{Asm().graph_zone(),
+                                                   &Asm().input_graph()};
+  GrowingOpIndexSidetable<Type>& output_graph_types_{
+      Asm().output_graph().operation_types()};
+  table_t table_{Asm().phase_zone()};
   const Block* current_block_ = nullptr;
-  GrowingSidetable<base::Optional<table_t::Key>> op_to_key_mapping_;
+  GrowingOpIndexSidetable<base::Optional<table_t::Key>> op_to_key_mapping_{
+      Asm().phase_zone(), &Asm().output_graph()};
   GrowingBlockSidetable<base::Optional<table_t::Snapshot>>
-      block_to_snapshot_mapping_;
+      block_to_snapshot_mapping_{Asm().input_graph().block_count(),
+                                 base::nullopt, Asm().phase_zone()};
   // {predecessors_} is used during merging, but we use an instance variable for
   // it, in order to save memory and not reallocate it for each merge.
-  ZoneVector<table_t::Snapshot> predecessors_;
-  TypeInferenceAnalysis analyzer_;
+  ZoneVector<table_t::Snapshot> predecessors_{Asm().phase_zone()};
+  TypeInferenceAnalysis analyzer_{Asm().modifiable_input_graph(),
+                                  Asm().phase_zone()};
 };
 
 #include "src/compiler/turboshaft/undef-assembler-macros.inc"
