@@ -6,6 +6,7 @@
 
 #include "src/base/logging.h"
 #include "src/common/globals.h"
+#include "src/compiler/access-builder.h"
 #include "src/compiler/common-operator.h"
 #include "src/compiler/compiler-source-position-table.h"
 #include "src/compiler/node-properties.h"
@@ -13,6 +14,7 @@
 #include "src/compiler/operator.h"
 #include "src/compiler/wasm-graph-assembler.h"
 #include "src/objects/heap-number.h"
+#include "src/objects/string.h"
 #include "src/wasm/object-access.h"
 #include "src/wasm/wasm-engine.h"
 #include "src/wasm/wasm-linkage.h"
@@ -23,6 +25,13 @@ namespace v8 {
 namespace internal {
 namespace compiler {
 
+namespace {
+int TaggedOffset(FieldAccess access) {
+  DCHECK(access.base_is_tagged);
+  return wasm::ObjectAccess::ToTagged(access.offset);
+}
+}  // namespace
+
 WasmGCLowering::WasmGCLowering(Editor* editor, MachineGraph* mcgraph,
                                const wasm::WasmModule* module,
                                bool disable_trap_handler,
@@ -30,8 +39,8 @@ WasmGCLowering::WasmGCLowering(Editor* editor, MachineGraph* mcgraph,
     : AdvancedReducer(editor),
       null_check_strategy_(trap_handler::IsTrapHandlerEnabled() &&
                                    V8_STATIC_ROOTS_BOOL && !disable_trap_handler
-                               ? kTrapHandler
-                               : kExplicitNullChecks),
+                               ? NullCheckStrategy::kTrapHandler
+                               : NullCheckStrategy::kExplicit),
       gasm_(mcgraph, mcgraph->zone()),
       module_(module),
       dead_(mcgraph->Dead()),
@@ -42,8 +51,12 @@ Reduction WasmGCLowering::Reduce(Node* node) {
   switch (node->opcode()) {
     case IrOpcode::kWasmTypeCheck:
       return ReduceWasmTypeCheck(node);
+    case IrOpcode::kWasmTypeCheckAbstract:
+      return ReduceWasmTypeCheckAbstract(node);
     case IrOpcode::kWasmTypeCast:
       return ReduceWasmTypeCast(node);
+    case IrOpcode::kWasmTypeCastAbstract:
+      return ReduceWasmTypeCastAbstract(node);
     case IrOpcode::kAssertNotNull:
       return ReduceAssertNotNull(node);
     case IrOpcode::kNull:
@@ -56,10 +69,10 @@ Reduction WasmGCLowering::Reduce(Node* node) {
       return ReduceRttCanon(node);
     case IrOpcode::kTypeGuard:
       return ReduceTypeGuard(node);
-    case IrOpcode::kWasmExternInternalize:
-      return ReduceWasmExternInternalize(node);
-    case IrOpcode::kWasmExternExternalize:
-      return ReduceWasmExternExternalize(node);
+    case IrOpcode::kWasmAnyConvertExtern:
+      return ReduceWasmAnyConvertExtern(node);
+    case IrOpcode::kWasmExternConvertAny:
+      return ReduceWasmExternConvertAny(node);
     case IrOpcode::kWasmStructGet:
       return ReduceWasmStructGet(node);
     case IrOpcode::kWasmStructSet:
@@ -82,7 +95,9 @@ Reduction WasmGCLowering::Reduce(Node* node) {
 }
 
 Node* WasmGCLowering::Null(wasm::ValueType type) {
-  RootIndex index = wasm::IsSubtypeOf(type, wasm::kWasmExternRef, module_)
+  // TODO(thibaudm): Can we use wasm null for exnref?
+  RootIndex index = wasm::IsSubtypeOf(type, wasm::kWasmExternRef, module_) ||
+                            wasm::IsSubtypeOf(type, wasm::kWasmExnRef, module_)
                         ? RootIndex::kNullValue
                         : RootIndex::kWasmNull;
   return gasm_.LoadImmutable(MachineType::Pointer(), gasm_.LoadRootRegister(),
@@ -92,10 +107,12 @@ Node* WasmGCLowering::Null(wasm::ValueType type) {
 Node* WasmGCLowering::IsNull(Node* object, wasm::ValueType type) {
   Tagged_t static_null =
       wasm::GetWasmEngine()->compressed_wasm_null_value_or_zero();
-  Node* null_value = !wasm::IsSubtypeOf(type, wasm::kWasmExternRef, module_) &&
-                             static_null != 0
-                         ? gasm_.UintPtrConstant(static_null)
-                         : Null(type);
+  Node* null_value =
+      !wasm::IsSubtypeOf(type, wasm::kWasmExternRef, module_) &&
+              !wasm::IsSubtypeOf(type, wasm::kWasmExnRef, module_) &&
+              static_null != 0
+          ? gasm_.UintPtrConstant(static_null)
+          : Null(type);
   return gasm_.TaggedEqual(object, null_value);
 }
 
@@ -120,9 +137,9 @@ Reduction WasmGCLowering::ReduceWasmTypeCheck(Node* node) {
   auto end_label = gasm_.MakeLabel(MachineRepresentation::kWord32);
   bool is_cast_from_any = config.from.is_reference_to(wasm::HeapType::kAny);
 
-  // Skip the null check if casting from any and if null results in check
-  // failure. In that case the instance type check will identify null as not
-  // being a wasm object and return 0 (failure).
+  // If we are casting from any and null results in check failure, then the
+  // {IsDataRefMap} check below subsumes the null check. Otherwise, perform
+  // an explicit null check now.
   if (object_can_be_null && (!is_cast_from_any || config.to.is_nullable())) {
     const int kResult = config.to.is_nullable() ? 1 : 0;
     gasm_.GotoIf(IsNull(object, wasm::kWasmAnyRef), &end_label,
@@ -181,6 +198,91 @@ Reduction WasmGCLowering::ReduceWasmTypeCheck(Node* node) {
   return Replace(end_label.PhiAt(0));  // Meaningless argument.
 }
 
+Reduction WasmGCLowering::ReduceWasmTypeCheckAbstract(Node* node) {
+  DCHECK_EQ(node->opcode(), IrOpcode::kWasmTypeCheckAbstract);
+
+  Node* object = node->InputAt(0);
+  Node* effect_input = NodeProperties::GetEffectInput(node);
+  Node* control_input = NodeProperties::GetControlInput(node);
+  WasmTypeCheckConfig config = OpParameter<WasmTypeCheckConfig>(node->op());
+  const bool object_can_be_null = config.from.is_nullable();
+  const bool null_succeeds = config.to.is_nullable();
+  const bool object_can_be_i31 =
+      wasm::IsSubtypeOf(wasm::kWasmI31Ref.AsNonNull(), config.from, module_) ||
+      config.from.heap_representation() == wasm::HeapType::kExtern;
+
+  gasm_.InitializeEffectControl(effect_input, control_input);
+
+  Node* result = nullptr;
+  auto end_label = gasm_.MakeLabel(MachineRepresentation::kWord32);
+
+  wasm::HeapType::Representation to_rep = config.to.heap_representation();
+  do {
+    // The none-types only perform a null check. They need no control flow.
+    if (to_rep == wasm::HeapType::kNone ||
+        to_rep == wasm::HeapType::kNoExtern ||
+        to_rep == wasm::HeapType::kNoFunc) {
+      result = IsNull(object, config.from);
+      break;
+    }
+    // Null checks performed by any other type check need control flow. We can
+    // skip the null check if null fails, because it's covered by the Smi check
+    // or instance type check we'll do later.
+    if (object_can_be_null && null_succeeds) {
+      const int kResult = null_succeeds ? 1 : 0;
+      gasm_.GotoIf(IsNull(object, wasm::kWasmAnyRef), &end_label,
+                   BranchHint::kFalse, gasm_.Int32Constant(kResult));
+    }
+    // i31 is special in that the Smi check is the last thing to do.
+    if (to_rep == wasm::HeapType::kI31) {
+      // If earlier optimization passes reached the limit of possible graph
+      // transformations, we could DCHECK(object_can_be_i31) here.
+      result = object_can_be_i31 ? gasm_.IsSmi(object) : gasm_.Int32Constant(0);
+      break;
+    }
+    if (to_rep == wasm::HeapType::kEq) {
+      if (object_can_be_i31) {
+        gasm_.GotoIf(gasm_.IsSmi(object), &end_label, BranchHint::kFalse,
+                     gasm_.Int32Constant(1));
+      }
+      result = gasm_.IsDataRefMap(gasm_.LoadMap(object));
+      break;
+    }
+    // array, struct, string: i31 fails.
+    if (object_can_be_i31) {
+      gasm_.GotoIf(gasm_.IsSmi(object), &end_label, BranchHint::kFalse,
+                   gasm_.Int32Constant(0));
+    }
+    if (to_rep == wasm::HeapType::kArray) {
+      result = gasm_.HasInstanceType(object, WASM_ARRAY_TYPE);
+      break;
+    }
+    if (to_rep == wasm::HeapType::kStruct) {
+      result = gasm_.HasInstanceType(object, WASM_STRUCT_TYPE);
+      break;
+    }
+    if (to_rep == wasm::HeapType::kString ||
+        to_rep == wasm::HeapType::kExternString) {
+      Node* instance_type = gasm_.LoadInstanceType(gasm_.LoadMap(object));
+      result = gasm_.Uint32LessThan(instance_type,
+                                    gasm_.Uint32Constant(FIRST_NONSTRING_TYPE));
+      break;
+    }
+    UNREACHABLE();
+  } while (false);
+
+  DCHECK_NOT_NULL(result);
+  if (end_label.IsUsed()) {
+    gasm_.Goto(&end_label, result);
+    gasm_.Bind(&end_label);
+    result = end_label.PhiAt(0);
+  }
+
+  ReplaceWithValue(node, result, gasm_.effect(), gasm_.control());
+  node->Kill();
+  return Replace(result);  // Meaningless argument.
+}
+
 Reduction WasmGCLowering::ReduceWasmTypeCast(Node* node) {
   DCHECK_EQ(node->opcode(), IrOpcode::kWasmTypeCast);
 
@@ -199,9 +301,9 @@ Reduction WasmGCLowering::ReduceWasmTypeCast(Node* node) {
   auto end_label = gasm_.MakeLabel();
   bool is_cast_from_any = config.from.is_reference_to(wasm::HeapType::kAny);
 
-  // Skip the null check if casting from any and if null results in check
-  // failure. In that case the instance type check will identify null as not
-  // being a wasm object and trap.
+  // If we are casting from any and null results in check failure, then the
+  // {IsDataRefMap} check below subsumes the null check. Otherwise, perform
+  // an explicit null check now.
   if (object_can_be_null && (!is_cast_from_any || config.to.is_nullable())) {
     Node* is_null = IsNull(object, wasm::kWasmAnyRef);
     if (config.to.is_nullable()) {
@@ -270,6 +372,99 @@ Reduction WasmGCLowering::ReduceWasmTypeCast(Node* node) {
   return Replace(object);
 }
 
+Reduction WasmGCLowering::ReduceWasmTypeCastAbstract(Node* node) {
+  DCHECK_EQ(node->opcode(), IrOpcode::kWasmTypeCastAbstract);
+
+  Node* object = node->InputAt(0);
+  Node* effect_input = NodeProperties::GetEffectInput(node);
+  Node* control_input = NodeProperties::GetControlInput(node);
+  WasmTypeCheckConfig config = OpParameter<WasmTypeCheckConfig>(node->op());
+  const bool object_can_be_null = config.from.is_nullable();
+  const bool null_succeeds = config.to.is_nullable();
+  const bool object_can_be_i31 =
+      wasm::IsSubtypeOf(wasm::kWasmI31Ref.AsNonNull(), config.from, module_) ||
+      config.from.heap_representation() == wasm::HeapType::kExtern;
+
+  gasm_.InitializeEffectControl(effect_input, control_input);
+
+  auto end_label = gasm_.MakeLabel();
+
+  wasm::HeapType::Representation to_rep = config.to.heap_representation();
+
+  do {
+    // The none-types only perform a null check.
+    if (to_rep == wasm::HeapType::kNone ||
+        to_rep == wasm::HeapType::kNoExtern ||
+        to_rep == wasm::HeapType::kNoFunc) {
+      gasm_.TrapUnless(IsNull(object, config.from), TrapId::kTrapIllegalCast);
+      UpdateSourcePosition(gasm_.effect(), node);
+      break;
+    }
+    // Null checks performed by any other type cast can be skipped if null
+    // fails, because it's covered by the Smi check
+    // or instance type check we'll do later.
+    if (object_can_be_null && null_succeeds &&
+        !v8_flags.experimental_wasm_skip_null_checks) {
+      gasm_.GotoIf(IsNull(object, config.from), &end_label, BranchHint::kFalse);
+    }
+    if (to_rep == wasm::HeapType::kI31) {
+      // If earlier optimization passes reached the limit of possible graph
+      // transformations, we could DCHECK(object_can_be_i31) here.
+      Node* success =
+          object_can_be_i31 ? gasm_.IsSmi(object) : gasm_.Int32Constant(0);
+      gasm_.TrapUnless(success, TrapId::kTrapIllegalCast);
+      UpdateSourcePosition(gasm_.effect(), node);
+      break;
+    }
+    if (to_rep == wasm::HeapType::kEq) {
+      if (object_can_be_i31) {
+        gasm_.GotoIf(gasm_.IsSmi(object), &end_label, BranchHint::kFalse);
+      }
+      gasm_.TrapUnless(gasm_.IsDataRefMap(gasm_.LoadMap(object)),
+                       TrapId::kTrapIllegalCast);
+      UpdateSourcePosition(gasm_.effect(), node);
+      break;
+    }
+    // array, struct, string: i31 fails.
+    if (object_can_be_i31) {
+      gasm_.TrapIf(gasm_.IsSmi(object), TrapId::kTrapIllegalCast);
+      UpdateSourcePosition(gasm_.effect(), node);
+    }
+    if (to_rep == wasm::HeapType::kArray) {
+      gasm_.TrapUnless(gasm_.HasInstanceType(object, WASM_ARRAY_TYPE),
+                       TrapId::kTrapIllegalCast);
+      UpdateSourcePosition(gasm_.effect(), node);
+      break;
+    }
+    if (to_rep == wasm::HeapType::kStruct) {
+      gasm_.TrapUnless(gasm_.HasInstanceType(object, WASM_STRUCT_TYPE),
+                       TrapId::kTrapIllegalCast);
+      UpdateSourcePosition(gasm_.effect(), node);
+      break;
+    }
+    if (to_rep == wasm::HeapType::kString ||
+        to_rep == wasm::HeapType::kExternString) {
+      Node* instance_type = gasm_.LoadInstanceType(gasm_.LoadMap(object));
+      gasm_.TrapUnless(
+          gasm_.Uint32LessThan(instance_type,
+                               gasm_.Uint32Constant(FIRST_NONSTRING_TYPE)),
+          TrapId::kTrapIllegalCast);
+      UpdateSourcePosition(gasm_.effect(), node);
+      break;
+    }
+    UNREACHABLE();
+  } while (false);
+
+  if (end_label.IsUsed()) {
+    gasm_.Goto(&end_label);
+    gasm_.Bind(&end_label);
+  }
+
+  ReplaceWithValue(node, object, gasm_.effect(), gasm_.control());
+  node->Kill();
+  return Replace(object);
+}
+
 Reduction WasmGCLowering::ReduceAssertNotNull(Node* node) {
   DCHECK_EQ(node->opcode(), IrOpcode::kAssertNotNull);
   Node* effect = NodeProperties::GetEffectInput(node);
@@ -287,16 +482,17 @@ Reduction WasmGCLowering::ReduceAssertNotNull(Node* node) {
       // for null.
       // For subtypes of externref, we use JS null, so we have to check
       // explicitly.
-      if (null_check_strategy_ == kExplicitNullChecks ||
+      if (null_check_strategy_ == NullCheckStrategy::kExplicit ||
           wasm::IsSubtypeOf(wasm::kWasmI31Ref.AsNonNull(), op_parameter.type,
                             module_) ||
-          wasm::IsSubtypeOf(op_parameter.type, wasm::kWasmExternRef, module_)) {
+          wasm::IsSubtypeOf(op_parameter.type, wasm::kWasmExternRef, module_) ||
+          wasm::IsSubtypeOf(op_parameter.type, wasm::kWasmExnRef, module_)) {
         gasm_.TrapIf(IsNull(object, op_parameter.type), op_parameter.trap_id);
         UpdateSourcePosition(gasm_.effect(), node);
       } else {
         static_assert(WasmStruct::kHeaderSize > kTaggedSize);
         static_assert(WasmArray::kHeaderSize > kTaggedSize);
-        // TODO(manoskouk): JSFunction::kHeaderSize also has to be >kTaggedSize.
+        static_assert(WasmInternalFunction::kHeaderSize > kTaggedSize);
         Node* trap_null = gasm_.LoadTrapOnNull(
             MachineType::Int32(), object,
             gasm_.IntPtrConstant(wasm::ObjectAccess::ToTagged(kTaggedSize)));
@@ -340,7 +536,7 @@ Reduction WasmGCLowering::ReduceRttCanon(Node* node) {
   Node* instance_node = node->InputAt(0);
   Node* maps_list = gasm_.LoadImmutable(
       MachineType::TaggedPointer(), instance_node,
-      WasmInstanceObject::kManagedObjectMapsOffset - kHeapObjectTag);
+      WasmTrustedInstanceData::kManagedObjectMapsOffset - kHeapObjectTag);
   return Replace(gasm_.LoadImmutable(
       MachineType::TaggedPointer(), maps_list,
       wasm::ObjectAccess::ElementOffsetInTaggedFixedArray(type_index)));
@@ -359,8 +555,8 @@ constexpr int32_t kInt31MaxValue = 0x3fffffff;
 constexpr int32_t kInt31MinValue = -kInt31MaxValue - 1;
 }  // namespace
 
-Reduction WasmGCLowering::ReduceWasmExternInternalize(Node* node) {
-  DCHECK_EQ(node->opcode(), IrOpcode::kWasmExternInternalize);
+Reduction WasmGCLowering::ReduceWasmAnyConvertExtern(Node* node) {
+  DCHECK_EQ(node->opcode(), IrOpcode::kWasmAnyConvertExtern);
   Node* input = NodeProperties::GetValueInput(node, 0);
   Node* effect = NodeProperties::GetEffectInput(node);
   Node* control = NodeProperties::GetControlInput(node);
@@ -409,7 +605,7 @@ Reduction WasmGCLowering::ReduceWasmExternInternalize(Node* node) {
   gasm_.Bind(&heap_number_label);
   Node* float_value = gasm_.LoadFromObject(
       MachineType::Float64(), input,
-      wasm::ObjectAccess::ToTagged(HeapNumber::kValueOffset));
+      wasm::ObjectAccess::ToTagged(AccessBuilder::ForHeapNumberValue().offset));
   // Check range of float value.
   gasm_.GotoIf(
       gasm_.Float64LessThan(float_value, gasm_.Float64Constant(kInt31MinValue)),
@@ -455,8 +651,8 @@ Reduction WasmGCLowering::ReduceWasmExternInternalize(Node* node) {
   return Replace(end_label.PhiAt(0));
 }
 
-Reduction WasmGCLowering::ReduceWasmExternExternalize(Node* node) {
-  DCHECK_EQ(node->opcode(), IrOpcode::kWasmExternExternalize);
+Reduction WasmGCLowering::ReduceWasmExternConvertAny(Node* node) {
+  DCHECK_EQ(node->opcode(), IrOpcode::kWasmExternConvertAny);
   Node* object = node->InputAt(0);
   gasm_.InitializeEffectControl(NodeProperties::GetEffectInput(node),
                                 NodeProperties::GetControlInput(node));
@@ -486,7 +682,7 @@ Reduction WasmGCLowering::ReduceWasmStructGet(Node* node) {
 
   bool explicit_null_check =
       info.null_check == kWithNullCheck &&
-      (null_check_strategy_ == kExplicitNullChecks ||
+      (null_check_strategy_ == NullCheckStrategy::kExplicit ||
        info.field_index > wasm::kMaxStructFieldIndexForImplicitNullCheck);
   bool implicit_null_check =
       info.null_check == kWithNullCheck && !explicit_null_check;
@@ -522,7 +718,7 @@ Reduction WasmGCLowering::ReduceWasmStructSet(Node* node) {
 
   bool explicit_null_check =
       info.null_check == kWithNullCheck &&
-      (null_check_strategy_ == kExplicitNullChecks ||
+      (null_check_strategy_ == NullCheckStrategy::kExplicit ||
        info.field_index > wasm::kMaxStructFieldIndexForImplicitNullCheck);
   bool implicit_null_check =
       info.null_check == kWithNullCheck && !explicit_null_check;
@@ -610,7 +806,7 @@ Reduction WasmGCLowering::ReduceWasmArrayLength(Node* node) {
 
   bool null_check = OpParameter<bool>(node->op());
 
-  if (null_check_strategy_ == kExplicitNullChecks &&
+  if (null_check_strategy_ == NullCheckStrategy::kExplicit &&
       null_check == kWithNullCheck) {
     gasm_.TrapIf(IsNull(object, wasm::kWasmAnyRef),
                  TrapId::kTrapNullDereference);
@@ -618,7 +814,8 @@ Reduction WasmGCLowering::ReduceWasmArrayLength(Node* node) {
   }
 
   bool use_null_trap =
-      null_check_strategy_ == kTrapHandler && null_check == kWithNullCheck;
+      null_check_strategy_ == NullCheckStrategy::kTrapHandler &&
+      null_check == kWithNullCheck;
   Node* length =
       use_null_trap
           ? gasm_.LoadTrapOnNull(
@@ -696,15 +893,15 @@ Reduction WasmGCLowering::ReduceStringPrepareForGetCodeunit(Node* node) {
                       MachineRepresentation::kWord32);        // Offset.
 
   // These values will be used to replace the original node's projections.
-  // The first, "string", is either a SeqString or Smi(0) (in case of external
-  // string). Notably this makes it GC-safe: if that string moves, this pointer
-  // will be updated accordingly.
-  // The second, "offset", has full register width so that it can be used to
-  // store external pointers: for external strings, we add up the character
-  // backing store's base address and any slice offset.
-  // The third, "character width", is a shift width, i.e. it is 0 for one-byte
-  // strings, 1 for two-byte strings, kCharWidthBailoutSentinel for uncached
-  // external strings (for which "string"/"offset" are invalid and unusable).
+  // The first, "string", is either a SeqString or Tagged<Smi>(0) (in case of
+  // external string). Notably this makes it GC-safe: if that string moves, this
+  // pointer will be updated accordingly. The second, "offset", has full
+  // register width so that it can be used to store external pointers: for
+  // external strings, we add up the character backing store's base address and
+  // any slice offset. The third, "character width", is a shift width, i.e. it
+  // is 0 for one-byte strings, 1 for two-byte strings,
+  // kCharWidthBailoutSentinel for uncached external strings (for which
+  // "string"/"offset" are invalid and unusable).
   auto done =
       gasm_.MakeLabel(MachineRepresentation::kTagged,        // String.
                       MachineType::PointerRepresentation(),  // Offset.
@@ -715,8 +912,8 @@ Reduction WasmGCLowering::ReduceStringPrepareForGetCodeunit(Node* node) {
 
   gasm_.Bind(&dispatch);
   {
-    auto thin_string = gasm_.MakeLabel(MachineRepresentation::kTaggedPointer);
-    auto cons_string = gasm_.MakeLabel(MachineRepresentation::kTaggedPointer);
+    auto thin_string = gasm_.MakeLabel();
+    auto cons_string = gasm_.MakeLabel();
 
     Node* string = dispatch.PhiAt(0);
     Node* instance_type = dispatch.PhiAt(1);
@@ -734,20 +931,19 @@ Reduction WasmGCLowering::ReduceStringPrepareForGetCodeunit(Node* node) {
         instance_type, gasm_.Int32Constant(kStringRepresentationMask));
     gasm_.GotoIf(gasm_.Word32Equal(string_representation,
                                    gasm_.Int32Constant(kThinStringTag)),
-                 &thin_string, string);
+                 &thin_string);
     gasm_.GotoIf(gasm_.Word32Equal(string_representation,
                                    gasm_.Int32Constant(kConsStringTag)),
-                 &cons_string, string);
+                 &cons_string);
 
     // Sliced string.
     Node* new_offset = gasm_.Int32Add(
-        offset,
-        gasm_.BuildChangeSmiToInt32(gasm_.LoadImmutableFromObject(
-            MachineType::TaggedSigned(), string,
-            wasm::ObjectAccess::ToTagged(SlicedString::kOffsetOffset))));
+        offset, gasm_.BuildChangeSmiToInt32(gasm_.LoadImmutableFromObject(
+                    MachineType::TaggedSigned(), string,
+                    TaggedOffset(AccessBuilder::ForSlicedStringOffset()))));
     Node* parent = gasm_.LoadImmutableFromObject(
         MachineType::TaggedPointer(), string,
-        wasm::ObjectAccess::ToTagged(SlicedString::kParentOffset));
+        TaggedOffset(AccessBuilder::ForSlicedStringParent()));
     Node* parent_type = gasm_.LoadInstanceType(gasm_.LoadMap(parent));
     gasm_.Goto(&next, parent, parent_type, new_offset);
 
@@ -755,7 +951,7 @@ Reduction WasmGCLowering::ReduceStringPrepareForGetCodeunit(Node* node) {
     gasm_.Bind(&thin_string);
     Node* actual = gasm_.LoadImmutableFromObject(
         MachineType::TaggedPointer(), string,
-        wasm::ObjectAccess::ToTagged(ThinString::kActualOffset));
+        TaggedOffset(AccessBuilder::ForThinStringActual()));
     Node* actual_type = gasm_.LoadInstanceType(gasm_.LoadMap(actual));
     // ThinStrings always reference (internalized) direct strings.
     gasm_.Goto(&direct_string, actual, actual_type, offset);
@@ -765,7 +961,7 @@ Reduction WasmGCLowering::ReduceStringPrepareForGetCodeunit(Node* node) {
     gasm_.Bind(&cons_string);
     Node* first = gasm_.LoadImmutableFromObject(
         MachineType::TaggedPointer(), string,
-        wasm::ObjectAccess::ToTagged(ConsString::kFirstOffset));
+        TaggedOffset(AccessBuilder::ForConsStringFirst()));
     Node* first_type = gasm_.LoadInstanceType(gasm_.LoadMap(first));
     gasm_.Goto(&next, first, first_type, offset);
 
@@ -795,11 +991,12 @@ Reduction WasmGCLowering::ReduceStringPrepareForGetCodeunit(Node* node) {
                  &external);
 
     // Sequential string.
-    static_assert(SeqOneByteString::kCharsOffset ==
-                  SeqTwoByteString::kCharsOffset);
+    DCHECK_EQ(AccessBuilder::ForSeqOneByteStringCharacter().header_size,
+              AccessBuilder::ForSeqTwoByteStringCharacter().header_size);
+    const int chars_start_offset =
+        AccessBuilder::ForSeqOneByteStringCharacter().header_size;
     Node* final_offset = gasm_.Int32Add(
-        gasm_.Int32Constant(
-            wasm::ObjectAccess::ToTagged(SeqOneByteString::kCharsOffset)),
+        gasm_.Int32Constant(wasm::ObjectAccess::ToTagged(chars_start_offset)),
         gasm_.Word32Shl(offset, charwidth_shift));
     gasm_.Goto(&done, string, gasm_.BuildChangeInt32ToIntPtr(final_offset),
                charwidth_shift);
@@ -812,7 +1009,7 @@ Reduction WasmGCLowering::ReduceStringPrepareForGetCodeunit(Node* node) {
         &done, string, gasm_.IntPtrConstant(0),
         gasm_.Int32Constant(kCharWidthBailoutSentinel));
     Node* resource = gasm_.BuildLoadExternalPointerFromObject(
-        string, ExternalString::kResourceDataOffset,
+        string, AccessBuilder::ForExternalStringResourceData().offset,
         kExternalStringResourceDataTag, gasm_.LoadRootRegister());
     Node* shifted_offset = gasm_.Word32Shl(offset, charwidth_shift);
     final_offset = gasm_.IntPtrAdd(
@@ -854,13 +1051,8 @@ void WasmGCLowering::UpdateSourcePosition(Node* new_node, Node* old_node) {
   if (source_position_table_) {
     SourcePosition position =
         source_position_table_->GetSourcePosition(old_node);
-    if (position.ScriptOffset() != kNoSourcePosition) {
-      source_position_table_->SetSourcePosition(new_node, position);
-    } else {
-      // TODO(mliedtke): Source positions are not yet supported for inlining
-      // wasm into JS. Add support for it and replace the if with a DCHECK.
-      DCHECK_EQ(kExplicitNullChecks, null_check_strategy_);
-    }
+    DCHECK(position.ScriptOffset() != kNoSourcePosition);
+    source_position_table_->SetSourcePosition(new_node, position);
   }
 }
 
