@@ -35,9 +35,10 @@ static MaybeHandle<SharedFunctionInfo> GetFunctionInfo(Isolate* isolate,
   ScriptDetails script_details(isolate->factory()->empty_string(),
                                ScriptOriginOptions(true, true));
   script_details.repl_mode = repl_mode;
+  ScriptCompiler::CompilationDetails compilation_details;
   return Compiler::GetSharedFunctionInfoForScript(
       isolate, source, script_details, ScriptCompiler::kNoCompileOptions,
-      ScriptCompiler::kNoCacheNoReason, NOT_NATIVES_CODE);
+      ScriptCompiler::kNoCacheNoReason, NOT_NATIVES_CODE, &compilation_details);
 }
 }  // namespace
 
@@ -51,32 +52,21 @@ MaybeHandle<Object> DebugEvaluate::Global(Isolate* isolate,
   }
 
   Handle<NativeContext> context = isolate->native_context();
-  Handle<JSFunction> fun =
+  Handle<JSFunction> function =
       Factory::JSFunctionBuilder{isolate, shared_info, context}.Build();
 
-  return Global(isolate, fun, mode, repl_mode);
-}
-
-MaybeHandle<Object> DebugEvaluate::Global(Isolate* isolate,
-                                          Handle<JSFunction> function,
-                                          debug::EvaluateGlobalMode mode,
-                                          REPLMode repl_mode) {
-  // Disable breaks in side-effect free mode.
   DisableBreak disable_break_scope(
       isolate->debug(),
       mode == debug::EvaluateGlobalMode::kDisableBreaks ||
           mode ==
               debug::EvaluateGlobalMode::kDisableBreaksAndThrowOnSideEffect);
 
-  Handle<NativeContext> context = isolate->native_context();
-  CHECK_EQ(function->native_context(), *context);
-
   if (mode == debug::EvaluateGlobalMode::kDisableBreaksAndThrowOnSideEffect) {
     isolate->debug()->StartSideEffectCheckMode();
   }
   // TODO(cbruni, 1244145): Use host-defined options from script context.
   Handle<FixedArray> host_defined_options(
-      Script::cast(function->shared().script()).host_defined_options(),
+      Script::cast(function->shared()->script())->host_defined_options(),
       isolate);
   MaybeHandle<Object> result = Execution::CallScript(
       isolate, function, Handle<JSObject>(context->global_proxy(), isolate),
@@ -101,7 +91,7 @@ MaybeHandle<Object> DebugEvaluate::Local(Isolate* isolate,
   if (it.is_wasm()) {
     WasmFrame* frame = WasmFrame::cast(it.frame());
     Handle<SharedFunctionInfo> outer_info(
-        isolate->native_context()->empty_function().shared(), isolate);
+        isolate->native_context()->empty_function()->shared(), isolate);
     Handle<JSObject> context_extension = GetWasmDebugProxy(frame);
     Handle<ScopeInfo> scope_info =
         ScopeInfo::CreateForWithScope(isolate, Handle<ScopeInfo>::null());
@@ -121,7 +111,7 @@ MaybeHandle<Object> DebugEvaluate::Local(Isolate* isolate,
   // Note that the native context is taken from the original context chain,
   // which may not be the current native context of the isolate.
   ContextBuilder context_builder(isolate, frame, inlined_jsframe_index);
-  if (isolate->has_pending_exception()) return {};
+  if (isolate->has_exception()) return {};
 
   Handle<Context> context = context_builder.evaluation_context();
   Handle<JSObject> receiver(context->global_proxy(), isolate);
@@ -141,7 +131,7 @@ MaybeHandle<Object> DebugEvaluate::WithTopmostArguments(Isolate* isolate,
 
   // Get context and receiver.
   Handle<Context> native_context(
-      Context::cast(it.frame()->context()).native_context(), isolate);
+      Context::cast(it.frame()->context())->native_context(), isolate);
 
   // Materialize arguments as property on an extension object.
   Handle<JSObject> materialized = factory->NewSlowJSObjectWithNullProto();
@@ -153,8 +143,8 @@ MaybeHandle<Object> DebugEvaluate::WithTopmostArguments(Isolate* isolate,
 
   // Materialize receiver.
   Handle<Object> this_value(it.frame()->receiver(), isolate);
-  DCHECK_EQ(it.frame()->IsConstructor(), this_value->IsTheHole(isolate));
-  if (!this_value->IsTheHole(isolate)) {
+  DCHECK_EQ(it.frame()->IsConstructor(), IsTheHole(*this_value, isolate));
+  if (!IsTheHole(*this_value, isolate)) {
     Handle<String> this_str = factory->this_string();
     JSObject::SetOwnPropertyIgnoreAttributes(materialized, this_str, this_value,
                                              NONE)
@@ -168,7 +158,7 @@ MaybeHandle<Object> DebugEvaluate::WithTopmostArguments(Isolate* isolate,
   Handle<Context> evaluation_context = factory->NewDebugEvaluateContext(
       native_context, scope_info, materialized, Handle<Context>());
   Handle<SharedFunctionInfo> outer_info(
-      native_context->empty_function().shared(), isolate);
+      native_context->empty_function()->shared(), isolate);
   Handle<JSObject> receiver(native_context->global_proxy(), isolate);
   const bool throw_on_side_effect = false;
   MaybeHandle<Object> maybe_result =
@@ -197,7 +187,7 @@ MaybeHandle<Object> DebugEvaluate::Evaluate(
   success = Execution::Call(isolate, eval_fun, receiver, 0, nullptr)
                 .ToHandle(&result);
   if (throw_on_side_effect) isolate->debug()->StopSideEffectCheckMode();
-  if (!success) DCHECK(isolate->has_pending_exception());
+  if (!success) DCHECK(isolate->has_exception());
   return success ? result : MaybeHandle<Object>();
 }
 
@@ -253,7 +243,7 @@ DebugEvaluate::ContextBuilder::ContextBuilder(Isolate* isolate,
   }
 
   Handle<ScopeInfo> scope_info =
-      evaluation_context_->IsNativeContext()
+      IsNativeContext(*evaluation_context_)
           ? Handle<ScopeInfo>::null()
           : handle(evaluation_context_->scope_info(), isolate);
   for (auto rit = context_chain_.rbegin(); rit != context_chain_.rend();
@@ -273,10 +263,10 @@ DebugEvaluate::ContextBuilder::ContextBuilder(Isolate* isolate,
       // and also associate the temporary scope_info we create here with that
       // blocklist.
       Handle<ScopeInfo> function_scope_info = handle(
-          frame_inspector_.GetFunction()->shared().scope_info(), isolate_);
+          frame_inspector_.GetFunction()->shared()->scope_info(), isolate_);
       Handle<Object> block_list = handle(
           isolate_->LocalsBlockListCacheGet(function_scope_info), isolate_);
-      CHECK(block_list->IsStringSet());
+      CHECK(IsStringSet(*block_list));
       isolate_->LocalsBlockListCacheSet(scope_info, Handle<ScopeInfo>::null(),
                                         Handle<StringSet>::cast(block_list));
     }
@@ -298,7 +288,7 @@ void DebugEvaluate::ContextBuilder::UpdateValues() {
               .ToHandleChecked();
 
       for (int i = 0; i < keys->length(); i++) {
-        DCHECK(keys->get(i).IsString());
+        DCHECK(IsString(keys->get(i)));
         Handle<String> key(String::cast(keys->get(i)), isolate_);
         Handle<Object> value = JSReceiver::GetDataProperty(
             isolate_, element.materialized_object, key);
@@ -312,103 +302,102 @@ void DebugEvaluate::ContextBuilder::UpdateValues() {
 // static
 bool DebugEvaluate::IsSideEffectFreeIntrinsic(Runtime::FunctionId id) {
 // Use macro to include only the non-inlined version of an intrinsic.
-#define INTRINSIC_ALLOWLIST(V)                \
-  /* Conversions */                           \
-  V(NumberToStringSlow)                       \
-  V(ToBigInt)                                 \
-  V(ToLength)                                 \
-  V(ToNumber)                                 \
-  V(ToObject)                                 \
-  V(ToString)                                 \
-  /* Type checks */                           \
-  V(IsArray)                                  \
-  V(IsFunction)                               \
-  V(IsJSProxy)                                \
-  V(IsJSReceiver)                             \
-  V(IsRegExp)                                 \
-  V(IsSmi)                                    \
-  /* Loads */                                 \
-  V(LoadLookupSlotForCall)                    \
-  V(GetProperty)                              \
-  /* Arrays */                                \
-  V(ArraySpeciesConstructor)                  \
-  V(HasFastPackedElements)                    \
-  V(NewArray)                                 \
-  V(NormalizeElements)                        \
-  V(TypedArrayGetBuffer)                      \
-  /* Errors */                                \
-  V(NewTypeError)                             \
-  V(ReThrow)                                  \
-  V(ThrowCalledNonCallable)                   \
-  V(ThrowInvalidStringLength)                 \
-  V(ThrowIteratorError)                       \
-  V(ThrowIteratorResultNotAnObject)           \
-  V(ThrowPatternAssignmentNonCoercible)       \
-  V(ThrowReferenceError)                      \
-  V(ThrowSymbolIteratorInvalid)               \
-  /* Strings */                               \
-  V(StringReplaceOneCharWithString)           \
-  V(StringSubstring)                          \
-  V(StringToNumber)                           \
-  /* BigInts */                               \
-  V(BigIntEqualToBigInt)                      \
-  V(BigIntToBoolean)                          \
-  V(BigIntToNumber)                           \
-  /* Literals */                              \
-  V(CreateArrayLiteral)                       \
-  V(CreateArrayLiteralWithoutAllocationSite)  \
-  V(CreateObjectLiteral)                      \
-  V(CreateObjectLiteralWithoutAllocationSite) \
-  V(CreateRegExpLiteral)                      \
-  V(DefineClass)                              \
-  /* Called from builtins */                  \
-  V(AllocateInYoungGeneration)                \
-  V(AllocateInOldGeneration)                  \
-  V(AllocateSeqOneByteString)                 \
-  V(AllocateSeqTwoByteString)                 \
-  V(ArrayIncludes_Slow)                       \
-  V(ArrayIndexOf)                             \
-  V(ArrayIsArray)                             \
-  V(GetFunctionName)                          \
-  V(GetOwnPropertyDescriptor)                 \
-  V(GlobalPrint)                              \
-  V(HasProperty)                              \
-  V(ObjectCreate)                             \
-  V(ObjectEntries)                            \
-  V(ObjectEntriesSkipFastPath)                \
-  V(ObjectHasOwnProperty)                     \
-  V(ObjectKeys)                               \
-  V(ObjectValues)                             \
-  V(ObjectValuesSkipFastPath)                 \
-  V(ObjectGetOwnPropertyNames)                \
-  V(ObjectGetOwnPropertyNamesTryFast)         \
-  V(ObjectIsExtensible)                       \
-  V(RegExpInitializeAndCompile)               \
-  V(StackGuard)                               \
-  V(StringAdd)                                \
-  V(StringCharCodeAt)                         \
-  V(StringEqual)                              \
-  V(StringParseFloat)                         \
-  V(StringParseInt)                           \
-  V(SymbolDescriptiveString)                  \
-  V(ThrowRangeError)                          \
-  V(ThrowTypeError)                           \
-  V(ToName)                                   \
-  V(TransitionElementsKind)                   \
-  /* Misc. */                                 \
-  V(Call)                                     \
-  V(CompleteInobjectSlackTrackingForMap)      \
-  V(HasInPrototypeChain)                      \
-  V(IncrementUseCounter)                      \
-  V(MaxSmi)                                   \
-  V(NewObject)                                \
-  V(StringMaxLength)                          \
-  V(StringToArray)                            \
-  V(AsyncFunctionEnter)                       \
-  V(AsyncFunctionResolve)                     \
-  /* Test */                                  \
-  V(GetOptimizationStatus)                    \
-  V(OptimizeFunctionOnNextCall)               \
+#define INTRINSIC_ALLOWLIST(V)           \
+  /* Conversions */                      \
+  V(NumberToStringSlow)                  \
+  V(ToBigInt)                            \
+  V(ToLength)                            \
+  V(ToNumber)                            \
+  V(ToObject)                            \
+  V(ToString)                            \
+  /* Type checks */                      \
+  V(IsArray)                             \
+  V(IsJSProxy)                           \
+  V(IsJSReceiver)                        \
+  V(IsRegExp)                            \
+  V(IsSmi)                               \
+  /* Loads */                            \
+  V(LoadLookupSlotForCall)               \
+  V(GetPrivateMember)                    \
+  V(GetProperty)                         \
+  /* Arrays */                           \
+  V(ArraySpeciesConstructor)             \
+  V(HasFastPackedElements)               \
+  V(NewArray)                            \
+  V(NormalizeElements)                   \
+  V(TypedArrayGetBuffer)                 \
+  /* Errors */                           \
+  V(NewTypeError)                        \
+  V(ReThrow)                             \
+  V(ThrowCalledNonCallable)              \
+  V(ThrowInvalidStringLength)            \
+  V(ThrowIteratorError)                  \
+  V(ThrowIteratorResultNotAnObject)      \
+  V(ThrowPatternAssignmentNonCoercible)  \
+  V(ThrowReferenceError)                 \
+  V(ThrowSymbolIteratorInvalid)          \
+  /* Strings */                          \
+  V(StringReplaceOneCharWithString)      \
+  V(StringSubstring)                     \
+  V(StringToNumber)                      \
+  /* BigInts */                          \
+  V(BigIntEqualToBigInt)                 \
+  V(BigIntToBoolean)                     \
+  V(BigIntToNumber)                      \
+  /* Literals */                         \
+  V(CreateArrayLiteral)                  \
+  V(CreateObjectLiteral)                 \
+  V(CreateRegExpLiteral)                 \
+  V(DefineClass)                         \
+  /* Called from builtins */             \
+  V(AllocateInYoungGeneration)           \
+  V(AllocateInOldGeneration)             \
+  V(AllocateSeqOneByteString)            \
+  V(AllocateSeqTwoByteString)            \
+  V(ArrayIncludes_Slow)                  \
+  V(ArrayIndexOf)                        \
+  V(ArrayIsArray)                        \
+  V(GetFunctionName)                     \
+  V(GetOwnPropertyDescriptor)            \
+  V(GlobalPrint)                         \
+  V(HasProperty)                         \
+  V(ObjectCreate)                        \
+  V(ObjectEntries)                       \
+  V(ObjectEntriesSkipFastPath)           \
+  V(ObjectHasOwnProperty)                \
+  V(ObjectKeys)                          \
+  V(ObjectValues)                        \
+  V(ObjectValuesSkipFastPath)            \
+  V(ObjectGetOwnPropertyNames)           \
+  V(ObjectGetOwnPropertyNamesTryFast)    \
+  V(ObjectIsExtensible)                  \
+  V(RegExpInitializeAndCompile)          \
+  V(StackGuard)                          \
+  V(HandleNoHeapWritesInterrupts)        \
+  V(StringAdd)                           \
+  V(StringCharCodeAt)                    \
+  V(StringEqual)                         \
+  V(StringParseFloat)                    \
+  V(StringParseInt)                      \
+  V(SymbolDescriptiveString)             \
+  V(ThrowRangeError)                     \
+  V(ThrowTypeError)                      \
+  V(ToName)                              \
+  V(TransitionElementsKind)              \
+  /* Misc. */                            \
+  V(Call)                                \
+  V(CompleteInobjectSlackTrackingForMap) \
+  V(HasInPrototypeChain)                 \
+  V(IncrementUseCounter)                 \
+  V(MaxSmi)                              \
+  V(NewObject)                           \
+  V(StringMaxLength)                     \
+  V(StringToArray)                       \
+  V(AsyncFunctionEnter)                  \
+  V(AsyncFunctionResolve)                \
+  /* Test */                             \
+  V(GetOptimizationStatus)               \
+  V(OptimizeFunctionOnNextCall)          \
   V(OptimizeOsr)
 
 // Intrinsics with inline versions have to be allowlisted here a second time.
@@ -560,6 +549,7 @@ DebugInfo::SideEffectState BuiltinGetSideEffectState(Builtin id) {
     case Builtin::kObjectGetOwnPropertyNames:
     case Builtin::kObjectGetOwnPropertySymbols:
     case Builtin::kObjectGetPrototypeOf:
+    case Builtin::kObjectGroupBy:
     case Builtin::kObjectHasOwn:
     case Builtin::kObjectIs:
     case Builtin::kObjectIsExtensible:
@@ -582,7 +572,6 @@ DebugInfo::SideEffectState BuiltinGetSideEffectState(Builtin id) {
     case Builtin::kArrayPrototypeAt:
     case Builtin::kArrayPrototypeConcat:
     case Builtin::kArrayPrototypeEntries:
-    case Builtin::kArrayPrototypeFill:
     case Builtin::kArrayPrototypeFind:
     case Builtin::kArrayPrototypeFindIndex:
     case Builtin::kArrayPrototypeFindLast:
@@ -590,8 +579,6 @@ DebugInfo::SideEffectState BuiltinGetSideEffectState(Builtin id) {
     case Builtin::kArrayPrototypeFlat:
     case Builtin::kArrayPrototypeFlatMap:
     case Builtin::kArrayPrototypeJoin:
-    case Builtin::kArrayPrototypeGroup:
-    case Builtin::kArrayPrototypeGroupToMap:
     case Builtin::kArrayPrototypeKeys:
     case Builtin::kArrayPrototypeLastIndexOf:
     case Builtin::kArrayPrototypeSlice:
@@ -707,6 +694,7 @@ DebugInfo::SideEffectState BuiltinGetSideEffectState(Builtin id) {
     case Builtin::kDatePrototypeValueOf:
     // Map builtins.
     case Builtin::kMapConstructor:
+    case Builtin::kMapGroupBy:
     case Builtin::kMapPrototypeForEach:
     case Builtin::kMapPrototypeGet:
     case Builtin::kMapPrototypeHas:
@@ -872,8 +860,6 @@ DebugInfo::SideEffectState BuiltinGetSideEffectState(Builtin id) {
     case Builtin::kStrictPoisonPillThrower:
     case Builtin::kAllocateInYoungGeneration:
     case Builtin::kAllocateInOldGeneration:
-    case Builtin::kAllocateRegularInYoungGeneration:
-    case Builtin::kAllocateRegularInOldGeneration:
     case Builtin::kConstructVarargs:
     case Builtin::kConstructWithArrayLike:
     case Builtin::kGetOwnPropertyDescriptor:
@@ -966,6 +952,7 @@ DebugInfo::SideEffectState BuiltinGetSideEffectState(Builtin id) {
     case Builtin::kSetPrototypeDelete:
     // Array builtins.
     case Builtin::kArrayIteratorPrototypeNext:
+    case Builtin::kArrayPrototypeFill:
     case Builtin::kArrayPrototypePop:
     case Builtin::kArrayPrototypePush:
     case Builtin::kArrayPrototypeReverse:
@@ -1052,7 +1039,7 @@ DebugInfo::SideEffectState DebugEvaluate::FunctionGetSideEffectState(
     Handle<BytecodeArray> bytecode_array(info->GetBytecodeArray(isolate),
                                          isolate);
     if (v8_flags.trace_side_effect_free_debug_evaluate) {
-      bytecode_array->Print();
+      Print(*bytecode_array);
     }
     bool requires_runtime_checks = false;
     for (interpreter::BytecodeArrayIterator it(bytecode_array); !it.done();
@@ -1075,9 +1062,9 @@ DebugInfo::SideEffectState DebugEvaluate::FunctionGetSideEffectState(
     return requires_runtime_checks ? DebugInfo::kRequiresRuntimeChecks
                                    : DebugInfo::kHasNoSideEffect;
   } else if (info->IsApiFunction()) {
-    Code code = info->GetCode(isolate);
-    if (code.is_builtin()) {
-      return code.builtin_id() == Builtin::kHandleApiCall
+    Tagged<Code> code = info->GetCode(isolate);
+    if (code->is_builtin()) {
+      return code->builtin_id() == Builtin::kHandleApiCallOrConstruct
                  ? DebugInfo::kHasNoSideEffect
                  : DebugInfo::kHasSideEffects;
     }
@@ -1133,6 +1120,7 @@ static bool TransitivelyCalledBuiltinHasNoSideEffect(Builtin caller,
     case Builtin::kCEntry_Return2_ArgvOnStack_NoBuiltinExit:
     case Builtin::kCEntry_Return2_ArgvOnStack_BuiltinExit:
     case Builtin::kCEntry_Return2_ArgvInRegister_NoBuiltinExit:
+    case Builtin::kWasmCEntry:
     case Builtin::kCloneFastJSArray:
     case Builtin::kConstruct:
     case Builtin::kConvertToLocaleString:
@@ -1143,11 +1131,13 @@ static bool TransitivelyCalledBuiltinHasNoSideEffect(Builtin caller,
     case Builtin::kFastNewObject:
     case Builtin::kFindOrderedHashMapEntry:
     case Builtin::kFindOrderedHashSetEntry:
-    case Builtin::kFlatMapIntoArray:
-    case Builtin::kFlattenIntoArray:
+    case Builtin::kFlattenIntoArrayWithMapFn:
+    case Builtin::kFlattenIntoArrayWithoutMapFn:
     case Builtin::kGenericArrayToReversed:
     case Builtin::kGenericArrayWith:
     case Builtin::kGetProperty:
+    case Builtin::kGetPropertyWithReceiver:
+    case Builtin::kGroupByGeneric:
     case Builtin::kHasProperty:
     case Builtin::kCreateHTML:
     case Builtin::kNonNumberToNumber:
@@ -1242,16 +1232,16 @@ void DebugEvaluate::VerifyTransitiveBuiltins(Isolate* isolate) {
   for (Builtin caller = Builtins::kFirst; caller <= Builtins::kLast; ++caller) {
     DebugInfo::SideEffectState state = BuiltinGetSideEffectState(caller);
     if (state != DebugInfo::kHasNoSideEffect) continue;
-    Code code = isolate->builtins()->code(caller);
+    Tagged<Code> code = isolate->builtins()->code(caller);
     int mode = RelocInfo::ModeMask(RelocInfo::CODE_TARGET) |
                RelocInfo::ModeMask(RelocInfo::RELATIVE_CODE_TARGET);
 
     for (RelocIterator it(code, mode); !it.done(); it.next()) {
       RelocInfo* rinfo = it.rinfo();
       DCHECK(RelocInfo::IsCodeTargetMode(rinfo->rmode()));
-      Code lookup_result =
+      Tagged<Code> lookup_result =
           isolate->heap()->FindCodeForInnerPointer(rinfo->target_address());
-      Builtin callee = lookup_result.builtin_id();
+      Builtin callee = lookup_result->builtin_id();
       if (BuiltinGetSideEffectState(callee) == DebugInfo::kHasNoSideEffect) {
         continue;
       }
