@@ -28,7 +28,7 @@ ScopeIterator::ScopeIterator(Isolate* isolate, FrameInspector* frame_inspector,
       function_(frame_inspector_->GetFunction()),
       script_(frame_inspector_->GetScript()),
       locals_(StringSet::New(isolate)) {
-  if (!frame_inspector->GetContext()->IsContext()) {
+  if (!IsContext(*frame_inspector->GetContext())) {
     // Optimized frame, context or function cannot be materialized. Give up.
     return;
   }
@@ -47,10 +47,10 @@ ScopeIterator::~ScopeIterator() = default;
 Handle<Object> ScopeIterator::GetFunctionDebugName() const {
   if (!function_.is_null()) return JSFunction::GetDebugName(function_);
 
-  if (!context_->IsNativeContext()) {
+  if (!IsNativeContext(*context_)) {
     DisallowGarbageCollection no_gc;
-    ScopeInfo closure_info = context_->closure_context().scope_info();
-    Handle<String> debug_name(closure_info.FunctionDebugName(), isolate_);
+    Tagged<ScopeInfo> closure_info = context_->closure_context()->scope_info();
+    Handle<String> debug_name(closure_info->FunctionDebugName(), isolate_);
     if (debug_name->length() > 0) return debug_name;
   }
   return isolate_->factory()->undefined_value();
@@ -60,11 +60,11 @@ ScopeIterator::ScopeIterator(Isolate* isolate, Handle<JSFunction> function)
     : isolate_(isolate),
       context_(function->context(), isolate),
       locals_(StringSet::New(isolate)) {
-  if (!function->shared().IsSubjectToDebugging()) {
+  if (!function->shared()->IsSubjectToDebugging()) {
     context_ = Handle<Context>();
     return;
   }
-  script_ = handle(Script::cast(function->shared().script()), isolate);
+  script_ = handle(Script::cast(function->shared()->script()), isolate);
   UnwrapEvaluationContext();
 }
 
@@ -74,9 +74,9 @@ ScopeIterator::ScopeIterator(Isolate* isolate,
       generator_(generator),
       function_(generator->function(), isolate),
       context_(generator->context(), isolate),
-      script_(Script::cast(function_->shared().script()), isolate),
+      script_(Script::cast(function_->shared()->script()), isolate),
       locals_(StringSet::New(isolate)) {
-  CHECK(function_->shared().IsSubjectToDebugging());
+  CHECK(function_->shared()->IsSubjectToDebugging());
   TryParseAndRetrieveScopes(ReparseStrategy::kFunctionLiteral);
 }
 
@@ -105,9 +105,9 @@ class ScopeChainRetriever {
   ScopeChainRetriever(DeclarationScope* scope, Handle<JSFunction> function,
                       int position)
       : scope_(scope),
-        break_scope_start_(function->shared().StartPosition()),
-        break_scope_end_(function->shared().EndPosition()),
-        break_scope_type_(function->shared().scope_info().scope_type()),
+        break_scope_start_(function->shared()->StartPosition()),
+        break_scope_end_(function->shared()->EndPosition()),
+        break_scope_type_(function->shared()->scope_info()->scope_type()),
         position_(position) {
     DCHECK_NOT_NULL(scope);
     RetrieveScopes();
@@ -189,8 +189,13 @@ class ScopeChainRetriever {
     // context on the stack with a range that starts at Token::CLASS, and the
     // source position will also point to Token::CLASS.  To identify the
     // matching scope we include start in the accepted range for class scopes.
+    //
+    // Similarly "with" scopes can already have bytecodes where the source
+    // position points to the closing parenthesis with the "with" context
+    // already pushed.
     const bool position_fits_start =
-        scope->is_class_scope() ? start <= position_ : start < position_;
+        scope->is_class_scope() || scope->is_with_scope() ? start <= position_
+                                                          : start < position_;
     return position_fits_start && position_fits_end;
   }
 };
@@ -201,7 +206,7 @@ void ScopeIterator::TryParseAndRetrieveScopes(ReparseStrategy strategy) {
   // Catch the case when the debugger stops in an internal function.
   Handle<SharedFunctionInfo> shared_info(function_->shared(), isolate_);
   Handle<ScopeInfo> scope_info(shared_info->scope_info(), isolate_);
-  if (shared_info->script().IsUndefined(isolate_)) {
+  if (IsUndefined(shared_info->script(), isolate_)) {
     current_scope_ = closure_scope_ = nullptr;
     context_ = handle(function_->context(), isolate_);
     function_ = Handle<JSFunction>();
@@ -209,14 +214,14 @@ void ScopeIterator::TryParseAndRetrieveScopes(ReparseStrategy strategy) {
   }
 
   bool ignore_nested_scopes = false;
-  if (shared_info->HasBreakInfo() && frame_inspector_ != nullptr) {
+  if (shared_info->HasBreakInfo(isolate_) && frame_inspector_ != nullptr) {
     // The source position at return is always the end of the function,
     // which is not consistent with the current scope chain. Therefore all
     // nested with, catch and block contexts are skipped, and we can only
     // inspect the function scope.
     // This can only happen if we set a break point inside right before the
     // return, which requires a debug info to be available.
-    Handle<DebugInfo> debug_info(shared_info->GetDebugInfo(), isolate_);
+    Handle<DebugInfo> debug_info(shared_info->GetDebugInfo(isolate_), isolate_);
 
     // Find the break point where execution has stopped.
     BreakLocation location = BreakLocation::FromFrame(debug_info, GetFrame());
@@ -225,8 +230,9 @@ void ScopeIterator::TryParseAndRetrieveScopes(ReparseStrategy strategy) {
   }
 
   if (strategy == ReparseStrategy::kScriptIfNeeded) {
-    Object maybe_block_list = isolate_->LocalsBlockListCacheGet(scope_info);
-    calculate_blocklists_ = maybe_block_list.IsTheHole();
+    Tagged<Object> maybe_block_list =
+        isolate_->LocalsBlockListCacheGet(scope_info);
+    calculate_blocklists_ = IsTheHole(maybe_block_list);
     strategy = calculate_blocklists_ ? ReparseStrategy::kScriptIfNeeded
                                      : ReparseStrategy::kFunctionLiteral;
   }
@@ -249,7 +255,7 @@ void ScopeIterator::TryParseAndRetrieveScopes(ReparseStrategy strategy) {
   MaybeHandle<ScopeInfo> maybe_outer_scope;
   if (scope_info->scope_type() == EVAL_SCOPE || script->is_wrapped()) {
     flags.set_is_eval(true);
-    if (!context_->IsNativeContext()) {
+    if (!IsNativeContext(*context_)) {
       maybe_outer_scope = handle(context_->scope_info(), isolate_);
     }
     // Language mode may be inherited from the eval caller.
@@ -319,16 +325,16 @@ void ScopeIterator::TryParseAndRetrieveScopes(ReparseStrategy strategy) {
 
 void ScopeIterator::UnwrapEvaluationContext() {
   if (!context_->IsDebugEvaluateContext()) return;
-  Context current = *context_;
+  Tagged<Context> current = *context_;
   do {
-    Object wrapped = current.get(Context::WRAPPED_CONTEXT_INDEX);
-    if (wrapped.IsContext()) {
+    Tagged<Object> wrapped = current->get(Context::WRAPPED_CONTEXT_INDEX);
+    if (IsContext(wrapped)) {
       current = Context::cast(wrapped);
     } else {
-      DCHECK(!current.previous().is_null());
-      current = current.previous();
+      DCHECK(!current->previous().is_null());
+      current = current->previous();
     }
-  } while (current.IsDebugEvaluateContext());
+  } while (current->IsDebugEvaluateContext());
   context_ = handle(current, isolate_);
 }
 
@@ -356,19 +362,19 @@ Handle<JSObject> ScopeIterator::MaterializeScopeDetails() {
 }
 
 bool ScopeIterator::HasPositionInfo() {
-  return InInnerScope() || !context_->IsNativeContext();
+  return InInnerScope() || !IsNativeContext(*context_);
 }
 
 int ScopeIterator::start_position() {
   if (InInnerScope()) return current_scope_->start_position();
-  if (context_->IsNativeContext()) return 0;
-  return context_->closure_context().scope_info().StartPosition();
+  if (IsNativeContext(*context_)) return 0;
+  return context_->closure_context()->scope_info()->StartPosition();
 }
 
 int ScopeIterator::end_position() {
   if (InInnerScope()) return current_scope_->end_position();
-  if (context_->IsNativeContext()) return 0;
-  return context_->closure_context().scope_info().EndPosition();
+  if (IsNativeContext(*context_)) return 0;
+  return context_->closure_context()->scope_info()->EndPosition();
 }
 
 bool ScopeIterator::DeclaresLocals(Mode mode) const {
@@ -404,8 +410,7 @@ bool ScopeIterator::NeedsContext() const {
   // context should the function need one. In that case the function has already
   // pushed the context and we are good.
   CHECK_IMPLIES(needs_context && current_scope_ == closure_scope_ &&
-                    current_scope_->is_function_scope() &&
-                    !function_->is_null(),
+                    current_scope_->is_function_scope() && !function_.is_null(),
                 function_->context() != *context_);
 
   return needs_context;
@@ -420,7 +425,7 @@ bool ScopeIterator::AdvanceOneScope() {
 }
 
 void ScopeIterator::AdvanceOneContext() {
-  DCHECK(!context_->IsNativeContext());
+  DCHECK(!IsNativeContext(*context_));
   DCHECK(!context_->previous().is_null());
   context_ = handle(context_->previous(), isolate_);
 
@@ -461,7 +466,7 @@ void ScopeIterator::Next() {
 
   if (scope_type == ScopeTypeGlobal) {
     // The global scope is always the last in the chain.
-    DCHECK(context_->IsNativeContext());
+    DCHECK(IsNativeContext(*context_));
     context_ = Handle<Context>();
     DCHECK(Done());
     return;
@@ -513,7 +518,7 @@ ScopeIterator::ScopeType ScopeIterator::Type() const {
         return ScopeTypeModule;
       case SCRIPT_SCOPE:
         DCHECK_IMPLIES(NeedsContext(), context_->IsScriptContext() ||
-                                           context_->IsNativeContext());
+                                           IsNativeContext(*context_));
         return ScopeTypeScript;
       case WITH_SCOPE:
         DCHECK_IMPLIES(NeedsContext(), context_->IsWithContext());
@@ -529,14 +534,14 @@ ScopeIterator::ScopeType ScopeIterator::Type() const {
         DCHECK_IMPLIES(NeedsContext(), context_->IsEvalContext());
         return ScopeTypeEval;
       case SHADOW_REALM_SCOPE:
-        DCHECK_IMPLIES(NeedsContext(), context_->IsNativeContext());
+        DCHECK_IMPLIES(NeedsContext(), IsNativeContext(*context_));
         // TODO(v8:11989): New ScopeType for ShadowRealms?
         return ScopeTypeScript;
     }
     UNREACHABLE();
   }
-  if (context_->IsNativeContext()) {
-    DCHECK(context_->global_object().IsJSGlobalObject());
+  if (IsNativeContext(*context_)) {
+    DCHECK(IsJSGlobalObject(context_->global_object()));
     // If we are at the native context and have not yet seen script scope,
     // fake it.
     return seen_script_scope_ ? ScopeTypeGlobal : ScopeTypeScript;
@@ -577,7 +582,7 @@ Handle<JSObject> ScopeIterator::ScopeObject(Mode mode) {
   Handle<JSObject> scope = isolate_->factory()->NewSlowJSObjectWithNullProto();
   auto visitor = [=](Handle<String> name, Handle<Object> value,
                      ScopeType scope_type) {
-    if (value->IsOptimizedOut(isolate_)) {
+    if (IsOptimizedOut(*value, isolate_)) {
       if (v8_flags.experimental_value_unavailable) {
         JSObject::SetAccessor(scope, name,
                               isolate_->factory()->value_unavailable_accessor(),
@@ -587,7 +592,7 @@ Handle<JSObject> ScopeIterator::ScopeObject(Mode mode) {
       }
       // Reflect optimized out variables as undefined in scope object.
       value = isolate_->factory()->undefined_value();
-    } else if (value->IsTheHole(isolate_)) {
+    } else if (IsTheHole(*value, isolate_)) {
       if (scope_type == ScopeTypeScript &&
           JSReceiver::HasOwnProperty(isolate_, scope, name).FromMaybe(true)) {
         // We also use the hole to represent overridden let-declarations via
@@ -606,8 +611,8 @@ Handle<JSObject> ScopeIterator::ScopeObject(Mode mode) {
     }
     // Overwrite properties. Sometimes names in the same scope can collide, e.g.
     // with extension objects introduced via local eval.
-    JSObject::SetPropertyOrElement(isolate_, scope, name, value,
-                                   Just(ShouldThrow::kDontThrow))
+    Object::SetPropertyOrElement(isolate_, scope, name, value,
+                                 Just(ShouldThrow::kDontThrow))
         .Check();
     return false;
   };
@@ -688,7 +693,7 @@ bool ScopeIterator::ClosureScopeHasThisReference() const {
 }
 
 void ScopeIterator::CollectLocalsFromCurrentScope() {
-  DCHECK(locals_->IsStringSet());
+  DCHECK(IsStringSet(*locals_));
   for (Variable* var : *current_scope_->locals()) {
     if (var->location() == VariableLocation::PARAMETER ||
         var->location() == VariableLocation::LOCAL) {
@@ -705,17 +710,17 @@ void ScopeIterator::DebugPrint() {
   switch (Type()) {
     case ScopeIterator::ScopeTypeGlobal:
       os << "Global:\n";
-      context_->Print(os);
+      Print(*context_, os);
       break;
 
     case ScopeIterator::ScopeTypeLocal: {
       os << "Local:\n";
       if (NeedsContext()) {
-        context_->Print(os);
+        Print(*context_, os);
         if (context_->has_extension()) {
           Handle<HeapObject> extension(context_->extension(), isolate_);
-          DCHECK(extension->IsJSContextExtensionObject());
-          extension->Print(os);
+          DCHECK(IsJSContextExtensionObject(*extension));
+          Print(*extension, os);
         }
       }
       break;
@@ -723,29 +728,28 @@ void ScopeIterator::DebugPrint() {
 
     case ScopeIterator::ScopeTypeWith:
       os << "With:\n";
-      context_->extension().Print(os);
+      Print(context_->extension(), os);
       break;
 
     case ScopeIterator::ScopeTypeCatch:
       os << "Catch:\n";
-      context_->extension().Print(os);
-      context_->get(Context::THROWN_OBJECT_INDEX).Print(os);
+      Print(context_->extension(), os);
+      Print(context_->get(Context::THROWN_OBJECT_INDEX), os);
       break;
 
     case ScopeIterator::ScopeTypeClosure:
       os << "Closure:\n";
-      context_->Print(os);
+      Print(*context_, os);
       if (context_->has_extension()) {
         Handle<HeapObject> extension(context_->extension(), isolate_);
-        DCHECK(extension->IsJSContextExtensionObject());
-        extension->Print(os);
+        DCHECK(IsJSContextExtensionObject(*extension));
+        Print(*extension, os);
       }
       break;
 
     case ScopeIterator::ScopeTypeScript:
       os << "Script:\n";
-      context_->global_object().native_context().script_context_table().Print(
-          os);
+      Print(context_->native_context()->script_context_table(), os);
       break;
 
     default:
@@ -761,24 +765,22 @@ int ScopeIterator::GetSourcePosition() const {
   } else {
     DCHECK(!generator_.is_null());
     SharedFunctionInfo::EnsureSourcePositionsAvailable(
-        isolate_, handle(generator_->function().shared(), isolate_));
+        isolate_, handle(generator_->function()->shared(), isolate_));
     return generator_->source_position();
   }
 }
 
 void ScopeIterator::VisitScriptScope(const Visitor& visitor) const {
-  Handle<JSGlobalObject> global(context_->global_object(), isolate_);
   Handle<ScriptContextTable> script_contexts(
-      global->native_context().script_context_table(), isolate_);
+      context_->native_context()->script_context_table(), isolate_);
 
   // Skip the first script since that just declares 'this'.
-  for (int context_index = 1;
-       context_index < script_contexts->used(kAcquireLoad); context_index++) {
-    Handle<Context> context = ScriptContextTable::GetContext(
-        isolate_, script_contexts, context_index);
+  for (int i = 1; i < script_contexts->length(kAcquireLoad); i++) {
+    Handle<Context> context(script_contexts->get(i), isolate_);
     Handle<ScopeInfo> scope_info(context->scope_info(), isolate_);
-    if (VisitContextLocals(visitor, scope_info, context, ScopeTypeScript))
+    if (VisitContextLocals(visitor, scope_info, context, ScopeTypeScript)) {
       return;
+    }
   }
 }
 
@@ -786,8 +788,9 @@ void ScopeIterator::VisitModuleScope(const Visitor& visitor) const {
   DCHECK(context_->IsModuleContext());
 
   Handle<ScopeInfo> scope_info(context_->scope_info(), isolate_);
-  if (VisitContextLocals(visitor, scope_info, context_, ScopeTypeModule))
+  if (VisitContextLocals(visitor, scope_info, context_, ScopeTypeModule)) {
     return;
+  }
 
   int module_variable_count = scope_info->ModuleVariableCount();
 
@@ -797,7 +800,7 @@ void ScopeIterator::VisitModuleScope(const Visitor& visitor) const {
     int index;
     Handle<String> name;
     {
-      String raw_name;
+      Tagged<String> raw_name;
       scope_info->ModuleVariable(i, &raw_name, &index);
       if (ScopeInfo::VariableIsSynthetic(raw_name)) continue;
       name = handle(raw_name, isolate_);
@@ -877,10 +880,10 @@ bool ScopeIterator::VisitLocals(const Visitor& visitor, Mode mode,
         if (frame_inspector_ == nullptr) {
           // Get the variable from the suspended generator.
           DCHECK(!generator_.is_null());
-          FixedArray parameters_and_registers =
+          Tagged<FixedArray> parameters_and_registers =
               generator_->parameters_and_registers();
-          DCHECK_LT(index, parameters_and_registers.length());
-          value = handle(parameters_and_registers.get(index), isolate_);
+          DCHECK_LT(index, parameters_and_registers->length());
+          value = handle(parameters_and_registers->get(index), isolate_);
         } else if (var->IsReceiver()) {
           value = frame_inspector_->GetReceiver();
         } else {
@@ -893,23 +896,23 @@ bool ScopeIterator::VisitLocals(const Visitor& visitor, Mode mode,
         if (frame_inspector_ == nullptr) {
           // Get the variable from the suspended generator.
           DCHECK(!generator_.is_null());
-          FixedArray parameters_and_registers =
+          Tagged<FixedArray> parameters_and_registers =
               generator_->parameters_and_registers();
           int parameter_count =
-              function_->shared().scope_info().ParameterCount();
+              function_->shared()->scope_info()->ParameterCount();
           index += parameter_count;
-          DCHECK_LT(index, parameters_and_registers.length());
-          value = handle(parameters_and_registers.get(index), isolate_);
+          DCHECK_LT(index, parameters_and_registers->length());
+          value = handle(parameters_and_registers->get(index), isolate_);
         } else {
           value = frame_inspector_->GetExpression(index);
-          if (value->IsOptimizedOut(isolate_)) {
+          if (IsOptimizedOut(*value, isolate_)) {
             // We'll rematerialize this later.
             if (current_scope_->is_declaration_scope() &&
                 current_scope_->AsDeclarationScope()->arguments() == var) {
               continue;
             }
           } else if (IsLexicalVariableMode(var->mode()) &&
-                     value->IsUndefined(isolate_) &&
+                     IsUndefined(*value, isolate_) &&
                      GetSourcePosition() != kNoSourcePosition &&
                      GetSourcePosition() <= var->initializer_position()) {
             // Variables that are `undefined` could also mean an elided hole
@@ -929,7 +932,7 @@ bool ScopeIterator::VisitLocals(const Visitor& visitor, Mode mode,
         // don't match (https://crbug.com/753338).
         // Return `undefined` if the context's ScopeInfo doesn't know anything
         // about this variable.
-        if (context_->scope_info().ContextSlotIndex(var->name()) != index) {
+        if (context_->scope_info()->ContextSlotIndex(var->name()) != index) {
           value = isolate_->factory()->undefined_value();
         } else {
           value = handle(context_->get(index), isolate_);
@@ -954,7 +957,7 @@ bool ScopeIterator::VisitLocals(const Visitor& visitor, Mode mode,
 // a proxy, return an empty object.
 Handle<JSObject> ScopeIterator::WithContextExtension() {
   DCHECK(context_->IsWithContext());
-  if (context_->extension_receiver().IsJSProxy()) {
+  if (IsJSProxy(context_->extension_receiver())) {
     return isolate_->factory()->NewSlowJSObjectWithNullProto();
   }
   return handle(JSObject::cast(context_->extension_receiver()), isolate_);
@@ -983,8 +986,9 @@ void ScopeIterator::VisitLocalScope(const Visitor& visitor, Mode mode,
       // FunctionGetArguments does.
       if (frame_inspector_ != nullptr && !closure_scope_->is_arrow_scope() &&
           (closure_scope_->arguments() == nullptr ||
-           frame_inspector_->GetExpression(closure_scope_->arguments()->index())
-               ->IsOptimizedOut(isolate_))) {
+           IsOptimizedOut(*frame_inspector_->GetExpression(
+                              closure_scope_->arguments()->index()),
+                          isolate_))) {
         JavaScriptFrame* frame = GetFrame();
         Handle<JSObject> arguments = Accessors::FunctionGetArguments(
             frame, frame_inspector_->inlined_frame_index());
@@ -1001,9 +1005,9 @@ void ScopeIterator::VisitLocalScope(const Visitor& visitor, Mode mode,
 
   if (mode == Mode::ALL && HasContext()) {
     DCHECK(!context_->IsScriptContext());
-    DCHECK(!context_->IsNativeContext());
+    DCHECK(!IsNativeContext(*context_));
     DCHECK(!context_->IsWithContext());
-    if (!context_->scope_info().SloppyEvalCanExtendVars()) return;
+    if (!context_->scope_info()->SloppyEvalCanExtendVars()) return;
     if (context_->extension_object().is_null()) return;
     Handle<JSObject> extension(context_->extension_object(), isolate_);
     Handle<FixedArray> keys =
@@ -1013,7 +1017,7 @@ void ScopeIterator::VisitLocalScope(const Visitor& visitor, Mode mode,
 
     for (int i = 0; i < keys->length(); i++) {
       // Names of variables introduced by eval are strings.
-      DCHECK(keys->get(i).IsString());
+      DCHECK(IsString(keys->get(i)));
       Handle<String> key(String::cast(keys->get(i)), isolate_);
       Handle<Object> value =
           JSReceiver::GetDataProperty(isolate_, extension, key);
@@ -1064,7 +1068,7 @@ bool ScopeIterator::SetLocalVariableValue(Handle<String> variable_name,
             // Set the variable in the suspended generator.
             DCHECK(!generator_.is_null());
             int parameter_count =
-                function_->shared().scope_info().ParameterCount();
+                function_->shared()->scope_info()->ParameterCount();
             index += parameter_count;
             Handle<FixedArray> parameters_and_registers(
                 generator_->parameters_and_registers(), isolate_);
@@ -1086,7 +1090,8 @@ bool ScopeIterator::SetLocalVariableValue(Handle<String> variable_name,
           // don't match (https://crbug.com/753338).
           // Skip the write if the context's ScopeInfo doesn't know anything
           // about this variable.
-          if (context_->scope_info().ContextSlotIndex(variable_name) != index) {
+          if (context_->scope_info()->ContextSlotIndex(variable_name) !=
+              index) {
             return false;
           }
           context_->set(index, *new_value);
@@ -1109,7 +1114,7 @@ bool ScopeIterator::SetContextExtensionValue(Handle<String> variable_name,
                                              Handle<Object> new_value) {
   if (!context_->has_extension()) return false;
 
-  DCHECK(context_->extension_object().IsJSContextExtensionObject());
+  DCHECK(IsJSContextExtensionObject(context_->extension_object()));
   Handle<JSObject> ext(context_->extension_object(), isolate_);
   LookupIterator it(isolate_, ext, variable_name, LookupIterator::OWN);
   Maybe<bool> maybe = JSReceiver::HasProperty(&it);
@@ -1122,7 +1127,7 @@ bool ScopeIterator::SetContextExtensionValue(Handle<String> variable_name,
 
 bool ScopeIterator::SetContextVariableValue(Handle<String> variable_name,
                                             Handle<Object> new_value) {
-  int slot_index = context_->scope_info().ContextSlotIndex(variable_name);
+  int slot_index = context_->scope_info()->ContextSlotIndex(variable_name);
   if (slot_index < 0) return false;
   context_->set(slot_index, *new_value);
   return true;
@@ -1135,7 +1140,7 @@ bool ScopeIterator::SetModuleVariableValue(Handle<String> variable_name,
   VariableMode mode;
   InitializationFlag init_flag;
   MaybeAssignedFlag maybe_assigned_flag;
-  cell_index = context_->scope_info().ModuleIndex(
+  cell_index = context_->scope_info()->ModuleIndex(
       *variable_name, &mode, &init_flag, &maybe_assigned_flag);
 
   // Setting imports is currently not supported.
@@ -1152,12 +1157,11 @@ bool ScopeIterator::SetModuleVariableValue(Handle<String> variable_name,
 bool ScopeIterator::SetScriptVariableValue(Handle<String> variable_name,
                                            Handle<Object> new_value) {
   Handle<ScriptContextTable> script_contexts(
-      context_->global_object().native_context().script_context_table(),
-      isolate_);
+      context_->native_context()->script_context_table(), isolate_);
   VariableLookupResult lookup_result;
   if (script_contexts->Lookup(variable_name, &lookup_result)) {
-    Handle<Context> script_context = ScriptContextTable::GetContext(
-        isolate_, script_contexts, lookup_result.context_index);
+    Handle<Context> script_context(
+        script_contexts->get(lookup_result.context_index), isolate_);
     script_context->set(lookup_result.slot_index, *new_value);
     return true;
   }
@@ -1251,13 +1255,13 @@ Handle<ScopeInfo> LocalBlocklistsCollector::FindScopeInfoForScope(
     Scope* scope) const {
   DisallowGarbageCollection no_gc;
   SharedFunctionInfo::ScriptIterator iterator(isolate_, *script_);
-  for (SharedFunctionInfo info = iterator.Next(); !info.is_null();
+  for (Tagged<SharedFunctionInfo> info = iterator.Next(); !info.is_null();
        info = iterator.Next()) {
-    ScopeInfo scope_info = info.scope_info();
-    if (info.is_compiled() && !scope_info.is_null() &&
-        scope->start_position() == info.StartPosition() &&
-        scope->end_position() == info.EndPosition() &&
-        scope->scope_type() == scope_info.scope_type()) {
+    Tagged<ScopeInfo> scope_info = info->scope_info();
+    if (info->is_compiled() && !scope_info.is_null() &&
+        scope->start_position() == info->StartPosition() &&
+        scope->end_position() == info->EndPosition() &&
+        scope->scope_type() == scope_info->scope_type()) {
       return handle(scope_info, isolate_);
     }
   }
@@ -1281,7 +1285,7 @@ void LocalBlocklistsCollector::StoreFunctionBlocklists(
 void LocalBlocklistsCollector::CollectAndStore() {
   InitializeWithClosureScope();
 
-  while (scope_->outer_scope() && !context_->IsNativeContext()) {
+  while (scope_->outer_scope() && !IsNativeContext(*context_)) {
     AdvanceToNextNonHiddenScope();
     // 1. Add all stack-allocated variables of `scope_` to the various lists.
     CollectCurrentLocalsIntoBlocklists();
@@ -1298,7 +1302,7 @@ void LocalBlocklistsCollector::CollectAndStore() {
         // need to advance `context_`.
         isolate_->LocalsBlockListCacheSet(
             handle(context_->scope_info(), isolate_),
-            handle(context_->previous().scope_info(), isolate_),
+            handle(context_->previous()->scope_info(), isolate_),
             context_blocklist_);
         context_ = handle(context_->previous(), isolate_);
       }
@@ -1328,10 +1332,8 @@ void ScopeIterator::MaybeCollectAndStoreLocalBlocklists() const {
     return;
   }
 
-  DCHECK(isolate_
-             ->LocalsBlockListCacheGet(
-                 handle(function_->shared().scope_info(), isolate_))
-             .IsTheHole());
+  DCHECK(IsTheHole(isolate_->LocalsBlockListCacheGet(
+      handle(function_->shared()->scope_info(), isolate_))));
   LocalBlocklistsCollector collector(isolate_, script_, context_,
                                      closure_scope_);
   collector.CollectAndStore();
