@@ -16,6 +16,7 @@
 #include "src/objects/objects.h"
 
 #if V8_ENABLE_WEBASSEMBLY
+#include "src/wasm/stacks.h"
 #include "src/wasm/wasm-code-manager.h"
 #endif  // V8_ENABLE_WEBASSEMBLY
 
@@ -46,6 +47,7 @@
 //       - CWasmEntryFrame
 //     - Internal
 //       - ConstructFrame
+//       - FastConstructFrame
 //       - BuiltinContinuationFrame
 //     - WasmFrame
 //       - WasmExitFrame
@@ -130,8 +132,10 @@ class StackHandler {
     JavaScriptBuiltinContinuationWithCatchFrame)                          \
   V(INTERNAL, InternalFrame)                                              \
   V(CONSTRUCT, ConstructFrame)                                            \
+  V(FAST_CONSTRUCT, FastConstructFrame)                                   \
   V(BUILTIN, BuiltinFrame)                                                \
   V(BUILTIN_EXIT, BuiltinExitFrame)                                       \
+  V(API_CALLBACK_EXIT, ApiCallbackExitFrame)                              \
   V(NATIVE, NativeFrame)                                                  \
   V(IRREGEXP, IrregexpFrame)
 
@@ -260,7 +264,9 @@ class StackFrame {
     return type() == JAVA_SCRIPT_BUILTIN_CONTINUATION_WITH_CATCH;
   }
   bool is_construct() const { return type() == CONSTRUCT; }
+  bool is_fast_construct() const { return type() == FAST_CONSTRUCT; }
   bool is_builtin_exit() const { return type() == BUILTIN_EXIT; }
+  bool is_api_callback_exit() const { return type() == API_CALLBACK_EXIT; }
   bool is_irregexp() const { return type() == IRREGEXP; }
 
   static bool IsJavaScript(Type t) {
@@ -310,15 +316,16 @@ class StackFrame {
   // This method is used by Isolate::PushStackTraceAndDie() for collecting a
   // stack trace on fatal error and thus it might be called in the middle of GC
   // and should be as safe as possible.
-  virtual HeapObject unchecked_code() const = 0;
+  virtual Tagged<HeapObject> unchecked_code() const = 0;
 
   // Search for the code associated with this frame.
-  V8_EXPORT_PRIVATE Code LookupCode() const;
-  V8_EXPORT_PRIVATE GcSafeCode GcSafeLookupCode() const;
+  V8_EXPORT_PRIVATE Tagged<Code> LookupCode() const;
+  V8_EXPORT_PRIVATE Tagged<GcSafeCode> GcSafeLookupCode() const;
 
   virtual void Iterate(RootVisitor* v) const = 0;
   void IteratePc(RootVisitor* v, Address* pc_address,
-                 Address* constant_pool_address, GcSafeCode holder) const;
+                 Address* constant_pool_address,
+                 Tagged<GcSafeCode> holder) const;
 
   // Sets a callback function for return-address rewriting profilers
   // to resolve the location of a return address to the location of the
@@ -376,7 +383,10 @@ class V8_EXPORT_PRIVATE FrameSummary {
 // Subclasses for the different summary kinds:
 #define FRAME_SUMMARY_VARIANTS(F)                                          \
   F(JAVA_SCRIPT, JavaScriptFrameSummary, java_script_summary_, JavaScript) \
-  IF_WASM(F, WASM, WasmFrameSummary, wasm_summary_, Wasm)
+  IF_WASM(F, BUILTIN, BuiltinFrameSummary, builtin_summary_, Builtin)      \
+  IF_WASM(F, WASM, WasmFrameSummary, wasm_summary_, Wasm)                  \
+  IF_WASM(F, WASM_INLINED, WasmInlinedFrameSummary, wasm_inlined_summary_, \
+          WasmInlined)
 
 #define FRAME_SUMMARY_KIND(kind, type, field, desc) kind,
   enum Kind { FRAME_SUMMARY_VARIANTS(FRAME_SUMMARY_KIND) };
@@ -396,10 +406,10 @@ class V8_EXPORT_PRIVATE FrameSummary {
 
   class JavaScriptFrameSummary : public FrameSummaryBase {
    public:
-    JavaScriptFrameSummary(Isolate* isolate, Object receiver,
-                           JSFunction function, AbstractCode abstract_code,
-                           int code_offset, bool is_constructor,
-                           FixedArray parameters);
+    JavaScriptFrameSummary(Isolate* isolate, Tagged<Object> receiver,
+                           Tagged<JSFunction> function,
+                           Tagged<AbstractCode> abstract_code, int code_offset,
+                           bool is_constructor, Tagged<FixedArray> parameters);
 
     void EnsureSourcePositionsAvailable();
     bool AreSourcePositionsAvailable() const;
@@ -429,8 +439,8 @@ class V8_EXPORT_PRIVATE FrameSummary {
 #if V8_ENABLE_WEBASSEMBLY
   class WasmFrameSummary : public FrameSummaryBase {
    public:
-    WasmFrameSummary(Isolate*, Handle<WasmInstanceObject>, wasm::WasmCode*,
-                     int byte_offset, int function_index,
+    WasmFrameSummary(Isolate* isolate, Handle<WasmInstanceObject> instance,
+                     wasm::WasmCode* code, int byte_offset, int function_index,
                      bool at_to_number_conversion);
 
     Handle<Object> receiver() const;
@@ -444,6 +454,7 @@ class V8_EXPORT_PRIVATE FrameSummary {
     int SourceStatementPosition() const { return SourcePosition(); }
     Handle<Script> script() const;
     Handle<WasmInstanceObject> wasm_instance() const { return wasm_instance_; }
+    Handle<WasmTrustedInstanceData> wasm_trusted_instance_data() const;
     Handle<Context> native_context() const;
     bool at_to_number_conversion() const { return at_to_number_conversion_; }
     Handle<StackFrameInfo> CreateStackFrameInfo() const;
@@ -455,6 +466,53 @@ class V8_EXPORT_PRIVATE FrameSummary {
     int byte_offset_;
     int function_index_;
   };
+
+  // Summary of a wasm frame inlined into JavaScript. (Wasm frames inlined into
+  // wasm are expressed by a WasmFrameSummary.)
+  class WasmInlinedFrameSummary : public FrameSummaryBase {
+   public:
+    WasmInlinedFrameSummary(Isolate* isolate,
+                            Handle<WasmInstanceObject> instance,
+                            int function_index, int op_wire_bytes_offset);
+
+    Handle<WasmInstanceObject> wasm_instance() const { return wasm_instance_; }
+    Handle<WasmTrustedInstanceData> wasm_trusted_instance_data() const;
+    Handle<Object> receiver() const;
+    uint32_t function_index() const;
+    int code_offset() const { return op_wire_bytes_offset_; }
+    bool is_constructor() const { return false; }
+    bool is_subject_to_debugging() const { return true; }
+    Handle<Script> script() const;
+    int SourcePosition() const;
+    int SourceStatementPosition() const { return SourcePosition(); }
+    Handle<Context> native_context() const;
+    Handle<StackFrameInfo> CreateStackFrameInfo() const;
+
+   private:
+    Handle<WasmInstanceObject> wasm_instance_;
+    int function_index_;
+    int op_wire_bytes_offset_;  // relative to function offset.
+  };
+
+  class BuiltinFrameSummary : public FrameSummaryBase {
+   public:
+    BuiltinFrameSummary(Isolate*, Builtin);
+
+    Builtin builtin() const { return builtin_; }
+
+    Handle<Object> receiver() const;
+    int code_offset() const { return 0; }
+    bool is_constructor() const { return false; }
+    bool is_subject_to_debugging() const { return false; }
+    Handle<Object> script() const;
+    int SourcePosition() const { return kNoSourcePosition; }
+    int SourceStatementPosition() const { return 0; }
+    Handle<Context> native_context() const;
+    Handle<StackFrameInfo> CreateStackFrameInfo() const;
+
+   private:
+    Builtin builtin_;
+  };
 #endif  // V8_ENABLE_WEBASSEMBLY
 
 #define FRAME_SUMMARY_CONS(kind, type, field, desc) \
@@ -504,13 +562,13 @@ class V8_EXPORT_PRIVATE FrameSummary {
 class CommonFrame : public StackFrame {
  public:
   // Accessors.
-  virtual Object context()
+  virtual Tagged<Object> context()
       const;  // TODO(victorgomes): CommonFrames don't have context.
   virtual int position() const;
 
   // Access the expressions in the stack frame including locals.
-  inline Object GetExpression(int index) const;
-  inline void SetExpression(int index, Object value);
+  inline Tagged<Object> GetExpression(int index) const;
+  inline void SetExpression(int index, Tagged<Object> value);
   int ComputeExpressionsCount() const;
 
   Address GetCallerStackPointer() const override;
@@ -528,7 +586,7 @@ class CommonFrame : public StackFrame {
  protected:
   inline explicit CommonFrame(StackFrameIteratorBase* iterator);
 
-  bool HasTaggedOutgoingParams(GcSafeCode code_lookup) const;
+  bool HasTaggedOutgoingParams(Tagged<GcSafeCode> code_lookup) const;
 
   void ComputeCallerState(State* state) const override;
 
@@ -552,7 +610,7 @@ class TurbofanStubWithContextFrame : public CommonFrame {
  public:
   Type type() const override { return TURBOFAN_STUB_WITH_CONTEXT; }
 
-  HeapObject unchecked_code() const override;
+  Tagged<HeapObject> unchecked_code() const override;
   void Iterate(RootVisitor* v) const override;
 
  protected:
@@ -565,9 +623,11 @@ class TurbofanStubWithContextFrame : public CommonFrame {
 
 class TypedFrame : public CommonFrame {
  public:
-  HeapObject unchecked_code() const override { return {}; }
+  Tagged<HeapObject> unchecked_code() const override { return {}; }
   void Iterate(RootVisitor* v) const override;
 
+  void IterateParamsOfWasmToJSWrapper(RootVisitor* v) const;
+
  protected:
   inline explicit TypedFrame(StackFrameIteratorBase* iterator);
 };
@@ -575,16 +635,16 @@ class TypedFrame : public CommonFrame {
 class CommonFrameWithJSLinkage : public CommonFrame {
  public:
   // Accessors.
-  virtual JSFunction function() const = 0;
+  virtual Tagged<JSFunction> function() const = 0;
 
   // Access the parameters.
-  virtual Object receiver() const;
-  virtual Object GetParameter(int index) const;
+  virtual Tagged<Object> receiver() const;
+  virtual Tagged<Object> GetParameter(int index) const;
   virtual int ComputeParametersCount() const;
   Handle<FixedArray> GetParameters() const;
   virtual int GetActualArgumentCount() const;
 
-  HeapObject unchecked_code() const override;
+  Tagged<HeapObject> unchecked_code() const override;
 
   // Lookup exception handler for current {pc}, returns -1 if none found. Also
   // returns data associated with the handler site specific to the frame type:
@@ -621,16 +681,16 @@ class JavaScriptFrame : public CommonFrameWithJSLinkage {
   Type type() const override = 0;
 
   // Accessors.
-  JSFunction function() const override;
-  Object unchecked_function() const;
-  Script script() const;
-  Object context() const override;
+  Tagged<JSFunction> function() const override;
+  Tagged<Object> unchecked_function() const;
+  Tagged<Script> script() const;
+  Tagged<Object> context() const override;
   int GetActualArgumentCount() const override;
 
-  inline void set_receiver(Object value);
+  inline void set_receiver(Tagged<Object> value);
 
   // Debugger access.
-  void SetParameterValue(int index, Object value) const;
+  void SetParameterValue(int index, Tagged<Object> value) const;
 
   // Check if this frame is a constructor frame invoked through 'new'.
   bool IsConstructor() const override;
@@ -643,7 +703,8 @@ class JavaScriptFrame : public CommonFrameWithJSLinkage {
              int index) const override;
 
   // Return a list with {SharedFunctionInfo} objects of this frame.
-  virtual void GetFunctions(std::vector<SharedFunctionInfo>* functions) const;
+  virtual void GetFunctions(
+      std::vector<Tagged<SharedFunctionInfo>>* functions) const;
 
   void GetFunctions(std::vector<Handle<SharedFunctionInfo>>* functions) const;
 
@@ -657,15 +718,16 @@ class JavaScriptFrame : public CommonFrameWithJSLinkage {
     return static_cast<JavaScriptFrame*>(frame);
   }
 
-  static void PrintFunctionAndOffset(JSFunction function, AbstractCode code,
-                                     int code_offset, FILE* file,
-                                     bool print_line_number);
+  static void PrintFunctionAndOffset(Tagged<JSFunction> function,
+                                     Tagged<AbstractCode> code, int code_offset,
+                                     FILE* file, bool print_line_number);
 
   static void PrintTop(Isolate* isolate, FILE* file, bool print_args,
                        bool print_line_number);
 
-  static void CollectFunctionAndOffsetForICStats(JSFunction function,
-                                                 AbstractCode code,
+  static void CollectFunctionAndOffsetForICStats(IsolateForSandbox isolate,
+                                                 Tagged<JSFunction> function,
+                                                 Tagged<AbstractCode> code,
                                                  int code_offset);
 
  protected:
@@ -676,7 +738,7 @@ class JavaScriptFrame : public CommonFrameWithJSLinkage {
   virtual void PrintFrameKind(StringStream* accumulator) const {}
 
  private:
-  inline Object function_slot_object() const;
+  inline Tagged<Object> function_slot_object() const;
 
   friend class StackFrameIteratorBase;
 };
@@ -702,7 +764,7 @@ class EntryFrame : public TypedFrame {
  public:
   Type type() const override { return ENTRY; }
 
-  HeapObject unchecked_code() const override;
+  Tagged<HeapObject> unchecked_code() const override;
 
   // Garbage collection support.
   void Iterate(RootVisitor* v) const override;
@@ -731,7 +793,7 @@ class ConstructEntryFrame : public EntryFrame {
  public:
   Type type() const override { return CONSTRUCT_ENTRY; }
 
-  HeapObject unchecked_code() const override;
+  Tagged<HeapObject> unchecked_code() const override;
 
   static ConstructEntryFrame* cast(StackFrame* frame) {
     DCHECK(frame->is_construct_entry());
@@ -783,10 +845,10 @@ class BuiltinExitFrame : public ExitFrame {
  public:
   Type type() const override { return BUILTIN_EXIT; }
 
-  JSFunction function() const;
+  Tagged<JSFunction> function() const;
 
-  Object receiver() const;
-  Object GetParameter(int i) const;
+  Tagged<Object> receiver() const;
+  Tagged<Object> GetParameter(int i) const;
   int ComputeParametersCount() const;
   Handle<FixedArray> GetParameters() const;
 
@@ -803,10 +865,60 @@ class BuiltinExitFrame : public ExitFrame {
   inline explicit BuiltinExitFrame(StackFrameIteratorBase* iterator);
 
  private:
-  inline Object receiver_slot_object() const;
-  inline Object argc_slot_object() const;
-  inline Object target_slot_object() const;
-  inline Object new_target_slot_object() const;
+  inline Tagged<Object> receiver_slot_object() const;
+  inline Tagged<Object> argc_slot_object() const;
+  inline Tagged<Object> target_slot_object() const;
+  inline Tagged<Object> new_target_slot_object() const;
+
+  friend class StackFrameIteratorBase;
+};
+
+// Api callback exit frames are a special case of exit frames, which are used
+// whenever an Api functions (such as v8::Function or v8::FunctionTemplate) are
+// called. Their main purpose is to allow these functions to appear in stack
+// traces.
+class ApiCallbackExitFrame : public ExitFrame {
+ public:
+  Type type() const override { return API_CALLBACK_EXIT; }
+
+  // ApiCallbackExitFrame might contain either FunctionTemplateInfo or
+  // JSFunction in the function slot.
+  Tagged<HeapObject> target() const;
+
+  // In case function slot contains FunctionTemplateInfo, instantiate the
+  // function, stores it in the function slot and returns JSFunction handle.
+  Handle<JSFunction> GetFunction() const;
+
+  Tagged<Object> receiver() const;
+  Tagged<Object> GetParameter(int i) const;
+  int ComputeParametersCount() const;
+  Handle<FixedArray> GetParameters() const;
+
+  // Check if this frame is a constructor frame invoked through 'new'.
+  bool IsConstructor() const;
+
+  void Print(StringStream* accumulator, PrintMode mode,
+             int index) const override;
+
+  // Summarize Frame
+  void Summarize(std::vector<FrameSummary>* frames) const override;
+
+  static ApiCallbackExitFrame* cast(StackFrame* frame) {
+    DCHECK(frame->is_api_callback_exit());
+    return static_cast<ApiCallbackExitFrame*>(frame);
+  }
+
+ protected:
+  inline explicit ApiCallbackExitFrame(StackFrameIteratorBase* iterator);
+
+ private:
+  inline void set_target(Tagged<HeapObject> function) const;
+
+  inline FullObjectSlot receiver_slot() const;
+  inline FullObjectSlot argc_slot() const;
+  inline FullObjectSlot context_slot() const;
+  inline FullObjectSlot target_slot() const;
+  inline FullObjectSlot new_target_slot() const;
 
   friend class StackFrameIteratorBase;
 };
@@ -815,12 +927,14 @@ class StubFrame : public TypedFrame {
  public:
   Type type() const override { return STUB; }
 
-  HeapObject unchecked_code() const override;
+  Tagged<HeapObject> unchecked_code() const override;
 
   // Lookup exception handler for current {pc}, returns -1 if none found. Only
   // TurboFan stub frames are supported.
   int LookupExceptionHandlerInTable();
 
+  void Summarize(std::vector<FrameSummary>* frames) const override;
+
  protected:
   inline explicit StubFrame(StackFrameIteratorBase* iterator);
 
@@ -833,11 +947,12 @@ class OptimizedFrame : public JavaScriptFrame {
   // Return a list with {SharedFunctionInfo} objects of this frame.
   // The functions are ordered bottom-to-top (i.e. functions.last()
   // is the top-most activation)
-  void GetFunctions(std::vector<SharedFunctionInfo>* functions) const override;
+  void GetFunctions(
+      std::vector<Tagged<SharedFunctionInfo>>* functions) const override;
 
   void Summarize(std::vector<FrameSummary>* frames) const override;
 
-  DeoptimizationData GetDeoptimizationData(int* deopt_index) const;
+  Tagged<DeoptimizationData> GetDeoptimizationData(int* deopt_index) const;
 
   static int StackSlotOffsetRelativeToFp(int slot_index);
 
@@ -845,7 +960,8 @@ class OptimizedFrame : public JavaScriptFrame {
   int LookupExceptionHandlerInTable(
       int* data, HandlerTable::CatchPrediction* prediction) override;
 
-  virtual int FindReturnPCForTrampoline(Code code, int trampoline_pc) const = 0;
+  virtual int FindReturnPCForTrampoline(Tagged<Code> code,
+                                        int trampoline_pc) const = 0;
 
  protected:
   inline explicit OptimizedFrame(StackFrameIteratorBase* iterator);
@@ -867,10 +983,12 @@ class UnoptimizedFrame : public JavaScriptFrame {
   virtual int GetBytecodeOffset() const = 0;
 
   // Returns the frame's current bytecode array.
-  BytecodeArray GetBytecodeArray() const;
+  Tagged<BytecodeArray> GetBytecodeArray() const;
 
   // Access to the interpreter register file for this frame.
-  Object ReadInterpreterRegister(int register_index) const;
+  Tagged<Object> ReadInterpreterRegister(int register_index) const;
+
+  inline void SetFeedbackVector(Tagged<FeedbackVector> feedback_vector);
 
   // Build a list with summaries for this frame including all inlined frames.
   void Summarize(std::vector<FrameSummary>* frames) const override;
@@ -902,7 +1020,7 @@ class InterpretedFrame : public UnoptimizedFrame {
 
   // Updates the frame's BytecodeArray with |bytecode_array|. Used by the
   // debugger to swap execution onto a BytecodeArray patched with breakpoints.
-  void PatchBytecodeArray(BytecodeArray bytecode_array);
+  void PatchBytecodeArray(Tagged<BytecodeArray> bytecode_array);
 
   static InterpretedFrame* cast(StackFrame* frame) {
     DCHECK(frame->is_interpreted());
@@ -931,7 +1049,7 @@ class BaselineFrame : public UnoptimizedFrame {
 
   intptr_t GetPCForBytecodeOffset(int lookup_offset) const;
 
-  void PatchContext(Context value);
+  void PatchContext(Tagged<Context> value);
 
   static BaselineFrame* cast(StackFrame* frame) {
     DCHECK(frame->is_baseline());
@@ -956,7 +1074,8 @@ class MaglevFrame : public OptimizedFrame {
 
   void Iterate(RootVisitor* v) const override;
 
-  int FindReturnPCForTrampoline(Code code, int trampoline_pc) const override;
+  int FindReturnPCForTrampoline(Tagged<Code> code,
+                                int trampoline_pc) const override;
 
   Handle<JSFunction> GetInnermostFunction() const;
   BytecodeOffset GetBytecodeOffsetForOSR() const;
@@ -978,7 +1097,8 @@ class TurbofanFrame : public OptimizedFrame {
 
   void Iterate(RootVisitor* v) const override;
 
-  int FindReturnPCForTrampoline(Code code, int trampoline_pc) const override;
+  int FindReturnPCForTrampoline(Tagged<Code> code,
+                                int trampoline_pc) const override;
 
  protected:
   inline explicit TurbofanFrame(StackFrameIteratorBase* iterator);
@@ -986,7 +1106,7 @@ class TurbofanFrame : public OptimizedFrame {
  private:
   friend class StackFrameIteratorBase;
 
-  Object StackSlotAt(int index) const;
+  Tagged<Object> StackSlotAt(int index) const;
 };
 
 // Builtin frames are built for builtins with JavaScript linkage, such as
@@ -1000,7 +1120,7 @@ class BuiltinFrame final : public TypedFrameWithJSLinkage {
     return static_cast<BuiltinFrame*>(frame);
   }
 
-  JSFunction function() const override;
+  Tagged<JSFunction> function() const override;
   int ComputeParametersCount() const override;
 
  protected:
@@ -1025,14 +1145,16 @@ class WasmFrame : public TypedFrame {
   void Iterate(RootVisitor* v) const override;
 
   // Accessors.
-  virtual V8_EXPORT_PRIVATE WasmInstanceObject wasm_instance() const;
+  virtual V8_EXPORT_PRIVATE Tagged<WasmInstanceObject> wasm_instance() const;
+  virtual Tagged<WasmTrustedInstanceData> trusted_instance_data() const;
   V8_EXPORT_PRIVATE wasm::NativeModule* native_module() const;
+
   wasm::WasmCode* wasm_code() const;
   int function_index() const;
-  Script script() const;
+  Tagged<Script> script() const;
   // Byte position in the module, or asm.js source position.
   int position() const override;
-  Object context() const override;
+  Tagged<Object> context() const override;
   bool at_to_number_conversion() const;
   // Generated code byte offset in the function.
   int generated_code_offset() const;
@@ -1050,7 +1172,7 @@ class WasmFrame : public TypedFrame {
 
  private:
   friend class StackFrameIteratorBase;
-  WasmModuleObject module_object() const;
+  Tagged<WasmModuleObject> module_object() const;
 };
 
 class WasmExitFrame : public WasmFrame {
@@ -1092,7 +1214,8 @@ class WasmToJsFrame : public WasmFrame {
   Type type() const override { return WASM_TO_JS; }
 
   int position() const override { return 0; }
-  WasmInstanceObject wasm_instance() const override;
+  Tagged<WasmInstanceObject> wasm_instance() const override;
+  Tagged<WasmTrustedInstanceData> trusted_instance_data() const override;
 
  protected:
   inline explicit WasmToJsFrame(StackFrameIteratorBase* iterator);
@@ -1213,6 +1336,24 @@ class ConstructFrame : public InternalFrame {
   friend class StackFrameIteratorBase;
 };
 
+// Fast construct frames are special construct trampoline frames that avoid
+// pushing arguments to the stack twice.
+class FastConstructFrame : public InternalFrame {
+ public:
+  Type type() const override { return FAST_CONSTRUCT; }
+
+  static FastConstructFrame* cast(StackFrame* frame) {
+    DCHECK(frame->is_fast_construct());
+    return static_cast<FastConstructFrame*>(frame);
+  }
+
+ protected:
+  inline explicit FastConstructFrame(StackFrameIteratorBase* iterator);
+
+ private:
+  friend class StackFrameIteratorBase;
+};
+
 class BuiltinContinuationFrame : public InternalFrame {
  public:
   Type type() const override { return BUILTIN_CONTINUATION; }
@@ -1238,11 +1379,11 @@ class JavaScriptBuiltinContinuationFrame : public TypedFrameWithJSLinkage {
     return static_cast<JavaScriptBuiltinContinuationFrame*>(frame);
   }
 
-  JSFunction function() const override;
+  Tagged<JSFunction> function() const override;
   int ComputeParametersCount() const override;
   intptr_t GetSPToFPDelta() const;
 
-  Object context() const override;
+  Tagged<Object> context() const override;
 
  protected:
   inline explicit JavaScriptBuiltinContinuationFrame(
@@ -1266,7 +1407,7 @@ class JavaScriptBuiltinContinuationWithCatchFrame
 
   // Patch in the exception object at the appropriate location into the stack
   // frame.
-  void SetException(Object exception);
+  void SetException(Tagged<Object> exception);
 
  protected:
   inline explicit JavaScriptBuiltinContinuationWithCatchFrame(
@@ -1440,8 +1581,11 @@ class V8_EXPORT_PRIVATE DebuggableStackFrameIterator {
 // Similar to StackFrameIterator, but can be created and used at any time and
 // any stack state. Currently, the only user is the profiler; if this ever
 // changes, find another name for this class.
-class V8_EXPORT_PRIVATE StackFrameIteratorForProfiler
-    : public StackFrameIteratorBase {
+// IMPORTANT: Do not mark this class as V8_EXPORT_PRIVATE. The profiler creates
+// instances of this class from a signal handler. If we use V8_EXPORT_PRIVATE
+// "ld" inserts a symbol stub for the constructor call that may crash with
+// a stackoverflow when called from a signal handler.
+class StackFrameIteratorForProfiler : public StackFrameIteratorBase {
  public:
   StackFrameIteratorForProfiler(Isolate* isolate, Address pc, Address fp,
                                 Address sp, Address lr, Address js_entry_sp);
@@ -1461,6 +1605,16 @@ class V8_EXPORT_PRIVATE StackFrameIteratorForProfiler
   void AdvanceOneFrame();
 
   bool IsValidStackAddress(Address addr) const {
+#if V8_ENABLE_WEBASSEMBLY
+    if (V8_UNLIKELY(v8_flags.experimental_wasm_stack_switching)) {
+      wasm::StackMemory* head = wasm_stacks_;
+      if (head->Contains(addr)) return true;
+      for (wasm::StackMemory* current = head->next(); current != head;
+           current = current->next()) {
+        if (current->Contains(addr)) return true;
+      }
+    }
+#endif
     return low_bound_ <= addr && addr <= high_bound_;
   }
   bool IsValidFrame(StackFrame* frame) const;
@@ -1482,6 +1636,24 @@ class V8_EXPORT_PRIVATE StackFrameIteratorForProfiler
   StackFrame::Type top_frame_type_;
   ExternalCallbackScope* external_callback_scope_;
   Address top_link_register_;
+#if V8_ENABLE_WEBASSEMBLY
+  wasm::StackMemory* wasm_stacks_;
+#endif
+};
+
+// We cannot export 'StackFrameIteratorForProfiler' for cctests since the
+// linker inserted symbol stub may cuase a stack overflow
+// (https://crbug.com/1449195).
+// We subclass it and export the subclass instead.
+class V8_EXPORT_PRIVATE StackFrameIteratorForProfilerForTesting
+    : public StackFrameIteratorForProfiler {
+ public:
+  StackFrameIteratorForProfilerForTesting(Isolate* isolate, Address pc,
+                                          Address fp, Address sp, Address lr,
+                                          Address js_entry_sp);
+  // Re-declare methods needed by the test. Otherwise we'd have to
+  // export individual methods on the base class (which we don't want to risk).
+  void Advance();
 };
 
 // Frame layout helper classes. Used by the deoptimizer and instruction
@@ -1570,6 +1742,29 @@ class ConstructStubFrameInfo {
   uint32_t frame_size_in_bytes_;
 };
 
+class FastConstructStubFrameInfo {
+ public:
+  static FastConstructStubFrameInfo Precise(bool is_topmost) {
+    return FastConstructStubFrameInfo(is_topmost);
+  }
+
+  static FastConstructStubFrameInfo Conservative() {
+    // Assume it is the top most frame when conservative.
+    return FastConstructStubFrameInfo(true);
+  }
+
+  uint32_t frame_size_in_bytes_without_fixed() const {
+    return frame_size_in_bytes_without_fixed_;
+  }
+  uint32_t frame_size_in_bytes() const { return frame_size_in_bytes_; }
+
+ private:
+  explicit FastConstructStubFrameInfo(bool is_topmost);
+
+  uint32_t frame_size_in_bytes_without_fixed_;
+  uint32_t frame_size_in_bytes_;
+};
+
 // Used by BuiltinContinuationFrameInfo.
 class CallInterfaceDescriptor;
 class RegisterConfiguration;
