@@ -8,6 +8,7 @@
 
 #include "include/cppgc/internal/name-trait.h"
 #include "include/cppgc/trace-trait.h"
+#include "include/cppgc/visitor.h"
 #include "include/v8-cppgc.h"
 #include "include/v8-profiler.h"
 #include "src/api/api-inl.h"
@@ -364,11 +365,11 @@ void* ExtractEmbedderDataBackref(Isolate* isolate, CppHeap& cpp_heap,
   if (!v8_value->IsObject()) return nullptr;
 
   Handle<Object> v8_object = Utils::OpenHandle(*v8_value);
-  if (!v8_object->IsJSObject() ||
-      !JSObject::cast(*v8_object).MayHaveEmbedderFields())
+  if (!IsJSObject(*v8_object) ||
+      !JSObject::cast(*v8_object)->MayHaveEmbedderFields())
     return nullptr;
 
-  JSObject js_object = JSObject::cast(*v8_object);
+  Tagged<JSObject> js_object = JSObject::cast(*v8_object);
 
   const auto maybe_info =
       WrappableInfo::From(isolate, js_object, cpp_heap.wrapper_descriptor());
@@ -472,53 +473,48 @@ class CppGraphBuilderImpl final {
     DCHECK(parent.IsVisibleNotDependent());
     v8::Local<v8::Value> v8_value =
         ref.Get(reinterpret_cast<v8::Isolate*>(cpp_heap_.isolate()));
-    if (!v8_value.IsEmpty()) {
-      if (!parent.get_node()) {
-        parent.set_node(AddNode(*parent.header()));
-      }
-      auto* v8_node = graph_.V8Node(v8_value);
-      if (!edge_name.empty()) {
-        graph_.AddEdge(parent.get_node(), v8_node,
-                       parent.get_node()->InternalizeEdgeName(edge_name));
-      } else {
-        graph_.AddEdge(parent.get_node(), v8_node);
-      }
+    if (v8_value.IsEmpty()) return;
 
-      // References that have a class id set may have their internal fields
-      // pointing back to the object. Set up a wrapper node for the graph so
-      // that the snapshot generator  can merge the nodes appropriately.
-      // Even with a set class id, do not set up a wrapper node when the edge
-      // has a specific name.
-      if (!ref.WrapperClassId() || !edge_name.empty()) return;
+    if (!parent.get_node()) {
+      parent.set_node(AddNode(*parent.header()));
+    }
+    auto* v8_node = graph_.V8Node(v8_value);
+    if (!edge_name.empty()) {
+      graph_.AddEdge(parent.get_node(), v8_node,
+                     parent.get_node()->InternalizeEdgeName(edge_name));
+    } else {
+      graph_.AddEdge(parent.get_node(), v8_node);
+    }
 
-      void* back_reference_object = ExtractEmbedderDataBackref(
-          reinterpret_cast<v8::internal::Isolate*>(cpp_heap_.isolate()),
-          cpp_heap_, v8_value);
-      if (back_reference_object) {
-        auto& back_header = HeapObjectHeader::FromObject(back_reference_object);
-        auto& back_state = states_.GetExistingState(back_header);
+    // Don't merge nodes if edges have a name.
+    if (!edge_name.empty()) return;
 
-        // Generally the back reference will point to `parent.header()`. In the
-        // case of global proxy set up the backreference will point to a
-        // different object, which may not have a node at t his point. Merge the
-        // nodes nevertheless as Window objects need to be able to query their
-        // detachedness state.
-        //
-        // TODO(chromium:1218404): See bug description on how to fix this
-        // inconsistency and only merge states when the backref points back
-        // to the same object.
-        if (!back_state.get_node()) {
-          back_state.set_node(AddNode(back_header));
-        }
-        back_state.get_node()->SetWrapperNode(v8_node);
+    // Try to extract the back reference. If the back reference matches
+    // `parent`, then the nodes are merged.
+    void* back_reference_object = ExtractEmbedderDataBackref(
+        reinterpret_cast<v8::internal::Isolate*>(cpp_heap_.isolate()),
+        cpp_heap_, v8_value);
+    if (!back_reference_object) return;
 
-        auto* profiler =
-            reinterpret_cast<Isolate*>(cpp_heap_.isolate())->heap_profiler();
-        if (profiler->HasGetDetachednessCallback()) {
-          back_state.get_node()->SetDetachedness(
-              profiler->GetDetachedness(v8_value, ref.WrapperClassId()));
-        }
-      }
+    auto& back_header = HeapObjectHeader::FromObject(back_reference_object);
+    auto& back_state = states_.GetExistingState(back_header);
+
+    // If the back reference doesn't point to the same header, just return. In
+    // such a case we have stand-alone references to a wrapper.
+    if (parent.header() != back_state.header()) return;
+
+    // Back reference points to parents header. In this case, the nodes should
+    // be merged and query the detachedness state of the embedder.
+    if (!back_state.get_node()) {
+      back_state.set_node(AddNode(back_header));
+    }
+    back_state.get_node()->SetWrapperNode(v8_node);
+
+    auto* profiler =
+        reinterpret_cast<Isolate*>(cpp_heap_.isolate())->heap_profiler();
+    if (profiler->HasGetDetachednessCallback()) {
+      back_state.get_node()->SetDetachedness(
+          profiler->GetDetachedness(v8_value, 0));
     }
   }
 
@@ -783,7 +779,10 @@ class CppGraphBuilderImpl::VisitationItem final : public WorkstackItemBase {
     }
     ParentScope parent_scope(current_);
     VisiblityVisitor object_visitor(graph_builder, parent_scope);
-    current_.header()->Trace(&object_visitor);
+    if (!current_.header()->IsInConstruction()) {
+      // TODO(mlippautz): Handle in construction objects.
+      current_.header()->Trace(&object_visitor);
+    }
     if (!parent_) {
       current_.UnmarkPending();
     }
@@ -867,6 +866,48 @@ void CppGraphBuilderImpl::VisitRootForGraphBuilding(
   AddRootEdge(root, current, loc.ToString());
 }
 
+namespace {
+
+// Visitor adds edges from native stack roots to objects.
+class GraphBuildingStackVisitor
+    : public cppgc::internal::ConservativeTracingVisitor,
+      public ::heap::base::StackVisitor,
+      public cppgc::Visitor {
+ public:
+  GraphBuildingStackVisitor(CppHeap& heap,
+                            GraphBuildingRootVisitor& root_visitor)
+      : cppgc::internal::ConservativeTracingVisitor(heap, *heap.page_backend(),
+                                                    *this),
+        cppgc::Visitor(cppgc::internal::VisitorFactory::CreateKey()),
+        root_visitor_(root_visitor) {}
+
+  void VisitPointer(const void* address) final {
+    // Entry point for stack walk. The conservative visitor dispatches as
+    // follows:
+    // - Fully constructed objects: VisitFullyConstructedConservatively()
+    // - Objects in construction: VisitInConstructionConservatively()
+    TraceConservativelyIfNeeded(address);
+  }
+
+  void VisitFullyConstructedConservatively(HeapObjectHeader& header) final {
+    root_visitor_.VisitRoot(header.ObjectStart(),
+                            {header.ObjectStart(), nullptr},
+                            cppgc::SourceLocation());
+  }
+
+  void VisitInConstructionConservatively(HeapObjectHeader& header,
+                                         TraceConservativelyCallback) final {
+    root_visitor_.VisitRoot(header.ObjectStart(),
+                            {header.ObjectStart(), nullptr},
+                            cppgc::SourceLocation());
+  }
+
+ private:
+  GraphBuildingRootVisitor& root_visitor_;
+};
+
+}  // namespace
+
 void CppGraphBuilderImpl::Run() {
   // Sweeping from a previous GC might still be running, in which case not all
   // pages have been returned to spaces yet.
@@ -887,7 +928,10 @@ void CppGraphBuilderImpl::Run() {
 
     ParentScope parent_scope(state);
     GraphBuildingVisitor object_visitor(*this, parent_scope);
-    state.header()->Trace(&object_visitor);
+    if (!state.header()->IsInConstruction()) {
+      // TODO(mlippautz): Handle in-construction objects.
+      state.header()->Trace(&object_visitor);
+    }
     state.ForAllEphemeronEdges([this, &state](const HeapObjectHeader& value) {
       AddEdge(state, value, "part of key -> value pair in ephemeron table");
     });
@@ -900,18 +944,29 @@ void CppGraphBuilderImpl::Run() {
   });
   // Add roots.
   {
-    ParentScope parent_scope(states_.CreateRootState(AddRootNode("C++ roots")));
+    ParentScope parent_scope(
+        states_.CreateRootState(AddRootNode("C++ Persistent roots")));
     GraphBuildingRootVisitor root_object_visitor(*this, parent_scope);
     cpp_heap_.GetStrongPersistentRegion().Iterate(root_object_visitor);
   }
   {
-    ParentScope parent_scope(
-        states_.CreateRootState(AddRootNode("C++ cross-thread roots")));
+    ParentScope parent_scope(states_.CreateRootState(
+        AddRootNode("C++ CrossThreadPersistent roots")));
     GraphBuildingRootVisitor root_object_visitor(*this, parent_scope);
     cppgc::internal::PersistentRegionLock guard;
     cpp_heap_.GetStrongCrossThreadPersistentRegion().Iterate(
         root_object_visitor);
   }
+  // Only add stack roots in case the callback is not run from generating a
+  // snapshot without stack. This avoids adding false-positive edges when
+  // conservatively scanning the stack.
+  if (cpp_heap_.isolate()->heap()->IsGCWithStack()) {
+    ParentScope parent_scope(
+        states_.CreateRootState(AddRootNode("C++ native stack roots")));
+    GraphBuildingRootVisitor root_object_visitor(*this, parent_scope);
+    GraphBuildingStackVisitor stack_visitor(cpp_heap_, root_object_visitor);
+    cpp_heap_.stack()->IteratePointersUntilMarker(&stack_visitor);
+  }
 }
 
 // static
