@@ -12,9 +12,11 @@
 #include "src/heap/array-buffer-sweeper.h"
 #include "src/heap/basic-memory-chunk.h"
 #include "src/heap/combined-heap.h"
+#include "src/heap/ephemeron-remembered-set.h"
 #include "src/heap/heap-write-barrier-inl.h"
 #include "src/heap/heap.h"
 #include "src/heap/large-spaces.h"
+#include "src/heap/memory-chunk-layout.h"
 #include "src/heap/new-spaces.h"
 #include "src/heap/objects-visiting-inl.h"
 #include "src/heap/paged-spaces.h"
@@ -33,16 +35,17 @@ namespace v8 {
 namespace internal {
 
 namespace {
-thread_local HeapObject pending_layout_change_object = HeapObject();
+thread_local Tagged<HeapObject> pending_layout_change_object =
+    Tagged<HeapObject>();
 }  // namespace
 
 // Verify that all objects are Smis.
 class VerifySmisVisitor final : public RootVisitor {
  public:
   void VisitRootPointers(Root root, const char* description,
-                         FullObjectSlot start, FullObjectSlot end) override {
+                         FullObjectSlot start, FullObjectSlot end) final {
     for (FullObjectSlot current = start; current < end; ++current) {
-      CHECK((*current).IsSmi());
+      CHECK(IsSmi(*current));
     }
   }
 };
@@ -58,53 +61,57 @@ class VerifyPointersVisitor : public ObjectVisitorWithCageBases,
   V8_INLINE explicit VerifyPointersVisitor(Heap* heap)
       : ObjectVisitorWithCageBases(heap), heap_(heap) {}
 
-  void VisitPointers(HeapObject host, ObjectSlot start,
+  void VisitPointers(Tagged<HeapObject> host, ObjectSlot start,
                      ObjectSlot end) override;
-  void VisitPointers(HeapObject host, MaybeObjectSlot start,
+  void VisitPointers(Tagged<HeapObject> host, MaybeObjectSlot start,
                      MaybeObjectSlot end) override;
-  void VisitCodePointer(Code host, CodeObjectSlot slot) override;
-  void VisitCodeTarget(InstructionStream host, RelocInfo* rinfo) override;
-  void VisitEmbeddedPointer(InstructionStream host, RelocInfo* rinfo) override;
+  void VisitInstructionStreamPointer(Tagged<Code> host,
+                                     InstructionStreamSlot slot) override;
+  void VisitCodeTarget(Tagged<InstructionStream> host,
+                       RelocInfo* rinfo) override;
+  void VisitEmbeddedPointer(Tagged<InstructionStream> host,
+                            RelocInfo* rinfo) override;
 
   void VisitRootPointers(Root root, const char* description,
                          FullObjectSlot start, FullObjectSlot end) override;
   void VisitRootPointers(Root root, const char* description,
                          OffHeapObjectSlot start,
                          OffHeapObjectSlot end) override;
-  void VisitMapPointer(HeapObject host) override;
+  void VisitMapPointer(Tagged<HeapObject> host) override;
 
  protected:
-  V8_INLINE void VerifyHeapObjectImpl(HeapObject heap_object);
-  V8_INLINE void VerifyCodeObjectImpl(HeapObject heap_object);
+  V8_INLINE void VerifyHeapObjectImpl(Tagged<HeapObject> heap_object);
+  V8_INLINE void VerifyCodeObjectImpl(Tagged<HeapObject> heap_object);
 
   template <typename TSlot>
   V8_INLINE void VerifyPointersImpl(TSlot start, TSlot end);
 
-  virtual void VerifyPointers(HeapObject host, MaybeObjectSlot start,
+  virtual void VerifyPointers(Tagged<HeapObject> host, MaybeObjectSlot start,
                               MaybeObjectSlot end);
 
   Heap* heap_;
 };
 
-void VerifyPointersVisitor::VisitPointers(HeapObject host, ObjectSlot start,
-                                          ObjectSlot end) {
+void VerifyPointersVisitor::VisitPointers(Tagged<HeapObject> host,
+                                          ObjectSlot start, ObjectSlot end) {
   VerifyPointers(host, MaybeObjectSlot(start), MaybeObjectSlot(end));
 }
 
-void VerifyPointersVisitor::VisitPointers(HeapObject host,
+void VerifyPointersVisitor::VisitPointers(Tagged<HeapObject> host,
                                           MaybeObjectSlot start,
                                           MaybeObjectSlot end) {
   VerifyPointers(host, start, end);
 }
 
-void VerifyPointersVisitor::VisitCodePointer(Code host, CodeObjectSlot slot) {
-  Object maybe_code = slot.load(code_cage_base());
-  HeapObject code;
+void VerifyPointersVisitor::VisitInstructionStreamPointer(
+    Tagged<Code> host, InstructionStreamSlot slot) {
+  Tagged<Object> maybe_code = slot.load(code_cage_base());
+  Tagged<HeapObject> code;
   // The slot might contain smi during Code creation.
   if (maybe_code.GetHeapObject(&code)) {
     VerifyCodeObjectImpl(code);
   } else {
-    CHECK(maybe_code.IsSmi());
+    CHECK(IsSmi(maybe_code));
   }
 }
 
@@ -122,19 +129,23 @@ void VerifyPointersVisitor::VisitRootPointers(Root root,
   VerifyPointersImpl(start, end);
 }
 
-void VerifyPointersVisitor::VisitMapPointer(HeapObject host) {
-  VerifyHeapObjectImpl(host.map(cage_base()));
+void VerifyPointersVisitor::VisitMapPointer(Tagged<HeapObject> host) {
+  VerifyHeapObjectImpl(host->map(cage_base()));
 }
 
-void VerifyPointersVisitor::VerifyHeapObjectImpl(HeapObject heap_object) {
+void VerifyPointersVisitor::VerifyHeapObjectImpl(
+    Tagged<HeapObject> heap_object) {
   CHECK(IsValidHeapObject(heap_, heap_object));
-  CHECK(heap_object.map(cage_base()).IsMap());
+  CHECK(IsMap(heap_object->map(cage_base())));
+  CHECK_IMPLIES(Heap::InYoungGeneration(heap_object),
+                Heap::InToPage(heap_object));
 }
 
-void VerifyPointersVisitor::VerifyCodeObjectImpl(HeapObject heap_object) {
+void VerifyPointersVisitor::VerifyCodeObjectImpl(
+    Tagged<HeapObject> heap_object) {
   CHECK(IsValidCodeObject(heap_, heap_object));
-  CHECK(heap_object.map(cage_base()).IsMap());
-  CHECK(heap_object.map(cage_base()).instance_type() ==
+  CHECK(IsMap(heap_object->map(cage_base())));
+  CHECK(heap_object->map(cage_base())->instance_type() ==
         INSTRUCTION_STREAM_TYPE);
 }
 
@@ -142,35 +153,38 @@ template <typename TSlot>
 void VerifyPointersVisitor::VerifyPointersImpl(TSlot start, TSlot end) {
   for (TSlot slot = start; slot < end; ++slot) {
     typename TSlot::TObject object = slot.load(cage_base());
-    HeapObject heap_object;
+#ifdef V8_ENABLE_DIRECT_LOCAL
+    if (object.ptr() == kTaggedNullAddress) continue;
+#endif
+    Tagged<HeapObject> heap_object;
     if (object.GetHeapObject(&heap_object)) {
       VerifyHeapObjectImpl(heap_object);
     } else {
-      CHECK(object.IsSmi() || object.IsCleared() ||
+      CHECK(IsSmi(object) || object.IsCleared() ||
             MapWord::IsPacked(object.ptr()));
     }
   }
 }
 
-void VerifyPointersVisitor::VerifyPointers(HeapObject host,
+void VerifyPointersVisitor::VerifyPointers(Tagged<HeapObject> host,
                                            MaybeObjectSlot start,
                                            MaybeObjectSlot end) {
   // If this DCHECK fires then you probably added a pointer field
   // to one of objects in DATA_ONLY_VISITOR_ID_LIST. You can fix
   // this by moving that object to POINTER_VISITOR_ID_LIST.
   DCHECK_EQ(ObjectFields::kMaybePointers,
-            Map::ObjectFieldsFrom(host.map(cage_base()).visitor_id()));
+            Map::ObjectFieldsFrom(host->map(cage_base())->visitor_id()));
   VerifyPointersImpl(start, end);
 }
 
-void VerifyPointersVisitor::VisitCodeTarget(InstructionStream host,
+void VerifyPointersVisitor::VisitCodeTarget(Tagged<InstructionStream> host,
                                             RelocInfo* rinfo) {
-  InstructionStream target =
+  Tagged<InstructionStream> target =
       InstructionStream::FromTargetAddress(rinfo->target_address());
   VerifyHeapObjectImpl(target);
 }
 
-void VerifyPointersVisitor::VisitEmbeddedPointer(InstructionStream host,
+void VerifyPointersVisitor::VisitEmbeddedPointer(Tagged<InstructionStream> host,
                                                  RelocInfo* rinfo) {
   VerifyHeapObjectImpl(rinfo->target_object(cage_base()));
 }
@@ -181,16 +195,16 @@ class VerifyReadOnlyPointersVisitor : public VerifyPointersVisitor {
       : VerifyPointersVisitor(heap) {}
 
  protected:
-  void VerifyPointers(HeapObject host, MaybeObjectSlot start,
+  void VerifyPointers(Tagged<HeapObject> host, MaybeObjectSlot start,
                       MaybeObjectSlot end) override {
     if (!host.is_null()) {
-      CHECK(ReadOnlyHeap::Contains(host.map()));
+      CHECK(ReadOnlyHeap::Contains(host->map()));
     }
     VerifyPointersVisitor::VerifyPointers(host, start, end);
 
     for (MaybeObjectSlot current = start; current < end; ++current) {
-      HeapObject heap_object;
-      if ((*current)->GetHeapObject(&heap_object)) {
+      Tagged<HeapObject> heap_object;
+      if ((*current).GetHeapObject(&heap_object)) {
         CHECK(ReadOnlyHeap::Contains(heap_object));
       }
     }
@@ -208,17 +222,17 @@ class VerifySharedHeapObjectVisitor : public VerifyPointersVisitor {
   }
 
  protected:
-  void VerifyPointers(HeapObject host, MaybeObjectSlot start,
+  void VerifyPointers(Tagged<HeapObject> host, MaybeObjectSlot start,
                       MaybeObjectSlot end) override {
     if (!host.is_null()) {
-      Map map = host.map();
+      Tagged<Map> map = host->map();
       CHECK(ReadOnlyHeap::Contains(map) || shared_space_->Contains(map));
     }
     VerifyPointersVisitor::VerifyPointers(host, start, end);
 
     for (MaybeObjectSlot current = start; current < end; ++current) {
-      HeapObject heap_object;
-      if ((*current)->GetHeapObject(&heap_object)) {
+      Tagged<HeapObject> heap_object;
+      if ((*current).GetHeapObject(&heap_object)) {
         CHECK(ReadOnlyHeap::Contains(heap_object) ||
               shared_space_->Contains(heap_object) ||
               shared_lo_space_->Contains(heap_object));
@@ -246,19 +260,17 @@ class HeapVerification final : public SpaceVerificationVisitor {
   void VerifyPage(const BasicMemoryChunk* chunk) final;
   void VerifyPageDone(const BasicMemoryChunk* chunk) final;
 
-  void VerifyObject(HeapObject object) final;
-  void VerifyObjectMap(HeapObject object);
-  void VerifyOutgoingPointers(HeapObject object);
-  // Verifies OLD_TO_NEW and OLD_TO_SHARED remembered sets for this object.
-  void VerifyRememberedSetFor(HeapObject object);
-
-  void VerifyInvalidatedObjectSize();
+  void VerifyObject(Tagged<HeapObject> object) final;
+  void VerifyObjectMap(Tagged<HeapObject> object);
+  void VerifyOutgoingPointers(Tagged<HeapObject> object);
+  // Verifies OLD_TO_NEW, OLD_TO_NEW_BACKGROUND and OLD_TO_SHARED remembered
+  // sets for this object.
+  void VerifyRememberedSetFor(Tagged<HeapObject> object);
 
   ReadOnlySpace* read_only_space() const { return heap_->read_only_space(); }
   NewSpace* new_space() const { return heap_->new_space(); }
   OldSpace* old_space() const { return heap_->old_space(); }
   SharedSpace* shared_space() const { return heap_->shared_space(); }
-
   CodeSpace* code_space() const { return heap_->code_space(); }
   LargeObjectSpace* lo_space() const { return heap_->lo_space(); }
   SharedLargeObjectSpace* shared_lo_space() const {
@@ -266,6 +278,10 @@ class HeapVerification final : public SpaceVerificationVisitor {
   }
   CodeLargeObjectSpace* code_lo_space() const { return heap_->code_lo_space(); }
   NewLargeObjectSpace* new_lo_space() const { return heap_->new_lo_space(); }
+  TrustedSpace* trusted_space() const { return heap_->trusted_space(); }
+  TrustedLargeObjectSpace* trusted_lo_space() const {
+    return heap_->trusted_lo_space();
+  }
 
   Isolate* isolate() const { return isolate_; }
   Heap* heap() const { return heap_; }
@@ -284,7 +300,6 @@ class HeapVerification final : public SpaceVerificationVisitor {
 void HeapVerification::Verify() {
   CHECK(heap()->HasBeenSetUp());
   AllowGarbageCollection allow_gc;
-  IgnoreLocalGCRequests ignore_gc_requests(heap());
   SafepointKind safepoint_kind = isolate()->is_shared_space_isolate()
                                      ? SafepointKind::kGlobal
                                      : SafepointKind::kIsolate;
@@ -292,6 +307,7 @@ void HeapVerification::Verify() {
   HandleScope scope(isolate());
 
   heap()->MakeHeapIterable();
+  heap()->FreeLinearAllocationAreas();
 
   // TODO(v8:13257): Currently we don't iterate through the stack conservatively
   // when verifying the heap.
@@ -301,12 +317,12 @@ void HeapVerification::Verify() {
 
   if (!isolate()->context().is_null() &&
       !isolate()->raw_native_context().is_null()) {
-    Object normalized_map_cache =
-        isolate()->raw_native_context().normalized_map_cache();
+    Tagged<Object> normalized_map_cache =
+        isolate()->raw_native_context()->normalized_map_cache();
 
-    if (normalized_map_cache.IsNormalizedMapCache()) {
+    if (IsNormalizedMapCache(normalized_map_cache)) {
       NormalizedMapCache::cast(normalized_map_cache)
-          .NormalizedMapCacheVerify(isolate());
+          ->NormalizedMapCacheVerify(isolate());
     }
   }
 
@@ -330,9 +346,10 @@ void HeapVerification::Verify() {
   VerifySpace(shared_lo_space());
   VerifySpace(code_lo_space());
 
-  isolate()->string_table()->VerifyIfOwnedBy(isolate());
+  VerifySpace(trusted_space());
+  VerifySpace(trusted_lo_space());
 
-  VerifyInvalidatedObjectSize();
+  isolate()->string_table()->VerifyIfOwnedBy(isolate());
 
 #if DEBUG
   heap()->VerifyCommittedPhysicalMemory();
@@ -349,7 +366,7 @@ void HeapVerification::VerifySpace(BaseSpace* space) {
 void HeapVerification::VerifyPage(const BasicMemoryChunk* chunk) {
   CHECK(!current_chunk_.has_value());
   CHECK(!chunk->IsFlagSet(Page::PAGE_NEW_OLD_PROMOTION));
-  CHECK(!chunk->IsFlagSet(Page::PAGE_NEW_NEW_PROMOTION));
+  CHECK(!chunk->IsFlagSet(Page::FROM_PAGE));
   if (V8_SHARED_RO_HEAP_BOOL && chunk->InReadOnlySpace()) {
     CHECK_NULL(chunk->owner());
   } else {
@@ -364,82 +381,63 @@ void HeapVerification::VerifyPageDone(const BasicMemoryChunk* chunk) {
   current_chunk_.reset();
 }
 
-void HeapVerification::VerifyObject(HeapObject object) {
+void HeapVerification::VerifyObject(Tagged<HeapObject> object) {
   CHECK_EQ(BasicMemoryChunk::FromHeapObject(object), *current_chunk_);
 
   // Verify object map.
   VerifyObjectMap(object);
 
   // The object itself should look OK.
-  object.ObjectVerify(isolate_);
+  Object::ObjectVerify(object, isolate_);
 
   // Verify outgoing references.
   VerifyOutgoingPointers(object);
 
   // Verify remembered set.
-  VerifyRememberedSetFor(object);
+  if (!heap_->incremental_marking()->IsMinorMarking()) {
+    // Minor incremental marking "steals" the remembered sets from pages.
+    VerifyRememberedSetFor(object);
+  }
 }
 
-void HeapVerification::VerifyOutgoingPointers(HeapObject object) {
+void HeapVerification::VerifyOutgoingPointers(Tagged<HeapObject> object) {
   switch (current_space_identity()) {
     case RO_SPACE: {
       VerifyReadOnlyPointersVisitor visitor(heap());
-      object.Iterate(cage_base_, &visitor);
+      object->Iterate(cage_base_, &visitor);
       break;
     }
 
     case SHARED_SPACE:
     case SHARED_LO_SPACE: {
       VerifySharedHeapObjectVisitor visitor(heap());
-      object.Iterate(cage_base_, &visitor);
+      object->Iterate(cage_base_, &visitor);
       break;
     }
 
     default: {
       VerifyPointersVisitor visitor(heap());
-      object.Iterate(cage_base_, &visitor);
+      object->Iterate(cage_base_, &visitor);
       break;
     }
   }
 }
 
-void HeapVerification::VerifyObjectMap(HeapObject object) {
+void HeapVerification::VerifyObjectMap(Tagged<HeapObject> object) {
   // The first word should be a map, and we expect all map pointers to be
   // in map space or read-only space.
-  Map map = object.map(cage_base_);
-  CHECK(map.IsMap(cage_base_));
+  Tagged<Map> map = object->map(cage_base_);
+  CHECK(IsMap(map, cage_base_));
   CHECK(ReadOnlyHeap::Contains(map) || old_space()->Contains(map) ||
         (shared_space() && shared_space()->Contains(map)));
 
   if (Heap::InYoungGeneration(object)) {
     // The object should not be code or a map.
-    CHECK(!object.IsMap(cage_base_));
-    CHECK(!object.IsAbstractCode(cage_base_));
+    CHECK(!IsMap(object, cage_base_));
+    CHECK(!IsAbstractCode(object, cage_base_));
   } else if (current_space_identity() == RO_SPACE) {
-    CHECK(!object.IsExternalString());
-    CHECK(!object.IsJSArrayBuffer());
-  }
-}
-
-namespace {
-void VerifyInvalidatedSlots(InvalidatedSlots* invalidated_slots) {
-  if (!invalidated_slots) return;
-  for (std::pair<HeapObject, int> object_and_size : *invalidated_slots) {
-    HeapObject object = object_and_size.first;
-    int size = object_and_size.second;
-    CHECK_EQ(object.Size(), size);
-  }
-}
-}  // namespace
-
-void HeapVerification::VerifyInvalidatedObjectSize() {
-  OldGenerationMemoryChunkIterator chunk_iterator(heap());
-  MemoryChunk* chunk;
-
-  while ((chunk = chunk_iterator.next()) != nullptr) {
-    VerifyInvalidatedSlots(chunk->invalidated_slots<OLD_TO_NEW>());
-    VerifyInvalidatedSlots(chunk->invalidated_slots<OLD_TO_OLD>());
-    VerifyInvalidatedSlots(chunk->invalidated_slots<OLD_TO_SHARED>());
+    CHECK(!IsExternalString(object));
+    CHECK(!IsJSArrayBuffer(object));
   }
 }
 
@@ -454,20 +452,21 @@ class SlotVerifyingVisitor : public ObjectVisitorWithCageBases {
                        std::set<std::pair<SlotType, Address>>* typed)
       : ObjectVisitorWithCageBases(isolate), untyped_(untyped), typed_(typed) {}
 
-  virtual bool ShouldHaveBeenRecorded(HeapObject host, MaybeObject target) = 0;
+  virtual bool ShouldHaveBeenRecorded(Tagged<HeapObject> host,
+                                      MaybeObject target) = 0;
 
-  void VisitPointers(HeapObject host, ObjectSlot start,
+  void VisitPointers(Tagged<HeapObject> host, ObjectSlot start,
                      ObjectSlot end) override {
 #ifdef DEBUG
     for (ObjectSlot slot = start; slot < end; ++slot) {
-      Object obj = slot.load(cage_base());
+      Tagged<Object> obj = slot.load(cage_base());
       CHECK(!MapWord::IsPacked(obj.ptr()) || !HasWeakHeapObjectTag(obj));
     }
 #endif  // DEBUG
     VisitPointers(host, MaybeObjectSlot(start), MaybeObjectSlot(end));
   }
 
-  void VisitPointers(HeapObject host, MaybeObjectSlot start,
+  void VisitPointers(Tagged<HeapObject> host, MaybeObjectSlot start,
                      MaybeObjectSlot end) final {
     for (MaybeObjectSlot slot = start; slot < end; ++slot) {
       if (ShouldHaveBeenRecorded(host, slot.load(cage_base()))) {
@@ -476,15 +475,17 @@ class SlotVerifyingVisitor : public ObjectVisitorWithCageBases {
     }
   }
 
-  void VisitCodePointer(Code host, CodeObjectSlot slot) override {
+  void VisitInstructionStreamPointer(Tagged<Code> host,
+                                     InstructionStreamSlot slot) override {
     if (ShouldHaveBeenRecorded(
             host, MaybeObject::FromObject(slot.load(code_cage_base())))) {
       CHECK_GT(untyped_->count(slot.address()), 0);
     }
   }
 
-  void VisitCodeTarget(InstructionStream host, RelocInfo* rinfo) override {
-    Object target =
+  void VisitCodeTarget(Tagged<InstructionStream> host,
+                       RelocInfo* rinfo) override {
+    Tagged<Object> target =
         InstructionStream::FromTargetAddress(rinfo->target_address());
     if (ShouldHaveBeenRecorded(host, MaybeObject::FromObject(target))) {
       CHECK(InTypedSet(SlotType::kCodeEntry, rinfo->pc()) ||
@@ -494,8 +495,9 @@ class SlotVerifyingVisitor : public ObjectVisitorWithCageBases {
     }
   }
 
-  void VisitEmbeddedPointer(InstructionStream host, RelocInfo* rinfo) override {
-    Object target = rinfo->target_object(cage_base());
+  void VisitEmbeddedPointer(Tagged<InstructionStream> host,
+                            RelocInfo* rinfo) override {
+    Tagged<Object> target = rinfo->target_object(cage_base());
     if (ShouldHaveBeenRecorded(host, MaybeObject::FromObject(target))) {
       CHECK(InTypedSet(SlotType::kEmbeddedObjectFull, rinfo->pc()) ||
             InTypedSet(SlotType::kEmbeddedObjectCompressed, rinfo->pc()) ||
@@ -523,28 +525,30 @@ class SlotVerifyingVisitor : public ObjectVisitorWithCageBases {
 
 class OldToNewSlotVerifyingVisitor : public SlotVerifyingVisitor {
  public:
-  OldToNewSlotVerifyingVisitor(Isolate* isolate, std::set<Address>* untyped,
-                               std::set<std::pair<SlotType, Address>>* typed,
-                               EphemeronRememberedSet* ephemeron_remembered_set)
+  OldToNewSlotVerifyingVisitor(
+      Isolate* isolate, std::set<Address>* untyped,
+      std::set<std::pair<SlotType, Address>>* typed,
+      EphemeronRememberedSet::TableMap* ephemeron_remembered_set)
       : SlotVerifyingVisitor(isolate, untyped, typed),
         ephemeron_remembered_set_(ephemeron_remembered_set) {}
 
-  bool ShouldHaveBeenRecorded(HeapObject host, MaybeObject target) override {
+  bool ShouldHaveBeenRecorded(Tagged<HeapObject> host,
+                              MaybeObject target) override {
     DCHECK_IMPLIES(target->IsStrongOrWeak() && Heap::InYoungGeneration(target),
                    Heap::InToPage(target));
     return target->IsStrongOrWeak() && Heap::InYoungGeneration(target) &&
            !Heap::InYoungGeneration(host);
   }
 
-  void VisitEphemeron(HeapObject host, int index, ObjectSlot key,
+  void VisitEphemeron(Tagged<HeapObject> host, int index, ObjectSlot key,
                       ObjectSlot target) override {
     VisitPointer(host, target);
-    if (v8_flags.minor_mc) return;
+    if (v8_flags.minor_ms) return;
     // Keys are handled separately and should never appear in this set.
     CHECK(!InUntypedSet(key));
-    Object k = *key;
+    Tagged<Object> k = *key;
     if (!ObjectInYoungGeneration(host) && ObjectInYoungGeneration(k)) {
-      EphemeronHashTable table = EphemeronHashTable::cast(host);
+      Tagged<EphemeronHashTable> table = EphemeronHashTable::cast(host);
       auto it = ephemeron_remembered_set_->find(table);
       CHECK(it != ephemeron_remembered_set_->end());
       int slot_index =
@@ -555,7 +559,7 @@ class OldToNewSlotVerifyingVisitor : public SlotVerifyingVisitor {
   }
 
  private:
-  EphemeronRememberedSet* ephemeron_remembered_set_;
+  EphemeronRememberedSet::TableMap* ephemeron_remembered_set_;
 };
 
 class OldToSharedSlotVerifyingVisitor : public SlotVerifyingVisitor {
@@ -564,9 +568,12 @@ class OldToSharedSlotVerifyingVisitor : public SlotVerifyingVisitor {
                                   std::set<std::pair<SlotType, Address>>* typed)
       : SlotVerifyingVisitor(isolate, untyped, typed) {}
 
-  bool ShouldHaveBeenRecorded(HeapObject host, MaybeObject target) override {
-    return target->IsStrongOrWeak() && Heap::InWritableSharedSpace(target) &&
-           !Heap::InYoungGeneration(host) && !host.InWritableSharedSpace();
+  bool ShouldHaveBeenRecorded(Tagged<HeapObject> host,
+                              MaybeObject target) override {
+    Tagged<HeapObject> target_heap_object;
+    return target.GetHeapObject(&target_heap_object) &&
+           InWritableSharedSpace(target_heap_object) &&
+           !Heap::InYoungGeneration(host) && !InWritableSharedSpace(host);
   }
 };
 
@@ -595,50 +602,54 @@ void CollectSlots(MemoryChunk* chunk, Address start, Address end,
 // Helper class for collecting slot addresses.
 class SlotCollectingVisitor final : public ObjectVisitor {
  public:
-  void VisitPointers(HeapObject host, ObjectSlot start,
+  void VisitPointers(Tagged<HeapObject> host, ObjectSlot start,
                      ObjectSlot end) override {
     VisitPointers(host, MaybeObjectSlot(start), MaybeObjectSlot(end));
   }
-  void VisitPointers(HeapObject host, MaybeObjectSlot start,
+  void VisitPointers(Tagged<HeapObject> host, MaybeObjectSlot start,
                      MaybeObjectSlot end) final {
     for (MaybeObjectSlot p = start; p < end; ++p) {
       slots_.push_back(p);
     }
   }
 
-  void VisitCodePointer(Code host, CodeObjectSlot slot) override {
+  void VisitInstructionStreamPointer(Tagged<Code> host,
+                                     InstructionStreamSlot slot) override {
     CHECK(V8_EXTERNAL_CODE_SPACE_BOOL);
 #ifdef V8_EXTERNAL_CODE_SPACE
     code_slots_.push_back(slot);
 #endif
   }
 
-  void VisitCodeTarget(InstructionStream host, RelocInfo* rinfo) final {
+  void VisitCodeTarget(Tagged<InstructionStream> host, RelocInfo* rinfo) final {
     UNREACHABLE();
   }
 
-  void VisitEmbeddedPointer(InstructionStream host, RelocInfo* rinfo) override {
+  void VisitEmbeddedPointer(Tagged<InstructionStream> host,
+                            RelocInfo* rinfo) override {
     UNREACHABLE();
   }
 
-  void VisitMapPointer(HeapObject object) override {}  // do nothing by default
+  void VisitMapPointer(Tagged<HeapObject> object) override {
+    slots_.push_back(MaybeObjectSlot(object->map_slot()));
+  }
 
   int number_of_slots() { return static_cast<int>(slots_.size()); }
 
   MaybeObjectSlot slot(int i) { return slots_[i]; }
 #ifdef V8_EXTERNAL_CODE_SPACE
-  CodeObjectSlot code_slot(int i) { return code_slots_[i]; }
+  InstructionStreamSlot code_slot(int i) { return code_slots_[i]; }
   int number_of_code_slots() { return static_cast<int>(code_slots_.size()); }
 #endif
 
  private:
   std::vector<MaybeObjectSlot> slots_;
 #ifdef V8_EXTERNAL_CODE_SPACE
-  std::vector<CodeObjectSlot> code_slots_;
+  std::vector<InstructionStreamSlot> code_slots_;
 #endif
 };
 
-void HeapVerification::VerifyRememberedSetFor(HeapObject object) {
+void HeapVerification::VerifyRememberedSetFor(Tagged<HeapObject> object) {
   if (current_space_identity() == RO_SPACE ||
       v8_flags.verify_heap_skip_remembered_set) {
     return;
@@ -647,15 +658,18 @@ void HeapVerification::VerifyRememberedSetFor(HeapObject object) {
   MemoryChunk* chunk = MemoryChunk::FromHeapObject(object);
 
   Address start = object.address();
-  Address end = start + object.Size(cage_base_);
+  Address end = start + object->Size(cage_base_);
 
   std::set<Address> old_to_new;
   std::set<std::pair<SlotType, Address>> typed_old_to_new;
   CollectSlots<OLD_TO_NEW>(chunk, start, end, &old_to_new, &typed_old_to_new);
+  CollectSlots<OLD_TO_NEW_BACKGROUND>(chunk, start, end, &old_to_new,
+                                      &typed_old_to_new);
+
   OldToNewSlotVerifyingVisitor old_to_new_visitor(
       isolate(), &old_to_new, &typed_old_to_new,
-      &heap()->ephemeron_remembered_set_);
-  object.IterateBody(cage_base_, &old_to_new_visitor);
+      heap()->ephemeron_remembered_set()->tables());
+  object->IterateBody(cage_base_, &old_to_new_visitor);
 
   std::set<Address> old_to_shared;
   std::set<std::pair<SlotType, Address>> typed_old_to_shared;
@@ -663,20 +677,26 @@ void HeapVerification::VerifyRememberedSetFor(HeapObject object) {
                               &typed_old_to_shared);
   OldToSharedSlotVerifyingVisitor old_to_shared_visitor(
       isolate(), &old_to_shared, &typed_old_to_shared);
-  object.IterateBody(cage_base_, &old_to_shared_visitor);
+  object->IterateBody(cage_base_, &old_to_shared_visitor);
 
-  if (object.InWritableSharedSpace()) {
+  if (InWritableSharedSpace(object)) {
     CHECK_NULL(chunk->slot_set<OLD_TO_SHARED>());
     CHECK_NULL(chunk->typed_slot_set<OLD_TO_SHARED>());
 
     CHECK_NULL(chunk->slot_set<OLD_TO_NEW>());
     CHECK_NULL(chunk->typed_slot_set<OLD_TO_NEW>());
+
+    CHECK_NULL(chunk->slot_set<OLD_TO_NEW_BACKGROUND>());
+    CHECK_NULL(chunk->typed_slot_set<OLD_TO_NEW_BACKGROUND>());
   }
 
   if (Heap::InYoungGeneration(object)) {
     CHECK_NULL(chunk->slot_set<OLD_TO_NEW>());
     CHECK_NULL(chunk->typed_slot_set<OLD_TO_NEW>());
 
+    CHECK_NULL(chunk->slot_set<OLD_TO_NEW_BACKGROUND>());
+    CHECK_NULL(chunk->typed_slot_set<OLD_TO_NEW_BACKGROUND>());
+
     CHECK_NULL(chunk->slot_set<OLD_TO_OLD>());
     CHECK_NULL(chunk->typed_slot_set<OLD_TO_OLD>());
 
@@ -702,13 +722,13 @@ void HeapVerifier::VerifyReadOnlyHeap(Heap* heap) {
 }
 
 // static
-void HeapVerifier::VerifyObjectLayoutChangeIsAllowed(Heap* heap,
-                                                     HeapObject object) {
-  if (object.InWritableSharedSpace()) {
+void HeapVerifier::VerifyObjectLayoutChangeIsAllowed(
+    Heap* heap, Tagged<HeapObject> object) {
+  if (InWritableSharedSpace(object)) {
     // Out of objects in the shared heap, only strings can change layout.
-    DCHECK(object.IsString());
+    DCHECK(IsString(object));
     // Shared strings only change layout under GC, never concurrently.
-    if (object.IsShared()) {
+    if (IsShared(object)) {
       Isolate* isolate = heap->isolate();
       Isolate* shared_space_isolate = isolate->is_shared_space_isolate()
                                           ? isolate
@@ -721,15 +741,17 @@ void HeapVerifier::VerifyObjectLayoutChangeIsAllowed(Heap* heap,
 }
 
 // static
-void HeapVerifier::SetPendingLayoutChangeObject(Heap* heap, HeapObject object) {
+void HeapVerifier::SetPendingLayoutChangeObject(Heap* heap,
+                                                Tagged<HeapObject> object) {
   VerifyObjectLayoutChangeIsAllowed(heap, object);
   DCHECK(pending_layout_change_object.is_null());
   pending_layout_change_object = object;
 }
 
 // static
-void HeapVerifier::VerifyObjectLayoutChange(Heap* heap, HeapObject object,
-                                            Map new_map) {
+void HeapVerifier::VerifyObjectLayoutChange(Heap* heap,
+                                            Tagged<HeapObject> object,
+                                            Tagged<Map> new_map) {
   // Object layout changes are currently not supported on background threads.
   DCHECK_NULL(LocalHeap::Current());
 
@@ -752,25 +774,27 @@ void HeapVerifier::VerifyObjectLayoutChange(Heap* heap, HeapObject object,
 }
 
 // static
-void HeapVerifier::VerifySafeMapTransition(Heap* heap, HeapObject object,
-                                           Map new_map) {
+void HeapVerifier::VerifySafeMapTransition(Heap* heap,
+                                           Tagged<HeapObject> object,
+                                           Tagged<Map> new_map) {
   PtrComprCageBase cage_base(heap->isolate());
 
-  if (object.IsJSObject(cage_base)) {
+  if (IsJSObject(object, cage_base)) {
     // Without double unboxing all in-object fields of a JSObject are tagged.
     return;
   }
 
-  if (object.IsString(cage_base) &&
-      new_map == ReadOnlyRoots(heap).thin_string_map()) {
+  if (IsString(object, cage_base) &&
+      (new_map == ReadOnlyRoots(heap).thin_two_byte_string_map() ||
+       new_map == ReadOnlyRoots(heap).thin_one_byte_string_map())) {
     // When transitioning a string to ThinString,
     // Heap::NotifyObjectLayoutChange doesn't need to be invoked because only
     // tagged fields are introduced.
     return;
   }
 
-  if (v8_flags.shared_string_table && object.IsString(cage_base) &&
-      InstanceTypeChecker::IsInternalizedString(new_map.instance_type())) {
+  if (v8_flags.shared_string_table && IsString(object, cage_base) &&
+      InstanceTypeChecker::IsInternalizedString(new_map->instance_type())) {
     // In-place internalization does not change a string's fields.
     //
     // When sharing the string table, the setting and re-setting of maps below
@@ -781,14 +805,14 @@ void HeapVerifier::VerifySafeMapTransition(Heap* heap, HeapObject object,
 
   // Check that the set of slots before and after the transition match.
   SlotCollectingVisitor old_visitor;
-  object.IterateFast(cage_base, &old_visitor);
-  MapWord old_map_word = object.map_word(cage_base, kRelaxedLoad);
+  object->IterateFast(cage_base, &old_visitor);
+  MapWord old_map_word = object->map_word(cage_base, kRelaxedLoad);
   // Temporarily set the new map to iterate new slots.
-  object.set_map_word(new_map, kRelaxedStore);
+  object->set_map_word(new_map, kRelaxedStore);
   SlotCollectingVisitor new_visitor;
-  object.IterateFast(cage_base, &new_visitor);
+  object->IterateFast(cage_base, &new_visitor);
   // Restore the old map.
-  object.set_map_word(old_map_word.ToMap(), kRelaxedStore);
+  object->set_map_word(old_map_word.ToMap(), kRelaxedStore);
   DCHECK_EQ(new_visitor.number_of_slots(), old_visitor.number_of_slots());
   for (int i = 0; i < new_visitor.number_of_slots(); i++) {
     DCHECK_EQ(new_visitor.slot(i), old_visitor.slot(i));
