@@ -33,17 +33,21 @@ size_t MemoryAllocator::commit_page_size_bits_ = 0;
 
 MemoryAllocator::MemoryAllocator(Isolate* isolate,
                                  v8::PageAllocator* code_page_allocator,
+                                 v8::PageAllocator* trusted_page_allocator,
                                  size_t capacity)
     : isolate_(isolate),
       data_page_allocator_(isolate->page_allocator()),
       code_page_allocator_(code_page_allocator),
+      trusted_page_allocator_(trusted_page_allocator),
       capacity_(RoundUp(capacity, Page::kPageSize)),
-      unmapper_(isolate->heap(), this) {
-  DCHECK_NOT_NULL(code_page_allocator);
+      pool_(this) {
+  DCHECK_NOT_NULL(data_page_allocator_);
+  DCHECK_NOT_NULL(code_page_allocator_);
+  DCHECK_NOT_NULL(trusted_page_allocator_);
 }
 
 void MemoryAllocator::TearDown() {
-  unmapper()->TearDown();
+  pool()->ReleasePooledChunks();
 
   // Check that spaces were torn down before MemoryAllocator.
   DCHECK_EQ(size_, 0u);
@@ -57,176 +61,35 @@ void MemoryAllocator::TearDown() {
 
   code_page_allocator_ = nullptr;
   data_page_allocator_ = nullptr;
+  trusted_page_allocator_ = nullptr;
 }
 
-class MemoryAllocator::Unmapper::UnmapFreeMemoryJob : public JobTask {
- public:
-  explicit UnmapFreeMemoryJob(Isolate* isolate, Unmapper* unmapper)
-      : unmapper_(unmapper),
-        tracer_(isolate->heap()->tracer()),
-        trace_id_(reinterpret_cast<uint64_t>(this) ^
-                  tracer_->CurrentEpoch(GCTracer::Scope::BACKGROUND_UNMAPPER)) {
+void MemoryAllocator::Pool::ReleasePooledChunks() {
+  std::vector<MemoryChunk*> copied_pooled;
+  {
+    base::MutexGuard guard(&mutex_);
+    std::swap(copied_pooled, pooled_chunks_);
   }
-
-  UnmapFreeMemoryJob(const UnmapFreeMemoryJob&) = delete;
-  UnmapFreeMemoryJob& operator=(const UnmapFreeMemoryJob&) = delete;
-
-  void Run(JobDelegate* delegate) override {
-    if (delegate->IsJoiningThread()) {
-      TRACE_GC_WITH_FLOW(tracer_, GCTracer::Scope::UNMAPPER, trace_id_,
-                         TRACE_EVENT_FLAG_FLOW_IN);
-      RunImpl(delegate);
-
-    } else {
-      TRACE_GC1_WITH_FLOW(tracer_, GCTracer::Scope::BACKGROUND_UNMAPPER,
-                          ThreadKind::kBackground, trace_id_,
-                          TRACE_EVENT_FLAG_FLOW_IN);
-      RunImpl(delegate);
-    }
-  }
-
-  size_t GetMaxConcurrency(size_t worker_count) const override {
-    const size_t kTaskPerChunk = 8;
-    return std::min<size_t>(
-        kMaxUnmapperTasks,
-        worker_count +
-            (unmapper_->NumberOfCommittedChunks() + kTaskPerChunk - 1) /
-                kTaskPerChunk);
-  }
-
-  uint64_t trace_id() const { return trace_id_; }
-
- private:
-  void RunImpl(JobDelegate* delegate) {
-    unmapper_->PerformFreeMemoryOnQueuedChunks(FreeMode::kUncommitPooled,
-                                               delegate);
-    if (v8_flags.trace_unmapper) {
-      PrintIsolate(unmapper_->heap_->isolate(), "UnmapFreeMemoryTask Done\n");
-    }
-  }
-  Unmapper* const unmapper_;
-  GCTracer* const tracer_;
-  const uint64_t trace_id_;
-};
-
-void MemoryAllocator::Unmapper::FreeQueuedChunks() {
-  if (NumberOfChunks() == 0) return;
-
-  if (!heap_->IsTearingDown() && v8_flags.concurrent_sweeping) {
-    if (job_handle_ && job_handle_->IsValid()) {
-      job_handle_->NotifyConcurrencyIncrease();
-    } else {
-      auto job = std::make_unique<UnmapFreeMemoryJob>(heap_->isolate(), this);
-      TRACE_GC_NOTE_WITH_FLOW("MemoryAllocator::Unmapper started",
-                              job->trace_id(), TRACE_EVENT_FLAG_FLOW_OUT);
-      job_handle_ = V8::GetCurrentPlatform()->PostJob(
-          TaskPriority::kUserVisible, std::move(job));
-      if (v8_flags.trace_unmapper) {
-        PrintIsolate(heap_->isolate(), "Unmapper::FreeQueuedChunks: new Job\n");
-      }
-    }
-  } else {
-    PerformFreeMemoryOnQueuedChunks(FreeMode::kUncommitPooled);
+  for (auto* chunk : copied_pooled) {
+    DCHECK_NOT_NULL(chunk);
+    VirtualMemory* reservation = chunk->reserved_memory();
+    DCHECK(reservation->IsReserved());
+    reservation->Free();
   }
 }
 
-void MemoryAllocator::Unmapper::CancelAndWaitForPendingTasks() {
-  if (job_handle_ && job_handle_->IsValid()) job_handle_->Join();
-
-  if (v8_flags.trace_unmapper) {
-    PrintIsolate(
-        heap_->isolate(),
-        "Unmapper::CancelAndWaitForPendingTasks: no tasks remaining\n");
-  }
-}
-
-void MemoryAllocator::Unmapper::PrepareForGC() {
-  // Free non-regular chunks because they cannot be re-used.
-  PerformFreeMemoryOnQueuedNonRegularChunks();
-}
-
-void MemoryAllocator::Unmapper::EnsureUnmappingCompleted() {
-  CancelAndWaitForPendingTasks();
-  PerformFreeMemoryOnQueuedChunks(FreeMode::kFreePooled);
-}
-
-void MemoryAllocator::Unmapper::PerformFreeMemoryOnQueuedNonRegularChunks(
-    JobDelegate* delegate) {
-  MemoryChunk* chunk = nullptr;
-  while ((chunk = GetMemoryChunkSafe(ChunkQueueType::kNonRegular)) != nullptr) {
-    allocator_->PerformFreeMemory(chunk);
-    if (delegate && delegate->ShouldYield()) return;
-  }
-}
-
-void MemoryAllocator::Unmapper::PerformFreeMemoryOnQueuedChunks(
-    MemoryAllocator::Unmapper::FreeMode mode, JobDelegate* delegate) {
-  MemoryChunk* chunk = nullptr;
-  if (v8_flags.trace_unmapper) {
-    PrintIsolate(
-        heap_->isolate(),
-        "Unmapper::PerformFreeMemoryOnQueuedChunks: %d queued chunks\n",
-        NumberOfChunks());
-  }
-  // Regular chunks.
-  while ((chunk = GetMemoryChunkSafe(ChunkQueueType::kRegular)) != nullptr) {
-    bool pooled = chunk->IsFlagSet(MemoryChunk::POOLED);
-    allocator_->PerformFreeMemory(chunk);
-    if (pooled) AddMemoryChunkSafe(ChunkQueueType::kPooled, chunk);
-    if (delegate && delegate->ShouldYield()) return;
-  }
-  if (mode == MemoryAllocator::Unmapper::FreeMode::kFreePooled) {
-    // The previous loop uncommitted any pages marked as pooled and added them
-    // to the pooled list. In case of kFreePooled we need to free them though as
-    // well.
-    while ((chunk = GetMemoryChunkSafe(ChunkQueueType::kPooled)) != nullptr) {
-      allocator_->FreePooledChunk(chunk);
-      if (delegate && delegate->ShouldYield()) return;
-    }
-  }
-  PerformFreeMemoryOnQueuedNonRegularChunks();
-}
-
-void MemoryAllocator::Unmapper::TearDown() {
-  CHECK(!job_handle_ || !job_handle_->IsValid());
-  PerformFreeMemoryOnQueuedChunks(FreeMode::kFreePooled);
-  for (int i = 0; i < ChunkQueueType::kNumberOfChunkQueues; i++) {
-    DCHECK(chunks_[i].empty());
-  }
-}
-
-size_t MemoryAllocator::Unmapper::NumberOfCommittedChunks() {
+size_t MemoryAllocator::Pool::NumberOfCommittedChunks() const {
   base::MutexGuard guard(&mutex_);
-  return chunks_[ChunkQueueType::kRegular].size() +
-         chunks_[ChunkQueueType::kNonRegular].size();
+  return pooled_chunks_.size();
 }
 
-int MemoryAllocator::Unmapper::NumberOfChunks() {
+int MemoryAllocator::Pool::NumberOfChunks() const {
   base::MutexGuard guard(&mutex_);
-  size_t result = 0;
-  for (int i = 0; i < ChunkQueueType::kNumberOfChunkQueues; i++) {
-    result += chunks_[i].size();
-  }
-  return static_cast<int>(result);
+  return static_cast<int>(pooled_chunks_.size());
 }
 
-size_t MemoryAllocator::Unmapper::CommittedBufferedMemory() {
-  base::MutexGuard guard(&mutex_);
-
-  size_t sum = 0;
-  // kPooled chunks are already uncommited. We only have to account for
-  // kRegular and kNonRegular chunks.
-  for (auto& chunk : chunks_[ChunkQueueType::kRegular]) {
-    sum += chunk->size();
-  }
-  for (auto& chunk : chunks_[ChunkQueueType::kNonRegular]) {
-    sum += chunk->size();
-  }
-  return sum;
-}
-
-bool MemoryAllocator::Unmapper::IsRunning() const {
-  return job_handle_ && job_handle_->IsValid();
+size_t MemoryAllocator::Pool::CommittedBufferedMemory() const {
+  return NumberOfCommittedChunks() * Page::kPageSize;
 }
 
 bool MemoryAllocator::CommitMemory(VirtualMemory* reservation,
@@ -258,7 +121,9 @@ Address MemoryAllocator::AllocateAlignedMemory(
     size_t chunk_size, size_t area_size, size_t alignment,
     AllocationSpace space, Executability executable, void* hint,
     VirtualMemory* controller) {
-  v8::PageAllocator* page_allocator = this->page_allocator(executable);
+  DCHECK_EQ(space == CODE_SPACE || space == CODE_LO_SPACE,
+            executable == EXECUTABLE);
+  v8::PageAllocator* page_allocator = this->page_allocator(space);
   DCHECK_LT(area_size, chunk_size);
 
   VirtualMemory reservation(page_allocator, chunk_size, hint, alignment);
@@ -508,7 +373,7 @@ void MemoryAllocator::FreeReadOnlyPage(ReadOnlyPage* chunk) {
 
   UnregisterSharedBasicMemoryChunk(chunk);
 
-  v8::PageAllocator* allocator = page_allocator(NOT_EXECUTABLE);
+  v8::PageAllocator* allocator = page_allocator(RO_SPACE);
   VirtualMemory* reservation = chunk->reserved_memory();
   if (reservation->IsReserved()) {
     reservation->FreeReadOnly();
@@ -544,12 +409,8 @@ void MemoryAllocator::PerformFreeMemory(MemoryChunk* chunk) {
   chunk->ReleaseAllAllocatedMemory();
 
   VirtualMemory* reservation = chunk->reserved_memory();
-  if (chunk->IsFlagSet(MemoryChunk::POOLED)) {
-    UncommitMemory(reservation);
-  } else {
-    DCHECK(reservation->IsReserved());
-    reservation->Free();
-  }
+  DCHECK(reservation->IsReserved());
+  reservation->Free();
 }
 
 void MemoryAllocator::Free(MemoryAllocator::FreeMode mode, MemoryChunk* chunk) {
@@ -563,35 +424,27 @@ void MemoryAllocator::Free(MemoryAllocator::FreeMode mode, MemoryChunk* chunk) {
       PreFreeMemory(chunk);
       PerformFreeMemory(chunk);
       break;
-    case FreeMode::kConcurrentlyAndPool:
+    case FreeMode::kPostpone:
+      PreFreeMemory(chunk);
+      // Record page to be freed later.
+      queued_pages_to_be_freed_.push_back(chunk);
+      break;
+    case FreeMode::kPool:
       DCHECK_EQ(chunk->size(), static_cast<size_t>(MemoryChunk::kPageSize));
       DCHECK_EQ(chunk->executable(), NOT_EXECUTABLE);
-      chunk->SetFlag(MemoryChunk::POOLED);
-      V8_FALLTHROUGH;
-    case FreeMode::kConcurrently:
       PreFreeMemory(chunk);
-      // The chunks added to this queue will be freed by a concurrent thread.
-      unmapper()->AddMemoryChunkSafe(chunk);
+      // The chunks added to this queue will be cached until memory reducing GC.
+      pool()->Add(chunk);
       break;
   }
 }
 
-void MemoryAllocator::FreePooledChunk(MemoryChunk* chunk) {
-  // Pooled pages cannot be touched anymore as their memory is uncommitted.
-  // Pooled pages are not-executable.
-  FreeMemoryRegion(data_page_allocator(), chunk->address(),
-                   static_cast<size_t>(MemoryChunk::kPageSize));
-}
-
 Page* MemoryAllocator::AllocatePage(MemoryAllocator::AllocationMode alloc_mode,
                                     Space* space, Executability executable) {
-  size_t size =
+  const size_t size =
       MemoryChunkLayout::AllocatableMemoryInMemoryChunk(space->identity());
   base::Optional<MemoryChunkAllocationResult> chunk_info;
   if (alloc_mode == AllocationMode::kUsePool) {
-    DCHECK_EQ(size, static_cast<size_t>(
-                        MemoryChunkLayout::AllocatableMemoryInMemoryChunk(
-                            space->identity())));
     DCHECK_EQ(executable, NOT_EXECUTABLE);
     chunk_info = AllocateUninitializedPageFromPool(space);
   }
@@ -658,7 +511,7 @@ LargePage* MemoryAllocator::AllocateLargePage(LargeObjectSpace* space,
 
 base::Optional<MemoryAllocator::MemoryChunkAllocationResult>
 MemoryAllocator::AllocateUninitializedPageFromPool(Space* space) {
-  void* chunk = unmapper()->TryGetPooledMemoryChunkSafe();
+  void* chunk = pool()->TryGetPooled();
   if (chunk == nullptr) return {};
   const int size = MemoryChunk::kPageSize;
   const Address start = reinterpret_cast<Address>(chunk);
@@ -668,8 +521,8 @@ MemoryAllocator::AllocateUninitializedPageFromPool(Space* space) {
   const Address area_end = start + size;
   // Pooled pages are always regular data pages.
   DCHECK_NE(CODE_SPACE, space->identity());
+  DCHECK_NE(TRUSTED_SPACE, space->identity());
   VirtualMemory reservation(data_page_allocator(), start, size);
-  if (!CommitMemory(&reservation, NOT_EXECUTABLE)) return {};
   if (heap::ShouldZapGarbage()) {
     heap::ZapBlock(start, size, kZapValue);
   }
@@ -868,5 +721,12 @@ void MemoryAllocator::RecordLargePageDestroyed(const LargePage& page) {
 #endif  // V8_ENABLE_CONSERVATIVE_STACK_SCANNING
 }
 
+void MemoryAllocator::ReleaseQueuedPages() {
+  for (auto* chunk : queued_pages_to_be_freed_) {
+    PerformFreeMemory(chunk);
+  }
+  queued_pages_to_be_freed_.clear();
+}
+
 }  // namespace internal
 }  // namespace v8
