@@ -9,6 +9,7 @@
 #include <limits>
 #include <memory>
 #include <utility>
+#include <variant>
 
 #include "src/base/bounds.h"
 #include "src/base/macros.h"
@@ -35,6 +36,78 @@ class ObjectVisitor;
 class PagedSpaceBase;
 class Sweeper;
 
+class HeapObjectRange final {
+ public:
+  class iterator final {
+   public:
+    using value_type = Tagged<HeapObject>;
+    using pointer = const value_type*;
+    using reference = const value_type&;
+    using iterator_category = std::forward_iterator_tag;
+
+    inline iterator();
+    explicit inline iterator(const Page* page);
+
+    inline iterator& operator++();
+    inline iterator operator++(int);
+
+    bool operator==(iterator other) const {
+      return cur_addr_ == other.cur_addr_;
+    }
+    bool operator!=(iterator other) const { return !(*this == other); }
+
+    value_type operator*() { return HeapObject::FromAddress(cur_addr_); }
+
+   private:
+    inline void AdvanceToNextObject();
+
+    PtrComprCageBase cage_base() const { return cage_base_; }
+
+    PtrComprCageBase cage_base_;
+    Address cur_addr_ = kNullAddress;  // Current iteration point.
+    int cur_size_ = 0;
+    Address cur_end_ = kNullAddress;  // End iteration point.
+  };
+
+  explicit HeapObjectRange(const Page* page) : page_(page) {}
+
+  inline iterator begin();
+  inline iterator end();
+
+ private:
+  const Page* const page_;
+};
+
+// Heap object iterator in paged spaces.
+//
+// A PagedSpaceObjectIterator iterates objects from the bottom of the given
+// space to its top or from the bottom of the given page to its top.
+//
+// If objects are allocated in the page during iteration the iterator may
+// or may not iterate over those objects.  The caller must create a new
+// iterator in order to be sure to visit these new objects.
+class V8_EXPORT_PRIVATE PagedSpaceObjectIterator : public ObjectIterator {
+ public:
+  // Creates a new object iterator in a given space.
+  PagedSpaceObjectIterator(Heap* heap, const PagedSpaceBase* space);
+
+  // Advance to the next object, skipping free spaces and other fillers and
+  // skipping the special garbage section of which there is one per space.
+  // Returns nullptr when the iteration has ended.
+  inline Tagged<HeapObject> Next() override;
+
+ private:
+  // Slow path of next(), goes into the next page.  Returns false if the
+  // iteration has ended.
+  bool AdvanceToNextPage();
+
+  HeapObjectRange::iterator cur_;
+  HeapObjectRange::iterator end_;
+  const PagedSpaceBase* const space_;
+  ConstPageRange page_range_;
+  ConstPageRange::iterator current_page_;
+};
+
 class V8_EXPORT_PRIVATE PagedSpaceBase
     : NON_EXPORTED_BASE(public SpaceWithLinearArea) {
  public:
@@ -43,19 +116,15 @@ class V8_EXPORT_PRIVATE PagedSpaceBase
 
   static const size_t kCompactionMemoryWanted = 500 * KB;
 
-  // Creates a space with an id.
-  PagedSpaceBase(
-      Heap* heap, AllocationSpace id, Executability executable,
-      FreeList* free_list, AllocationCounter& allocation_counter,
-      LinearAllocationArea& allocation_info,
-      LinearAreaOriginalData& linear_area_original_data,
-      CompactionSpaceKind compaction_space_kind = CompactionSpaceKind::kNone);
+  PagedSpaceBase(Heap* heap, AllocationSpace id, Executability executable,
+                 std::unique_ptr<FreeList> free_list,
+                 CompactionSpaceKind compaction_space_kind);
 
   ~PagedSpaceBase() override { TearDown(); }
 
   // Checks whether an object/address is in this space.
   inline bool Contains(Address a) const;
-  inline bool Contains(Object o) const;
+  inline bool Contains(Tagged<Object> o) const;
   bool ContainsSlow(Address addr) const;
 
   // Does the space need executable memory?
@@ -106,47 +175,13 @@ class V8_EXPORT_PRIVATE PagedSpaceBase
   RawAllocateBackground(LocalHeap* local_heap, size_t min_size_in_bytes,
                         size_t max_size_in_bytes, AllocationOrigin origin);
 
-  size_t Free(Address start, size_t size_in_bytes, SpaceAccountingMode mode) {
-    if (size_in_bytes == 0) return 0;
-    heap()->CreateFillerObjectAtBackground(start,
-                                           static_cast<int>(size_in_bytes));
-    if (mode == SpaceAccountingMode::kSpaceAccounted) {
-      return AccountedFree(start, size_in_bytes);
-    } else {
-      return UnaccountedFree(start, size_in_bytes);
-    }
-  }
-
-  // Give a block of memory to the space's free list.  It might be added to
-  // the free list or accounted as waste.
-  // If add_to_freelist is false then just accounting stats are updated and
-  // no attempt to add area to free list is made.
-  size_t AccountedFree(Address start, size_t size_in_bytes) {
-    size_t wasted = free_list_->Free(start, size_in_bytes, kLinkCategory);
-    Page* page = Page::FromAddress(start);
-    accounting_stats_.DecreaseAllocatedBytes(size_in_bytes, page);
-    DCHECK_GE(size_in_bytes, wasted);
-    return size_in_bytes - wasted;
-  }
-
-  size_t UnaccountedFree(Address start, size_t size_in_bytes) {
-    size_t wasted = free_list_->Free(start, size_in_bytes, kDoNotLinkCategory);
-    DCHECK_GE(size_in_bytes, wasted);
-    return size_in_bytes - wasted;
-  }
-
-  inline bool TryFreeLast(Address object_address, int object_size);
+  // Free a block of memory. During sweeping, we don't update the accounting
+  // stats and don't link the free list category.
+  V8_INLINE size_t Free(Address start, size_t size_in_bytes);
+  V8_INLINE size_t FreeDuringSweep(Address start, size_t size_in_bytes);
 
   void ResetFreeList();
 
-  // Empty space linear allocation area, returning unused area to free list.
-  void FreeLinearAllocationArea() override;
-
-  void MakeLinearAllocationAreaIterable();
-
-  void MarkLinearAllocationAreaBlack();
-  void UnmarkLinearAllocationArea();
-
   void DecreaseAllocatedBytes(size_t bytes, Page* page) {
     accounting_stats_.DecreaseAllocatedBytes(bytes, page);
   }
@@ -172,18 +207,6 @@ class V8_EXPORT_PRIVATE PagedSpaceBase
   // be used for allocation.
   Page* RemovePageSafe(int size_in_bytes);
 
-  void SetReadable();
-  void SetReadAndExecutable();
-  void SetCodeModificationPermissions();
-
-  void SetDefaultCodePermissions() {
-    if (v8_flags.jitless) {
-      SetReadable();
-    } else {
-      SetReadAndExecutable();
-    }
-  }
-
 #ifdef VERIFY_HEAP
   // Verify integrity of this space.
   void Verify(Isolate* isolate,
@@ -225,7 +248,7 @@ class V8_EXPORT_PRIVATE PagedSpaceBase
 
   // Refills the free list from the corresponding free list filled by the
   // sweeper.
-  virtual void RefillFreeList() = 0;
+  virtual void RefillFreeList();
 
   base::Mutex* mutex() { return &space_mutex_; }
 
@@ -260,59 +283,16 @@ class V8_EXPORT_PRIVATE PagedSpaceBase
 
   std::unique_ptr<ObjectIterator> GetObjectIterator(Heap* heap) override;
 
-  void SetLinearAllocationArea(Address top, Address limit, Address end);
-
   void AddRangeToActiveSystemPages(Page* page, Address start, Address end);
   void ReduceActiveSystemPages(Page* page,
                                ActiveSystemPages active_system_pages);
 
-  // Allocates memory with the given size constraints from the space's free
-  // list.
-  V8_WARN_UNUSED_RESULT base::Optional<std::pair<Address, size_t>>
-  TryAllocationFromFreeListBackground(size_t min_size_in_bytes,
-                                      size_t max_size_in_bytes,
-                                      AllocationOrigin origin);
-
-  // Expands the space by a single page from a background thread and allocates
-  // a memory area of the given size in it. If successful the method returns
-  // the address and size of the area.
-  base::Optional<std::pair<Address, size_t>> TryExpandBackground(
-      size_t size_in_bytes);
-
- private:
-  class ConcurrentAllocationMutex {
-   public:
-    explicit ConcurrentAllocationMutex(const PagedSpaceBase* space) {
-      if (space->SupportsConcurrentAllocation()) {
-        guard_.emplace(&space->space_mutex_);
-      }
-    }
-
-    base::Optional<base::MutexGuard> guard_;
-  };
-
-  bool SupportsConcurrentAllocation() const {
-    return !is_compaction_space() && (identity() != NEW_SPACE);
-  }
-
-  // Set space linear allocation area.
-  void SetTopAndLimit(Address top, Address limit, Address end);
-  void DecreaseLimit(Address new_limit);
-  bool SupportsAllocationObserver() const override {
-    return !is_compaction_space();
-  }
-
- protected:
-  // Updates the current lab limit without updating top, original_top or
-  // original_limit.
-  void SetLimit(Address limit);
-
-  bool SupportsExtendingLAB() const { return identity() == NEW_SPACE; }
+  // Expands the space by a single page and returns true on success.
+  bool TryExpand(LocalHeap* local_heap, AllocationOrigin origin);
 
   void RefineAllocatedBytesAfterSweeping(Page* page);
 
-  void UpdateInlineAllocationLimit() override;
-
+ protected:
   // PagedSpaces that should be included in snapshots have different, i.e.,
   // smaller, initial pages.
   virtual bool snapshotable() const { return true; }
@@ -323,38 +303,8 @@ class V8_EXPORT_PRIVATE PagedSpaceBase
   // to the initial chunk, uncommits addresses in the initial chunk.
   void TearDown();
 
-  // Expands the space by allocating a fixed number of pages. Returns false if
-  // it cannot allocate requested number of pages from OS, or if the hard heap
-  // size limit has been hit.
-  virtual Page* TryExpandImpl(MemoryAllocator::AllocationMode allocation_mode);
-
-  bool EnsureAllocation(int size_in_bytes, AllocationAlignment alignment,
-                        AllocationOrigin origin,
-                        int* out_max_aligned_size) override;
-
-  V8_WARN_UNUSED_RESULT bool TryAllocationFromFreeListMain(
-      size_t size_in_bytes, AllocationOrigin origin);
-
-  V8_WARN_UNUSED_RESULT bool ContributeToSweepingMain(
-      int required_freed_bytes, int max_pages, int size_in_bytes,
-      AllocationOrigin origin, GCTracer::Scope::ScopeId sweeping_scope_id,
-      ThreadKind sweeping_scope_kind);
-
-  // Refills LAB for EnsureLabMain. This function is space-dependent. Returns
-  // false if there is not enough space and the caller has to retry after
-  // collecting garbage.
-  V8_WARN_UNUSED_RESULT virtual bool RefillLabMain(int size_in_bytes,
-                                                   AllocationOrigin origin);
-
-  // Actual implementation of refilling LAB. Returns false if there is not
-  // enough space and the caller has to retry after collecting garbage.
-  V8_WARN_UNUSED_RESULT bool RawRefillLabMain(int size_in_bytes,
-                                              AllocationOrigin origin);
-
-  V8_WARN_UNUSED_RESULT bool TryExtendLAB(int size_in_bytes);
-
-  V8_WARN_UNUSED_RESULT bool TryExpand(int size_in_bytes,
-                                       AllocationOrigin origin);
+  // Spaces can use this method to get notified about pages added to it.
+  virtual void NotifyNewPage(Page* page) {}
 
   size_t committed_physical_memory() const {
     return committed_physical_memory_.load(std::memory_order_relaxed);
@@ -362,6 +312,8 @@ class V8_EXPORT_PRIVATE PagedSpaceBase
 
   void ReleasePageImpl(Page* page, MemoryAllocator::FreeMode free_mode);
 
+  void AddPageImpl(Page* page);
+
   Executability executable_;
 
   CompactionSpaceKind compaction_space_kind_;
@@ -376,8 +328,31 @@ class V8_EXPORT_PRIVATE PagedSpaceBase
 
   std::atomic<size_t> committed_physical_memory_{0};
 
+  // Used for tracking bytes allocated since last gc in new space.
+  size_t size_at_last_gc_ = 0;
+
+ private:
+  template <bool during_sweep>
+  V8_INLINE size_t FreeInternal(Address start, size_t size_in_bytes);
+
+  class ConcurrentAllocationMutex {
+   public:
+    explicit ConcurrentAllocationMutex(const PagedSpaceBase* space) {
+      if (space->SupportsConcurrentAllocation()) {
+        guard_.emplace(&space->space_mutex_);
+      }
+    }
+
+    base::Optional<base::MutexGuard> guard_;
+  };
+
+  bool SupportsConcurrentAllocation() const {
+    return !is_compaction_space() && (identity() != NEW_SPACE);
+  }
+
   friend class IncrementalMarking;
   friend class MarkCompactCollector;
+  friend class PagedSpaceAllocatorPolicy;
 
   // Used in cctest.
   friend class heap::HeapTester;
@@ -385,20 +360,13 @@ class V8_EXPORT_PRIVATE PagedSpaceBase
 
 class V8_EXPORT_PRIVATE PagedSpace : public PagedSpaceBase {
  public:
-  // Creates a space with an id.
-  PagedSpace(
-      Heap* heap, AllocationSpace id, Executability executable,
-      FreeList* free_list, LinearAllocationArea& allocation_info,
-      CompactionSpaceKind compaction_space_kind = CompactionSpaceKind::kNone)
-      : PagedSpaceBase(heap, id, executable, free_list, allocation_counter_,
-                       allocation_info, linear_area_original_data_,
+  PagedSpace(Heap* heap, AllocationSpace id, Executability executable,
+             std::unique_ptr<FreeList> free_list,
+             CompactionSpaceKind compaction_space_kind)
+      : PagedSpaceBase(heap, id, executable, std::move(free_list),
                        compaction_space_kind) {}
 
-  void RefillFreeList() final;
-
- private:
-  AllocationCounter allocation_counter_;
-  LinearAreaOriginalData linear_area_original_data_;
+  AllocatorPolicy* CreateAllocatorPolicy(MainAllocator* allocator) final;
 };
 
 // -----------------------------------------------------------------------------
@@ -409,20 +377,17 @@ class V8_EXPORT_PRIVATE CompactionSpace final : public PagedSpace {
   CompactionSpace(Heap* heap, AllocationSpace id, Executability executable,
                   CompactionSpaceKind compaction_space_kind)
       : PagedSpace(heap, id, executable, FreeList::CreateFreeList(),
-                   allocation_info_, compaction_space_kind) {
+                   compaction_space_kind) {
     DCHECK(is_compaction_space());
   }
 
   const std::vector<Page*>& GetNewPages() { return new_pages_; }
 
- private:
-  LinearAllocationArea allocation_info_;
+  void RefillFreeList() final;
 
  protected:
-  V8_WARN_UNUSED_RESULT bool RefillLabMain(int size_in_bytes,
-                                           AllocationOrigin origin) override;
+  void NotifyNewPage(Page* page) final;
 
-  Page* TryExpandImpl(MemoryAllocator::AllocationMode allocation_mode) final;
   // The space is temporary and not included in any snapshots.
   bool snapshotable() const final { return false; }
   // Pages that were allocated in this local space and need to be merged
@@ -434,13 +399,7 @@ class V8_EXPORT_PRIVATE CompactionSpace final : public PagedSpace {
 class CompactionSpaceCollection : public Malloced {
  public:
   explicit CompactionSpaceCollection(Heap* heap,
-                                     CompactionSpaceKind compaction_space_kind)
-      : old_space_(heap, OLD_SPACE, Executability::NOT_EXECUTABLE,
-                   compaction_space_kind),
-        code_space_(heap, CODE_SPACE, Executability::EXECUTABLE,
-                    compaction_space_kind),
-        shared_space_(heap, SHARED_SPACE, Executability::NOT_EXECUTABLE,
-                      compaction_space_kind) {}
+                                     CompactionSpaceKind compaction_space_kind);
 
   CompactionSpace* Get(AllocationSpace space) {
     switch (space) {
@@ -450,6 +409,8 @@ class CompactionSpaceCollection : public Malloced {
         return &code_space_;
       case SHARED_SPACE:
         return &shared_space_;
+      case TRUSTED_SPACE:
+        return &trusted_space_;
       default:
         UNREACHABLE();
     }
@@ -460,28 +421,33 @@ class CompactionSpaceCollection : public Malloced {
   CompactionSpace old_space_;
   CompactionSpace code_space_;
   CompactionSpace shared_space_;
+  CompactionSpace trusted_space_;
 };
 
 // -----------------------------------------------------------------------------
 // Old generation regular object space.
 
-class OldSpace final : public PagedSpace {
+class V8_EXPORT_PRIVATE OldSpace final : public PagedSpace {
  public:
   // Creates an old space object. The constructor does not allocate pages
   // from OS.
-  explicit OldSpace(Heap* heap, LinearAllocationArea& allocation_info)
+  explicit OldSpace(Heap* heap)
       : PagedSpace(heap, OLD_SPACE, NOT_EXECUTABLE, FreeList::CreateFreeList(),
-                   allocation_info) {}
+                   CompactionSpaceKind::kNone) {}
 
   static bool IsAtPageStart(Address addr) {
     return static_cast<intptr_t>(addr & kPageAlignmentMask) ==
            MemoryChunkLayout::ObjectStartOffsetInDataPage();
   }
 
+  void AddPromotedPage(Page* page);
+
+  void ReleasePage(Page* page) override;
+
   size_t ExternalBackingStoreBytes(ExternalBackingStoreType type) const final {
     if (type == ExternalBackingStoreType::kArrayBuffer)
       return heap()->OldArrayBufferBytes();
-    return external_backing_store_bytes_[type];
+    return external_backing_store_bytes_[static_cast<int>(type)];
   }
 };
 
@@ -490,14 +456,11 @@ class OldSpace final : public PagedSpace {
 
 class CodeSpace final : public PagedSpace {
  public:
-  // Creates an old space object. The constructor does not allocate pages
-  // from OS.
+  // Creates a code space object. The constructor does not allocate pages from
+  // OS.
   explicit CodeSpace(Heap* heap)
       : PagedSpace(heap, CODE_SPACE, EXECUTABLE, FreeList::CreateFreeList(),
-                   paged_allocation_info_) {}
-
- private:
-  LinearAllocationArea paged_allocation_info_;
+                   CompactionSpaceKind::kNone) {}
 };
 
 // -----------------------------------------------------------------------------
@@ -505,11 +468,11 @@ class CodeSpace final : public PagedSpace {
 
 class SharedSpace final : public PagedSpace {
  public:
-  // Creates an old space object. The constructor does not allocate pages
-  // from OS.
+  // Creates a shared space object. The constructor does not allocate pages from
+  // OS.
   explicit SharedSpace(Heap* heap)
       : PagedSpace(heap, SHARED_SPACE, NOT_EXECUTABLE,
-                   FreeList::CreateFreeList(), allocation_info) {}
+                   FreeList::CreateFreeList(), CompactionSpaceKind::kNone) {}
 
   static bool IsAtPageStart(Address addr) {
     return static_cast<intptr_t>(addr & kPageAlignmentMask) ==
@@ -519,11 +482,34 @@ class SharedSpace final : public PagedSpace {
   size_t ExternalBackingStoreBytes(ExternalBackingStoreType type) const final {
     if (type == ExternalBackingStoreType::kArrayBuffer) return 0;
     DCHECK_EQ(type, ExternalBackingStoreType::kExternalString);
-    return external_backing_store_bytes_[type];
+    return external_backing_store_bytes_[static_cast<int>(type)];
+  }
+};
+
+// -----------------------------------------------------------------------------
+// Trusted space.
+// Essentially another old space that, when the sandbox is enabled, will be
+// located outside of the sandbox. As such an attacker cannot corrupt objects
+// located in this space and therefore these objects can be considered trusted.
+
+class TrustedSpace final : public PagedSpace {
+ public:
+  // Creates a trusted space object. The constructor does not allocate pages
+  // from OS.
+  explicit TrustedSpace(Heap* heap)
+      : PagedSpace(heap, TRUSTED_SPACE, NOT_EXECUTABLE,
+                   FreeList::CreateFreeList(), CompactionSpaceKind::kNone) {}
+
+  static bool IsAtPageStart(Address addr) {
+    return static_cast<intptr_t>(addr & kPageAlignmentMask) ==
+           MemoryChunkLayout::ObjectStartOffsetInDataPage();
   }
 
- private:
-  LinearAllocationArea allocation_info;
+  size_t ExternalBackingStoreBytes(ExternalBackingStoreType type) const final {
+    if (type == ExternalBackingStoreType::kArrayBuffer) return 0;
+    DCHECK_EQ(type, ExternalBackingStoreType::kExternalString);
+    return external_backing_store_bytes_[static_cast<int>(type)];
+  }
 };
 
 // Iterates over the chunks (pages and large object pages) that can contain
@@ -535,20 +521,29 @@ class OldGenerationMemoryChunkIterator {
   // Return nullptr when the iterator is done.
   inline MemoryChunk* next();
 
+  // Applies `callback` to all `MemoryChunk` returned by the iterator.
+  template <typename Callback>
+  static void ForAll(Heap* heap, Callback callback) {
+    OldGenerationMemoryChunkIterator it(heap);
+    while (MemoryChunk* chunk = it.next()) {
+      callback(chunk);
+    }
+  }
+
  private:
   enum State {
-    kOldSpaceState,
-    kCodeState,
-    kLargeObjectState,
-    kCodeLargeObjectState,
-    kFinishedState
+    kOldSpace,
+    kCodeSpace,
+    kLargeObjectSpace,
+    kCodeLargeObjectSpace,
+    kTrustedSpace,
+    kTrustedLargeObjectSpace,
+    kFinished
   };
   Heap* const heap_;
   State state_;
-  PageIterator old_iterator_;
-  PageIterator code_iterator_;
-  LargePageIterator lo_iterator_;
-  LargePageIterator code_lo_iterator_;
+  // The current type of {iterator_} depends on {state_}.
+  std::variant<PageIterator, LargePageIterator> iterator_;
 };
 
 }  // namespace internal
