@@ -16,18 +16,18 @@ namespace internal {
 namespace {
 
 template <typename BitField>
-Handle<Smi> SetBitFieldValue(Isolate* isolate, Handle<Smi> smi_handler,
+Tagged<Smi> SetBitFieldValue(Isolate* isolate, Tagged<Smi> smi_handler,
                              typename BitField::FieldType value) {
-  int config = smi_handler->value();
+  int config = smi_handler.value();
   config = BitField::update(config, true);
-  return handle(Smi::FromInt(config), isolate);
+  return Smi::FromInt(config);
 }
 
 // TODO(ishell): Remove templatezation once we move common bits from
 // Load/StoreHandler to the base class.
 template <typename ICHandler, bool fill_handler = true>
 int InitPrototypeChecksImpl(Isolate* isolate, Handle<ICHandler> handler,
-                            Handle<Smi>* smi_handler,
+                            Tagged<Smi>* smi_handler,
                             Handle<Map> lookup_start_object_map,
                             MaybeObjectHandle data1,
                             MaybeObjectHandle maybe_data2) {
@@ -35,12 +35,12 @@ int InitPrototypeChecksImpl(Isolate* isolate, Handle<ICHandler> handler,
   // Holder-is-receiver case itself does not add entries unless there is an
   // optional data2 value provided.
 
-  DCHECK_IMPLIES(lookup_start_object_map->IsJSGlobalObjectMap(),
+  DCHECK_IMPLIES(IsJSGlobalObjectMap(*lookup_start_object_map),
                  lookup_start_object_map->is_prototype_map());
 
-  if (lookup_start_object_map->IsPrimitiveMap() ||
+  if (IsPrimitiveMap(*lookup_start_object_map) ||
       lookup_start_object_map->is_access_check_needed()) {
-    DCHECK(!lookup_start_object_map->IsJSGlobalObjectMap());
+    DCHECK(!IsJSGlobalObjectMap(*lookup_start_object_map));
     // The validity cell check for primitive and global proxy receivers does
     // not guarantee that certain native context ever had access to other
     // native context. However, a handler created for one native context could
@@ -58,7 +58,7 @@ int InitPrototypeChecksImpl(Isolate* isolate, Handle<ICHandler> handler,
     }
     data_size++;
   } else if (lookup_start_object_map->is_dictionary_map() &&
-             !lookup_start_object_map->IsJSGlobalObjectMap()) {
+             !IsJSGlobalObjectMap(*lookup_start_object_map)) {
     if (!fill_handler) {
       // Enable lookup on lookup start object.
       *smi_handler =
@@ -90,7 +90,7 @@ int InitPrototypeChecksImpl(Isolate* isolate, Handle<ICHandler> handler,
 // If the |holder| is an empty handle then the full prototype chain is
 // checked.
 template <typename ICHandler>
-int GetHandlerDataSize(Isolate* isolate, Handle<Smi>* smi_handler,
+int GetHandlerDataSize(Isolate* isolate, Tagged<Smi>* smi_handler,
                        Handle<Map> lookup_start_object_map,
                        MaybeObjectHandle data1,
                        MaybeObjectHandle maybe_data2 = MaybeObjectHandle()) {
@@ -114,7 +114,7 @@ void InitPrototypeChecks(Isolate* isolate, Handle<ICHandler> handler,
 // static
 Handle<Object> LoadHandler::LoadFromPrototype(
     Isolate* isolate, Handle<Map> lookup_start_object_map,
-    Handle<JSReceiver> holder, Handle<Smi> smi_handler,
+    Handle<JSReceiver> holder, Tagged<Smi> smi_handler,
     MaybeObjectHandle maybe_data1, MaybeObjectHandle maybe_data2) {
   MaybeObjectHandle data1;
   if (maybe_data1.is_null()) {
@@ -131,7 +131,7 @@ Handle<Object> LoadHandler::LoadFromPrototype(
 
   Handle<LoadHandler> handler = isolate->factory()->NewLoadHandler(data_size);
 
-  handler->set_smi_handler(*smi_handler);
+  handler->set_smi_handler(smi_handler);
   handler->set_validity_cell(*validity_cell);
   InitPrototypeChecks(isolate, handler, lookup_start_object_map, data1,
                       maybe_data2);
@@ -142,25 +142,26 @@ Handle<Object> LoadHandler::LoadFromPrototype(
 Handle<Object> LoadHandler::LoadFullChain(Isolate* isolate,
                                           Handle<Map> lookup_start_object_map,
                                           const MaybeObjectHandle& holder,
-                                          Handle<Smi> smi_handler) {
+                                          Handle<Smi> smi_handler_handle) {
+  Tagged<Smi> smi_handler = *smi_handler_handle;
   MaybeObjectHandle data1 = holder;
   int data_size = GetHandlerDataSize<LoadHandler>(
       isolate, &smi_handler, lookup_start_object_map, data1);
 
   Handle<Object> validity_cell = Map::GetOrCreatePrototypeChainValidityCell(
       lookup_start_object_map, isolate);
-  if (validity_cell->IsSmi()) {
+  if (IsSmi(*validity_cell)) {
     DCHECK_EQ(1, data_size);
     // Lookup on lookup start object isn't supported in case of a simple smi
     // handler.
-    if (!LookupOnLookupStartObjectBits::decode(smi_handler->value())) {
-      return smi_handler;
+    if (!LookupOnLookupStartObjectBits::decode(smi_handler.value())) {
+      return smi_handler_handle;
     }
   }
 
   Handle<LoadHandler> handler = isolate->factory()->NewLoadHandler(data_size);
 
-  handler->set_smi_handler(*smi_handler);
+  handler->set_smi_handler(smi_handler);
   handler->set_validity_cell(*validity_cell);
   InitPrototypeChecks(isolate, handler, lookup_start_object_map, data1);
   return handler;
@@ -169,36 +170,46 @@ Handle<Object> LoadHandler::LoadFullChain(Isolate* isolate,
 // static
 KeyedAccessLoadMode LoadHandler::GetKeyedAccessLoadMode(MaybeObject handler) {
   DisallowGarbageCollection no_gc;
-  if (handler->IsSmi()) {
+  if (IsSmi(handler)) {
     int const raw_handler = handler.ToSmi().value();
     Kind const kind = KindBits::decode(raw_handler);
     if ((kind == Kind::kElement || kind == Kind::kIndexedString) &&
         AllowOutOfBoundsBits::decode(raw_handler)) {
-      return LOAD_IGNORE_OUT_OF_BOUNDS;
+      return KeyedAccessLoadMode::kHandleOOB;
     }
   }
-  return STANDARD_LOAD;
+  return KeyedAccessLoadMode::kInBounds;
+}
+
+// static
+bool LoadHandler::GetConvertHole(MaybeObject handler) {
+  DisallowGarbageCollection no_gc;
+  if (IsSmi(handler)) {
+    int const raw_handler = handler.ToSmi().value();
+    return ConvertHoleBits::decode(raw_handler);
+  }
+  return false;
 }
 
 // static
 KeyedAccessStoreMode StoreHandler::GetKeyedAccessStoreMode(
     MaybeObject handler) {
   DisallowGarbageCollection no_gc;
-  if (handler->IsSmi()) {
+  if (IsSmi(handler)) {
     int const raw_handler = handler.ToSmi().value();
     Kind const kind = KindBits::decode(raw_handler);
     // All the handlers except the Slow Handler that use the
     // KeyedAccessStoreMode, compute it using KeyedAccessStoreModeForBuiltin
     // method. Hence if any other Handler get to this path, just return
-    // STANDARD_STORE.
+    // KeyedAccessStoreMode::kInBounds.
     if (kind != Kind::kSlow) {
-      return STANDARD_STORE;
+      return KeyedAccessStoreMode::kInBounds;
     }
     KeyedAccessStoreMode store_mode =
         KeyedAccessStoreModeBits::decode(raw_handler);
     return store_mode;
   }
-  return STANDARD_STORE;
+  return KeyedAccessStoreMode::kInBounds;
 }
 
 // static
@@ -228,7 +239,7 @@ MaybeObjectHandle StoreHandler::StoreOwnTransition(Isolate* isolate,
     Handle<DescriptorArray> descriptors(
         transition_map->instance_descriptors(isolate), isolate);
     PropertyDetails details = descriptors->GetDetails(descriptor);
-    if (descriptors->GetKey(descriptor).IsPrivate()) {
+    if (descriptors->GetKey(descriptor)->IsPrivate()) {
       DCHECK_EQ(DONT_ENUM, details.attributes());
     } else {
       DCHECK_EQ(NONE, details.attributes());
@@ -242,7 +253,7 @@ MaybeObjectHandle StoreHandler::StoreOwnTransition(Isolate* isolate,
 
   // StoreOwnTransition does not involve any prototype checks.
   if (is_dictionary_map) {
-    DCHECK(!transition_map->IsJSGlobalObjectMap());
+    DCHECK(!IsJSGlobalObjectMap(*transition_map));
     int config = KindBits::encode(Kind::kNormal);
     return MaybeObjectHandle(Smi::FromInt(config), isolate);
 
@@ -261,9 +272,9 @@ MaybeObjectHandle StoreHandler::StoreTransition(Isolate* isolate,
     Handle<DescriptorArray> descriptors(
         transition_map->instance_descriptors(isolate), isolate);
     // Private fields must be added via StoreOwnTransition handler.
-    DCHECK(!descriptors->GetKey(descriptor).IsPrivateName());
+    DCHECK(!descriptors->GetKey(descriptor)->IsPrivateName());
     PropertyDetails details = descriptors->GetDetails(descriptor);
-    if (descriptors->GetKey(descriptor).IsPrivate()) {
+    if (descriptors->GetKey(descriptor)->IsPrivate()) {
       DCHECK_EQ(DONT_ENUM, details.attributes());
     } else {
       DCHECK_EQ(NONE, details.attributes());
@@ -283,7 +294,7 @@ MaybeObjectHandle StoreHandler::StoreTransition(Isolate* isolate,
   }
 
   if (is_dictionary_map) {
-    DCHECK(!transition_map->IsJSGlobalObjectMap());
+    DCHECK(!IsJSGlobalObjectMap(*transition_map));
     Handle<StoreHandler> handler = isolate->factory()->NewStoreHandler(0);
     // Store normal with enabled lookup on receiver.
     int config = KindBits::encode(Kind::kNormal) |
@@ -305,7 +316,7 @@ MaybeObjectHandle StoreHandler::StoreTransition(Isolate* isolate,
 // static
 Handle<Object> StoreHandler::StoreThroughPrototype(
     Isolate* isolate, Handle<Map> receiver_map, Handle<JSReceiver> holder,
-    Handle<Smi> smi_handler, MaybeObjectHandle maybe_data1,
+    Tagged<Smi> smi_handler, MaybeObjectHandle maybe_data1,
     MaybeObjectHandle maybe_data2) {
   MaybeObjectHandle data1;
   if (maybe_data1.is_null()) {
@@ -322,7 +333,7 @@ Handle<Object> StoreHandler::StoreThroughPrototype(
 
   Handle<StoreHandler> handler = isolate->factory()->NewStoreHandler(data_size);
 
-  handler->set_smi_handler(*smi_handler);
+  handler->set_smi_handler(smi_handler);
   handler->set_validity_cell(*validity_cell);
   InitPrototypeChecks(isolate, handler, receiver_map, data1, maybe_data2);
   return handler;
@@ -340,17 +351,17 @@ Handle<Object> StoreHandler::StoreProxy(Isolate* isolate,
                                         Handle<JSReceiver> receiver) {
   Handle<Smi> smi_handler = StoreProxy(isolate);
   if (receiver.is_identical_to(proxy)) return smi_handler;
-  return StoreThroughPrototype(isolate, receiver_map, proxy, smi_handler,
+  return StoreThroughPrototype(isolate, receiver_map, proxy, *smi_handler,
                                MaybeObjectHandle::Weak(proxy));
 }
 
-bool LoadHandler::CanHandleHolderNotLookupStart(Object handler) {
-  if (handler.IsSmi()) {
+bool LoadHandler::CanHandleHolderNotLookupStart(Tagged<Object> handler) {
+  if (IsSmi(handler)) {
     auto kind = LoadHandler::KindBits::decode(handler.ToSmi().value());
     return kind == LoadHandler::Kind::kSlow ||
            kind == LoadHandler::Kind::kNonExistent;
   }
-  return handler.IsLoadHandler();
+  return IsLoadHandler(handler);
 }
 
 #if defined(OBJECT_PRINT)
@@ -440,20 +451,6 @@ void PrintSmiLoadHandler(int raw_handler, std::ostream& os) {
   }
 }
 
-const char* KeyedAccessStoreModeToString(KeyedAccessStoreMode mode) {
-  switch (mode) {
-    case STANDARD_STORE:
-      return "STANDARD_STORE";
-    case STORE_AND_GROW_HANDLE_COW:
-      return "STORE_AND_GROW_HANDLE_COW";
-    case STORE_IGNORE_OUT_OF_BOUNDS:
-      return "STORE_IGNORE_OUT_OF_BOUNDS";
-    case STORE_HANDLE_COW:
-      return "STORE_HANDLE_COW";
-  }
-  UNREACHABLE();
-}
-
 void PrintSmiStoreHandler(int raw_handler, std::ostream& os) {
   StoreHandler::Kind kind = StoreHandler::KindBits::decode(raw_handler);
   os << "kind = ";
@@ -501,8 +498,7 @@ void PrintSmiStoreHandler(int raw_handler, std::ostream& os) {
     case StoreHandler::Kind::kSlow: {
       KeyedAccessStoreMode keyed_access_store_mode =
           StoreHandler::KeyedAccessStoreModeBits::decode(raw_handler);
-      os << "kSlow, keyed access store mode = "
-         << KeyedAccessStoreModeToString(keyed_access_store_mode);
+      os << "kSlow, keyed access store mode = " << keyed_access_store_mode;
       break;
     }
     case StoreHandler::Kind::kProxy:
@@ -519,84 +515,84 @@ void PrintSmiStoreHandler(int raw_handler, std::ostream& os) {
 }  // namespace
 
 // static
-void LoadHandler::PrintHandler(Object handler, std::ostream& os) {
+void LoadHandler::PrintHandler(Tagged<Object> handler, std::ostream& os) {
   DisallowGarbageCollection no_gc;
-  if (handler.IsSmi()) {
+  if (IsSmi(handler)) {
     int raw_handler = handler.ToSmi().value();
     os << "LoadHandler(Smi)(";
     PrintSmiLoadHandler(raw_handler, os);
     os << ")";
-  } else if (handler.IsCode()) {
+  } else if (IsCode(handler)) {
     os << "LoadHandler(Code)("
-       << Builtins::name(Code::cast(handler).builtin_id()) << ")";
-  } else if (handler.IsSymbol()) {
+       << Builtins::name(Code::cast(handler)->builtin_id()) << ")";
+  } else if (IsSymbol(handler)) {
     os << "LoadHandler(Symbol)(" << Brief(Symbol::cast(handler)) << ")";
-  } else if (handler.IsLoadHandler()) {
-    LoadHandler load_handler = LoadHandler::cast(handler);
-    int raw_handler = load_handler.smi_handler().ToSmi().value();
+  } else if (IsLoadHandler(handler)) {
+    Tagged<LoadHandler> load_handler = LoadHandler::cast(handler);
+    int raw_handler = Smi::cast(load_handler->smi_handler()).value();
     os << "LoadHandler(do access check on lookup start object = "
        << DoAccessCheckOnLookupStartObjectBits::decode(raw_handler)
        << ", lookup on lookup start object = "
        << LookupOnLookupStartObjectBits::decode(raw_handler) << ", ";
     PrintSmiLoadHandler(raw_handler, os);
-    if (load_handler.data_field_count() >= 1) {
+    if (load_handler->data_field_count() >= 1) {
       os << ", data1 = ";
-      load_handler.data1().ShortPrint(os);
+      ShortPrint(load_handler->data1(), os);
     }
-    if (load_handler.data_field_count() >= 2) {
+    if (load_handler->data_field_count() >= 2) {
       os << ", data2 = ";
-      load_handler.data2().ShortPrint(os);
+      ShortPrint(load_handler->data2(), os);
     }
-    if (load_handler.data_field_count() >= 3) {
+    if (load_handler->data_field_count() >= 3) {
       os << ", data3 = ";
-      load_handler.data3().ShortPrint(os);
+      ShortPrint(load_handler->data3(), os);
     }
     os << ", validity cell = ";
-    load_handler.validity_cell().ShortPrint(os);
+    ShortPrint(load_handler->validity_cell(), os);
     os << ")";
   } else {
     os << "LoadHandler(<unexpected>)(" << Brief(handler) << ")";
   }
 }
 
-void StoreHandler::PrintHandler(Object handler, std::ostream& os) {
+void StoreHandler::PrintHandler(Tagged<Object> handler, std::ostream& os) {
   DisallowGarbageCollection no_gc;
-  if (handler.IsSmi()) {
+  if (IsSmi(handler)) {
     int raw_handler = handler.ToSmi().value();
     os << "StoreHandler(Smi)(";
     PrintSmiStoreHandler(raw_handler, os);
     os << ")" << std::endl;
-  } else if (handler.IsStoreHandler()) {
+  } else if (IsStoreHandler(handler)) {
     os << "StoreHandler(";
-    StoreHandler store_handler = StoreHandler::cast(handler);
-    if (store_handler.smi_handler().IsCode()) {
-      Code code = Code::cast(store_handler.smi_handler());
+    Tagged<StoreHandler> store_handler = StoreHandler::cast(handler);
+    if (IsCode(store_handler->smi_handler())) {
+      Tagged<Code> code = Code::cast(store_handler->smi_handler());
       os << "builtin = ";
-      code.ShortPrint(os);
+      ShortPrint(code, os);
     } else {
-      int raw_handler = store_handler.smi_handler().ToSmi().value();
+      int raw_handler = Smi::cast(store_handler->smi_handler()).value();
       os << "do access check on lookup start object = "
          << DoAccessCheckOnLookupStartObjectBits::decode(raw_handler)
          << ", lookup on lookup start object = "
          << LookupOnLookupStartObjectBits::decode(raw_handler) << ", ";
       PrintSmiStoreHandler(raw_handler, os);
     }
-    if (store_handler.data_field_count() >= 1) {
+    if (store_handler->data_field_count() >= 1) {
       os << ", data1 = ";
-      store_handler.data1().ShortPrint(os);
+      ShortPrint(store_handler->data1(), os);
     }
-    if (store_handler.data_field_count() >= 2) {
+    if (store_handler->data_field_count() >= 2) {
       os << ", data2 = ";
-      store_handler.data2().ShortPrint(os);
+      ShortPrint(store_handler->data2(), os);
     }
-    if (store_handler.data_field_count() >= 3) {
+    if (store_handler->data_field_count() >= 3) {
       os << ", data3 = ";
-      store_handler.data3().ShortPrint(os);
+      ShortPrint(store_handler->data3(), os);
     }
     os << ", validity cell = ";
-    store_handler.validity_cell().ShortPrint(os);
+    ShortPrint(store_handler->validity_cell(), os);
     os << ")" << std::endl;
-  } else if (handler.IsMap()) {
+  } else if (IsMap(handler)) {
     os << "StoreHandler(field transition to " << Brief(handler) << ")"
        << std::endl;
   } else {
