@@ -31,6 +31,14 @@ class BytecodeJumpTable;
 
 class BytecodeGenerator final : public AstVisitor<BytecodeGenerator> {
  public:
+  enum TypeHint : uint8_t {
+    kBoolean = 1 << 0,
+    kInternalizedString = 1 << 1,
+    kString = kInternalizedString | (1 << 2),
+    kAny = kBoolean | kString,
+    kUnknown = 0xFFu
+  };
+
   explicit BytecodeGenerator(
       LocalIsolate* local_isolate, Zone* zone, UnoptimizedCompilationInfo* info,
       const AstStringConstants* ast_string_constants,
@@ -44,8 +52,17 @@ class BytecodeGenerator final : public AstVisitor<BytecodeGenerator> {
   template <typename IsolateT>
   Handle<ByteArray> FinalizeSourcePositionTable(IsolateT* isolate);
 
+  // Check if hint2 is same or the subtype of hint1.
+  static bool IsSameOrSubTypeHint(TypeHint hint1, TypeHint hint2) {
+    return hint1 == (hint1 | hint2);
+  }
+
+  static bool IsStringTypeHint(TypeHint hint) {
+    return IsSameOrSubTypeHint(TypeHint::kString, hint);
+  }
+
 #ifdef DEBUG
-  int CheckBytecodeMatches(BytecodeArray bytecode);
+  int CheckBytecodeMatches(Tagged<BytecodeArray> bytecode);
 #endif
 
 #define DECLARE_VISIT(type) void Visit##type(type* node);
@@ -63,6 +80,7 @@ class BytecodeGenerator final : public AstVisitor<BytecodeGenerator> {
   class ContextScope;
   class ControlScope;
   class ControlScopeForBreakable;
+  class ControlScopeForDerivedConstructor;
   class ControlScopeForIteration;
   class ControlScopeForTopLevel;
   class ControlScopeForTryCatch;
@@ -86,7 +104,6 @@ class BytecodeGenerator final : public AstVisitor<BytecodeGenerator> {
   using ToBooleanMode = BytecodeArrayBuilder::ToBooleanMode;
 
   enum class TestFallthrough { kThen, kElse, kNone };
-  enum class TypeHint { kAny, kBoolean, kString };
   enum class AccumulatorPreservingMode { kNone, kPreserve };
 
   // An assignment has to evaluate its LHS before its RHS, but has to assign to
@@ -179,6 +196,7 @@ class BytecodeGenerator final : public AstVisitor<BytecodeGenerator> {
   };
 
   void GenerateBytecodeBody();
+  void GenerateBytecodeBodyWithoutImplicitFinalReturn();
   template <typename IsolateT>
   void AllocateDeferredConstants(IsolateT* isolate, Handle<Script> script);
 
@@ -334,6 +352,8 @@ class BytecodeGenerator final : public AstVisitor<BytecodeGenerator> {
   void VisitArgumentsObject(Variable* variable);
   void VisitRestArgumentsArray(Variable* rest);
   void VisitCallSuper(Call* call);
+  void BuildInstanceInitializationAfterSuperCall(Register this_function,
+                                                 Register instance);
   void BuildInvalidPropertyAccess(MessageTemplate tmpl, Property* property);
   void BuildPrivateBrandCheck(Property* property, Register object);
   void BuildPrivateMethodIn(Variable* private_name,
@@ -418,6 +438,10 @@ class BytecodeGenerator final : public AstVisitor<BytecodeGenerator> {
   template <typename ExpressionFunc>
   void BuildOptionalChain(ExpressionFunc expression_func);
 
+  void BuildGetAndCheckSuperConstructor(Register this_function,
+                                        Register new_target,
+                                        Register constructor,
+                                        BytecodeLabel* super_ctor_call_done);
   void BuildSuperCallOptimization(Register this_function, Register new_target,
                                   Register constructor_then_instance,
                                   BytecodeLabel* super_ctor_call_done);
@@ -440,12 +464,18 @@ class BytecodeGenerator final : public AstVisitor<BytecodeGenerator> {
   // Convenience visitors that put a HoleCheckElisionScope on stack.
   template <typename T>
   void VisitInHoleCheckElisionScope(T* node);
+  void VisitIterationBodyInHoleCheckElisionScope(IterationStatement* stmt,
+                                                 LoopBuilder* loop_builder);
   TypeHint VisitInHoleCheckElisionScopeForAccumulatorValue(Expression* expr);
 
   void VisitInSameTestExecutionScope(Expression* expr);
 
   Register GetRegisterForLocalVariable(Variable* variable);
 
+  bool IsLocalVariableWithInternalizedStringHint(Expression* expr);
+
+  TypeHint GetTypeHintForLocalVariable(Variable* variable);
+
   // Returns the runtime function id for a store to super for the function's
   // language mode.
   inline Runtime::FunctionId StoreToSuperRuntimeId();
