@@ -7,7 +7,8 @@
 #include <limits>
 #include <ostream>
 
-#include "src/codegen/code-factory.h"
+#include "src/builtins/builtins-inl.h"
+#include "src/codegen/code-stub-assembler-inl.h"
 #include "src/codegen/interface-descriptors-inl.h"
 #include "src/codegen/machine-type.h"
 #include "src/interpreter/bytecodes.h"
@@ -702,7 +703,7 @@ TNode<JSFunction> InterpreterAssembler::LoadFunctionClosure() {
 }
 
 TNode<HeapObject> InterpreterAssembler::LoadFeedbackVector() {
-  return CodeStubAssembler::LoadFeedbackVector(LoadFunctionClosure());
+  return CAST(LoadRegister(Register::feedback_vector()));
 }
 
 void InterpreterAssembler::CallPrologue() {
@@ -735,13 +736,11 @@ void InterpreterAssembler::CallJSAndDispatch(
     args_count = Int32Add(args_count, Int32Constant(kJSArgcReceiverSlots));
   }
 
-  Callable callable = CodeFactory::InterpreterPushArgsThenCall(
-      isolate(), receiver_mode, InterpreterPushArgsMode::kOther);
-  TNode<Code> code_target = HeapConstant(callable.code());
+  Builtin builtin = Builtins::InterpreterPushArgsThenCall(
+      receiver_mode, InterpreterPushArgsMode::kOther);
 
-  TailCallStubThenBytecodeDispatch(callable.descriptor(), code_target, context,
-                                   args_count, args.base_reg_location(),
-                                   function);
+  TailCallBuiltinThenBytecodeDispatch(builtin, context, args_count,
+                                      args.base_reg_location(), function);
   // TailCallStubThenDispatch updates accumulator with result.
   implicit_register_use_ =
       implicit_register_use_ | ImplicitRegisterUse::kWriteAccumulator;
@@ -757,18 +756,16 @@ void InterpreterAssembler::CallJSAndDispatch(TNode<Object> function,
   DCHECK(Bytecodes::IsCallOrConstruct(bytecode_) ||
          bytecode_ == Bytecode::kInvokeIntrinsic);
   DCHECK_EQ(Bytecodes::GetReceiverMode(bytecode_), receiver_mode);
-  Callable callable = CodeFactory::Call(isolate());
-  TNode<Code> code_target = HeapConstant(callable.code());
+  Builtin builtin = Builtins::Call();
 
   arg_count = JSParameterCount(arg_count);
   if (receiver_mode == ConvertReceiverMode::kNullOrUndefined) {
     // The first argument parameter (the receiver) is implied to be undefined.
-    TailCallStubThenBytecodeDispatch(callable.descriptor(), code_target,
-                                     context, function, arg_count, args...,
-                                     UndefinedConstant());
+    TailCallBuiltinThenBytecodeDispatch(builtin, context, function, arg_count,
+                                        args..., UndefinedConstant());
   } else {
-    TailCallStubThenBytecodeDispatch(callable.descriptor(), code_target,
-                                     context, function, arg_count, args...);
+    TailCallBuiltinThenBytecodeDispatch(builtin, context, function, arg_count,
+                                        args...);
   }
   // TailCallStubThenDispatch updates accumulator with result.
   implicit_register_use_ =
@@ -805,15 +802,12 @@ void InterpreterAssembler::CallJSWithSpreadAndDispatch(
 #endif  // !V8_JITLESS
 
   Comment("call using CallWithSpread builtin");
-  Callable callable = CodeFactory::InterpreterPushArgsThenCall(
-      isolate(), ConvertReceiverMode::kAny,
-      InterpreterPushArgsMode::kWithFinalSpread);
-  TNode<Code> code_target = HeapConstant(callable.code());
+  Builtin builtin = Builtins::InterpreterPushArgsThenCall(
+      ConvertReceiverMode::kAny, InterpreterPushArgsMode::kWithFinalSpread);
 
   TNode<Word32T> args_count = args.reg_count();
-  TailCallStubThenBytecodeDispatch(callable.descriptor(), code_target, context,
-                                   args_count, args.base_reg_location(),
-                                   function);
+  TailCallBuiltinThenBytecodeDispatch(builtin, context, args_count,
+                                      args.base_reg_location(), function);
   // TailCallStubThenDispatch updates accumulator with result.
   implicit_register_use_ =
       implicit_register_use_ | ImplicitRegisterUse::kWriteAccumulator;
@@ -826,23 +820,35 @@ TNode<Object> InterpreterAssembler::Construct(
   DCHECK(Bytecodes::MakesCallAlongCriticalPath(bytecode_));
   TVARIABLE(Object, var_result);
   TVARIABLE(AllocationSite, var_site);
-  Label return_result(this), construct_generic(this),
+  Label return_result(this), try_fast_construct(this), construct_generic(this),
       construct_array(this, &var_site);
 
   TNode<Word32T> args_count = JSParameterCount(args.reg_count());
   CollectConstructFeedback(context, target, new_target, maybe_feedback_vector,
                            slot_id, UpdateFeedbackMode::kOptionalFeedback,
-                           &construct_generic, &construct_array, &var_site);
+                           &try_fast_construct, &construct_array, &var_site);
+
+  BIND(&try_fast_construct);
+  {
+    Comment("call using FastConstruct builtin");
+    GotoIf(TaggedIsSmi(target), &construct_generic);
+    GotoIfNot(IsJSFunction(CAST(target)), &construct_generic);
+    var_result =
+        CallBuiltin(Builtin::kInterpreterPushArgsThenFastConstructFunction,
+                    context, args_count, args.base_reg_location(), target,
+                    new_target, UndefinedConstant());
+    Goto(&return_result);
+  }
 
   BIND(&construct_generic);
   {
     // TODO(bmeurer): Remove the generic type_info parameter from the Construct.
     Comment("call using Construct builtin");
-    Callable callable = CodeFactory::InterpreterPushArgsThenConstruct(
-        isolate(), InterpreterPushArgsMode::kOther);
+    Builtin builtin = Builtins::InterpreterPushArgsThenConstruct(
+        InterpreterPushArgsMode::kOther);
     var_result =
-        CallStub(callable, context, args_count, args.base_reg_location(),
-                 target, new_target, UndefinedConstant());
+        CallBuiltin(builtin, context, args_count, args.base_reg_location(),
+                    target, new_target, UndefinedConstant());
     Goto(&return_result);
   }
 
@@ -851,11 +857,11 @@ TNode<Object> InterpreterAssembler::Construct(
     // TODO(bmeurer): Introduce a dedicated builtin to deal with the Array
     // constructor feedback collection inside of Ignition.
     Comment("call using ConstructArray builtin");
-    Callable callable = CodeFactory::InterpreterPushArgsThenConstruct(
-        isolate(), InterpreterPushArgsMode::kArrayFunction);
+    Builtin builtin = Builtins::InterpreterPushArgsThenConstruct(
+        InterpreterPushArgsMode::kArrayFunction);
     var_result =
-        CallStub(callable, context, args_count, args.base_reg_location(),
-                 target, new_target, var_site.value());
+        CallBuiltin(builtin, context, args_count, args.base_reg_location(),
+                    target, new_target, var_site.value());
     Goto(&return_result);
   }
 
@@ -872,6 +878,8 @@ TNode<Object> InterpreterAssembler::ConstructWithSpread(
   DCHECK(Bytecodes::MakesCallAlongCriticalPath(bytecode_));
 
 #ifndef V8_JITLESS
+  // TODO(syg): Is the feedback collection logic here the same as
+  // CollectConstructFeedback?
   Label extra_checks(this, Label::kDeferred), construct(this);
   TNode<HeapObject> maybe_feedback_vector = LoadFeedbackVector();
   GotoIf(IsUndefined(maybe_feedback_vector), &construct);
@@ -893,7 +901,8 @@ TNode<Object> InterpreterAssembler::ConstructWithSpread(
     // Check if it is a megamorphic {new_target}.
     Comment("check if megamorphic");
     TNode<BoolT> is_megamorphic = TaggedEqual(
-        feedback, HeapConstant(FeedbackVector::MegamorphicSentinel(isolate())));
+        feedback,
+        HeapConstantNoHole(FeedbackVector::MegamorphicSentinel(isolate())));
     GotoIf(is_megamorphic, &construct);
 
     Comment("check if weak reference");
@@ -969,7 +978,7 @@ TNode<Object> InterpreterAssembler::ConstructWithSpread(
       DCHECK(RootsTable::IsImmortalImmovable(RootIndex::kmegamorphic_symbol));
       StoreFeedbackVectorSlot(
           feedback_vector, slot_id,
-          HeapConstant(FeedbackVector::MegamorphicSentinel(isolate())),
+          HeapConstantNoHole(FeedbackVector::MegamorphicSentinel(isolate())),
           SKIP_WRITE_BARRIER);
       ReportFeedbackUpdate(feedback_vector, slot_id,
                            "ConstructWithSpread:TransitionMegamorphic");
@@ -980,11 +989,36 @@ TNode<Object> InterpreterAssembler::ConstructWithSpread(
   BIND(&construct);
 #endif  // !V8_JITLESS
   Comment("call using ConstructWithSpread builtin");
-  Callable callable = CodeFactory::InterpreterPushArgsThenConstruct(
-      isolate(), InterpreterPushArgsMode::kWithFinalSpread);
+  Builtin builtin = Builtins::InterpreterPushArgsThenConstruct(
+      InterpreterPushArgsMode::kWithFinalSpread);
   TNode<Word32T> args_count = JSParameterCount(args.reg_count());
-  return CallStub(callable, context, args_count, args.base_reg_location(),
-                  target, new_target, UndefinedConstant());
+  return CallBuiltin(builtin, context, args_count, args.base_reg_location(),
+                     target, new_target, UndefinedConstant());
+}
+
+// TODO(v8:13249): Add a FastConstruct variant to avoid pushing arguments twice
+// (once here, and once again in construct stub).
+TNode<Object> InterpreterAssembler::ConstructForwardAllArgs(
+    TNode<Object> target, TNode<Context> context, TNode<Object> new_target,
+    TNode<UintPtrT> slot_id) {
+  DCHECK(Bytecodes::MakesCallAlongCriticalPath(bytecode_));
+  TVARIABLE(Object, var_result);
+  TVARIABLE(AllocationSite, var_site);
+
+#ifndef V8_JITLESS
+  Label construct(this);
+
+  TNode<HeapObject> maybe_feedback_vector = LoadFeedbackVector();
+  GotoIf(IsUndefined(maybe_feedback_vector), &construct);
+
+  CollectConstructFeedback(context, target, new_target, maybe_feedback_vector,
+                           slot_id, UpdateFeedbackMode::kOptionalFeedback,
+                           &construct, &construct, &var_site);
+  BIND(&construct);
+#endif  // !V8_JITLESS
+
+  return CallBuiltin(Builtin::kInterpreterForwardAllArgsThenConstruct, context,
+                     target, new_target);
 }
 
 template <class T>
@@ -994,8 +1028,6 @@ TNode<T> InterpreterAssembler::CallRuntimeN(TNode<Uint32T> function_id,
                                             int return_count) {
   DCHECK(Bytecodes::MakesCallAlongCriticalPath(bytecode_));
   DCHECK(Bytecodes::IsCallRuntime(bytecode_));
-  Callable callable = CodeFactory::InterpreterCEntry(isolate(), return_count);
-  TNode<Code> code_target = HeapConstant(callable.code());
 
   // Get the function entry from the function id.
   TNode<RawPtrT> function_table = ReinterpretCast<RawPtrT>(ExternalConstant(
@@ -1007,9 +1039,9 @@ TNode<T> InterpreterAssembler::CallRuntimeN(TNode<Uint32T> function_id,
   TNode<RawPtrT> function_entry = Load<RawPtrT>(
       function, IntPtrConstant(offsetof(Runtime::Function, entry)));
 
-  return CallStub<T>(callable.descriptor(), code_target, context,
-                     args.reg_count(), args.base_reg_location(),
-                     function_entry);
+  Builtin centry = Builtins::InterpreterCEntry(return_count);
+  return CallBuiltin<T>(centry, context, args.reg_count(),
+                        args.base_reg_location(), function_entry);
 }
 
 template V8_EXPORT_PRIVATE TNode<Object> InterpreterAssembler::CallRuntimeN(
@@ -1023,7 +1055,7 @@ InterpreterAssembler::CallRuntimeN(TNode<Uint32T> function_id,
 
 TNode<Int32T> InterpreterAssembler::UpdateInterruptBudget(
     TNode<Int32T> weight) {
-  TNode<JSFunction> function = CAST(LoadRegister(Register::function_closure()));
+  TNode<JSFunction> function = LoadFunctionClosure();
   TNode<FeedbackCell> feedback_cell =
       LoadObjectField<FeedbackCell>(function, JSFunction::kFeedbackCellOffset);
   TNode<Int32T> old_budget = LoadObjectField<Int32T>(
@@ -1053,7 +1085,7 @@ void InterpreterAssembler::DecreaseInterruptBudget(
          &interrupt_check);
 
   BIND(&interrupt_check);
-  TNode<JSFunction> function = CAST(LoadRegister(Register::function_closure()));
+  TNode<JSFunction> function = LoadFunctionClosure();
   CallRuntime(stack_check_behavior == kEnableStackCheck
                   ? Runtime::kBytecodeBudgetInterruptWithStackCheck_Ignition
                   : Runtime::kBytecodeBudgetInterrupt_Ignition,
@@ -1360,13 +1392,13 @@ void InterpreterAssembler::OnStackReplacement(
     OnStackReplacementParams params) {
   // Three cases may cause us to attempt OSR, in the following order:
   //
-  // 1) Presence of cached OSR Turbofan code.
+  // 1) Presence of cached OSR Turbofan/Maglev code.
   // 2) Presence of cached OSR Sparkplug code.
   // 3) The OSR urgency exceeds the current loop depth - in that case, trigger
   //    a Turbofan OSR compilation.
 
   TVARIABLE(Object, maybe_target_code, SmiConstant(0));
-  Label osr_to_turbofan(this), osr_to_sparkplug(this);
+  Label osr_to_opt(this), osr_to_sparkplug(this);
 
   // Case 1).
   {
@@ -1377,8 +1409,11 @@ void InterpreterAssembler::OnStackReplacement(
     maybe_target_code = GetHeapObjectAssumeWeak(maybe_cached_osr_code);
 
     // Is it marked_for_deoptimization? If yes, clear the slot.
+    TNode<CodeWrapper> code_wrapper = CAST(maybe_target_code.value());
+    maybe_target_code =
+        LoadCodePointerFromObject(code_wrapper, CodeWrapper::kCodeOffset);
     GotoIfNot(IsMarkedForDeoptimization(CAST(maybe_target_code.value())),
-              &osr_to_turbofan);
+              &osr_to_opt);
     StoreFeedbackVectorSlot(feedback_vector, Unsigned(feedback_slot),
                             ClearedValue(), UNSAFE_SKIP_WRITE_BARRIER);
     maybe_target_code = SmiConstant(0);
@@ -1394,44 +1429,39 @@ void InterpreterAssembler::OnStackReplacement(
     DCHECK_EQ(params, OnStackReplacementParams::kDefault);
     TNode<SharedFunctionInfo> sfi = LoadObjectField<SharedFunctionInfo>(
         LoadFunctionClosure(), JSFunction::kSharedFunctionInfoOffset);
-    TNode<HeapObject> sfi_data = LoadObjectField<HeapObject>(
-        sfi, SharedFunctionInfo::kFunctionDataOffset);
-    GotoIf(InstanceTypeEqual(LoadInstanceType(sfi_data), CODE_TYPE),
-           &osr_to_sparkplug);
+    GotoIf(SharedFunctionInfoHasBaselineCode(sfi), &osr_to_sparkplug);
 
     // Case 3).
     {
       static_assert(FeedbackVector::OsrUrgencyBits::kShift == 0);
       TNode<Int32T> osr_urgency = Word32And(
           osr_state, Int32Constant(FeedbackVector::OsrUrgencyBits::kMask));
-      GotoIf(Uint32LessThan(loop_depth, osr_urgency), &osr_to_turbofan);
+      GotoIf(Uint32LessThan(loop_depth, osr_urgency), &osr_to_opt);
       JumpBackward(relative_jump);
     }
   }
 
-  BIND(&osr_to_turbofan);
+  BIND(&osr_to_opt);
   {
-    TNode<IntPtrT> length =
-        LoadAndUntagFixedArrayBaseLength(BytecodeArrayTaggedPointer());
-    TNode<IntPtrT> weight =
-        IntPtrMul(length, IntPtrConstant(v8_flags.osr_to_tierup));
-    DecreaseInterruptBudget(TruncateWordToInt32(weight), kDisableStackCheck);
-    Callable callable = CodeFactory::InterpreterOnStackReplacement(isolate());
-    CallStub(callable, context, maybe_target_code.value());
-    UpdateInterruptBudget(
-        Int32Mul(TruncateWordToInt32(weight), Int32Constant(-1)));
+    TNode<Uint32T> length =
+        LoadAndUntagBytecodeArrayLength(BytecodeArrayTaggedPointer());
+    TNode<Uint32T> weight =
+        Uint32Mul(length, Uint32Constant(v8_flags.osr_to_tierup));
+    DecreaseInterruptBudget(Signed(weight), kDisableStackCheck);
+    CallBuiltin(Builtin::kInterpreterOnStackReplacement, context,
+                maybe_target_code.value());
+    UpdateInterruptBudget(Int32Mul(Signed(weight), Int32Constant(-1)));
     JumpBackward(relative_jump);
   }
 
   BIND(&osr_to_sparkplug);
   {
-    Callable callable =
-        CodeFactory::InterpreterOnStackReplacement_ToBaseline(isolate());
     // We already compiled the baseline code, so we don't need to handle failed
     // compilation as in the Ignition -> Turbofan case. Therefore we can just
     // tailcall to the OSR builtin.
     SaveBytecodeOffset();
-    TailCallStub(callable, context);
+    TailCallBuiltin(Builtin::kInterpreterOnStackReplacement_ToBaseline,
+                    context);
   }
 }
 
@@ -1481,7 +1511,7 @@ bool InterpreterAssembler::TargetSupportsUnalignedAccess() {
 }
 
 void InterpreterAssembler::AbortIfRegisterCountInvalid(
-    TNode<FixedArrayBase> parameters_and_registers,
+    TNode<FixedArray> parameters_and_registers,
     TNode<IntPtrT> formal_parameter_count, TNode<UintPtrT> register_count) {
   TNode<IntPtrT> array_size =
       LoadAndUntagFixedArrayBaseLength(parameters_and_registers);
