@@ -136,6 +136,7 @@ class RuntimeCallTimer final {
   V(ArrayBuffer_NewBackingStore)                           \
   V(ArrayBuffer_BackingStore_Reallocate)                   \
   V(Array_CloneElementAt)                                  \
+  V(Array_Iterate)                                         \
   V(Array_New)                                             \
   V(BigInt64Array_New)                                     \
   V(BigInt_NewFromWords)                                   \
@@ -226,7 +227,9 @@ class RuntimeCallTimer final {
   V(Object_ToInt32)                                        \
   V(Object_ToInteger)                                      \
   V(Object_ToNumber)                                       \
+  V(Object_ToNumeric)                                      \
   V(Object_ToObject)                                       \
+  V(Object_ToPrimitive)                                    \
   V(Object_ToString)                                       \
   V(Object_ToUint32)                                       \
   V(Persistent_New)                                        \
@@ -309,100 +312,116 @@ class RuntimeCallTimer final {
   V(Prefix##Suffix)                                    \
   V(Prefix##Background##Suffix)
 
-#define FOR_EACH_THREAD_SPECIFIC_COUNTER(V)                                 \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Compile, Analyse)                          \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Compile, Eval)                             \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Compile, Function)                         \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Compile, Ignition)                         \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Compile, IgnitionFinalization)             \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Compile, RewriteReturnResult)              \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Compile, ScopeAnalysis)                    \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Compile, Script)                           \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Compile, CompileTask)                      \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, AllocateFPRegisters)             \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, AllocateSIMD128Registers)        \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, AllocateGeneralRegisters)        \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, AssembleCode)                    \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, AssignSpillSlots)                \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, BitcastElision)                  \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, BranchConditionDuplication)      \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, BuildLiveRangeBundles)           \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, BuildLiveRanges)                 \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, BytecodeGraphBuilder)            \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, CommitAssignment)                \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, ConnectRanges)                   \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, ControlFlowOptimization)         \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, CSAEarlyOptimization)            \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, CSAOptimization)                 \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, DecideSpillingMode)              \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, DecompressionOptimization)       \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, EarlyGraphTrimming)              \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, EarlyOptimization)               \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, EffectLinearization)             \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, EscapeAnalysis)                  \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, FinalizeCode)                    \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, FrameElision)                    \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, GenericLowering)                 \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, Inlining)                        \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, JSWasmInlining)                  \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, JSWasmLowering)                  \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, JumpThreading)                   \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, MidTierPopulateReferenceMaps)    \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, MidTierRegisterAllocator)        \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, MidTierRegisterOutputDefinition) \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, MidTierSpillSlotAllocator)       \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, LateOptimization)                \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, LoadElimination)                 \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, LocateSpillSlots)                \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, LoopExitElimination)             \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, LoopPeeling)                     \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, MachineOperatorOptimization)     \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, MeetRegisterConstraints)         \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, MemoryOptimization)              \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, OptimizeMoves)                   \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, PopulatePointerMaps)             \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, PrintGraph)                      \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, PrintTurboshaftGraph)            \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, ResolveControlFlow)              \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, ResolvePhis)                     \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize,                                  \
-                              ScheduledEffectControlLinearization)          \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, ScheduledMachineLowering)        \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, Scheduling)                      \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, SelectInstructions)              \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, SimplifiedLowering)              \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, SimplifyLoops)                   \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, StoreStoreElimination)           \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, TraceScheduleAndVerify)          \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, TurboshaftBuildGraph)            \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, TurboshaftDeadCodeElimination)   \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize,                                  \
-                              TurboshaftDecompressionOptimization)          \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, TurboshaftLateOptimization)      \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, TurboshaftMachineLowering)       \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, TurboshaftOptimize)              \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, TurboshaftRecreateSchedule)      \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, TurboshaftTagUntagLowering)      \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, TurboshaftTypeAssertions)        \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, TurboshaftTypedOptimizations)    \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, TypeAssertions)                  \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, TypedLowering)                   \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, Typer)                           \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, Untyper)                         \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, VerifyGraph)                     \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, WasmBaseOptimization)            \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, WasmGCLowering)                  \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, WasmGCOptimization)              \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, WasmInlining)                    \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, WasmLoopPeeling)                 \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, WasmLoopUnrolling)               \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, WasmOptimization)                \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, WasmTyping)                      \
-                                                                            \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Parse, ArrowFunctionLiteral)               \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Parse, FunctionLiteral)                    \
-  ADD_THREAD_SPECIFIC_COUNTER(V, Parse, Program)                            \
-  ADD_THREAD_SPECIFIC_COUNTER(V, PreParse, ArrowFunctionLiteral)            \
+#define FOR_EACH_THREAD_SPECIFIC_COUNTER(V)                                   \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Compile, Analyse)                            \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Compile, Eval)                               \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Compile, Function)                           \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Compile, Ignition)                           \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Compile, IgnitionFinalization)               \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Compile, RewriteReturnResult)                \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Compile, ScopeAnalysis)                      \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Compile, Script)                             \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Compile, CompileTask)                        \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, AllocateFPRegisters)               \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, AllocateSIMD128Registers)          \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, AllocateGeneralRegisters)          \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, AssembleCode)                      \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, AssignSpillSlots)                  \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, BitcastElision)                    \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, BranchConditionDuplication)        \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, BuildLiveRangeBundles)             \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, BuildLiveRanges)                   \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, BytecodeGraphBuilder)              \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, CommitAssignment)                  \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, ConnectRanges)                     \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, ControlFlowOptimization)           \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, CSAEarlyOptimization)              \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, CSAOptimization)                   \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, DecideSpillingMode)                \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, DecompressionOptimization)         \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, EarlyGraphTrimming)                \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, EarlyOptimization)                 \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, EffectLinearization)               \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, EscapeAnalysis)                    \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, FinalizeCode)                      \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, FrameElision)                      \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, GenericLowering)                   \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, Inlining)                          \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, JSWasmInlining)                    \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, JSWasmLowering)                    \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, JumpThreading)                     \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, LateOptimization)                  \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, LoadElimination)                   \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, LocateSpillSlots)                  \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, LoopExitElimination)               \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, LoopPeeling)                       \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, MachineOperatorOptimization)       \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, PairingOptimization)               \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, MeetRegisterConstraints)           \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, MemoryOptimization)                \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, OptimizeMoves)                     \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, PopulatePointerMaps)               \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, PrintGraph)                        \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, PrintTurboshaftGraph)              \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, ResolveControlFlow)                \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, ResolvePhis)                       \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize,                                    \
+                              ScheduledEffectControlLinearization)            \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, ScheduledMachineLowering)          \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, Scheduling)                        \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, SelectInstructions)                \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, SimplifiedLowering)                \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, SimplifyLoops)                     \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, StoreStoreElimination)             \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, TraceScheduleAndVerify)            \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, TurboshaftBuildGraph)              \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize,                                    \
+                              TurboshaftCodeEliminationAndSimplification)     \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, TurboshaftCsaBranchElimination)    \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, TurboshaftCsaLateEscapeAnalysis)   \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, TurboshaftCsaLoadElimination)      \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, TurboshaftCsaOptimize)             \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, TurboshaftDebugFeatureLowering)    \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, TurboshaftDecompressionOpt)        \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, TurboshaftInstructionSelection)    \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, TurboshaftInt64Lowering)           \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, TurboshaftLateOptimization)        \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, TurboshaftLoopPeeling)             \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, TurboshaftLoopUnrolling)           \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, TurboshaftMachineLowering)         \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, TurboshaftMaglevGraphBuilding)     \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, TurboshaftOptimize)                \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, TurboshaftRecreateSchedule)        \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, TurboshaftSimplifiedLowering)      \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, TurboshaftSpecialRPOScheduling)    \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, TurboshaftStoreStoreElim)          \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, TurboshaftTagUntagLowering)        \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, TurboshaftTypeAssertions)          \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, TurboshaftTypedOptimizations)      \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, TurboshaftWasmDeadCodeElimination) \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, TurboshaftWasmGCOptimize)          \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, TurboshaftWasmOptimize)            \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, TurboshaftWasmLowering)            \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, TurboshaftWasmRevec)               \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, TypeAssertions)                    \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, TypedLowering)                     \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, Typer)                             \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, Untyper)                           \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, VerifyGraph)                       \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, WasmBaseOptimization)              \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, WasmGCLowering)                    \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, WasmGCOptimization)                \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, WasmInlining)                      \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, WasmLoopPeeling)                   \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, WasmLoopUnrolling)                 \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, WasmOptimization)                  \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, WasmJSLowering)                    \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Optimize, WasmTyping)                        \
+                                                                              \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Parse, ArrowFunctionLiteral)                 \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Parse, FunctionLiteral)                      \
+  ADD_THREAD_SPECIFIC_COUNTER(V, Parse, Program)                              \
+  ADD_THREAD_SPECIFIC_COUNTER(V, PreParse, ArrowFunctionLiteral)              \
   ADD_THREAD_SPECIFIC_COUNTER(V, PreParse, WithVariableResolution)
 
 #define FOR_EACH_MANUAL_COUNTER(V)             \
