@@ -5,6 +5,7 @@
 #include "src/maglev/maglev-phi-representation-selector.h"
 
 #include "src/base/enum-set.h"
+#include "src/base/logging.h"
 #include "src/flags/flags.h"
 #include "src/handles/handles-inl.h"
 #include "src/maglev/maglev-graph-processor.h"
@@ -22,15 +23,63 @@ namespace maglev {
     }                                             \
   } while (false)
 
-ProcessResult MaglevPhiRepresentationSelector::Process(Phi* node,
-                                                       const ProcessingState&) {
-  DCHECK_EQ(node->value_representation(), ValueRepresentation::kTagged);
+void MaglevPhiRepresentationSelector::PreProcessBasicBlock(BasicBlock* block) {
+  MergeNewNodesInBlock(current_block_);
+  PreparePhiTaggings(current_block_, block);
+  current_block_ = block;
+
+  if (block->has_phi()) {
+    auto& phis = *block->phis();
+
+    auto first_retry = phis.begin();
+    auto end_retry = first_retry;
+    bool any_change = false;
+
+    for (auto it = phis.begin(); it != phis.end(); ++it) {
+      Phi* phi = *it;
+      ProcessPhiResult res = ProcessPhi(phi);
+      if (res == ProcessPhiResult::kChanged) {
+        any_change = true;
+      } else if (ProcessPhiResult::kRetryOnChange) {
+        if (end_retry == first_retry) {
+          first_retry = it;
+        }
+        end_retry = it;
+        ++end_retry;
+      }
+    }
+    // Give it one more shot in case an earlier phi has a later one as input.
+    if (any_change) {
+      for (auto it = first_retry; it != end_retry; ++it) {
+        ProcessPhi(*it);
+      }
+    }
+  }
+}
+
+namespace {
+
+bool CanHoistUntaggingTo(BasicBlock* block) {
+  if (block->successors().size() != 1) return false;
+  if (!block->successors()[0]->is_loop()) return true;
+  // To be able to hoist above resumable loops we would have to be able to
+  // convert during resumption.
+  return !block->successors()[0]->state()->is_resumable_loop();
+}
+
+}  // namespace
+
+MaglevPhiRepresentationSelector::ProcessPhiResult
+MaglevPhiRepresentationSelector::ProcessPhi(Phi* node) {
+  if (node->value_representation() != ValueRepresentation::kTagged) {
+    return ProcessPhiResult::kNone;
+  }
 
   if (node->is_exception_phi()) {
     // Exception phis have no inputs (or, at least, none accessible through
     // `node->input(...)`), so we don't know if the inputs could be untagged or
     // not, so we just keep those Phis tagged.
-    return ProcessResult::kContinue;
+    return ProcessPhiResult::kNone;
   }
 
   TRACE_UNTAGGING(
@@ -39,7 +88,9 @@ ProcessResult MaglevPhiRepresentationSelector::Process(Phi* node,
   // {input_mask} represents the ValueRepresentation that {node} could have,
   // based on the ValueRepresentation of its inputs.
   ValueRepresentationSet input_reprs;
+  HoistType hoist_untagging = HoistType::kNone;
 
+  bool has_tagged_phi_input = false;
   for (int i = 0; i < node->input_count(); i++) {
     ValueNode* input = node->input(i).node();
     if (input->Is<SmiConstant>()) {
@@ -80,12 +131,60 @@ ProcessResult MaglevPhiRepresentationSelector::Process(Phi* node,
           DCHECK_EQ(i, 1);
           break;
         }
+        has_tagged_phi_input = true;
         input_reprs.RemoveAll();
         break;
       }
     } else {
-      // This input is tagged (and didn't require a tagging operation to be
-      // tagged); we won't untag {node}.
+      // This is the case where we don't have an existing conversion to attach
+      // the untagging to. In the general case we give up, however in the
+      // special case of the value originating from the loop entry branch, we
+      // can try to hoist untagging out of the loop.
+      if (builder_->graph()->is_osr() &&
+          v8_flags.maglev_hoist_osr_value_phi_untagging &&
+          input->Is<InitialValue>() &&
+          CanHoistUntaggingTo(*builder_->graph()->begin())) {
+        hoist_untagging = HoistType::kPrologue;
+        continue;
+      }
+      if (node->is_loop_phi()) {
+        BasicBlock* dominator = node->merge_state()->predecessor_at(0);
+        if ((input->Is<InitialValue>() || i == 0 ||
+             (input->has_id() &&
+              input->id() < dominator->control_node()->id())) &&
+            CanHoistUntaggingTo(dominator)) {
+          auto static_type = StaticTypeForNode(
+              builder_->broker(), builder_->local_isolate(), input);
+          if (NodeTypeIs(static_type, NodeType::kSmi)) {
+            input_reprs.Add(ValueRepresentation::kInt32);
+            hoist_untagging = HoistType::kLoopEntryUnchecked;
+            continue;
+          }
+          if (NodeTypeIs(static_type, NodeType::kNumber)) {
+            input_reprs.Add(ValueRepresentation::kFloat64);
+            hoist_untagging = HoistType::kLoopEntryUnchecked;
+            continue;
+          }
+
+          // TODO(olivf): Unless we untag OSR values, speculatively untagging
+          // could end us in deopt loops. To enable this by default we need to
+          // add some feedback to be able to back off. Or, ideally find the
+          // respective checked conversion from within the loop to wire up the
+          // feedback collection.
+          if (v8_flags.maglev_speculative_hoist_phi_untagging) {
+            // TODO(olivf): Currently there is no hard guarantee that the phi
+            // merge state has a checkpointed jump.
+            if (dominator->control_node()->Is<CheckpointedJump>()) {
+              DCHECK(!node->merge_state()->is_resumable_loop());
+              hoist_untagging = HoistType::kLoopEntry;
+              continue;
+            }
+          }
+        }
+      }
+
+      // This input is tagged, didn't require a tagging operation to be
+      // tagged and we decided not to hosit; we won't untag {node}.
       // TODO(dmercadier): this is a bit suboptimal, because some nodes start
       // tagged, and later become untagged (parameters for instance). Such nodes
       // will have their untagged alternative passed to {node} without any
@@ -95,8 +194,18 @@ ProcessResult MaglevPhiRepresentationSelector::Process(Phi* node,
       break;
     }
   }
+  ProcessPhiResult default_result = has_tagged_phi_input
+                                        ? ProcessPhiResult::kRetryOnChange
+                                        : ProcessPhiResult::kNone;
 
-  UseRepresentationSet use_reprs = node->get_uses_repr_hints();
+  UseRepresentationSet use_reprs;
+  if (node->is_loop_phi() && !node->get_same_loop_uses_repr_hints().empty()) {
+    // {node} is a loop phi that has uses inside the loop; we will tag/untag
+    // based on those uses, ignoring uses after the loop.
+    use_reprs = node->get_same_loop_uses_repr_hints();
+  } else {
+    use_reprs = node->get_uses_repr_hints();
+  }
 
   TRACE_UNTAGGING("  + use_reprs  : " << use_reprs);
   TRACE_UNTAGGING("  + input_reprs: " << input_reprs);
@@ -112,7 +221,7 @@ ProcessResult MaglevPhiRepresentationSelector::Process(Phi* node,
     // ignored.
     TRACE_UNTAGGING("  => Leaving tagged [incompatible uses]");
     EnsurePhiInputsTagged(node);
-    return ProcessResult::kContinue;
+    return default_result;
   }
 
   if (input_reprs.contains(ValueRepresentation::kTagged) ||
@@ -120,7 +229,7 @@ ProcessResult MaglevPhiRepresentationSelector::Process(Phi* node,
       input_reprs.empty()) {
     TRACE_UNTAGGING("  => Leaving tagged [tagged or uint32 inputs]");
     EnsurePhiInputsTagged(node);
-    return ProcessResult::kContinue;
+    return default_result;
   }
 
   // Only allowed to have Int32, Float64 and HoleyFloat64 inputs from here.
@@ -186,23 +295,24 @@ ProcessResult MaglevPhiRepresentationSelector::Process(Phi* node,
   TRACE_UNTAGGING("  + intersection reprs: " << intersection);
   if (intersection.contains(ValueRepresentation::kInt32)) {
     TRACE_UNTAGGING("  => Untagging to Int32");
-    ConvertTaggedPhiTo(node, ValueRepresentation::kInt32);
-    return ProcessResult::kContinue;
+    ConvertTaggedPhiTo(node, ValueRepresentation::kInt32, hoist_untagging);
+    return ProcessPhiResult::kChanged;
   } else if (intersection.contains(ValueRepresentation::kFloat64)) {
     TRACE_UNTAGGING("  => Untagging to kFloat64");
-    ConvertTaggedPhiTo(node, ValueRepresentation::kFloat64);
-    return ProcessResult::kContinue;
+    ConvertTaggedPhiTo(node, ValueRepresentation::kFloat64, hoist_untagging);
+    return ProcessPhiResult::kChanged;
   } else if (intersection.contains(ValueRepresentation::kHoleyFloat64)) {
     TRACE_UNTAGGING("  => Untagging to HoleyFloat64");
-    ConvertTaggedPhiTo(node, ValueRepresentation::kHoleyFloat64);
-    return ProcessResult::kContinue;
+    ConvertTaggedPhiTo(node, ValueRepresentation::kHoleyFloat64,
+                       hoist_untagging);
+    return ProcessPhiResult::kChanged;
   }
 
   DCHECK(intersection.empty());
   // We don't untag the Phi.
   TRACE_UNTAGGING("  => Leaving tagged [incompatible inputs/uses]");
   EnsurePhiInputsTagged(node);
-  return ProcessResult::kContinue;
+  return default_result;
 }
 
 void MaglevPhiRepresentationSelector::EnsurePhiInputsTagged(Phi* phi) {
@@ -215,8 +325,8 @@ void MaglevPhiRepresentationSelector::EnsurePhiInputsTagged(Phi* phi) {
   for (int i = 0; i < phi->input_count(); i++) {
     ValueNode* input = phi->input(i).node();
     if (Phi* phi_input = input->TryCast<Phi>()) {
-      phi->set_input(i, EnsurePhiTagged(phi_input, phi->predecessor_at(i),
-                                        NewNodePosition::kEnd));
+      phi->change_input(i, EnsurePhiTagged(phi_input, phi->predecessor_at(i),
+                                           NewNodePosition::kEnd, i));
     } else {
       // Inputs of Phis that aren't Phi should always be tagged (except for the
       // phis untagged by this class, but {phi} isn't one of them).
@@ -243,7 +353,7 @@ Opcode GetOpcodeForConversion(ValueRepresentation from, ValueRepresentation to,
 
         case ValueRepresentation::kInt32:
         case ValueRepresentation::kTagged:
-        case ValueRepresentation::kWord64:
+        case ValueRepresentation::kIntPtr:
           UNREACHABLE();
       }
     case ValueRepresentation::kUint32:
@@ -257,7 +367,7 @@ Opcode GetOpcodeForConversion(ValueRepresentation from, ValueRepresentation to,
 
         case ValueRepresentation::kUint32:
         case ValueRepresentation::kTagged:
-        case ValueRepresentation::kWord64:
+        case ValueRepresentation::kIntPtr:
           UNREACHABLE();
       }
     case ValueRepresentation::kFloat64:
@@ -276,7 +386,7 @@ Opcode GetOpcodeForConversion(ValueRepresentation from, ValueRepresentation to,
 
         case ValueRepresentation::kFloat64:
         case ValueRepresentation::kTagged:
-        case ValueRepresentation::kWord64:
+        case ValueRepresentation::kIntPtr:
           UNREACHABLE();
       }
     case ValueRepresentation::kHoleyFloat64:
@@ -296,12 +406,12 @@ Opcode GetOpcodeForConversion(ValueRepresentation from, ValueRepresentation to,
 
         case ValueRepresentation::kHoleyFloat64:
         case ValueRepresentation::kTagged:
-        case ValueRepresentation::kWord64:
+        case ValueRepresentation::kIntPtr:
           UNREACHABLE();
       }
 
     case ValueRepresentation::kTagged:
-    case ValueRepresentation::kWord64:
+    case ValueRepresentation::kIntPtr:
       UNREACHABLE();
   }
   UNREACHABLE();
@@ -310,12 +420,15 @@ Opcode GetOpcodeForConversion(ValueRepresentation from, ValueRepresentation to,
 }  // namespace
 
 void MaglevPhiRepresentationSelector::ConvertTaggedPhiTo(
-    Phi* phi, ValueRepresentation repr) {
+    Phi* phi, ValueRepresentation repr, HoistType hoist_untagging) {
   // We currently only support Int32, Float64, and HoleyFloat64 untagged phis.
   DCHECK(repr == ValueRepresentation::kInt32 ||
          repr == ValueRepresentation::kFloat64 ||
          repr == ValueRepresentation::kHoleyFloat64);
   phi->change_representation(repr);
+  // Re-initialise register data, since we might have changed from integer
+  // registers to floating registers.
+  phi->InitializeRegisterData();
 
   for (int i = 0; i < phi->input_count(); i++) {
     ValueNode* input = phi->input(i).node();
@@ -383,7 +496,7 @@ void MaglevPhiRepresentationSelector::ConvertTaggedPhiTo(
             UNREACHABLE();
         }
       }
-      phi->set_input(i, new_input);
+      phi->change_input(i, new_input);
     } else if (Phi* input_phi = input->TryCast<Phi>()) {
       ValueRepresentation from_repr = input_phi->value_representation();
       if (from_repr == ValueRepresentation::kTagged) {
@@ -403,10 +516,10 @@ void MaglevPhiRepresentationSelector::ConvertTaggedPhiTo(
 
         DeoptFrame* deopt_frame = phi->merge_state()->backedge_deopt_frame();
         if (repr == ValueRepresentation::kInt32) {
-          phi->set_input(i, AddNode(NodeBase::New<CheckedSmiUntag>(
-                                        builder_->zone(), {input_phi}),
-                                    phi->predecessor_at(i),
-                                    NewNodePosition::kEnd, deopt_frame));
+          phi->change_input(i, AddNode(NodeBase::New<CheckedSmiUntag>(
+                                           builder_->zone(), {input_phi}),
+                                       phi->predecessor_at(i),
+                                       NewNodePosition::kEnd, deopt_frame));
         } else {
           DCHECK(repr == ValueRepresentation::kFloat64 ||
                  repr == ValueRepresentation::kHoleyFloat64);
@@ -414,7 +527,7 @@ void MaglevPhiRepresentationSelector::ConvertTaggedPhiTo(
               repr == ValueRepresentation::kFloat64
                   ? TaggedToFloat64ConversionType::kOnlyNumber
                   : TaggedToFloat64ConversionType::kNumberOrOddball;
-          phi->set_input(
+          phi->change_input(
               i, AddNode(NodeBase::New<CheckedNumberOrOddballToFloat64>(
                              builder_->zone(), {input_phi}, convertion_type),
                          phi->predecessor_at(i), NewNodePosition::kEnd,
@@ -429,7 +542,7 @@ void MaglevPhiRepresentationSelector::ConvertTaggedPhiTo(
         // inputs, which will require a Int32ToFloat64 conversion.
         DCHECK(repr == ValueRepresentation::kFloat64 ||
                repr == ValueRepresentation::kHoleyFloat64);
-        phi->set_input(
+        phi->change_input(
             i, AddNode(NodeBase::New<ChangeInt32ToFloat64>(builder_->zone(),
                                                            {input_phi}),
                        phi->predecessor_at(i), NewNodePosition::kEnd));
@@ -444,6 +557,79 @@ void MaglevPhiRepresentationSelector::ConvertTaggedPhiTo(
         TRACE_UNTAGGING(TRACE_INPUT_LABEL
                         << ": Keeping untagged Phi input as-is");
       }
+    } else if (hoist_untagging != HoistType::kNone) {
+      CHECK_EQ(input->value_representation(), ValueRepresentation::kTagged);
+      BasicBlock* block;
+      DeoptFrame* deopt_frame;
+      auto GetDeoptFrame = [](BasicBlock* block) {
+        return &block->control_node()
+                    ->Cast<CheckpointedJump>()
+                    ->eager_deopt_info()
+                    ->top_frame();
+      };
+      switch (hoist_untagging) {
+        case HoistType::kLoopEntryUnchecked:
+          block = phi->merge_state()->predecessor_at(0);
+          deopt_frame = nullptr;
+          break;
+        case HoistType::kLoopEntry:
+          block = phi->merge_state()->predecessor_at(0);
+          deopt_frame = GetDeoptFrame(block);
+          break;
+        case HoistType::kPrologue:
+          block = *builder_->graph()->begin();
+          deopt_frame = GetDeoptFrame(block);
+          break;
+        case HoistType::kNone:
+          UNREACHABLE();
+      }
+      ValueNode* untagged;
+      switch (repr) {
+        case ValueRepresentation::kUint32:
+        case ValueRepresentation::kInt32:
+          if (!deopt_frame) {
+            DCHECK(
+                NodeTypeIs(StaticTypeForNode(builder_->broker(),
+                                             builder_->local_isolate(), input),
+                           NodeType::kSmi));
+            untagged = AddNode(
+                NodeBase::New<UnsafeSmiUntag>(builder_->zone(), {input}), block,
+                NewNodePosition::kEnd);
+
+          } else {
+            untagged = AddNode(NodeBase::New<CheckedNumberOrOddballToFloat64>(
+                                   builder_->zone(), {input},
+                                   TaggedToFloat64ConversionType::kOnlyNumber),
+                               block, NewNodePosition::kEnd, deopt_frame);
+            untagged = AddNode(NodeBase::New<CheckedTruncateFloat64ToInt32>(
+                                   builder_->zone(), {untagged}),
+                               block, NewNodePosition::kEnd, deopt_frame);
+          }
+          break;
+        case ValueRepresentation::kTagged:
+          UNREACHABLE();
+        case ValueRepresentation::kFloat64:
+        case ValueRepresentation::kHoleyFloat64:
+          if (!deopt_frame) {
+            DCHECK(
+                NodeTypeIs(StaticTypeForNode(builder_->broker(),
+                                             builder_->local_isolate(), input),
+                           NodeType::kNumber));
+            untagged = AddNode(NodeBase::New<UncheckedNumberOrOddballToFloat64>(
+                                   builder_->zone(), {input},
+                                   TaggedToFloat64ConversionType::kOnlyNumber),
+                               block, NewNodePosition::kEnd);
+          } else {
+            untagged = AddNode(NodeBase::New<CheckedNumberOrOddballToFloat64>(
+                                   builder_->zone(), {input},
+                                   TaggedToFloat64ConversionType::kOnlyNumber),
+                               block, NewNodePosition::kEnd, deopt_frame);
+          }
+          break;
+        case ValueRepresentation::kIntPtr:
+          UNREACHABLE();
+      }
+      phi->change_input(i, untagged);
     } else {
       TRACE_UNTAGGING(TRACE_INPUT_LABEL << ": Invalid input for untagged phi");
       UNREACHABLE();
@@ -467,7 +653,7 @@ bool MaglevPhiRepresentationSelector::IsUntagging(Opcode op) {
 }
 
 void MaglevPhiRepresentationSelector::UpdateUntaggingOfPhi(
-    ValueNode* old_untagging) {
+    Phi* phi, ValueNode* old_untagging) {
   DCHECK_EQ(old_untagging->input_count(), 1);
   DCHECK(old_untagging->input(0).node()->Is<Phi>());
 
@@ -488,6 +674,13 @@ void MaglevPhiRepresentationSelector::UpdateUntaggingOfPhi(
   }
 
   if (from_repr == to_repr) {
+    if (from_repr == ValueRepresentation::kInt32) {
+      if (phi->uses_require_31_bit_value() &&
+          old_untagging->Is<CheckedSmiUntag>()) {
+        old_untagging->OverwriteWith<CheckedSmiSizedInt32>();
+        return;
+      }
+    }
     old_untagging->OverwriteWith<Identity>();
     return;
   }
@@ -537,8 +730,12 @@ ProcessResult MaglevPhiRepresentationSelector::UpdateNodePhiInput(
       return ProcessResult::kContinue;
 
     case ValueRepresentation::kInt32:
-      node->OverwriteWith<CheckInt32IsSmi>();
-      return ProcessResult::kContinue;
+      if (!SmiValuesAre32Bits()) {
+        node->OverwriteWith<CheckInt32IsSmi>();
+        return ProcessResult::kContinue;
+      } else {
+        return ProcessResult::kRemove;
+      }
 
     case ValueRepresentation::kFloat64:
     case ValueRepresentation::kHoleyFloat64:
@@ -546,7 +743,7 @@ ProcessResult MaglevPhiRepresentationSelector::UpdateNodePhiInput(
       return ProcessResult::kContinue;
 
     case ValueRepresentation::kUint32:
-    case ValueRepresentation::kWord64:
+    case ValueRepresentation::kIntPtr:
       UNREACHABLE();
   }
 }
@@ -648,7 +845,7 @@ ProcessResult MaglevPhiRepresentationSelector::UpdateNodePhiInput(
       return ProcessResult::kContinue;
 
     case ValueRepresentation::kUint32:
-    case ValueRepresentation::kWord64:
+    case ValueRepresentation::kIntPtr:
       UNREACHABLE();
   }
 }
@@ -676,34 +873,80 @@ ProcessResult MaglevPhiRepresentationSelector::UpdateNodePhiInput(
 }
 
 ValueNode* MaglevPhiRepresentationSelector::EnsurePhiTagged(
-    Phi* phi, BasicBlock* block, NewNodePosition pos) {
+    Phi* phi, BasicBlock* block, NewNodePosition pos,
+    base::Optional<int> predecessor_index) {
+  if (phi->value_representation() == ValueRepresentation::kTagged) {
+    return phi;
+  }
+
+  // Try to find an existing Tagged conversion for {phi} in {phi_taggings_}.
+  if (phi->has_key()) {
+    if (predecessor_index.has_value()) {
+      if (ValueNode* tagging = phi_taggings_.GetPredecessorValue(
+              phi->key(), predecessor_index.value())) {
+        return tagging;
+      }
+    } else {
+      if (ValueNode* tagging = phi_taggings_.Get(phi->key())) {
+        return tagging;
+      }
+    }
+  }
+
+  // We didn't already Tag {phi} on the current path; creating this tagging now.
+  ValueNode* tagged = nullptr;
   switch (phi->value_representation()) {
     case ValueRepresentation::kFloat64:
       // It's important to use kCanonicalizeSmi for Float64ToTagged, as
       // otherwise, we could end up storing HeapNumbers in Smi fields.
-      return AddNode(NodeBase::New<Float64ToTagged>(
-                         builder_->zone(), {phi},
-                         Float64ToTagged::ConversionMode::kCanonicalizeSmi),
-                     block, pos);
+      tagged = AddNode(NodeBase::New<Float64ToTagged>(
+                           builder_->zone(), {phi},
+                           Float64ToTagged::ConversionMode::kCanonicalizeSmi),
+                       block, pos);
+      break;
     case ValueRepresentation::kHoleyFloat64:
       // It's important to use kCanonicalizeSmi for HoleyFloat64ToTagged, as
       // otherwise, we could end up storing HeapNumbers in Smi fields.
-      return AddNode(
-          NodeBase::New<HoleyFloat64ToTagged>(
-              builder_->zone(), {phi},
-              HoleyFloat64ToTagged::ConversionMode::kCanonicalizeSmi),
-          block, pos);
+      tagged =
+          AddNode(NodeBase::New<HoleyFloat64ToTagged>(
+                      builder_->zone(), {phi},
+                      HoleyFloat64ToTagged::ConversionMode::kCanonicalizeSmi),
+                  block, pos);
+      break;
     case ValueRepresentation::kInt32:
-      return AddNode(NodeBase::New<Int32ToNumber>(builder_->zone(), {phi}),
-                     block, pos);
+      tagged = AddNode(NodeBase::New<Int32ToNumber>(builder_->zone(), {phi}),
+                       block, pos);
+      break;
     case ValueRepresentation::kUint32:
-      return AddNode(NodeBase::New<Uint32ToNumber>(builder_->zone(), {phi}),
-                     block, pos);
+      tagged = AddNode(NodeBase::New<Uint32ToNumber>(builder_->zone(), {phi}),
+                       block, pos);
+      break;
     case ValueRepresentation::kTagged:
-      return phi;
-    case ValueRepresentation::kWord64:
+      // Already handled at the begining of this function.
+    case ValueRepresentation::kIntPtr:
       UNREACHABLE();
   }
+
+  if (predecessor_index.has_value()) {
+    // We inserted the new tagging node in a predecessor of the current block,
+    // so we shouldn't update the snapshot table for the current block (and we
+    // can't update it for the predecessor either since its snapshot is sealed).
+    DCHECK_IMPLIES(block == current_block_,
+                   block->is_loop() && block->successors().size() == 1 &&
+                       block->successors().at(0) == block);
+    return tagged;
+  }
+
+  if (phi->has_key()) {
+    // The Key already existed, but wasn't set on the current path.
+    phi_taggings_.Set(phi->key(), tagged);
+  } else {
+    // The Key didn't already exist, so we create it now.
+    auto key = phi_taggings_.NewKey();
+    phi->set_key(key);
+    phi_taggings_.Set(key, tagged);
+  }
+  return tagged;
 }
 
 void MaglevPhiRepresentationSelector::FixLoopPhisBackedge(BasicBlock* block) {
@@ -727,9 +970,9 @@ void MaglevPhiRepresentationSelector::FixLoopPhisBackedge(BasicBlock* block) {
         // Since all Phi inputs are initially tagged, the fact that the backedge
         // is not tagged means that it's a Phi that we recently untagged.
         DCHECK(backedge->Is<Phi>());
-        phi->set_input(last_input_idx,
-                       EnsurePhiTagged(backedge->Cast<Phi>(), current_block_,
-                                       NewNodePosition::kEnd));
+        phi->change_input(last_input_idx,
+                          EnsurePhiTagged(backedge->Cast<Phi>(), current_block_,
+                                          NewNodePosition::kEnd));
       }
     } else {
       // If {phi} was untagged and its backedge became Identity, then we need to
@@ -738,7 +981,7 @@ void MaglevPhiRepresentationSelector::FixLoopPhisBackedge(BasicBlock* block) {
       if (backedge->Is<Identity>()) {
         DCHECK_EQ(backedge->input(0).node()->value_representation(),
                   phi->value_representation());
-        phi->set_input(last_input_idx, backedge->input(0).node());
+        phi->change_input(last_input_idx, backedge->input(0).node());
       }
     }
   }
@@ -782,13 +1025,17 @@ ValueNode* MaglevPhiRepresentationSelector::AddNode(ValueNode* node,
     DCHECK_EQ(pos, NewNodePosition::kEnd);
     block->nodes().Add(node);
   }
+  RegisterNewNode(node);
+  return node;
+}
+
+void MaglevPhiRepresentationSelector::RegisterNewNode(ValueNode* node) {
   if (builder_->has_graph_labeller()) {
     builder_->graph_labeller()->RegisterNode(node);
   }
 #ifdef DEBUG
   new_nodes_.insert(node);
 #endif
-  return node;
 }
 
 void MaglevPhiRepresentationSelector::MergeNewNodesInBlock(BasicBlock* block) {
@@ -807,6 +1054,68 @@ void MaglevPhiRepresentationSelector::MergeNewNodesInBlock(BasicBlock* block) {
   new_nodes_current_block_end_.clear();
 }
 
+void MaglevPhiRepresentationSelector::PreparePhiTaggings(
+    BasicBlock* old_block, const BasicBlock* new_block) {
+  // Sealing and saving current snapshot
+  if (phi_taggings_.IsSealed()) {
+    phi_taggings_.StartNewSnapshot();
+    return;
+  }
+  old_block->SetSnapshot(phi_taggings_.Seal());
+
+  // Setting up new snapshot
+  predecessors_.clear();
+
+  if (!new_block->is_merge_block()) {
+    BasicBlock* pred = new_block->predecessor();
+    predecessors_.push_back(pred->snapshot());
+  } else {
+    int skip_backedge = new_block->is_loop();
+    for (int i = 0; i < new_block->predecessor_count() - skip_backedge; i++) {
+      BasicBlock* pred = new_block->predecessor_at(i);
+      predecessors_.push_back(pred->snapshot());
+    }
+  }
+
+  auto merge_taggings =
+      [&](Key key, base::Vector<ValueNode* const> predecessors) -> ValueNode* {
+    for (ValueNode* node : predecessors) {
+      if (node == nullptr) {
+        // There is a predecessor that doesn't have this Tagging, so we'll
+        // return nullptr, and if we need it in the future, we'll have to
+        // recreate it. An alternative would be to eagerly insert this Tagging
+        // in all of the other predecesors, but it's possible that it's not used
+        // anymore or not on all future path, so this could also introduce
+        // unnecessary tagging.
+        return static_cast<Phi*>(nullptr);
+      }
+    }
+
+    // Only merge blocks should require Phis.
+    DCHECK(new_block->is_merge_block());
+
+    // We create a Phi to merge all of the existing taggings.
+    int predecessor_count = new_block->predecessor_count();
+    Phi* phi = Node::New<Phi>(builder_->zone(), predecessor_count,
+                              new_block->state(), interpreter::Register());
+    for (int i = 0; static_cast<size_t>(i) < predecessors.size(); i++) {
+      phi->set_input(i, predecessors[i]);
+    }
+    if (predecessors.size() != static_cast<size_t>(predecessor_count)) {
+      // The backedge is omitted from {predecessors}. With set the Phi as its
+      // own backedge.
+      DCHECK(new_block->is_loop());
+      phi->set_input(predecessor_count - 1, phi);
+    }
+    RegisterNewNode(phi);
+    new_block->AddPhi(phi);
+
+    return phi;
+  };
+
+  phi_taggings_.StartNewSnapshot(base::VectorOf(predecessors_), merge_taggings);
+}
+
 }  // namespace maglev
 }  // namespace internal
 }  // namespace v8
