@@ -27,7 +27,9 @@
 #include "src/maglev/maglev-regalloc-data.h"
 #include "src/zone/zone-containers.h"
 
-#ifdef V8_TARGET_ARCH_ARM64
+#ifdef V8_TARGET_ARCH_ARM
+#include "src/codegen/arm/register-arm.h"
+#elif V8_TARGET_ARCH_ARM64
 #include "src/codegen/arm64/register-arm64.h"
 #elif V8_TARGET_ARCH_X64
 #include "src/codegen/x64/register-x64.h"
@@ -66,9 +68,15 @@ ControlNode* NearestPostDominatingHole(ControlNode* node) {
   // If the node is a Jump, it may be a hole, but only if it is not a
   // fallthrough (jump to the immediately next block). Otherwise, it will point
   // to the nearest post-dominating hole in its own "next" field.
-  if (Jump* jump = node->TryCast<Jump>()) {
-    if (IsTargetOfNodeFallthrough(jump, jump->target())) {
-      return jump->next_post_dominating_hole();
+  if (node->Is<Jump>() || node->Is<CheckpointedJump>()) {
+    BasicBlock* target;
+    if (auto jmp = node->TryCast<Jump>()) {
+      target = jmp->target();
+    } else {
+      target = node->Cast<CheckpointedJump>()->target();
+    }
+    if (IsTargetOfNodeFallthrough(node, target)) {
+      return node->next_post_dominating_hole();
     }
   }
 
@@ -136,7 +144,7 @@ ControlNode* HighestPostDominatingHole(
 
 bool IsLiveAtTarget(ValueNode* node, ControlNode* source, BasicBlock* target) {
   DCHECK_NOT_NULL(node);
-  DCHECK(!node->is_dead());
+  DCHECK(!node->has_no_more_uses());
 
   // If we're looping, a value can only be live if it was live before the loop.
   if (target->control_node()->id() <= source->id()) {
@@ -180,9 +188,10 @@ void ClearDeadFallthroughRegisters(RegisterFrameState<RegisterT>& registers,
 }
 
 bool IsDeadNodeToSkip(Node* node) {
-  return node->Is<ValueNode>() && node->Cast<ValueNode>()->is_dead() &&
+  return node->Is<ValueNode>() && node->Cast<ValueNode>()->has_no_more_uses() &&
          !node->properties().is_required_when_unused();
 }
+
 }  // namespace
 
 StraightForwardRegisterAllocator::StraightForwardRegisterAllocator(
@@ -192,6 +201,26 @@ StraightForwardRegisterAllocator::StraightForwardRegisterAllocator(
   AllocateRegisters();
   uint32_t tagged_stack_slots = tagged_.top;
   uint32_t untagged_stack_slots = untagged_.top;
+  if (graph_->is_osr()) {
+    // Fix our stack frame to be compatible with the source stack frame of this
+    // OSR transition:
+    // 1) Ensure the section with tagged slots is big enough to receive all
+    //    live OSR-in values.
+    for (auto val : graph_->osr_values()) {
+      if (val->result().operand().IsAllocated() &&
+          val->stack_slot() >= tagged_stack_slots) {
+        tagged_stack_slots = val->stack_slot() + 1;
+      }
+    }
+    // 2) Ensure we never have to shrink stack frames when OSR'ing into Maglev.
+    //    We don't grow tagged slots or they might end up being uninitialized.
+    uint32_t source_frame_size =
+        graph_->min_maglev_stackslots_for_unoptimized_frame_size();
+    uint32_t target_frame_size = tagged_stack_slots + untagged_stack_slots;
+    if (source_frame_size > target_frame_size) {
+      untagged_stack_slots += source_frame_size - target_frame_size;
+    }
+  }
 #ifdef V8_TARGET_ARCH_ARM64
   // Due to alignment constraints, we add one untagged slot if
   // stack_slots + fixed_slot_count is odd.
@@ -340,10 +369,18 @@ void StraightForwardRegisterAllocator::AllocateRegisters() {
     constant->SetConstantLocation();
     USE(value);
   }
+  for (const auto& [value, constant] : graph_->tagged_index()) {
+    constant->SetConstantLocation();
+    USE(value);
+  }
   for (const auto& [value, constant] : graph_->int32()) {
     constant->SetConstantLocation();
     USE(value);
   }
+  for (const auto& [value, constant] : graph_->uint32()) {
+    constant->SetConstantLocation();
+    USE(value);
+  }
   for (const auto& [value, constant] : graph_->float64()) {
     constant->SetConstantLocation();
     USE(value);
@@ -386,8 +423,12 @@ void StraightForwardRegisterAllocator::AllocateRegisters() {
       if (!control->Is<JumpLoop>()) {
         printing_visitor_->os() << "\n[holes:";
         while (true) {
-          if (control->Is<Jump>()) {
-            BasicBlock* target = control->Cast<Jump>()->target();
+          if (control->Is<JumpLoop>()) {
+            printing_visitor_->os() << " " << control->id() << "↰";
+            break;
+          } else if (control->Is<UnconditionalControlNode>()) {
+            BasicBlock* target =
+                control->Cast<UnconditionalControlNode>()->target();
             printing_visitor_->os()
                 << " " << control->id() << "-" << target->first_id();
             control = control->next_post_dominating_hole();
@@ -409,9 +450,6 @@ void StraightForwardRegisterAllocator::AllocateRegisters() {
           } else if (control->Is<Deopt>() || control->Is<Abort>()) {
             printing_visitor_->os() << " " << control->id() << "✖️";
             break;
-          } else if (control->Is<JumpLoop>()) {
-            printing_visitor_->os() << " " << control->id() << "↰";
-            break;
           }
           UNREACHABLE();
         }
@@ -422,65 +460,68 @@ void StraightForwardRegisterAllocator::AllocateRegisters() {
 
     // Activate phis.
     if (block->has_phi()) {
+      Phi::List& phis = *block->phis();
       // Firstly, make the phi live, and try to assign it to an input
       // location.
-      for (Phi* phi : *block->phis()) {
-        // Ignore dead phis.
-        // TODO(leszeks): We should remove dead phis entirely and turn this
-        // into a DCHECK.
-        if (!phi->has_valid_live_range()) continue;
-        phi->SetNoSpill();
-        TryAllocateToInput(phi);
+      for (auto phi_it = phis.begin(); phi_it != phis.end();) {
+        Phi* phi = *phi_it;
+        if (!phi->has_valid_live_range()) {
+          // We might still have left over dead Phis, due to phis being kept
+          // alive by deopts that the representation analysis dropped. Clear
+          // them out now.
+          phi_it = phis.RemoveAt(phi_it);
+        } else {
+          DCHECK(phi->has_valid_live_range());
+          phi->SetNoSpill();
+          TryAllocateToInput(phi);
+          ++phi_it;
+        }
       }
       if (block->is_exception_handler_block()) {
         // If we are in exception handler block, then we find the ExceptionPhi
         // (the first one by default) that is marked with the
         // virtual_accumulator and force kReturnRegister0. This corresponds to
         // the exception message object.
-        Phi::List::Iterator phi_it = block->phis()->begin();
-        Phi* phi = *phi_it;
-        DCHECK_EQ(phi->input_count(), 0);
-        if (phi->owner() == interpreter::Register::virtual_accumulator() &&
-            !phi->is_dead()) {
-          phi->result().SetAllocated(ForceAllocate(kReturnRegister0, phi));
-          if (v8_flags.trace_maglev_regalloc) {
-            printing_visitor_->Process(phi, ProcessingState(block_it_));
-            printing_visitor_->os() << "phi (exception message object) "
-                                    << phi->result().operand() << std::endl;
+        for (Phi* phi : phis) {
+          DCHECK_EQ(phi->input_count(), 0);
+          DCHECK(phi->is_exception_phi());
+          if (phi->owner() == interpreter::Register::virtual_accumulator()) {
+            if (!phi->has_no_more_uses()) {
+              phi->result().SetAllocated(ForceAllocate(kReturnRegister0, phi));
+              if (v8_flags.trace_maglev_regalloc) {
+                printing_visitor_->Process(phi, ProcessingState(block_it_));
+                printing_visitor_->os() << "phi (exception message object) "
+                                        << phi->result().operand() << std::endl;
+              }
+            }
+          } else if (phi->owner().is_parameter() &&
+                     phi->owner().is_receiver()) {
+            // The receiver is a special case for a fairly silly reason:
+            // OptimizedFrame::Summarize requires the receiver (and the
+            // function) to be in a stack slot, since its value must be
+            // available even though we're not deoptimizing (and thus register
+            // states are not available).
+            //
+            // TODO(leszeks):
+            // For inlined functions / nested graph generation, this a) doesn't
+            // work (there's no receiver stack slot); and b) isn't necessary
+            // (Summarize only looks at noninlined functions).
+            phi->Spill(compiler::AllocatedOperand(
+                compiler::AllocatedOperand::STACK_SLOT,
+                MachineRepresentation::kTagged,
+                (StandardFrameConstants::kExpressionsOffset -
+                 UnoptimizedFrameConstants::kRegisterFileFromFp) /
+                        kSystemPointerSize +
+                    interpreter::Register::receiver().index()));
+            phi->result().SetAllocated(phi->spill_slot());
+            // Break once both accumulator and receiver have been processed.
+            break;
           }
         }
-        // The receiver is the next phi after the accumulator (or the first phi
-        // if there is no accumulator).
-        if (phi->owner() == interpreter::Register::virtual_accumulator()) {
-          ++phi_it;
-          phi = *phi_it;
-        }
-        DCHECK(phi->owner().is_receiver());
-        // The receiver is a special case for a fairly silly reason:
-        // OptimizedFrame::Summarize requires the receiver (and the function)
-        // to be in a stack slot, since its value must be available even
-        // though we're not deoptimizing (and thus register states are not
-        // available).
-        //
-        // TODO(leszeks):
-        // For inlined functions / nested graph generation, this a) doesn't
-        // work (there's no receiver stack slot); and b) isn't necessary
-        // (Summarize only looks at noninlined functions).
-        phi->Spill(compiler::AllocatedOperand(
-            compiler::AllocatedOperand::STACK_SLOT,
-            MachineRepresentation::kTagged,
-            (StandardFrameConstants::kExpressionsOffset -
-             UnoptimizedFrameConstants::kRegisterFileFromFp) /
-                    kSystemPointerSize +
-                interpreter::Register::receiver().index()));
-        phi->result().SetAllocated(phi->spill_slot());
       }
       // Secondly try to assign the phi to a free register.
-      for (Phi* phi : *block->phis()) {
-        // Ignore dead phis.
-        // TODO(leszeks): We should remove dead phis entirely and turn this into
-        // a DCHECK.
-        if (!phi->has_valid_live_range()) continue;
+      for (Phi* phi : phis) {
+        DCHECK(phi->has_valid_live_range());
         if (phi->result().operand().IsAllocated()) continue;
         if (phi->use_double_register()) {
           if (!double_registers_.UnblockedFreeIsEmpty()) {
@@ -510,11 +551,8 @@ void StraightForwardRegisterAllocator::AllocateRegisters() {
         }
       }
       // Finally just use a stack slot.
-      for (Phi* phi : *block->phis()) {
-        // Ignore dead phis.
-        // TODO(leszeks): We should remove dead phis entirely and turn this into
-        // a DCHECK.
-        if (!phi->has_valid_live_range()) continue;
+      for (Phi* phi : phis) {
+        DCHECK(phi->has_valid_live_range());
         if (phi->result().operand().IsAllocated()) continue;
         AllocateSpillSlot(phi);
         // TODO(verwaest): Will this be used at all?
@@ -586,12 +624,12 @@ void StraightForwardRegisterAllocator::UpdateUse(
         << "Using " << PrintNodeLabel(graph_labeller(), node) << "...\n";
   }
 
-  DCHECK(!node->is_dead());
+  DCHECK(!node->has_no_more_uses());
 
   // Update the next use.
-  node->set_next_use(input_location->next_use_id());
+  node->advance_next_use(input_location->next_use_id());
 
-  if (!node->is_dead()) return;
+  if (!node->has_no_more_uses()) return;
 
   if (v8_flags.trace_maglev_regalloc) {
     printing_visitor_->os()
@@ -611,7 +649,10 @@ void StraightForwardRegisterAllocator::UpdateUse(
       DCHECK_IMPLIES(
           slots.free_slots.size() > 0,
           slots.free_slots.back().freed_at_position <= node->live_range().end);
-      slots.free_slots.emplace_back(slot.index(), node->live_range().end);
+      bool double_slot =
+          IsDoubleRepresentation(node->properties().value_representation());
+      slots.free_slots.emplace_back(slot.index(), node->live_range().end,
+                                    double_slot);
     }
   }
 }
@@ -620,6 +661,7 @@ void StraightForwardRegisterAllocator::AllocateEagerDeopt(
     const EagerDeoptInfo& deopt_info) {
   detail::DeepForEachInput(
       &deopt_info, [&](ValueNode* node, InputLocation* input) {
+        DCHECK(!node->Is<Identity>());
         // We might have dropped this node without spilling it. Spill it now.
         if (!node->has_register() && !node->is_loadable()) {
           Spill(node);
@@ -633,6 +675,7 @@ void StraightForwardRegisterAllocator::AllocateLazyDeopt(
     const LazyDeoptInfo& deopt_info) {
   detail::DeepForEachInput(&deopt_info,
                            [&](ValueNode* node, InputLocation* input) {
+                             DCHECK(!node->Is<Identity>());
                              // Lazy deopts always need spilling, and should
                              // always be loaded from their loadable slot.
                              Spill(node);
@@ -714,6 +757,7 @@ void StraightForwardRegisterAllocator::AllocateNode(Node* node) {
     AllocateLazyDeopt(*node->lazy_deopt_info());
   }
 
+  // Make sure to save snapshot after allocate eager deopt registers.
   if (node->properties().needs_register_snapshot()) SaveRegisterSnapshot(node);
 
   if (v8_flags.trace_maglev_regalloc) {
@@ -768,13 +812,28 @@ void StraightForwardRegisterAllocator::AllocateNodeResult(ValueNode* node) {
 
   if (operand.basic_policy() == compiler::UnallocatedOperand::FIXED_SLOT) {
     DCHECK(node->Is<InitialValue>());
-    DCHECK_LT(operand.fixed_slot_index(), 0);
+    DCHECK_IMPLIES(!graph_->is_osr(), operand.fixed_slot_index() < 0);
     // Set the stack slot to exactly where the value is.
     compiler::AllocatedOperand location(compiler::AllocatedOperand::STACK_SLOT,
                                         node->GetMachineRepresentation(),
                                         operand.fixed_slot_index());
     node->result().SetAllocated(location);
     node->Spill(location);
+
+    int idx = operand.fixed_slot_index();
+    if (idx > 0) {
+      // Reserve this slot by increasing the top and also marking slots below as
+      // free. Assumes slots are processed in increasing order.
+      CHECK(node->is_tagged());
+      CHECK_GE(idx, tagged_.top);
+      for (int i = tagged_.top; i < idx; ++i) {
+        bool double_slot =
+            IsDoubleRepresentation(node->properties().value_representation());
+        tagged_.free_slots.emplace_back(i, node->live_range().start,
+                                        double_slot);
+      }
+      tagged_.top = idx + 1;
+    }
     return;
   }
 
@@ -824,7 +883,7 @@ void StraightForwardRegisterAllocator::AllocateNodeResult(ValueNode* node) {
     DCHECK(node->has_register());
     FreeRegistersUsedBy(node);
     DCHECK(!node->has_register());
-    DCHECK(node->is_dead());
+    DCHECK(node->has_no_more_uses());
   }
 }
 
@@ -889,15 +948,19 @@ void StraightForwardRegisterAllocator::InitializeBranchTargetPhis(
   // which means we shouldn't update register state as we go (as if we were
   // emitting a series of serialised moves) but rather take 'old' register
   // state as the phi input.
-  Phi::List* phis = target->phis();
-  for (Phi* phi : *phis) {
-    // Ignore dead phis.
-    // TODO(leszeks): We should remove dead phis entirely and turn this into a
-    // DCHECK.
-    if (!phi->has_valid_live_range()) continue;
-
-    Input& input = phi->input(predecessor_id);
-    input.InjectLocation(input.node()->allocation());
+  Phi::List& phis = *target->phis();
+  for (auto phi_it = phis.begin(); phi_it != phis.end();) {
+    Phi* phi = *phi_it;
+    if (!phi->has_valid_live_range()) {
+      // We might still have left over dead Phis, due to phis being kept
+      // alive by deopts that the representation analysis dropped. Clear
+      // them out now.
+      phi_it = phis.RemoveAt(phi_it);
+    } else {
+      Input& input = phi->input(predecessor_id);
+      input.InjectLocation(input.node()->allocation());
+      ++phi_it;
+    }
   }
 }
 
@@ -928,7 +991,7 @@ void StraightForwardRegisterAllocator::AllocateControlNode(ControlNode* node,
   // Control nodes can't lazy deopt at the moment.
   DCHECK(!node->properties().can_lazy_deopt());
 
-  if (node->Is<JumpToInlined>() || node->Is<Abort>()) {
+  if (node->Is<Abort>()) {
     // Do nothing.
     DCHECK(node->general_temporaries().is_empty());
     DCHECK(node->double_temporaries().is_empty());
@@ -1317,7 +1380,7 @@ void StraightForwardRegisterAllocator::AssignArbitraryRegisterInput(
   UpdateUse(&input);
   // Only need to mark the location as clobbered if the node wasn't already
   // killed by UpdateUse.
-  if (is_clobbered && !node->is_dead()) {
+  if (is_clobbered && !node->has_no_more_uses()) {
     MarkAsClobbered(node, location);
   }
   // Clobbered inputs should no longer be in the allocated location, as far as
@@ -1462,16 +1525,12 @@ void StraightForwardRegisterAllocator::VerifyRegisterState() {
   for (BasicBlock* block : *graph_) {
     if (block->has_phi()) {
       for (Phi* phi : *block->phis()) {
-        // Ignore dead phis.
-        // TODO(leszeks): We should remove dead phis entirely and turn this into
-        // a DCHECK.
-        if (!phi->has_valid_live_range()) continue;
         ValidateValueNode(phi);
       }
     }
     for (Node* node : block->nodes()) {
-      if (IsDeadNodeToSkip(node)) continue;
       if (ValueNode* value_node = node->TryCast<ValueNode>()) {
+        if (node->Is<Identity>()) continue;
         ValidateValueNode(value_node);
       }
     }
@@ -1528,6 +1587,25 @@ void StraightForwardRegisterAllocator::SaveRegisterSnapshot(NodeBase* node) {
       snapshot.live_tagged_registers.clear(reg);
     }
   }
+  if (node->properties().can_eager_deopt()) {
+    // If we eagerly deopt after a deferred call, the registers saved by the
+    // runtime call might not include the inputs into the eager deopt. Here, we
+    // make sure that all the eager deopt registers are included in the
+    // snapshot.
+    detail::DeepForEachInput(
+        node->eager_deopt_info(), [&](ValueNode* node, InputLocation* input) {
+          if (!input->IsAnyRegister()) return;
+          if (input->IsDoubleRegister()) {
+            snapshot.live_double_registers.set(input->AssignedDoubleRegister());
+          } else {
+            snapshot.live_registers.set(input->AssignedGeneralRegister());
+            if (node->is_tagged()) {
+              snapshot.live_tagged_registers.set(
+                  input->AssignedGeneralRegister());
+            }
+          }
+        });
+  }
   node->set_register_snapshot(snapshot);
 }
 
@@ -1536,12 +1614,21 @@ void StraightForwardRegisterAllocator::AllocateSpillSlot(ValueNode* node) {
   uint32_t free_slot;
   bool is_tagged = (node->properties().value_representation() ==
                     ValueRepresentation::kTagged);
-  // TODO(v8:7700): We will need a new class of SpillSlots for doubles in 32-bit
-  // architectures.
+  uint32_t slot_size = 1;
+  bool double_slot =
+      IsDoubleRepresentation(node->properties().value_representation());
+  if constexpr (kDoubleSize != kSystemPointerSize) {
+    if (double_slot) {
+      slot_size = kDoubleSize / kSystemPointerSize;
+    }
+  }
   SpillSlots& slots = is_tagged ? tagged_ : untagged_;
   MachineRepresentation representation = node->GetMachineRepresentation();
-  if (!v8_flags.maglev_reuse_stack_slots || slots.free_slots.empty()) {
-    free_slot = slots.top++;
+  // TODO(victorgomes): We don't currently reuse double slots on arm.
+  if (!v8_flags.maglev_reuse_stack_slots || slot_size > 1 ||
+      slots.free_slots.empty()) {
+    free_slot = slots.top + slot_size - 1;
+    slots.top += slot_size;
   } else {
     NodeIdT start = node->live_range().start;
     auto it =
@@ -1549,10 +1636,23 @@ void StraightForwardRegisterAllocator::AllocateSpillSlot(ValueNode* node) {
                          start, [](NodeIdT s, const SpillSlotInfo& slot_info) {
                            return slot_info.freed_at_position >= s;
                          });
+    // {it} points to the first invalid slot. Decrement it to get to the last
+    // valid slot freed before {start}.
     if (it != slots.free_slots.begin()) {
-      // {it} points to the first invalid slot. Decrement it to get to the last
-      // valid slot freed before {start}.
       --it;
+    }
+
+    // TODO(olivf): Currently we cannot mix double and normal stack slots since
+    // the gap resolver treats them independently and cannot detect cycles via
+    // shared slots.
+    while (it != slots.free_slots.begin()) {
+      if (it->double_slot == double_slot) break;
+      --it;
+    }
+
+    if (it != slots.free_slots.begin()) {
+      CHECK_EQ(double_slot, it->double_slot);
+      CHECK_GT(start, it->freed_at_position);
       free_slot = it->slot_index;
       slots.free_slots.erase(it);
     } else {
@@ -1582,7 +1682,7 @@ RegisterT StraightForwardRegisterAllocator::PickRegisterToFree(
       best = reg;
       break;
     }
-    int use = value->next_use();
+    int use = value->current_next_use();
     if (use > furthest_use) {
       furthest_use = use;
       best = reg;
@@ -1840,6 +1940,10 @@ RegListBase<RegisterT> GetReservedRegisters(NodeBase* node_base) {
   compiler::UnallocatedOperand operand =
       compiler::UnallocatedOperand::cast(node->result().operand());
   RegListBase<RegisterT> reserved = {node->GetRegisterHint<RegisterT>()};
+  if (operand.basic_policy() == compiler::UnallocatedOperand::FIXED_SLOT) {
+    DCHECK(node->Is<InitialValue>());
+    return reserved;
+  }
   if constexpr (std::is_same_v<RegisterT, Register>) {
     if (operand.extended_policy() ==
         compiler::UnallocatedOperand::FIXED_REGISTER) {
@@ -1934,8 +2038,10 @@ void StraightForwardRegisterAllocator::ClearRegisterValues() {
   ClearRegisterState(double_registers_);
 
   // All registers should be free by now.
-  DCHECK_EQ(general_registers_.unblocked_free(), kAllocatableGeneralRegisters);
-  DCHECK_EQ(double_registers_.unblocked_free(), kAllocatableDoubleRegisters);
+  DCHECK_EQ(general_registers_.unblocked_free(),
+            MaglevAssembler::GetAllocatableRegisters());
+  DCHECK_EQ(double_registers_.unblocked_free(),
+            MaglevAssembler::GetAllocatableDoubleRegisters());
 }
 
 void StraightForwardRegisterAllocator::InitializeRegisterValues(
@@ -2204,7 +2310,7 @@ void StraightForwardRegisterAllocator::MergeRegisterValues(ControlNode* control,
         // though).
         DCHECK_IMPLIES(
             !incoming->is_loadable() && !IsInRegister(target_state, incoming),
-            !IsForwardReachable(target, incoming->next_use(),
+            !IsForwardReachable(target, incoming->current_next_use(),
                                 incoming->live_range().end));
       }
 
