@@ -34,16 +34,14 @@ class AccessorInfo
   // returns/accepts C function and converts the value from and to redirected
   // pointer.
   DECL_EXTERNAL_POINTER_ACCESSORS_MAYBE_READ_ONLY_HOST(getter, Address)
-  inline void init_getter_redirection(Isolate* isolate);
-  inline void remove_getter_redirection(Isolate* isolate);
+  inline void init_getter_redirection(IsolateForSandbox isolate);
+  inline void remove_getter_redirection(IsolateForSandbox isolate);
   inline bool has_getter(Isolate* isolate);
 
   // The field contains the address of the C function.
   DECL_EXTERNAL_POINTER_ACCESSORS_MAYBE_READ_ONLY_HOST(setter, Address)
   inline bool has_setter(Isolate* isolate);
 
-  DECL_BOOLEAN_ACCESSORS(all_can_read)
-  DECL_BOOLEAN_ACCESSORS(all_can_write)
   DECL_BOOLEAN_ACCESSORS(is_special_data_property)
   DECL_BOOLEAN_ACCESSORS(replace_on_access)
   DECL_BOOLEAN_ACCESSORS(is_sloppy)
@@ -107,7 +105,6 @@ class InterceptorInfo
     : public TorqueGeneratedInterceptorInfo<InterceptorInfo, Struct> {
  public:
   DECL_BOOLEAN_ACCESSORS(can_intercept_symbols)
-  DECL_BOOLEAN_ACCESSORS(all_can_read)
   DECL_BOOLEAN_ACCESSORS(non_masking)
   DECL_BOOLEAN_ACCESSORS(is_named)
   DECL_BOOLEAN_ACCESSORS(has_no_side_effect)
@@ -133,8 +130,8 @@ class CallHandlerInfo
   // returns/accepts C function and converts the value from and to redirected
   // pointer.
   DECL_EXTERNAL_POINTER_ACCESSORS_MAYBE_READ_ONLY_HOST(callback, Address)
-  inline void init_callback_redirection(i::Isolate* isolate);
-  inline void remove_callback_redirection(i::Isolate* isolate);
+  inline void init_callback_redirection(i::IsolateForSandbox isolate);
+  inline void remove_callback_redirection(i::IsolateForSandbox isolate);
 
   class BodyDescriptor;
 
