@@ -4,6 +4,7 @@
 
 #include "src/objects/feedback-vector.h"
 
+#include "src/base/optional.h"
 #include "src/common/globals.h"
 #include "src/deoptimizer/deoptimizer.h"
 #include "src/diagnostics/code-tracer.h"
@@ -15,7 +16,6 @@
 #include "src/objects/feedback-vector-inl.h"
 #include "src/objects/hash-table-inl.h"
 #include "src/objects/map-inl.h"
-#include "src/objects/object-macros.h"
 #include "src/objects/objects.h"
 
 namespace v8 {
@@ -32,15 +32,15 @@ FeedbackSlot FeedbackVectorSpec::AddSlot(FeedbackSlotKind kind) {
 }
 
 static bool IsPropertyNameFeedback(MaybeObject feedback) {
-  HeapObject heap_object;
-  if (!feedback->GetHeapObjectIfStrong(&heap_object)) return false;
-  if (heap_object.IsString()) {
-    DCHECK(heap_object.IsInternalizedString());
+  Tagged<HeapObject> heap_object;
+  if (!feedback.GetHeapObjectIfStrong(&heap_object)) return false;
+  if (IsString(heap_object)) {
+    DCHECK(IsInternalizedString(heap_object));
     return true;
   }
-  if (!heap_object.IsSymbol()) return false;
-  Symbol symbol = Symbol::cast(heap_object);
-  ReadOnlyRoots roots = symbol.GetReadOnlyRoots();
+  if (!IsSymbol(heap_object)) return false;
+  Tagged<Symbol> symbol = Symbol::cast(heap_object);
+  ReadOnlyRoots roots = symbol->GetReadOnlyRoots();
   return symbol != roots.uninitialized_symbol() &&
          symbol != roots.mega_dom_symbol() &&
          symbol != roots.megamorphic_symbol();
@@ -183,32 +183,40 @@ const char* FeedbackMetadata::Kind2String(FeedbackSlotKind kind) {
 
 FeedbackSlotKind FeedbackVector::GetKind(FeedbackSlot slot) const {
   DCHECK(!is_empty());
-  return metadata().GetKind(slot);
+  return metadata()->GetKind(slot);
 }
 
 FeedbackSlotKind FeedbackVector::GetKind(FeedbackSlot slot,
                                          AcquireLoadTag tag) const {
   DCHECK(!is_empty());
-  return metadata(tag).GetKind(slot);
+  return metadata(tag)->GetKind(slot);
 }
 
 // static
 Handle<ClosureFeedbackCellArray> ClosureFeedbackCellArray::New(
-    Isolate* isolate, Handle<SharedFunctionInfo> shared) {
-  Factory* factory = isolate->factory();
-
-  int num_feedback_cells =
-      shared->feedback_metadata().create_closure_slot_count();
-
-  Handle<ClosureFeedbackCellArray> feedback_cell_array =
-      factory->NewClosureFeedbackCellArray(num_feedback_cells);
-
-  for (int i = 0; i < num_feedback_cells; i++) {
-    Handle<FeedbackCell> cell =
-        factory->NewNoClosuresCell(factory->undefined_value());
-    feedback_cell_array->set(i, *cell);
+    Isolate* isolate, Handle<SharedFunctionInfo> shared,
+    AllocationType allocation) {
+  int length = shared->feedback_metadata()->create_closure_slot_count();
+  if (length == 0) {
+    return isolate->factory()->empty_closure_feedback_cell_array();
   }
-  return feedback_cell_array;
+
+  // Pre-allocate the cells s.t. we can initialize `result` without further
+  // allocation.
+  Handle<HeapObject> undefined = isolate->factory()->undefined_value();
+  std::vector<Handle<FeedbackCell>> cells;
+  cells.reserve(length);
+  for (int i = 0; i < length; i++) {
+    cells.push_back(isolate->factory()->NewNoClosuresCell(undefined));
+  }
+
+  base::Optional<DisallowGarbageCollection> no_gc;
+  auto result = Allocate(isolate, length, &no_gc, allocation);
+  for (int i = 0; i < length; i++) {
+    result->set(i, *cells[i]);
+  }
+
+  return result;
 }
 
 // static
@@ -339,36 +347,45 @@ Handle<FeedbackVector> FeedbackVector::NewWithOneCompareSlotForTesting(
 void FeedbackVector::AddToVectorsForProfilingTools(
     Isolate* isolate, Handle<FeedbackVector> vector) {
   DCHECK(!isolate->is_best_effort_code_coverage());
-  if (!vector->shared_function_info().IsSubjectToDebugging()) return;
+  if (!vector->shared_function_info()->IsSubjectToDebugging()) return;
   Handle<ArrayList> list = Handle<ArrayList>::cast(
       isolate->factory()->feedback_vectors_for_profiling_tools());
   list = ArrayList::Add(isolate, list, vector);
   isolate->SetFeedbackVectorsForProfilingTools(*list);
 }
 
-void FeedbackVector::SetOptimizedCode(Code code) {
-  DCHECK(CodeKindIsOptimizedJSFunction(code.kind()));
-  // We should set optimized code only when there is no valid optimized code.
-  DCHECK(!has_optimized_code() ||
-         optimized_code().marked_for_deoptimization() ||
-         (CodeKindCanTierUp(optimized_code().kind()) &&
-          optimized_code().kind() < code.kind()) ||
-         v8_flags.stress_concurrent_inlining_attach_code);
+void FeedbackVector::SetOptimizedCode(IsolateForSandbox isolate,
+                                      Tagged<Code> code) {
+  DCHECK(CodeKindIsOptimizedJSFunction(code->kind()));
+  int32_t state = flags();
+  // Skip setting optimized code if it would cause us to tier down.
+  if (!has_optimized_code()) {
+    state = MaybeHasTurbofanCodeBit::update(state, false);
+  } else if (!CodeKindCanTierUp(optimized_code(isolate)->kind()) ||
+             optimized_code(isolate)->kind() > code->kind()) {
+    if (!v8_flags.stress_concurrent_inlining_attach_code &&
+        !optimized_code(isolate)->marked_for_deoptimization()) {
+      return;
+    }
+    // If we fall through, we may be tiering down. This is fine since we only do
+    // that when the current code is marked for deoptimization, or because we're
+    // stress testing.
+    state = MaybeHasTurbofanCodeBit::update(state, false);
+  }
   // TODO(mythria): We could see a CompileOptimized state here either from
   // tests that use %OptimizeFunctionOnNextCall, --always-turbofan or because we
   // re-mark the function for non-concurrent optimization after an OSR. We
   // should avoid these cases and also check that marker isn't
   // TieringState::kRequestTurbofan*.
-  set_maybe_optimized_code(HeapObjectReference::Weak(code));
-  int32_t state = flags();
+  set_maybe_optimized_code(HeapObjectReference::Weak(code->wrapper()));
   // TODO(leszeks): Reconsider whether this could clear the tiering state vs.
   // the callers doing so.
   state = TieringStateBits::update(state, TieringState::kNone);
-  if (code.is_maglevved()) {
+  if (code->is_maglevved()) {
     DCHECK(!MaybeHasTurbofanCodeBit::decode(state));
     state = MaybeHasMaglevCodeBit::update(state, true);
   } else {
-    DCHECK(code.is_turbofanned());
+    DCHECK(code->is_turbofanned());
     state = MaybeHasTurbofanCodeBit::update(state, true);
     state = MaybeHasMaglevCodeBit::update(state, false);
   }
@@ -383,11 +400,16 @@ void FeedbackVector::ClearOptimizedCode() {
   set_maybe_has_turbofan_code(false);
 }
 
-void FeedbackVector::SetOptimizedOsrCode(FeedbackSlot slot, Code code) {
-  DCHECK(CodeKindIsOptimizedJSFunction(code.kind()));
+void FeedbackVector::SetOptimizedOsrCode(Isolate* isolate, FeedbackSlot slot,
+                                         Tagged<Code> code) {
+  DCHECK(CodeKindIsOptimizedJSFunction(code->kind()));
   DCHECK(!slot.IsInvalid());
-  Set(slot, HeapObjectReference::Weak(code));
-  set_maybe_has_optimized_osr_code(true);
+  auto current = GetOptimizedOsrCode(isolate, slot);
+  if (V8_UNLIKELY(current && current.value()->kind() > code->kind())) {
+    return;
+  }
+  Set(slot, HeapObjectReference::Weak(code->wrapper()));
+  set_maybe_has_optimized_osr_code(true, code->kind());
 }
 
 void FeedbackVector::reset_tiering_state() {
@@ -406,7 +428,8 @@ void FeedbackVector::reset_flags() {
             MaybeHasMaglevCodeBit::encode(false) |
             MaybeHasTurbofanCodeBit::encode(false) |
             OsrTieringStateBit::encode(TieringState::kNone) |
-            MaybeHasOptimizedOsrCodeBit::encode(false));
+            MaybeHasMaglevOsrCodeBit::encode(false) |
+            MaybeHasTurbofanOsrCodeBit::encode(false));
 }
 
 TieringState FeedbackVector::osr_tiering_state() {
@@ -423,7 +446,7 @@ void FeedbackVector::set_osr_tiering_state(TieringState marker) {
 }
 
 void FeedbackVector::EvictOptimizedCodeMarkedForDeoptimization(
-    Isolate* isolate, SharedFunctionInfo shared, const char* reason) {
+    Isolate* isolate, Tagged<SharedFunctionInfo> shared, const char* reason) {
   MaybeObject slot = maybe_optimized_code();
   if (slot->IsCleared()) {
     set_maybe_has_maglev_code(false);
@@ -431,15 +454,15 @@ void FeedbackVector::EvictOptimizedCodeMarkedForDeoptimization(
     return;
   }
 
-  Code code = Code::cast(slot->GetHeapObject());
-  if (code.marked_for_deoptimization()) {
+  Tagged<Code> code = CodeWrapper::cast(slot.GetHeapObject())->code(isolate);
+  if (code->marked_for_deoptimization()) {
     Deoptimizer::TraceEvictFromOptimizedCodeCache(isolate, shared, reason);
     ClearOptimizedCode();
   }
 }
 
 bool FeedbackVector::ClearSlots(Isolate* isolate, ClearBehavior behavior) {
-  if (!shared_function_info().HasFeedbackMetadata()) return false;
+  if (!shared_function_info()->HasFeedbackMetadata()) return false;
   MaybeObject uninitialized_sentinel = MaybeObject::FromObject(
       FeedbackVector::RawUninitializedSentinel(isolate));
 
@@ -457,6 +480,36 @@ bool FeedbackVector::ClearSlots(Isolate* isolate, ClearBehavior behavior) {
   return feedback_updated;
 }
 
+#ifdef V8_TRACE_FEEDBACK_UPDATES
+
+// static
+void FeedbackVector::TraceFeedbackChange(Isolate* isolate,
+                                         Tagged<FeedbackVector> vector,
+                                         FeedbackSlot slot,
+                                         const char* reason) {
+  int slot_count = vector->metadata()->slot_count();
+  StdoutStream os;
+  if (slot.IsInvalid()) {
+    os << "[Feedback slots in ";
+  } else {
+    FeedbackSlotKind kind = vector->metadata()->GetKind(slot);
+    os << "[Feedback slot " << slot.ToInt() << "/" << slot_count << " ("
+       << FeedbackMetadata::Kind2String(kind) << ")"
+       << " in ";
+  }
+  ShortPrint(vector->shared_function_info(), os);
+  if (slot.IsInvalid()) {
+    os << " updated - ";
+  } else {
+    os << " updated to ";
+    vector->FeedbackSlotPrint(os, slot);
+    os << " - ";
+  }
+  os << reason << "]" << std::endl;
+}
+
+#endif
+
 MaybeObjectHandle NexusConfig::NewHandle(MaybeObject object) const {
   if (mode() == Mode::MainThread) {
     return handle(object, isolate_);
@@ -466,7 +519,7 @@ MaybeObjectHandle NexusConfig::NewHandle(MaybeObject object) const {
 }
 
 template <typename T>
-Handle<T> NexusConfig::NewHandle(T object) const {
+Handle<T> NexusConfig::NewHandle(Tagged<T> object) const {
   if (mode() == Mode::MainThread) {
     return handle(object, isolate_);
   }
@@ -474,25 +527,25 @@ Handle<T> NexusConfig::NewHandle(T object) const {
   return handle(object, local_heap_);
 }
 
-void NexusConfig::SetFeedbackPair(FeedbackVector vector,
+void NexusConfig::SetFeedbackPair(Tagged<FeedbackVector> vector,
                                   FeedbackSlot start_slot, MaybeObject feedback,
                                   WriteBarrierMode mode,
                                   MaybeObject feedback_extra,
                                   WriteBarrierMode mode_extra) const {
   CHECK(can_write());
-  CHECK_GT(vector.length(), start_slot.WithOffset(1).ToInt());
+  CHECK_GT(vector->length(), start_slot.WithOffset(1).ToInt());
   base::SharedMutexGuard<base::kExclusive> shared_mutex_guard(
       isolate()->feedback_vector_access());
-  vector.Set(start_slot, feedback, mode);
-  vector.Set(start_slot.WithOffset(1), feedback_extra, mode_extra);
+  vector->Set(start_slot, feedback, mode);
+  vector->Set(start_slot.WithOffset(1), feedback_extra, mode_extra);
 }
 
 std::pair<MaybeObject, MaybeObject> NexusConfig::GetFeedbackPair(
-    FeedbackVector vector, FeedbackSlot slot) const {
+    Tagged<FeedbackVector> vector, FeedbackSlot slot) const {
   base::SharedMutexGuardIf<base::kShared> scope(
       isolate()->feedback_vector_access(), mode() == BackgroundThread);
-  MaybeObject feedback = vector.Get(slot);
-  MaybeObject feedback_extra = vector.Get(slot.WithOffset(1));
+  MaybeObject feedback = vector->Get(slot);
+  MaybeObject feedback_extra = vector->Get(slot.WithOffset(1));
   return std::make_pair(feedback, feedback_extra);
 }
 
@@ -504,12 +557,12 @@ FeedbackNexus::FeedbackNexus(Handle<FeedbackVector> vector, FeedbackSlot slot)
   kind_ = vector.is_null() ? FeedbackSlotKind::kInvalid : vector->GetKind(slot);
 }
 
-FeedbackNexus::FeedbackNexus(FeedbackVector vector, FeedbackSlot slot)
+FeedbackNexus::FeedbackNexus(Tagged<FeedbackVector> vector, FeedbackSlot slot)
     : vector_(vector),
       slot_(slot),
       config_(NexusConfig::FromMainThread(
-          vector.is_null() ? nullptr : vector.GetIsolate())) {
-  kind_ = vector.is_null() ? FeedbackSlotKind::kInvalid : vector.GetKind(slot);
+          vector.is_null() ? nullptr : vector->GetIsolate())) {
+  kind_ = vector.is_null() ? FeedbackSlotKind::kInvalid : vector->GetKind(slot);
 }
 
 FeedbackNexus::FeedbackNexus(Handle<FeedbackVector> vector, FeedbackSlot slot,
@@ -657,7 +710,7 @@ bool FeedbackNexus::ConfigureMegamorphic(IcCheckType property_type) {
   return update_required;
 }
 
-Map FeedbackNexus::GetFirstMap() const {
+Tagged<Map> FeedbackNexus::GetFirstMap() const {
   FeedbackIterator it(this);
   if (!it.done()) {
     return it.map();
@@ -672,7 +725,7 @@ InlineCacheState FeedbackNexus::ic_state() const {
 
   switch (kind()) {
     case FeedbackSlotKind::kLiteral:
-      if (feedback->IsSmi()) return InlineCacheState::UNINITIALIZED;
+      if (IsSmi(feedback)) return InlineCacheState::UNINITIALIZED;
       return InlineCacheState::MONOMORPHIC;
 
     case FeedbackSlotKind::kStoreGlobalSloppy:
@@ -680,7 +733,7 @@ InlineCacheState FeedbackNexus::ic_state() const {
     case FeedbackSlotKind::kLoadGlobalNotInsideTypeof:
     case FeedbackSlotKind::kLoadGlobalInsideTypeof:
     case FeedbackSlotKind::kJumpLoop: {
-      if (feedback->IsSmi()) return InlineCacheState::MONOMORPHIC;
+      if (IsSmi(feedback)) return InlineCacheState::MONOMORPHIC;
 
       DCHECK(feedback->IsWeakOrCleared());
       if (!feedback->IsCleared() || extra != UninitializedSentinel()) {
@@ -713,20 +766,21 @@ InlineCacheState FeedbackNexus::ic_state() const {
         // Don't check if the map is cleared.
         return InlineCacheState::MONOMORPHIC;
       }
-      HeapObject heap_object;
-      if (feedback->GetHeapObjectIfStrong(&heap_object)) {
-        if (heap_object.IsWeakFixedArray()) {
+      Tagged<HeapObject> heap_object;
+      if (feedback.GetHeapObjectIfStrong(&heap_object)) {
+        if (IsWeakFixedArray(heap_object)) {
           // Determine state purely by our structure, don't check if the maps
           // are cleared.
           return InlineCacheState::POLYMORPHIC;
         }
-        if (heap_object.IsName()) {
+        if (IsName(heap_object)) {
           DCHECK(IsKeyedLoadICKind(kind()) || IsKeyedStoreICKind(kind()) ||
                  IsKeyedHasICKind(kind()) || IsDefineKeyedOwnICKind(kind()));
-          Object extra_object = extra->GetHeapObjectAssumeStrong();
-          WeakFixedArray extra_array = WeakFixedArray::cast(extra_object);
-          return extra_array.length() > 2 ? InlineCacheState::POLYMORPHIC
-                                          : InlineCacheState::MONOMORPHIC;
+          Tagged<Object> extra_object = extra.GetHeapObjectAssumeStrong();
+          Tagged<WeakFixedArray> extra_array =
+              WeakFixedArray::cast(extra_object);
+          return extra_array->length() > 2 ? InlineCacheState::POLYMORPHIC
+                                           : InlineCacheState::MONOMORPHIC;
         }
       }
       // TODO(1393773): Remove once the issue is solved.
@@ -743,19 +797,19 @@ InlineCacheState FeedbackNexus::ic_state() const {
       UNREACHABLE();
     }
     case FeedbackSlotKind::kCall: {
-      HeapObject heap_object;
+      Tagged<HeapObject> heap_object;
       if (feedback == MegamorphicSentinel()) {
         return InlineCacheState::GENERIC;
       } else if (feedback->IsWeakOrCleared()) {
-        if (feedback->GetHeapObjectIfWeak(&heap_object)) {
-          if (heap_object.IsFeedbackCell()) {
+        if (feedback.GetHeapObjectIfWeak(&heap_object)) {
+          if (IsFeedbackCell(heap_object)) {
             return InlineCacheState::POLYMORPHIC;
           }
-          CHECK(heap_object.IsJSFunction() || heap_object.IsJSBoundFunction());
+          CHECK(IsJSFunction(heap_object) || IsJSBoundFunction(heap_object));
         }
         return InlineCacheState::MONOMORPHIC;
-      } else if (feedback->GetHeapObjectIfStrong(&heap_object) &&
-                 heap_object.IsAllocationSite()) {
+      } else if (feedback.GetHeapObjectIfStrong(&heap_object) &&
+                 IsAllocationSite(heap_object)) {
         return InlineCacheState::MONOMORPHIC;
       }
 
@@ -821,7 +875,7 @@ InlineCacheState FeedbackNexus::ic_state() const {
         return InlineCacheState::MONOMORPHIC;
       }
 
-      DCHECK(feedback->GetHeapObjectAssumeStrong().IsWeakFixedArray());
+      DCHECK(IsWeakFixedArray(feedback.GetHeapObjectAssumeStrong()));
       return InlineCacheState::POLYMORPHIC;
     }
 
@@ -864,15 +918,24 @@ void FeedbackNexus::ConfigureHandlerMode(const MaybeObjectHandle& handler) {
               UPDATE_WRITE_BARRIER, *handler, UPDATE_WRITE_BARRIER);
 }
 
-void FeedbackNexus::ConfigureCloneObject(Handle<Map> source_map,
-                                         Handle<Map> result_map) {
+void FeedbackNexus::ConfigureCloneObject(
+    Handle<Map> source_map, const MaybeObjectHandle& handler_handle) {
+  // TODO(olivf): Introduce a CloneHandler to deal with all the logic of this
+  // state machine which is now spread between Runtime_CloneObjectIC_Miss and
+  // this method.
+  auto GetHandler = [=]() {
+    if (handler_handle->IsSmi()) {
+      return *handler_handle;
+    }
+    return HeapObjectReference::MakeWeak(*handler_handle);
+  };
   DCHECK(config()->can_write());
   Isolate* isolate = GetIsolate();
   Handle<HeapObject> feedback;
   {
     MaybeObject maybe_feedback = GetFeedback();
     if (maybe_feedback->IsStrongOrWeak()) {
-      feedback = handle(maybe_feedback->GetHeapObject(), isolate);
+      feedback = handle(maybe_feedback.GetHeapObject(), isolate);
     } else {
       DCHECK(maybe_feedback->IsCleared());
     }
@@ -881,23 +944,23 @@ void FeedbackNexus::ConfigureCloneObject(Handle<Map> source_map,
     case InlineCacheState::UNINITIALIZED:
       // Cache the first map seen which meets the fast case requirements.
       SetFeedback(HeapObjectReference::Weak(*source_map), UPDATE_WRITE_BARRIER,
-                  *result_map);
+                  GetHandler());
       break;
     case InlineCacheState::MONOMORPHIC:
       if (feedback.is_null() || feedback.is_identical_to(source_map) ||
-          Map::cast(*feedback).is_deprecated()) {
+          Map::cast(*feedback)->is_deprecated()) {
         SetFeedback(HeapObjectReference::Weak(*source_map),
-                    UPDATE_WRITE_BARRIER, *result_map);
+                    UPDATE_WRITE_BARRIER, GetHandler());
       } else {
         // Transition to POLYMORPHIC.
         Handle<WeakFixedArray> array =
             CreateArrayOfSize(2 * kCloneObjectPolymorphicEntrySize);
         DisallowGarbageCollection no_gc;
-        auto raw_array = *array;
-        raw_array.Set(0, HeapObjectReference::Weak(*feedback));
-        raw_array.Set(1, GetFeedbackExtra());
-        raw_array.Set(2, HeapObjectReference::Weak(*source_map));
-        raw_array.Set(3, MaybeObject::FromObject(*result_map));
+        Tagged<WeakFixedArray> raw_array = *array;
+        raw_array->set(0, HeapObjectReference::Weak(*feedback));
+        raw_array->set(1, GetFeedbackExtra());
+        raw_array->set(2, HeapObjectReference::Weak(*source_map));
+        raw_array->set(3, GetHandler());
         SetFeedback(raw_array, UPDATE_WRITE_BARRIER,
                     HeapObjectReference::ClearedValue(isolate));
       }
@@ -908,9 +971,9 @@ void FeedbackNexus::ConfigureCloneObject(Handle<Map> source_map,
       Handle<WeakFixedArray> array = Handle<WeakFixedArray>::cast(feedback);
       int i = 0;
       for (; i < array->length(); i += kCloneObjectPolymorphicEntrySize) {
-        MaybeObject feedback_map = array->Get(i);
+        MaybeObject feedback_map = array->get(i);
         if (feedback_map->IsCleared()) break;
-        Handle<Map> cached_map(Map::cast(feedback_map->GetHeapObject()),
+        Handle<Map> cached_map(Map::cast(feedback_map.GetHeapObject()),
                                isolate);
         if (cached_map.is_identical_to(source_map) ||
             cached_map->is_deprecated())
@@ -930,14 +993,14 @@ void FeedbackNexus::ConfigureCloneObject(Handle<Map> source_map,
         Handle<WeakFixedArray> new_array = CreateArrayOfSize(
             array->length() + kCloneObjectPolymorphicEntrySize);
         for (int j = 0; j < array->length(); ++j) {
-          new_array->Set(j, array->Get(j));
+          new_array->set(j, array->get(j));
         }
         SetFeedback(*new_array);
         array = new_array;
       }
 
-      array->Set(i, HeapObjectReference::Weak(*source_map));
-      array->Set(i + 1, MaybeObject::FromObject(*result_map));
+      array->set(i, HeapObjectReference::Weak(*source_map));
+      array->set(i + 1, GetHandler());
       break;
     }
 
@@ -949,8 +1012,8 @@ void FeedbackNexus::ConfigureCloneObject(Handle<Map> source_map,
 int FeedbackNexus::GetCallCount() {
   DCHECK(IsCallICKind(kind()));
 
-  Object call_count = GetFeedbackExtra()->cast<Object>();
-  CHECK(call_count.IsSmi());
+  Tagged<Object> call_count = GetFeedbackExtra()->cast<Object>();
+  CHECK(IsSmi(call_count));
   uint32_t value = static_cast<uint32_t>(Smi::ToInt(call_count));
   return CallCountField::decode(value);
 }
@@ -958,8 +1021,8 @@ int FeedbackNexus::GetCallCount() {
 void FeedbackNexus::SetSpeculationMode(SpeculationMode mode) {
   DCHECK(IsCallICKind(kind()));
 
-  Object call_count = GetFeedbackExtra()->cast<Object>();
-  CHECK(call_count.IsSmi());
+  Tagged<Object> call_count = GetFeedbackExtra()->cast<Object>();
+  CHECK(IsSmi(call_count));
   uint32_t count = static_cast<uint32_t>(Smi::ToInt(call_count));
   count = SpeculationModeField::update(count, mode);
   MaybeObject feedback = GetFeedback();
@@ -972,8 +1035,8 @@ void FeedbackNexus::SetSpeculationMode(SpeculationMode mode) {
 SpeculationMode FeedbackNexus::GetSpeculationMode() {
   DCHECK(IsCallICKind(kind()));
 
-  Object call_count = GetFeedbackExtra()->cast<Object>();
-  CHECK(call_count.IsSmi());
+  Tagged<Object> call_count = GetFeedbackExtra()->cast<Object>();
+  CHECK(IsSmi(call_count));
   uint32_t value = static_cast<uint32_t>(Smi::ToInt(call_count));
   return SpeculationModeField::decode(value);
 }
@@ -981,8 +1044,8 @@ SpeculationMode FeedbackNexus::GetSpeculationMode() {
 CallFeedbackContent FeedbackNexus::GetCallFeedbackContent() {
   DCHECK(IsCallICKind(kind()));
 
-  Object call_count = GetFeedbackExtra()->cast<Object>();
-  CHECK(call_count.IsSmi());
+  Tagged<Object> call_count = GetFeedbackExtra()->cast<Object>();
+  CHECK(IsSmi(call_count));
   uint32_t value = static_cast<uint32_t>(Smi::ToInt(call_count));
   return CallFeedbackContentField::decode(value);
 }
@@ -990,7 +1053,7 @@ CallFeedbackContent FeedbackNexus::GetCallFeedbackContent() {
 float FeedbackNexus::ComputeCallFrequency() {
   DCHECK(IsCallICKind(kind()));
 
-  double const invocation_count = vector().invocation_count(kRelaxedLoad);
+  double const invocation_count = vector()->invocation_count(kRelaxedLoad);
   double const call_count = GetCallCount();
   if (invocation_count == 0.0) {  // Prevent division by 0.
     return 0.0f;
@@ -1011,8 +1074,8 @@ void FeedbackNexus::ConfigureMonomorphic(Handle<Name> name,
                   UPDATE_WRITE_BARRIER, *handler);
     } else {
       Handle<WeakFixedArray> array = CreateArrayOfSize(2);
-      array->Set(0, HeapObjectReference::Weak(*receiver_map));
-      array->Set(1, *handler);
+      array->set(0, HeapObjectReference::Weak(*receiver_map));
+      array->set(1, *handler);
       SetFeedback(*name, UPDATE_WRITE_BARRIER, *array);
     }
   }
@@ -1026,10 +1089,10 @@ void FeedbackNexus::ConfigurePolymorphic(
 
   for (int current = 0; current < receiver_count; ++current) {
     Handle<Map> map = maps_and_handlers[current].first;
-    array->Set(current * 2, HeapObjectReference::Weak(*map));
+    array->set(current * 2, HeapObjectReference::Weak(*map));
     MaybeObjectHandle handler = maps_and_handlers[current].second;
     DCHECK(IC::IsHandler(*handler));
-    array->Set(current * 2 + 1, *handler);
+    array->set(current * 2 + 1, *handler);
   }
 
   if (name.is_null()) {
@@ -1120,37 +1183,40 @@ MaybeObjectHandle FeedbackNexus::FindHandlerForMap(Handle<Map> map) const {
   return MaybeObjectHandle();
 }
 
-Name FeedbackNexus::GetName() const {
+Tagged<Name> FeedbackNexus::GetName() const {
   if (IsKeyedStoreICKind(kind()) || IsKeyedLoadICKind(kind()) ||
       IsKeyedHasICKind(kind()) || IsDefineKeyedOwnICKind(kind())) {
     MaybeObject feedback = GetFeedback();
     if (IsPropertyNameFeedback(feedback)) {
-      return Name::cast(feedback->GetHeapObjectAssumeStrong());
+      return Name::cast(feedback.GetHeapObjectAssumeStrong());
     }
   }
   if (IsDefineKeyedOwnPropertyInLiteralKind(kind())) {
     MaybeObject extra = GetFeedbackExtra();
     if (IsPropertyNameFeedback(extra)) {
-      return Name::cast(extra->GetHeapObjectAssumeStrong());
+      return Name::cast(extra.GetHeapObjectAssumeStrong());
     }
   }
-  return Name();
+  return {};
 }
 
 KeyedAccessLoadMode FeedbackNexus::GetKeyedAccessLoadMode() const {
   DCHECK(IsKeyedLoadICKind(kind()) || IsKeyedHasICKind(kind()));
 
-  if (GetKeyType() == IcCheckType::kProperty) return STANDARD_LOAD;
+  // TODO(victorgomes): The KeyedAccessLoadMode::kInBounds is doing double duty
+  // here. It shouldn't be used for property loads.
+  if (GetKeyType() == IcCheckType::kProperty)
+    return KeyedAccessLoadMode::kInBounds;
 
   std::vector<MapAndHandler> maps_and_handlers;
   ExtractMapsAndHandlers(&maps_and_handlers);
   for (MapAndHandler map_and_handler : maps_and_handlers) {
     KeyedAccessLoadMode mode =
         LoadHandler::GetKeyedAccessLoadMode(*map_and_handler.second);
-    if (mode != STANDARD_LOAD) return mode;
+    if (LoadModeHandlesOOB(mode)) return mode;
   }
 
-  return STANDARD_LOAD;
+  return KeyedAccessLoadMode::kInBounds;
 }
 
 namespace {
@@ -1158,17 +1224,17 @@ namespace {
 bool BuiltinHasKeyedAccessStoreMode(Builtin builtin) {
   DCHECK(Builtins::IsBuiltinId(builtin));
   switch (builtin) {
-    case Builtin::kKeyedStoreIC_SloppyArguments_Standard:
-    case Builtin::kKeyedStoreIC_SloppyArguments_GrowNoTransitionHandleCOW:
-    case Builtin::kKeyedStoreIC_SloppyArguments_NoTransitionIgnoreOOB:
+    case Builtin::kKeyedStoreIC_SloppyArguments_InBounds:
+    case Builtin::kKeyedStoreIC_SloppyArguments_NoTransitionGrowAndHandleCOW:
+    case Builtin::kKeyedStoreIC_SloppyArguments_NoTransitionIgnoreTypedArrayOOB:
     case Builtin::kKeyedStoreIC_SloppyArguments_NoTransitionHandleCOW:
-    case Builtin::kStoreFastElementIC_Standard:
-    case Builtin::kStoreFastElementIC_GrowNoTransitionHandleCOW:
-    case Builtin::kStoreFastElementIC_NoTransitionIgnoreOOB:
+    case Builtin::kStoreFastElementIC_InBounds:
+    case Builtin::kStoreFastElementIC_NoTransitionGrowAndHandleCOW:
+    case Builtin::kStoreFastElementIC_NoTransitionIgnoreTypedArrayOOB:
     case Builtin::kStoreFastElementIC_NoTransitionHandleCOW:
-    case Builtin::kElementsTransitionAndStore_Standard:
-    case Builtin::kElementsTransitionAndStore_GrowNoTransitionHandleCOW:
-    case Builtin::kElementsTransitionAndStore_NoTransitionIgnoreOOB:
+    case Builtin::kElementsTransitionAndStore_InBounds:
+    case Builtin::kElementsTransitionAndStore_NoTransitionGrowAndHandleCOW:
+    case Builtin::kElementsTransitionAndStore_NoTransitionIgnoreTypedArrayOOB:
     case Builtin::kElementsTransitionAndStore_NoTransitionHandleCOW:
       return true;
     default:
@@ -1180,22 +1246,22 @@ bool BuiltinHasKeyedAccessStoreMode(Builtin builtin) {
 KeyedAccessStoreMode KeyedAccessStoreModeForBuiltin(Builtin builtin) {
   DCHECK(BuiltinHasKeyedAccessStoreMode(builtin));
   switch (builtin) {
-    case Builtin::kKeyedStoreIC_SloppyArguments_Standard:
-    case Builtin::kStoreFastElementIC_Standard:
-    case Builtin::kElementsTransitionAndStore_Standard:
-      return STANDARD_STORE;
-    case Builtin::kKeyedStoreIC_SloppyArguments_GrowNoTransitionHandleCOW:
-    case Builtin::kStoreFastElementIC_GrowNoTransitionHandleCOW:
-    case Builtin::kElementsTransitionAndStore_GrowNoTransitionHandleCOW:
-      return STORE_AND_GROW_HANDLE_COW;
-    case Builtin::kKeyedStoreIC_SloppyArguments_NoTransitionIgnoreOOB:
-    case Builtin::kStoreFastElementIC_NoTransitionIgnoreOOB:
-    case Builtin::kElementsTransitionAndStore_NoTransitionIgnoreOOB:
-      return STORE_IGNORE_OUT_OF_BOUNDS;
+    case Builtin::kKeyedStoreIC_SloppyArguments_InBounds:
+    case Builtin::kStoreFastElementIC_InBounds:
+    case Builtin::kElementsTransitionAndStore_InBounds:
+      return KeyedAccessStoreMode::kInBounds;
+    case Builtin::kKeyedStoreIC_SloppyArguments_NoTransitionGrowAndHandleCOW:
+    case Builtin::kStoreFastElementIC_NoTransitionGrowAndHandleCOW:
+    case Builtin::kElementsTransitionAndStore_NoTransitionGrowAndHandleCOW:
+      return KeyedAccessStoreMode::kGrowAndHandleCOW;
+    case Builtin::kKeyedStoreIC_SloppyArguments_NoTransitionIgnoreTypedArrayOOB:
+    case Builtin::kStoreFastElementIC_NoTransitionIgnoreTypedArrayOOB:
+    case Builtin::kElementsTransitionAndStore_NoTransitionIgnoreTypedArrayOOB:
+      return KeyedAccessStoreMode::kIgnoreTypedArrayOOB;
     case Builtin::kKeyedStoreIC_SloppyArguments_NoTransitionHandleCOW:
     case Builtin::kStoreFastElementIC_NoTransitionHandleCOW:
     case Builtin::kElementsTransitionAndStore_NoTransitionHandleCOW:
-      return STORE_HANDLE_COW;
+      return KeyedAccessStoreMode::kHandleCOW;
     default:
       UNREACHABLE();
   }
@@ -1207,7 +1273,7 @@ KeyedAccessStoreMode FeedbackNexus::GetKeyedAccessStoreMode() const {
   DCHECK(IsKeyedStoreICKind(kind()) || IsStoreInArrayLiteralICKind(kind()) ||
          IsDefineKeyedOwnPropertyInLiteralKind(kind()) ||
          IsDefineKeyedOwnICKind(kind()));
-  KeyedAccessStoreMode mode = STANDARD_STORE;
+  KeyedAccessStoreMode mode = KeyedAccessStoreMode::kInBounds;
 
   if (GetKeyType() == IcCheckType::kProperty) return mode;
 
@@ -1217,38 +1283,38 @@ KeyedAccessStoreMode FeedbackNexus::GetKeyedAccessStoreMode() const {
     const MaybeObjectHandle maybe_code_handler = map_and_handler.second;
     // The first handler that isn't the slow handler will have the bits we need.
     Builtin builtin_handler = Builtin::kNoBuiltinId;
-    if (maybe_code_handler.object()->IsStoreHandler()) {
+    if (IsStoreHandler(*maybe_code_handler.object())) {
       Handle<StoreHandler> data_handler =
           Handle<StoreHandler>::cast(maybe_code_handler.object());
 
-      if ((data_handler->smi_handler()).IsSmi()) {
+      if (IsSmi(data_handler->smi_handler())) {
         // Decode the KeyedAccessStoreMode information from the Handler.
         mode = StoreHandler::GetKeyedAccessStoreMode(
             MaybeObject::FromObject(data_handler->smi_handler()));
-        if (mode != STANDARD_STORE) return mode;
+        if (!StoreModeIsInBounds(mode)) return mode;
         continue;
       } else {
-        Code code = Code::cast(data_handler->smi_handler());
-        builtin_handler = code.builtin_id();
+        Tagged<Code> code = Code::cast(data_handler->smi_handler());
+        builtin_handler = code->builtin_id();
       }
 
-    } else if (maybe_code_handler.object()->IsSmi()) {
+    } else if (IsSmi(*maybe_code_handler.object())) {
       // Skip for Proxy Handlers.
       if (*maybe_code_handler.object() == StoreHandler::StoreProxy()) {
         continue;
       }
       // Decode the KeyedAccessStoreMode information from the Handler.
       mode = StoreHandler::GetKeyedAccessStoreMode(*maybe_code_handler);
-      if (mode != STANDARD_STORE) return mode;
+      if (!StoreModeIsInBounds(mode)) return mode;
       continue;
     } else if (IsDefineKeyedOwnICKind(kind())) {
       mode = StoreHandler::GetKeyedAccessStoreMode(*maybe_code_handler);
-      if (mode != STANDARD_STORE) return mode;
+      if (!StoreModeIsInBounds(mode)) return mode;
       continue;
     } else {
       // Element store without prototype chain check.
-      Code code = Code::cast(*maybe_code_handler.object());
-      builtin_handler = code.builtin_id();
+      Tagged<Code> code = Code::cast(*maybe_code_handler.object());
+      builtin_handler = code->builtin_id();
     }
 
     if (Builtins::IsBuiltinId(builtin_handler)) {
@@ -1302,8 +1368,8 @@ ForInHint FeedbackNexus::GetForInFeedback() const {
 MaybeHandle<JSObject> FeedbackNexus::GetConstructorFeedback() const {
   DCHECK_EQ(kind(), FeedbackSlotKind::kInstanceOf);
   MaybeObject feedback = GetFeedback();
-  HeapObject heap_object;
-  if (feedback->GetHeapObjectIfWeak(&heap_object)) {
+  Tagged<HeapObject> heap_object;
+  if (feedback.GetHeapObjectIfWeak(&heap_object)) {
     return config()->NewHandle(JSObject::cast(heap_object));
   }
   return MaybeHandle<JSObject>();
@@ -1323,23 +1389,23 @@ FeedbackIterator::FeedbackIterator(const FeedbackNexus* nexus)
   auto pair = nexus->GetFeedbackPair();
   MaybeObject feedback = pair.first;
   bool is_named_feedback = IsPropertyNameFeedback(feedback);
-  HeapObject heap_object;
+  Tagged<HeapObject> heap_object;
 
-  if ((feedback->GetHeapObjectIfStrong(&heap_object) &&
-       heap_object.IsWeakFixedArray()) ||
+  if ((feedback.GetHeapObjectIfStrong(&heap_object) &&
+       IsWeakFixedArray(heap_object)) ||
       is_named_feedback) {
     index_ = 0;
     state_ = kPolymorphic;
-    heap_object = feedback->GetHeapObjectAssumeStrong();
+    heap_object = feedback.GetHeapObjectAssumeStrong();
     if (is_named_feedback) {
       polymorphic_feedback_ = nexus->config()->NewHandle(
-          WeakFixedArray::cast(pair.second->GetHeapObjectAssumeStrong()));
+          WeakFixedArray::cast(pair.second.GetHeapObjectAssumeStrong()));
     } else {
       polymorphic_feedback_ =
           nexus->config()->NewHandle(WeakFixedArray::cast(heap_object));
     }
     AdvancePolymorphic();
-  } else if (feedback->GetHeapObjectIfWeak(&heap_object)) {
+  } else if (feedback.GetHeapObjectIfWeak(&heap_object)) {
     state_ = kMonomorphic;
     MaybeObject handler = pair.second;
     map_ = Map::cast(heap_object);
@@ -1365,11 +1431,11 @@ void FeedbackIterator::AdvancePolymorphic() {
   CHECK(!done_);
   CHECK_EQ(state_, kPolymorphic);
   int length = polymorphic_feedback_->length();
-  HeapObject heap_object;
+  Tagged<HeapObject> heap_object;
 
   while (index_ < length) {
-    if (polymorphic_feedback_->Get(index_)->GetHeapObjectIfWeak(&heap_object)) {
-      MaybeObject handler = polymorphic_feedback_->Get(index_ + kHandlerOffset);
+    if (polymorphic_feedback_->get(index_).GetHeapObjectIfWeak(&heap_object)) {
+      MaybeObject handler = polymorphic_feedback_->get(index_ + kHandlerOffset);
       map_ = Map::cast(heap_object);
       handler_ = handler;
       index_ += kEntrySize;
