@@ -7,12 +7,15 @@
 
 #include "src/common/globals.h"
 #include "src/heap/heap-write-barrier.h"
+#include "src/objects/dictionary.h"
 #include "src/objects/elements.h"
 #include "src/objects/embedder-data-slot-inl.h"
 #include "src/objects/feedback-vector.h"
 #include "src/objects/field-index-inl.h"
+#include "src/objects/fixed-array.h"
 #include "src/objects/hash-table-inl.h"
 #include "src/objects/heap-number-inl.h"
+#include "src/objects/instance-type-inl.h"
 #include "src/objects/js-objects.h"
 #include "src/objects/keys.h"
 #include "src/objects/lookup-inl.h"
@@ -22,6 +25,7 @@
 #include "src/objects/shared-function-info.h"
 #include "src/objects/slots.h"
 #include "src/objects/smi-inl.h"
+#include "src/objects/string.h"
 #include "src/objects/swiss-name-dictionary-inl.h"
 
 // Has to be the last include (doesn't have include guards):
@@ -51,22 +55,23 @@ NEVER_READ_ONLY_SPACE_IMPL(JSReceiver)
 
 CAST_ACCESSOR(JSIteratorResult)
 
-DEF_GETTER(JSObject, elements, FixedArrayBase) {
+DEF_GETTER(JSObject, elements, Tagged<FixedArrayBase>) {
   return TaggedField<FixedArrayBase, kElementsOffset>::load(cage_base, *this);
 }
 
-FixedArrayBase JSObject::elements(RelaxedLoadTag tag) const {
+Tagged<FixedArrayBase> JSObject::elements(RelaxedLoadTag tag) const {
   PtrComprCageBase cage_base = GetPtrComprCageBase(*this);
   return elements(cage_base, tag);
 }
 
-FixedArrayBase JSObject::elements(PtrComprCageBase cage_base,
-                                  RelaxedLoadTag) const {
+Tagged<FixedArrayBase> JSObject::elements(PtrComprCageBase cage_base,
+                                          RelaxedLoadTag) const {
   return TaggedField<FixedArrayBase, kElementsOffset>::Relaxed_Load(cage_base,
                                                                     *this);
 }
 
-void JSObject::set_elements(FixedArrayBase value, WriteBarrierMode mode) {
+void JSObject::set_elements(Tagged<FixedArrayBase> value,
+                            WriteBarrierMode mode) {
   // Note the relaxed atomic store.
   TaggedField<FixedArrayBase, kElementsOffset>::Relaxed_Store(*this, value);
   CONDITIONAL_WRITE_BARRIER(*this, kElementsOffset, value, mode);
@@ -100,7 +105,7 @@ Handle<Object> JSReceiver::GetDataProperty(Isolate* isolate,
 MaybeHandle<HeapObject> JSReceiver::GetPrototype(Isolate* isolate,
                                                  Handle<JSReceiver> receiver) {
   // We don't expect access checks to be needed on JSProxy objects.
-  DCHECK(!receiver->IsAccessCheckNeeded() || receiver->IsJSObject());
+  DCHECK(!IsAccessCheckNeeded(*receiver) || IsJSObject(*receiver));
 
   PrototypeIterator iter(isolate, receiver, kStartAtReceiver,
                          PrototypeIterator::END_AT_NON_HIDDEN);
@@ -125,34 +130,36 @@ V8_WARN_UNUSED_RESULT MaybeHandle<FixedArray> JSReceiver::OwnPropertyKeys(
                                  GetKeysConversion::kConvertToString);
 }
 
-bool JSObject::PrototypeHasNoElements(Isolate* isolate, JSObject object) {
+bool JSObject::PrototypeHasNoElements(Isolate* isolate,
+                                      Tagged<JSObject> object) {
   DisallowGarbageCollection no_gc;
-  HeapObject prototype = HeapObject::cast(object.map().prototype());
+  Tagged<HeapObject> prototype = HeapObject::cast(object->map()->prototype());
   ReadOnlyRoots roots(isolate);
-  HeapObject null = roots.null_value();
-  FixedArrayBase empty_fixed_array = roots.empty_fixed_array();
-  FixedArrayBase empty_slow_element_dictionary =
+  Tagged<HeapObject> null = roots.null_value();
+  Tagged<FixedArrayBase> empty_fixed_array = roots.empty_fixed_array();
+  Tagged<FixedArrayBase> empty_slow_element_dictionary =
       roots.empty_slow_element_dictionary();
   while (prototype != null) {
-    Map map = prototype.map();
-    if (map.IsCustomElementsReceiverMap()) return false;
-    FixedArrayBase elements = JSObject::cast(prototype).elements();
+    Tagged<Map> map = prototype->map();
+    if (IsCustomElementsReceiverMap(map)) return false;
+    Tagged<FixedArrayBase> elements = JSObject::cast(prototype)->elements();
     if (elements != empty_fixed_array &&
         elements != empty_slow_element_dictionary) {
       return false;
     }
-    prototype = HeapObject::cast(map.prototype());
+    prototype = HeapObject::cast(map->prototype());
   }
   return true;
 }
 
-ACCESSORS(JSReceiver, raw_properties_or_hash, Object, kPropertiesOrHashOffset)
-RELAXED_ACCESSORS(JSReceiver, raw_properties_or_hash, Object,
+ACCESSORS(JSReceiver, raw_properties_or_hash, Tagged<Object>,
+          kPropertiesOrHashOffset)
+RELAXED_ACCESSORS(JSReceiver, raw_properties_or_hash, Tagged<Object>,
                   kPropertiesOrHashOffset)
 
 void JSObject::EnsureCanContainHeapObjectElements(Handle<JSObject> object) {
   JSObject::ValidateElements(*object);
-  ElementsKind elements_kind = object->map().elements_kind();
+  ElementsKind elements_kind = object->map()->elements_kind();
   if (!IsObjectElementsKind(elements_kind)) {
     if (IsHoleyElementsKind(elements_kind)) {
       TransitionElementsKind(object, HOLEY_ELEMENTS);
@@ -176,14 +183,14 @@ void JSObject::EnsureCanContainElements(Handle<JSObject> object, TSlot objects,
     DCHECK(mode != ALLOW_COPIED_DOUBLE_ELEMENTS);
     bool is_holey = IsHoleyElementsKind(current_kind);
     if (current_kind == HOLEY_ELEMENTS) return;
-    Object the_hole = object->GetReadOnlyRoots().the_hole_value();
+    Tagged<Object> the_hole = object->GetReadOnlyRoots().the_hole_value();
     for (uint32_t i = 0; i < count; ++i, ++objects) {
-      Object current = *objects;
+      Tagged<Object> current = *objects;
       if (current == the_hole) {
         is_holey = true;
         target_kind = GetHoleyElementsKind(target_kind);
-      } else if (!current.IsSmi()) {
-        if (mode == ALLOW_CONVERTED_DOUBLE_ELEMENTS && current.IsNumber()) {
+      } else if (!IsSmi(current)) {
+        if (mode == ALLOW_CONVERTED_DOUBLE_ELEMENTS && IsNumber(current)) {
           if (IsSmiElementsKind(target_kind)) {
             if (is_holey) {
               target_kind = HOLEY_DOUBLE_ELEMENTS;
@@ -217,7 +224,7 @@ void JSObject::EnsureCanContainElements(Handle<JSObject> object,
       mode = DONT_ALLOW_DOUBLE_ELEMENTS;
     }
     ObjectSlot objects =
-        Handle<FixedArray>::cast(elements)->GetFirstElementAddress();
+        Handle<FixedArray>::cast(elements)->RawFieldOfFirstElement();
     EnsureCanContainElements(object, objects, length, mode);
     return;
   }
@@ -242,43 +249,43 @@ void JSObject::SetMapAndElements(Handle<JSObject> object, Handle<Map> new_map,
                                  Handle<FixedArrayBase> value) {
   Isolate* isolate = object->GetIsolate();
   JSObject::MigrateToMap(isolate, object, new_map);
-  DCHECK((object->map().has_fast_smi_or_object_elements() ||
+  DCHECK((object->map()->has_fast_smi_or_object_elements() ||
           (*value == ReadOnlyRoots(isolate).empty_fixed_array()) ||
-          object->map().has_fast_string_wrapper_elements()) ==
+          object->map()->has_fast_string_wrapper_elements()) ==
          (value->map() == ReadOnlyRoots(isolate).fixed_array_map() ||
           value->map() == ReadOnlyRoots(isolate).fixed_cow_array_map()));
   DCHECK((*value == ReadOnlyRoots(isolate).empty_fixed_array()) ||
-         (object->map().has_fast_double_elements() ==
-          value->IsFixedDoubleArray()));
+         (object->map()->has_fast_double_elements() ==
+          IsFixedDoubleArray(*value)));
   object->set_elements(*value);
 }
 
 void JSObject::initialize_elements() {
-  FixedArrayBase elements = map().GetInitialElements();
+  Tagged<FixedArrayBase> elements = map()->GetInitialElements();
   set_elements(elements, SKIP_WRITE_BARRIER);
 }
 
-DEF_GETTER(JSObject, GetIndexedInterceptor, InterceptorInfo) {
-  return map(cage_base).GetIndexedInterceptor(cage_base);
+DEF_GETTER(JSObject, GetIndexedInterceptor, Tagged<InterceptorInfo>) {
+  return map(cage_base)->GetIndexedInterceptor(cage_base);
 }
 
-DEF_GETTER(JSObject, GetNamedInterceptor, InterceptorInfo) {
-  return map(cage_base).GetNamedInterceptor(cage_base);
+DEF_GETTER(JSObject, GetNamedInterceptor, Tagged<InterceptorInfo>) {
+  return map(cage_base)->GetNamedInterceptor(cage_base);
 }
 
 // static
-int JSObject::GetHeaderSize(Map map) {
+int JSObject::GetHeaderSize(Tagged<Map> map) {
   // Check for the most common kind of JavaScript object before
   // falling into the generic switch. This speeds up the internal
   // field operations considerably on average.
-  InstanceType instance_type = map.instance_type();
+  InstanceType instance_type = map->instance_type();
   return instance_type == JS_OBJECT_TYPE
              ? JSObject::kHeaderSize
-             : GetHeaderSize(instance_type, map.has_prototype_slot());
+             : GetHeaderSize(instance_type, map->has_prototype_slot());
 }
 
 // static
-int JSObject::GetEmbedderFieldsStartOffset(Map map) {
+int JSObject::GetEmbedderFieldsStartOffset(Tagged<Map> map) {
   // Embedder fields are located after the object header.
   return GetHeaderSize(map);
 }
@@ -288,8 +295,8 @@ int JSObject::GetEmbedderFieldsStartOffset() {
 }
 
 // static
-bool JSObject::MayHaveEmbedderFields(Map map) {
-  InstanceType instance_type = map.instance_type();
+bool JSObject::MayHaveEmbedderFields(Tagged<Map> map) {
+  InstanceType instance_type = map->instance_type();
   // TODO(v8) It'd be nice if all objects with embedder data slots inherited
   // from JSObjectWithEmbedderSlots, but this is currently not possible due to
   // instance_type constraints.
@@ -302,8 +309,8 @@ bool JSObject::MayHaveEmbedderFields() const {
 }
 
 // static
-int JSObject::GetEmbedderFieldCount(Map map) {
-  int instance_size = map.instance_size();
+int JSObject::GetEmbedderFieldCount(Tagged<Map> map) {
+  int instance_size = map->instance_size();
   if (instance_size == kVariableSizeSentinel) return 0;
   // Embedder fields are located after the object header, whereas in-object
   // properties are located at the end of the object. We don't have to round up
@@ -312,7 +319,7 @@ int JSObject::GetEmbedderFieldCount(Map map) {
   // kSystemPointerSize) anyway.
   return (((instance_size - GetEmbedderFieldsStartOffset(map)) >>
            kTaggedSizeLog2) -
-          map.GetInObjectProperties()) /
+          map->GetInObjectProperties()) /
          kEmbedderDataSlotSizeInTaggedSlots;
 }
 
@@ -326,62 +333,69 @@ int JSObject::GetEmbedderFieldOffset(int index) {
   return GetEmbedderFieldsStartOffset() + (kEmbedderDataSlotSize * index);
 }
 
-Object JSObject::GetEmbedderField(int index) {
-  return EmbedderDataSlot(*this, index).load_tagged();
+Tagged<Object> JSObject::GetEmbedderField(int index) {
+  return EmbedderDataSlot(Tagged(*this), index).load_tagged();
 }
 
-void JSObject::SetEmbedderField(int index, Object value) {
-  EmbedderDataSlot::store_tagged(*this, index, value);
+void JSObject::SetEmbedderField(int index, Tagged<Object> value) {
+  EmbedderDataSlot::store_tagged(Tagged(*this), index, value);
 }
 
-void JSObject::SetEmbedderField(int index, Smi value) {
-  EmbedderDataSlot(*this, index).store_smi(value);
+void JSObject::SetEmbedderField(int index, Tagged<Smi> value) {
+  EmbedderDataSlot(Tagged(*this), index).store_smi(value);
+}
+
+// static
+bool JSObject::IsDroppableApiObject(const Tagged<Map> map) {
+  auto instance_type = map->instance_type();
+  return InstanceTypeChecker::IsJSApiObject(instance_type) ||
+         instance_type == JS_SPECIAL_API_OBJECT_TYPE;
 }
 
 bool JSObject::IsDroppableApiObject() const {
-  auto instance_type = map().instance_type();
-  return InstanceTypeChecker::IsJSApiObject(instance_type) ||
-         instance_type == JS_SPECIAL_API_OBJECT_TYPE;
+  return IsDroppableApiObject(map());
 }
 
 // Access fast-case object properties at index. The use of these routines
 // is needed to correctly distinguish between properties stored in-object and
 // properties stored in the properties array.
-Object JSObject::RawFastPropertyAt(FieldIndex index) const {
+Tagged<Object> JSObject::RawFastPropertyAt(FieldIndex index) const {
   PtrComprCageBase cage_base = GetPtrComprCageBase(*this);
   return RawFastPropertyAt(cage_base, index);
 }
 
-Object JSObject::RawFastPropertyAt(PtrComprCageBase cage_base,
-                                   FieldIndex index) const {
+Tagged<Object> JSObject::RawFastPropertyAt(PtrComprCageBase cage_base,
+                                           FieldIndex index) const {
   if (index.is_inobject()) {
     return TaggedField<Object>::Relaxed_Load(cage_base, *this, index.offset());
   } else {
-    return property_array(cage_base).get(cage_base,
-                                         index.outobject_array_index());
+    return property_array(cage_base)->get(cage_base,
+                                          index.outobject_array_index());
   }
 }
 
 // The SeqCst versions of RawFastPropertyAt are used for atomically accessing
 // shared struct fields.
-Object JSObject::RawFastPropertyAt(FieldIndex index,
-                                   SeqCstAccessTag tag) const {
+Tagged<Object> JSObject::RawFastPropertyAt(FieldIndex index,
+                                           SeqCstAccessTag tag) const {
   PtrComprCageBase cage_base = GetPtrComprCageBase(*this);
   return RawFastPropertyAt(cage_base, index, tag);
 }
 
-Object JSObject::RawFastPropertyAt(PtrComprCageBase cage_base, FieldIndex index,
-                                   SeqCstAccessTag tag) const {
+Tagged<Object> JSObject::RawFastPropertyAt(PtrComprCageBase cage_base,
+                                           FieldIndex index,
+                                           SeqCstAccessTag tag) const {
   if (index.is_inobject()) {
     return TaggedField<Object>::SeqCst_Load(cage_base, *this, index.offset());
   } else {
-    return property_array(cage_base).get(cage_base,
-                                         index.outobject_array_index(), tag);
+    return property_array(cage_base)->get(cage_base,
+                                          index.outobject_array_index(), tag);
   }
 }
 
-base::Optional<Object> JSObject::RawInobjectPropertyAt(
-    PtrComprCageBase cage_base, Map original_map, FieldIndex index) const {
+base::Optional<Tagged<Object>> JSObject::RawInobjectPropertyAt(
+    PtrComprCageBase cage_base, Tagged<Map> original_map,
+    FieldIndex index) const {
   CHECK(index.is_inobject());
 
   // This method implements a "snapshot" protocol to protect against reading out
@@ -409,13 +423,14 @@ base::Optional<Object> JSObject::RawInobjectPropertyAt(
   // Only if the maps match can the property be inspected. It may have a "wrong"
   // value, but it will be within the bounds of the objects instance size as
   // given by the map and it will be a valid Smi or object pointer.
-  Object maybe_tagged_object =
+  Tagged<Object> maybe_tagged_object =
       TaggedField<Object>::Acquire_Load(cage_base, *this, index.offset());
   if (original_map != map(cage_base, kAcquireLoad)) return {};
   return maybe_tagged_object;
 }
 
-void JSObject::RawFastInobjectPropertyAtPut(FieldIndex index, Object value,
+void JSObject::RawFastInobjectPropertyAtPut(FieldIndex index,
+                                            Tagged<Object> value,
                                             WriteBarrierMode mode) {
   DCHECK(index.is_inobject());
   int offset = index.offset();
@@ -423,35 +438,36 @@ void JSObject::RawFastInobjectPropertyAtPut(FieldIndex index, Object value,
   CONDITIONAL_WRITE_BARRIER(*this, offset, value, mode);
 }
 
-void JSObject::RawFastInobjectPropertyAtPut(FieldIndex index, Object value,
+void JSObject::RawFastInobjectPropertyAtPut(FieldIndex index,
+                                            Tagged<Object> value,
                                             SeqCstAccessTag tag) {
   DCHECK(index.is_inobject());
-  DCHECK(value.IsShared());
+  DCHECK(IsShared(value));
   SEQ_CST_WRITE_FIELD(*this, index.offset(), value);
   CONDITIONAL_WRITE_BARRIER(*this, index.offset(), value, UPDATE_WRITE_BARRIER);
 }
 
-void JSObject::FastPropertyAtPut(FieldIndex index, Object value,
+void JSObject::FastPropertyAtPut(FieldIndex index, Tagged<Object> value,
                                  WriteBarrierMode mode) {
   if (index.is_inobject()) {
     RawFastInobjectPropertyAtPut(index, value, mode);
   } else {
     DCHECK_EQ(UPDATE_WRITE_BARRIER, mode);
-    property_array().set(index.outobject_array_index(), value);
+    property_array()->set(index.outobject_array_index(), value);
   }
 }
 
-void JSObject::FastPropertyAtPut(FieldIndex index, Object value,
+void JSObject::FastPropertyAtPut(FieldIndex index, Tagged<Object> value,
                                  SeqCstAccessTag tag) {
   if (index.is_inobject()) {
     RawFastInobjectPropertyAtPut(index, value, tag);
   } else {
-    property_array().set(index.outobject_array_index(), value, tag);
+    property_array()->set(index.outobject_array_index(), value, tag);
   }
 }
 
 void JSObject::WriteToField(InternalIndex descriptor, PropertyDetails details,
-                            Object value) {
+                            Tagged<Object> value) {
   DCHECK_EQ(PropertyLocation::kField, details.location());
   DCHECK_EQ(PropertyKind::kData, details.kind());
   DisallowGarbageCollection no_gc;
@@ -463,49 +479,76 @@ void JSObject::WriteToField(InternalIndex descriptor, PropertyDetails details,
     // to return values and stores to the stack silently clear the signalling
     // bit).
     uint64_t bits;
-    if (value.IsSmi()) {
+    if (IsSmi(value)) {
       bits = base::bit_cast<uint64_t>(static_cast<double>(Smi::ToInt(value)));
-    } else if (value.IsUninitialized()) {
+    } else if (IsUninitialized(value)) {
       bits = kHoleNanInt64;
     } else {
-      DCHECK(value.IsHeapNumber());
-      bits = HeapNumber::cast(value).value_as_bits(kRelaxedLoad);
+      DCHECK(IsHeapNumber(value));
+      bits = HeapNumber::cast(value)->value_as_bits();
     }
     auto box = HeapNumber::cast(RawFastPropertyAt(index));
-    box.set_value_as_bits(bits, kRelaxedStore);
+    box->set_value_as_bits(bits);
   } else {
     FastPropertyAtPut(index, value);
   }
 }
 
-Object JSObject::RawFastPropertyAtSwap(FieldIndex index, Object value,
-                                       SeqCstAccessTag tag) {
-  PtrComprCageBase cage_base = GetPtrComprCageBase(*this);
-  return RawFastPropertyAtSwap(cage_base, index, value, tag);
+Tagged<Object> JSObject::RawFastInobjectPropertyAtSwap(FieldIndex index,
+                                                       Tagged<Object> value,
+                                                       SeqCstAccessTag tag) {
+  DCHECK(index.is_inobject());
+  DCHECK(IsShared(value));
+  int offset = index.offset();
+  Tagged<Object> old_value = SEQ_CST_SWAP_FIELD(*this, offset, value);
+  CONDITIONAL_WRITE_BARRIER(*this, offset, value, UPDATE_WRITE_BARRIER);
+  return old_value;
 }
 
-Object JSObject::RawFastPropertyAtSwap(PtrComprCageBase cage_base,
-                                       FieldIndex index, Object value,
-                                       SeqCstAccessTag tag) {
+Tagged<Object> JSObject::RawFastPropertyAtSwap(FieldIndex index,
+                                               Tagged<Object> value,
+                                               SeqCstAccessTag tag) {
   if (index.is_inobject()) {
-    return TaggedField<Object>::SeqCst_Swap(cage_base, *this, index.offset(),
-                                            value);
+    return RawFastInobjectPropertyAtSwap(index, value, tag);
   }
-  return property_array().Swap(cage_base, index.outobject_array_index(), value,
-                               tag);
+  return property_array()->Swap(index.outobject_array_index(), value, tag);
+}
+
+Tagged<Object> JSObject::RawFastInobjectPropertyAtCompareAndSwap(
+    FieldIndex index, Tagged<Object> expected, Tagged<Object> value,
+    SeqCstAccessTag tag) {
+  DCHECK(index.is_inobject());
+  DCHECK(IsShared(value));
+  Tagged<Object> previous_value =
+      SEQ_CST_COMPARE_AND_SWAP_FIELD(*this, index.offset(), expected, value);
+  if (previous_value == expected) {
+    CONDITIONAL_WRITE_BARRIER(*this, index.offset(), value,
+                              UPDATE_WRITE_BARRIER);
+  }
+  return previous_value;
+}
+
+Tagged<Object> JSObject::RawFastPropertyAtCompareAndSwapInternal(
+    FieldIndex index, Tagged<Object> expected, Tagged<Object> value,
+    SeqCstAccessTag tag) {
+  if (index.is_inobject()) {
+    return RawFastInobjectPropertyAtCompareAndSwap(index, expected, value, tag);
+  }
+  return property_array()->CompareAndSwap(index.outobject_array_index(),
+                                          expected, value, tag);
 }
 
 int JSObject::GetInObjectPropertyOffset(int index) {
-  return map().GetInObjectPropertyOffset(index);
+  return map()->GetInObjectPropertyOffset(index);
 }
 
-Object JSObject::InObjectPropertyAt(int index) {
+Tagged<Object> JSObject::InObjectPropertyAt(int index) {
   int offset = GetInObjectPropertyOffset(index);
   return TaggedField<Object>::load(*this, offset);
 }
 
-Object JSObject::InObjectPropertyAtPut(int index, Object value,
-                                       WriteBarrierMode mode) {
+Tagged<Object> JSObject::InObjectPropertyAtPut(int index, Tagged<Object> value,
+                                               WriteBarrierMode mode) {
   // Adjust for the number of properties stored in the object.
   int offset = GetInObjectPropertyOffset(index);
   WRITE_FIELD(*this, offset, value);
@@ -513,10 +556,11 @@ Object JSObject::InObjectPropertyAtPut(int index, Object value,
   return value;
 }
 
-void JSObject::InitializeBody(Map map, int start_offset,
+void JSObject::InitializeBody(Tagged<Map> map, int start_offset,
                               bool is_slack_tracking_in_progress,
-                              MapWord filler_map, Object undefined_filler) {
-  int size = map.instance_size();
+                              MapWord filler_map,
+                              Tagged<Object> undefined_filler) {
+  int size = map->instance_size();
   int offset = start_offset;
 
   // embedder data slots need to be initialized separately
@@ -535,7 +579,7 @@ void JSObject::InitializeBody(Map map, int start_offset,
     DCHECK_EQ(offset, embedder_field_start);
     for (int i = 0; i < embedder_field_count; i++) {
       // TODO(v8): consider initializing embedded data slots with Smi::zero().
-      EmbedderDataSlot(*this, i).Initialize(undefined_filler);
+      EmbedderDataSlot(Tagged<JSObject>(*this), i).Initialize(undefined_filler);
       offset += kEmbedderDataSlotSize;
     }
   } else {
@@ -545,7 +589,7 @@ void JSObject::InitializeBody(Map map, int start_offset,
   DCHECK_LE(offset, size);
   if (is_slack_tracking_in_progress) {
     int end_of_pre_allocated_offset =
-        size - (map.UnusedPropertyFields() * kTaggedSize);
+        size - (map->UnusedPropertyFields() * kTaggedSize);
     DCHECK_LE(kHeaderSize, end_of_pre_allocated_offset);
     DCHECK_LE(offset, end_of_pre_allocated_offset);
     // fill pre allocated slots with references to the undefined value object
@@ -555,7 +599,7 @@ void JSObject::InitializeBody(Map map, int start_offset,
     }
     // fill the remainder with one word filler objects (ie just a map word)
     while (offset < size) {
-      Object fm = Object(filler_map.ptr());
+      Tagged<Object> fm = Tagged<Object>(filler_map.ptr());
       WRITE_FIELD(*this, offset, fm);
       offset += kTaggedSize;
     }
@@ -573,11 +617,6 @@ TQ_OBJECT_CONSTRUCTORS_IMPL(JSExternalObject)
 EXTERNAL_POINTER_ACCESSORS(JSExternalObject, value, void*, kValueOffset,
                            kExternalObjectValueTag)
 
-DEF_GETTER(JSGlobalObject, native_context_unchecked, Object) {
-  return TaggedField<Object, kNativeContextOffset>::Relaxed_Load(cage_base,
-                                                                 *this);
-}
-
 bool JSMessageObject::DidEnsureSourcePositionsAvailable() const {
   return shared_info() == Smi::zero();
 }
@@ -586,7 +625,7 @@ bool JSMessageObject::DidEnsureSourcePositionsAvailable() const {
 void JSMessageObject::EnsureSourcePositionsAvailable(
     Isolate* isolate, Handle<JSMessageObject> message) {
   if (message->DidEnsureSourcePositionsAvailable()) {
-    DCHECK(message->script().has_line_ends());
+    DCHECK(message->script()->has_line_ends());
   } else {
     JSMessageObject::InitializeSourcePositions(isolate, message);
   }
@@ -612,38 +651,38 @@ void JSMessageObject::set_type(MessageTemplate value) {
   set_raw_type(static_cast<int>(value));
 }
 
-ACCESSORS(JSMessageObject, shared_info, Object, kSharedInfoOffset)
-ACCESSORS(JSMessageObject, bytecode_offset, Smi, kBytecodeOffsetOffset)
+ACCESSORS(JSMessageObject, shared_info, Tagged<Object>, kSharedInfoOffset)
+ACCESSORS(JSMessageObject, bytecode_offset, Tagged<Smi>, kBytecodeOffsetOffset)
 SMI_ACCESSORS(JSMessageObject, start_position, kStartPositionOffset)
 SMI_ACCESSORS(JSMessageObject, end_position, kEndPositionOffset)
 SMI_ACCESSORS(JSMessageObject, error_level, kErrorLevelOffset)
 SMI_ACCESSORS(JSMessageObject, raw_type, kMessageTypeOffset)
 
 DEF_GETTER(JSObject, GetElementsKind, ElementsKind) {
-  ElementsKind kind = map(cage_base).elements_kind();
+  ElementsKind kind = map(cage_base)->elements_kind();
 #if VERIFY_HEAP && DEBUG
-  FixedArrayBase fixed_array = FixedArrayBase::unchecked_cast(
+  Tagged<FixedArrayBase> fixed_array = FixedArrayBase::unchecked_cast(
       TaggedField<HeapObject, kElementsOffset>::load(cage_base, *this));
 
   // If a GC was caused while constructing this object, the elements
   // pointer may point to a one pointer filler map.
   if (ElementsAreSafeToExamine(cage_base)) {
-    Map map = fixed_array.map(cage_base);
+    Tagged<Map> map = fixed_array->map(cage_base);
     if (IsSmiOrObjectElementsKind(kind)) {
       DCHECK(map == GetReadOnlyRoots(cage_base).fixed_array_map() ||
              map == GetReadOnlyRoots(cage_base).fixed_cow_array_map());
     } else if (IsDoubleElementsKind(kind)) {
-      DCHECK(fixed_array.IsFixedDoubleArray(cage_base) ||
+      DCHECK(IsFixedDoubleArray(fixed_array, cage_base) ||
              fixed_array == GetReadOnlyRoots(cage_base).empty_fixed_array());
     } else if (kind == DICTIONARY_ELEMENTS) {
-      DCHECK(fixed_array.IsFixedArray(cage_base));
-      DCHECK(fixed_array.IsNumberDictionary(cage_base));
+      DCHECK(IsFixedArray(fixed_array, cage_base));
+      DCHECK(IsNumberDictionary(fixed_array, cage_base));
     } else {
       DCHECK(kind > DICTIONARY_ELEMENTS ||
              IsAnyNonextensibleElementsKind(kind));
     }
     DCHECK(!IsSloppyArgumentsElementsKind(kind) ||
-           elements(cage_base).IsSloppyArgumentsElements());
+           IsSloppyArgumentsElements(elements(cage_base)));
   }
 #endif
   return kind;
@@ -731,12 +770,12 @@ DEF_GETTER(JSObject, HasSlowStringWrapperElements, bool) {
 
 DEF_GETTER(JSObject, HasTypedArrayOrRabGsabTypedArrayElements, bool) {
   DCHECK(!elements(cage_base).is_null());
-  return map(cage_base).has_typed_array_or_rab_gsab_typed_array_elements();
+  return map(cage_base)->has_typed_array_or_rab_gsab_typed_array_elements();
 }
 
-#define FIXED_TYPED_ELEMENTS_CHECK(Type, type, TYPE, ctype)   \
-  DEF_GETTER(JSObject, HasFixed##Type##Elements, bool) {      \
-    return map(cage_base).elements_kind() == TYPE##_ELEMENTS; \
+#define FIXED_TYPED_ELEMENTS_CHECK(Type, type, TYPE, ctype)    \
+  DEF_GETTER(JSObject, HasFixed##Type##Elements, bool) {       \
+    return map(cage_base)->elements_kind() == TYPE##_ELEMENTS; \
   }
 
 TYPED_ARRAYS(FIXED_TYPED_ELEMENTS_CHECK)
@@ -744,18 +783,19 @@ TYPED_ARRAYS(FIXED_TYPED_ELEMENTS_CHECK)
 #undef FIXED_TYPED_ELEMENTS_CHECK
 
 DEF_GETTER(JSObject, HasNamedInterceptor, bool) {
-  return map(cage_base).has_named_interceptor();
+  return map(cage_base)->has_named_interceptor();
 }
 
 DEF_GETTER(JSObject, HasIndexedInterceptor, bool) {
-  return map(cage_base).has_indexed_interceptor();
+  return map(cage_base)->has_indexed_interceptor();
 }
 
 RELEASE_ACQUIRE_ACCESSORS_CHECKED2(JSGlobalObject, global_dictionary,
-                                   GlobalDictionary, kPropertiesOrHashOffset,
+                                   Tagged<GlobalDictionary>,
+                                   kPropertiesOrHashOffset,
                                    !HasFastProperties(cage_base), true)
 
-DEF_GETTER(JSObject, element_dictionary, NumberDictionary) {
+DEF_GETTER(JSObject, element_dictionary, Tagged<NumberDictionary>) {
   DCHECK(HasDictionaryElements(cage_base) ||
          HasSlowStringWrapperElements(cage_base));
   return NumberDictionary::cast(elements(cage_base));
@@ -766,7 +806,7 @@ void JSReceiver::initialize_properties(Isolate* isolate) {
   DCHECK(!ObjectInYoungGeneration(roots.empty_fixed_array()));
   DCHECK(!ObjectInYoungGeneration(roots.empty_property_dictionary()));
   DCHECK(!ObjectInYoungGeneration(roots.empty_ordered_property_dictionary()));
-  if (map(isolate).is_dictionary_map()) {
+  if (map(isolate)->is_dictionary_map()) {
     if (V8_ENABLE_SWISS_NAME_DICTIONARY_BOOL) {
       WRITE_FIELD(*this, kPropertiesOrHashOffset,
                   roots.empty_swiss_property_dictionary());
@@ -780,36 +820,36 @@ void JSReceiver::initialize_properties(Isolate* isolate) {
 }
 
 DEF_GETTER(JSReceiver, HasFastProperties, bool) {
-  Object raw_properties_or_hash_obj =
+  Tagged<Object> raw_properties_or_hash_obj =
       raw_properties_or_hash(cage_base, kRelaxedLoad);
-  DCHECK(raw_properties_or_hash_obj.IsSmi() ||
-         ((raw_properties_or_hash_obj.IsGlobalDictionary(cage_base) ||
-           raw_properties_or_hash_obj.IsNameDictionary(cage_base) ||
-           raw_properties_or_hash_obj.IsSwissNameDictionary(cage_base)) ==
-          map(cage_base).is_dictionary_map()));
+  DCHECK(IsSmi(raw_properties_or_hash_obj) ||
+         ((IsGlobalDictionary(raw_properties_or_hash_obj, cage_base) ||
+           IsNameDictionary(raw_properties_or_hash_obj, cage_base) ||
+           IsSwissNameDictionary(raw_properties_or_hash_obj, cage_base)) ==
+          map(cage_base)->is_dictionary_map()));
   USE(raw_properties_or_hash_obj);
-  return !map(cage_base).is_dictionary_map();
+  return !map(cage_base)->is_dictionary_map();
 }
 
-DEF_GETTER(JSReceiver, property_dictionary, NameDictionary) {
-  DCHECK(!IsJSGlobalObject(cage_base));
+DEF_GETTER(JSReceiver, property_dictionary, Tagged<NameDictionary>) {
+  DCHECK(!IsJSGlobalObject(*this, cage_base));
   DCHECK(!HasFastProperties(cage_base));
   DCHECK(!V8_ENABLE_SWISS_NAME_DICTIONARY_BOOL);
 
-  Object prop = raw_properties_or_hash(cage_base);
-  if (prop.IsSmi()) {
+  Tagged<Object> prop = raw_properties_or_hash(cage_base);
+  if (IsSmi(prop)) {
     return GetReadOnlyRoots(cage_base).empty_property_dictionary();
   }
   return NameDictionary::cast(prop);
 }
 
-DEF_GETTER(JSReceiver, property_dictionary_swiss, SwissNameDictionary) {
-  DCHECK(!IsJSGlobalObject(cage_base));
+DEF_GETTER(JSReceiver, property_dictionary_swiss, Tagged<SwissNameDictionary>) {
+  DCHECK(!IsJSGlobalObject(*this, cage_base));
   DCHECK(!HasFastProperties(cage_base));
   DCHECK(V8_ENABLE_SWISS_NAME_DICTIONARY_BOOL);
 
-  Object prop = raw_properties_or_hash(cage_base);
-  if (prop.IsSmi()) {
+  Tagged<Object> prop = raw_properties_or_hash(cage_base);
+  if (IsSmi(prop)) {
     return GetReadOnlyRoots(cage_base).empty_swiss_property_dictionary();
   }
   return SwissNameDictionary::cast(prop);
@@ -817,15 +857,32 @@ DEF_GETTER(JSReceiver, property_dictionary_swiss, SwissNameDictionary) {
 
 // TODO(gsathya): Pass isolate directly to this function and access
 // the heap from this.
-DEF_GETTER(JSReceiver, property_array, PropertyArray) {
+DEF_GETTER(JSReceiver, property_array, Tagged<PropertyArray>) {
   DCHECK(HasFastProperties(cage_base));
-  Object prop = raw_properties_or_hash(cage_base);
-  if (prop.IsSmi() || prop == GetReadOnlyRoots(cage_base).empty_fixed_array()) {
+  Tagged<Object> prop = raw_properties_or_hash(cage_base);
+  if (IsSmi(prop) || prop == GetReadOnlyRoots(cage_base).empty_fixed_array()) {
     return GetReadOnlyRoots(cage_base).empty_property_array();
   }
   return PropertyArray::cast(prop);
 }
 
+base::Optional<Tagged<NativeContext>> JSReceiver::GetCreationContext() {
+  DisallowGarbageCollection no_gc;
+  Tagged<Map> meta_map = map()->map();
+  DCHECK(IsMapMap(meta_map));
+  Tagged<Object> maybe_native_context = meta_map->native_context_or_null();
+  if (IsNull(maybe_native_context)) return {};
+  DCHECK(IsNativeContext(maybe_native_context));
+  return NativeContext::cast(maybe_native_context);
+}
+
+MaybeHandle<NativeContext> JSReceiver::GetCreationContext(Isolate* isolate) {
+  DisallowGarbageCollection no_gc;
+  base::Optional<Tagged<NativeContext>> maybe_context = GetCreationContext();
+  if (!maybe_context.has_value()) return {};
+  return handle(maybe_context.value(), isolate);
+}
+
 Maybe<bool> JSReceiver::HasProperty(Isolate* isolate, Handle<JSReceiver> object,
                                     Handle<Name> name) {
   PropertyKey key(isolate, name);
@@ -836,7 +893,7 @@ Maybe<bool> JSReceiver::HasProperty(Isolate* isolate, Handle<JSReceiver> object,
 Maybe<bool> JSReceiver::HasOwnProperty(Isolate* isolate,
                                        Handle<JSReceiver> object,
                                        uint32_t index) {
-  if (object->IsJSObject()) {  // Shortcut.
+  if (IsJSObject(*object)) {  // Shortcut.
     LookupIterator it(isolate, object, index, object, LookupIterator::OWN);
     return HasProperty(&it);
   }
@@ -890,12 +947,16 @@ Maybe<PropertyAttributes> JSReceiver::GetOwnElementAttributes(
   return GetPropertyAttributes(&it);
 }
 
-bool JSGlobalObject::IsDetached() {
-  return global_proxy().IsDetachedFrom(*this);
+Tagged<NativeContext> JSGlobalObject::native_context() {
+  return *GetCreationContext();
 }
 
-bool JSGlobalProxy::IsDetachedFrom(JSGlobalObject global) const {
-  const PrototypeIterator iter(this->GetIsolate(), *this);
+bool JSGlobalObject::IsDetached() {
+  return global_proxy()->IsDetachedFrom(*this);
+}
+
+bool JSGlobalProxy::IsDetachedFrom(Tagged<JSGlobalObject> global) const {
+  const PrototypeIterator iter(this->GetIsolate(), Tagged<JSReceiver>(*this));
   return iter.GetCurrent() != global;
 }
 
@@ -904,8 +965,8 @@ inline int JSGlobalProxy::SizeWithEmbedderFields(int embedder_field_count) {
   return kHeaderSize + embedder_field_count * kEmbedderDataSlotSize;
 }
 
-ACCESSORS(JSIteratorResult, value, Object, kValueOffset)
-ACCESSORS(JSIteratorResult, done, Object, kDoneOffset)
+ACCESSORS(JSIteratorResult, value, Tagged<Object>, kValueOffset)
+ACCESSORS(JSIteratorResult, done, Tagged<Object>, kDoneOffset)
 
 // If the fast-case backing storage takes up much more memory than a dictionary
 // backing storage would, the object should have slow elements.
@@ -918,7 +979,7 @@ static inline bool ShouldConvertToSlowElements(uint32_t used_elements,
   return size_threshold <= new_capacity;
 }
 
-static inline bool ShouldConvertToSlowElements(JSObject object,
+static inline bool ShouldConvertToSlowElements(Tagged<JSObject> object,
                                                uint32_t capacity,
                                                uint32_t index,
                                                uint32_t* new_capacity) {
@@ -936,7 +997,7 @@ static inline bool ShouldConvertToSlowElements(JSObject object,
        ObjectInYoungGeneration(object))) {
     return false;
   }
-  return ShouldConvertToSlowElements(object.GetFastElementsUsage(),
+  return ShouldConvertToSlowElements(object->GetFastElementsUsage(),
                                      *new_capacity);
 }
 
