@@ -16,6 +16,52 @@
 
 #include "src/base/memory.h"
 
+// V8 objects are defined as:
+//
+//     V8_OBJECT class Foo : public Base {
+//       ...
+//     } V8_OBJECT_END;
+//
+// These macros are to enable packing down to 4-byte alignment (i.e. int32
+// alignment, since we have int32 fields), and to add warnings which ensure that
+// there is no unwanted within-object padding.
+#if V8_CC_GNU
+
+#define V8_OBJECT_PUSH                                                    \
+  _Pragma("pack(push)") _Pragma("pack(4)") _Pragma("GCC diagnostic push") \
+      _Pragma("GCC diagnostic error \"-Wpadded\"")
+#define V8_OBJECT_POP _Pragma("pack(pop)") _Pragma("GCC diagnostic pop")
+
+#define V8_OBJECT V8_OBJECT_PUSH
+
+// GCC wants this pragma to be a new statement, but we prefer to have
+// V8_OBJECT_END look like part of the definition. Insert a semicolon before the
+// pragma to make gcc happy, and use static_assert(true) to swallow the next
+// semicolon.
+#define V8_OBJECT_END \
+  ;                   \
+  V8_OBJECT_POP static_assert(true)
+
+#define V8_OBJECT_INNER_CLASS V8_OBJECT_POP
+#define V8_OBJECT_INNER_CLASS_END \
+  ;                               \
+  V8_OBJECT_PUSH static_assert(true)
+
+#elif V8_CC_MSVC
+#define V8_OBJECT_PUSH                                           \
+  __pragma(pack(push)) __pragma(pack(4)) __pragma(warning(push)) \
+      __pragma(warning(default : 4820))
+#define V8_OBJECT_POP __pragma(pack(pop)) __pragma(warning(pop))
+
+#define V8_OBJECT V8_OBJECT_PUSH
+#define V8_OBJECT_END V8_OBJECT_POP
+
+#define V8_OBJECT_INNER_CLASS V8_OBJECT_POP
+#define V8_OBJECT_INNER_CLASS_END V8_OBJECT_PUSH
+#else
+#error Unsupported compiler
+#endif
+
 // Since this changes visibility, it should always be last in a class
 // definition.
 #define OBJECT_CONSTRUCTORS(Type, ...)                                         \
@@ -28,19 +74,19 @@
   const Type* operator->() const { return this; }                              \
                                                                                \
  protected:                                                                    \
-  template <typename TFieldType, int kFieldOffset, typename CompressionScheme> \
-  friend class TaggedField;                                                    \
   friend class Tagged<Type>;                                                   \
                                                                                \
   /* Special constructor for constexpr construction which allows skipping type \
    * checks. */                                                                \
-  explicit constexpr inline Type(Address ptr, Object::SkipTypeCheckTag)        \
-      : __VA_ARGS__(ptr, Object::SkipTypeCheckTag()) {}                        \
+  explicit constexpr V8_INLINE Type(Address ptr, HeapObject::SkipTypeCheckTag) \
+      : __VA_ARGS__(ptr, HeapObject::SkipTypeCheckTag()) {}                    \
                                                                                \
+  inline void CheckTypeOnCast();                                               \
   explicit inline Type(Address ptr)
 
-#define OBJECT_CONSTRUCTORS_IMPL(Type, Super) \
-  inline Type::Type(Address ptr) : Super(ptr) { SLOW_DCHECK(Is##Type(*this)); }
+#define OBJECT_CONSTRUCTORS_IMPL(Type, Super)                           \
+  inline void Type::CheckTypeOnCast() { SLOW_DCHECK(Is##Type(*this)); } \
+  inline Type::Type(Address ptr) : Super(ptr) { CheckTypeOnCast(); }
 
 #define NEVER_READ_ONLY_SPACE   \
   inline Heap* GetHeap() const; \
@@ -91,9 +137,6 @@
 #define DECL_RELAXED_UINT16_ACCESSORS(name) \
   DECL_RELAXED_PRIMITIVE_ACCESSORS(name, uint16_t)
 
-// TODO(ishell): eventually isolate-less getters should not be used anymore.
-// For full pointer-mode the C++ compiler should optimize away unused isolate
-// parameter.
 #define DECL_GETTER(name, type) \
   inline type name() const;     \
   inline type name(PtrComprCageBase cage_base) const;
@@ -244,6 +287,20 @@
 #define ACCESSORS(holder, name, type, offset) \
   ACCESSORS_CHECKED(holder, name, type, offset, true)
 
+// TODO(jgruber): Eventually, all accessors should be ported to the NOCAGE
+// variant (which doesn't define a PtrComprCageBase overload). Once that's
+// done, remove the cage-ful macros (e.g. ACCESSORS) and rename the cage-less
+// macros (e.g. ACCESSORS_NOCAGE).
+#define ACCESSORS_NOCAGE(holder, name, type, offset)           \
+  type holder::name() const {                                  \
+    PtrComprCageBase cage_base = GetPtrComprCageBase(*this);   \
+    return TaggedField<type, offset>::load(cage_base, *this);  \
+  }                                                            \
+  void holder::set_##name(type value, WriteBarrierMode mode) { \
+    TaggedField<type, offset>::store(*this, value);            \
+    CONDITIONAL_WRITE_BARRIER(*this, offset, value, mode);     \
+  }
+
 #define RENAME_TORQUE_ACCESSORS(holder, name, torque_name, type)      \
   inline type holder::name() const {                                  \
     return TorqueGeneratedClass::torque_name();                       \
@@ -382,7 +439,7 @@
 #define SMI_ACCESSORS_CHECKED(holder, name, offset, condition)   \
   int holder::name() const {                                     \
     DCHECK(condition);                                           \
-    Smi value = TaggedField<Smi, offset>::load(*this);           \
+    Tagged<Smi> value = TaggedField<Smi, offset>::load(*this);   \
     return value.value();                                        \
   }                                                              \
   void holder::set_##name(int value) {                           \
@@ -399,20 +456,20 @@
 
 #define RELEASE_ACQUIRE_SMI_ACCESSORS(holder, name, offset)              \
   int holder::name(AcquireLoadTag) const {                               \
-    Smi value = TaggedField<Smi, offset>::Acquire_Load(*this);           \
+    Tagged<Smi> value = TaggedField<Smi, offset>::Acquire_Load(*this);   \
     return value.value();                                                \
   }                                                                      \
   void holder::set_##name(int value, ReleaseStoreTag) {                  \
     TaggedField<Smi, offset>::Release_Store(*this, Smi::FromInt(value)); \
   }
 
-#define DECL_RELAXED_SMI_ACCESSORS(name) \
+#define DECL_RELAXED_INT_ACCESSORS(name) \
   inline int name(RelaxedLoadTag) const; \
   inline void set_##name(int value, RelaxedStoreTag);
 
 #define RELAXED_SMI_ACCESSORS(holder, name, offset)                      \
   int holder::name(RelaxedLoadTag) const {                               \
-    Smi value = TaggedField<Smi, offset>::Relaxed_Load(*this);           \
+    Tagged<Smi> value = TaggedField<Smi, offset>::Relaxed_Load(*this);   \
     return value.value();                                                \
   }                                                                      \
   void holder::set_##name(int value, RelaxedStoreTag) {                  \
@@ -442,23 +499,25 @@
   }
 
 // Host objects in ReadOnlySpace can't define the isolate-less accessor.
-#define DECL_EXTERNAL_POINTER_ACCESSORS_MAYBE_READ_ONLY_HOST(name, type)  \
-  inline type name(i::Isolate* isolate_for_sandbox) const;                \
-  inline void init_##name(i::Isolate* isolate, const type initial_value); \
-  inline void set_##name(i::Isolate* isolate, const type value);
+#define DECL_EXTERNAL_POINTER_ACCESSORS_MAYBE_READ_ONLY_HOST(name, type) \
+  inline type name(i::IsolateForSandbox isolate) const;                  \
+  inline void init_##name(i::IsolateForSandbox isolate,                  \
+                          const type initial_value);                     \
+  inline void set_##name(i::IsolateForSandbox isolate, const type value);
 
 // Host objects in ReadOnlySpace can't define the isolate-less accessor.
 #define EXTERNAL_POINTER_ACCESSORS_MAYBE_READ_ONLY_HOST(holder, name, type, \
                                                         offset, tag)        \
-  type holder::name(i::Isolate* isolate_for_sandbox) const {                \
+  type holder::name(i::IsolateForSandbox isolate) const {                   \
     /* This is a workaround for MSVC error C2440 not allowing  */           \
     /* reinterpret casts to the same type. */                               \
     struct C2440 {};                                                        \
-    Address result = HeapObject::ReadExternalPointerField<tag>(             \
-        offset, isolate_for_sandbox);                                       \
+    Address result =                                                        \
+        HeapObject::ReadExternalPointerField<tag>(offset, isolate);         \
     return reinterpret_cast<type>(reinterpret_cast<C2440*>(result));        \
   }                                                                         \
-  void holder::init_##name(i::Isolate* isolate, const type initial_value) { \
+  void holder::init_##name(i::IsolateForSandbox isolate,                    \
+                           const type initial_value) {                      \
     /* This is a workaround for MSVC error C2440 not allowing  */           \
     /* reinterpret casts to the same type. */                               \
     struct C2440 {};                                                        \
@@ -466,7 +525,7 @@
         reinterpret_cast<const C2440*>(initial_value));                     \
     HeapObject::InitExternalPointerField<tag>(offset, isolate, the_value);  \
   }                                                                         \
-  void holder::set_##name(i::Isolate* isolate, const type value) {          \
+  void holder::set_##name(i::IsolateForSandbox isolate, const type value) { \
     /* This is a workaround for MSVC error C2440 not allowing  */           \
     /* reinterpret casts to the same type. */                               \
     struct C2440 {};                                                        \
@@ -481,12 +540,74 @@
 
 #define EXTERNAL_POINTER_ACCESSORS(holder, name, type, offset, tag)           \
   type holder::name() const {                                                 \
-    i::Isolate* isolate_for_sandbox = GetIsolateForSandbox(*this);            \
-    return holder::name(isolate_for_sandbox);                                 \
+    i::IsolateForSandbox isolate = GetIsolateForSandbox(*this);               \
+    return holder::name(isolate);                                             \
   }                                                                           \
   EXTERNAL_POINTER_ACCESSORS_MAYBE_READ_ONLY_HOST(holder, name, type, offset, \
                                                   tag)
 
+#define DECL_TRUSTED_POINTER_ACCESSORS(name, type)                           \
+  /* Trusted pointers currently always have release-acquire semantics. */    \
+  /* However, we still expose explicit release-acquire accessors so it */    \
+  /* can be made clear when they are required. */                            \
+  /* If desired, we could create separate {Read|Write}TrustedPointer */      \
+  /* routines for relaxed- and release-acquire semantics in the future. */   \
+  inline Tagged<type> name(IsolateForSandbox isolate) const;                 \
+  inline Tagged<type> name(IsolateForSandbox isolate, AcquireLoadTag) const; \
+  inline void set_##name(Tagged<type> value,                                 \
+                         WriteBarrierMode mode = UPDATE_WRITE_BARRIER);      \
+  inline void set_##name(Tagged<type> value, ReleaseStoreTag,                \
+                         WriteBarrierMode mode = UPDATE_WRITE_BARRIER);      \
+  inline bool has_##name() const;                                            \
+  inline void clear_##name();
+
+#define TRUSTED_POINTER_ACCESSORS(holder, name, type, offset, tag)             \
+  Tagged<type> holder::name(IsolateForSandbox isolate) const {                 \
+    return name(isolate, kAcquireLoad);                                        \
+  }                                                                            \
+  Tagged<type> holder::name(IsolateForSandbox isolate, AcquireLoadTag) const { \
+    DCHECK(has_##name());                                                      \
+    return type::cast(ReadTrustedPointerField<tag>(offset, isolate));          \
+  }                                                                            \
+  void holder::set_##name(Tagged<type> value, WriteBarrierMode mode) {         \
+    set_##name(value, kReleaseStore, mode);                                    \
+  }                                                                            \
+  void holder::set_##name(Tagged<type> value, ReleaseStoreTag,                 \
+                          WriteBarrierMode mode) {                             \
+    WriteTrustedPointerField<tag>(offset, value);                              \
+    CONDITIONAL_TRUSTED_POINTER_WRITE_BARRIER(*this, offset, tag, value,       \
+                                              mode);                           \
+  }                                                                            \
+  bool holder::has_##name() const {                                            \
+    return !IsTrustedPointerFieldCleared(offset);                              \
+  }                                                                            \
+  void holder::clear_##name() { ClearTrustedPointerField(offset); }
+
+#define DECL_CODE_POINTER_ACCESSORS(name) \
+  DECL_TRUSTED_POINTER_ACCESSORS(name, Code)
+#define CODE_POINTER_ACCESSORS(holder, name, offset) \
+  TRUSTED_POINTER_ACCESSORS(holder, name, Code, offset, kCodeIndirectPointerTag)
+
+// Accessors for "protected" pointers, i.e. references from one trusted object
+// to another trusted object. For these pointers it can be assumed that neither
+// the pointer nor the pointed-to object can be manipulated by an attacker.
+#define DECL_PROTECTED_POINTER_ACCESSORS(name, type) \
+  inline Tagged<type> name() const;                  \
+  inline void set_##name(Tagged<type> value,         \
+                         WriteBarrierMode mode = UPDATE_WRITE_BARRIER);
+
+#define PROTECTED_POINTER_ACCESSORS(holder, name, type, offset)              \
+  static_assert(std::is_base_of<TrustedObject, holder>::value);              \
+  Tagged<type> holder::name() const {                                        \
+    return TaggedField<type, offset, TrustedSpaceCompressionScheme>::load(   \
+        *this);                                                              \
+  }                                                                          \
+  void holder::set_##name(Tagged<type> value, WriteBarrierMode mode) {       \
+    TaggedField<type, offset, TrustedSpaceCompressionScheme>::store(*this,   \
+                                                                    value);  \
+    CONDITIONAL_PROTECTED_POINTER_WRITE_BARRIER(*this, offset, value, mode); \
+  }
+
 #define BIT_FIELD_ACCESSORS2(holder, get_field, set_field, name, BitField) \
   typename BitField::FieldType holder::name() const {                      \
     return BitField::decode(get_field());                                  \
@@ -543,6 +664,7 @@
 
 #ifdef V8_DISABLE_WRITE_BARRIERS
 #define WRITE_BARRIER(object, offset, value)
+#define WRITE_BARRIER_CPP(object, offset, value)
 #else
 #define WRITE_BARRIER(object, offset, value)                              \
   do {                                                                    \
@@ -581,14 +703,14 @@
 #endif
 
 #ifdef V8_DISABLE_WRITE_BARRIERS
-#define INDIRECT_POINTER_WRITE_BARRIER(object, offset, value)
+#define INDIRECT_POINTER_WRITE_BARRIER(object, offset, tag, value)
 #else
-#define INDIRECT_POINTER_WRITE_BARRIER(object, offset, value)           \
-  do {                                                                  \
-    DCHECK_NOT_NULL(GetHeapFromWritableObject(object));                 \
-    IndirectPointerWriteBarrier(                                        \
-        object, Tagged(object)->RawIndirectPointerField(offset), value, \
-        UPDATE_WRITE_BARRIER);                                          \
+#define INDIRECT_POINTER_WRITE_BARRIER(object, offset, tag, value)           \
+  do {                                                                       \
+    DCHECK_NOT_NULL(GetHeapFromWritableObject(object));                      \
+    IndirectPointerWriteBarrier(                                             \
+        object, Tagged(object)->RawIndirectPointerField(offset, tag), value, \
+        UPDATE_WRITE_BARRIER);                                               \
   } while (false)
 #endif
 
@@ -631,17 +753,39 @@
 #endif
 
 #ifdef V8_DISABLE_WRITE_BARRIERS
-#define CONDITIONAL_INDIRECT_POINTER_WRITE_BARRIER(object, offset, value, mode)
+#define CONDITIONAL_INDIRECT_POINTER_WRITE_BARRIER(object, offset, tag, value, \
+                                                   mode)
 #else
-#define CONDITIONAL_INDIRECT_POINTER_WRITE_BARRIER(object, offset, value, \
-                                                   mode)                  \
-  do {                                                                    \
-    DCHECK_NOT_NULL(GetHeapFromWritableObject(object));                   \
-    IndirectPointerWriteBarrier(                                          \
-        object, (object).RawIndirectPointerField(offset), value, mode);   \
+#define CONDITIONAL_INDIRECT_POINTER_WRITE_BARRIER(object, offset, tag, value, \
+                                                   mode)                       \
+  do {                                                                         \
+    DCHECK_NOT_NULL(GetHeapFromWritableObject(object));                        \
+    IndirectPointerWriteBarrier(                                               \
+        object, (object).RawIndirectPointerField(offset, tag), value, mode);   \
   } while (false)
 #endif
 
+#ifdef V8_ENABLE_SANDBOX
+#define CONDITIONAL_TRUSTED_POINTER_WRITE_BARRIER(object, offset, tag, value, \
+                                                  mode)                       \
+  CONDITIONAL_INDIRECT_POINTER_WRITE_BARRIER(object, offset, tag, value, mode)
+#else
+#define CONDITIONAL_TRUSTED_POINTER_WRITE_BARRIER(object, offset, tag, value, \
+                                                  mode)                       \
+  CONDITIONAL_WRITE_BARRIER(*this, offset, value, mode);
+#endif
+#define CONDITIONAL_CODE_POINTER_WRITE_BARRIER(object, offset, value, mode) \
+  CONDITIONAL_TRUSTED_POINTER_WRITE_BARRIER(                                \
+      object, offset, kCodeIndirectPointerTag, value, mode)
+
+#define CONDITIONAL_PROTECTED_POINTER_WRITE_BARRIER(object, offset, value, \
+                                                    mode)                  \
+  do {                                                                     \
+    DCHECK_NOT_NULL(GetHeapFromWritableObject(object));                    \
+    ProtectedPointerWriteBarrier(                                          \
+        object, (object).RawProtectedPointerField(offset), value, mode);   \
+  } while (false)
+
 #define ACQUIRE_READ_INT8_FIELD(p, offset) \
   static_cast<int8_t>(base::Acquire_Load(  \
       reinterpret_cast<const base::Atomic8*>(FIELD_ADDR(p, offset))))
