@@ -23,22 +23,22 @@ namespace {
 void LogExecution(Isolate* isolate, Handle<JSFunction> function) {
   DCHECK(v8_flags.log_function_events);
   if (!function->has_feedback_vector()) return;
-  if (!function->feedback_vector().log_next_execution()) return;
+  if (!function->feedback_vector()->log_next_execution()) return;
   Handle<SharedFunctionInfo> sfi(function->shared(), isolate);
   Handle<String> name = SharedFunctionInfo::DebugName(isolate, sfi);
   DisallowGarbageCollection no_gc;
-  auto raw_sfi = *sfi;
+  Tagged<SharedFunctionInfo> raw_sfi = *sfi;
   std::string event_name = "first-execution";
-  CodeKind kind = function->abstract_code(isolate).kind(isolate);
+  CodeKind kind = function->abstract_code(isolate)->kind(isolate);
   // Not adding "-interpreter" for tooling backwards compatiblity.
   if (kind != CodeKind::INTERPRETED_FUNCTION) {
     event_name += "-";
     event_name += CodeKindToString(kind);
   }
-  LOG(isolate,
-      FunctionEvent(event_name.c_str(), Script::cast(raw_sfi.script()).id(), 0,
-                    raw_sfi.StartPosition(), raw_sfi.EndPosition(), *name));
-  function->feedback_vector().set_log_next_execution(false);
+  LOG(isolate, FunctionEvent(
+                   event_name.c_str(), Script::cast(raw_sfi->script())->id(), 0,
+                   raw_sfi->StartPosition(), raw_sfi->EndPosition(), *name));
+  function->feedback_vector()->set_log_next_execution(false);
 }
 }  // namespace
 
@@ -54,7 +54,7 @@ RUNTIME_FUNCTION(Runtime_CompileLazy) {
 
   Handle<SharedFunctionInfo> sfi(function->shared(), isolate);
 
-  DCHECK(!function->is_compiled());
+  DCHECK(!function->is_compiled(isolate));
 #ifdef DEBUG
   if (v8_flags.trace_lazy && sfi->is_compiled()) {
     PrintF("[unoptimized: %s]\n", function->DebugNameCStr().get());
@@ -68,8 +68,8 @@ RUNTIME_FUNCTION(Runtime_CompileLazy) {
   if (V8_UNLIKELY(v8_flags.log_function_events)) {
     LogExecution(isolate, function);
   }
-  DCHECK(function->is_compiled());
-  return function->code();
+  DCHECK(function->is_compiled(isolate));
+  return function->code(isolate);
 }
 
 RUNTIME_FUNCTION(Runtime_InstallBaselineCode) {
@@ -79,13 +79,13 @@ RUNTIME_FUNCTION(Runtime_InstallBaselineCode) {
   Handle<SharedFunctionInfo> sfi(function->shared(), isolate);
   DCHECK(sfi->HasBaselineCode());
   IsCompiledScope is_compiled_scope(*sfi, isolate);
-  DCHECK(!function->HasAvailableOptimizedCode());
+  DCHECK(!function->HasAvailableOptimizedCode(isolate));
   DCHECK(!function->has_feedback_vector());
   JSFunction::CreateAndAttachFeedbackVector(isolate, function,
                                             &is_compiled_scope);
   {
     DisallowGarbageCollection no_gc;
-    Code baseline_code = sfi->baseline_code(kAcquireLoad);
+    Tagged<Code> baseline_code = sfi->baseline_code(kAcquireLoad);
     function->set_code(baseline_code);
     if V8_LIKELY (!v8_flags.log_function_events) return baseline_code;
   }
@@ -139,11 +139,11 @@ RUNTIME_FUNCTION(Runtime_CompileOptimized) {
 
   Compiler::CompileOptimized(isolate, function, mode, target_kind);
 
-  DCHECK(function->is_compiled());
+  DCHECK(function->is_compiled(isolate));
   if (V8_UNLIKELY(v8_flags.log_function_events)) {
     LogExecution(isolate, function);
   }
-  return function->code();
+  return function->code(isolate);
 }
 
 RUNTIME_FUNCTION(Runtime_FunctionLogNextExecution) {
@@ -152,7 +152,7 @@ RUNTIME_FUNCTION(Runtime_FunctionLogNextExecution) {
   Handle<JSFunction> js_function = args.at<JSFunction>(0);
   DCHECK(v8_flags.log_function_events);
   LogExecution(isolate, js_function);
-  return js_function->code();
+  return js_function->code(isolate);
 }
 
 RUNTIME_FUNCTION(Runtime_HealOptimizedCodeSlot) {
@@ -160,28 +160,35 @@ RUNTIME_FUNCTION(Runtime_HealOptimizedCodeSlot) {
   DCHECK_EQ(1, args.length());
   Handle<JSFunction> function = args.at<JSFunction>(0);
 
-  DCHECK(function->shared().is_compiled());
+  DCHECK(function->shared()->is_compiled());
 
-  function->feedback_vector().EvictOptimizedCodeMarkedForDeoptimization(
+  function->feedback_vector()->EvictOptimizedCodeMarkedForDeoptimization(
       isolate, function->shared(), "Runtime_HealOptimizedCodeSlot");
-  return function->code();
+  return function->code(isolate);
 }
 
+// The enum values need to match "AsmJsInstantiateResult" in
+// tools/metrics/histograms/enums.xml.
+enum AsmJsInstantiateResult {
+  kAsmJsInstantiateSuccess = 0,
+  kAsmJsInstantiateFail = 1,
+};
+
 RUNTIME_FUNCTION(Runtime_InstantiateAsmJs) {
   HandleScope scope(isolate);
   DCHECK_EQ(args.length(), 4);
   Handle<JSFunction> function = args.at<JSFunction>(0);
 
   Handle<JSReceiver> stdlib;
-  if (args[1].IsJSReceiver()) {
+  if (IsJSReceiver(args[1])) {
     stdlib = args.at<JSReceiver>(1);
   }
   Handle<JSReceiver> foreign;
-  if (args[2].IsJSReceiver()) {
+  if (IsJSReceiver(args[2])) {
     foreign = args.at<JSReceiver>(2);
   }
   Handle<JSArrayBuffer> memory;
-  if (args[3].IsJSArrayBuffer()) {
+  if (IsJSArrayBuffer(args[3])) {
     memory = args.at<JSArrayBuffer>(3);
   }
   Handle<SharedFunctionInfo> shared(function->shared(), isolate);
@@ -190,70 +197,40 @@ RUNTIME_FUNCTION(Runtime_InstantiateAsmJs) {
     Handle<AsmWasmData> data(shared->asm_wasm_data(), isolate);
     MaybeHandle<Object> result = AsmJs::InstantiateAsmWasm(
         isolate, shared, data, stdlib, foreign, memory);
-    if (!result.is_null()) return *result.ToHandleChecked();
+    if (!result.is_null()) {
+      isolate->counters()->asmjs_instantiate_result()->AddSample(
+          kAsmJsInstantiateSuccess);
+      return *result.ToHandleChecked();
+    }
+    if (isolate->has_exception()) {
+      // If instantiation fails, we do not propagate the exception but instead
+      // fall back to JS execution. The only exception (to that rule) is the
+      // termination exception.
+      DCHECK(isolate->is_execution_terminating());
+      return ReadOnlyRoots{isolate}.exception();
+    }
+    isolate->counters()->asmjs_instantiate_result()->AddSample(
+        kAsmJsInstantiateFail);
+
     // Remove wasm data, mark as broken for asm->wasm, replace function code
     // with UncompiledData, and return a smi 0 to indicate failure.
     SharedFunctionInfo::DiscardCompiled(isolate, shared);
   }
   shared->set_is_asm_wasm_broken(true);
 #endif
-  DCHECK_EQ(function->code(), *BUILTIN_CODE(isolate, InstantiateAsmJs));
+  DCHECK_EQ(function->code(isolate), *BUILTIN_CODE(isolate, InstantiateAsmJs));
   function->set_code(*BUILTIN_CODE(isolate, CompileLazy));
-  DCHECK(!isolate->has_pending_exception());
+  DCHECK(!isolate->has_exception());
   return Smi::zero();
 }
 
 namespace {
 
-// Whether the deopt exit is contained by the outermost loop containing the
-// osr'd loop. For example:
-//
-//  for (;;) {
-//    for (;;) {
-//    }  // OSR is triggered on this backedge.
-//  }  // This is the outermost loop containing the osr'd loop.
-bool DeoptExitIsInsideOsrLoop(Isolate* isolate, JSFunction function,
-                              BytecodeOffset deopt_exit_offset,
-                              BytecodeOffset osr_offset) {
-  DisallowGarbageCollection no_gc;
-  DCHECK(!deopt_exit_offset.IsNone());
-  DCHECK(!osr_offset.IsNone());
-
-  Handle<BytecodeArray> bytecode_array(
-      function.shared().GetBytecodeArray(isolate), isolate);
-  DCHECK(interpreter::BytecodeArrayIterator::IsValidOffset(
-      bytecode_array, deopt_exit_offset.ToInt()));
-
-  interpreter::BytecodeArrayIterator it(bytecode_array, osr_offset.ToInt());
-  DCHECK_EQ(it.current_bytecode(), interpreter::Bytecode::kJumpLoop);
-
-  for (; !it.done(); it.Advance()) {
-    const int current_offset = it.current_offset();
-    // If we've reached the deopt exit, it's contained in the current loop
-    // (this is covered by IsInRange below, but this check lets us avoid
-    // useless iteration).
-    if (current_offset == deopt_exit_offset.ToInt()) return true;
-    // We're only interested in loop ranges.
-    if (it.current_bytecode() != interpreter::Bytecode::kJumpLoop) continue;
-    // Is the deopt exit contained in the current loop?
-    if (base::IsInRange(deopt_exit_offset.ToInt(), it.GetJumpTargetOffset(),
-                        current_offset)) {
-      return true;
-    }
-    // We've reached nesting level 0, i.e. the current JumpLoop concludes a
-    // top-level loop.
-    const int loop_nesting_level = it.GetImmediateOperand(1);
-    if (loop_nesting_level == 0) return false;
-  }
-
-  UNREACHABLE();
-}
-
-bool TryGetOptimizedOsrCode(Isolate* isolate, FeedbackVector vector,
+bool TryGetOptimizedOsrCode(Isolate* isolate, Tagged<FeedbackVector> vector,
                             const interpreter::BytecodeArrayIterator& it,
-                            Code* code_out) {
-  base::Optional<Code> maybe_code =
-      vector.GetOptimizedOsrCode(isolate, it.GetSlotOperand(2));
+                            Tagged<Code>* code_out) {
+  base::Optional<Tagged<Code>> maybe_code =
+      vector->GetOptimizedOsrCode(isolate, it.GetSlotOperand(2));
   if (maybe_code.has_value()) {
     *code_out = maybe_code.value();
     return true;
@@ -272,26 +249,27 @@ bool TryGetOptimizedOsrCode(Isolate* isolate, FeedbackVector vector,
 //      }  // Type b: deopt exit < loop start < OSR backedge
 //    } // Type c: loop start < deopt exit < OSR backedge
 //  }  // The outermost loop
-void DeoptAllOsrLoopsContainingDeoptExit(Isolate* isolate, JSFunction function,
+void DeoptAllOsrLoopsContainingDeoptExit(Isolate* isolate,
+                                         Tagged<JSFunction> function,
                                          BytecodeOffset deopt_exit_offset) {
   DisallowGarbageCollection no_gc;
   DCHECK(!deopt_exit_offset.IsNone());
 
   if (!v8_flags.use_ic ||
-      !function.feedback_vector().maybe_has_optimized_osr_code()) {
+      !function->feedback_vector()->maybe_has_optimized_osr_code()) {
     return;
   }
   Handle<BytecodeArray> bytecode_array(
-      function.shared().GetBytecodeArray(isolate), isolate);
+      function->shared()->GetBytecodeArray(isolate), isolate);
   DCHECK(interpreter::BytecodeArrayIterator::IsValidOffset(
       bytecode_array, deopt_exit_offset.ToInt()));
 
   interpreter::BytecodeArrayIterator it(bytecode_array,
                                         deopt_exit_offset.ToInt());
 
-  FeedbackVector vector = function.feedback_vector();
-  Code code;
-  base::SmallVector<Code, 8> osr_codes;
+  Tagged<FeedbackVector> vector = function->feedback_vector();
+  Tagged<Code> code;
+  base::SmallVector<Tagged<Code>, 8> osr_codes;
   // Visit before the first loop-with-deopt is found
   for (; !it.done(); it.Advance()) {
     // We're only interested in loop ranges.
@@ -367,6 +345,10 @@ RUNTIME_FUNCTION(Runtime_NotifyDeoptimized) {
   TimerEventScope<TimerEventDeoptimizeCode> timer(isolate);
   TRACE_EVENT0("v8", "V8.DeoptimizeCode");
   Handle<JSFunction> function = deoptimizer->function();
+  if (v8_flags.profile_guided_optimization) {
+    function->shared()->set_cached_tiering_decision(
+        CachedTieringDecision::kNormal);
+  }
   // For OSR the optimized code isn't installed on the function, so get the
   // code object from deoptimizer.
   Handle<Code> optimized_code = deoptimizer->compiled_code();
@@ -417,8 +399,9 @@ RUNTIME_FUNCTION(Runtime_NotifyDeoptimized) {
   if (osr_offset.IsNone()) {
     Deoptimizer::DeoptimizeFunction(*function, *optimized_code);
     DeoptAllOsrLoopsContainingDeoptExit(isolate, *function, deopt_exit_offset);
-  } else if (DeoptExitIsInsideOsrLoop(isolate, *function, deopt_exit_offset,
-                                      osr_offset)) {
+  } else if (deopt_reason != DeoptimizeReason::kOSREarlyExit &&
+             Deoptimizer::DeoptExitIsInsideOsrLoop(
+                 isolate, *function, deopt_exit_offset, osr_offset)) {
     Deoptimizer::DeoptimizeFunction(*function, *optimized_code);
   }
 
@@ -461,64 +444,58 @@ void GetOsrOffsetAndFunctionForOSR(Isolate* isolate, BytecodeOffset* osr_offset,
   JavaScriptStackFrameIterator it(isolate);
   UnoptimizedFrame* frame = UnoptimizedFrame::cast(it.frame());
   DCHECK_IMPLIES(frame->is_interpreted(),
-                 frame->LookupCode().is_interpreter_trampoline_builtin());
+                 frame->LookupCode()->is_interpreter_trampoline_builtin());
   DCHECK_IMPLIES(frame->is_baseline(),
-                 frame->LookupCode().kind() == CodeKind::BASELINE);
+                 frame->LookupCode()->kind() == CodeKind::BASELINE);
 
   *osr_offset = BytecodeOffset(frame->GetBytecodeOffset());
   *function = handle(frame->function(), isolate);
 
   DCHECK(!osr_offset->IsNone());
-  DCHECK((*function)->shared().HasBytecodeArray());
+  DCHECK((*function)->shared()->HasBytecodeArray());
 }
 
-Object CompileOptimizedOSR(Isolate* isolate, Handle<JSFunction> function,
-                           BytecodeOffset osr_offset) {
+Tagged<Object> CompileOptimizedOSR(Isolate* isolate,
+                                   Handle<JSFunction> function,
+                                   CodeKind min_opt_level,
+                                   BytecodeOffset osr_offset) {
   const ConcurrencyMode mode =
       V8_LIKELY(isolate->concurrent_recompilation_enabled() &&
-                v8_flags.concurrent_osr)
+                v8_flags.concurrent_osr &&
+                !isolate->UseEfficiencyModeForTiering())
           ? ConcurrencyMode::kConcurrent
           : ConcurrencyMode::kSynchronous;
 
   Handle<Code> result;
-  if (!Compiler::CompileOptimizedOSR(isolate, function, osr_offset, mode)
-           .ToHandle(&result)) {
+  if (!Compiler::CompileOptimizedOSR(
+           isolate, function, osr_offset, mode,
+           (maglev::IsMaglevOsrEnabled() && min_opt_level == CodeKind::MAGLEV)
+               ? CodeKind::MAGLEV
+               : CodeKind::TURBOFAN)
+           .ToHandle(&result) ||
+      result->marked_for_deoptimization()) {
     // An empty result can mean one of two things:
     // 1) we've started a concurrent compilation job - everything is fine.
     // 2) synchronous compilation failed for some reason.
 
-    if (!function->HasAttachedOptimizedCode()) {
-      function->set_code(function->shared().GetCode(isolate));
+    if (!function->HasAttachedOptimizedCode(isolate)) {
+      function->set_code(function->shared()->GetCode(isolate));
     }
 
-    return {};
+    return Smi::zero();
   }
 
   DCHECK(!result.is_null());
-  DCHECK(result->is_turbofanned());  // TODO(v8:7700): Support Maglev.
+  DCHECK(result->is_turbofanned() || result->is_maglevved());
   DCHECK(CodeKindIsOptimizedJSFunction(result->kind()));
 
 #ifdef DEBUG
-  DeoptimizationData data =
+  Tagged<DeoptimizationData> data =
       DeoptimizationData::cast(result->deoptimization_data());
-  DCHECK_EQ(BytecodeOffset(data.OsrBytecodeOffset().value()), osr_offset);
-  DCHECK_GE(data.OsrPcOffset().value(), 0);
+  DCHECK_EQ(BytecodeOffset(data->OsrBytecodeOffset().value()), osr_offset);
+  DCHECK_GE(data->OsrPcOffset().value(), 0);
 #endif  // DEBUG
 
-  if (function->feedback_vector().invocation_count() <= 1 &&
-      !IsNone(function->tiering_state()) &&
-      !IsInProgress(function->tiering_state())) {
-    // With lazy feedback allocation we may not have feedback for the
-    // initial part of the function that was executed before we allocated a
-    // feedback vector. Reset any tiering states for such functions.
-    //
-    // TODO(mythria): Instead of resetting the tiering state here we
-    // should only mark a function for optimization if it has sufficient
-    // feedback. We cannot do this currently since we OSR only after we mark
-    // a function for optimization. We should instead change it to be based
-    // based on number of ticks.
-    function->reset_tiering_state();
-  }
   // First execution logging happens in LogOrTraceOptimizedOSREntry
   return *result;
 }
@@ -534,14 +511,14 @@ RUNTIME_FUNCTION(Runtime_CompileOptimizedOSR) {
   Handle<JSFunction> function;
   GetOsrOffsetAndFunctionForOSR(isolate, &osr_offset, &function);
 
-  return CompileOptimizedOSR(isolate, function, osr_offset);
+  return CompileOptimizedOSR(isolate, function, CodeKind::MAGLEV, osr_offset);
 }
 
 namespace {
 
-Object CompileOptimizedOSRFromMaglev(Isolate* isolate,
-                                     Handle<JSFunction> function,
-                                     BytecodeOffset osr_offset) {
+Tagged<Object> CompileOptimizedOSRFromMaglev(Isolate* isolate,
+                                             Handle<JSFunction> function,
+                                             BytecodeOffset osr_offset) {
   // This path is only relevant for tests (all production configurations enable
   // concurrent OSR). It's quite subtle, if interested read on:
   if (V8_UNLIKELY(!isolate->concurrent_recompilation_enabled() ||
@@ -560,10 +537,17 @@ Object CompileOptimizedOSRFromMaglev(Isolate* isolate,
     // letting it handle OSR. How do we trigger the early bailout? Returning
     // any non-null InstructionStream from this function triggers the deopt in
     // JumpLoop.
-    return function->code();
+    if (v8_flags.trace_osr) {
+      CodeTracer::Scope scope(isolate->GetCodeTracer());
+      PrintF(scope.file(),
+             "[OSR - Tiering from Maglev to Turbofan failed because "
+             "concurrent_osr is disabled. function: %s, osr offset: %d]\n",
+             function->DebugNameCStr().get(), osr_offset.ToInt());
+    }
+    return function->code(isolate);
   }
 
-  return CompileOptimizedOSR(isolate, function, osr_offset);
+  return CompileOptimizedOSR(isolate, function, CodeKind::TURBOFAN, osr_offset);
 }
 
 }  // namespace
@@ -577,7 +561,7 @@ RUNTIME_FUNCTION(Runtime_CompileOptimizedOSRFromMaglev) {
 
   JavaScriptStackFrameIterator it(isolate);
   MaglevFrame* frame = MaglevFrame::cast(it.frame());
-  DCHECK_EQ(frame->LookupCode().kind(), CodeKind::MAGLEV);
+  DCHECK_EQ(frame->LookupCode()->kind(), CodeKind::MAGLEV);
   Handle<JSFunction> function = handle(frame->function(), isolate);
 
   return CompileOptimizedOSRFromMaglev(isolate, function, osr_offset);
@@ -593,12 +577,15 @@ RUNTIME_FUNCTION(Runtime_CompileOptimizedOSRFromMaglevInlined) {
 
   JavaScriptStackFrameIterator it(isolate);
   MaglevFrame* frame = MaglevFrame::cast(it.frame());
-  DCHECK_EQ(frame->LookupCode().kind(), CodeKind::MAGLEV);
+  DCHECK_EQ(frame->LookupCode()->kind(), CodeKind::MAGLEV);
 
   if (*function != frame->function()) {
     // We are OSRing an inlined function. Mark the top frame one for
     // optimization.
-    isolate->tiering_manager()->MarkForTurboFanOptimization(frame->function());
+    if (!frame->function()->ActiveTierIsTurbofan(isolate)) {
+      isolate->tiering_manager()->MarkForTurboFanOptimization(
+          frame->function());
+    }
   }
 
   return CompileOptimizedOSRFromMaglev(isolate, function, osr_offset);
@@ -624,13 +611,13 @@ RUNTIME_FUNCTION(Runtime_LogOrTraceOptimizedOSREntry) {
   return ReadOnlyRoots(isolate).undefined_value();
 }
 
-static Object CompileGlobalEval(Isolate* isolate,
-                                Handle<i::Object> source_object,
-                                Handle<SharedFunctionInfo> outer_info,
-                                LanguageMode language_mode,
-                                int eval_scope_position, int eval_position) {
-  Handle<Context> context(isolate->context(), isolate);
-  Handle<Context> native_context(context->native_context(), isolate);
+static Tagged<Object> CompileGlobalEval(Isolate* isolate,
+                                        Handle<i::Object> source_object,
+                                        Handle<SharedFunctionInfo> outer_info,
+                                        LanguageMode language_mode,
+                                        int eval_scope_position,
+                                        int eval_position) {
+  Handle<NativeContext> native_context = isolate->native_context();
 
   // Check if native context allows code generation from
   // strings. Throw an exception if it doesn't.
@@ -656,6 +643,7 @@ static Object CompileGlobalEval(Isolate* isolate,
   // and return the compiled function bound in the local context.
   static const ParseRestriction restriction = NO_PARSE_RESTRICTION;
   Handle<JSFunction> compiled;
+  Handle<Context> context(isolate->context(), isolate);
   ASSIGN_RETURN_ON_EXCEPTION_VALUE(
       isolate, compiled,
       Compiler::GetFunctionFromEval(
