@@ -57,7 +57,7 @@ RUNTIME_FUNCTION_RETURN_PAIR(Runtime_DebugBreakOnBytecode) {
   // If the user requested to restart a frame, there is no need
   // to get the return value or check the bytecode for side-effects.
   if (isolate->debug()->IsRestartFrameScheduled()) {
-    Object exception = isolate->TerminateExecution();
+    Tagged<Object> exception = isolate->TerminateExecution();
     return MakePair(exception,
                     Smi::FromInt(static_cast<uint8_t>(Bytecode::kIllegal)));
   }
@@ -75,10 +75,10 @@ RUNTIME_FUNCTION_RETURN_PAIR(Runtime_DebugBreakOnBytecode) {
 
   // Make sure to only access these objects after the side effect check, as the
   // check can allocate on failure.
-  SharedFunctionInfo shared = interpreted_frame->function().shared();
-  BytecodeArray bytecode_array = shared.GetBytecodeArray(isolate);
+  Tagged<SharedFunctionInfo> shared = interpreted_frame->function()->shared();
+  Tagged<BytecodeArray> bytecode_array = shared->GetBytecodeArray(isolate);
   int bytecode_offset = interpreted_frame->GetBytecodeOffset();
-  Bytecode bytecode = Bytecodes::FromByte(bytecode_array.get(bytecode_offset));
+  Bytecode bytecode = Bytecodes::FromByte(bytecode_array->get(bytecode_offset));
 
   if (Bytecodes::Returns(bytecode)) {
     // If we are returning (or suspending), reset the bytecode array on the
@@ -100,8 +100,8 @@ RUNTIME_FUNCTION_RETURN_PAIR(Runtime_DebugBreakOnBytecode) {
     return MakePair(ReadOnlyRoots(isolate).exception(),
                     Smi::FromInt(static_cast<uint8_t>(bytecode)));
   }
-  Object interrupt_object = isolate->stack_guard()->HandleInterrupts();
-  if (interrupt_object.IsException(isolate)) {
+  Tagged<Object> interrupt_object = isolate->stack_guard()->HandleInterrupts();
+  if (IsException(interrupt_object, isolate)) {
     return MakePair(interrupt_object,
                     Smi::FromInt(static_cast<uint8_t>(bytecode)));
   }
@@ -113,10 +113,8 @@ RUNTIME_FUNCTION(Runtime_DebugBreakAtEntry) {
   HandleScope scope(isolate);
   DCHECK_EQ(1, args.length());
   Handle<JSFunction> function = args.at<JSFunction>(0);
-  USE(function);
 
-  DCHECK(function->shared().HasDebugInfo());
-  DCHECK(function->shared().GetDebugInfo().BreakAtEntry());
+  DCHECK(function->shared()->BreakAtEntry(isolate));
 
   // Get the top-most JavaScript frame. This is the debug target function.
   JavaScriptStackFrameIterator it(isolate);
@@ -166,7 +164,7 @@ template <class IteratorType>
 static Handle<ArrayList> AddIteratorInternalProperties(
     Isolate* isolate, Handle<ArrayList> result, Handle<IteratorType> iterator) {
   const char* kind = nullptr;
-  switch (iterator->map().instance_type()) {
+  switch (iterator->map()->instance_type()) {
     case JS_MAP_KEY_ITERATOR_TYPE:
       kind = "keys";
       break;
@@ -201,20 +199,20 @@ static Handle<ArrayList> AddIteratorInternalProperties(
 
 MaybeHandle<JSArray> Runtime::GetInternalProperties(Isolate* isolate,
                                                     Handle<Object> object) {
-  auto result = ArrayList::New(isolate, 8 * 2);
-  if (object->IsJSObject()) {
+  Handle<ArrayList> result = ArrayList::New(isolate, 8 * 2);
+  if (IsJSObject(*object)) {
     PrototypeIterator iter(isolate, Handle<JSObject>::cast(object),
                            kStartAtReceiver);
     if (iter.HasAccess()) {
       iter.Advance();
       Handle<Object> prototype = PrototypeIterator::GetCurrent(iter);
-      if (!iter.IsAtEnd() && iter.HasAccess() && object->IsJSGlobalProxy()) {
+      if (!iter.IsAtEnd() && iter.HasAccess() && IsJSGlobalProxy(*object)) {
         // Skip JSGlobalObject as the [[Prototype]].
-        DCHECK(prototype->IsJSGlobalObject());
+        DCHECK(IsJSGlobalObject(*prototype));
         iter.Advance();
         prototype = PrototypeIterator::GetCurrent(iter);
       }
-      if (!prototype->IsNull(isolate)) {
+      if (!IsNull(*prototype, isolate)) {
         result = ArrayList::Add(
             isolate, result,
             isolate->factory()->NewStringFromStaticChars("[[Prototype]]"),
@@ -222,7 +220,7 @@ MaybeHandle<JSArray> Runtime::GetInternalProperties(Isolate* isolate,
       }
     }
   }
-  if (object->IsJSBoundFunction()) {
+  if (IsJSBoundFunction(*object)) {
     Handle<JSBoundFunction> function = Handle<JSBoundFunction>::cast(object);
 
     result = ArrayList::Add(
@@ -239,13 +237,13 @@ MaybeHandle<JSArray> Runtime::GetInternalProperties(Isolate* isolate,
         isolate->factory()->NewJSArrayWithElements(
             isolate->factory()->CopyFixedArray(
                 handle(function->bound_arguments(), isolate))));
-  } else if (object->IsJSMapIterator()) {
+  } else if (IsJSMapIterator(*object)) {
     Handle<JSMapIterator> iterator = Handle<JSMapIterator>::cast(object);
     result = AddIteratorInternalProperties(isolate, result, iterator);
-  } else if (object->IsJSSetIterator()) {
+  } else if (IsJSSetIterator(*object)) {
     Handle<JSSetIterator> iterator = Handle<JSSetIterator>::cast(object);
     result = AddIteratorInternalProperties(isolate, result, iterator);
-  } else if (object->IsJSGeneratorObject()) {
+  } else if (IsJSGeneratorObject(*object)) {
     Handle<JSGeneratorObject> generator =
         Handle<JSGeneratorObject>::cast(object);
 
@@ -270,7 +268,7 @@ MaybeHandle<JSArray> Runtime::GetInternalProperties(Isolate* isolate,
         isolate, result,
         isolate->factory()->NewStringFromAsciiChecked("[[GeneratorReceiver]]"),
         handle(generator->receiver(), isolate));
-  } else if (object->IsJSPromise()) {
+  } else if (IsJSPromise(*object)) {
     Handle<JSPromise> promise = Handle<JSPromise>::cast(object);
 
     result = ArrayList::Add(
@@ -284,7 +282,7 @@ MaybeHandle<JSArray> Runtime::GetInternalProperties(Isolate* isolate,
         promise->status() == Promise::kPending
             ? isolate->factory()->undefined_value()
             : handle(promise->result(), isolate));
-  } else if (object->IsJSProxy()) {
+  } else if (IsJSProxy(*object)) {
     Handle<JSProxy> js_proxy = Handle<JSProxy>::cast(object);
 
     result = ArrayList::Add(
@@ -299,7 +297,7 @@ MaybeHandle<JSArray> Runtime::GetInternalProperties(Isolate* isolate,
         isolate, result,
         isolate->factory()->NewStringFromAsciiChecked("[[IsRevoked]]"),
         isolate->factory()->ToBoolean(js_proxy->IsRevoked()));
-  } else if (object->IsJSPrimitiveWrapper()) {
+  } else if (IsJSPrimitiveWrapper(*object)) {
     Handle<JSPrimitiveWrapper> js_value =
         Handle<JSPrimitiveWrapper>::cast(object);
 
@@ -307,14 +305,14 @@ MaybeHandle<JSArray> Runtime::GetInternalProperties(Isolate* isolate,
         isolate, result,
         isolate->factory()->NewStringFromAsciiChecked("[[PrimitiveValue]]"),
         handle(js_value->value(), isolate));
-  } else if (object->IsJSWeakRef()) {
+  } else if (IsJSWeakRef(*object)) {
     Handle<JSWeakRef> js_weak_ref = Handle<JSWeakRef>::cast(object);
 
     result = ArrayList::Add(
         isolate, result,
         isolate->factory()->NewStringFromAsciiChecked("[[WeakRefTarget]]"),
         handle(js_weak_ref->target(), isolate));
-  } else if (object->IsJSArrayBuffer()) {
+  } else if (IsJSArrayBuffer(*object)) {
     Handle<JSArrayBuffer> js_array_buffer = Handle<JSArrayBuffer>::cast(object);
     if (js_array_buffer->was_detached()) {
       // Mark a detached JSArrayBuffer and such and don't even try to
@@ -326,10 +324,8 @@ MaybeHandle<JSArray> Runtime::GetInternalProperties(Isolate* isolate,
           isolate->factory()->true_value());
     } else {
       const size_t byte_length = js_array_buffer->byte_length();
-      // TODO(v8:4153): Remove this code once the maximum lengths are equal (and
-      // add a static assertion that it stays that way).
-      static_assert(JSTypedArray::kMaxLength < JSArrayBuffer::kMaxByteLength);
-      CHECK_LE(byte_length, JSArrayBuffer::kMaxByteLength);
+      static_assert(JSTypedArray::kMaxByteLength ==
+                    JSArrayBuffer::kMaxByteLength);
       using DataView = std::tuple<const char*, ExternalArrayType, size_t>;
       for (auto [name, type, elem_size] :
            {DataView{"[[Int8Array]]", kExternalInt8Array, 1},
@@ -338,7 +334,6 @@ MaybeHandle<JSArray> Runtime::GetInternalProperties(Isolate* isolate,
             DataView{"[[Int32Array]]", kExternalInt32Array, 4}}) {
         if ((byte_length % elem_size) != 0) continue;
         size_t length = byte_length / elem_size;
-        if (length > JSTypedArray::kMaxLength) continue;
         result =
             ArrayList::Add(isolate, result,
                            isolate->factory()->NewStringFromAsciiChecked(name),
@@ -365,7 +360,7 @@ MaybeHandle<JSArray> Runtime::GetInternalProperties(Isolate* isolate,
           isolate->factory()->array_buffer_wasm_memory_symbol();
       Handle<Object> memory_object =
           JSObject::GetDataProperty(isolate, js_array_buffer, memory_symbol);
-      if (!memory_object->IsUndefined(isolate)) {
+      if (!IsUndefined(*memory_object, isolate)) {
         result = ArrayList::Add(isolate, result,
                                 isolate->factory()->NewStringFromAsciiChecked(
                                     "[[WebAssemblyMemory]]"),
@@ -373,26 +368,26 @@ MaybeHandle<JSArray> Runtime::GetInternalProperties(Isolate* isolate,
       }
     }
 #if V8_ENABLE_WEBASSEMBLY
-  } else if (object->IsWasmInstanceObject()) {
+  } else if (IsWasmInstanceObject(*object)) {
     result = AddWasmInstanceObjectInternalProperties(
         isolate, result, Handle<WasmInstanceObject>::cast(object));
-  } else if (object->IsWasmModuleObject()) {
+  } else if (IsWasmModuleObject(*object)) {
     result = AddWasmModuleObjectInternalProperties(
         isolate, result, Handle<WasmModuleObject>::cast(object));
-  } else if (object->IsWasmTableObject()) {
+  } else if (IsWasmTableObject(*object)) {
     result = AddWasmTableObjectInternalProperties(
         isolate, result, Handle<WasmTableObject>::cast(object));
 #endif  // V8_ENABLE_WEBASSEMBLY
   }
   return isolate->factory()->NewJSArrayWithElements(
-      ArrayList::Elements(isolate, result), PACKED_ELEMENTS);
+      ArrayList::ToFixedArray(isolate, result), PACKED_ELEMENTS);
 }
 
 RUNTIME_FUNCTION(Runtime_GetGeneratorScopeCount) {
   HandleScope scope(isolate);
   DCHECK_EQ(1, args.length());
 
-  if (!args[0].IsJSGeneratorObject()) return Smi::zero();
+  if (!IsJSGeneratorObject(args[0])) return Smi::zero();
 
   // Check arguments.
   Handle<JSGeneratorObject> gen = args.at<JSGeneratorObject>(0);
@@ -415,7 +410,7 @@ RUNTIME_FUNCTION(Runtime_GetGeneratorScopeDetails) {
   HandleScope scope(isolate);
   DCHECK_EQ(2, args.length());
 
-  if (!args[0].IsJSGeneratorObject()) {
+  if (!IsJSGeneratorObject(args[0])) {
     return ReadOnlyRoots(isolate).undefined_value();
   }
 
@@ -483,7 +478,7 @@ RUNTIME_FUNCTION(Runtime_GetBreakLocations) {
   // Find the number of break points
   Handle<Object> break_locations =
       Debug::GetSourceBreakLocations(isolate, shared);
-  if (break_locations->IsUndefined(isolate)) {
+  if (IsUndefined(*break_locations, isolate)) {
     return ReadOnlyRoots(isolate).undefined_value();
   }
   // Return array as JS array
@@ -539,9 +534,9 @@ RUNTIME_FUNCTION(Runtime_FunctionGetInferredName) {
   SealHandleScope shs(isolate);
   DCHECK_EQ(1, args.length());
 
-  Object f = args[0];
-  if (f.IsJSFunction()) {
-    return JSFunction::cast(f).shared().inferred_name();
+  Tagged<Object> f = args[0];
+  if (IsJSFunction(f)) {
+    return JSFunction::cast(f)->shared()->inferred_name();
   }
   return ReadOnlyRoots(isolate).empty_string();
 }
@@ -552,7 +547,7 @@ RUNTIME_FUNCTION(Runtime_FunctionGetInferredName) {
 RUNTIME_FUNCTION(Runtime_CollectGarbage) {
   SealHandleScope shs(isolate);
   DCHECK_EQ(1, args.length());
-  isolate->heap()->PreciseCollectAllGarbage(Heap::kNoGCFlags,
+  isolate->heap()->PreciseCollectAllGarbage(GCFlag::kNoFlags,
                                             GarbageCollectionReason::kRuntime);
   return ReadOnlyRoots(isolate).undefined_value();
 }
@@ -571,14 +566,14 @@ int ScriptLinePosition(Handle<Script> script, int line) {
 
   Script::InitLineEnds(script->GetIsolate(), script);
 
-  FixedArray line_ends_array = FixedArray::cast(script->line_ends());
-  const int line_count = line_ends_array.length();
+  Tagged<FixedArray> line_ends_array = FixedArray::cast(script->line_ends());
+  const int line_count = line_ends_array->length();
   DCHECK_LT(0, line_count);
 
   if (line == 0) return 0;
   // If line == line_count, we return the first position beyond the last line.
   if (line > line_count) return -1;
-  return Smi::ToInt(line_ends_array.get(line - 1)) + 1;
+  return Smi::ToInt(line_ends_array->get(line - 1)) + 1;
 }
 
 int ScriptLinePositionWithOffset(Handle<Script> script, int line, int offset) {
@@ -642,14 +637,14 @@ Handle<Object> ScriptLocationFromLine(Isolate* isolate, Handle<Script> script,
   // additionally subtracting corresponding offsets.
 
   int32_t line = 0;
-  if (!opt_line->IsNullOrUndefined(isolate)) {
-    CHECK(opt_line->IsNumber());
+  if (!IsNullOrUndefined(*opt_line, isolate)) {
+    CHECK(IsNumber(*opt_line));
     line = NumberToInt32(*opt_line) - script->line_offset();
   }
 
   int32_t column = 0;
-  if (!opt_column->IsNullOrUndefined(isolate)) {
-    CHECK(opt_column->IsNumber());
+  if (!IsNullOrUndefined(*opt_column, isolate)) {
+    CHECK(IsNumber(*opt_column));
     column = NumberToInt32(*opt_column);
     if (line == 0) column -= script->column_offset();
   }
@@ -664,9 +659,9 @@ Handle<Object> ScriptLocationFromLine(Isolate* isolate, Handle<Script> script,
 // Slow traversal over all scripts on the heap.
 bool GetScriptById(Isolate* isolate, int needle, Handle<Script>* result) {
   Script::Iterator iterator(isolate);
-  for (Script script = iterator.Next(); !script.is_null();
+  for (Tagged<Script> script = iterator.Next(); !script.is_null();
        script = iterator.Next()) {
-    if (script.id() == needle) {
+    if (script->id() == needle) {
       *result = handle(script, isolate);
       return true;
     }
@@ -811,7 +806,7 @@ RUNTIME_FUNCTION(Runtime_DebugCollectCoverage) {
 
 RUNTIME_FUNCTION(Runtime_DebugTogglePreciseCoverage) {
   SealHandleScope shs(isolate);
-  bool enable = Oddball::cast(args[0]).ToBool(isolate);
+  bool enable = Boolean::cast(args[0])->ToBool(isolate);
   Coverage::SelectMode(isolate, enable ? debug::CoverageMode::kPreciseCount
                                        : debug::CoverageMode::kBestEffort);
   return ReadOnlyRoots(isolate).undefined_value();
@@ -819,7 +814,7 @@ RUNTIME_FUNCTION(Runtime_DebugTogglePreciseCoverage) {
 
 RUNTIME_FUNCTION(Runtime_DebugToggleBlockCoverage) {
   SealHandleScope shs(isolate);
-  bool enable = Oddball::cast(args[0]).ToBool(isolate);
+  bool enable = Boolean::cast(args[0])->ToBool(isolate);
   Coverage::SelectMode(isolate, enable ? debug::CoverageMode::kBlockCount
                                        : debug::CoverageMode::kBestEffort);
   return ReadOnlyRoots(isolate).undefined_value();
@@ -836,7 +831,7 @@ RUNTIME_FUNCTION(Runtime_DebugAsyncFunctionSuspended) {
   Handle<JSPromise> outer_promise = args.at<JSPromise>(1);
   Handle<JSFunction> reject_handler = args.at<JSFunction>(2);
   Handle<JSGeneratorObject> generator = args.at<JSGeneratorObject>(3);
-  bool is_predicted_as_caught = Oddball::cast(args[4]).ToBool(isolate);
+  bool is_predicted_as_caught = Boolean::cast(args[4])->ToBool(isolate);
 
   // Allocate the throwaway promise and fire the appropriate init
   // hook for the throwaway promise (passing the {promise} as its
@@ -868,7 +863,7 @@ RUNTIME_FUNCTION(Runtime_DebugAsyncFunctionSuspended) {
 
     Handle<WeakFixedArray> awaited_by_holder(
         isolate->factory()->NewWeakFixedArray(1));
-    awaited_by_holder->Set(
+    awaited_by_holder->set(
         0, MaybeObject::MakeWeak(MaybeObject::FromObject(*generator)));
     Object::SetProperty(isolate, promise,
                         isolate->factory()->promise_awaited_by_symbol(),
@@ -884,7 +879,7 @@ RUNTIME_FUNCTION(Runtime_DebugPromiseThen) {
   DCHECK_EQ(1, args.length());
   HandleScope scope(isolate);
   Handle<JSReceiver> promise = args.at<JSReceiver>(0);
-  if (promise->IsJSPromise()) {
+  if (IsJSPromise(*promise)) {
     isolate->OnPromiseThen(Handle<JSPromise>::cast(promise));
   }
   return *promise;
@@ -896,7 +891,7 @@ RUNTIME_FUNCTION(Runtime_LiveEditPatchScript) {
   Handle<JSFunction> script_function = args.at<JSFunction>(0);
   Handle<String> new_source = args.at<String>(1);
 
-  Handle<Script> script(Script::cast(script_function->shared().script()),
+  Handle<Script> script(Script::cast(script_function->shared()->script()),
                         isolate);
   v8::debug::LiveEditResult result;
   LiveEdit::PatchScript(isolate, script, new_source, /* preview */ false,
@@ -931,8 +926,16 @@ RUNTIME_FUNCTION(Runtime_ProfileCreateSnapshotDataBlob) {
 
   DisableEmbeddedBlobRefcounting();
 
+  static constexpr char* kNoEmbeddedSource = nullptr;
+  // Have the SnapshotCreator create a new Isolate from scratch.
+  static constexpr Isolate* kNoIsolate = nullptr;
+  // We use this flag to tell the serializer not to finalize/seal RO space -
+  // this already happened after deserializing the main Isolate.
+  static constexpr Snapshot::SerializerFlags kSerializerFlags =
+      Snapshot::SerializerFlag::kAllowActiveIsolateForTesting;
   v8::StartupData blob = CreateSnapshotDataBlobInternal(
-      v8::SnapshotCreator::FunctionCodeHandling::kClear, nullptr);
+      v8::SnapshotCreator::FunctionCodeHandling::kClear, kNoEmbeddedSource,
+      kNoIsolate, kSerializerFlags);
   delete[] blob.data;
 
   // Track the embedded blob size as well.
