@@ -49,23 +49,23 @@ uint32_t GetArgcForReplaceCallable(uint32_t num_captures,
 
 // Looks up the capture of the given name. Returns the (1-based) numbered
 // capture index or -1 on failure.
-int LookupNamedCapture(const std::function<bool(String)>& name_matches,
-                       FixedArray capture_name_map) {
+int LookupNamedCapture(const std::function<bool(Tagged<String>)>& name_matches,
+                       Tagged<FixedArray> capture_name_map) {
   // TODO(jgruber): Sort capture_name_map and do binary search via
   // internalized strings.
 
   int maybe_capture_index = -1;
-  const int named_capture_count = capture_name_map.length() >> 1;
+  const int named_capture_count = capture_name_map->length() >> 1;
   for (int j = 0; j < named_capture_count; j++) {
     // The format of {capture_name_map} is documented at
     // JSRegExp::kIrregexpCaptureNameMapIndex.
     const int name_ix = j * 2;
     const int index_ix = j * 2 + 1;
 
-    String capture_name = String::cast(capture_name_map.get(name_ix));
+    Tagged<String> capture_name = String::cast(capture_name_map->get(name_ix));
     if (!name_matches(capture_name)) continue;
 
-    maybe_capture_index = Smi::ToInt(capture_name_map.get(index_ix));
+    maybe_capture_index = Smi::ToInt(capture_name_map->get(index_ix));
     break;
   }
 
@@ -150,8 +150,8 @@ class CompiledReplacement {
 
   template <typename Char>
   bool ParseReplacementPattern(base::Vector<Char> characters,
-                               FixedArray capture_name_map, int capture_count,
-                               int subject_length) {
+                               Tagged<FixedArray> capture_name_map,
+                               int capture_count, int subject_length) {
     // Equivalent to String::GetSubstitution, except that this method converts
     // the replacement string into an internal representation that avoids
     // repeated parsing when used repeatedly.
@@ -275,8 +275,8 @@ class CompiledReplacement {
             // Let capture be ? Get(namedCaptures, groupName).
 
             const int capture_index = LookupNamedCapture(
-                [=](String capture_name) {
-                  return capture_name.IsEqualTo(requested_name);
+                [=](Tagged<String> capture_name) {
+                  return capture_name->IsEqualTo(requested_name);
                 },
                 capture_name_map);
 
@@ -330,11 +330,11 @@ bool CompiledReplacement::Compile(Isolate* isolate, Handle<JSRegExp> regexp,
     String::FlatContent content = replacement->GetFlatContent(no_gc);
     DCHECK(content.IsFlat());
 
-    FixedArray capture_name_map;
+    Tagged<FixedArray> capture_name_map;
     if (capture_count > 0) {
       DCHECK(JSRegExp::TypeSupportsCaptures(regexp->type_tag()));
-      Object maybe_capture_name_map = regexp->capture_name_map();
-      if (maybe_capture_name_map.IsFixedArray()) {
+      Tagged<Object> maybe_capture_name_map = regexp->capture_name_map();
+      if (IsFixedArray(maybe_capture_name_map)) {
         capture_name_map = FixedArray::cast(maybe_capture_name_map);
       }
     }
@@ -464,12 +464,13 @@ void FindStringIndices(Isolate* isolate,
   }
 }
 
-void FindStringIndicesDispatch(Isolate* isolate, String subject, String pattern,
+void FindStringIndicesDispatch(Isolate* isolate, Tagged<String> subject,
+                               Tagged<String> pattern,
                                std::vector<int>* indices, unsigned int limit) {
   {
     DisallowGarbageCollection no_gc;
-    String::FlatContent subject_content = subject.GetFlatContent(no_gc);
-    String::FlatContent pattern_content = pattern.GetFlatContent(no_gc);
+    String::FlatContent subject_content = subject->GetFlatContent(no_gc);
+    String::FlatContent pattern_content = pattern->GetFlatContent(no_gc);
     DCHECK(subject_content.IsFlat());
     DCHECK(pattern_content.IsFlat());
     if (subject_content.IsOneByte()) {
@@ -541,7 +542,8 @@ void TruncateRegexpIndicesList(Isolate* isolate) {
 }  // namespace
 
 template <typename ResultSeqString>
-V8_WARN_UNUSED_RESULT static Object StringReplaceGlobalAtomRegExpWithString(
+V8_WARN_UNUSED_RESULT static Tagged<Object>
+StringReplaceGlobalAtomRegExpWithString(
     Isolate* isolate, Handle<String> subject, Handle<JSRegExp> pattern_regexp,
     Handle<String> replacement, Handle<RegExpMatchInfo> last_match_info) {
   DCHECK(subject->IsFlat());
@@ -550,9 +552,9 @@ V8_WARN_UNUSED_RESULT static Object StringReplaceGlobalAtomRegExpWithString(
   std::vector<int>* indices = GetRewoundRegexpIndicesList(isolate);
 
   DCHECK_EQ(JSRegExp::ATOM, pattern_regexp->type_tag());
-  String pattern = pattern_regexp->atom_pattern();
+  Tagged<String> pattern = pattern_regexp->atom_pattern();
   int subject_len = subject->length();
-  int pattern_len = pattern.length();
+  int pattern_len = pattern->length();
   int replacement_len = replacement->length();
 
   FindStringIndicesDispatch(isolate, *subject, pattern, indices, 0xFFFFFFFF);
@@ -620,7 +622,7 @@ V8_WARN_UNUSED_RESULT static Object StringReplaceGlobalAtomRegExpWithString(
   return *result;
 }
 
-V8_WARN_UNUSED_RESULT static Object StringReplaceGlobalRegExpWithString(
+V8_WARN_UNUSED_RESULT static Tagged<Object> StringReplaceGlobalRegExpWithString(
     Isolate* isolate, Handle<String> subject, Handle<JSRegExp> regexp,
     Handle<String> replacement, Handle<RegExpMatchInfo> last_match_info) {
   DCHECK(subject->IsFlat());
@@ -700,7 +702,8 @@ V8_WARN_UNUSED_RESULT static Object StringReplaceGlobalRegExpWithString(
 }
 
 template <typename ResultSeqString>
-V8_WARN_UNUSED_RESULT static Object StringReplaceGlobalRegExpWithEmptyString(
+V8_WARN_UNUSED_RESULT static Tagged<Object>
+StringReplaceGlobalRegExpWithEmptyString(
     Isolate* isolate, Handle<String> subject, Handle<JSRegExp> regexp,
     Handle<RegExpMatchInfo> last_match_info) {
   DCHECK(subject->IsFlat());
@@ -811,7 +814,7 @@ RUNTIME_FUNCTION(Runtime_StringSplit) {
   CHECK_LT(0, pattern_length);
 
   if (limit == 0xFFFFFFFFu) {
-    FixedArray last_match_cache_unused;
+    Tagged<FixedArray> last_match_cache_unused;
     Handle<Object> cached_answer(
         RegExpResultsCache::Lookup(isolate->heap(), *subject, *pattern,
                                    &last_match_cache_unused,
@@ -845,9 +848,9 @@ RUNTIME_FUNCTION(Runtime_StringSplit) {
   // Create JSArray of substrings separated by separator.
   int part_count = static_cast<int>(indices->size());
 
-  Handle<JSArray> result =
-      isolate->factory()->NewJSArray(PACKED_ELEMENTS, part_count, part_count,
-                                     INITIALIZE_ARRAY_ELEMENTS_WITH_HOLE);
+  Handle<JSArray> result = isolate->factory()->NewJSArray(
+      PACKED_ELEMENTS, part_count, part_count,
+      ArrayStorageAllocationMode::INITIALIZE_ARRAY_ELEMENTS_WITH_HOLE);
 
   DCHECK(result->HasObjectElements());
 
@@ -915,7 +918,7 @@ RUNTIME_FUNCTION(Runtime_RegExpExec) {
   Handle<JSRegExp> regexp = args.at<JSRegExp>(0);
   Handle<String> subject = args.at<String>(1);
   int32_t index = 0;
-  CHECK(args[2].ToInt32(&index));
+  CHECK(Object::ToInt32(args[2], &index));
   Handle<RegExpMatchInfo> last_match_info = args.at<RegExpMatchInfo>(3);
   RETURN_RESULT_OR_FAILURE(
       isolate, RegExpExec(isolate, regexp, subject, index, last_match_info,
@@ -928,7 +931,7 @@ RUNTIME_FUNCTION(Runtime_RegExpExecTreatMatchAtEndAsFailure) {
   Handle<JSRegExp> regexp = args.at<JSRegExp>(0);
   Handle<String> subject = args.at<String>(1);
   int32_t index = 0;
-  CHECK(args[2].ToInt32(&index));
+  CHECK(Object::ToInt32(args[2], &index));
   Handle<RegExpMatchInfo> last_match_info = args.at<RegExpMatchInfo>(3);
   RETURN_RESULT_OR_FAILURE(
       isolate, RegExpExec(isolate, regexp, subject, index, last_match_info,
@@ -941,7 +944,7 @@ RUNTIME_FUNCTION(Runtime_RegExpExperimentalOneshotExec) {
   Handle<JSRegExp> regexp = args.at<JSRegExp>(0);
   Handle<String> subject = args.at<String>(1);
   int32_t index = 0;
-  CHECK(args[2].ToInt32(&index));
+  CHECK(Object::ToInt32(args[2], &index));
   Handle<RegExpMatchInfo> last_match_info = args.at<RegExpMatchInfo>(3);
   RETURN_RESULT_OR_FAILURE(
       isolate,
@@ -956,7 +959,7 @@ RUNTIME_FUNCTION(
   Handle<JSRegExp> regexp = args.at<JSRegExp>(0);
   Handle<String> subject = args.at<String>(1);
   int32_t index = 0;
-  CHECK(args[2].ToInt32(&index));
+  CHECK(Object::ToInt32(args[2], &index));
   Handle<RegExpMatchInfo> last_match_info = args.at<RegExpMatchInfo>(3);
   RETURN_RESULT_OR_FAILURE(
       isolate,
@@ -988,8 +991,8 @@ class MatchInfoBackedMatch : public String::Match {
     subject_ = String::Flatten(isolate, subject);
 
     if (JSRegExp::TypeSupportsCaptures(regexp->type_tag())) {
-      Object o = regexp->capture_name_map();
-      has_named_captures_ = o.IsFixedArray();
+      Tagged<Object> o = regexp->capture_name_map();
+      has_named_captures_ = IsFixedArray(o);
       if (has_named_captures_) {
         capture_name_map_ = handle(FixedArray::cast(o), isolate);
       }
@@ -1003,12 +1006,12 @@ class MatchInfoBackedMatch : public String::Match {
   }
 
   Handle<String> GetPrefix() override {
-    const int match_start = match_info_->Capture(0);
+    const int match_start = match_info_->capture(0);
     return isolate_->factory()->NewSubString(subject_, 0, match_start);
   }
 
   Handle<String> GetSuffix() override {
-    const int match_end = match_info_->Capture(1);
+    const int match_end = match_info_->capture(1);
     return isolate_->factory()->NewSubString(subject_, match_end,
                                              subject_->length());
   }
@@ -1016,7 +1019,7 @@ class MatchInfoBackedMatch : public String::Match {
   bool HasNamedCaptures() override { return has_named_captures_; }
 
   int CaptureCount() override {
-    return match_info_->NumberOfCaptureRegisters() / 2;
+    return match_info_->number_of_capture_registers() / 2;
   }
 
   MaybeHandle<String> GetCapture(int i, bool* capture_exists) override {
@@ -1030,7 +1033,9 @@ class MatchInfoBackedMatch : public String::Match {
                                       CaptureState* state) override {
     DCHECK(has_named_captures_);
     const int capture_index = LookupNamedCapture(
-        [=](String capture_name) { return capture_name.Equals(*name); },
+        [=](Tagged<String> capture_name) {
+          return capture_name->Equals(*name);
+        },
         *capture_name_map_);
 
     if (capture_index == -1) {
@@ -1076,8 +1081,8 @@ class VectorBackedMatch : public String::Match {
         captures_(captures) {
     subject_ = String::Flatten(isolate, subject);
 
-    DCHECK(groups_obj->IsUndefined(isolate) || groups_obj->IsJSReceiver());
-    has_named_captures_ = !groups_obj->IsUndefined(isolate);
+    DCHECK(IsUndefined(*groups_obj, isolate) || IsJSReceiver(*groups_obj));
+    has_named_captures_ = !IsUndefined(*groups_obj, isolate);
     if (has_named_captures_) groups_obj_ = Handle<JSReceiver>::cast(groups_obj);
   }
 
@@ -1099,7 +1104,7 @@ class VectorBackedMatch : public String::Match {
 
   MaybeHandle<String> GetCapture(int i, bool* capture_exists) override {
     Handle<Object> capture_obj = captures_[i];
-    if (capture_obj->IsUndefined(isolate_)) {
+    if (IsUndefined(*capture_obj, isolate_)) {
       *capture_exists = false;
       return isolate_->factory()->empty_string();
     }
@@ -1111,11 +1116,20 @@ class VectorBackedMatch : public String::Match {
                                       CaptureState* state) override {
     DCHECK(has_named_captures_);
 
+    // Strings representing integer indices are not valid identifiers (and
+    // therefore not valid capture names).
+    {
+      size_t unused;
+      if (name->AsIntegerIndex(&unused)) {
+        *state = UNMATCHED;
+        return isolate_->factory()->empty_string();
+      }
+    }
     Handle<Object> capture_obj;
     ASSIGN_RETURN_ON_EXCEPTION(isolate_, capture_obj,
                                Object::GetProperty(isolate_, groups_obj_, name),
                                String);
-    if (capture_obj->IsUndefined(isolate_)) {
+    if (IsUndefined(*capture_obj, isolate_)) {
       *state = UNMATCHED;
       return isolate_->factory()->empty_string();
     } else {
@@ -1139,7 +1153,7 @@ class VectorBackedMatch : public String::Match {
 // RegExpBuiltinsAssembler::ConstructNewResultFromMatchInfo).
 Handle<JSObject> ConstructNamedCaptureGroupsObject(
     Isolate* isolate, Handle<FixedArray> capture_map,
-    const std::function<Object(int)>& f_get_capture) {
+    const std::function<Tagged<Object>(int)>& f_get_capture) {
   Handle<JSObject> groups = isolate->factory()->NewJSObjectWithNullProto();
 
   const int named_capture_count = capture_map->length() >> 1;
@@ -1153,7 +1167,7 @@ Handle<JSObject> ConstructNamedCaptureGroupsObject(
     DCHECK_GE(capture_ix, 1);  // Explicit groups start at index 1.
 
     Handle<Object> capture_value(f_get_capture(capture_ix), isolate);
-    DCHECK(capture_value->IsUndefined(isolate) || capture_value->IsString());
+    DCHECK(IsUndefined(*capture_value, isolate) || IsString(*capture_value));
 
     JSObject::AddProperty(isolate, groups, capture_name, capture_value, NONE);
   }
@@ -1164,9 +1178,9 @@ Handle<JSObject> ConstructNamedCaptureGroupsObject(
 // Only called from Runtime_RegExpExecMultiple so it doesn't need to maintain
 // separate last match info.  See comment on that function.
 template <bool has_capture>
-static Object SearchRegExpMultiple(Isolate* isolate, Handle<String> subject,
-                                   Handle<JSRegExp> regexp,
-                                   Handle<RegExpMatchInfo> last_match_array) {
+static Tagged<Object> SearchRegExpMultiple(
+    Isolate* isolate, Handle<String> subject, Handle<JSRegExp> regexp,
+    Handle<RegExpMatchInfo> last_match_array) {
   DCHECK(RegExpUtils::IsUnmodifiedRegExp(isolate, regexp));
   DCHECK_NE(has_capture, regexp->capture_count() == 0);
   DCHECK(subject->IsFlat());
@@ -1190,16 +1204,16 @@ static Object SearchRegExpMultiple(Isolate* isolate, Handle<String> subject,
   static const int kMinLengthToCache = 0x1000;
 
   if (subject_length > kMinLengthToCache) {
-    FixedArray last_match_cache;
-    Object cached_answer = RegExpResultsCache::Lookup(
+    Tagged<FixedArray> last_match_cache;
+    Tagged<Object> cached_answer = RegExpResultsCache::Lookup(
         isolate->heap(), *subject, regexp->data(), &last_match_cache,
         RegExpResultsCache::REGEXP_MULTIPLE_INDICES);
-    if (cached_answer.IsFixedArray()) {
+    if (IsFixedArray(cached_answer)) {
       int capture_registers = JSRegExp::RegistersForCaptureCount(capture_count);
       std::unique_ptr<int32_t[]> last_match(new int32_t[capture_registers]);
       int32_t* raw_last_match = last_match.get();
       for (int i = 0; i < capture_registers; i++) {
-        raw_last_match[i] = Smi::ToInt(last_match_cache.get(i));
+        raw_last_match[i] = Smi::ToInt(last_match_cache->get(i));
       }
       Handle<FixedArray> cached_fixed_array =
           Handle<FixedArray>(FixedArray::cast(cached_answer), isolate);
@@ -1255,7 +1269,7 @@ static Object SearchRegExpMultiple(Isolate* isolate, Handle<String> subject,
         // named captures, they are also passed as the last argument.
 
         Handle<Object> maybe_capture_map(regexp->capture_name_map(), isolate);
-        const bool has_named_captures = maybe_capture_map->IsFixedArray();
+        const bool has_named_captures = IsFixedArray(*maybe_capture_map);
 
         const int argc =
             has_named_captures ? 4 + capture_count : 3 + capture_count;
@@ -1318,8 +1332,8 @@ static Object SearchRegExpMultiple(Isolate* isolate, Handle<String> subject,
       for (int i = 0; i < capture_registers; i++) {
         last_match_cache->set(i, Smi::FromInt(last_match[i]));
       }
-      Handle<FixedArray> result_fixed_array =
-          FixedArray::ShrinkOrEmpty(isolate, builder.array(), builder.length());
+      Handle<FixedArray> result_fixed_array = FixedArray::RightTrimOrEmpty(
+          isolate, builder.array(), builder.length());
       // Cache the result and copy the FixedArray into a COW array.
       Handle<FixedArray> copied_fixed_array =
           isolate->factory()->CopyFixedArrayWithMap(
@@ -1376,15 +1390,15 @@ V8_WARN_UNUSED_RESULT MaybeHandle<String> RegExpReplace(
           String);
     }
 
-    if (match_indices_obj->IsNull(isolate)) {
+    if (IsNull(*match_indices_obj, isolate)) {
       if (sticky) regexp->set_last_index(Smi::zero(), SKIP_WRITE_BARRIER);
       return string;
     }
 
     auto match_indices = Handle<RegExpMatchInfo>::cast(match_indices_obj);
 
-    const int start_index = match_indices->Capture(0);
-    const int end_index = match_indices->Capture(1);
+    const int start_index = match_indices->capture(0);
+    const int end_index = match_indices->capture(1);
 
     if (sticky) {
       regexp->set_last_index(Smi::FromInt(end_index), SKIP_WRITE_BARRIER);
@@ -1426,21 +1440,21 @@ V8_WARN_UNUSED_RESULT MaybeHandle<String> RegExpReplace(
 
     if (replace->length() == 0) {
       if (string->IsOneByteRepresentation()) {
-        Object result =
+        Tagged<Object> result =
             StringReplaceGlobalRegExpWithEmptyString<SeqOneByteString>(
                 isolate, string, regexp, last_match_info);
         return handle(String::cast(result), isolate);
       } else {
-        Object result =
+        Tagged<Object> result =
             StringReplaceGlobalRegExpWithEmptyString<SeqTwoByteString>(
                 isolate, string, regexp, last_match_info);
         return handle(String::cast(result), isolate);
       }
     }
 
-    Object result = StringReplaceGlobalRegExpWithString(
+    Tagged<Object> result = StringReplaceGlobalRegExpWithString(
         isolate, string, regexp, replace, last_match_info);
-    if (result.IsString()) {
+    if (IsString(result)) {
       return handle(String::cast(result), isolate);
     } else {
       return MaybeHandle<String>();
@@ -1466,7 +1480,7 @@ RUNTIME_FUNCTION(Runtime_RegExpExecMultiple) {
   subject = String::Flatten(isolate, subject);
   CHECK(regexp->flags() & JSRegExp::kGlobal);
 
-  Object result;
+  Tagged<Object> result;
   if (regexp->capture_count() == 0) {
     result =
         SearchRegExpMultiple<false>(isolate, subject, regexp, last_match_info);
@@ -1486,7 +1500,7 @@ RUNTIME_FUNCTION(Runtime_StringReplaceNonGlobalRegExpWithFunction) {
   Handle<JSReceiver> replace_obj = args.at<JSReceiver>(2);
 
   DCHECK(RegExpUtils::IsUnmodifiedRegExp(isolate, regexp));
-  DCHECK(replace_obj->map().is_callable());
+  DCHECK(replace_obj->map()->is_callable());
 
   Factory* factory = isolate->factory();
   Handle<RegExpMatchInfo> last_match_info = isolate->regexp_last_match_info();
@@ -1516,7 +1530,7 @@ RUNTIME_FUNCTION(Runtime_StringReplaceNonGlobalRegExpWithFunction) {
         RegExp::Exec(isolate, regexp, subject, last_index, last_match_info));
   }
 
-  if (match_indices_obj->IsNull(isolate)) {
+  if (IsNull(*match_indices_obj, isolate)) {
     if (sticky) regexp->set_last_index(Smi::zero(), SKIP_WRITE_BARRIER);
     return *subject;
   }
@@ -1524,8 +1538,8 @@ RUNTIME_FUNCTION(Runtime_StringReplaceNonGlobalRegExpWithFunction) {
   Handle<RegExpMatchInfo> match_indices =
       Handle<RegExpMatchInfo>::cast(match_indices_obj);
 
-  const int index = match_indices->Capture(0);
-  const int end_of_match = match_indices->Capture(1);
+  const int index = match_indices->capture(0);
+  const int end_of_match = match_indices->capture(1);
 
   if (sticky) {
     regexp->set_last_index(Smi::FromInt(end_of_match), SKIP_WRITE_BARRIER);
@@ -1539,15 +1553,15 @@ RUNTIME_FUNCTION(Runtime_StringReplaceNonGlobalRegExpWithFunction) {
   // named captures, they are also passed as the last argument.
 
   // The number of captures plus one for the match.
-  const int m = match_indices->NumberOfCaptureRegisters() / 2;
+  const int m = match_indices->number_of_capture_registers() / 2;
 
   bool has_named_captures = false;
   Handle<FixedArray> capture_map;
   if (m > 1) {
     DCHECK(JSRegExp::TypeSupportsCaptures(regexp->type_tag()));
 
-    Object maybe_capture_map = regexp->capture_name_map();
-    if (maybe_capture_map.IsFixedArray()) {
+    Tagged<Object> maybe_capture_map = regexp->capture_name_map();
+    if (IsFixedArray(maybe_capture_map)) {
       has_named_captures = true;
       capture_map = handle(FixedArray::cast(maybe_capture_map), isolate);
     }
@@ -1604,7 +1618,7 @@ namespace {
 V8_WARN_UNUSED_RESULT MaybeHandle<Object> ToUint32(Isolate* isolate,
                                                    Handle<Object> object,
                                                    uint32_t* out) {
-  if (object->IsUndefined(isolate)) {
+  if (IsUndefined(*object, isolate)) {
     *out = kMaxUInt32;
     return object;
   }
@@ -1620,7 +1634,7 @@ Handle<JSArray> NewJSArrayWithElements(Isolate* isolate,
                                        Handle<FixedArray> elems,
                                        int num_elems) {
   return isolate->factory()->NewJSArrayWithElements(
-      FixedArray::ShrinkOrEmpty(isolate, elems, num_elems));
+      FixedArray::RightTrimOrEmpty(isolate, elems, num_elems));
 }
 
 }  // namespace
@@ -1693,7 +1707,7 @@ RUNTIME_FUNCTION(Runtime_RegExpSplit) {
         isolate, result, RegExpUtils::RegExpExec(isolate, splitter, string,
                                                  factory->undefined_value()));
 
-    if (!result->IsNull(isolate)) return *factory->NewJSArray(0);
+    if (!IsNull(*result, isolate)) return *factory->NewJSArray(0);
 
     Handle<FixedArray> elems = factory->NewFixedArray(1);
     elems->set(0, *string);
@@ -1715,7 +1729,7 @@ RUNTIME_FUNCTION(Runtime_RegExpSplit) {
         isolate, result, RegExpUtils::RegExpExec(isolate, splitter, string,
                                                  factory->undefined_value()));
 
-    if (result->IsNull(isolate)) {
+    if (IsNull(*result, isolate)) {
       string_index = static_cast<uint32_t>(
           RegExpUtils::AdvanceStringIndex(string, string_index, unicode));
       continue;
@@ -1794,7 +1808,7 @@ RUNTIME_FUNCTION(Runtime_RegExpReplaceRT) {
 
   string = String::Flatten(isolate, string);
 
-  const bool functional_replace = replace_obj->IsCallable();
+  const bool functional_replace = IsCallable(*replace_obj);
 
   Handle<String> replace;
   if (!functional_replace) {
@@ -1821,7 +1835,7 @@ RUNTIME_FUNCTION(Runtime_RegExpReplaceRT) {
   ASSIGN_RETURN_FAILURE_ON_EXCEPTION(
       isolate, global_obj,
       JSReceiver::GetProperty(isolate, recv, factory->global_string()));
-  const bool global = global_obj->BooleanValue(isolate);
+  const bool global = Object::BooleanValue(*global_obj, isolate);
 
   bool unicode = false;
   if (global) {
@@ -1829,7 +1843,7 @@ RUNTIME_FUNCTION(Runtime_RegExpReplaceRT) {
     ASSIGN_RETURN_FAILURE_ON_EXCEPTION(
         isolate, unicode_obj,
         JSReceiver::GetProperty(isolate, recv, factory->unicode_string()));
-    unicode = unicode_obj->BooleanValue(isolate);
+    unicode = Object::BooleanValue(*unicode_obj, isolate);
 
     RETURN_FAILURE_ON_EXCEPTION(isolate,
                                 RegExpUtils::SetLastIndex(isolate, recv, 0));
@@ -1843,7 +1857,7 @@ RUNTIME_FUNCTION(Runtime_RegExpReplaceRT) {
         isolate, result, RegExpUtils::RegExpExec(isolate, recv, string,
                                                  factory->undefined_value()));
 
-    if (result->IsNull(isolate)) break;
+    if (IsNull(*result, isolate)) break;
 
     results.emplace_back(result);
     if (!global) break;
@@ -1907,7 +1921,7 @@ RUNTIME_FUNCTION(Runtime_RegExpReplaceRT) {
       ASSIGN_RETURN_FAILURE_ON_EXCEPTION(
           isolate, capture, Object::GetElement(isolate, result, n));
 
-      if (!capture->IsUndefined(isolate)) {
+      if (!IsUndefined(*capture, isolate)) {
         ASSIGN_RETURN_FAILURE_ON_EXCEPTION(isolate, capture,
                                            Object::ToString(isolate, capture));
       }
@@ -1919,7 +1933,7 @@ RUNTIME_FUNCTION(Runtime_RegExpReplaceRT) {
         isolate, groups_obj,
         Object::GetProperty(isolate, result, factory->groups_string()));
 
-    const bool has_named_captures = !groups_obj->IsUndefined(isolate);
+    const bool has_named_captures = !IsUndefined(*groups_obj, isolate);
 
     Handle<String> replacement;
     if (functional_replace) {
@@ -1953,7 +1967,7 @@ RUNTIME_FUNCTION(Runtime_RegExpReplaceRT) {
           isolate, replacement, Object::ToString(isolate, replacement_obj));
     } else {
       DCHECK(!functional_replace);
-      if (!groups_obj->IsUndefined(isolate)) {
+      if (!IsUndefined(*groups_obj, isolate)) {
         ASSIGN_RETURN_FAILURE_ON_EXCEPTION(
             isolate, groups_obj, Object::ToObject(isolate, groups_obj));
       }
@@ -1999,15 +2013,15 @@ RUNTIME_FUNCTION(Runtime_RegExpInitializeAndCompile) {
 RUNTIME_FUNCTION(Runtime_IsRegExp) {
   SealHandleScope shs(isolate);
   DCHECK_EQ(1, args.length());
-  Object obj = args[0];
-  return isolate->heap()->ToBoolean(obj.IsJSRegExp());
+  Tagged<Object> obj = args[0];
+  return isolate->heap()->ToBoolean(IsJSRegExp(obj));
 }
 
 RUNTIME_FUNCTION(Runtime_RegExpStringFromFlags) {
   HandleScope scope(isolate);
   DCHECK_EQ(1, args.length());
   auto regexp = JSRegExp::cast(args[0]);
-  Handle<String> flags = JSRegExp::StringFromFlags(isolate, regexp.flags());
+  Handle<String> flags = JSRegExp::StringFromFlags(isolate, regexp->flags());
   return *flags;
 }
 
