@@ -66,6 +66,7 @@ namespace internal {
   F(AtomicsLoadSharedStructOrArray, 2, 1)              \
   F(AtomicsStoreSharedStructOrArray, 3, 1)             \
   F(AtomicsExchangeSharedStructOrArray, 3, 1)          \
+  F(AtomicsCompareExchangeSharedStructOrArray, 4, 1)   \
   F(AtomicsConditionNumWaitersForTesting, 1, 1)
 
 #define FOR_EACH_INTRINSIC_BIGINT(F, I) \
@@ -99,11 +100,14 @@ namespace internal {
 #define FOR_EACH_INTRINSIC_COLLECTIONS(F, I) \
   F(MapGrow, 1, 1)                           \
   F(MapShrink, 1, 1)                         \
+  F(OrderedHashSetGrow, 2, 1)                \
   F(SetGrow, 1, 1)                           \
   F(SetShrink, 1, 1)                         \
+  F(OrderedHashSetShrink, 1, 1)              \
   F(TheHole, 0, 1)                           \
   F(WeakCollectionDelete, 3, 1)              \
-  F(WeakCollectionSet, 4, 1)
+  F(WeakCollectionSet, 4, 1)                 \
+  F(OrderedHashMapGrow, 2, 1)
 
 #define FOR_EACH_INTRINSIC_COMPILER(F, I)       \
   F(CompileOptimizedOSR, 0, 1)                  \
@@ -180,8 +184,7 @@ namespace internal {
   F(FunctionGetScriptId, 1, 1)             \
   F(FunctionGetScriptSourcePosition, 1, 1) \
   F(FunctionGetSourceCode, 1, 1)           \
-  F(FunctionIsAPIFunction, 1, 1)           \
-  F(IsFunction, 1, 1)
+  F(FunctionIsAPIFunction, 1, 1)
 
 #define FOR_EACH_INTRINSIC_GENERATOR(F, I)    \
   I(AsyncFunctionAwaitCaught, 2, 1)           \
@@ -224,6 +227,7 @@ namespace internal {
   F(DoubleToStringWithRadix, 2, 1)                         \
   F(FatalProcessOutOfMemoryInAllocateRaw, 0, 1)            \
   F(FatalProcessOutOfMemoryInvalidArrayLength, 0, 1)       \
+  F(FatalInvalidSize, 0, 1)                                \
   F(GetAndResetRuntimeCallStats, -1 /* <= 2 */, 1)         \
   F(GetAndResetTurboProfilingData, 0, 1)                   \
   F(GetTemplateObject, 3, 1)                               \
@@ -240,7 +244,7 @@ namespace internal {
   F(NewSyntaxError, 2, 1)                                  \
   F(NewTypeError, -1 /* [1, 4] */, 1)                      \
   F(OrdinaryHasInstance, 2, 1)                             \
-  F(PromoteScheduledException, 0, 1)                       \
+  F(PropagateException, 0, 1)                              \
   F(ReportMessageFromMicrotask, 1, 1)                      \
   F(ReThrow, 1, 1)                                         \
   F(ReThrowWithMessage, 2, 1)                              \
@@ -248,6 +252,7 @@ namespace internal {
   F(PerformMicrotaskCheckpoint, 0, 1)                      \
   F(SharedValueBarrierSlow, 1, 1)                          \
   F(StackGuard, 0, 1)                                      \
+  F(HandleNoHeapWritesInterrupts, 0, 1)                    \
   F(StackGuardWithGap, 1, 1)                               \
   F(Throw, 1, 1)                                           \
   F(ThrowApplyNonFunction, 1, 1)                           \
@@ -275,11 +280,9 @@ namespace internal {
   F(Typeof, 1, 1)                                          \
   F(UnwindAndFindExceptionHandler, 0, 1)
 
-#define FOR_EACH_INTRINSIC_LITERALS(F, I)           \
-  F(CreateArrayLiteral, 4, 1)                       \
-  F(CreateArrayLiteralWithoutAllocationSite, 2, 1)  \
-  F(CreateObjectLiteral, 4, 1)                      \
-  F(CreateObjectLiteralWithoutAllocationSite, 2, 1) \
+#define FOR_EACH_INTRINSIC_LITERALS(F, I) \
+  F(CreateArrayLiteral, 4, 1)             \
+  F(CreateObjectLiteral, 4, 1)            \
   F(CreateRegExpLiteral, 4, 1)
 
 #define FOR_EACH_INTRINSIC_MODULE(F, I)    \
@@ -297,8 +300,7 @@ namespace internal {
   F(NumberToStringSlow, 1, 1)            \
   F(StringParseFloat, 1, 1)              \
   F(StringParseInt, 2, 1)                \
-  F(StringToNumber, 1, 1)                \
-  F(TypedArrayMaxLength, 0, 1)
+  F(StringToNumber, 1, 1)
 
 #define FOR_EACH_INTRINSIC_OBJECT(F, I)                                \
   F(AddDictionaryProperty, 3, 1)                                       \
@@ -434,6 +436,7 @@ namespace internal {
   F(DeleteLookupSlot, 1, 1)                        \
   F(LoadLookupSlot, 1, 1)                          \
   F(LoadLookupSlotInsideTypeof, 1, 1)              \
+  F(LoadLookupSlotForCall_Baseline, 2, 1)          \
                                                    \
   F(NewClosure, 2, 1)                              \
   F(NewClosure_Tenured, 2, 1)                      \
@@ -500,6 +503,7 @@ namespace internal {
   F(ArraySpeciesProtector, 0, 1)              \
   F(BaselineOsr, -1, 1)                       \
   F(BenchMaglev, 2, 1)                        \
+  F(BenchTurbofan, 2, 1)                      \
   F(ClearFunctionFeedback, 1, 1)              \
   F(ClearMegamorphicStubCache, 0, 1)          \
   F(CompleteInobjectSlackTracking, 1, 1)      \
@@ -510,7 +514,9 @@ namespace internal {
   F(ConstructThinString, 1, 1)                \
   F(CurrentFrameIsTurbofan, 0, 1)             \
   F(DebugPrint, -1, 1)                        \
+  F(DebugPrintFloat, 5, 1)                    \
   F(DebugPrintPtr, 1, 1)                      \
+  F(DebugPrintWord, 5, 1)                     \
   F(DebugTrace, 0, 1)                         \
   F(DebugTrackRetainingPath, -1, 1)           \
   F(DeoptimizeFunction, 1, 1)                 \
@@ -522,6 +528,7 @@ namespace internal {
   F(FinalizeOptimization, 0, 1)               \
   F(ForceFlush, 1, 1)                         \
   F(GetCallable, 0, 1)                        \
+  F(GetFunctionForCurrentFrame, 0, 1)         \
   F(GetInitializerFunction, 1, 1)             \
   F(GetOptimizationStatus, 1, 1)              \
   F(GetUndetectable, 0, 1)                    \
@@ -561,6 +568,7 @@ namespace internal {
   F(IsConcatSpreadableProtector, 0, 1)        \
   F(IsConcurrentRecompilationSupported, 0, 1) \
   F(IsDictPropertyConstTrackingEnabled, 0, 1) \
+  F(IsEfficiencyModeEnabled, 0, 1)            \
   F(IsInPlaceInternalizableString, 1, 1)      \
   F(IsInternalizedString, 1, 1)               \
   F(IsMaglevEnabled, 0, 1)                    \
@@ -571,7 +579,10 @@ namespace internal {
   F(MapIteratorProtector, 0, 1)               \
   F(NeverOptimizeFunction, 1, 1)              \
   F(NewRegExpWithBacktrackLimit, 3, 1)        \
+  F(NoElementsProtector, 0, 1)                \
   F(NotifyContextDisposed, 0, 1)              \
+  F(NotifyIsolateForeground, 0, 1)            \
+  F(NotifyIsolateBackground, 0, 1)            \
   F(OptimizeMaglevOnNextCall, 1, 1)           \
   F(OptimizeFunctionOnNextCall, -1, 1)        \
   F(OptimizeOsr, -1, 1)                       \
@@ -589,9 +600,11 @@ namespace internal {
   F(ScheduleGCInStackCheck, 0, 1)             \
   F(SerializeDeserializeNow, 0, 1)            \
   F(SetAllocationTimeout, -1 /* 2 || 3 */, 1) \
+  F(SetBatterySaverMode, 1, 1)                \
   F(SetForceSlowPath, 1, 1)                   \
   F(SetIteratorProtector, 0, 1)               \
   F(SharedGC, 0, 1)                           \
+  F(ShareObject, 1, 1)                        \
   F(SimulateNewspaceFull, 0, 1)               \
   F(StringIteratorProtector, 0, 1)            \
   F(SystemBreak, 0, 1)                        \
@@ -601,7 +614,8 @@ namespace internal {
   F(TurbofanStaticAssert, 1, 1)               \
   F(TypedArraySpeciesProtector, 0, 1)         \
   F(WaitForBackgroundOptimization, 0, 1)      \
-  I(DeoptimizeNow, 0, 1)
+  I(DeoptimizeNow, 0, 1)                      \
+  F(LeakHole, 0, 1)
 
 #define FOR_EACH_INTRINSIC_TYPEDARRAY(F, I)    \
   F(ArrayBufferDetach, -1, 1)                  \
@@ -615,16 +629,19 @@ namespace internal {
 #define FOR_EACH_INTRINSIC_WASM(F, I)         \
   F(ThrowBadSuspenderError, 0, 1)             \
   F(ThrowWasmError, 1, 1)                     \
+  F(TrapHandlerThrowWasmError, 0, 1)          \
   F(ThrowWasmStackOverflow, 0, 1)             \
   F(WasmI32AtomicWait, 4, 1)                  \
   F(WasmI64AtomicWait, 5, 1)                  \
-  F(WasmAtomicNotify, 3, 1)                   \
   F(WasmMemoryGrow, 2, 1)                     \
   F(WasmStackGuard, 0, 1)                     \
   F(WasmThrow, 2, 1)                          \
   F(WasmReThrow, 1, 1)                        \
   F(WasmThrowJSTypeError, 0, 1)               \
   F(WasmThrowTypeError, 2, 1)                 \
+  F(WasmThrowRangeError, 1, 1)                \
+  F(WasmThrowDataViewTypeError, 2, 1)         \
+  F(WasmThrowDataViewDetachedError, 1, 1)     \
   F(WasmRefFunc, 1, 1)                        \
   F(WasmInternalFunctionCreateExternal, 1, 1) \
   F(WasmFunctionTableGet, 3, 1)               \
@@ -634,17 +651,22 @@ namespace internal {
   F(WasmTableGrow, 3, 1)                      \
   F(WasmTableFill, 5, 1)                      \
   F(WasmJSToWasmObject, 2, 1)                 \
+  F(WasmGenericJSToWasmObject, 3, 1)          \
+  F(WasmGenericWasmToJSObject, 1, 1)          \
   F(WasmCompileLazy, 2, 1)                    \
   F(WasmAllocateFeedbackVector, 3, 1)         \
-  F(WasmCompileWrapper, 2, 1)                 \
+  F(WasmCompileWrapper, 1, 1)                 \
+  F(TierUpJSToJSWrapper, 1, 1)                \
+  F(IsWasmExternalFunction, 1, 1)             \
+  F(TierUpWasmToJSWrapper, 1, 1)              \
   F(WasmTriggerTierUp, 1, 1)                  \
   F(WasmDebugBreak, 0, 1)                     \
   F(WasmArrayCopy, 5, 1)                      \
   F(WasmArrayNewSegment, 5, 1)                \
   F(WasmArrayInitSegment, 6, 1)               \
   F(WasmAllocateSuspender, 0, 1)              \
-  F(WasmSyncStackLimit, 0, 1)                 \
-  F(WasmCreateResumePromise, 2, 1)            \
+  F(WasmCastToSpecialPrimitiveArray, 2, 1)    \
+  F(WasmStringNewSegmentWtf8, 4, 1)           \
   F(WasmStringNewWtf8, 5, 1)                  \
   F(WasmStringNewWtf8Array, 4, 1)             \
   F(WasmStringNewWtf16, 4, 1)                 \
@@ -659,38 +681,44 @@ namespace internal {
   F(WasmStringViewWtf8Encode, 6, 1)           \
   F(WasmStringViewWtf8Slice, 3, 1)            \
   F(WasmStringFromCodePoint, 1, 1)            \
-  F(WasmStringHash, 1, 1)
+  F(WasmStringHash, 1, 1)                     \
+  F(WasmSubstring, 3, 1)
 
-#define FOR_EACH_INTRINSIC_WASM_TEST(F, I) \
-  F(DeserializeWasmModule, 2, 1)           \
-  F(DisallowWasmCodegen, 1, 1)             \
-  F(FlushWasmCode, 0, 1)                   \
-  F(FreezeWasmLazyCompilation, 1, 1)       \
-  F(GetWasmExceptionTagId, 2, 1)           \
-  F(GetWasmExceptionValues, 1, 1)          \
-  F(GetWasmRecoveredTrapCount, 0, 1)       \
-  F(IsAsmWasmCode, 1, 1)                   \
-  F(IsLiftoffFunction, 1, 1)               \
-  F(IsTurboFanFunction, 1, 1)              \
-  F(IsWasmDebugFunction, 1, 1)             \
-  F(IsUncompiledWasmFunction, 1, 1)        \
-  F(IsThreadInWasm, 0, 1)                  \
-  F(IsWasmCode, 1, 1)                      \
-  F(IsWasmTrapHandlerEnabled, 0, 1)        \
-  F(IsWasmPartialOOBWriteNoop, 0, 1)       \
-  F(SerializeWasmModule, 1, 1)             \
-  F(SetWasmCompileControls, 2, 1)          \
-  F(SetWasmInstantiateControls, 0, 1)      \
-  F(SetWasmGCEnabled, 1, 1)                \
-  F(WasmCompiledExportWrappersCount, 0, 1) \
-  F(WasmGetNumberOfInstances, 1, 1)        \
-  F(WasmNumCodeSpaces, 1, 1)               \
-  F(WasmEnterDebugging, 0, 1)              \
-  F(WasmLeaveDebugging, 0, 1)              \
-  F(WasmTierUpFunction, 1, 1)              \
-  F(WasmTraceEnter, 0, 1)                  \
-  F(WasmTraceExit, 1, 1)                   \
-  F(WasmTraceMemory, 1, 1)
+#define FOR_EACH_INTRINSIC_WASM_TEST(F, I)  \
+  F(CountUnoptimizedWasmToJSWrapper, 1, 1)  \
+  F(DeserializeWasmModule, 2, 1)            \
+  F(DisallowWasmCodegen, 1, 1)              \
+  F(FlushWasmCode, 0, 1)                    \
+  F(FreezeWasmLazyCompilation, 1, 1)        \
+  F(GetWasmExceptionTagId, 2, 1)            \
+  F(GetWasmExceptionValues, 1, 1)           \
+  F(GetWasmRecoveredTrapCount, 0, 1)        \
+  F(HasUnoptimizedWasmToJSWrapper, 1, 1)    \
+  F(HasUnoptimizedJSToJSWrapper, 1, 1)      \
+  F(IsAsmWasmCode, 1, 1)                    \
+  F(IsLiftoffFunction, 1, 1)                \
+  F(IsTurboFanFunction, 1, 1)               \
+  F(IsWasmDebugFunction, 1, 1)              \
+  F(IsUncompiledWasmFunction, 1, 1)         \
+  F(IsThreadInWasm, 0, 1)                   \
+  F(IsWasmCode, 1, 1)                       \
+  F(IsWasmTrapHandlerEnabled, 0, 1)         \
+  F(IsWasmPartialOOBWriteNoop, 0, 1)        \
+  F(SerializeWasmModule, 1, 1)              \
+  F(SetWasmCompileControls, 2, 1)           \
+  F(SetWasmInstantiateControls, 0, 1)       \
+  F(SetWasmGCEnabled, 1, 1)                 \
+  F(WasmCompiledExportWrappersCount, 0, 1)  \
+  F(WasmGetNumberOfInstances, 1, 1)         \
+  F(WasmNumCodeSpaces, 1, 1)                \
+  F(WasmEnterDebugging, 0, 1)               \
+  F(WasmLeaveDebugging, 0, 1)               \
+  F(WasmSwitchToTheCentralStackCount, 0, 1) \
+  F(WasmTierUpFunction, 1, 1)               \
+  F(WasmTraceEnter, 0, 1)                   \
+  F(WasmTraceExit, 1, 1)                    \
+  F(WasmTraceMemory, 1, 1)                  \
+  F(CheckIsOnCentralStack, 0, 1)
 
 #define FOR_EACH_INTRINSIC_WEAKREF(F, I)                             \
   F(JSFinalizationRegistryRegisterWeakCellWithUnregisterToken, 4, 1) \
@@ -728,6 +756,7 @@ namespace internal {
   F(DefineNamedOwnIC_Miss, 5, 1)             \
   F(StoreInArrayLiteralIC_Slow, 5, 1)        \
   F(StorePropertyWithInterceptor, 5, 1)      \
+  F(CloneObjectIC_Slow, 2, 1)                \
   F(CloneObjectIC_Miss, 4, 1)                \
   F(KeyedHasIC_Miss, 4, 1)                   \
   F(HasElementWithInterceptor, 2, 1)
@@ -966,8 +995,6 @@ V8_EXPORT_PRIVATE std::ostream& operator<<(std::ostream&, Runtime::FunctionId);
 
 using AllocateDoubleAlignFlag = base::BitField<bool, 0, 1>;
 
-using AllowLargeObjectAllocationFlag = base::BitField<bool, 1, 1>;
-
 // A set of bits returned by Runtime_GetOptimizationStatus.
 // These bits must be in sync with bits defined in test/mjsunit/mjsunit.js
 enum class OptimizationStatus {
