@@ -21,13 +21,14 @@
 #include "src/base/macros.h"
 #include "src/base/vector.h"
 #include "src/builtins/builtins.h"
+#include "src/codegen/safepoint-table.h"
 #include "src/codegen/source-position.h"
-#include "src/common/code-memory-access.h"
 #include "src/handles/handles.h"
 #include "src/tasks/operations-barrier.h"
 #include "src/trap-handler/trap-handler.h"
 #include "src/wasm/compilation-environment.h"
 #include "src/wasm/wasm-features.h"
+#include "src/wasm/wasm-import-wrapper-cache.h"
 #include "src/wasm/wasm-limits.h"
 #include "src/wasm/wasm-module-sourcemap.h"
 #include "src/wasm/wasm-tier.h"
@@ -51,114 +52,6 @@ class WasmImportWrapperCache;
 struct WasmModule;
 enum class WellKnownImport : uint8_t;
 
-// Convenience macro listing all wasm runtime stubs. Note that the first few
-// elements of the list coincide with {compiler::TrapId}, order matters.
-#define WASM_RUNTIME_STUB_LIST(V, VTRAP) \
-  FOREACH_WASM_TRAPREASON(VTRAP)         \
-  V(WasmCompileLazy)                     \
-  V(WasmTriggerTierUp)                   \
-  V(WasmLiftoffFrameSetup)               \
-  V(WasmDebugBreak)                      \
-  V(WasmInt32ToHeapNumber)               \
-  V(WasmTaggedNonSmiToInt32)             \
-  V(WasmFloat32ToNumber)                 \
-  V(WasmFloat64ToNumber)                 \
-  V(WasmTaggedToFloat64)                 \
-  V(WasmAllocateJSArray)                 \
-  V(WasmAtomicNotify)                    \
-  V(WasmI32AtomicWait)                   \
-  V(WasmI64AtomicWait)                   \
-  V(WasmGetOwnProperty)                  \
-  V(WasmRefFunc)                         \
-  V(WasmInternalFunctionCreateExternal)  \
-  V(WasmMemoryGrow)                      \
-  V(WasmTableInit)                       \
-  V(WasmTableCopy)                       \
-  V(WasmTableFill)                       \
-  V(WasmTableGrow)                       \
-  V(WasmTableGet)                        \
-  V(WasmTableSet)                        \
-  V(WasmTableGetFuncRef)                 \
-  V(WasmTableSetFuncRef)                 \
-  V(WasmStackGuard)                      \
-  V(WasmStackOverflow)                   \
-  V(WasmAllocateFixedArray)              \
-  V(WasmThrow)                           \
-  V(WasmRethrow)                         \
-  V(WasmRethrowExplicitContext)          \
-  V(WasmTraceEnter)                      \
-  V(WasmTraceExit)                       \
-  V(WasmTraceMemory)                     \
-  V(BigIntToI32Pair)                     \
-  V(BigIntToI64)                         \
-  V(CallRefIC)                           \
-  V(DoubleToI)                           \
-  V(I32PairToBigInt)                     \
-  V(I64ToBigInt)                         \
-  V(RecordWriteSaveFP)                   \
-  V(RecordWriteIgnoreFP)                 \
-  V(ToNumber)                            \
-  IF_TSAN(V, TSANRelaxedStore8IgnoreFP)  \
-  IF_TSAN(V, TSANRelaxedStore8SaveFP)    \
-  IF_TSAN(V, TSANRelaxedStore16IgnoreFP) \
-  IF_TSAN(V, TSANRelaxedStore16SaveFP)   \
-  IF_TSAN(V, TSANRelaxedStore32IgnoreFP) \
-  IF_TSAN(V, TSANRelaxedStore32SaveFP)   \
-  IF_TSAN(V, TSANRelaxedStore64IgnoreFP) \
-  IF_TSAN(V, TSANRelaxedStore64SaveFP)   \
-  IF_TSAN(V, TSANSeqCstStore8IgnoreFP)   \
-  IF_TSAN(V, TSANSeqCstStore8SaveFP)     \
-  IF_TSAN(V, TSANSeqCstStore16IgnoreFP)  \
-  IF_TSAN(V, TSANSeqCstStore16SaveFP)    \
-  IF_TSAN(V, TSANSeqCstStore32IgnoreFP)  \
-  IF_TSAN(V, TSANSeqCstStore32SaveFP)    \
-  IF_TSAN(V, TSANSeqCstStore64IgnoreFP)  \
-  IF_TSAN(V, TSANSeqCstStore64SaveFP)    \
-  IF_TSAN(V, TSANRelaxedLoad32IgnoreFP)  \
-  IF_TSAN(V, TSANRelaxedLoad32SaveFP)    \
-  IF_TSAN(V, TSANRelaxedLoad64IgnoreFP)  \
-  IF_TSAN(V, TSANRelaxedLoad64SaveFP)    \
-  V(WasmAllocateArray_Uninitialized)     \
-  V(WasmArrayCopy)                       \
-  V(WasmArrayCopyWithChecks)             \
-  V(WasmArrayNewSegment)                 \
-  V(WasmArrayInitSegment)                \
-  V(WasmAllocateStructWithRtt)           \
-  V(WasmSubtypeCheck)                    \
-  V(WasmOnStackReplace)                  \
-  V(WasmSuspend)                         \
-  V(WasmStringNewWtf8)                   \
-  V(WasmStringNewWtf16)                  \
-  V(WasmStringConst)                     \
-  V(WasmStringMeasureUtf8)               \
-  V(WasmStringMeasureWtf8)               \
-  V(WasmStringEncodeWtf8)                \
-  V(WasmStringEncodeWtf16)               \
-  V(WasmStringConcat)                    \
-  V(WasmStringEqual)                     \
-  V(WasmStringIsUSVSequence)             \
-  V(WasmStringAsWtf16)                   \
-  V(WasmStringViewWtf16GetCodeUnit)      \
-  V(WasmStringViewWtf16Encode)           \
-  V(WasmStringViewWtf16Slice)            \
-  V(WasmStringNewWtf8Array)              \
-  V(WasmStringNewWtf16Array)             \
-  V(WasmStringEncodeWtf8Array)           \
-  V(WasmStringEncodeWtf16Array)          \
-  V(WasmStringAsWtf8)                    \
-  V(WasmStringViewWtf8Advance)           \
-  V(WasmStringViewWtf8Encode)            \
-  V(WasmStringViewWtf8Slice)             \
-  V(WasmStringAsIter)                    \
-  V(WasmStringViewIterNext)              \
-  V(WasmStringViewIterAdvance)           \
-  V(WasmStringViewIterRewind)            \
-  V(WasmStringViewIterSlice)             \
-  V(StringCompare)                       \
-  V(WasmStringFromCodePoint)             \
-  V(WasmStringHash)                      \
-  V(WasmExternInternalize)
-
 // Sorted, disjoint and non-overlapping memory regions. A region is of the
 // form [start, end). So there's no [start, end), [end, other_end),
 // because that should have been reduced to [start, other_end).
@@ -193,101 +86,89 @@ class V8_EXPORT_PRIVATE WasmCode final {
  public:
   enum Kind { kWasmFunction, kWasmToCapiWrapper, kWasmToJsWrapper, kJumpTable };
 
-  // Each runtime stub is identified by an id. This id is used to reference the
-  // stub via {RelocInfo::WASM_STUB_CALL} and gets resolved during relocation.
-  enum RuntimeStubId {
-#define DEF_ENUM(Name) k##Name,
-#define DEF_ENUM_TRAP(Name) kThrowWasm##Name,
-    WASM_RUNTIME_STUB_LIST(DEF_ENUM, DEF_ENUM_TRAP)
-#undef DEF_ENUM_TRAP
-#undef DEF_ENUM
-        kRuntimeStubCount
-  };
-
-  static constexpr RuntimeStubId GetRecordWriteStub(SaveFPRegsMode fp_mode) {
+  static constexpr Builtin GetRecordWriteBuiltin(SaveFPRegsMode fp_mode) {
     switch (fp_mode) {
       case SaveFPRegsMode::kIgnore:
-        return RuntimeStubId::kRecordWriteIgnoreFP;
+        return Builtin::kRecordWriteIgnoreFP;
       case SaveFPRegsMode::kSave:
-        return RuntimeStubId::kRecordWriteSaveFP;
+        return Builtin::kRecordWriteSaveFP;
     }
   }
 
 #ifdef V8_IS_TSAN
-  static RuntimeStubId GetTSANStoreStub(SaveFPRegsMode fp_mode, int size,
-                                        std::memory_order order) {
+  static Builtin GetTSANStoreBuiltin(SaveFPRegsMode fp_mode, int size,
+                                     std::memory_order order) {
     if (order == std::memory_order_relaxed) {
       if (size == kInt8Size) {
         return fp_mode == SaveFPRegsMode::kIgnore
-                   ? RuntimeStubId::kTSANRelaxedStore8IgnoreFP
-                   : RuntimeStubId::kTSANRelaxedStore8SaveFP;
+                   ? Builtin::kTSANRelaxedStore8IgnoreFP
+                   : Builtin::kTSANRelaxedStore8SaveFP;
       } else if (size == kInt16Size) {
         return fp_mode == SaveFPRegsMode::kIgnore
-                   ? RuntimeStubId::kTSANRelaxedStore16IgnoreFP
-                   : RuntimeStubId::kTSANRelaxedStore16SaveFP;
+                   ? Builtin::kTSANRelaxedStore16IgnoreFP
+                   : Builtin::kTSANRelaxedStore16SaveFP;
       } else if (size == kInt32Size) {
         return fp_mode == SaveFPRegsMode::kIgnore
-                   ? RuntimeStubId::kTSANRelaxedStore32IgnoreFP
-                   : RuntimeStubId::kTSANRelaxedStore32SaveFP;
+                   ? Builtin::kTSANRelaxedStore32IgnoreFP
+                   : Builtin::kTSANRelaxedStore32SaveFP;
       } else {
         CHECK_EQ(size, kInt64Size);
         return fp_mode == SaveFPRegsMode::kIgnore
-                   ? RuntimeStubId::kTSANRelaxedStore64IgnoreFP
-                   : RuntimeStubId::kTSANRelaxedStore64SaveFP;
+                   ? Builtin::kTSANRelaxedStore64IgnoreFP
+                   : Builtin::kTSANRelaxedStore64SaveFP;
       }
     } else {
       DCHECK_EQ(order, std::memory_order_seq_cst);
       if (size == kInt8Size) {
         return fp_mode == SaveFPRegsMode::kIgnore
-                   ? RuntimeStubId::kTSANSeqCstStore8IgnoreFP
-                   : RuntimeStubId::kTSANSeqCstStore8SaveFP;
+                   ? Builtin::kTSANSeqCstStore8IgnoreFP
+                   : Builtin::kTSANSeqCstStore8SaveFP;
       } else if (size == kInt16Size) {
         return fp_mode == SaveFPRegsMode::kIgnore
-                   ? RuntimeStubId::kTSANSeqCstStore16IgnoreFP
-                   : RuntimeStubId::kTSANSeqCstStore16SaveFP;
+                   ? Builtin::kTSANSeqCstStore16IgnoreFP
+                   : Builtin::kTSANSeqCstStore16SaveFP;
       } else if (size == kInt32Size) {
         return fp_mode == SaveFPRegsMode::kIgnore
-                   ? RuntimeStubId::kTSANSeqCstStore32IgnoreFP
-                   : RuntimeStubId::kTSANSeqCstStore32SaveFP;
+                   ? Builtin::kTSANSeqCstStore32IgnoreFP
+                   : Builtin::kTSANSeqCstStore32SaveFP;
       } else {
         CHECK_EQ(size, kInt64Size);
         return fp_mode == SaveFPRegsMode::kIgnore
-                   ? RuntimeStubId::kTSANSeqCstStore64IgnoreFP
-                   : RuntimeStubId::kTSANSeqCstStore64SaveFP;
+                   ? Builtin::kTSANSeqCstStore64IgnoreFP
+                   : Builtin::kTSANSeqCstStore64SaveFP;
       }
     }
   }
 
-  static RuntimeStubId GetTSANRelaxedLoadStub(SaveFPRegsMode fp_mode,
-                                              int size) {
+  static Builtin GetTSANRelaxedLoadBuiltin(SaveFPRegsMode fp_mode, int size) {
     if (size == kInt32Size) {
       return fp_mode == SaveFPRegsMode::kIgnore
-                 ? RuntimeStubId::kTSANRelaxedLoad32IgnoreFP
-                 : RuntimeStubId::kTSANRelaxedLoad32SaveFP;
+                 ? Builtin::kTSANRelaxedLoad32IgnoreFP
+                 : Builtin::kTSANRelaxedLoad32SaveFP;
     } else {
       CHECK_EQ(size, kInt64Size);
       return fp_mode == SaveFPRegsMode::kIgnore
-                 ? RuntimeStubId::kTSANRelaxedLoad64IgnoreFP
-                 : RuntimeStubId::kTSANRelaxedLoad64SaveFP;
+                 ? Builtin::kTSANRelaxedLoad64IgnoreFP
+                 : Builtin::kTSANRelaxedLoad64SaveFP;
     }
   }
 #endif  // V8_IS_TSAN
 
-  base::Vector<byte> instructions() const {
+  base::Vector<uint8_t> instructions() const {
     return base::VectorOf(instructions_,
                           static_cast<size_t>(instructions_size_));
   }
   Address instruction_start() const {
     return reinterpret_cast<Address>(instructions_);
   }
-  base::Vector<const byte> reloc_info() const {
+  base::Vector<const uint8_t> reloc_info() const {
     return {protected_instructions_data().end(),
             static_cast<size_t>(reloc_info_size_)};
   }
-  base::Vector<const byte> source_positions() const {
+  base::Vector<const uint8_t> source_positions() const {
     return {reloc_info().end(), static_cast<size_t>(source_positions_size_)};
   }
-  base::Vector<const byte> inlining_positions() const {
+  base::Vector<const uint8_t> inlining_positions() const {
     return {source_positions().end(),
             static_cast<size_t>(inlining_positions_size_)};
   }
@@ -343,6 +224,8 @@ class V8_EXPORT_PRIVATE WasmCode final {
         protected_instructions_data());
   }
 
+  bool IsProtectedInstruction(Address pc);
+
   void Validate() const;
   void Print(const char* name = nullptr) const;
   void MaybePrint() const;
@@ -357,10 +240,10 @@ class V8_EXPORT_PRIVATE WasmCode final {
   ~WasmCode();
 
   void IncRef() {
-    int old_val = ref_count_.fetch_add(1, std::memory_order_acq_rel);
+    [[maybe_unused]] int old_val =
+        ref_count_.fetch_add(1, std::memory_order_acq_rel);
     DCHECK_LE(1, old_val);
     DCHECK_GT(kMaxInt, old_val);
-    USE(old_val);
   }
 
   // Decrement the ref count. Returns whether this code becomes dead and needs
@@ -380,9 +263,9 @@ class V8_EXPORT_PRIVATE WasmCode final {
   // Decrement the ref count on code that is known to be in use (i.e. the ref
   // count cannot drop to zero here).
   void DecRefOnLiveCode() {
-    int old_count = ref_count_.fetch_sub(1, std::memory_order_acq_rel);
+    [[maybe_unused]] int old_count =
+        ref_count_.fetch_sub(1, std::memory_order_acq_rel);
     DCHECK_LE(2, old_count);
-    USE(old_count);
   }
 
   // Decrement the ref count on code that is known to be dead, even though there
@@ -422,14 +305,14 @@ class V8_EXPORT_PRIVATE WasmCode final {
   friend class NativeModule;
 
   WasmCode(NativeModule* native_module, int index,
-           base::Vector<byte> instructions, int stack_slots,
+           base::Vector<uint8_t> instructions, int stack_slots,
            uint32_t tagged_parameter_slots, int safepoint_table_offset,
            int handler_table_offset, int constant_pool_offset,
            int code_comments_offset, int unpadded_binary_size,
-           base::Vector<const byte> protected_instructions_data,
-           base::Vector<const byte> reloc_info,
-           base::Vector<const byte> source_position_table,
-           base::Vector<const byte> inlining_positions, Kind kind,
+           base::Vector<const uint8_t> protected_instructions_data,
+           base::Vector<const uint8_t> reloc_info,
+           base::Vector<const uint8_t> source_position_table,
+           base::Vector<const uint8_t> inlining_positions, Kind kind,
            ExecutionTier tier, ForDebugging for_debugging,
            bool frame_has_feedback_slot = false)
       : native_module_(native_module),
@@ -459,8 +342,8 @@ class V8_EXPORT_PRIVATE WasmCode final {
     DCHECK_LE(constant_pool_offset, unpadded_binary_size);
   }
 
-  std::unique_ptr<const byte[]> ConcatenateBytes(
-      std::initializer_list<base::Vector<const byte>>);
+  std::unique_ptr<const uint8_t[]> ConcatenateBytes(
+      std::initializer_list<base::Vector<const uint8_t>>);
 
   // Tries to get a reasonable name. Lazily looks up the name section, and falls
   // back to the function index. Return value is guaranteed to not be empty.
@@ -488,13 +371,13 @@ class V8_EXPORT_PRIVATE WasmCode final {
   V8_NOINLINE bool DecRefOnPotentiallyDeadCode();
 
   NativeModule* const native_module_ = nullptr;
-  byte* const instructions_;
+  uint8_t* const instructions_;
   // {meta_data_} contains several byte vectors concatenated into one:
   //  - protected instructions data of size {protected_instructions_size_}
   //  - relocation info of size {reloc_info_size_}
   //  - source positions of size {source_positions_size_}
   // Note that the protected instructions come first to ensure alignment.
-  std::unique_ptr<const byte[]> meta_data_;
+  std::unique_ptr<const uint8_t[]> meta_data_;
   const int instructions_size_;
   const int reloc_info_size_;
   const int source_positions_size_;
@@ -561,13 +444,13 @@ class WasmCodeAllocator {
 
   // Allocate code space. Returns a valid buffer or fails with OOM (crash).
   // Hold the {NativeModule}'s {allocation_mutex_} when calling this method.
-  base::Vector<byte> AllocateForCode(NativeModule*, size_t size);
+  base::Vector<uint8_t> AllocateForCode(NativeModule*, size_t size);
 
   // Allocate code space within a specific region. Returns a valid buffer or
   // fails with OOM (crash).
   // Hold the {NativeModule}'s {allocation_mutex_} when calling this method.
-  base::Vector<byte> AllocateForCodeInRegion(NativeModule*, size_t size,
-                                             base::AddressRegion);
+  base::Vector<uint8_t> AllocateForCodeInRegion(NativeModule*, size_t size,
+                                                base::AddressRegion);
 
   // Free memory pages of all given code objects. Used for wasm code GC.
   // Hold the {NativeModule}'s {allocation_mutex_} when calling this method.
@@ -605,7 +488,7 @@ class WasmCodeAllocator {
 class V8_EXPORT_PRIVATE NativeModule final {
  public:
 #if V8_TARGET_ARCH_X64 || V8_TARGET_ARCH_S390X || V8_TARGET_ARCH_ARM64 || \
-    V8_TARGET_ARCH_PPC64
+    V8_TARGET_ARCH_PPC64 || V8_TARGET_ARCH_LOONG64 || V8_TARGET_ARCH_RISCV64
   static constexpr bool kNeedsFarJumpsBetweenCodeSpaces = true;
 #else
   static constexpr bool kNeedsFarJumpsBetweenCodeSpaces = false;
@@ -621,8 +504,8 @@ class V8_EXPORT_PRIVATE NativeModule final {
   std::unique_ptr<WasmCode> AddCode(
       int index, const CodeDesc& desc, int stack_slots,
       uint32_t tagged_parameter_slots,
-      base::Vector<const byte> protected_instructions,
-      base::Vector<const byte> source_position_table, WasmCode::Kind kind,
+      base::Vector<const uint8_t> protected_instructions,
+      base::Vector<const uint8_t> source_position_table, WasmCode::Kind kind,
       ExecutionTier tier, ForDebugging for_debugging);
 
   // {PublishCode} makes the code available to the system by entering it into
@@ -652,14 +535,14 @@ class V8_EXPORT_PRIVATE NativeModule final {
       size_t total_code_size);
 
   std::unique_ptr<WasmCode> AddDeserializedCode(
-      int index, base::Vector<byte> instructions, int stack_slots,
+      int index, base::Vector<uint8_t> instructions, int stack_slots,
       uint32_t tagged_parameter_slots, int safepoint_table_offset,
       int handler_table_offset, int constant_pool_offset,
       int code_comments_offset, int unpadded_binary_size,
-      base::Vector<const byte> protected_instructions_data,
-      base::Vector<const byte> reloc_info,
-      base::Vector<const byte> source_position_table,
-      base::Vector<const byte> inlining_positions, WasmCode::Kind kind,
+      base::Vector<const uint8_t> protected_instructions_data,
+      base::Vector<const uint8_t> reloc_info,
+      base::Vector<const uint8_t> source_position_table,
+      base::Vector<const uint8_t> inlining_positions, WasmCode::Kind kind,
       ExecutionTier tier);
 
   // Adds anonymous code for testing purposes.
@@ -700,10 +583,9 @@ class V8_EXPORT_PRIVATE NativeModule final {
   Address GetNearCallTargetForFunction(uint32_t func_index,
                                        const JumpTablesRef&) const;
 
-  // Get a runtime stub entry (which is a far jump table slot) in the jump table
-  // previously looked up via {FindJumpTablesForRegionLocked}.
-  Address GetNearRuntimeStubEntry(WasmCode::RuntimeStubId index,
-                                  const JumpTablesRef&) const;
+  // Get the slot offset in the far jump table that jumps to the given builtin.
+  Address GetJumpTableEntryForBuiltin(Builtin builtin,
+                                      const JumpTablesRef&) const;
 
   // Reverse lookup from a given call target (which must be a jump table slot)
   // to a function index.
@@ -713,7 +595,10 @@ class V8_EXPORT_PRIVATE NativeModule final {
   // on the fly, and bypass the instance builder pipeline.
   void ReserveCodeTableForTesting(uint32_t max_functions);
 
-  void LogWasmCodes(Isolate*, Script);
+  // Log all owned code in the given isolate, using the given script as the
+  // containing script. Use this after transferring the module to a new isolate
+  // or when enabling a component that needs all code to be logged (profiler).
+  void LogWasmCodes(Isolate*, Tagged<Script>);
 
   CompilationState* compilation_state() const {
     return compilation_state_.get();
@@ -730,7 +615,6 @@ class V8_EXPORT_PRIVATE NativeModule final {
   uint32_t num_imported_functions() const {
     return module_->num_imported_functions;
   }
-  BoundsCheckStrategy bounds_checks() const { return bounds_checks_; }
   void set_lazy_compile_frozen(bool frozen) { lazy_compile_frozen_ = frozen; }
   bool lazy_compile_frozen() const { return lazy_compile_frozen_; }
   base::Vector<const uint8_t> wire_bytes() const {
@@ -782,8 +666,8 @@ class V8_EXPORT_PRIVATE NativeModule final {
   // Similar to above, scheduling a repeated task to write out PGO data is only
   // needed once per module, not per instantiation.
   bool ShouldPgoDataBeWritten() {
-    return should_pgo_data_be_written.exchange(false,
-                                               std::memory_order_relaxed);
+    return should_pgo_data_be_written_.exchange(false,
+                                                std::memory_order_relaxed);
   }
 
   bool HasWireBytes() const {
@@ -798,15 +682,16 @@ class V8_EXPORT_PRIVATE NativeModule final {
 
   WasmCode* Lookup(Address) const;
 
-  WasmImportWrapperCache* import_wrapper_cache() const {
-    return import_wrapper_cache_.get();
+  WasmImportWrapperCache* import_wrapper_cache() {
+    return &import_wrapper_cache_;
   }
 
-  const WasmFeatures& enabled_features() const { return enabled_features_; }
+  WasmFeatures enabled_features() const { return enabled_features_; }
+  CompileTimeImports compile_imports() const { return compile_imports_; }
 
-  // Returns the runtime stub id that corresponds to the given address (which
-  // must be a far jump table slot). Returns {kRuntimeStubCount} on failure.
-  WasmCode::RuntimeStubId GetRuntimeStubId(Address runtime_stub_target) const;
+  // Returns the builtin that corresponds to the given address (which
+  // must be a far jump table slot). Returns {kNoBuiltinId} on failure.
+  Builtin GetBuiltinInJumptableSlot(Address target) const;
 
   // Sample the current code size of this modules to the given counters.
   void SampleCodeSize(Counters*) const;
@@ -860,6 +745,22 @@ class V8_EXPORT_PRIVATE NativeModule final {
 
   Counters* counters() const { return code_allocator_.counters(); }
 
+  size_t EstimateCurrentMemoryConsumption() const;
+
+  bool log_code() const { return log_code_.load(std::memory_order_relaxed); }
+
+  void EnableCodeLogging() { log_code_.store(true, std::memory_order_relaxed); }
+
+  void DisableCodeLogging() {
+    log_code_.store(false, std::memory_order_relaxed);
+  }
+
+  enum class JumpTableType {
+    kJumpTable,
+    kFarJumpTable,
+    kLazyCompileTable,
+  };
+
  private:
   friend class WasmCode;
   friend class WasmCodeAllocator;
@@ -873,7 +774,8 @@ class V8_EXPORT_PRIVATE NativeModule final {
   };
 
   // Private constructor, called via {WasmCodeManager::NewNativeModule()}.
-  NativeModule(const WasmFeatures& enabled_features,
+  NativeModule(WasmFeatures enabled_features,
+               CompileTimeImports compile_imports,
                DynamicTiering dynamic_tiering, VirtualMemory code_space,
                std::shared_ptr<const WasmModule> module,
                std::shared_ptr<Counters> async_counters,
@@ -882,17 +784,18 @@ class V8_EXPORT_PRIVATE NativeModule final {
   std::unique_ptr<WasmCode> AddCodeWithCodeSpace(
       int index, const CodeDesc& desc, int stack_slots,
       uint32_t tagged_parameter_slots,
-      base::Vector<const byte> protected_instructions_data,
-      base::Vector<const byte> source_position_table,
-      base::Vector<const byte> inlining_positions, WasmCode::Kind kind,
+      base::Vector<const uint8_t> protected_instructions_data,
+      base::Vector<const uint8_t> source_position_table,
+      base::Vector<const uint8_t> inlining_positions, WasmCode::Kind kind,
       ExecutionTier tier, ForDebugging for_debugging,
       bool frame_has_feedback_slot, base::Vector<uint8_t> code_space,
       const JumpTablesRef& jump_tables_ref);
 
-  WasmCode* CreateEmptyJumpTableLocked(int jump_table_size);
+  WasmCode* CreateEmptyJumpTableLocked(int jump_table_size, JumpTableType type);
 
   WasmCode* CreateEmptyJumpTableInRegionLocked(int jump_table_size,
-                                               base::AddressRegion);
+                                               base::AddressRegion,
+                                               JumpTableType type);
 
   // Finds the jump tables that should be used for given code region. This
   // information is then passed to {GetNearCallTargetForFunction} and
@@ -942,6 +845,9 @@ class V8_EXPORT_PRIVATE NativeModule final {
   // to be consistent across asynchronous compilations later.
   const WasmFeatures enabled_features_;
 
+  // Compile-time imports requested for this module.
+  const CompileTimeImports compile_imports_;
+
   // The decoded module, stored in a shared_ptr such that background compile
   // tasks can keep this alive.
   std::shared_ptr<const WasmModule> module_;
@@ -970,7 +876,7 @@ class V8_EXPORT_PRIVATE NativeModule final {
   std::unique_ptr<CompilationState> compilation_state_;
 
   // A cache of the import wrappers, keyed on the kind and signature.
-  std::unique_ptr<WasmImportWrapperCache> import_wrapper_cache_;
+  WasmImportWrapperCache import_wrapper_cache_;
 
   // Array to handle number of function calls.
   std::unique_ptr<uint32_t[]> tiering_budgets_;
@@ -1024,7 +930,6 @@ class V8_EXPORT_PRIVATE NativeModule final {
   // End of fields protected by {allocation_mutex_}.
   //////////////////////////////////////////////////////////////////////////////
 
-  const BoundsCheckStrategy bounds_checks_;
   bool lazy_compile_frozen_ = false;
   std::atomic<size_t> liftoff_bailout_count_{0};
   std::atomic<size_t> liftoff_code_size_{0};
@@ -1038,7 +943,14 @@ class V8_EXPORT_PRIVATE NativeModule final {
 
   // Whether the next instantiation should trigger repeated output of PGO data
   // (if --experimental-wasm-pgo-to-file is enabled).
-  std::atomic<bool> should_pgo_data_be_written{true};
+  std::atomic<bool> should_pgo_data_be_written_{true};
+
+  // A lock-free quick-access flag to indicate whether code for this
+  // NativeModule might need to be logged in any isolate. This is updated by the
+  // {WasmEngine}, which keeps the source of truth. After checking this flag,
+  // you would typically call into {WasmEngine::LogCode} which then checks
+  // (under a mutex) which isolate needs logging.
+  std::atomic<bool> log_code_{false};
 };
 
 class V8_EXPORT_PRIVATE WasmCodeManager final {
@@ -1054,7 +966,16 @@ class V8_EXPORT_PRIVATE WasmCodeManager final {
 #endif  // V8_OS_WIN64
 
   NativeModule* LookupNativeModule(Address pc) const;
-  WasmCode* LookupCode(Address pc) const;
+  // Returns the Wasm code that contains the given address. The result
+  // is cached. There is one cache per isolate for performance reasons
+  // (to avoid locking and reference counting). Note that the returned
+  // value is not reference counted. This should not be an issue since
+  // we expect that the code is currently being executed. If 'isolate'
+  // is nullptr, no caching occurs.
+  WasmCode* LookupCode(Isolate* isolate, Address pc) const;
+  std::pair<WasmCode*, SafepointEntry> LookupCodeAndSafepoint(Isolate* isolate,
+                                                              Address pc);
+  void FlushCodeLookupCache(Isolate* isolate);
   size_t committed_code_space() const {
     return total_committed_code_space_.load();
   }
@@ -1090,19 +1011,15 @@ class V8_EXPORT_PRIVATE WasmCodeManager final {
   // Can only be called if {HasMemoryProtectionKeySupport()} is {true}.
   static bool MemoryProtectionKeyWritable();
 
-  // Allocate new memory for assembler buffers, potentially protected by PKU.
-  base::AddressRegion AllocateAssemblerBufferSpace(int size);
-
-  // Free previously allocated space for assembler buffers.
-  void FreeAssemblerBufferSpace(base::AddressRegion region);
-
  private:
   friend class WasmCodeAllocator;
   friend class WasmEngine;
+  friend class WasmCodeLookupCache;
 
   std::shared_ptr<NativeModule> NewNativeModule(
-      Isolate* isolate, const WasmFeatures& enabled_features,
-      size_t code_size_estimate, std::shared_ptr<const WasmModule> module);
+      Isolate* isolate, WasmFeatures enabled_features,
+      CompileTimeImports compile_imports, size_t code_size_estimate,
+      std::shared_ptr<const WasmModule> module);
 
   V8_WARN_UNUSED_RESULT VirtualMemory TryAllocate(size_t size,
                                                   void* hint = nullptr);
@@ -1114,6 +1031,8 @@ class V8_EXPORT_PRIVATE WasmCodeManager final {
 
   void AssignRange(base::AddressRegion, NativeModule*);
 
+  WasmCode* LookupCode(Address pc) const;
+
   const size_t max_committed_code_space_;
 
   std::atomic<size_t> total_committed_code_space_{0};
@@ -1178,8 +1097,29 @@ class GlobalWasmCodeRef {
   const std::shared_ptr<NativeModule> native_module_;
 };
 
-Builtin RuntimeStubIdToBuiltinName(WasmCode::RuntimeStubId);
-const char* GetRuntimeStubName(WasmCode::RuntimeStubId);
+class WasmCodeLookupCache final {
+  friend WasmCodeManager;
+
+ public:
+  WasmCodeLookupCache() { Flush(); }
+
+  WasmCodeLookupCache(const WasmCodeLookupCache&) = delete;
+  WasmCodeLookupCache& operator=(const WasmCodeLookupCache&) = delete;
+
+ private:
+  struct CacheEntry {
+    std::atomic<Address> pc;
+    wasm::WasmCode* code;
+    SafepointEntry safepoint_entry;
+    CacheEntry() : safepoint_entry() {}
+  };
+
+  void Flush();
+  CacheEntry* GetCacheEntry(Address pc);
+
+  static const int kWasmCodeLookupCacheSize = 1024;
+  CacheEntry cache_[kWasmCodeLookupCacheSize];
+};
 
 }  // namespace wasm
 }  // namespace internal
