@@ -16,6 +16,7 @@
 #include "src/wasm/baseline/liftoff-compiler.h"
 #include "src/wasm/baseline/liftoff-register.h"
 #include "src/wasm/module-decoder.h"
+#include "src/wasm/std-object-sizes.h"
 #include "src/wasm/value-type.h"
 #include "src/wasm/wasm-code-manager.h"
 #include "src/wasm/wasm-engine.h"
@@ -110,6 +111,20 @@ void DebugSideTable::Entry::Print(std::ostream& os) const {
   os << " ]\n";
 }
 
+size_t DebugSideTable::Entry::EstimateCurrentMemoryConsumption() const {
+  UPDATE_WHEN_CLASS_CHANGES(DebugSideTable::Entry, 32);
+  return ContentSize(changed_values_);
+}
+
+size_t DebugSideTable::EstimateCurrentMemoryConsumption() const {
+  UPDATE_WHEN_CLASS_CHANGES(DebugSideTable, 32);
+  size_t result = sizeof(DebugSideTable) + ContentSize(entries_);
+  for (const Entry& entry : entries_) {
+    result += entry.EstimateCurrentMemoryConsumption();
+  }
+  return result;
+}
+
 class DebugInfoImpl {
  public:
   explicit DebugInfoImpl(NativeModule* native_module)
@@ -118,21 +133,21 @@ class DebugInfoImpl {
   DebugInfoImpl(const DebugInfoImpl&) = delete;
   DebugInfoImpl& operator=(const DebugInfoImpl&) = delete;
 
-  int GetNumLocals(Address pc) {
-    FrameInspectionScope scope(this, pc);
+  int GetNumLocals(Address pc, Isolate* isolate) {
+    FrameInspectionScope scope(this, pc, isolate);
     if (!scope.is_inspectable()) return 0;
     return scope.debug_side_table->num_locals();
   }
 
   WasmValue GetLocalValue(int local, Address pc, Address fp,
                           Address debug_break_fp, Isolate* isolate) {
-    FrameInspectionScope scope(this, pc);
+    FrameInspectionScope scope(this, pc, isolate);
     return GetValue(scope.debug_side_table, scope.debug_side_table_entry, local,
                     fp, debug_break_fp, isolate);
   }
 
-  int GetStackDepth(Address pc) {
-    FrameInspectionScope scope(this, pc);
+  int GetStackDepth(Address pc, Isolate* isolate) {
+    FrameInspectionScope scope(this, pc, isolate);
     if (!scope.is_inspectable()) return 0;
     int num_locals = scope.debug_side_table->num_locals();
     int stack_height = scope.debug_side_table_entry->stack_height();
@@ -141,7 +156,7 @@ class DebugInfoImpl {
 
   WasmValue GetStackValue(int index, Address pc, Address fp,
                           Address debug_break_fp, Isolate* isolate) {
-    FrameInspectionScope scope(this, pc);
+    FrameInspectionScope scope(this, pc, isolate);
     int num_locals = scope.debug_side_table->num_locals();
     int value_count = scope.debug_side_table_entry->stack_height();
     if (num_locals + index >= value_count) return {};
@@ -149,8 +164,8 @@ class DebugInfoImpl {
                     num_locals + index, fp, debug_break_fp, isolate);
   }
 
-  const WasmFunction& GetFunctionAtAddress(Address pc) {
-    FrameInspectionScope scope(this, pc);
+  const WasmFunction& GetFunctionAtAddress(Address pc, Isolate* isolate) {
+    FrameInspectionScope scope(this, pc, isolate);
     auto* module = native_module_->module();
     return module->functions[scope.code->index()];
   }
@@ -221,8 +236,10 @@ class DebugInfoImpl {
     // If lazy validation is on, we might need to lazily validate here.
     if (V8_UNLIKELY(!env.module->function_was_validated(func_index))) {
       WasmFeatures unused_detected_features;
-      DecodeResult validation_result = ValidateFunctionBody(
-          env.enabled_features, env.module, &unused_detected_features, body);
+      Zone validation_zone(wasm::GetWasmEngine()->allocator(), ZONE_NAME);
+      DecodeResult validation_result =
+          ValidateFunctionBody(&validation_zone, env.enabled_features,
+                               env.module, &unused_detected_features, body);
       // Handling illegal modules here is tricky. As lazy validation is off by
       // default anyway and this is for debugging only, we just crash for now.
       CHECK_WITH_MSG(validation_result.ok(),
@@ -391,7 +408,7 @@ class DebugInfoImpl {
   }
 
   bool IsStepping(WasmFrame* frame) {
-    Isolate* isolate = frame->wasm_instance().GetIsolate();
+    Isolate* isolate = frame->wasm_instance()->GetIsolate();
     if (isolate->debug()->last_step_action() == StepInto) return true;
     base::MutexGuard guard(&mutex_);
     auto it = per_isolate_data_.find(isolate);
@@ -478,10 +495,44 @@ class DebugInfoImpl {
     }
   }
 
+  size_t EstimateCurrentMemoryConsumption() const {
+    UPDATE_WHEN_CLASS_CHANGES(DebugInfoImpl, 208);
+    UPDATE_WHEN_CLASS_CHANGES(CachedDebuggingCode, 40);
+    UPDATE_WHEN_CLASS_CHANGES(PerIsolateDebugData, 48);
+    size_t result = sizeof(DebugInfoImpl);
+    {
+      base::MutexGuard lock(&debug_side_tables_mutex_);
+      result += ContentSize(debug_side_tables_);
+      for (const auto& [code, table] : debug_side_tables_) {
+        result += table->EstimateCurrentMemoryConsumption();
+      }
+    }
+    {
+      base::MutexGuard lock(&mutex_);
+      result += ContentSize(cached_debugging_code_);
+      for (const CachedDebuggingCode& code : cached_debugging_code_) {
+        result += code.breakpoint_offsets.size() * sizeof(int);
+      }
+      result += ContentSize(per_isolate_data_);
+      for (const auto& [isolate, data] : per_isolate_data_) {
+        // Inlined handling of {PerIsolateDebugData}.
+        result += ContentSize(data.breakpoints_per_function);
+        for (const auto& [idx, breakpoints] : data.breakpoints_per_function) {
+          result += ContentSize(breakpoints);
+        }
+      }
+    }
+    if (v8_flags.trace_wasm_offheap_memory) {
+      PrintF("DebugInfo: %zu\n", result);
+    }
+    return result;
+  }
+
  private:
   struct FrameInspectionScope {
-    FrameInspectionScope(DebugInfoImpl* debug_info, Address pc)
-        : code(wasm::GetWasmCodeManager()->LookupCode(pc)),
+    FrameInspectionScope(DebugInfoImpl* debug_info, Address pc,
+                         Isolate* isolate)
+        : code(wasm::GetWasmCodeManager()->LookupCode(isolate, pc)),
           pc_offset(static_cast<int>(pc - code->instruction_start())),
           debug_side_table(code->is_inspectable()
                                ? debug_info->GetDebugSideTable(code)
@@ -565,7 +616,8 @@ class DebugInfoImpl {
           return WasmValue(ReadUnalignedValue<uint64_t>(gp_addr(reg.gp())));
         } else if (value->type.is_reference()) {
           Handle<Object> obj(
-              Object(ReadUnalignedValue<Address>(gp_addr(reg.gp()))), isolate);
+              Tagged<Object>(ReadUnalignedValue<Address>(gp_addr(reg.gp()))),
+              isolate);
           return WasmValue(obj, value->type);
         } else {
           UNREACHABLE();
@@ -609,8 +661,9 @@ class DebugInfoImpl {
       case kRef:
       case kRefNull:
       case kRtt: {
-        Handle<Object> obj(Object(ReadUnalignedValue<Address>(stack_address)),
-                           isolate);
+        Handle<Object> obj(
+            Tagged<Object>(ReadUnalignedValue<Address>(stack_address)),
+            isolate);
         return WasmValue(obj, value->type);
       }
       case kI8:
@@ -669,7 +722,7 @@ class DebugInfoImpl {
     DisallowGarbageCollection no_gc;
     int position = frame->position();
     NativeModule* native_module =
-        frame->wasm_instance().module_object().native_module();
+        frame->wasm_instance()->module_object()->native_module();
     uint8_t opcode = native_module->wire_bytes()[position];
     if (opcode == kExprReturn) return true;
     // Another implicit return is at the last kExprEnd in the function body.
@@ -723,22 +776,27 @@ DebugInfo::DebugInfo(NativeModule* native_module)
 
 DebugInfo::~DebugInfo() = default;
 
-int DebugInfo::GetNumLocals(Address pc) { return impl_->GetNumLocals(pc); }
+int DebugInfo::GetNumLocals(Address pc, Isolate* isolate) {
+  return impl_->GetNumLocals(pc, isolate);
+}
 
 WasmValue DebugInfo::GetLocalValue(int local, Address pc, Address fp,
                                    Address debug_break_fp, Isolate* isolate) {
   return impl_->GetLocalValue(local, pc, fp, debug_break_fp, isolate);
 }
 
-int DebugInfo::GetStackDepth(Address pc) { return impl_->GetStackDepth(pc); }
+int DebugInfo::GetStackDepth(Address pc, Isolate* isolate) {
+  return impl_->GetStackDepth(pc, isolate);
+}
 
 WasmValue DebugInfo::GetStackValue(int index, Address pc, Address fp,
                                    Address debug_break_fp, Isolate* isolate) {
   return impl_->GetStackValue(index, pc, fp, debug_break_fp, isolate);
 }
 
-const wasm::WasmFunction& DebugInfo::GetFunctionAtAddress(Address pc) {
-  return impl_->GetFunctionAtAddress(pc);
+const wasm::WasmFunction& DebugInfo::GetFunctionAtAddress(Address pc,
+                                                          Isolate* isolate) {
+  return impl_->GetFunctionAtAddress(pc, isolate);
 }
 
 void DebugInfo::SetBreakpoint(int func_index, int offset,
@@ -782,25 +840,28 @@ void DebugInfo::RemoveIsolate(Isolate* isolate) {
   return impl_->RemoveIsolate(isolate);
 }
 
+size_t DebugInfo::EstimateCurrentMemoryConsumption() const {
+  return impl_->EstimateCurrentMemoryConsumption();
+}
+
 }  // namespace wasm
 
 namespace {
 
 // Return the next breakable position at or after {offset_in_func} in function
 // {func_index}, or 0 if there is none.
-// Note that 0 is never a breakable position in wasm, since the first byte
+// Note that 0 is never a breakable position in wasm, since the first uint8_t
 // contains the locals count for the function.
 int FindNextBreakablePosition(wasm::NativeModule* native_module, int func_index,
                               int offset_in_func) {
-  AccountingAllocator alloc;
-  Zone tmp(&alloc, ZONE_NAME);
+  Zone zone{wasm::GetWasmEngine()->allocator(), ZONE_NAME};
   wasm::BodyLocalDecls locals;
-  const byte* module_start = native_module->wire_bytes().begin();
+  const uint8_t* module_start = native_module->wire_bytes().begin();
   const wasm::WasmFunction& func =
       native_module->module()->functions[func_index];
   wasm::BytecodeIterator iterator(module_start + func.code.offset(),
                                   module_start + func.code.end_offset(),
-                                  &locals, &tmp);
+                                  &locals, &zone);
   DCHECK_LT(0, locals.encoded_size);
   if (offset_in_func < 0) return 0;
   for (; iterator.has_next(); iterator.next()) {
@@ -811,17 +872,19 @@ int FindNextBreakablePosition(wasm::NativeModule* native_module, int func_index,
   return 0;
 }
 
-void SetBreakOnEntryFlag(Script script, bool enabled) {
-  if (script.break_on_entry() == enabled) return;
+void SetBreakOnEntryFlag(Tagged<Script> script, bool enabled) {
+  if (script->break_on_entry() == enabled) return;
 
-  script.set_break_on_entry(enabled);
+  script->set_break_on_entry(enabled);
   // Update the "break_on_entry" flag on all live instances.
-  i::WeakArrayList weak_instance_list = script.wasm_weak_instance_list();
-  for (int i = 0; i < weak_instance_list.length(); ++i) {
-    if (weak_instance_list.Get(i)->IsCleared()) continue;
-    i::WasmInstanceObject instance =
-        i::WasmInstanceObject::cast(weak_instance_list.Get(i)->GetHeapObject());
-    instance.set_break_on_entry(enabled);
+  i::Tagged<i::WeakArrayList> weak_instance_list =
+      script->wasm_weak_instance_list();
+  i::Isolate* isolate = script->GetIsolate();
+  for (int i = 0; i < weak_instance_list->length(); ++i) {
+    if (weak_instance_list->Get(i)->IsCleared()) continue;
+    i::Tagged<i::WasmInstanceObject> instance =
+        i::WasmInstanceObject::cast(weak_instance_list->Get(i).GetHeapObject());
+    instance->trusted_data(isolate)->set_break_on_entry(enabled);
   }
 }
 }  // namespace
@@ -894,9 +957,10 @@ bool WasmScript::SetBreakPointForFunction(Handle<Script> script, int func_index,
 
 namespace {
 
-int GetBreakpointPos(Isolate* isolate, Object break_point_info_or_undef) {
-  if (break_point_info_or_undef.IsUndefined(isolate)) return kMaxInt;
-  return BreakPointInfo::cast(break_point_info_or_undef).source_position();
+int GetBreakpointPos(Isolate* isolate,
+                     Tagged<Object> break_point_info_or_undef) {
+  if (IsUndefined(break_point_info_or_undef, isolate)) return kMaxInt;
+  return BreakPointInfo::cast(break_point_info_or_undef)->source_position();
 }
 
 int FindBreakpointInfoInsertPos(Isolate* isolate,
@@ -911,7 +975,7 @@ int FindBreakpointInfoInsertPos(Isolate* isolate,
   int right = breakpoint_infos->length();  // exclusive
   while (right - left > 1) {
     int mid = left + (right - left) / 2;
-    Object mid_obj = breakpoint_infos->get(mid);
+    Tagged<Object> mid_obj = breakpoint_infos->get(mid);
     if (GetBreakpointPos(isolate, mid_obj) <= position) {
       left = mid;
     } else {
@@ -946,12 +1010,14 @@ bool WasmScript::ClearBreakPoint(Handle<Script> script, int position,
   if (info->GetBreakPointCount(isolate) == 0) {
     // Update array by moving breakpoints up one position.
     for (int i = pos; i < breakpoint_infos->length() - 1; i++) {
-      Object entry = breakpoint_infos->get(i + 1);
+      Tagged<Object> entry = breakpoint_infos->get(i + 1);
       breakpoint_infos->set(i, entry);
-      if (entry.IsUndefined(isolate)) break;
+      if (IsUndefined(entry, isolate)) break;
     }
     // Make sure last array element is empty as a result.
-    breakpoint_infos->set_undefined(breakpoint_infos->length() - 1);
+    breakpoint_infos->set(breakpoint_infos->length() - 1,
+                          ReadOnlyRoots{isolate}.undefined_value(),
+                          SKIP_WRITE_BARRIER);
   }
 
   if (break_point->id() == v8::internal::Debug::kInstrumentationId) {
@@ -981,7 +1047,7 @@ bool WasmScript::ClearBreakPointById(Handle<Script> script, int breakpoint_id) {
 
   for (int i = 0, e = breakpoint_infos->length(); i < e; ++i) {
     Handle<Object> obj(breakpoint_infos->get(i), isolate);
-    if (obj->IsUndefined(isolate)) {
+    if (IsUndefined(*obj, isolate)) {
       continue;
     }
     Handle<BreakPointInfo> breakpoint_info = Handle<BreakPointInfo>::cast(obj);
@@ -998,9 +1064,9 @@ bool WasmScript::ClearBreakPointById(Handle<Script> script, int breakpoint_id) {
 }
 
 // static
-void WasmScript::ClearAllBreakpoints(Script script) {
-  script.set_wasm_breakpoint_infos(
-      ReadOnlyRoots(script.GetIsolate()).empty_fixed_array());
+void WasmScript::ClearAllBreakpoints(Tagged<Script> script) {
+  script->set_wasm_breakpoint_infos(
+      ReadOnlyRoots(script->GetIsolate()).empty_fixed_array());
   SetBreakOnEntryFlag(script, false);
 }
 
@@ -1032,8 +1098,8 @@ void WasmScript::AddBreakpointToInfo(Handle<Script> script, int position,
   }
 
   // Enlarge break positions array if necessary.
-  bool need_realloc = !breakpoint_infos->get(breakpoint_infos->length() - 1)
-                           .IsUndefined(isolate);
+  bool need_realloc = !IsUndefined(
+      breakpoint_infos->get(breakpoint_infos->length() - 1), isolate);
   Handle<FixedArray> new_breakpoint_infos = breakpoint_infos;
   if (need_realloc) {
     new_breakpoint_infos = isolate->factory()->NewFixedArray(
@@ -1046,8 +1112,8 @@ void WasmScript::AddBreakpointToInfo(Handle<Script> script, int position,
 
   // Move elements [insert_pos, ...] up by one.
   for (int i = breakpoint_infos->length() - 1; i >= insert_pos; --i) {
-    Object entry = breakpoint_infos->get(i);
-    if (entry.IsUndefined(isolate)) continue;
+    Tagged<Object> entry = breakpoint_infos->get(i);
+    if (IsUndefined(entry, isolate)) continue;
     new_breakpoint_infos->set(i + 1, entry);
   }
 
@@ -1101,9 +1167,8 @@ bool WasmScript::GetPossibleBreakpoints(
   if (start_func_index == end_func_index &&
       start_offset > functions[end_func_index].code.end_offset())
     return false;
-  AccountingAllocator alloc;
-  Zone tmp(&alloc, ZONE_NAME);
-  const byte* module_start = native_module->wire_bytes().begin();
+  Zone zone{wasm::GetWasmEngine()->allocator(), ZONE_NAME};
+  const uint8_t* module_start = native_module->wire_bytes().begin();
 
   for (int func_idx = start_func_index; func_idx <= end_func_index;
        ++func_idx) {
@@ -1113,7 +1178,7 @@ bool WasmScript::GetPossibleBreakpoints(
     wasm::BodyLocalDecls locals;
     wasm::BytecodeIterator iterator(module_start + func.code.offset(),
                                     module_start + func.code.end_offset(),
-                                    &locals, &tmp);
+                                    &locals, &zone);
     DCHECK_LT(0u, locals.encoded_size);
     for (; iterator.has_next(); iterator.next()) {
       uint32_t total_offset = func.code.offset() + iterator.pc_offset();
@@ -1133,7 +1198,7 @@ namespace {
 
 bool CheckBreakPoint(Isolate* isolate, Handle<BreakPoint> break_point,
                      StackFrameId frame_id) {
-  if (break_point->condition().length() == 0) return true;
+  if (break_point->condition()->length() == 0) return true;
 
   HandleScope scope(isolate);
   Handle<String> condition(break_point->condition(), isolate);
@@ -1144,10 +1209,10 @@ bool CheckBreakPoint(Isolate* isolate, Handle<BreakPoint> break_point,
   if (!DebugEvaluate::Local(isolate, frame_id, inlined_jsframe_index, condition,
                             throw_on_side_effect)
            .ToHandle(&result)) {
-    isolate->clear_pending_exception();
+    isolate->clear_exception();
     return false;
   }
-  return result->BooleanValue(isolate);
+  return Object::BooleanValue(*result, isolate);
 }
 
 }  // namespace
@@ -1166,13 +1231,13 @@ MaybeHandle<FixedArray> WasmScript::CheckBreakPoints(Isolate* isolate,
 
   Handle<Object> maybe_breakpoint_info(breakpoint_infos->get(insert_pos),
                                        isolate);
-  if (maybe_breakpoint_info->IsUndefined(isolate)) return {};
+  if (IsUndefined(*maybe_breakpoint_info, isolate)) return {};
   Handle<BreakPointInfo> breakpoint_info =
       Handle<BreakPointInfo>::cast(maybe_breakpoint_info);
   if (breakpoint_info->source_position() != position) return {};
 
   Handle<Object> break_points(breakpoint_info->break_points(), isolate);
-  if (!break_points->IsFixedArray()) {
+  if (!IsFixedArray(*break_points)) {
     if (!CheckBreakPoint(isolate, Handle<BreakPoint>::cast(break_points),
                          frame_id)) {
       return {};
@@ -1193,7 +1258,7 @@ MaybeHandle<FixedArray> WasmScript::CheckBreakPoints(Isolate* isolate,
     }
   }
   if (break_points_hit_count == 0) return {};
-  break_points_hit->Shrink(isolate, break_points_hit_count);
+  break_points_hit->RightTrim(isolate, break_points_hit_count);
   return break_points_hit;
 }
 
