@@ -13,6 +13,7 @@
 #include "src/objects/objects.h"
 #include "src/wasm/jump-table-assembler.h"
 #include "src/wasm/module-decoder.h"
+#include "src/wasm/std-object-sizes.h"
 #include "src/wasm/wasm-code-manager.h"
 #include "src/wasm/wasm-engine.h"
 #include "src/wasm/wasm-init-expr.h"
@@ -24,6 +25,26 @@
 
 namespace v8::internal::wasm {
 
+// Ensure that the max subtyping depth can be stored in the TypeDefinition.
+static_assert(
+    kV8MaxRttSubtypingDepth <=
+    std::numeric_limits<decltype(TypeDefinition().subtyping_depth)>::max());
+
+// static
+int WasmMemory::GetMemory64GuardsShift(uint64_t max_memory_size) {
+  // For memory64 we need a guard region that is at least twice the size of the
+  // maximum size of the Wasm memory. In order to speed-up bounds checks, we
+  // allocate the greater power-of-two size.
+  DCHECK_NE(max_memory_size, 0U);
+  size_t min_guards_size = 2 * max_memory_size;
+  int guards_shift = 63 - base::bits::CountLeadingZeros64(min_guards_size);
+  DCHECK_GE(guards_shift, 0);
+  if (!base::bits::IsPowerOfTwo(min_guards_size)) {
+    guards_shift++;
+  }
+  return guards_shift;
+}
+
 template <class Value>
 void AdaptiveMap<Value>::FinishInitialization() {
   uint32_t count = 0;
@@ -112,17 +133,10 @@ int GetContainingWasmFunction(const WasmModule* module, uint32_t byte_offset) {
   return func_index;
 }
 
-// TODO(7748): Measure whether this iterative implementation is fast enough.
-// We could cache the result on the module, in yet another vector indexed by
-// type index.
 int GetSubtypingDepth(const WasmModule* module, uint32_t type_index) {
-  uint32_t starting_point = type_index;
-  int depth = 0;
-  while ((type_index = module->supertype(type_index)) != kNoSuperType) {
-    if (type_index == starting_point) return -1;  // Cycle detected.
-    depth++;
-    if (depth > static_cast<int>(kV8MaxRttSubtypingDepth)) break;
-  }
+  DCHECK_LT(type_index, module->types.size());
+  int depth = module->types[type_index].subtyping_depth;
+  DCHECK_LE(depth, kV8MaxRttSubtypingDepth);
   return depth;
 }
 
@@ -133,7 +147,7 @@ void LazilyGeneratedNames::AddForTesting(int function_index,
 }
 
 AsmJsOffsetInformation::AsmJsOffsetInformation(
-    base::Vector<const byte> encoded_offsets)
+    base::Vector<const uint8_t> encoded_offsets)
     : encoded_offsets_(base::OwnedVector<const uint8_t>::Of(encoded_offsets)) {}
 
 AsmJsOffsetInformation::~AsmJsOffsetInformation() = default;
@@ -154,6 +168,9 @@ int AsmJsOffsetInformation::GetSourcePosition(int declared_func_index,
   };
   SLOW_DCHECK(std::is_sorted(function_offsets.begin(), function_offsets.end(),
                              byte_offset_less));
+  // If there are no positions recorded, map offset 0 (for function entry) to
+  // position 0.
+  if (function_offsets.empty() && byte_offset == 0) return 0;
   auto it =
       std::lower_bound(function_offsets.begin(), function_offsets.end(),
                        AsmJsOffsetEntry{byte_offset, 0, 0}, byte_offset_less);
@@ -218,7 +235,7 @@ WasmModule::WasmModule(ModuleOrigin origin)
     : signature_zone(GetWasmEngine()->allocator(), "signature zone"),
       origin(origin) {}
 
-bool IsWasmCodegenAllowed(Isolate* isolate, Handle<Context> context) {
+bool IsWasmCodegenAllowed(Isolate* isolate, Handle<NativeContext> context) {
   // TODO(wasm): Once wasm has its own CSP policy, we should introduce a
   // separate callback that includes information about the module about to be
   // compiled. For the time being, pass an empty string as placeholder for the
@@ -308,7 +325,7 @@ Handle<JSObject> GetTypeForGlobal(Isolate* isolate, bool is_mutable,
 
 Handle<JSObject> GetTypeForMemory(Isolate* isolate, uint32_t min_size,
                                   base::Optional<uint32_t> max_size,
-                                  bool shared) {
+                                  bool shared, bool is_memory64) {
   Factory* factory = isolate->factory();
 
   Handle<JSFunction> object_function = isolate->object_function();
@@ -316,6 +333,7 @@ Handle<JSObject> GetTypeForMemory(Isolate* isolate, uint32_t min_size,
   Handle<String> minimum_string = factory->InternalizeUtf8String("minimum");
   Handle<String> maximum_string = factory->InternalizeUtf8String("maximum");
   Handle<String> shared_string = factory->InternalizeUtf8String("shared");
+  Handle<String> index_string = factory->InternalizeUtf8String("index");
   JSObject::AddProperty(isolate, object, minimum_string,
                         factory->NewNumberFromUint(min_size), NONE);
   if (max_size.has_value()) {
@@ -325,6 +343,10 @@ Handle<JSObject> GetTypeForMemory(Isolate* isolate, uint32_t min_size,
   JSObject::AddProperty(isolate, object, shared_string,
                         factory->ToBoolean(shared), NONE);
 
+  auto index = is_memory64 ? "i64" : "i32";
+  JSObject::AddProperty(isolate, object, index_string,
+                        factory->InternalizeUtf8String(index), NONE);
+
   return object;
 }
 
@@ -374,12 +396,14 @@ Handle<JSArray> GetImports(Isolate* isolate,
   Handle<JSArray> array_object = factory->NewJSArray(PACKED_ELEMENTS, 0, 0);
   Handle<FixedArray> storage = factory->NewFixedArray(num_imports);
   JSArray::SetContent(array_object, storage);
-  array_object->set_length(Smi::FromInt(num_imports));
 
   Handle<JSFunction> object_function =
       Handle<JSFunction>(isolate->native_context()->object_function(), isolate);
 
   // Populate the result array.
+  const WellKnownImportsList& well_known_imports =
+      module->type_feedback.well_known_imports;
+  int cursor = 0;
   for (int index = 0; index < num_imports; ++index) {
     const WasmImport& import = module->import_table[index];
 
@@ -389,6 +413,9 @@ Handle<JSArray> GetImports(Isolate* isolate,
     Handle<JSObject> type_value;
     switch (import.kind) {
       case kExternalFunction:
+        if (IsCompileTimeImport(well_known_imports.get(import.index))) {
+          continue;
+        }
         if (enabled_features.has_type_reflection()) {
           auto& func = module->functions[import.index];
           type_value = GetTypeForFunction(isolate, func.sig);
@@ -407,14 +434,14 @@ Handle<JSArray> GetImports(Isolate* isolate,
         break;
       case kExternalMemory:
         if (enabled_features.has_type_reflection()) {
-          DCHECK_EQ(0, import.index);  // Only one memory supported.
+          auto& memory = module->memories[import.index];
           base::Optional<uint32_t> maximum_size;
-          if (module->has_maximum_pages) {
-            maximum_size.emplace(module->maximum_pages);
+          if (memory.has_maximum_pages) {
+            maximum_size.emplace(memory.maximum_pages);
           }
           type_value =
-              GetTypeForMemory(isolate, module->initial_pages, maximum_size,
-                               module->has_shared_memory);
+              GetTypeForMemory(isolate, memory.initial_pages, maximum_size,
+                               memory.is_shared, memory.is_memory64);
         }
         import_kind = memory_string;
         break;
@@ -430,7 +457,7 @@ Handle<JSArray> GetImports(Isolate* isolate,
         import_kind = tag_string;
         break;
     }
-    DCHECK(!import_kind->is_null());
+    DCHECK(!import_kind.is_null());
 
     Handle<String> import_module =
         WasmModuleObject::ExtractUtf8StringFromModuleBytes(
@@ -447,9 +474,10 @@ Handle<JSArray> GetImports(Isolate* isolate,
       JSObject::AddProperty(isolate, entry, type_string, type_value, NONE);
     }
 
-    storage->set(index, *entry);
+    storage->set(cursor++, *entry);
   }
 
+  array_object->set_length(Smi::FromInt(cursor));
   return array_object;
 }
 
@@ -505,14 +533,14 @@ Handle<JSArray> GetExports(Isolate* isolate,
         break;
       case kExternalMemory:
         if (enabled_features.has_type_reflection()) {
-          DCHECK_EQ(0, exp.index);  // Only one memory supported.
+          auto& memory = module->memories[exp.index];
           base::Optional<uint32_t> maximum_size;
-          if (module->has_maximum_pages) {
-            maximum_size.emplace(module->maximum_pages);
+          if (memory.has_maximum_pages) {
+            maximum_size.emplace(memory.maximum_pages);
           }
           type_value =
-              GetTypeForMemory(isolate, module->initial_pages, maximum_size,
-                               module->has_shared_memory);
+              GetTypeForMemory(isolate, memory.initial_pages, maximum_size,
+                               memory.is_shared, memory.is_memory64);
         }
         export_kind = memory_string;
         break;
@@ -617,21 +645,93 @@ int GetSourcePosition(const WasmModule* module, uint32_t func_index,
       is_at_number_conversion);
 }
 
-namespace {
-template <typename T>
-inline size_t VectorSize(const std::vector<T>& vector) {
-  return sizeof(T) * vector.size();
+size_t WasmModule::EstimateStoredSize() const {
+  UPDATE_WHEN_CLASS_CHANGES(WasmModule, 848);
+  return sizeof(WasmModule) +                            // --
+         signature_zone.allocation_size_for_tracing() +  // --
+         ContentSize(types) +                            // --
+         ContentSize(isorecursive_canonical_type_ids) +  // --
+         ContentSize(explicit_recursive_type_groups) +   // --
+         ContentSize(functions) +                        // --
+         ContentSize(globals) +                          // --
+         ContentSize(data_segments) +                    // --
+         ContentSize(tables) +                           // --
+         ContentSize(memories) +                         // --
+         ContentSize(import_table) +                     // --
+         ContentSize(export_table) +                     // --
+         ContentSize(tags) +                             // --
+         ContentSize(stringref_literals) +               // --
+         ContentSize(elem_segments) +                    // --
+         ContentSize(compilation_hints) +                // --
+         ContentSize(branch_hints) +                     // --
+         ContentSize(inst_traces) +                      // --
+         (num_declared_functions + 7) / 8;               // validated_functions
 }
-}  // namespace
 
-size_t EstimateStoredSize(const WasmModule* module) {
-  return sizeof(WasmModule) + VectorSize(module->globals) +
-         module->signature_zone.allocation_size() + VectorSize(module->types) +
-         VectorSize(module->isorecursive_canonical_type_ids) +
-         VectorSize(module->functions) + VectorSize(module->data_segments) +
-         VectorSize(module->tables) + VectorSize(module->import_table) +
-         VectorSize(module->export_table) + VectorSize(module->tags) +
-         VectorSize(module->elem_segments);
+template <class Value>
+size_t AdaptiveMap<Value>::EstimateCurrentMemoryConsumption() const {
+  UNREACHABLE();  // Explicit implementations below.
+}
+
+template <>
+size_t NameMap::EstimateCurrentMemoryConsumption() const {
+  size_t result = ContentSize(vector_);
+  if (map_) result += ContentSize(*map_);
+  return result;
+}
+
+size_t LazilyGeneratedNames::EstimateCurrentMemoryConsumption() const {
+  base::MutexGuard lock(&mutex_);
+  return function_names_.EstimateCurrentMemoryConsumption();
+}
+
+template <>
+size_t IndirectNameMap::EstimateCurrentMemoryConsumption() const {
+  size_t result = ContentSize(vector_);
+  for (const auto& inner_map : vector_) {
+    result += inner_map.EstimateCurrentMemoryConsumption();
+  }
+  if (map_) {
+    result += ContentSize(*map_);
+    for (const auto& [outer_index, inner_map] : *map_) {
+      result += inner_map.EstimateCurrentMemoryConsumption();
+    }
+  }
+  return result;
+}
+
+size_t TypeFeedbackStorage::EstimateCurrentMemoryConsumption() const {
+  UPDATE_WHEN_CLASS_CHANGES(TypeFeedbackStorage, 160);
+  UPDATE_WHEN_CLASS_CHANGES(FunctionTypeFeedback, 48);
+  // Not including sizeof(TFS) because that's contained in sizeof(WasmModule).
+  size_t result = ContentSize(feedback_for_function);
+  base::SharedMutexGuard<base::kShared> lock(&mutex);
+  for (const auto& [func_idx, feedback] : feedback_for_function) {
+    result += ContentSize(feedback.feedback_vector);
+    result += feedback.call_targets.size() * sizeof(uint32_t);
+  }
+  // The size of {well_known_imports} can only be estimated at the WasmModule
+  // level.
+  if (v8_flags.trace_wasm_offheap_memory) {
+    PrintF("TypeFeedback: %zu\n", result);
+  }
+  return result;
+}
+
+size_t WasmModule::EstimateCurrentMemoryConsumption() const {
+  UPDATE_WHEN_CLASS_CHANGES(WasmModule, 848);
+  size_t result = EstimateStoredSize();
+
+  result += type_feedback.EstimateCurrentMemoryConsumption();
+  // For type_feedback.well_known_imports:
+  result += num_imported_functions * sizeof(WellKnownImport);
+
+  result += lazily_generated_names.EstimateCurrentMemoryConsumption();
+
+  if (v8_flags.trace_wasm_offheap_memory) {
+    PrintF("WasmModule: %zu\n", result);
+  }
+  return result;
 }
 
 size_t PrintSignature(base::Vector<char> buffer, const wasm::FunctionSig* sig,
