@@ -31,26 +31,28 @@ V8_EXPORT_PRIVATE bool IsJSCompatibleSignature(const FunctionSig* sig);
 // Format of all opcode macros: kExprName, binary, signature, wat name
 
 // Control expressions and blocks.
-#define FOREACH_CONTROL_OPCODE(V)                              \
-  V(Unreachable, 0x00, _, "unreachable")                       \
-  V(Nop, 0x01, _, "nop")                                       \
-  V(Block, 0x02, _, "block")                                   \
-  V(Loop, 0x03, _, "loop")                                     \
-  V(If, 0x04, _, "if")                                         \
-  V(Else, 0x05, _, "else")                                     \
-  V(Try, 0x06, _, "try")                                       \
-  V(Catch, 0x07, _, "catch")                                   \
-  V(Throw, 0x08, _, "throw")                                   \
-  V(Rethrow, 0x09, _, "rethrow")                               \
-  V(End, 0x0b, _, "end")                                       \
-  V(Br, 0x0c, _, "br")                                         \
-  V(BrIf, 0x0d, _, "br_if")                                    \
-  V(BrTable, 0x0e, _, "br_table")                              \
-  V(Return, 0x0f, _, "return")                                 \
-  V(Delegate, 0x18, _, "delegate")                             \
-  V(CatchAll, 0x19, _, "catch_all")                            \
-  V(BrOnNull, 0xd4, _, "br_on_null")        /* gc prototype */ \
-  V(BrOnNonNull, 0xd6, _, "br_on_non_null") /* gc prototype */ \
+#define FOREACH_CONTROL_OPCODE(V) /* 80 columns                             */ \
+  V(Unreachable, 0x00, _, "unreachable")                                       \
+  V(Nop, 0x01, _, "nop")                                                       \
+  V(Block, 0x02, _, "block")                                                   \
+  V(Loop, 0x03, _, "loop")                                                     \
+  V(If, 0x04, _, "if")                                                         \
+  V(Else, 0x05, _, "else")                                                     \
+  V(Try, 0x06, _, "try")                                                       \
+  V(Catch, 0x07, _, "catch")                                                   \
+  V(Throw, 0x08, _, "throw")                                                   \
+  V(Rethrow, 0x09, _, "rethrow")                                               \
+  V(TryTable, 0x1f, _, "try_table")                                            \
+  V(ThrowRef, 0x0a, _, "throw_ref")                                            \
+  V(End, 0x0b, _, "end")                                                       \
+  V(Br, 0x0c, _, "br")                                                         \
+  V(BrIf, 0x0d, _, "br_if")                                                    \
+  V(BrTable, 0x0e, _, "br_table")                                              \
+  V(Return, 0x0f, _, "return")                                                 \
+  V(Delegate, 0x18, _, "delegate")                                             \
+  V(CatchAll, 0x19, _, "catch_all")                                            \
+  V(BrOnNull, 0xd5, _, "br_on_null")        /* typed_funcref prototype */      \
+  V(BrOnNonNull, 0xd6, _, "br_on_non_null") /* typed_funcref prototype */      \
   V(NopForTestingUnsupportedInLiftoff, 0x16, _, "nop_for_testing")
 
 // Constants, locals, globals, and calls.
@@ -78,7 +80,7 @@ V8_EXPORT_PRIVATE bool IsJSCompatibleSignature(const FunctionSig* sig);
   V(RefNull, 0xd0, _, "ref.null")                                            \
   V(RefIsNull, 0xd1, _, "ref.is_null")                                       \
   V(RefFunc, 0xd2, _, "ref.func")                                            \
-  V(RefAsNonNull, 0xd3, _, "ref.as_non_null") /* typed_funcref prototype */
+  V(RefAsNonNull, 0xd4, _, "ref.as_non_null") /* typed_funcref prototype */
 
 // Load memory expressions.
 #define FOREACH_LOAD_MEM_OPCODE(V)            \
@@ -116,8 +118,8 @@ V8_EXPORT_PRIVATE bool IsJSCompatibleSignature(const FunctionSig* sig);
 
 // Expressions with signatures.
 
-// The following opcodes can be used as constant expressions under
-// --experimental-wasm-extended-const.
+// Opcodes that can also be used in constant expressions (via the 'extended
+// constant expressions' proposal).
 #define FOREACH_SIMPLE_EXTENDED_CONST_OPCODE(V) \
   V(I32Add, 0x6a, i_ii, "i32.add")              \
   V(I32Sub, 0x6b, i_ii, "i32.sub")              \
@@ -254,7 +256,7 @@ V8_EXPORT_PRIVATE bool IsJSCompatibleSignature(const FunctionSig* sig);
   FOREACH_SIMPLE_EXTENDED_CONST_OPCODE(V) \
   FOREACH_SIMPLE_NON_CONST_OPCODE(V)
 
-#define FOREACH_SIMPLE_PROTOTYPE_OPCODE(V) V(RefEq, 0xd5, i_qq, "ref.eq")
+#define FOREACH_SIMPLE_PROTOTYPE_OPCODE(V) V(RefEq, 0xd3, i_qq, "ref.eq")
 
 // For compatibility with Asm.js.
 // These opcodes are not spec'ed (or visible) externally; the idea is
@@ -610,130 +612,118 @@ V8_EXPORT_PRIVATE bool IsJSCompatibleSignature(const FunctionSig* sig);
 #define FOREACH_NUMERIC_OPCODE(V) \
   FOREACH_NUMERIC_OPCODE_WITH_SIG(V) FOREACH_NUMERIC_OPCODE_VARIADIC(V)
 
+// kExprName, binary, signature for memory32, wat name, signature for memory64.
 #define FOREACH_ATOMIC_OPCODE(V)                                              \
-  V(AtomicNotify, 0xfe00, i_ii, "memory.atomic.notify")                       \
-  V(I32AtomicWait, 0xfe01, i_iil, "memory.atomic.wait32")                     \
-  V(I64AtomicWait, 0xfe02, i_ill, "memory.atomic.wait64")                     \
-  V(I32AtomicLoad, 0xfe10, i_i, "i32.atomic.load")                            \
-  V(I64AtomicLoad, 0xfe11, l_i, "i64.atomic.load")                            \
-  V(I32AtomicLoad8U, 0xfe12, i_i, "i32.atomic.load8_u")                       \
-  V(I32AtomicLoad16U, 0xfe13, i_i, "i32.atomic.load16_u")                     \
-  V(I64AtomicLoad8U, 0xfe14, l_i, "i64.atomic.load8_u")                       \
-  V(I64AtomicLoad16U, 0xfe15, l_i, "i64.atomic.load16_u")                     \
-  V(I64AtomicLoad32U, 0xfe16, l_i, "i64.atomic.load32_u")                     \
-  V(I32AtomicStore, 0xfe17, v_ii, "i32.atomic.store")                         \
-  V(I64AtomicStore, 0xfe18, v_il, "i64.atomic.store")                         \
-  V(I32AtomicStore8U, 0xfe19, v_ii, "i32.atomic.store8")                      \
-  V(I32AtomicStore16U, 0xfe1a, v_ii, "i32.atomic.store16")                    \
-  V(I64AtomicStore8U, 0xfe1b, v_il, "i64.atomic.store8")                      \
-  V(I64AtomicStore16U, 0xfe1c, v_il, "i64.atomic.store16")                    \
-  V(I64AtomicStore32U, 0xfe1d, v_il, "i64.atomic.store32")                    \
-  V(I32AtomicAdd, 0xfe1e, i_ii, "i32.atomic.rmw.add")                         \
-  V(I64AtomicAdd, 0xfe1f, l_il, "i64.atomic.rmw.add")                         \
-  V(I32AtomicAdd8U, 0xfe20, i_ii, "i32.atomic.rmw8.add_u")                    \
-  V(I32AtomicAdd16U, 0xfe21, i_ii, "i32.atomic.rmw16.add_u")                  \
-  V(I64AtomicAdd8U, 0xfe22, l_il, "i64.atomic.rmw8.add_u")                    \
-  V(I64AtomicAdd16U, 0xfe23, l_il, "i64.atomic.rmw16.add_u")                  \
-  V(I64AtomicAdd32U, 0xfe24, l_il, "i64.atomic.rmw32.add_u")                  \
-  V(I32AtomicSub, 0xfe25, i_ii, "i32.atomic.rmw.sub")                         \
-  V(I64AtomicSub, 0xfe26, l_il, "i64.atomic.rmw.sub")                         \
-  V(I32AtomicSub8U, 0xfe27, i_ii, "i32.atomic.rmw8.sub_u")                    \
-  V(I32AtomicSub16U, 0xfe28, i_ii, "i32.atomic.rmw16.sub_u")                  \
-  V(I64AtomicSub8U, 0xfe29, l_il, "i64.atomic.rmw8.sub_u")                    \
-  V(I64AtomicSub16U, 0xfe2a, l_il, "i64.atomic.rmw16.sub_u")                  \
-  V(I64AtomicSub32U, 0xfe2b, l_il, "i64.atomic.rmw32.sub_u")                  \
-  V(I32AtomicAnd, 0xfe2c, i_ii, "i32.atomic.rmw.and")                         \
-  V(I64AtomicAnd, 0xfe2d, l_il, "i64.atomic.rmw.and")                         \
-  V(I32AtomicAnd8U, 0xfe2e, i_ii, "i32.atomic.rmw8.and_u")                    \
-  V(I32AtomicAnd16U, 0xfe2f, i_ii, "i32.atomic.rmw16.and_u")                  \
-  V(I64AtomicAnd8U, 0xfe30, l_il, "i64.atomic.rmw8.and_u")                    \
-  V(I64AtomicAnd16U, 0xfe31, l_il, "i64.atomic.rmw16.and_u")                  \
-  V(I64AtomicAnd32U, 0xfe32, l_il, "i64.atomic.rmw32.and_u")                  \
-  V(I32AtomicOr, 0xfe33, i_ii, "i32.atomic.rmw.or")                           \
-  V(I64AtomicOr, 0xfe34, l_il, "i64.atomic.rmw.or")                           \
-  V(I32AtomicOr8U, 0xfe35, i_ii, "i32.atomic.rmw8.or_u")                      \
-  V(I32AtomicOr16U, 0xfe36, i_ii, "i32.atomic.rmw16.or_u")                    \
-  V(I64AtomicOr8U, 0xfe37, l_il, "i64.atomic.rmw8.or_u")                      \
-  V(I64AtomicOr16U, 0xfe38, l_il, "i64.atomic.rmw16.or_u")                    \
-  V(I64AtomicOr32U, 0xfe39, l_il, "i64.atomic.rmw32.or_u")                    \
-  V(I32AtomicXor, 0xfe3a, i_ii, "i32.atomic.rmw.xor")                         \
-  V(I64AtomicXor, 0xfe3b, l_il, "i64.atomic.rmw.xor")                         \
-  V(I32AtomicXor8U, 0xfe3c, i_ii, "i32.atomic.rmw8.xor_u")                    \
-  V(I32AtomicXor16U, 0xfe3d, i_ii, "i32.atomic.rmw16.xor_u")                  \
-  V(I64AtomicXor8U, 0xfe3e, l_il, "i64.atomic.rmw8.xor_u")                    \
-  V(I64AtomicXor16U, 0xfe3f, l_il, "i64.atomic.rmw16.xor_u")                  \
-  V(I64AtomicXor32U, 0xfe40, l_il, "i64.atomic.rmw32.xor_u")                  \
-  V(I32AtomicExchange, 0xfe41, i_ii, "i32.atomic.rmw.xchg")                   \
-  V(I64AtomicExchange, 0xfe42, l_il, "i64.atomic.rmw.xchg")                   \
-  V(I32AtomicExchange8U, 0xfe43, i_ii, "i32.atomic.rmw8.xchg_u")              \
-  V(I32AtomicExchange16U, 0xfe44, i_ii, "i32.atomic.rmw16.xchg_u")            \
-  V(I64AtomicExchange8U, 0xfe45, l_il, "i64.atomic.rmw8.xchg_u")              \
-  V(I64AtomicExchange16U, 0xfe46, l_il, "i64.atomic.rmw16.xchg_u")            \
-  V(I64AtomicExchange32U, 0xfe47, l_il, "i64.atomic.rmw32.xchg_u")            \
-  V(I32AtomicCompareExchange, 0xfe48, i_iii, "i32.atomic.rmw.cmpxchg")        \
-  V(I64AtomicCompareExchange, 0xfe49, l_ill, "i64.atomic.rmw.cmpxchg")        \
-  V(I32AtomicCompareExchange8U, 0xfe4a, i_iii, "i32.atomic.rmw8.cmpxchg_u")   \
-  V(I32AtomicCompareExchange16U, 0xfe4b, i_iii, "i32.atomic.rmw16.cmpxchg_u") \
-  V(I64AtomicCompareExchange8U, 0xfe4c, l_ill, "i64.atomic.rmw8.cmpxchg_u")   \
-  V(I64AtomicCompareExchange16U, 0xfe4d, l_ill, "i64.atomic.rmw16.cmpxchg_u") \
-  V(I64AtomicCompareExchange32U, 0xfe4e, l_ill, "i64.atomic.rmw32.cmpxchg_u")
+  V(AtomicNotify, 0xfe00, i_ii, "memory.atomic.notify", i_li)                 \
+  V(I32AtomicWait, 0xfe01, i_iil, "memory.atomic.wait32", i_lil)              \
+  V(I64AtomicWait, 0xfe02, i_ill, "memory.atomic.wait64", i_lll)              \
+  V(I32AtomicLoad, 0xfe10, i_i, "i32.atomic.load", i_l)                       \
+  V(I64AtomicLoad, 0xfe11, l_i, "i64.atomic.load", l_l)                       \
+  V(I32AtomicLoad8U, 0xfe12, i_i, "i32.atomic.load8_u", i_l)                  \
+  V(I32AtomicLoad16U, 0xfe13, i_i, "i32.atomic.load16_u", i_l)                \
+  V(I64AtomicLoad8U, 0xfe14, l_i, "i64.atomic.load8_u", l_l)                  \
+  V(I64AtomicLoad16U, 0xfe15, l_i, "i64.atomic.load16_u", l_l)                \
+  V(I64AtomicLoad32U, 0xfe16, l_i, "i64.atomic.load32_u", l_l)                \
+  V(I32AtomicStore, 0xfe17, v_ii, "i32.atomic.store", v_li)                   \
+  V(I64AtomicStore, 0xfe18, v_il, "i64.atomic.store", v_ll)                   \
+  V(I32AtomicStore8U, 0xfe19, v_ii, "i32.atomic.store8", v_li)                \
+  V(I32AtomicStore16U, 0xfe1a, v_ii, "i32.atomic.store16", v_li)              \
+  V(I64AtomicStore8U, 0xfe1b, v_il, "i64.atomic.store8", v_ll)                \
+  V(I64AtomicStore16U, 0xfe1c, v_il, "i64.atomic.store16", v_ll)              \
+  V(I64AtomicStore32U, 0xfe1d, v_il, "i64.atomic.store32", v_ll)              \
+  V(I32AtomicAdd, 0xfe1e, i_ii, "i32.atomic.rmw.add", i_li)                   \
+  V(I64AtomicAdd, 0xfe1f, l_il, "i64.atomic.rmw.add", l_ll)                   \
+  V(I32AtomicAdd8U, 0xfe20, i_ii, "i32.atomic.rmw8.add_u", i_li)              \
+  V(I32AtomicAdd16U, 0xfe21, i_ii, "i32.atomic.rmw16.add_u", i_li)            \
+  V(I64AtomicAdd8U, 0xfe22, l_il, "i64.atomic.rmw8.add_u", l_ll)              \
+  V(I64AtomicAdd16U, 0xfe23, l_il, "i64.atomic.rmw16.add_u", l_ll)            \
+  V(I64AtomicAdd32U, 0xfe24, l_il, "i64.atomic.rmw32.add_u", l_ll)            \
+  V(I32AtomicSub, 0xfe25, i_ii, "i32.atomic.rmw.sub", i_li)                   \
+  V(I64AtomicSub, 0xfe26, l_il, "i64.atomic.rmw.sub", l_ll)                   \
+  V(I32AtomicSub8U, 0xfe27, i_ii, "i32.atomic.rmw8.sub_u", i_li)              \
+  V(I32AtomicSub16U, 0xfe28, i_ii, "i32.atomic.rmw16.sub_u", i_li)            \
+  V(I64AtomicSub8U, 0xfe29, l_il, "i64.atomic.rmw8.sub_u", l_ll)              \
+  V(I64AtomicSub16U, 0xfe2a, l_il, "i64.atomic.rmw16.sub_u", l_ll)            \
+  V(I64AtomicSub32U, 0xfe2b, l_il, "i64.atomic.rmw32.sub_u", l_ll)            \
+  V(I32AtomicAnd, 0xfe2c, i_ii, "i32.atomic.rmw.and", i_li)                   \
+  V(I64AtomicAnd, 0xfe2d, l_il, "i64.atomic.rmw.and", l_ll)                   \
+  V(I32AtomicAnd8U, 0xfe2e, i_ii, "i32.atomic.rmw8.and_u", i_li)              \
+  V(I32AtomicAnd16U, 0xfe2f, i_ii, "i32.atomic.rmw16.and_u", i_li)            \
+  V(I64AtomicAnd8U, 0xfe30, l_il, "i64.atomic.rmw8.and_u", l_ll)              \
+  V(I64AtomicAnd16U, 0xfe31, l_il, "i64.atomic.rmw16.and_u", l_ll)            \
+  V(I64AtomicAnd32U, 0xfe32, l_il, "i64.atomic.rmw32.and_u", l_ll)            \
+  V(I32AtomicOr, 0xfe33, i_ii, "i32.atomic.rmw.or", i_li)                     \
+  V(I64AtomicOr, 0xfe34, l_il, "i64.atomic.rmw.or", l_ll)                     \
+  V(I32AtomicOr8U, 0xfe35, i_ii, "i32.atomic.rmw8.or_u", i_li)                \
+  V(I32AtomicOr16U, 0xfe36, i_ii, "i32.atomic.rmw16.or_u", i_li)              \
+  V(I64AtomicOr8U, 0xfe37, l_il, "i64.atomic.rmw8.or_u", l_ll)                \
+  V(I64AtomicOr16U, 0xfe38, l_il, "i64.atomic.rmw16.or_u", l_ll)              \
+  V(I64AtomicOr32U, 0xfe39, l_il, "i64.atomic.rmw32.or_u", l_ll)              \
+  V(I32AtomicXor, 0xfe3a, i_ii, "i32.atomic.rmw.xor", i_li)                   \
+  V(I64AtomicXor, 0xfe3b, l_il, "i64.atomic.rmw.xor", l_ll)                   \
+  V(I32AtomicXor8U, 0xfe3c, i_ii, "i32.atomic.rmw8.xor_u", i_li)              \
+  V(I32AtomicXor16U, 0xfe3d, i_ii, "i32.atomic.rmw16.xor_u", i_li)            \
+  V(I64AtomicXor8U, 0xfe3e, l_il, "i64.atomic.rmw8.xor_u", l_ll)              \
+  V(I64AtomicXor16U, 0xfe3f, l_il, "i64.atomic.rmw16.xor_u", l_ll)            \
+  V(I64AtomicXor32U, 0xfe40, l_il, "i64.atomic.rmw32.xor_u", l_ll)            \
+  V(I32AtomicExchange, 0xfe41, i_ii, "i32.atomic.rmw.xchg", i_li)             \
+  V(I64AtomicExchange, 0xfe42, l_il, "i64.atomic.rmw.xchg", l_ll)             \
+  V(I32AtomicExchange8U, 0xfe43, i_ii, "i32.atomic.rmw8.xchg_u", i_li)        \
+  V(I32AtomicExchange16U, 0xfe44, i_ii, "i32.atomic.rmw16.xchg_u", i_li)      \
+  V(I64AtomicExchange8U, 0xfe45, l_il, "i64.atomic.rmw8.xchg_u", l_ll)        \
+  V(I64AtomicExchange16U, 0xfe46, l_il, "i64.atomic.rmw16.xchg_u", l_ll)      \
+  V(I64AtomicExchange32U, 0xfe47, l_il, "i64.atomic.rmw32.xchg_u", l_ll)      \
+  V(I32AtomicCompareExchange, 0xfe48, i_iii, "i32.atomic.rmw.cmpxchg", i_lii) \
+  V(I64AtomicCompareExchange, 0xfe49, l_ill, "i64.atomic.rmw.cmpxchg", l_lll) \
+  V(I32AtomicCompareExchange8U, 0xfe4a, i_iii, "i32.atomic.rmw8.cmpxchg_u",   \
+    i_lii)                                                                    \
+  V(I32AtomicCompareExchange16U, 0xfe4b, i_iii, "i32.atomic.rmw16.cmpxchg_u", \
+    i_lii)                                                                    \
+  V(I64AtomicCompareExchange8U, 0xfe4c, l_ill, "i64.atomic.rmw8.cmpxchg_u",   \
+    l_lll)                                                                    \
+  V(I64AtomicCompareExchange16U, 0xfe4d, l_ill, "i64.atomic.rmw16.cmpxchg_u", \
+    l_lll)                                                                    \
+  V(I64AtomicCompareExchange32U, 0xfe4e, l_ill, "i64.atomic.rmw32.cmpxchg_u", \
+    l_lll)
 
 #define FOREACH_ATOMIC_0_OPERAND_OPCODE(V)                      \
   /* AtomicFence does not target a particular linear memory. */ \
-  V(AtomicFence, 0xfe03, v_v, "atomic.fence")
+  V(AtomicFence, 0xfe03, v_v, "atomic.fence", v_v)
 
 #define FOREACH_GC_OPCODE(V) /*              Force 80 columns               */ \
-  V(StructGet, 0xfb03, _, "struct.get")                                        \
-  V(StructGetS, 0xfb04, _, "struct.get_s")                                     \
-  V(StructGetU, 0xfb05, _, "struct.get_u")                                     \
-  V(StructSet, 0xfb06, _, "struct.set")                                        \
-  V(StructNew, 0xfb07, _, "struct.new")                                        \
-  V(StructNewDefault, 0xfb08, _, "struct.new_default")                         \
-  V(ArrayGet, 0xfb13, _, "array.get")                                          \
-  V(ArrayGetS, 0xfb14, _, "array.get_s")                                       \
-  V(ArrayGetU, 0xfb15, _, "array.get_u")                                       \
-  V(ArraySet, 0xfb16, _, "array.set")                                          \
-  V(ArrayCopy, 0xfb18, _, "array.copy")                                        \
-  V(ArrayLen, 0xfb19, _, "array.len")                                          \
-  V(ArrayNewFixed, 0xfb1a, _, "array.new_fixed")                               \
-  V(ArrayNew, 0xfb1b, _, "array.new")                                          \
-  V(ArrayNewDefault, 0xfb1c, _, "array.new_default")                           \
-  V(ArrayNewData, 0xfb1d, _, "array.new_data")                                 \
-  V(ArrayNewElem, 0xfb1f, _, "array.new_elem")                                 \
-  V(ArrayFill, 0xfb0f, _, "array.fill")                                        \
-  V(ArrayInitData, 0xfb54, _, "array.init_data")                               \
-  V(ArrayInitElem, 0xfb55, _, "array.init_elem")                               \
-  V(I31New, 0xfb20, _, "i31.new")                                              \
-  V(I31GetS, 0xfb21, _, "i31.get_s")                                           \
-  V(I31GetU, 0xfb22, _, "i31.get_u")                                           \
-  V(RefTest, 0xfb40, _, "ref.test")                                            \
-  V(RefTestNull, 0xfb48, _, "ref.test null")                                   \
-  V(RefTestDeprecated, 0xfb44, _, "ref.test")                                  \
-  V(RefCast, 0xfb41, _, "ref.cast")                                            \
-  V(RefCastNull, 0xfb49, _, "ref.cast null")                                   \
-  V(RefCastDeprecated, 0xfb45, _, "ref.cast")                                  \
-  V(BrOnCast, 0xfb42, _, "br_on_cast")                                         \
-  V(BrOnCastNull, 0xfb4a, _, "br_on_cast null")                                \
-  V(BrOnCastDeprecated, 0xfb46, _, "br_on_cast")                               \
-  V(BrOnCastFail, 0xfb43, _, "br_on_cast_fail")                                \
-  V(BrOnCastFailNull, 0xfb4b, _, "br_on_cast_fail null")                       \
-  V(BrOnCastFailDeprecated, 0xfb47, _, "br_on_cast_fail")                      \
-  V(BrOnCastGeneric, 0xfb4f, _, "br_on_cast")                                  \
+  V(StructNew, 0xfb00, _, "struct.new")                                        \
+  V(StructNewDefault, 0xfb01, _, "struct.new_default")                         \
+  V(StructGet, 0xfb02, _, "struct.get")                                        \
+  V(StructGetS, 0xfb03, _, "struct.get_s")                                     \
+  V(StructGetU, 0xfb04, _, "struct.get_u")                                     \
+  V(StructSet, 0xfb05, _, "struct.set")                                        \
+  V(ArrayNew, 0xfb06, _, "array.new")                                          \
+  V(ArrayNewDefault, 0xfb07, _, "array.new_default")                           \
+  V(ArrayNewFixed, 0xfb08, _, "array.new_fixed")                               \
+  V(ArrayNewData, 0xfb09, _, "array.new_data")                                 \
+  V(ArrayNewElem, 0xfb0a, _, "array.new_elem")                                 \
+  V(ArrayGet, 0xfb0b, _, "array.get")                                          \
+  V(ArrayGetS, 0xfb0c, _, "array.get_s")                                       \
+  V(ArrayGetU, 0xfb0d, _, "array.get_u")                                       \
+  V(ArraySet, 0xfb0e, _, "array.set")                                          \
+  V(ArrayLen, 0xfb0f, _, "array.len")                                          \
+  V(ArrayFill, 0xfb10, _, "array.fill")                                        \
+  V(ArrayCopy, 0xfb11, _, "array.copy")                                        \
+  V(ArrayInitData, 0xfb12, _, "array.init_data")                               \
+  V(ArrayInitElem, 0xfb13, _, "array.init_elem")                               \
+  V(RefTest, 0xfb14, _, "ref.test")                                            \
+  V(RefTestNull, 0xfb15, _, "ref.test null")                                   \
+  V(RefCast, 0xfb16, _, "ref.cast")                                            \
+  V(RefCastNull, 0xfb17, _, "ref.cast null")                                   \
+  V(BrOnCastGeneric, 0xfb18, _, "br_on_cast")                                  \
+  V(BrOnCastFailGeneric, 0xfb19, _, "br_on_cast_fail")                         \
+  V(AnyConvertExtern, 0xfb1a, _, "any.convert_extern")                         \
+  V(ExternConvertAny, 0xfb1b, _, "extern.convert_any")                         \
+  V(RefI31, 0xfb1c, _, "ref.i31")                                              \
+  V(I31GetS, 0xfb1d, _, "i31.get_s")                                           \
+  V(I31GetU, 0xfb1e, _, "i31.get_u")                                           \
   V(RefCastNop, 0xfb4c, _, "ref.cast_nop")                                     \
-  V(RefIsStruct, 0xfb51, _, "ref.is_struct")                                   \
-  V(RefIsI31, 0xfb52, _, "ref.is_i31")                                         \
-  V(RefIsArray, 0xfb53, _, "ref.is_array")                                     \
-  V(RefAsStruct, 0xfb59, _, "ref.as_struct")                                   \
-  V(RefAsI31, 0xfb5a, _, "ref.as_i31")                                         \
-  V(RefAsArray, 0xfb5b, _, "ref.as_array")                                     \
-  V(BrOnStruct, 0xfb61, _, "br_on_struct")                                     \
-  V(BrOnI31, 0xfb62, _, "br_on_i31")                                           \
-  V(BrOnArray, 0xfb66, _, "br_on_array")                                       \
-  V(BrOnNonStruct, 0xfb64, _, "br_on_non_struct")                              \
-  V(BrOnNonI31, 0xfb65, _, "br_on_non_i31")                                    \
-  V(BrOnNonArray, 0xfb67, _, "br_on_non_array")                                \
-  V(ExternInternalize, 0xfb70, _, "extern.internalize")                        \
-  V(ExternExternalize, 0xfb71, _, "extern.externalize")                        \
+  /* Stringref proposal. */                                                    \
   V(StringNewUtf8, 0xfb80, _, "string.new_utf8")                               \
   V(StringNewWtf16, 0xfb81, _, "string.new_wtf16")                             \
   V(StringConst, 0xfb82, _, "string.const")                                    \
@@ -759,7 +749,7 @@ V8_EXPORT_PRIVATE bool IsJSCompatibleSignature(const FunctionSig* sig);
   V(StringViewWtf8EncodeWtf8, 0xfb95, _, "stringview_wtf8.encode_wtf8")        \
   V(StringAsWtf16, 0xfb98, _, "string.as_wtf16")                               \
   V(StringViewWtf16Length, 0xfb99, _, "stringview_wtf16.length")               \
-  V(StringViewWtf16GetCodeUnit, 0xfb9a, _, "stringview_wtf16.get_codeunit")    \
+  V(StringViewWtf16GetCodeunit, 0xfb9a, _, "stringview_wtf16.get_codeunit")    \
   V(StringViewWtf16Encode, 0xfb9b, _, "stringview_wtf16.encode")               \
   V(StringViewWtf16Slice, 0xfb9c, _, "stringview_wtf16.slice")                 \
   V(StringAsIter, 0xfba0, _, "string.as_iter")                                 \
@@ -799,47 +789,54 @@ V8_EXPORT_PRIVATE bool IsJSCompatibleSignature(const FunctionSig* sig);
 // All signatures.
 #define FOREACH_SIGNATURE(V)                        \
   FOREACH_SIMD_SIGNATURE(V)                         \
-  V(v_v, kWasmVoid)                                 \
-  V(i_ii, kWasmI32, kWasmI32, kWasmI32)             \
-  V(i_i, kWasmI32, kWasmI32)                        \
-  V(i_v, kWasmI32)                                  \
-  V(i_ff, kWasmI32, kWasmF32, kWasmF32)             \
-  V(i_f, kWasmI32, kWasmF32)                        \
-  V(i_dd, kWasmI32, kWasmF64, kWasmF64)             \
-  V(i_d, kWasmI32, kWasmF64)                        \
-  V(i_l, kWasmI32, kWasmI64)                        \
-  V(l_ll, kWasmI64, kWasmI64, kWasmI64)             \
-  V(i_ll, kWasmI32, kWasmI64, kWasmI64)             \
-  V(l_l, kWasmI64, kWasmI64)                        \
-  V(l_i, kWasmI64, kWasmI32)                        \
-  V(l_f, kWasmI64, kWasmF32)                        \
-  V(l_d, kWasmI64, kWasmF64)                        \
-  V(f_ff, kWasmF32, kWasmF32, kWasmF32)             \
-  V(f_f, kWasmF32, kWasmF32)                        \
-  V(f_d, kWasmF32, kWasmF64)                        \
-  V(f_i, kWasmF32, kWasmI32)                        \
-  V(f_l, kWasmF32, kWasmI64)                        \
-  V(d_dd, kWasmF64, kWasmF64, kWasmF64)             \
   V(d_d, kWasmF64, kWasmF64)                        \
+  V(d_dd, kWasmF64, kWasmF64, kWasmF64)             \
   V(d_f, kWasmF64, kWasmF32)                        \
   V(d_i, kWasmF64, kWasmI32)                        \
-  V(d_l, kWasmF64, kWasmI64)                        \
-  V(v_i, kWasmVoid, kWasmI32)                       \
-  V(v_ii, kWasmVoid, kWasmI32, kWasmI32)            \
-  V(v_id, kWasmVoid, kWasmI32, kWasmF64)            \
   V(d_id, kWasmF64, kWasmI32, kWasmF64)             \
-  V(v_if, kWasmVoid, kWasmI32, kWasmF32)            \
+  V(d_l, kWasmF64, kWasmI64)                        \
+  V(f_d, kWasmF32, kWasmF64)                        \
+  V(f_f, kWasmF32, kWasmF32)                        \
+  V(f_ff, kWasmF32, kWasmF32, kWasmF32)             \
+  V(f_i, kWasmF32, kWasmI32)                        \
   V(f_if, kWasmF32, kWasmI32, kWasmF32)             \
-  V(v_il, kWasmVoid, kWasmI32, kWasmI64)            \
-  V(l_il, kWasmI64, kWasmI32, kWasmI64)             \
-  V(v_iii, kWasmVoid, kWasmI32, kWasmI32, kWasmI32) \
-  V(i_iii, kWasmI32, kWasmI32, kWasmI32, kWasmI32)  \
-  V(l_ill, kWasmI64, kWasmI32, kWasmI64, kWasmI64)  \
-  V(i_iil, kWasmI32, kWasmI32, kWasmI32, kWasmI64)  \
-  V(i_ill, kWasmI32, kWasmI32, kWasmI64, kWasmI64)  \
+  V(f_l, kWasmF32, kWasmI64)                        \
   V(i_a, kWasmI32, kWasmAnyRef)                     \
   V(i_ci, kWasmI32, kWasmFuncRef, kWasmI32)         \
-  V(i_qq, kWasmI32, kWasmEqRef, kWasmEqRef)
+  V(i_d, kWasmI32, kWasmF64)                        \
+  V(i_dd, kWasmI32, kWasmF64, kWasmF64)             \
+  V(i_f, kWasmI32, kWasmF32)                        \
+  V(i_ff, kWasmI32, kWasmF32, kWasmF32)             \
+  V(i_i, kWasmI32, kWasmI32)                        \
+  V(i_ii, kWasmI32, kWasmI32, kWasmI32)             \
+  V(i_iii, kWasmI32, kWasmI32, kWasmI32, kWasmI32)  \
+  V(i_iil, kWasmI32, kWasmI32, kWasmI32, kWasmI64)  \
+  V(i_ill, kWasmI32, kWasmI32, kWasmI64, kWasmI64)  \
+  V(i_l, kWasmI32, kWasmI64)                        \
+  V(i_li, kWasmI32, kWasmI64, kWasmI32)             \
+  V(i_lii, kWasmI32, kWasmI64, kWasmI32, kWasmI32)  \
+  V(i_lil, kWasmI32, kWasmI64, kWasmI32, kWasmI64)  \
+  V(i_lll, kWasmI32, kWasmI64, kWasmI64, kWasmI64)  \
+  V(i_ll, kWasmI32, kWasmI64, kWasmI64)             \
+  V(i_qq, kWasmI32, kWasmEqRef, kWasmEqRef)         \
+  V(i_v, kWasmI32)                                  \
+  V(l_d, kWasmI64, kWasmF64)                        \
+  V(l_f, kWasmI64, kWasmF32)                        \
+  V(l_i, kWasmI64, kWasmI32)                        \
+  V(l_il, kWasmI64, kWasmI32, kWasmI64)             \
+  V(l_ill, kWasmI64, kWasmI32, kWasmI64, kWasmI64)  \
+  V(l_l, kWasmI64, kWasmI64)                        \
+  V(l_ll, kWasmI64, kWasmI64, kWasmI64)             \
+  V(l_lll, kWasmI64, kWasmI64, kWasmI64, kWasmI64)  \
+  V(v_id, kWasmVoid, kWasmI32, kWasmF64)            \
+  V(v_if, kWasmVoid, kWasmI32, kWasmF32)            \
+  V(v_i, kWasmVoid, kWasmI32)                       \
+  V(v_ii, kWasmVoid, kWasmI32, kWasmI32)            \
+  V(v_iii, kWasmVoid, kWasmI32, kWasmI32, kWasmI32) \
+  V(v_il, kWasmVoid, kWasmI32, kWasmI64)            \
+  V(v_li, kWasmVoid, kWasmI64, kWasmI32)            \
+  V(v_ll, kWasmVoid, kWasmI64, kWasmI64)            \
+  V(v_v, kWasmVoid)
 
 #define FOREACH_SIMD_SIGNATURE(V)                      \
   V(s_s, kWasmS128, kWasmS128)                         \
@@ -899,12 +896,17 @@ class V8_EXPORT_PRIVATE WasmOpcodes {
  public:
   static constexpr const char* OpcodeName(WasmOpcode);
   static constexpr const FunctionSig* Signature(WasmOpcode);
+  static constexpr const FunctionSig* SignatureForAtomicOp(WasmOpcode opcode,
+                                                           bool is_memory64);
   static constexpr const FunctionSig* AsmjsSignature(WasmOpcode);
   static constexpr bool IsPrefixOpcode(WasmOpcode);
   static constexpr bool IsControlOpcode(WasmOpcode);
   static constexpr bool IsExternRefOpcode(WasmOpcode);
   static constexpr bool IsThrowingOpcode(WasmOpcode);
   static constexpr bool IsRelaxedSimdOpcode(WasmOpcode);
+#if DEBUG
+  static constexpr bool IsMemoryAccessOpcode(WasmOpcode);
+#endif  // DEBUG
   // Check whether the given opcode always jumps, i.e. all instructions after
   // this one in the current block are dead. Returns false for |end|.
   static constexpr bool IsUnconditionalJump(WasmOpcode);
@@ -914,7 +916,7 @@ class V8_EXPORT_PRIVATE WasmOpcodes {
   static constexpr TrapReason MessageIdToTrapReason(MessageTemplate message);
 
   // Extract the prefix byte (or 0x00) from a {WasmOpcode}.
-  static constexpr byte ExtractPrefix(WasmOpcode);
+  static constexpr uint8_t ExtractPrefix(WasmOpcode);
   static inline const char* TrapReasonMessage(TrapReason);
 };
 
