@@ -163,9 +163,9 @@ TEST(ReduceJSLoadContext0) {
   native->set(slot, *expected);
 
   Node* const_context =
-      t.jsgraph()->Constant(MakeRef(t.broker(), native), t.broker());
+      t.jsgraph()->ConstantNoHole(MakeRef(t.broker(), native), t.broker());
   Node* deep_const_context =
-      t.jsgraph()->Constant(MakeRef(t.broker(), subcontext2), t.broker());
+      t.jsgraph()->ConstantNoHole(MakeRef(t.broker(), subcontext2), t.broker());
   Node* param_context = t.graph()->NewNode(t.common()->Parameter(0), start);
 
   {
@@ -290,7 +290,8 @@ TEST(ReduceJSLoadContext2) {
   // The graph's context chain ends in a constant context (context_object1),
   // which has another outer context (context_object0).
   //
-  //   context2 <-- context1 <-- context0 (= HeapConstant(context_object1))
+  //   context2 <-- context1 <-- context0 (=
+  //   HeapConstantNoHole(context_object1))
   //   context_object1 <~~ context_object0
 
   ContextSpecializationTester t(Nothing<OuterContext>());
@@ -313,8 +314,8 @@ TEST(ReduceJSLoadContext2) {
   context_object0->set_extension(*slot_value0);
   context_object1->set_extension(*slot_value1);
 
-  Node* context0 =
-      t.jsgraph()->Constant(MakeRef(t.broker(), context_object1), t.broker());
+  Node* context0 = t.jsgraph()->ConstantNoHole(
+      MakeRef(t.broker(), context_object1), t.broker());
   Node* context1 =
       t.graph()->NewNode(create_function_context, context0, start, start);
   Node* context2 =
@@ -476,9 +477,9 @@ TEST(ReduceJSStoreContext0) {
   native->set(slot, *expected);
 
   Node* const_context =
-      t.jsgraph()->Constant(MakeRef(t.broker(), native), t.broker());
+      t.jsgraph()->ConstantNoHole(MakeRef(t.broker(), native), t.broker());
   Node* deep_const_context =
-      t.jsgraph()->Constant(MakeRef(t.broker(), subcontext2), t.broker());
+      t.jsgraph()->ConstantNoHole(MakeRef(t.broker(), subcontext2), t.broker());
   Node* param_context = t.graph()->NewNode(t.common()->Parameter(0), start);
 
   {
@@ -591,8 +592,8 @@ TEST(ReduceJSStoreContext2) {
   context_object0->set_extension(*slot_value0);
   context_object1->set_extension(*slot_value1);
 
-  Node* context0 =
-      t.jsgraph()->Constant(MakeRef(t.broker(), context_object1), t.broker());
+  Node* context0 = t.jsgraph()->ConstantNoHole(
+      MakeRef(t.broker(), context_object1), t.broker());
   Node* context1 =
       t.graph()->NewNode(create_function_context, context0, start, start);
   Node* context2 =
@@ -730,13 +731,12 @@ TEST(SpecializeJSFunction_ToConstant_uninit) {
         "(function() { if (false) { var x = 1; } function inc(a)"
         " { return x; } return inc; })()");  // x is undefined!
     i::Isolate* isolate = CcTest::i_isolate();
-    CHECK(
-        T.Call(T.Val(0.0), T.Val(0.0)).ToHandleChecked()->IsUndefined(isolate));
-    CHECK(
-        T.Call(T.Val(2.0), T.Val(0.0)).ToHandleChecked()->IsUndefined(isolate));
-    CHECK(T.Call(T.Val(-2.1), T.Val(0.0))
-              .ToHandleChecked()
-              ->IsUndefined(isolate));
+    CHECK(IsUndefined(*T.Call(T.Val(0.0), T.Val(0.0)).ToHandleChecked(),
+                      isolate));
+    CHECK(IsUndefined(*T.Call(T.Val(2.0), T.Val(0.0)).ToHandleChecked(),
+                      isolate));
+    CHECK(IsUndefined(*T.Call(T.Val(-2.1), T.Val(0.0)).ToHandleChecked(),
+                      isolate));
   }
 
   {
@@ -744,9 +744,9 @@ TEST(SpecializeJSFunction_ToConstant_uninit) {
         "(function() { if (false) { var x = 1; } function inc(a)"
         " { return a + x; } return inc; })()");  // x is undefined!
 
-    CHECK(T.Call(T.Val(0.0), T.Val(0.0)).ToHandleChecked()->IsNaN());
-    CHECK(T.Call(T.Val(2.0), T.Val(0.0)).ToHandleChecked()->IsNaN());
-    CHECK(T.Call(T.Val(-2.1), T.Val(0.0)).ToHandleChecked()->IsNaN());
+    CHECK(IsNaN(*T.Call(T.Val(0.0), T.Val(0.0)).ToHandleChecked()));
+    CHECK(IsNaN(*T.Call(T.Val(2.0), T.Val(0.0)).ToHandleChecked()));
+    CHECK(IsNaN(*T.Call(T.Val(-2.1), T.Val(0.0)).ToHandleChecked()));
   }
 }
 
