@@ -57,15 +57,18 @@ TEST(WeakReferencesBasic) {
         Factory::CodeBuilder(isolate, desc, CodeKind::FOR_TESTING).Build();
     CHECK(IsCode(*code));
 
-    lh->set_data1(HeapObjectReference::Weak(*code));
-    Tagged<HeapObject> code_heap_object;
-    CHECK(lh->data1().GetHeapObjectIfWeak(&code_heap_object));
-    CHECK_EQ(*code, code_heap_object);
+    // We cannot store the Code object itself into the tagged field as it will
+    // be located outside of the main pointer compression cage when the sandbox
+    // is enabled. So instead we use the Code's wrapper object.
+    lh->set_data1(HeapObjectReference::Weak(code->wrapper()));
+    Tagged<HeapObject> code_wrapper_heap_object;
+    CHECK(lh->data1().GetHeapObjectIfWeak(&code_wrapper_heap_object));
+    CHECK_EQ(code->wrapper(), code_wrapper_heap_object);
 
     heap::InvokeMajorGC(CcTest::heap());
 
-    CHECK(lh->data1().GetHeapObjectIfWeak(&code_heap_object));
-    CHECK_EQ(*code, code_heap_object);
+    CHECK(lh->data1().GetHeapObjectIfWeak(&code_wrapper_heap_object));
+    CHECK_EQ(code->wrapper(), code_wrapper_heap_object);
   }  // code will go out of scope.
 
   heap::InvokeMajorGC(CcTest::heap());
@@ -193,7 +196,7 @@ TEST(ObjectMovesBeforeClearingWeakField) {
   HandleScope outer_scope(isolate);
   Handle<LoadHandler> lh = CreateLoadHandlerForTest(factory);
   CHECK(InCorrectGeneration(*lh));
-  Tagged<LoadHandler> lh_location = *lh;
+  Address lh_object_location = lh->address();
   {
     HandleScope inner_scope(isolate);
     // Create a new FixedArray which the LoadHandler will point to.
@@ -210,8 +213,7 @@ TEST(ObjectMovesBeforeClearingWeakField) {
 
   // Scavenger will move *lh.
   heap::InvokeMinorGC(heap);
-  Tagged<LoadHandler> new_lh_location = *lh;
-  CHECK_NE(lh_location, new_lh_location);
+  CHECK_NE(lh_object_location, lh.address());
   CHECK(lh->data1()->IsWeak());
 
   // Now we try to clear *lh.
@@ -373,7 +375,7 @@ TEST(WeakArraysBasic) {
 
   for (int i = 0; i < length; ++i) {
     Tagged<HeapObject> heap_object;
-    CHECK(array->Get(i).GetHeapObjectIfStrong(&heap_object));
+    CHECK(array->get(i).GetHeapObjectIfStrong(&heap_object));
     CHECK_EQ(heap_object, ReadOnlyRoots(heap).undefined_value());
   }
 
@@ -390,10 +392,10 @@ TEST(WeakArraysBasic) {
     Handle<FixedArray> index3 = factory->NewFixedArray(1);
     index3->set(0, Smi::FromInt(2019));
 
-    array->Set(0, HeapObjectReference::Weak(*index0));
-    array->Set(1, HeapObjectReference::Weak(*index1));
-    array->Set(2, HeapObjectReference::Strong(*index2));
-    array->Set(3, HeapObjectReference::Weak(*index3));
+    array->set(0, HeapObjectReference::Weak(*index0));
+    array->set(1, HeapObjectReference::Weak(*index1));
+    array->set(2, HeapObjectReference::Strong(*index2));
+    array->set(3, HeapObjectReference::Weak(*index3));
     saved = inner_scope.CloseAndEscape(index1);
   }  // inner_scope goes out of scope.
 
@@ -405,23 +407,23 @@ TEST(WeakArraysBasic) {
   // space.
   heap::InvokeMinorGC(heap);
   Tagged<HeapObject> heap_object;
-  CHECK(array->Get(0).GetHeapObjectIfWeak(&heap_object));
+  CHECK(array->get(0).GetHeapObjectIfWeak(&heap_object));
   CHECK_EQ(Smi::cast(FixedArray::cast(heap_object)->get(0)).value(), 2016);
-  CHECK(array->Get(1).GetHeapObjectIfWeak(&heap_object));
+  CHECK(array->get(1).GetHeapObjectIfWeak(&heap_object));
   CHECK_EQ(Smi::cast(FixedArray::cast(heap_object)->get(0)).value(), 2017);
-  CHECK(array->Get(2).GetHeapObjectIfStrong(&heap_object));
+  CHECK(array->get(2).GetHeapObjectIfStrong(&heap_object));
   CHECK_EQ(Smi::cast(FixedArray::cast(heap_object)->get(0)).value(), 2018);
-  CHECK(array->Get(3).GetHeapObjectIfWeak(&heap_object));
+  CHECK(array->get(3).GetHeapObjectIfWeak(&heap_object));
   CHECK_EQ(Smi::cast(FixedArray::cast(heap_object)->get(0)).value(), 2019);
 
   heap::InvokeMajorGC(heap);
   CHECK(heap->InOldSpace(*array));
-  CHECK(array->Get(0)->IsCleared());
-  CHECK(array->Get(1).GetHeapObjectIfWeak(&heap_object));
+  CHECK(array->get(0)->IsCleared());
+  CHECK(array->get(1).GetHeapObjectIfWeak(&heap_object));
   CHECK_EQ(Smi::cast(FixedArray::cast(heap_object)->get(0)).value(), 2017);
-  CHECK(array->Get(2).GetHeapObjectIfStrong(&heap_object));
+  CHECK(array->get(2).GetHeapObjectIfStrong(&heap_object));
   CHECK_EQ(Smi::cast(FixedArray::cast(heap_object)->get(0)).value(), 2018);
-  CHECK(array->Get(3)->IsCleared());
+  CHECK(array->get(3)->IsCleared());
 }
 
 TEST(WeakArrayListBasic) {
@@ -481,16 +483,16 @@ TEST(WeakArrayListBasic) {
 
     CHECK(InCorrectGeneration(*array));
 
-    CHECK_EQ(array->Get(0), HeapObjectReference::Weak(*index0));
-    CHECK_EQ(array->Get(1).ToSmi().value(), 1);
+    CHECK_EQ(array->get(0), HeapObjectReference::Weak(*index0));
+    CHECK_EQ(array->get(1).ToSmi().value(), 1);
 
-    CHECK_EQ(array->Get(2), HeapObjectReference::Weak(*index2));
-    CHECK_EQ(array->Get(3).ToSmi().value(), 3);
+    CHECK_EQ(array->get(2), HeapObjectReference::Weak(*index2));
+    CHECK_EQ(array->get(3).ToSmi().value(), 3);
 
-    CHECK_EQ(array->Get(4), HeapObjectReference::Weak(*index4));
-    CHECK_EQ(array->Get(5).ToSmi().value(), 5);
+    CHECK_EQ(array->get(4), HeapObjectReference::Weak(*index4));
+    CHECK_EQ(array->get(5).ToSmi().value(), 5);
 
-    CHECK_EQ(array->Get(6), HeapObjectReference::Weak(*index6));
+    CHECK_EQ(array->get(6), HeapObjectReference::Weak(*index6));
     array = inner_scope.CloseAndEscape(array);
   }  // inner_scope goes out of scope.
 
@@ -503,37 +505,37 @@ TEST(WeakArrayListBasic) {
   heap::InvokeMinorGC(heap);
   Tagged<HeapObject> heap_object;
   CHECK_EQ(array->length(), 8);
-  CHECK(array->Get(0).GetHeapObjectIfWeak(&heap_object));
+  CHECK(array->get(0).GetHeapObjectIfWeak(&heap_object));
   CHECK_EQ(Smi::cast(FixedArray::cast(heap_object)->get(0)).value(), 2016);
-  CHECK_EQ(array->Get(1).ToSmi().value(), 1);
+  CHECK_EQ(array->get(1).ToSmi().value(), 1);
 
-  CHECK(array->Get(2).GetHeapObjectIfWeak(&heap_object));
+  CHECK(array->get(2).GetHeapObjectIfWeak(&heap_object));
   CHECK_EQ(Smi::cast(FixedArray::cast(heap_object)->get(0)).value(), 2017);
-  CHECK_EQ(array->Get(3).ToSmi().value(), 3);
+  CHECK_EQ(array->get(3).ToSmi().value(), 3);
 
-  CHECK(array->Get(4).GetHeapObjectIfWeak(&heap_object));
+  CHECK(array->get(4).GetHeapObjectIfWeak(&heap_object));
   CHECK_EQ(Smi::cast(FixedArray::cast(heap_object)->get(0)).value(), 2018);
-  CHECK_EQ(array->Get(5).ToSmi().value(), 5);
+  CHECK_EQ(array->get(5).ToSmi().value(), 5);
 
-  CHECK(array->Get(6).GetHeapObjectIfWeak(&heap_object));
+  CHECK(array->get(6).GetHeapObjectIfWeak(&heap_object));
   CHECK_EQ(Smi::cast(FixedArray::cast(heap_object)->get(0)).value(), 2019);
-  CHECK_EQ(array->Get(7).ToSmi().value(), 7);
+  CHECK_EQ(array->get(7).ToSmi().value(), 7);
 
   heap::InvokeMajorGC(heap);
   CHECK(heap->InOldSpace(*array));
   CHECK_EQ(array->length(), 8);
-  CHECK(array->Get(0)->IsCleared());
-  CHECK_EQ(array->Get(1).ToSmi().value(), 1);
+  CHECK(array->get(0)->IsCleared());
+  CHECK_EQ(array->get(1).ToSmi().value(), 1);
 
-  CHECK(array->Get(2).GetHeapObjectIfWeak(&heap_object));
+  CHECK(array->get(2).GetHeapObjectIfWeak(&heap_object));
   CHECK_EQ(Smi::cast(FixedArray::cast(heap_object)->get(0)).value(), 2017);
-  CHECK_EQ(array->Get(3).ToSmi().value(), 3);
+  CHECK_EQ(array->get(3).ToSmi().value(), 3);
 
-  CHECK(array->Get(4)->IsCleared());
-  CHECK_EQ(array->Get(5).ToSmi().value(), 5);
+  CHECK(array->get(4)->IsCleared());
+  CHECK_EQ(array->get(5).ToSmi().value(), 5);
 
-  CHECK(array->Get(6)->IsCleared());
-  CHECK_EQ(array->Get(7).ToSmi().value(), 7);
+  CHECK(array->get(6)->IsCleared());
+  CHECK_EQ(array->get(7).ToSmi().value(), 7);
 }
 
 TEST(WeakArrayListRemove) {
@@ -559,26 +561,26 @@ TEST(WeakArrayListRemove) {
       WeakArrayList::AddToEnd(isolate, array, MaybeObjectHandle::Weak(elem2));
 
   CHECK_EQ(array->length(), 3);
-  CHECK_EQ(array->Get(0), HeapObjectReference::Weak(*elem0));
-  CHECK_EQ(array->Get(1), HeapObjectReference::Weak(*elem1));
-  CHECK_EQ(array->Get(2), HeapObjectReference::Weak(*elem2));
+  CHECK_EQ(array->get(0), HeapObjectReference::Weak(*elem0));
+  CHECK_EQ(array->get(1), HeapObjectReference::Weak(*elem1));
+  CHECK_EQ(array->get(2), HeapObjectReference::Weak(*elem2));
 
   CHECK(array->RemoveOne(MaybeObjectHandle::Weak(elem1)));
 
   CHECK_EQ(array->length(), 2);
-  CHECK_EQ(array->Get(0), HeapObjectReference::Weak(*elem0));
-  CHECK_EQ(array->Get(1), HeapObjectReference::Weak(*elem2));
+  CHECK_EQ(array->get(0), HeapObjectReference::Weak(*elem0));
+  CHECK_EQ(array->get(1), HeapObjectReference::Weak(*elem2));
 
   CHECK(!array->RemoveOne(MaybeObjectHandle::Weak(elem1)));
 
   CHECK_EQ(array->length(), 2);
-  CHECK_EQ(array->Get(0), HeapObjectReference::Weak(*elem0));
-  CHECK_EQ(array->Get(1), HeapObjectReference::Weak(*elem2));
+  CHECK_EQ(array->get(0), HeapObjectReference::Weak(*elem0));
+  CHECK_EQ(array->get(1), HeapObjectReference::Weak(*elem2));
 
   CHECK(array->RemoveOne(MaybeObjectHandle::Weak(elem0)));
 
   CHECK_EQ(array->length(), 1);
-  CHECK_EQ(array->Get(0), HeapObjectReference::Weak(*elem2));
+  CHECK_EQ(array->get(0), HeapObjectReference::Weak(*elem2));
 
   CHECK(array->RemoveOne(MaybeObjectHandle::Weak(elem2)));
 
@@ -643,7 +645,8 @@ TEST(PrototypeUsersBasic) {
   // Add some objects into the array.
   int index = -1;
   {
-    Handle<Map> map = factory->NewMap(JS_OBJECT_TYPE, JSObject::kHeaderSize);
+    Handle<Map> map = factory->NewContextfulMapForCurrentContext(
+        JS_OBJECT_TYPE, JSObject::kHeaderSize);
     array = PrototypeUsers::Add(isolate, array, map, &index);
     CHECK_EQ(array->length(), index + 1);
   }
@@ -656,7 +659,8 @@ TEST(PrototypeUsersBasic) {
   int last_index = index;
   int old_capacity = array->capacity();
   while (!array->IsFull()) {
-    Handle<Map> map = factory->NewMap(JS_OBJECT_TYPE, JSObject::kHeaderSize);
+    Handle<Map> map = factory->NewContextfulMapForCurrentContext(
+        JS_OBJECT_TYPE, JSObject::kHeaderSize);
     array = PrototypeUsers::Add(isolate, array, map, &index);
     CHECK_EQ(index, last_index + 1);
     CHECK_EQ(array->length(), index + 1);
@@ -665,14 +669,16 @@ TEST(PrototypeUsersBasic) {
 
   // The next addition will fill the empty slot.
   {
-    Handle<Map> map = factory->NewMap(JS_OBJECT_TYPE, JSObject::kHeaderSize);
+    Handle<Map> map = factory->NewContextfulMapForCurrentContext(
+        JS_OBJECT_TYPE, JSObject::kHeaderSize);
     array = PrototypeUsers::Add(isolate, array, map, &index);
   }
   CHECK_EQ(index, empty_index);
 
   // The next addition will make the arrow grow again.
   {
-    Handle<Map> map = factory->NewMap(JS_OBJECT_TYPE, JSObject::kHeaderSize);
+    Handle<Map> map = factory->NewContextfulMapForCurrentContext(
+        JS_OBJECT_TYPE, JSObject::kHeaderSize);
     array = PrototypeUsers::Add(isolate, array, map, &index);
     CHECK_EQ(array->length(), index + 1);
     last_index = index;
@@ -688,7 +694,8 @@ TEST(PrototypeUsersBasic) {
   // Fill the array (still adding to the end)
   old_capacity = array->capacity();
   while (!array->IsFull()) {
-    Handle<Map> map = factory->NewMap(JS_OBJECT_TYPE, JSObject::kHeaderSize);
+    Handle<Map> map = factory->NewContextfulMapForCurrentContext(
+        JS_OBJECT_TYPE, JSObject::kHeaderSize);
     array = PrototypeUsers::Add(isolate, array, map, &index);
     CHECK_EQ(index, last_index + 1);
     CHECK_EQ(array->length(), index + 1);
@@ -697,13 +704,15 @@ TEST(PrototypeUsersBasic) {
 
   // Make sure we use the empty slots in (reverse) order.
   {
-    Handle<Map> map = factory->NewMap(JS_OBJECT_TYPE, JSObject::kHeaderSize);
+    Handle<Map> map = factory->NewContextfulMapForCurrentContext(
+        JS_OBJECT_TYPE, JSObject::kHeaderSize);
     array = PrototypeUsers::Add(isolate, array, map, &index);
   }
   CHECK_EQ(index, empty_index2);
 
   {
-    Handle<Map> map = factory->NewMap(JS_OBJECT_TYPE, JSObject::kHeaderSize);
+    Handle<Map> map = factory->NewContextfulMapForCurrentContext(
+        JS_OBJECT_TYPE, JSObject::kHeaderSize);
     array = PrototypeUsers::Add(isolate, array, map, &index);
   }
   CHECK_EQ(index, empty_index1);
@@ -737,17 +746,18 @@ TEST(PrototypeUsersCompacted) {
 
   // Add some objects into the array.
   int index = -1;
-  Handle<Map> map_cleared_by_user =
-      factory->NewMap(JS_OBJECT_TYPE, JSObject::kHeaderSize);
+  Handle<Map> map_cleared_by_user = factory->NewContextfulMapForCurrentContext(
+      JS_OBJECT_TYPE, JSObject::kHeaderSize);
   array = PrototypeUsers::Add(isolate, array, map_cleared_by_user, &index);
   CHECK_EQ(index, 1);
-  Handle<Map> live_map = factory->NewMap(JS_OBJECT_TYPE, JSObject::kHeaderSize);
+  Handle<Map> live_map = factory->NewContextfulMapForCurrentContext(
+      JS_OBJECT_TYPE, JSObject::kHeaderSize);
   array = PrototypeUsers::Add(isolate, array, live_map, &index);
   CHECK_EQ(index, 2);
   {
     HandleScope inner_scope(isolate);
-    Handle<Map> soon_dead_map =
-        factory->NewMap(JS_OBJECT_TYPE, JSObject::kHeaderSize);
+    Handle<Map> soon_dead_map = factory->NewContextfulMapForCurrentContext(
+        JS_OBJECT_TYPE, JSObject::kHeaderSize);
     array = PrototypeUsers::Add(isolate, array, soon_dead_map, &index);
     CHECK_EQ(index, 3);
 
@@ -756,7 +766,7 @@ TEST(PrototypeUsersCompacted) {
 
   PrototypeUsers::MarkSlotEmpty(*array, 1);
   heap::InvokeMajorGC(heap);
-  CHECK(array->Get(3)->IsCleared());
+  CHECK(array->get(3)->IsCleared());
 
   CHECK_EQ(array->length(), 3 + PrototypeUsers::kFirstIndex);
   Tagged<WeakArrayList> new_array =
