@@ -3,8 +3,12 @@
 // found in the LICENSE file.
 
 #include "src/api/api-inl.h"
+#include "src/base/logging.h"
 #include "src/base/strings.h"
+#include "src/common/globals.h"
 #include "src/objects/js-array-buffer-inl.h"
+#include "src/sandbox/sandbox.h"
+#include "test/cctest/heap/heap-utils.h"
 #include "test/cctest/test-api.h"
 #include "test/common/flag-utils.h"
 
@@ -67,7 +71,7 @@ THREADED_TEST(ArrayBuffer_ApiInternalToExternal) {
   Local<v8::ArrayBuffer> ab = v8::ArrayBuffer::New(isolate, 1024);
   CheckInternalFieldsAreZero(ab);
   CHECK_EQ(1024, ab->ByteLength());
-  CcTest::CollectAllGarbage();
+  i::heap::InvokeMajorGC(CcTest::heap());
 
   std::shared_ptr<v8::BackingStore> backing_store = Externalize(ab);
   CHECK_EQ(1024, backing_store->ByteLength());
@@ -306,7 +310,7 @@ THREADED_TEST(SharedArrayBuffer_ApiInternalToExternal) {
   Local<v8::SharedArrayBuffer> ab = v8::SharedArrayBuffer::New(isolate, 1024);
   CheckInternalFieldsAreZero(ab);
   CHECK_EQ(1024, ab->ByteLength());
-  CcTest::CollectAllGarbage();
+  i::heap::InvokeMajorGC(CcTest::heap());
 
   std::shared_ptr<v8::BackingStore> backing_store = Externalize(ab);
 
@@ -389,10 +393,9 @@ THREADED_TEST(SkipArrayBufferBackingStoreDuringGC) {
       v8::ArrayBuffer::New(isolate, std::move(backing_store));
 
   // Should not crash
-  CcTest::CollectGarbage(i::NEW_SPACE);  // in survivor space now
-  CcTest::CollectGarbage(i::NEW_SPACE);  // in old gen now
-  CcTest::CollectAllGarbage();
-  CcTest::CollectAllGarbage();
+  i::heap::EmptyNewSpaceUsingGC(CcTest::heap());
+  i::heap::InvokeMajorGC(CcTest::heap());
+  i::heap::InvokeMajorGC(CcTest::heap());
 
   // Should not move the pointer
   CHECK_EQ(ab->GetBackingStore()->Data(), store_ptr);
@@ -413,8 +416,7 @@ THREADED_TEST(SkipArrayBufferDuringScavenge) {
   auto backing_store = v8::ArrayBuffer::NewBackingStore(
       store_ptr, 8, [](void*, size_t, void*) {}, nullptr);
 
-  // Make `store_ptr` point to from space
-  CcTest::CollectGarbage(i::NEW_SPACE);
+  i::heap::InvokeMinorGC(CcTest::heap());
 
   // Create ArrayBuffer with pointer-that-cannot-be-visited in the backing store
   Local<v8::ArrayBuffer> ab =
@@ -422,8 +424,7 @@ THREADED_TEST(SkipArrayBufferDuringScavenge) {
 
   // Should not crash,
   // i.e. backing store pointer should not be treated as a heap object pointer
-  CcTest::CollectGarbage(i::NEW_SPACE);  // in survivor space now
-  CcTest::CollectGarbage(i::NEW_SPACE);  // in old gen now
+  i::heap::EmptyNewSpaceUsingGC(CcTest::heap());
 
   CHECK_EQ(ab->GetBackingStore()->Data(), store_ptr);
   CHECK_EQ(ab->Data(), store_ptr);
@@ -873,3 +874,30 @@ TEST(ArrayBuffer_FixedLength) {
   CHECK_EQ(32, sab->MaxByteLength());
   CHECK_EQ(sab->MaxByteLength(), sab->GetBackingStore()->MaxByteLength());
 }
+
+THREADED_TEST(ArrayBuffer_DataApiWithEmptyExternal) {
+  LocalContext env;
+  v8::Isolate* isolate = env->GetIsolate();
+  v8::HandleScope handle_scope(isolate);
+
+  Local<v8::ArrayBuffer> ab = v8::ArrayBuffer::New(isolate, 0);
+  void* expected_data_ptr = V8_ENABLE_SANDBOX_BOOL
+                                ? v8::internal::EmptyBackingStoreBuffer()
+                                : nullptr;
+  CHECK_EQ(expected_data_ptr, ab->Data());
+  CHECK_EQ(0, ab->ByteLength());
+  CHECK_NULL(ab->GetBackingStore()->Data());
+  // Repeat test to make sure that accessing the backing store buffer hasn't
+  // changed what sandboxed AB's Data method returns.
+  CHECK_EQ(expected_data_ptr, ab->Data());
+  CHECK_EQ(0, ab->ByteLength());
+
+  void* buffer = CcTest::array_buffer_allocator()->Allocate(1);
+  std::unique_ptr<v8::BackingStore> backing_store =
+      v8::ArrayBuffer::NewBackingStore(buffer, 0,
+                                       v8::BackingStore::EmptyDeleter, nullptr);
+  Local<v8::ArrayBuffer> ab2 =
+      v8::ArrayBuffer::New(isolate, std::move(backing_store));
+  CHECK_EQ(buffer, ab2->Data());
+  CHECK_EQ(0, ab->ByteLength());
+}
