@@ -50,7 +50,6 @@ void CheckPropBase(const d::PropertyBase& property, const char* expected_type,
                    const char* expected_name) {
   CHECK(property.type == std::string("v8::internal::TaggedValue") ||
         property.type == std::string(expected_type));
-  CHECK(property.decompressed_type == std::string(expected_type));
   CHECK(property.name == std::string(expected_name));
 }
 
@@ -126,6 +125,7 @@ TEST(GetObjectProperties) {
   CcTest::InitializeVM();
   v8::Isolate* isolate = CcTest::isolate();
   i::Isolate* i_isolate = reinterpret_cast<i::Isolate*>(isolate);
+  PtrComprCageAccessScope ptr_compr_cage_access_scope(i_isolate);
   v8::HandleScope scope(isolate);
   LocalContext context;
   // Claim we don't know anything about the heap layout.
@@ -146,11 +146,16 @@ TEST(GetObjectProperties) {
   CHECK(props->type_check_result == d::TypeCheckResult::kUsedMap);
   CHECK(props->type == std::string("v8::internal::JSArray"));
   CHECK_EQ(props->num_properties, 4);
-  CheckProp(*props->properties[0], "v8::internal::Map", "map");
-  CheckProp(*props->properties[1], "v8::internal::Object",
+  CheckProp(*props->properties[0],
+            "v8::internal::TaggedMember<v8::internal::Map>", "map");
+  CheckProp(*props->properties[1],
+            "v8::internal::TaggedMember<v8::internal::Object>",
             "properties_or_hash");
-  CheckProp(*props->properties[2], "v8::internal::FixedArrayBase", "elements");
-  CheckProp(*props->properties[3], "v8::internal::Object", "length",
+  CheckProp(*props->properties[2],
+            "v8::internal::TaggedMember<v8::internal::FixedArrayBase>",
+            "elements");
+  CheckProp(*props->properties[3],
+            "v8::internal::TaggedMember<v8::internal::Object>", "length",
             static_cast<i::Tagged_t>(IntToSmi(2)));
 
   // We need to supply some valid address for decompression before reading the
@@ -173,7 +178,8 @@ TEST(GetObjectProperties) {
           d::TypeCheckResult::kObjectPointerValidButInaccessible);
     CHECK(props->type == std::string("v8::internal::HeapObject"));
     CHECK_EQ(props->num_properties, 1);
-    CheckProp(*props->properties[0], "v8::internal::Map", "map");
+    CheckProp(*props->properties[0],
+              "v8::internal::TaggedMember<v8::internal::Map>", "map");
     // "maybe" prefix indicates that GetObjectProperties recognized the offset
     // within the page as matching a known object, but didn't know whether the
     // object is on the right page. This response can only happen in builds
@@ -192,7 +198,8 @@ TEST(GetObjectProperties) {
           d::TypeCheckResult::kObjectPointerValidButInaccessible);
     CHECK(props->type == std::string("v8::internal::HeapObject"));
     CHECK_EQ(props->num_properties, 1);
-    CheckProp(*props->properties[0], "v8::internal::Map", "map");
+    CheckProp(*props->properties[0],
+              "v8::internal::TaggedMember<v8::internal::Map>", "map");
     CHECK(StartsWith(props->brief, "EmptyFixedArray"));
   }
 
@@ -200,10 +207,13 @@ TEST(GetObjectProperties) {
   CHECK(props->type_check_result == d::TypeCheckResult::kUsedMap);
   CHECK(props->type == std::string("v8::internal::FixedArray"));
   CHECK_EQ(props->num_properties, 3);
-  CheckProp(*props->properties[0], "v8::internal::Map", "map");
-  CheckProp(*props->properties[1], "v8::internal::Object", "length",
+  CheckProp(*props->properties[0],
+            "v8::internal::TaggedMember<v8::internal::Map>", "map");
+  CheckProp(*props->properties[1],
+            "v8::internal::TaggedMember<v8::internal::Object>", "length",
             static_cast<i::Tagged_t>(IntToSmi(2)));
-  CheckProp(*props->properties[2], "v8::internal::Object", "objects",
+  CheckProp(*props->properties[2],
+            "v8::internal::TaggedMember<v8::internal::Object>", "objects",
             d::PropertyKind::kArrayOfKnownSize, 2);
 
   // Get the second string value from the FixedArray.
@@ -214,7 +224,8 @@ TEST(GetObjectProperties) {
   CHECK(props->type_check_result == d::TypeCheckResult::kUsedMap);
   CHECK(props->type == std::string("v8::internal::SeqOneByteString"));
   CHECK_EQ(props->num_properties, 4);
-  CheckProp(*props->properties[0], "v8::internal::Map", "map");
+  CheckProp(*props->properties[0],
+            "v8::internal::TaggedMember<v8::internal::Map>", "map");
   CheckProp(*props->properties[1], "uint32_t", "raw_hash_field");
   CheckProp(*props->properties[2], "int32_t", "length", 2);
   CheckProp(*props->properties[3], "char", "chars",
@@ -266,7 +277,8 @@ TEST(GetObjectProperties) {
       CHECK(std::string(props2->guessed_types[0]) ==
             std::string("v8::internal::SeqOneByteString"));
     }
-    CheckProp(*props2->properties[0], "v8::internal::Map", "map",
+    CheckProp(*props2->properties[0],
+              "v8::internal::TaggedMember<v8::internal::Map>", "map",
               *reinterpret_cast<i::Tagged_t*>(props->properties[0]->address));
     CheckProp(*props2->properties[1], "uint32_t", "raw_hash_field",
               *reinterpret_cast<int32_t*>(props->properties[1]->address));
@@ -281,7 +293,8 @@ TEST(GetObjectProperties) {
   CHECK(props2->type_check_result == d::TypeCheckResult::kUsedMap);
   CHECK(props2->type == std::string("v8::internal::SeqOneByteString"));
   CHECK_EQ(props2->num_properties, 4);
-  CheckProp(*props2->properties[0], "v8::internal::Map", "map",
+  CheckProp(*props2->properties[0],
+            "v8::internal::TaggedMember<v8::internal::Map>", "map",
             *reinterpret_cast<i::Tagged_t*>(props->properties[0]->address));
   CheckProp(*props2->properties[1], "uint32_t", "raw_hash_field",
             *reinterpret_cast<i::Tagged_t*>(props->properties[1]->address));
@@ -299,7 +312,8 @@ TEST(GetObjectProperties) {
   // Cause a failure when reading the "second" pointer within the top-level
   // ConsString.
   {
-    CheckProp(*props->properties[4], "v8::internal::String", "second");
+    CheckProp(*props->properties[4],
+              "v8::internal::TaggedMember<v8::internal::String>", "second");
     uintptr_t second_address = props->properties[4]->address;
     MemoryFailureRegion failure(second_address, second_address + 4);
     props = d::GetObjectProperties((*o).ptr(), &ReadMemory, heap_addresses);
@@ -338,8 +352,8 @@ TEST(GetObjectProperties) {
   // Objects constructed from literals get their properties placed inline, so
   // the GetObjectProperties response should include an array.
   const d::ObjectProperty& prop = FindProp(*props, "in-object properties");
-  CheckProp(prop, "v8::internal::Object", "in-object properties",
-            d::PropertyKind::kArrayOfKnownSize, 2);
+  CheckProp(prop, "v8::internal::TaggedMember<v8::internal::Object>",
+            "in-object properties", d::PropertyKind::kArrayOfKnownSize, 2);
   // The second item in that array is the SMI value 2 from the object literal.
   props2 =
       d::GetObjectProperties(reinterpret_cast<i::Tagged_t*>(prop.address)[1],
@@ -372,13 +386,16 @@ TEST(GetObjectProperties) {
             d::PropertyKind::kArrayOfKnownSize, number_of_all_descriptors);
   CHECK_EQ(descriptors.size, 3 * i::kTaggedSize);
   CHECK_EQ(descriptors.num_struct_fields, 3);
-  CheckStructProp(*descriptors.struct_fields[0],
-                  "v8::internal::PrimitiveHeapObject", "key",
-                  0 * i::kTaggedSize);
-  CheckStructProp(*descriptors.struct_fields[1], "v8::internal::Object",
-                  "details", 1 * i::kTaggedSize);
-  CheckStructProp(*descriptors.struct_fields[2], "v8::internal::Object",
-                  "value", 2 * i::kTaggedSize);
+  CheckStructProp(
+      *descriptors.struct_fields[0],
+      "v8::internal::TaggedMember<v8::internal::PrimitiveHeapObject>", "key",
+      0 * i::kTaggedSize);
+  CheckStructProp(*descriptors.struct_fields[1],
+                  "v8::internal::TaggedMember<v8::internal::Object>", "details",
+                  1 * i::kTaggedSize);
+  CheckStructProp(*descriptors.struct_fields[2],
+                  "v8::internal::TaggedMember<v8::internal::Object>", "value",
+                  2 * i::kTaggedSize);
 
   // Build a basic JS function and get its properties. This will allow us to
   // exercise bitfield functionality.
@@ -410,23 +427,6 @@ TEST(GetObjectProperties) {
                   0, 1, i::kSmiTagSize + i::kSmiShiftSize + 4);
 }
 
-TEST(ListObjectClasses) {
-  CcTest::InitializeVM();
-
-  // The ListObjectClasses result will change as classes are added, removed, or
-  // renamed. Just check that a few expected classes are included in the list,
-  // and that there are no duplicates.
-  const d::ClassList* class_list = d::ListObjectClasses();
-  std::unordered_set<std::string> class_set;
-  for (size_t i = 0; i < class_list->num_class_names; ++i) {
-    CHECK_WITH_MSG(class_set.insert(class_list->class_names[i]).second,
-                   "there should be no duplicate entries");
-  }
-  CHECK_NE(class_set.find("v8::internal::HeapObject"), class_set.end());
-  CHECK_NE(class_set.find("v8::internal::String"), class_set.end());
-  CHECK_NE(class_set.find("v8::internal::JSRegExp"), class_set.end());
-}
-
 static void FrameIterationCheck(
     v8::Local<v8::String> name,
     const v8::PropertyCallbackInfo<v8::Value>& info) {
@@ -439,17 +439,24 @@ static void FrameIterationCheck(
       JavaScriptFrame* js_frame = JavaScriptFrame::cast(frame);
       CHECK_EQ(props->num_properties, 5);
       auto js_function = js_frame->function();
-      CheckProp(*props->properties[0], "v8::internal::JSFunction",
+      // This one is Tagged, not TaggedMember, because it's from the stack.
+      CheckProp(*props->properties[0],
+                "v8::internal::Tagged<v8::internal::JSFunction>",
                 "currently_executing_jsfunction", js_function.ptr());
       auto shared_function_info = js_function->shared();
       auto script = i::Script::cast(shared_function_info->script());
-      CheckProp(*props->properties[1], "v8::internal::Object", "script_name",
-                static_cast<i::Tagged_t>(script->name().ptr()));
-      CheckProp(*props->properties[2], "v8::internal::Object", "script_source",
+      CheckProp(*props->properties[1],
+                "v8::internal::TaggedMember<v8::internal::Object>",
+                "script_name", static_cast<i::Tagged_t>(script->name().ptr()));
+      CheckProp(*props->properties[2],
+                "v8::internal::TaggedMember<v8::internal::Object>",
+                "script_source",
                 static_cast<i::Tagged_t>(script->source().ptr()));
 
       auto scope_info = shared_function_info->scope_info();
-      CheckProp(*props->properties[3], "v8::internal::Object", "function_name",
+      CheckProp(*props->properties[3],
+                "v8::internal::TaggedMember<v8::internal::Object>",
+                "function_name",
                 static_cast<i::Tagged_t>(scope_info->FunctionName().ptr()));
 
       CheckProp(*props->properties[4], "", "function_character_offset");
@@ -457,9 +464,11 @@ static void FrameIterationCheck(
           *props->properties[4];
       CHECK_EQ(function_character_offset.num_struct_fields, 2);
       CheckStructProp(*function_character_offset.struct_fields[0],
-                      "v8::internal::Object", "start", 0);
+                      "v8::internal::TaggedMember<v8::internal::Object>",
+                      "start", 0);
       CheckStructProp(*function_character_offset.struct_fields[1],
-                      "v8::internal::Object", "end", 4);
+                      "v8::internal::TaggedMember<v8::internal::Object>", "end",
+                      4);
     } else {
       CHECK_EQ(props->num_properties, 0);
     }
@@ -470,6 +479,8 @@ static void FrameIterationCheck(
 THREADED_TEST(GetFrameStack) {
   LocalContext env;
   v8::Isolate* isolate = env->GetIsolate();
+  i::Isolate* i_isolate = reinterpret_cast<Isolate*>(isolate);
+  PtrComprCageAccessScope ptr_compr_cage_access_scope(i_isolate);
   v8::HandleScope scope(isolate);
   v8::Local<v8::ObjectTemplate> obj = v8::ObjectTemplate::New(isolate);
   obj->SetAccessor(v8_str("xxx"), FrameIterationCheck);
@@ -490,6 +501,7 @@ TEST(SmallOrderedHashSetGetObjectProperties) {
   LocalContext context;
   Isolate* isolate = reinterpret_cast<Isolate*>((*context)->GetIsolate());
   Factory* factory = isolate->factory();
+  PtrComprCageAccessScope ptr_compr_cage_access_scope(isolate);
   HandleScope scope(isolate);
 
   Handle<SmallOrderedHashSet> set = factory->NewSmallOrderedHashSet();
@@ -506,7 +518,8 @@ TEST(SmallOrderedHashSetGetObjectProperties) {
   CHECK_EQ(props->type, std::string("v8::internal::SmallOrderedHashSet"));
   CHECK_EQ(props->num_properties, 8);
 
-  CheckProp(*props->properties[0], "v8::internal::Map", "map");
+  CheckProp(*props->properties[0],
+            "v8::internal::TaggedMember<v8::internal::Map>", "map");
   CheckProp(*props->properties[1], "uint8_t", "number_of_elements");
   CheckProp(*props->properties[2], "uint8_t", "number_of_deleted_elements");
   CheckProp(*props->properties[3], "uint8_t", "number_of_buckets");
@@ -517,7 +530,8 @@ TEST(SmallOrderedHashSetGetObjectProperties) {
   CheckProp(*props->properties[4], "uint8_t", "padding",
             d::PropertyKind::kArrayOfKnownSize, 1);
 #endif
-  CheckProp(*props->properties[5], "v8::internal::Object", "data_table",
+  CheckProp(*props->properties[5],
+            "v8::internal::TaggedMember<v8::internal::Object>", "data_table",
             d::PropertyKind::kArrayOfKnownSize,
             number_of_buckets * OrderedHashMap::kLoadFactor);
   CheckProp(*props->properties[6], "uint8_t", "hash_table",
