@@ -6,6 +6,7 @@
 
 #include "include/v8-exception.h"
 #include "include/v8-local-handle.h"
+#include "include/v8-memory-span.h"
 #include "include/v8-primitive.h"
 #include "include/v8-value.h"
 #include "src/wasm/wasm-module-builder.h"
@@ -286,8 +287,6 @@ class FastJSWasmCallTester {
         zone_(&allocator_, ZONE_NAME),
         builder_(zone_.New<WasmModuleBuilder>(&zone_)),
         old_budget_(i::v8_flags.invocation_count_for_turbofan) {
-    i::v8_flags.experimental_wasm_typed_funcref = true;
-    i::v8_flags.experimental_wasm_typed_funcref = true;
     i::v8_flags.allow_natives_syntax = true;
     i::v8_flags.turbo_inline_js_wasm_calls = true;
     i::v8_flags.stress_background_compile = false;
@@ -326,7 +325,7 @@ class FastJSWasmCallTester {
   // Executes a test function that returns a value of type T.
   template <typename T>
   void CallAndCheckWasmFunction(const std::string& exported_function_name,
-                                const std::vector<v8::Local<v8::Value>>& args,
+                                v8::MemorySpan<v8::Local<v8::Value>> args,
                                 const T& expected_result,
                                 bool test_lazy_deopt = false) {
     LocalContext env;
@@ -344,10 +343,9 @@ class FastJSWasmCallTester {
   }
 
   // Executes a test function that returns NaN.
-  void CallAndCheckWasmFunctionNaN(
-      const std::string& exported_function_name,
-      const std::vector<v8::Local<v8::Value>>& args,
-      bool test_lazy_deopt = false) {
+  void CallAndCheckWasmFunctionNaN(const std::string& exported_function_name,
+                                   v8::MemorySpan<v8::Local<v8::Value>> args,
+                                   bool test_lazy_deopt = false) {
     LocalContext env;
     v8::Local<v8::Value> result_value = DoCallAndCheckWasmFunction(
         env, exported_function_name, args, test_lazy_deopt);
@@ -361,7 +359,7 @@ class FastJSWasmCallTester {
   // Executes a test function that returns a BigInt.
   void CallAndCheckWasmFunctionBigInt(
       const std::string& exported_function_name,
-      const std::vector<v8::Local<v8::Value>>& args,
+      v8::MemorySpan<v8::Local<v8::Value>> args,
       const v8::Local<v8::BigInt> expected_result,
       bool test_lazy_deopt = false) {
     LocalContext env;
@@ -376,7 +374,7 @@ class FastJSWasmCallTester {
 
   // Executes a test function that returns void.
   void CallAndCheckWasmFunction(const std::string& exported_function_name,
-                                const std::vector<v8::Local<v8::Value>>& args,
+                                v8::MemorySpan<v8::Local<v8::Value>> args,
                                 bool test_lazy_deopt = false) {
     LocalContext env;
     v8::Local<v8::Value> result_value = DoCallAndCheckWasmFunction(
@@ -525,7 +523,7 @@ class FastJSWasmCallTester {
   // void.
   void CallAndCheckWithTryCatch_void(
       const std::string& exported_function_name,
-      const std::vector<v8::Local<v8::Value>>& args) {
+      v8::MemorySpan<v8::Local<v8::Value>> args) {
     LocalContext env;
     for (size_t i = 0; i < args.size(); i++) {
       CHECK((*env)
@@ -605,8 +603,7 @@ class FastJSWasmCallTester {
 
   v8::Local<v8::Value> DoCallAndCheckWasmFunction(
       LocalContext& env, const std::string& exported_function_name,
-      const std::vector<v8::Local<v8::Value>>& args,
-      bool test_lazy_deopt = false) {
+      v8::MemorySpan<v8::Local<v8::Value>> args, bool test_lazy_deopt = false) {
     for (size_t i = 0; i < args.size(); i++) {
       CHECK((*env)
                 ->Global()
@@ -800,38 +797,41 @@ TEST(TestFastJSWasmCall_I32Arg) {
   v8::HandleScope scope(CcTest::isolate());
   FastJSWasmCallTester tester;
   tester.AddExportedFunction(k_i32_square);
-  tester.CallAndCheckWasmFunction<int32_t>("i32_square", {v8_num(42)}, 42 * 42);
+  auto args = v8::to_array<v8::Local<v8::Value>>({v8_num(42)});
+  tester.CallAndCheckWasmFunction<int32_t>("i32_square", args, 42 * 42);
 }
 
 TEST(TestFastJSWasmCall_I32ArgNotSmi) {
   v8::HandleScope scope(CcTest::isolate());
   FastJSWasmCallTester tester;
   tester.AddExportedFunction(k_add);
-  tester.CallAndCheckWasmFunction<int32_t>(
-      "add", {v8_num(0x7fffffff), v8_int(1)}, 0x80000000);
+  auto args =
+      v8::to_array<v8::Local<v8::Value>>({v8_num(0x7fffffff), v8_int(1)});
+  tester.CallAndCheckWasmFunction<int32_t>("add", args, 0x80000000);
 }
 
 TEST(TestFastJSWasmCall_F32Arg) {
   v8::HandleScope scope(CcTest::isolate());
   FastJSWasmCallTester tester;
   tester.AddExportedFunction(k_f32_square);
-  tester.CallAndCheckWasmFunction<float>("f32_square", {v8_num(42.0)},
-                                         42.0 * 42.0);
+  auto args = v8::to_array<v8::Local<v8::Value>>({v8_num(42.0)});
+  tester.CallAndCheckWasmFunction<float>("f32_square", args, 42.0 * 42.0);
 }
 
 TEST(TestFastJSWasmCall_F64Arg) {
   v8::HandleScope scope(CcTest::isolate());
   FastJSWasmCallTester tester;
   tester.AddExportedFunction(k_f64_square);
-  tester.CallAndCheckWasmFunction<double>("f64_square", {v8_num(42.0)},
-                                          42.0 * 42.0);
+  auto args = v8::to_array<v8::Local<v8::Value>>({v8_num(42.0)});
+  tester.CallAndCheckWasmFunction<double>("f64_square", args, 42.0 * 42.0);
 }
 
 TEST(TestFastJSWasmCall_I64Arg) {
   v8::HandleScope scope(CcTest::isolate());
   FastJSWasmCallTester tester;
   tester.AddExportedFunction(k_i64_square);
-  tester.CallAndCheckWasmFunctionBigInt("i64_square", {v8_bigint(1234567890ll)},
+  auto args = v8::to_array<v8::Local<v8::Value>>({v8_bigint(1234567890ll)});
+  tester.CallAndCheckWasmFunctionBigInt("i64_square", args,
                                         v8_bigint(1234567890ll * 1234567890ll));
 }
 
@@ -839,8 +839,9 @@ TEST(TestFastJSWasmCall_I64NegativeResult) {
   v8::HandleScope scope(CcTest::isolate());
   FastJSWasmCallTester tester;
   tester.AddExportedFunction(k_i64_add);
-  tester.CallAndCheckWasmFunctionBigInt(
-      "i64_add", {v8_bigint(1ll), v8_bigint(-2ll)}, v8_bigint(-1ll));
+  auto args =
+      v8::to_array<v8::Local<v8::Value>>({v8_bigint(1ll), v8_bigint(-2ll)});
+  tester.CallAndCheckWasmFunctionBigInt("i64_add", args, v8_bigint(-1ll));
 }
 
 TEST(TestFastJSWasmCall_ExternrefNullArg) {
@@ -848,43 +849,49 @@ TEST(TestFastJSWasmCall_ExternrefNullArg) {
   FastJSWasmCallTester tester;
   tester.AddExportedFunction(k_externref_null_id);
   Local<Primitive> v8_null = v8::Null(CcTest::isolate());
-  tester.CallAndCheckWasmFunction("externref_null_id", {v8_null}, nullptr);
-  tester.CallAndCheckWasmFunction("externref_null_id", {v8_num(42)}, 42);
-  tester.CallAndCheckWasmFunctionBigInt("externref_null_id", {v8_bigint(42)},
+  auto args1 = v8::to_array<v8::Local<v8::Value>>({v8_null});
+  tester.CallAndCheckWasmFunction("externref_null_id", args1, nullptr);
+  auto args2 = v8::to_array<v8::Local<v8::Value>>({v8_num(42)});
+  tester.CallAndCheckWasmFunction("externref_null_id", args2, 42);
+  auto args3 = v8::to_array<v8::Local<v8::Value>>({v8_bigint(42)});
+  tester.CallAndCheckWasmFunctionBigInt("externref_null_id", args3,
                                         v8_bigint(42));
   auto str = v8_str("test");
-  tester.CallAndCheckWasmFunction("externref_null_id", {str}, str);
+  auto args4 = v8::to_array<v8::Local<v8::Value>>({str});
+  tester.CallAndCheckWasmFunction("externref_null_id", args4, str);
 }
 
 TEST(TestFastJSWasmCall_ExternrefArg) {
   v8::HandleScope scope(CcTest::isolate());
   FastJSWasmCallTester tester;
   tester.AddExportedFunction(k_externref_id);
-  tester.CallAndCheckWasmFunction("externref_id", {v8_num(42)}, 42);
-  tester.CallAndCheckWasmFunctionBigInt("externref_id", {v8_bigint(42)},
-                                        v8_bigint(42));
+  auto args1 = v8::to_array<v8::Local<v8::Value>>({v8_num(42)});
+  tester.CallAndCheckWasmFunction("externref_id", args1, 42);
+  auto args2 = v8::to_array<v8::Local<v8::Value>>({v8_bigint(42)});
+  tester.CallAndCheckWasmFunctionBigInt("externref_id", args2, v8_bigint(42));
   auto str = v8_str("test");
-  tester.CallAndCheckWasmFunction("externref_id", {str}, str);
+  auto args3 = v8::to_array<v8::Local<v8::Value>>({str});
+  tester.CallAndCheckWasmFunction("externref_id", args3, str);
 }
 
 TEST(TestFastJSWasmCall_MultipleArgs) {
   v8::HandleScope scope(CcTest::isolate());
   FastJSWasmCallTester tester;
   tester.AddExportedFunction(k_sum10);
-  tester.CallAndCheckWasmFunction<int32_t>(
-      "sum10",
+  auto args = v8::to_array<v8::Local<v8::Value>>(
       {v8_num(1), v8_num(2), v8_num(3), v8_num(4), v8_num(5), v8_num(6),
-       v8_num(7), v8_num(8), v8_num(9), v8_num(10)},
-      55);
+       v8_num(7), v8_num(8), v8_num(9), v8_num(10)});
+  tester.CallAndCheckWasmFunction<int32_t>("sum10", args, 55);
 }
 
 TEST(TestFastJSWasmCall_MixedArgs) {
   v8::HandleScope scope(CcTest::isolate());
   FastJSWasmCallTester tester;
   tester.AddExportedFunction(k_sum_mixed);
-  tester.CallAndCheckWasmFunction<double>(
-      "sum_mixed", {v8_num(1), v8_bigint(0x80000000), v8_num(42.0), v8_num(.5)},
-      1 + 0x80000000 + 42 + .5);
+  auto args = v8::to_array<v8::Local<v8::Value>>(
+      {v8_num(1), v8_bigint(0x80000000), v8_num(42.0), v8_num(.5)});
+  tester.CallAndCheckWasmFunction<double>("sum_mixed", args,
+                                          1 + 0x80000000 + 42 + .5);
 }
 
 TEST(TestFastJSWasmCall_MistypedArgs) {
@@ -892,7 +899,8 @@ TEST(TestFastJSWasmCall_MistypedArgs) {
   FastJSWasmCallTester tester;
 
   tester.AddExportedFunction(k_i32_square);
-  tester.CallAndCheckWasmFunction<int32_t>("i32_square", {v8_str("test")}, 0);
+  auto args = v8::to_array<v8::Local<v8::Value>>({v8_str("test")});
+  tester.CallAndCheckWasmFunction<int32_t>("i32_square", args, 0);
 }
 
 TEST(TestFastJSWasmCall_MixedMistypedArgs) {
@@ -900,9 +908,10 @@ TEST(TestFastJSWasmCall_MixedMistypedArgs) {
   FastJSWasmCallTester tester;
 
   tester.AddExportedFunction(k_sum_mixed);
-  tester.CallAndCheckWasmFunctionNaN(
-      "sum_mixed", {v8_str("alpha"), v8_bigint(0x80000000), v8_str("beta"),
-                    v8_str("gamma")});
+  auto args = v8::to_array<v8::Local<v8::Value>>(
+      {v8_str("alpha"), v8_bigint(0x80000000), v8_str("beta"),
+       v8_str("gamma")});
+  tester.CallAndCheckWasmFunctionNaN("sum_mixed", args);
 }
 
 TEST(TestFastJSWasmCall_NoArgs) {
@@ -918,7 +927,8 @@ TEST(TestFastJSWasmCall_NoReturnTypes) {
   FastJSWasmCallTester tester;
 
   tester.AddExportedFunction(k_void_square);
-  tester.CallAndCheckWasmFunction("void_square", {v8_num(42)});
+  auto args = v8::to_array<v8::Local<v8::Value>>({v8_num(42)});
+  tester.CallAndCheckWasmFunction("void_square", args);
 }
 
 TEST(TestFastJSWasmCall_MismatchedArity) {
@@ -926,10 +936,11 @@ TEST(TestFastJSWasmCall_MismatchedArity) {
   FastJSWasmCallTester tester;
 
   tester.AddExportedFunction(k_sum3);
-  tester.CallAndCheckWasmFunction<int32_t>("sum3", {v8_num(1), v8_num(2)}, 3);
-  tester.CallAndCheckWasmFunction<int32_t>(
-      "sum3",
-      {v8_num(1), v8_num(2), v8_num(3), v8_num(4), v8_num(5), v8_num(6)}, 6);
+  auto args1 = v8::to_array<v8::Local<v8::Value>>({v8_num(1), v8_num(2)});
+  tester.CallAndCheckWasmFunction<int32_t>("sum3", args1, 3);
+  auto args2 = v8::to_array<v8::Local<v8::Value>>(
+      {v8_num(1), v8_num(2), v8_num(3), v8_num(4), v8_num(5), v8_num(6)});
+  tester.CallAndCheckWasmFunction<int32_t>("sum3", args2, 6);
   tester.CallAndCheckWasmFunction<int32_t>("sum3", {}, 0);
 }
 
@@ -940,7 +951,8 @@ TEST(TestFastJSWasmCall_LazyDeopt_I32Result) {
   FastJSWasmCallTester tester;
   tester.DeclareCallback("callback", sigs.v_d(), "env");
   tester.AddExportedFunction(k_i32_square_deopt);
-  tester.CallAndCheckWasmFunction<int32_t>("i32_square_deopt", {v8_num(42)},
+  auto args = v8::to_array<v8::Local<v8::Value>>({v8_num(42)});
+  tester.CallAndCheckWasmFunction<int32_t>("i32_square_deopt", args,
                                            43 * 43 + 1, true);
 }
 
@@ -950,13 +962,15 @@ TEST(TestFastJSWasmCall_LazyDeopt_I64Result) {
   tester.DeclareCallback("callback", sigs.v_d(), "env");
   tester.AddExportedFunction(k_i64_square_deopt);
 
-  tester.CallAndCheckWasmFunctionBigInt("i64_square_deopt", {v8_bigint(42)},
+  auto args1 = v8::to_array<v8::Local<v8::Value>>({v8_bigint(42)});
+  tester.CallAndCheckWasmFunctionBigInt("i64_square_deopt", args1,
                                         v8_bigint(43 * 43 + 1), true);
 
   // This test would fail if the result was converted into a HeapNumber through
   // a double, losing precision.
+  auto args2 = v8::to_array<v8::Local<v8::Value>>({v8_bigint(1234567890ll)});
   tester.CallAndCheckWasmFunctionBigInt(
-      "i64_square_deopt", {v8_bigint(1234567890ll)},
+      "i64_square_deopt", args2,
       v8_bigint(1524157877488187882ll),  // (1234567890 + 1)*(1234567890 + 1)+1
       true);
 }
@@ -966,8 +980,9 @@ TEST(TestFastJSWasmCall_LazyDeopt_F32Result) {
   FastJSWasmCallTester tester;
   tester.DeclareCallback("callback", sigs.v_d(), "env");
   tester.AddExportedFunction(k_f32_square_deopt);
-  tester.CallAndCheckWasmFunction<float>("f32_square_deopt", {v8_num(42.0)},
-                                         43 * 43 + 1, true);
+  auto args = v8::to_array<v8::Local<v8::Value>>({v8_num(42.0)});
+  tester.CallAndCheckWasmFunction<float>("f32_square_deopt", args, 43 * 43 + 1,
+                                         true);
 }
 
 TEST(TestFastJSWasmCall_LazyDeopt_F64Result) {
@@ -975,8 +990,9 @@ TEST(TestFastJSWasmCall_LazyDeopt_F64Result) {
   FastJSWasmCallTester tester;
   tester.DeclareCallback("callback", sigs.v_d(), "env");
   tester.AddExportedFunction(k_f64_square_deopt);
-  tester.CallAndCheckWasmFunction<float>("f64_square_deopt", {v8_num(42.0)},
-                                         43 * 43 + 1, true);
+  auto args = v8::to_array<v8::Local<v8::Value>>({v8_num(42.0)});
+  tester.CallAndCheckWasmFunction<float>("f64_square_deopt", args, 43 * 43 + 1,
+                                         true);
 }
 
 TEST(TestFastJSWasmCall_LazyDeopt_VoidResult) {
@@ -984,7 +1000,8 @@ TEST(TestFastJSWasmCall_LazyDeopt_VoidResult) {
   FastJSWasmCallTester tester;
   tester.DeclareCallback("callback", sigs.v_d(), "env");
   tester.AddExportedFunction(k_void_square_deopt);
-  tester.CallAndCheckWasmFunction("void_square_deopt", {v8_num(42.0)}, true);
+  auto args = v8::to_array<v8::Local<v8::Value>>({v8_num(42.0)});
+  tester.CallAndCheckWasmFunction("void_square_deopt", args, true);
 }
 
 // Eager deoptimization tests
@@ -1012,36 +1029,37 @@ TEST(TestFastJSWasmCall_Trap_i32) {
   v8::HandleScope scope(CcTest::isolate());
   FastJSWasmCallTester tester;
   tester.AddExportedFunction(k_load_i32);
-  tester.CallAndCheckWithTryCatch("load_i32", {v8_int(0x7fffffff)});
+  tester.CallAndCheckWithTryCatch("load_i32", v8_int(0x7fffffff));
 }
 
 TEST(TestFastJSWasmCall_Trap_i64) {
   v8::HandleScope scope(CcTest::isolate());
   FastJSWasmCallTester tester;
   tester.AddExportedFunction(k_load_i64);
-  tester.CallAndCheckWithTryCatch("load_i64", {v8_bigint(0x7fffffff)});
+  tester.CallAndCheckWithTryCatch("load_i64", v8_bigint(0x7fffffff));
 }
 
 TEST(TestFastJSWasmCall_Trap_f32) {
   v8::HandleScope scope(CcTest::isolate());
   FastJSWasmCallTester tester;
   tester.AddExportedFunction(k_load_f32);
-  tester.CallAndCheckWithTryCatch("load_f32", {v8_num(0x7fffffff)});
+  tester.CallAndCheckWithTryCatch("load_f32", v8_num(0x7fffffff));
 }
 
 TEST(TestFastJSWasmCall_Trap_f64) {
   v8::HandleScope scope(CcTest::isolate());
   FastJSWasmCallTester tester;
   tester.AddExportedFunction(k_load_f64);
-  tester.CallAndCheckWithTryCatch("load_f64", {v8_num(0x7fffffff)});
+  tester.CallAndCheckWithTryCatch("load_f64", v8_num(0x7fffffff));
 }
 
 TEST(TestFastJSWasmCall_Trap_void) {
   v8::HandleScope scope(CcTest::isolate());
   FastJSWasmCallTester tester;
   tester.AddExportedFunction(k_store_i32);
-  tester.CallAndCheckWithTryCatch_void("store_i32",
-                                       {v8_int(0x7fffffff), v8_int(42)});
+  auto args =
+      v8::to_array<v8::Local<v8::Value>>({v8_int(0x7fffffff), v8_int(42)});
+  tester.CallAndCheckWithTryCatch_void("store_i32", args);
 }
 
 // BigInt
