@@ -2,10 +2,6 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-#if !defined(_WIN32) && !defined(_WIN64)
-#include <unistd.h>
-#endif  // !defined(_WIN32) && !defined(_WIN64)
-
 #include <locale.h>
 
 #include <string>
@@ -26,6 +22,10 @@
 #include "test/inspector/task-runner.h"
 #include "test/inspector/tasks.h"
 
+#if !defined(V8_OS_WIN)
+#include <unistd.h>
+#endif  // !defined(V8_OS_WIN)
+
 namespace v8 {
 namespace internal {
 namespace {
@@ -78,8 +78,6 @@ class UtilsExtension : public InspectorIsolateData::SetupGlobalTask {
     backend_runner_ = runner;
   }
 
-  static void ClearAllSessions() { channels_.clear(); }
-
  private:
   static TaskRunner* backend_runner_;
 
@@ -169,32 +167,46 @@ class UtilsExtension : public InspectorIsolateData::SetupGlobalTask {
                 });
   }
 
+  static bool IsValidConnectSessionArgs(
+      const v8::FunctionCallbackInfo<v8::Value>& info) {
+    if (info.Length() < 3 || info.Length() > 4) return false;
+    if (!info[0]->IsInt32() || !info[1]->IsString() || !info[2]->IsFunction()) {
+      return false;
+    }
+    return info.Length() == 3 || info[3]->IsBoolean();
+  }
+
   static void ConnectSession(const v8::FunctionCallbackInfo<v8::Value>& info) {
     DCHECK(ValidateCallbackInfo(info));
-    if (info.Length() != 3 || !info[0]->IsInt32() || !info[1]->IsString() ||
-        !info[2]->IsFunction()) {
-      return;
-    }
+    if (!IsValidConnectSessionArgs(info)) return;
     v8::Local<v8::Context> context = info.GetIsolate()->GetCurrentContext();
-    FrontendChannelImpl* channel = new FrontendChannelImpl(
-        InspectorIsolateData::FromContext(context)->task_runner(),
-        InspectorIsolateData::FromContext(context)->GetContextGroupId(context),
-        info.GetIsolate(), info[2].As<v8::Function>());
+    std::unique_ptr<FrontendChannelImpl> channel =
+        std::make_unique<FrontendChannelImpl>(
+            InspectorIsolateData::FromContext(context)->task_runner(),
+            InspectorIsolateData::FromContext(context)->GetContextGroupId(
+                context),
+            info.GetIsolate(), info[2].As<v8::Function>());
 
     std::vector<uint8_t> state =
         ToBytes(info.GetIsolate(), info[1].As<v8::String>());
     int context_group_id = info[0].As<v8::Int32>()->Value();
-    int session_id = 0;
-    RunSyncTask(backend_runner_, [&context_group_id, &session_id, &channel,
-                                  &state](InspectorIsolateData* data) {
-      session_id = data->ConnectSession(
-          context_group_id,
-          v8_inspector::StringView(state.data(), state.size()), channel);
-      channel->set_session_id(session_id);
-    });
+    bool is_fully_trusted =
+        info.Length() == 3 || info[3].As<v8::Boolean>()->Value();
+    base::Optional<int> session_id;
+    RunSyncTask(backend_runner_,
+                [context_group_id, &session_id, &channel, &state,
+                 is_fully_trusted](InspectorIsolateData* data) {
+                  session_id = data->ConnectSession(
+                      context_group_id,
+                      v8_inspector::StringView(state.data(), state.size()),
+                      std::move(channel), is_fully_trusted);
+                });
 
-    channels_[session_id].reset(channel);
-    info.GetReturnValue().Set(v8::Int32::New(info.GetIsolate(), session_id));
+    if (session_id.has_value()) {
+      info.GetReturnValue().Set(v8::Int32::New(info.GetIsolate(), *session_id));
+    } else {
+      info.GetIsolate()->ThrowError("Unable to connect to context group");
+    }
   }
 
   static void DisconnectSession(
@@ -204,12 +216,15 @@ class UtilsExtension : public InspectorIsolateData::SetupGlobalTask {
       return;
     }
     int session_id = info[0].As<v8::Int32>()->Value();
+    v8::Local<v8::Context> context = info.GetIsolate()->GetCurrentContext();
+    TaskRunner* context_task_runner =
+        InspectorIsolateData::FromContext(context)->task_runner();
     std::vector<uint8_t> state;
-    RunSyncTask(backend_runner_,
-                [&session_id, &state](InspectorIsolateData* data) {
-                  state = data->DisconnectSession(session_id);
-                });
-    channels_.erase(session_id);
+    RunSyncTask(backend_runner_, [&session_id, &context_task_runner,
+                                  &state](InspectorIsolateData* data) {
+      state = data->DisconnectSession(session_id, context_task_runner);
+    });
+
     info.GetReturnValue().Set(ToV8String(info.GetIsolate(), state));
   }
 
@@ -223,12 +238,9 @@ class UtilsExtension : public InspectorIsolateData::SetupGlobalTask {
         info[0].As<v8::Int32>()->Value(),
         ToVector(info.GetIsolate(), info[1].As<v8::String>())));
   }
-
-  static std::map<int, std::unique_ptr<FrontendChannelImpl>> channels_;
 };
 
 TaskRunner* UtilsExtension::backend_runner_ = nullptr;
-std::map<int, std::unique_ptr<FrontendChannelImpl>> UtilsExtension::channels_;
 
 bool StrictAccessCheck(v8::Local<v8::Context> accessing_context,
                        v8::Local<v8::Object> accessed_object,
@@ -613,8 +625,6 @@ void FuzzInspector(const uint8_t* data, size_t size) {
   ready_semaphore.Signal();
   watchdog.Join();
 
-  UtilsExtension::ClearAllSessions();
-
   // TaskRunners go out of scope here, which causes Isolate teardown and all
   // running background tasks to be properly joined.
 }
