@@ -2,10 +2,6 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-#if !defined(_WIN32) && !defined(_WIN64)
-#include <unistd.h>
-#endif  // !defined(_WIN32) && !defined(_WIN64)
-
 #include <locale.h>
 
 #include <string>
@@ -26,17 +22,19 @@
 #include "test/inspector/tasks.h"
 #include "test/inspector/utils.h"
 
+#if !defined(V8_OS_WIN)
+#include <unistd.h>
+#endif  // !defined(V8_OS_WIN)
+
 namespace v8 {
 namespace internal {
 
 extern void DisableEmbeddedBlobRefcounting();
 extern void FreeCurrentEmbeddedBlob();
 
-extern v8::StartupData CreateSnapshotDataBlobInternal(
+extern v8::StartupData CreateSnapshotDataBlobInternalForInspectorTest(
     v8::SnapshotCreator::FunctionCodeHandling function_code_handling,
-    const char* embedded_source, v8::Isolate* isolate);
-extern v8::StartupData WarmUpSnapshotDataBlobInternal(
-    v8::StartupData cold_snapshot_blob, const char* warmup_source);
+    const char* embedded_source);
 
 namespace {
 
@@ -116,8 +114,6 @@ class UtilsExtension : public InspectorIsolateData::SetupGlobalTask {
     backend_runner_ = runner;
   }
 
-  static void ClearAllSessions() { channels_.clear(); }
-
  private:
   static TaskRunner* backend_runner_;
 
@@ -360,33 +356,46 @@ class UtilsExtension : public InspectorIsolateData::SetupGlobalTask {
                 });
   }
 
+  static bool IsValidConnectSessionArgs(
+      const v8::FunctionCallbackInfo<v8::Value>& info) {
+    if (info.Length() < 3 || info.Length() > 4) return false;
+    if (!info[0]->IsInt32() || !info[1]->IsString() || !info[2]->IsFunction()) {
+      return false;
+    }
+    return info.Length() == 3 || info[3]->IsBoolean();
+  }
+
   static void ConnectSession(const v8::FunctionCallbackInfo<v8::Value>& info) {
-    if (info.Length() != 3 || !info[0]->IsInt32() || !info[1]->IsString() ||
-        !info[2]->IsFunction()) {
+    if (!IsValidConnectSessionArgs(info)) {
       FATAL(
           "Internal error: connectionSession(context_group_id, state, "
-          "dispatch).");
+          "dispatch, is_fully_trusted).");
     }
     v8::Local<v8::Context> context = info.GetIsolate()->GetCurrentContext();
-    FrontendChannelImpl* channel = new FrontendChannelImpl(
-        InspectorIsolateData::FromContext(context)->task_runner(),
-        InspectorIsolateData::FromContext(context)->GetContextGroupId(context),
-        info.GetIsolate(), info[2].As<v8::Function>());
+    std::unique_ptr<FrontendChannelImpl> channel =
+        std::make_unique<FrontendChannelImpl>(
+            InspectorIsolateData::FromContext(context)->task_runner(),
+            InspectorIsolateData::FromContext(context)->GetContextGroupId(
+                context),
+            info.GetIsolate(), info[2].As<v8::Function>());
 
     std::vector<uint8_t> state =
         ToBytes(info.GetIsolate(), info[1].As<v8::String>());
     int context_group_id = info[0].As<v8::Int32>()->Value();
-    int session_id = 0;
-    RunSyncTask(backend_runner_, [&context_group_id, &session_id, &channel,
-                                  &state](InspectorIsolateData* data) {
-      session_id = data->ConnectSession(
-          context_group_id,
-          v8_inspector::StringView(state.data(), state.size()), channel);
-      channel->set_session_id(session_id);
-    });
+    bool is_fully_trusted =
+        info.Length() == 3 || info[3].As<v8::Boolean>()->Value();
+    base::Optional<int> session_id;
+    RunSyncTask(backend_runner_,
+                [context_group_id, &session_id, &channel, &state,
+                 is_fully_trusted](InspectorIsolateData* data) {
+                  session_id = data->ConnectSession(
+                      context_group_id,
+                      v8_inspector::StringView(state.data(), state.size()),
+                      std::move(channel), is_fully_trusted);
+                });
 
-    channels_[session_id].reset(channel);
-    info.GetReturnValue().Set(v8::Int32::New(info.GetIsolate(), session_id));
+    CHECK(session_id.has_value());
+    info.GetReturnValue().Set(v8::Int32::New(info.GetIsolate(), *session_id));
   }
 
   static void DisconnectSession(
@@ -394,13 +403,16 @@ class UtilsExtension : public InspectorIsolateData::SetupGlobalTask {
     if (info.Length() != 1 || !info[0]->IsInt32()) {
       FATAL("Internal error: disconnectionSession(session_id).");
     }
+    v8::Local<v8::Context> context = info.GetIsolate()->GetCurrentContext();
+    TaskRunner* context_task_runner =
+        InspectorIsolateData::FromContext(context)->task_runner();
     int session_id = info[0].As<v8::Int32>()->Value();
     std::vector<uint8_t> state;
-    RunSyncTask(backend_runner_,
-                [&session_id, &state](InspectorIsolateData* data) {
-                  state = data->DisconnectSession(session_id);
-                });
-    channels_.erase(session_id);
+    RunSyncTask(backend_runner_, [&session_id, &context_task_runner,
+                                  &state](InspectorIsolateData* data) {
+      state = data->DisconnectSession(session_id, context_task_runner);
+    });
+
     info.GetReturnValue().Set(ToV8String(info.GetIsolate(), state));
   }
 
@@ -431,12 +443,9 @@ class UtilsExtension : public InspectorIsolateData::SetupGlobalTask {
         },
         info[1].As<v8::Function>());
   }
-
-  static std::map<int, std::unique_ptr<FrontendChannelImpl>> channels_;
 };
 
 TaskRunner* UtilsExtension::backend_runner_ = nullptr;
-std::map<int, std::unique_ptr<FrontendChannelImpl>> UtilsExtension::channels_;
 
 bool StrictAccessCheck(v8::Local<v8::Context> accessing_context,
                        v8::Local<v8::Object> accessed_object,
@@ -452,8 +461,8 @@ class ConsoleExtension : public InspectorIsolateData::SetupGlobalTask {
            v8::Local<v8::ObjectTemplate> global) override {
     v8::Local<v8::String> name =
         v8::String::NewFromUtf8Literal(isolate, "console");
-    global->SetAccessor(name, &ConsoleGetterCallback, nullptr, {}, v8::DEFAULT,
-                        v8::DontEnum);
+    global->SetNativeDataProperty(name, &ConsoleGetterCallback, nullptr, {},
+                                  v8::DontEnum);
   }
 
  private:
@@ -815,9 +824,11 @@ class InspectorExtension : public InspectorIsolateData::SetupGlobalTask {
     v8::Local<v8::Context> context = isolate->GetCurrentContext();
 
     v8::Local<v8::Function> callback = v8::Local<v8::Function>::Cast(info[0]);
-    v8::MaybeLocal<v8::Value> result =
-        callback->Call(context, v8::Undefined(isolate), 0, nullptr);
-    info.GetReturnValue().Set(result.ToLocalChecked());
+    v8::Local<v8::Value> result;
+    if (callback->Call(context, v8::Undefined(isolate), 0, nullptr)
+            .ToLocal(&result)) {
+      info.GetReturnValue().Set(result);
+    }
   }
 
   static void RunNestedMessageLoop(
@@ -847,8 +858,8 @@ int InspectorTestMain(int argc, char* argv[]) {
     if (strcmp(argv[i], "--embed") == 0) {
       argv[i++] = nullptr;
       printf("Embedding script '%s'\n", argv[i]);
-      startup_data = i::CreateSnapshotDataBlobInternal(
-          SnapshotCreator::FunctionCodeHandling::kClear, argv[i], nullptr);
+      startup_data = i::CreateSnapshotDataBlobInternalForInspectorTest(
+          SnapshotCreator::FunctionCodeHandling::kClear, argv[i]);
       argv[i] = nullptr;
     }
   }
@@ -898,7 +909,6 @@ int InspectorTestMain(int argc, char* argv[]) {
     frontend_runner.Join();
     backend_runner.Join();
 
-    UtilsExtension::ClearAllSessions();
     delete[] startup_data.data;
 
     // TaskRunners go out of scope here, which causes Isolate teardown and all
