@@ -10,6 +10,7 @@
 #include "include/v8-template.h"
 #include "src/init/v8.h"
 #include "src/inspector/test-interface.h"
+#include "test/inspector/frontend-channel.h"
 #include "test/inspector/task-runner.h"
 #include "test/inspector/utils.h"
 
@@ -77,6 +78,30 @@ InspectorIsolateData* InspectorIsolateData::FromContext(
       context->GetAlignedPointerFromEmbedderData(kIsolateDataIndex));
 }
 
+InspectorIsolateData::~InspectorIsolateData() {
+  // Enter the isolate before destructing this InspectorIsolateData, so that
+  // destructors that run before the Isolate's destructor still see it as
+  // entered. Use a v8::Locker, in case the thread destroying the isolate is
+  // not the last one that entered it.
+  locker_.emplace(isolate());
+  isolate()->Enter();
+
+  // Sessions need to be deleted before channels can be cleaned up, and channels
+  // must be deleted before the isolate gets cleaned up. This means we first
+  // clean up all the sessions and immedatly after all the channels used by
+  // those sessions.
+  for (const auto& pair : sessions_) {
+    session_ids_for_cleanup_.insert(pair.first);
+  }
+
+  context_group_by_session_.clear();
+  sessions_.clear();
+
+  for (int session_id : session_ids_for_cleanup_) {
+    ChannelHolder::RemoveChannel(session_id);
+  }
+}
+
 int InspectorIsolateData::CreateContextGroup() {
   int context_group_id = ++last_context_group_id_;
   if (!CreateContext(context_group_id, v8_inspector::StringView())) {
@@ -159,14 +184,21 @@ v8::MaybeLocal<v8::Module> InspectorIsolateData::ModuleResolveCallback(
   return maybe_module;
 }
 
-int InspectorIsolateData::ConnectSession(
+base::Optional<int> InspectorIsolateData::ConnectSession(
     int context_group_id, const v8_inspector::StringView& state,
-    v8_inspector::V8Inspector::Channel* channel) {
+    std::unique_ptr<FrontendChannelImpl> channel, bool is_fully_trusted) {
+  if (contexts_.find(context_group_id) == contexts_.end()) return base::nullopt;
+
   v8::SealHandleScope seal_handle_scope(isolate());
   int session_id = ++last_session_id_;
+  // It's important that we register the channel before the `connect` as the
+  // inspector will already send notifications.
+  auto* c = channel.get();
+  ChannelHolder::AddChannel(session_id, std::move(channel));
   sessions_[session_id] = inspector_->connect(
-      context_group_id, channel, state,
-      v8_inspector::V8Inspector::kFullyTrusted,
+      context_group_id, c, state,
+      is_fully_trusted ? v8_inspector::V8Inspector::kFullyTrusted
+                       : v8_inspector::V8Inspector::kUntrusted,
       waiting_for_debugger_
           ? v8_inspector::V8Inspector::kWaitingForDebugger
           : v8_inspector::V8Inspector::kNotWaitingForDebugger);
@@ -174,13 +206,43 @@ int InspectorIsolateData::ConnectSession(
   return session_id;
 }
 
-std::vector<uint8_t> InspectorIsolateData::DisconnectSession(int session_id) {
+namespace {
+
+class RemoveChannelTask : public TaskRunner::Task {
+ public:
+  explicit RemoveChannelTask(int session_id) : session_id_(session_id) {}
+  ~RemoveChannelTask() override = default;
+  bool is_priority_task() final { return false; }
+
+ private:
+  void Run(InspectorIsolateData* data) override {
+    ChannelHolder::RemoveChannel(session_id_);
+  }
+  int session_id_;
+};
+
+}  // namespace
+
+std::vector<uint8_t> InspectorIsolateData::DisconnectSession(
+    int session_id, TaskRunner* context_task_runner) {
   v8::SealHandleScope seal_handle_scope(isolate());
   auto it = sessions_.find(session_id);
   CHECK(it != sessions_.end());
   context_group_by_session_.erase(it->second.get());
   std::vector<uint8_t> result = it->second->state();
   sessions_.erase(it);
+
+  // The InspectorSession destructor does cleanup work like disabling agents.
+  // This could send some more notifications. We'll delay removing the channel
+  // so notification tasks have time to get sent.
+  // Note: This only works for tasks scheduled immediately by the desctructor.
+  //       Any task scheduled in turn by one of the "cleanup tasks" will run
+  //       AFTER the channel was removed.
+  context_task_runner->Append(std::make_unique<RemoveChannelTask>(session_id));
+
+  // In case we shutdown the test runner before the above task can run, we
+  // let the desctructor clean up the channel.
+  session_ids_for_cleanup_.insert(session_id);
   return result;
 }
 
@@ -463,8 +525,8 @@ void InspectorIsolateData::installAdditionalCommandLineAPI(
   CHECK(context->GetIsolate() == isolate());
   v8::HandleScope handle_scope(isolate());
   v8::Context::Scope context_scope(context);
-  v8::ScriptOrigin origin(isolate(), v8::String::NewFromUtf8Literal(
-                                         isolate(), "internal-console-api"));
+  v8::ScriptOrigin origin(
+      v8::String::NewFromUtf8Literal(isolate(), "internal-console-api"));
   v8::ScriptCompiler::Source scriptSource(
       additional_console_api_.Get(isolate()), origin);
   v8::MaybeLocal<v8::Script> script =
@@ -560,5 +622,27 @@ int64_t InspectorIsolateData::generateUniqueId() {
   return ++last_unique_id;
 }
 
+// static
+void ChannelHolder::AddChannel(int session_id,
+                               std::unique_ptr<FrontendChannelImpl> channel) {
+  CHECK_NE(channel.get(), nullptr);
+  channel->set_session_id(session_id);
+  channels_[session_id] = std::move(channel);
+}
+
+// static
+FrontendChannelImpl* ChannelHolder::GetChannel(int session_id) {
+  auto it = channels_.find(session_id);
+  return it != channels_.end() ? it->second.get() : nullptr;
+}
+
+// static
+void ChannelHolder::RemoveChannel(int session_id) {
+  channels_.erase(session_id);
+}
+
+// static
+std::map<int, std::unique_ptr<FrontendChannelImpl>> ChannelHolder::channels_;
+
 }  // namespace internal
 }  // namespace v8
