@@ -3,8 +3,8 @@
 // found in the LICENSE file.
 
 // Flags: --experimental-wasm-stringref --allow-natives-syntax
-// Flags: --experimental-wasm-gc --experimental-wasm-imported-strings
-// Flags: --trace-wasm-inlining --liftoff --experimental-wasm-typed-funcref
+// Flags: --experimental-wasm-imported-strings
+// Flags: --trace-wasm-inlining --liftoff
 // Also explicitly enable inlining and disable debug code to avoid differences
 // between --future and --no-future or debug and release builds.
 // Flags: --experimental-wasm-inlining --no-debug-code
@@ -95,42 +95,62 @@ d8.file.execute('test/mjsunit/wasm/wasm-module-builder.js');
   let kSig_e_d = makeSig([kWasmF64], [kRefExtern]);
   let kSig_i_ri = makeSig([kWasmExternRef, kWasmI32], [kWasmI32]);
   let kSig_i_rr = makeSig([kWasmExternRef, kWasmExternRef], [kWasmI32]);
+  let kSig_i_rri =
+      makeSig([kWasmExternRef, kWasmExternRef, kWasmI32], [kWasmI32]);
+  let kSig_e_r = makeSig([kWasmExternRef], [kRefExtern]);
   let kSig_e_i = makeSig([kWasmI32], [kRefExtern]);
   let kSig_e_rii = makeSig([kWasmExternRef, kWasmI32, kWasmI32],
                           [kRefExtern]);
   let kSig_e_rr = makeSig([kWasmExternRef, kWasmExternRef], [kRefExtern]);
 
   let builder = new WasmModuleBuilder();
-  let kArrayI16 = builder.addArray(kWasmI16, true);
-  let kArrayI8 = builder.addArray(kWasmI8, true);
+  builder.startRecGroup();
+  let kArrayI16 = builder.addArray(kWasmI16, true, kNoSuperType, true);
+  builder.endRecGroup();
+  builder.startRecGroup();
+  let kArrayI8 = builder.addArray(kWasmI8, true, kNoSuperType, true);
+  builder.endRecGroup();
   let a16ref = wasmRefNullType(kArrayI16);
   let a8ref = wasmRefNullType(kArrayI8);
 
-  builder.addImport('String', 'fromWtf16Array',
+  builder.addImport('wasm:js-string', 'cast', kSig_e_r);
+  builder.addImport('wasm:js-string', 'test', kSig_i_r);
+  builder.addImport('wasm:js-string', 'fromCharCodeArray',
                     makeSig([a16ref, kWasmI32, kWasmI32], [kRefExtern]));
-  builder.addImport('String', 'fromWtf8Array',
+  builder.addImport('wasm:text-decoder', 'decodeStringFromUTF8Array',
                     makeSig([a8ref, kWasmI32, kWasmI32], [kRefExtern]));
-  builder.addImport('String', 'toWtf16Array',
+  builder.addImport('wasm:js-string', 'intoCharCodeArray',
                     makeSig([kWasmExternRef, a16ref, kWasmI32], [kWasmI32]));
-  builder.addImport('String', 'fromCharCode', kSig_e_i);
-  builder.addImport('String', 'fromCodePoint', kSig_e_i);
-  builder.addImport('String', 'charCodeAt', kSig_i_ri);
-  builder.addImport('String', 'codePointAt', kSig_i_ri);
-  builder.addImport('String', 'length', kSig_i_r);
-  builder.addImport('String', 'concat', kSig_e_rr);
-  builder.addImport('String', 'substring', kSig_e_rii);
-  builder.addImport('String', 'equals', kSig_i_rr);
-  builder.addImport('String', 'compare', kSig_i_rr);
+  builder.addImport('wasm:js-string', 'fromCharCode', kSig_e_i);
+  builder.addImport('wasm:js-string', 'fromCodePoint', kSig_e_i);
+  builder.addImport('wasm:js-string', 'charCodeAt', kSig_i_ri);
+  builder.addImport('wasm:js-string', 'codePointAt', kSig_i_ri);
+  builder.addImport('wasm:js-string', 'length', kSig_i_r);
+  builder.addImport('wasm:js-string', 'concat', kSig_e_rr);
+  builder.addImport('wasm:js-string', 'substring', kSig_e_rii);
+  builder.addImport('wasm:js-string', 'equals', kSig_i_rr);
+  builder.addImport('wasm:js-string', 'compare', kSig_i_rr);
+
+  builder.addImport('wasm:text-encoder', 'measureStringAsUTF8', kSig_i_r);
+  builder.addImport('wasm:text-encoder', 'encodeStringIntoUTF8Array',
+                    makeSig([kWasmExternRef, a8ref, kWasmI32], [kWasmI32]));
 
   builder.addImport('related', 'intToString', kSig_e_ii);
   builder.addImport('related', 'doubleToString', kSig_e_d);
+  builder.addImport('related', 'stringIndexOf', kSig_i_rri);
+  builder.addImport('related', 'stringToLowerCase', kSig_r_r);
+
   // String-consuming imports like "toLowerCase" are not (yet?) supported for
   // special-casing with imported strings due to lack of static typing.
   let intToString = Function.prototype.call.bind(Number.prototype.toString);
   let doubleToString = intToString;
+  let stringIndexOf = Function.prototype.call.bind(String.prototype.indexOf);
+  let stringToLowerCase =
+      Function.prototype.call.bind(String.prototype.toLowerCase);
 
   builder.instantiate({
-    String: WebAssembly.String,
-    related: { intToString, doubleToString }
+    related: {intToString, doubleToString, stringIndexOf, stringToLowerCase},
+  }, {
+    builtins: ['js-string', 'text-decoder', 'text-encoder'],
   });
 })();
