@@ -15,10 +15,16 @@ const fg1 = new FinalizationRegistry(callback);
 const fg2 = new FinalizationRegistry(callback);
 
 (function() {
-let x = {};
-fg1.register(x, {});
-fg2.register(x, {});
-x = null;
+  let x = {};
+  fg1.register(x, {});
+  fg2.register(x, {});
+  x = null;
 })();
 
-gc();
+// We need to invoke GC asynchronously and wait for it to finish, so that
+// it doesn't need to scan the stack. Otherwise, the objects may not be
+// reclaimed because of conservative stack scanning and the test may not
+// work as intended.
+(async function () {
+  await gc({ type: 'major', execution: 'async' });
+})();
