@@ -2,5 +2,4 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-let x;
-export default x = 0;
+export default 1, 2;
