@@ -2,13 +2,13 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-// Flags: --wasm-staging --experimental-wasm-gc
+// Flags: --wasm-staging
 
 d8.file.execute('test/mjsunit/wasm/wasm-module-builder.js');
 
 const builder = new WasmModuleBuilder();
 builder.addType(makeSig([kWasmI32, kWasmI32, kWasmI32], [kWasmI32]));
-builder.addMemory(16, 32, false);
+builder.addMemory(16, 32);
 // Generate function 1 (out of 1).
 builder.addFunction(undefined, 0 /* sig */)
   .addBodyWithEnd([
