@@ -9,7 +9,7 @@ d8.file.execute("test/mjsunit/wasm/wasm-module-builder.js");
 (function() {
 "use asm";
 var builder = new WasmModuleBuilder();
-builder.addMemory(0, 5, false);
+builder.addMemory(0, 5);
 builder.addFunction("regression_699485", kSig_i_v)
   .addBody([
       kExprI32Const, 0x04,
