@@ -2,7 +2,7 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-// Flags: --experimental-wasm-gc --experimental-wasm-stringref
+// Flags: --experimental-wasm-stringref
 
 d8.file.execute('test/mjsunit/wasm/wasm-module-builder.js');
 
@@ -20,19 +20,19 @@ d8.file.execute('test/mjsunit/wasm/wasm-module-builder.js');
     .addBody([
       kExprLocalGet, 0,
       kGCPrefix, kExprStructNew, structSuper,
-      kGCPrefix, kExprExternExternalize,
+      kGCPrefix, kExprExternConvertAny,
     ]).exportFunc();
   builder.addFunction('createStructSub', makeSig([kWasmI32], [kWasmExternRef]))
     .addBody([
       kExprLocalGet, 0,
       kGCPrefix, kExprStructNew, structSub,
-      kGCPrefix, kExprExternExternalize,
+      kGCPrefix, kExprExternConvertAny,
     ]).exportFunc();
   builder.addFunction('createArray', makeSig([kWasmI32], [kWasmExternRef]))
     .addBody([
       kExprLocalGet, 0,
       kGCPrefix, kExprArrayNewFixed, array, 1,
-      kGCPrefix, kExprExternExternalize,
+      kGCPrefix, kExprExternConvertAny,
     ]).exportFunc();
   builder.addFunction('createFuncRef', makeSig([], [kWasmFuncRef]))
     .addBody([
@@ -56,10 +56,10 @@ d8.file.execute('test/mjsunit/wasm/wasm-module-builder.js');
                         makeSig([kWasmExternRef], [kWasmI32, kWasmI32]))
     .addBody([
       kExprLocalGet, 0,
-      kGCPrefix, kExprExternInternalize,
+      kGCPrefix, kExprAnyConvertExtern,
       kGCPrefix, kExprRefTest, typeCode,
       kExprLocalGet, 0,
-      kGCPrefix, kExprExternInternalize,
+      kGCPrefix, kExprAnyConvertExtern,
       kGCPrefix, kExprRefTestNull, typeCode,
     ]).exportFunc();
 
@@ -67,75 +67,18 @@ d8.file.execute('test/mjsunit/wasm/wasm-module-builder.js');
                         makeSig([kWasmExternRef], [kWasmExternRef]))
     .addBody([
       kExprLocalGet, 0,
-      kGCPrefix, kExprExternInternalize,
+      kGCPrefix, kExprAnyConvertExtern,
       kGCPrefix, kExprRefCast, typeCode,
-      kGCPrefix, kExprExternExternalize,
+      kGCPrefix, kExprExternConvertAny,
     ]).exportFunc();
 
     builder.addFunction(`refCastNull${typeName}`,
                         makeSig([kWasmExternRef], [kWasmExternRef]))
     .addBody([
       kExprLocalGet, 0,
-      kGCPrefix, kExprExternInternalize,
+      kGCPrefix, kExprAnyConvertExtern,
       kGCPrefix, kExprRefCastNull, typeCode,
-      kGCPrefix, kExprExternExternalize,
-    ]).exportFunc();
-
-    builder.addFunction(`brOnCast${typeName}`,
-                        makeSig([kWasmExternRef], [kWasmI32]))
-    .addBody([
-      kExprBlock, kWasmRef, typeCode,
-        kExprLocalGet, 0,
-        kGCPrefix, kExprExternInternalize,
-        kGCPrefix, kExprBrOnCast, 0, typeCode,
-        kExprI32Const, 0,
-        kExprReturn,
-      kExprEnd,
-      kExprDrop,
-      kExprI32Const, 1,
-      kExprReturn,
-    ]).exportFunc();
-    builder.addFunction(`brOnCastNull${typeName}`,
-                        makeSig([kWasmExternRef], [kWasmI32]))
-    .addBody([
-      kExprBlock, kWasmRefNull, typeCode,
-        kExprLocalGet, 0,
-        kGCPrefix, kExprExternInternalize,
-        kGCPrefix, kExprBrOnCastNull, 0, typeCode,
-        kExprI32Const, 0,
-        kExprReturn,
-      kExprEnd,
-      kExprDrop,
-      kExprI32Const, 1,
-      kExprReturn,
-    ]).exportFunc();
-    builder.addFunction(`brOnCastFail${typeName}`,
-                        makeSig([kWasmExternRef], [kWasmI32]))
-    .addBody([
-      kExprBlock, kAnyRefCode,
-        kExprLocalGet, 0,
-        kGCPrefix, kExprExternInternalize,
-        kGCPrefix, kExprBrOnCastFail, 0, typeCode,
-        kExprI32Const, 0,
-        kExprReturn,
-      kExprEnd,
-      kExprDrop,
-      kExprI32Const, 1,
-      kExprReturn,
-    ]).exportFunc();
-    builder.addFunction(`brOnCastFailNull${typeName}`,
-                        makeSig([kWasmExternRef], [kWasmI32]))
-    .addBody([
-      kExprBlock, kAnyRefCode,
-        kExprLocalGet, 0,
-        kGCPrefix, kExprExternInternalize,
-        kGCPrefix, kExprBrOnCastFailNull, 0, typeCode,
-        kExprI32Const, 0,
-        kExprReturn,
-      kExprEnd,
-      kExprDrop,
-      kExprI32Const, 1,
-      kExprReturn,
+      kGCPrefix, kExprExternConvertAny,
     ]).exportFunc();
 
     builder.addFunction(`brOnCastGeneric${typeName}`,
@@ -143,7 +86,7 @@ d8.file.execute('test/mjsunit/wasm/wasm-module-builder.js');
     .addBody([
       kExprBlock, kWasmRef, typeCode,
         kExprLocalGet, 0,
-        kGCPrefix, kExprExternInternalize,
+        kGCPrefix, kExprAnyConvertExtern,
         kGCPrefix, kExprBrOnCastGeneric, 0b01, 0, kAnyRefCode, typeCode,
         kExprI32Const, 0,
         kExprReturn,
@@ -157,7 +100,7 @@ d8.file.execute('test/mjsunit/wasm/wasm-module-builder.js');
     .addBody([
       kExprBlock, kWasmRefNull, typeCode,
         kExprLocalGet, 0,
-        kGCPrefix, kExprExternInternalize,
+        kGCPrefix, kExprAnyConvertExtern,
         kGCPrefix, kExprBrOnCastGeneric, 0b11, 0, kAnyRefCode, typeCode,
         kExprI32Const, 0,
         kExprReturn,
@@ -171,7 +114,7 @@ d8.file.execute('test/mjsunit/wasm/wasm-module-builder.js');
     .addBody([
       kExprBlock, kAnyRefCode,
         kExprLocalGet, 0,
-        kGCPrefix, kExprExternInternalize,
+        kGCPrefix, kExprAnyConvertExtern,
         kGCPrefix, kExprBrOnCastFailGeneric, 0b01, 0, kAnyRefCode, typeCode,
         kExprI32Const, 0,
         kExprReturn,
@@ -185,7 +128,7 @@ d8.file.execute('test/mjsunit/wasm/wasm-module-builder.js');
     .addBody([
       kExprBlock, kAnyRefCode,
         kExprLocalGet, 0,
-        kGCPrefix, kExprExternInternalize,
+        kGCPrefix, kExprAnyConvertExtern,
         kGCPrefix, kExprBrOnCastFailGeneric, 0b11, 0, kAnyRefCode, typeCode,
         kExprI32Const, 0,
         kExprReturn,
@@ -508,410 +451,6 @@ d8.file.execute('test/mjsunit/wasm/wasm-module-builder.js');
   assertTraps(kTrapIllegalCast, () => wasm.refCastNullNone(jsObj));
   assertTraps(kTrapIllegalCast, () => wasm.refCastNullNone(strObj));
 
-  // br_on_cast
-  assertEquals(0, wasm.brOnCastStructSuper(null));
-  assertEquals(0, wasm.brOnCastStructSuper(undefined));
-  assertEquals(1, wasm.brOnCastStructSuper(structSuperObj));
-  assertEquals(1, wasm.brOnCastStructSuper(structSubObj));
-  assertEquals(0, wasm.brOnCastStructSuper(arrayObj));
-  assertEquals(0, wasm.brOnCastStructSuper(funcObj));
-  assertEquals(0, wasm.brOnCastStructSuper(1));
-  assertEquals(0, wasm.brOnCastStructSuper(jsObj));
-  assertEquals(0, wasm.brOnCastStructSuper(strObj));
-
-  assertEquals(0, wasm.brOnCastStructSub(null));
-  assertEquals(0, wasm.brOnCastStructSub(undefined));
-  assertEquals(0, wasm.brOnCastStructSub(structSuperObj));
-  assertEquals(1, wasm.brOnCastStructSub(structSubObj));
-  assertEquals(0, wasm.brOnCastStructSub(arrayObj));
-  assertEquals(0, wasm.brOnCastStructSub(funcObj));
-  assertEquals(0, wasm.brOnCastStructSub(1));
-  assertEquals(0, wasm.brOnCastStructSub(jsObj));
-  assertEquals(0, wasm.brOnCastStructSub(strObj));
-
-  assertEquals(0, wasm.brOnCastArray(null));
-  assertEquals(0, wasm.brOnCastArray(undefined));
-  assertEquals(0, wasm.brOnCastArray(structSuperObj));
-  assertEquals(0, wasm.brOnCastArray(structSubObj));
-  assertEquals(1, wasm.brOnCastArray(arrayObj));
-  assertEquals(0, wasm.brOnCastArray(funcObj));
-  assertEquals(0, wasm.brOnCastArray(1));
-  assertEquals(0, wasm.brOnCastArray(jsObj));
-  assertEquals(0, wasm.brOnCastArray(strObj));
-
-  assertEquals(0, wasm.brOnCastI31(null));
-  assertEquals(0, wasm.brOnCastI31(undefined));
-  assertEquals(0, wasm.brOnCastI31(structSuperObj));
-  assertEquals(0, wasm.brOnCastI31(structSubObj));
-  assertEquals(0, wasm.brOnCastI31(arrayObj));
-  assertEquals(0, wasm.brOnCastI31(funcObj));
-  assertEquals(1, wasm.brOnCastI31(1));
-  assertEquals(0, wasm.brOnCastI31(jsObj));
-  assertEquals(0, wasm.brOnCastI31(strObj));
-
-  assertEquals(0, wasm.brOnCastAnyArray(null));
-  assertEquals(0, wasm.brOnCastAnyArray(undefined));
-  assertEquals(0, wasm.brOnCastAnyArray(structSuperObj));
-  assertEquals(0, wasm.brOnCastAnyArray(structSubObj));
-  assertEquals(1, wasm.brOnCastAnyArray(arrayObj));
-  assertEquals(0, wasm.brOnCastAnyArray(funcObj));
-  assertEquals(0, wasm.brOnCastAnyArray(1));
-  assertEquals(0, wasm.brOnCastAnyArray(jsObj));
-  assertEquals(0, wasm.brOnCastAnyArray(strObj));
-
-  assertEquals(0, wasm.brOnCastStruct(null));
-  assertEquals(0, wasm.brOnCastStruct(undefined));
-  assertEquals(1, wasm.brOnCastStruct(structSuperObj));
-  assertEquals(1, wasm.brOnCastStruct(structSubObj));
-  assertEquals(0, wasm.brOnCastStruct(arrayObj));
-  assertEquals(0, wasm.brOnCastStruct(funcObj));
-  assertEquals(0, wasm.brOnCastStruct(1));
-  assertEquals(0, wasm.brOnCastStruct(jsObj));
-  assertEquals(0, wasm.brOnCastStruct(strObj));
-
-  assertEquals(0, wasm.brOnCastString(null));
-  assertEquals(0, wasm.brOnCastString(undefined));
-  assertEquals(0, wasm.brOnCastString(structSuperObj));
-  assertEquals(0, wasm.brOnCastString(structSubObj));
-  assertEquals(0, wasm.brOnCastString(arrayObj));
-  assertEquals(0, wasm.brOnCastString(funcObj));
-  assertEquals(0, wasm.brOnCastString(1));
-  assertEquals(0, wasm.brOnCastString(jsObj));
-  assertEquals(1, wasm.brOnCastString(strObj));
-
-  assertEquals(0, wasm.brOnCastEq(null));
-  assertEquals(0, wasm.brOnCastEq(undefined));
-  assertEquals(1, wasm.brOnCastEq(structSuperObj));
-  assertEquals(1, wasm.brOnCastEq(structSubObj));
-  assertEquals(1, wasm.brOnCastEq(arrayObj));
-  assertEquals(0, wasm.brOnCastEq(funcObj));
-  assertEquals(1, wasm.brOnCastEq(1));
-  assertEquals(0, wasm.brOnCastEq(jsObj));
-  assertEquals(0, wasm.brOnCastEq(strObj));
-
-  assertEquals(0, wasm.brOnCastAny(null));
-  assertEquals(1, wasm.brOnCastAny(undefined));
-  assertEquals(1, wasm.brOnCastAny(structSuperObj));
-  assertEquals(1, wasm.brOnCastAny(structSubObj));
-  assertEquals(1, wasm.brOnCastAny(arrayObj));
-  assertEquals(1, wasm.brOnCastAny(funcObj));
-  assertEquals(1, wasm.brOnCastAny(1));
-  assertEquals(1, wasm.brOnCastAny(jsObj));
-  assertEquals(1, wasm.brOnCastAny(strObj));
-
-  assertEquals(0, wasm.brOnCastNone(null));
-  assertEquals(0, wasm.brOnCastNone(undefined));
-  assertEquals(0, wasm.brOnCastNone(structSuperObj));
-  assertEquals(0, wasm.brOnCastNone(structSubObj));
-  assertEquals(0, wasm.brOnCastNone(arrayObj));
-  assertEquals(0, wasm.brOnCastNone(funcObj));
-  assertEquals(0, wasm.brOnCastNone(1));
-  assertEquals(0, wasm.brOnCastNone(jsObj));
-  assertEquals(0, wasm.brOnCastNone(strObj));
-
-  // br_on_cast null
-  assertEquals(1, wasm.brOnCastNullStructSuper(null));
-  assertEquals(0, wasm.brOnCastNullStructSuper(undefined));
-  assertEquals(1, wasm.brOnCastNullStructSuper(structSuperObj));
-  assertEquals(1, wasm.brOnCastNullStructSuper(structSubObj));
-  assertEquals(0, wasm.brOnCastNullStructSuper(arrayObj));
-  assertEquals(0, wasm.brOnCastNullStructSuper(funcObj));
-  assertEquals(0, wasm.brOnCastNullStructSuper(1));
-  assertEquals(0, wasm.brOnCastNullStructSuper(jsObj));
-  assertEquals(0, wasm.brOnCastNullStructSuper(strObj));
-
-  assertEquals(1, wasm.brOnCastNullStructSub(null));
-  assertEquals(0, wasm.brOnCastNullStructSub(undefined));
-  assertEquals(0, wasm.brOnCastNullStructSub(structSuperObj));
-  assertEquals(1, wasm.brOnCastNullStructSub(structSubObj));
-  assertEquals(0, wasm.brOnCastNullStructSub(arrayObj));
-  assertEquals(0, wasm.brOnCastNullStructSub(funcObj));
-  assertEquals(0, wasm.brOnCastNullStructSub(1));
-  assertEquals(0, wasm.brOnCastNullStructSub(jsObj));
-  assertEquals(0, wasm.brOnCastNullStructSub(strObj));
-
-  assertEquals(1, wasm.brOnCastNullArray(null));
-  assertEquals(0, wasm.brOnCastNullArray(undefined));
-  assertEquals(0, wasm.brOnCastNullArray(structSuperObj));
-  assertEquals(0, wasm.brOnCastNullArray(structSubObj));
-  assertEquals(1, wasm.brOnCastNullArray(arrayObj));
-  assertEquals(0, wasm.brOnCastNullArray(funcObj));
-  assertEquals(0, wasm.brOnCastNullArray(1));
-  assertEquals(0, wasm.brOnCastNullArray(jsObj));
-  assertEquals(0, wasm.brOnCastNullArray(strObj));
-
-  assertEquals(1, wasm.brOnCastNullI31(null));
-  assertEquals(0, wasm.brOnCastNullI31(undefined));
-  assertEquals(0, wasm.brOnCastNullI31(structSuperObj));
-  assertEquals(0, wasm.brOnCastNullI31(structSubObj));
-  assertEquals(0, wasm.brOnCastNullI31(arrayObj));
-  assertEquals(0, wasm.brOnCastNullI31(funcObj));
-  assertEquals(1, wasm.brOnCastNullI31(1));
-  assertEquals(0, wasm.brOnCastNullI31(jsObj));
-  assertEquals(0, wasm.brOnCastNullI31(strObj));
-
-  assertEquals(1, wasm.brOnCastNullAnyArray(null));
-  assertEquals(0, wasm.brOnCastNullAnyArray(undefined));
-  assertEquals(0, wasm.brOnCastNullAnyArray(structSuperObj));
-  assertEquals(0, wasm.brOnCastNullAnyArray(structSubObj));
-  assertEquals(1, wasm.brOnCastNullAnyArray(arrayObj));
-  assertEquals(0, wasm.brOnCastNullAnyArray(funcObj));
-  assertEquals(0, wasm.brOnCastNullAnyArray(1));
-  assertEquals(0, wasm.brOnCastNullAnyArray(jsObj));
-  assertEquals(0, wasm.brOnCastNullAnyArray(strObj));
-
-  assertEquals(1, wasm.brOnCastNullStruct(null));
-  assertEquals(0, wasm.brOnCastNullStruct(undefined));
-  assertEquals(1, wasm.brOnCastNullStruct(structSuperObj));
-  assertEquals(1, wasm.brOnCastNullStruct(structSubObj));
-  assertEquals(0, wasm.brOnCastNullStruct(arrayObj));
-  assertEquals(0, wasm.brOnCastNullStruct(funcObj));
-  assertEquals(0, wasm.brOnCastNullStruct(1));
-  assertEquals(0, wasm.brOnCastNullStruct(jsObj));
-  assertEquals(0, wasm.brOnCastNullStruct(strObj));
-
-  assertEquals(1, wasm.brOnCastNullEq(null));
-  assertEquals(0, wasm.brOnCastNullEq(undefined));
-  assertEquals(1, wasm.brOnCastNullEq(structSuperObj));
-  assertEquals(1, wasm.brOnCastNullEq(structSubObj));
-  assertEquals(1, wasm.brOnCastNullEq(arrayObj));
-  assertEquals(0, wasm.brOnCastNullEq(funcObj));
-  assertEquals(1, wasm.brOnCastNullEq(1));
-  assertEquals(0, wasm.brOnCastNullEq(jsObj));
-  assertEquals(0, wasm.brOnCastNullEq(strObj));
-
-  assertEquals(1, wasm.brOnCastNullString(null));
-  assertEquals(0, wasm.brOnCastNullString(undefined));
-  assertEquals(0, wasm.brOnCastNullString(structSuperObj));
-  assertEquals(0, wasm.brOnCastNullString(structSubObj));
-  assertEquals(0, wasm.brOnCastNullString(arrayObj));
-  assertEquals(0, wasm.brOnCastNullString(funcObj));
-  assertEquals(0, wasm.brOnCastNullString(1));
-  assertEquals(0, wasm.brOnCastNullString(jsObj));
-  assertEquals(1, wasm.brOnCastNullString(strObj));
-
-  assertEquals(1, wasm.brOnCastNullAny(null));
-  assertEquals(1, wasm.brOnCastNullAny(undefined));
-  assertEquals(1, wasm.brOnCastNullAny(structSuperObj));
-  assertEquals(1, wasm.brOnCastNullAny(structSubObj));
-  assertEquals(1, wasm.brOnCastNullAny(arrayObj));
-  assertEquals(1, wasm.brOnCastNullAny(funcObj));
-  assertEquals(1, wasm.brOnCastNullAny(1));
-  assertEquals(1, wasm.brOnCastNullAny(jsObj));
-  assertEquals(1, wasm.brOnCastNullAny(strObj));
-
-  assertEquals(1, wasm.brOnCastNullNone(null));
-  assertEquals(0, wasm.brOnCastNullNone(undefined));
-  assertEquals(0, wasm.brOnCastNullNone(structSuperObj));
-  assertEquals(0, wasm.brOnCastNullNone(structSubObj));
-  assertEquals(0, wasm.brOnCastNullNone(arrayObj));
-  assertEquals(0, wasm.brOnCastNullNone(funcObj));
-  assertEquals(0, wasm.brOnCastNullNone(1));
-  assertEquals(0, wasm.brOnCastNullNone(jsObj));
-  assertEquals(0, wasm.brOnCastNullNone(strObj));
-
-  // br_on_cast_fail
-  assertEquals(1, wasm.brOnCastFailStructSuper(null));
-  assertEquals(1, wasm.brOnCastFailStructSuper(undefined));
-  assertEquals(0, wasm.brOnCastFailStructSuper(structSuperObj));
-  assertEquals(0, wasm.brOnCastFailStructSuper(structSubObj));
-  assertEquals(1, wasm.brOnCastFailStructSuper(arrayObj));
-  assertEquals(1, wasm.brOnCastFailStructSuper(funcObj));
-  assertEquals(1, wasm.brOnCastFailStructSuper(1));
-  assertEquals(1, wasm.brOnCastFailStructSuper(jsObj));
-  assertEquals(1, wasm.brOnCastFailStructSuper(strObj));
-
-  assertEquals(1, wasm.brOnCastFailStructSub(null));
-  assertEquals(1, wasm.brOnCastFailStructSub(undefined));
-  assertEquals(1, wasm.brOnCastFailStructSub(structSuperObj));
-  assertEquals(0, wasm.brOnCastFailStructSub(structSubObj));
-  assertEquals(1, wasm.brOnCastFailStructSub(arrayObj));
-  assertEquals(1, wasm.brOnCastFailStructSub(funcObj));
-  assertEquals(1, wasm.brOnCastFailStructSub(1));
-  assertEquals(1, wasm.brOnCastFailStructSub(jsObj));
-  assertEquals(1, wasm.brOnCastFailStructSub(strObj));
-
-  assertEquals(1, wasm.brOnCastFailArray(null));
-  assertEquals(1, wasm.brOnCastFailArray(undefined));
-  assertEquals(1, wasm.brOnCastFailArray(structSuperObj));
-  assertEquals(1, wasm.brOnCastFailArray(structSubObj));
-  assertEquals(0, wasm.brOnCastFailArray(arrayObj));
-  assertEquals(1, wasm.brOnCastFailArray(funcObj));
-  assertEquals(1, wasm.brOnCastFailArray(1));
-  assertEquals(1, wasm.brOnCastFailArray(jsObj));
-  assertEquals(1, wasm.brOnCastFailArray(strObj));
-
-  assertEquals(1, wasm.brOnCastFailI31(null));
-  assertEquals(1, wasm.brOnCastFailI31(undefined));
-  assertEquals(1, wasm.brOnCastFailI31(structSuperObj));
-  assertEquals(1, wasm.brOnCastFailI31(structSubObj));
-  assertEquals(1, wasm.brOnCastFailI31(arrayObj));
-  assertEquals(1, wasm.brOnCastFailI31(funcObj));
-  assertEquals(0, wasm.brOnCastFailI31(1));
-  assertEquals(1, wasm.brOnCastFailI31(jsObj));
-  assertEquals(1, wasm.brOnCastFailI31(strObj));
-
-  assertEquals(1, wasm.brOnCastFailAnyArray(null));
-  assertEquals(1, wasm.brOnCastFailAnyArray(undefined));
-  assertEquals(1, wasm.brOnCastFailAnyArray(structSuperObj));
-  assertEquals(1, wasm.brOnCastFailAnyArray(structSubObj));
-  assertEquals(0, wasm.brOnCastFailAnyArray(arrayObj));
-  assertEquals(1, wasm.brOnCastFailAnyArray(funcObj));
-  assertEquals(1, wasm.brOnCastFailAnyArray(1));
-  assertEquals(1, wasm.brOnCastFailAnyArray(jsObj));
-  assertEquals(1, wasm.brOnCastFailAnyArray(strObj));
-
-  assertEquals(1, wasm.brOnCastFailStruct(null));
-  assertEquals(1, wasm.brOnCastFailStruct(undefined));
-  assertEquals(0, wasm.brOnCastFailStruct(structSuperObj));
-  assertEquals(0, wasm.brOnCastFailStruct(structSubObj));
-  assertEquals(1, wasm.brOnCastFailStruct(arrayObj));
-  assertEquals(1, wasm.brOnCastFailStruct(funcObj));
-  assertEquals(1, wasm.brOnCastFailStruct(1));
-  assertEquals(1, wasm.brOnCastFailStruct(jsObj));
-  assertEquals(1, wasm.brOnCastFailStruct(strObj));
-
-  assertEquals(1, wasm.brOnCastFailEq(null));
-  assertEquals(1, wasm.brOnCastFailEq(undefined));
-  assertEquals(0, wasm.brOnCastFailEq(structSuperObj));
-  assertEquals(0, wasm.brOnCastFailEq(structSubObj));
-  assertEquals(0, wasm.brOnCastFailEq(arrayObj));
-  assertEquals(1, wasm.brOnCastFailEq(funcObj));
-  assertEquals(0, wasm.brOnCastFailEq(1));
-  assertEquals(1, wasm.brOnCastFailEq(jsObj));
-  assertEquals(1, wasm.brOnCastFailEq(strObj));
-
-  assertEquals(1, wasm.brOnCastFailString(null));
-  assertEquals(1, wasm.brOnCastFailString(undefined));
-  assertEquals(1, wasm.brOnCastFailString(structSuperObj));
-  assertEquals(1, wasm.brOnCastFailString(structSubObj));
-  assertEquals(1, wasm.brOnCastFailString(arrayObj));
-  assertEquals(1, wasm.brOnCastFailString(funcObj));
-  assertEquals(1, wasm.brOnCastFailString(1));
-  assertEquals(1, wasm.brOnCastFailString(jsObj));
-  assertEquals(0, wasm.brOnCastFailString(strObj));
-
-  assertEquals(1, wasm.brOnCastFailAny(null));
-  assertEquals(0, wasm.brOnCastFailAny(undefined));
-  assertEquals(0, wasm.brOnCastFailAny(structSuperObj));
-  assertEquals(0, wasm.brOnCastFailAny(structSubObj));
-  assertEquals(0, wasm.brOnCastFailAny(arrayObj));
-  assertEquals(0, wasm.brOnCastFailAny(funcObj));
-  assertEquals(0, wasm.brOnCastFailAny(1));
-  assertEquals(0, wasm.brOnCastFailAny(jsObj));
-  assertEquals(0, wasm.brOnCastFailAny(strObj));
-
-  assertEquals(1, wasm.brOnCastFailNone(null));
-  assertEquals(1, wasm.brOnCastFailNone(undefined));
-  assertEquals(1, wasm.brOnCastFailNone(structSuperObj));
-  assertEquals(1, wasm.brOnCastFailNone(structSubObj));
-  assertEquals(1, wasm.brOnCastFailNone(arrayObj));
-  assertEquals(1, wasm.brOnCastFailNone(funcObj));
-  assertEquals(1, wasm.brOnCastFailNone(1));
-  assertEquals(1, wasm.brOnCastFailNone(jsObj));
-  assertEquals(1, wasm.brOnCastFailNone(strObj));
-
-  // br_on_cast_fail null
-  assertEquals(0, wasm.brOnCastFailNullStructSuper(null));
-  assertEquals(1, wasm.brOnCastFailNullStructSuper(undefined));
-  assertEquals(0, wasm.brOnCastFailNullStructSuper(structSuperObj));
-  assertEquals(0, wasm.brOnCastFailNullStructSuper(structSubObj));
-  assertEquals(1, wasm.brOnCastFailNullStructSuper(arrayObj));
-  assertEquals(1, wasm.brOnCastFailNullStructSuper(funcObj));
-  assertEquals(1, wasm.brOnCastFailNullStructSuper(1));
-  assertEquals(1, wasm.brOnCastFailNullStructSuper(jsObj));
-  assertEquals(1, wasm.brOnCastFailNullStructSuper(strObj));
-
-  assertEquals(0, wasm.brOnCastFailNullStructSub(null));
-  assertEquals(1, wasm.brOnCastFailNullStructSub(undefined));
-  assertEquals(1, wasm.brOnCastFailNullStructSub(structSuperObj));
-  assertEquals(0, wasm.brOnCastFailNullStructSub(structSubObj));
-  assertEquals(1, wasm.brOnCastFailNullStructSub(arrayObj));
-  assertEquals(1, wasm.brOnCastFailNullStructSub(funcObj));
-  assertEquals(1, wasm.brOnCastFailNullStructSub(1));
-  assertEquals(1, wasm.brOnCastFailNullStructSub(jsObj));
-  assertEquals(1, wasm.brOnCastFailNullStructSub(strObj));
-
-  assertEquals(0, wasm.brOnCastFailNullArray(null));
-  assertEquals(1, wasm.brOnCastFailNullArray(undefined));
-  assertEquals(1, wasm.brOnCastFailNullArray(structSuperObj));
-  assertEquals(1, wasm.brOnCastFailNullArray(structSubObj));
-  assertEquals(0, wasm.brOnCastFailNullArray(arrayObj));
-  assertEquals(1, wasm.brOnCastFailNullArray(funcObj));
-  assertEquals(1, wasm.brOnCastFailNullArray(1));
-  assertEquals(1, wasm.brOnCastFailNullArray(jsObj));
-  assertEquals(1, wasm.brOnCastFailNullArray(strObj));
-
-  assertEquals(0, wasm.brOnCastFailNullI31(null));
-  assertEquals(1, wasm.brOnCastFailNullI31(undefined));
-  assertEquals(1, wasm.brOnCastFailNullI31(structSuperObj));
-  assertEquals(1, wasm.brOnCastFailNullI31(structSubObj));
-  assertEquals(1, wasm.brOnCastFailNullI31(arrayObj));
-  assertEquals(1, wasm.brOnCastFailNullI31(funcObj));
-  assertEquals(0, wasm.brOnCastFailNullI31(1));
-  assertEquals(1, wasm.brOnCastFailNullI31(jsObj));
-  assertEquals(1, wasm.brOnCastFailNullI31(strObj));
-
-  assertEquals(0, wasm.brOnCastFailNullAnyArray(null));
-  assertEquals(1, wasm.brOnCastFailNullAnyArray(undefined));
-  assertEquals(1, wasm.brOnCastFailNullAnyArray(structSuperObj));
-  assertEquals(1, wasm.brOnCastFailNullAnyArray(structSubObj));
-  assertEquals(0, wasm.brOnCastFailNullAnyArray(arrayObj));
-  assertEquals(1, wasm.brOnCastFailNullAnyArray(funcObj));
-  assertEquals(1, wasm.brOnCastFailNullAnyArray(1));
-  assertEquals(1, wasm.brOnCastFailNullAnyArray(jsObj));
-  assertEquals(1, wasm.brOnCastFailNullAnyArray(strObj));
-
-  assertEquals(0, wasm.brOnCastFailNullStruct(null));
-  assertEquals(1, wasm.brOnCastFailNullStruct(undefined));
-  assertEquals(0, wasm.brOnCastFailNullStruct(structSuperObj));
-  assertEquals(0, wasm.brOnCastFailNullStruct(structSubObj));
-  assertEquals(1, wasm.brOnCastFailNullStruct(arrayObj));
-  assertEquals(1, wasm.brOnCastFailNullStruct(funcObj));
-  assertEquals(1, wasm.brOnCastFailNullStruct(1));
-  assertEquals(1, wasm.brOnCastFailNullStruct(jsObj));
-  assertEquals(1, wasm.brOnCastFailNullStruct(strObj));
-
-  assertEquals(0, wasm.brOnCastFailNullEq(null));
-  assertEquals(1, wasm.brOnCastFailNullEq(undefined));
-  assertEquals(0, wasm.brOnCastFailNullEq(structSuperObj));
-  assertEquals(0, wasm.brOnCastFailNullEq(structSubObj));
-  assertEquals(0, wasm.brOnCastFailNullEq(arrayObj));
-  assertEquals(1, wasm.brOnCastFailNullEq(funcObj));
-  assertEquals(0, wasm.brOnCastFailNullEq(1));
-  assertEquals(1, wasm.brOnCastFailNullEq(jsObj));
-  assertEquals(1, wasm.brOnCastFailNullEq(strObj));
-
-  assertEquals(0, wasm.brOnCastFailNullString(null));
-  assertEquals(1, wasm.brOnCastFailNullString(undefined));
-  assertEquals(1, wasm.brOnCastFailNullString(structSuperObj));
-  assertEquals(1, wasm.brOnCastFailNullString(structSubObj));
-  assertEquals(1, wasm.brOnCastFailNullString(arrayObj));
-  assertEquals(1, wasm.brOnCastFailNullString(funcObj));
-  assertEquals(1, wasm.brOnCastFailNullString(1));
-  assertEquals(1, wasm.brOnCastFailNullString(jsObj));
-  assertEquals(0, wasm.brOnCastFailNullString(strObj));
-
-  assertEquals(0, wasm.brOnCastFailNullAny(null));
-  assertEquals(0, wasm.brOnCastFailNullAny(undefined));
-  assertEquals(0, wasm.brOnCastFailNullAny(structSuperObj));
-  assertEquals(0, wasm.brOnCastFailNullAny(structSubObj));
-  assertEquals(0, wasm.brOnCastFailNullAny(arrayObj));
-  assertEquals(0, wasm.brOnCastFailNullAny(funcObj));
-  assertEquals(0, wasm.brOnCastFailNullAny(1));
-  assertEquals(0, wasm.brOnCastFailNullAny(jsObj));
-  assertEquals(0, wasm.brOnCastFailNullAny(strObj));
-
-  assertEquals(0, wasm.brOnCastFailNullNone(null));
-  assertEquals(1, wasm.brOnCastFailNullNone(undefined));
-  assertEquals(1, wasm.brOnCastFailNullNone(structSuperObj));
-  assertEquals(1, wasm.brOnCastFailNullNone(structSubObj));
-  assertEquals(1, wasm.brOnCastFailNullNone(arrayObj));
-  assertEquals(1, wasm.brOnCastFailNullNone(funcObj));
-  assertEquals(1, wasm.brOnCastFailNullNone(1));
-  assertEquals(1, wasm.brOnCastFailNullNone(jsObj));
-  assertEquals(1, wasm.brOnCastFailNullNone(strObj));
-
   // br_on_cast
   assertEquals(0, wasm.brOnCastGenericStructSuper(null));
   assertEquals(0, wasm.brOnCastGenericStructSuper(undefined));
