@@ -2,7 +2,7 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-// Flags: --experimental-wasm-imported-strings --experimental-wasm-gc
+// Flags: --experimental-wasm-imported-strings
 // For {isOneByteString}:
 // Flags: --expose-externalize-string
 
@@ -17,6 +17,7 @@ let kSig_e_v = makeSig([], [kRefExtern]);
 let kSig_i_ri = makeSig([kWasmExternRef, kWasmI32], [kWasmI32]);
 let kSig_i_rii = makeSig([kWasmExternRef, kWasmI32, kWasmI32], [kWasmI32]);
 let kSig_i_rr = makeSig([kWasmExternRef, kWasmExternRef], [kWasmI32]);
+let kSig_i_rri = makeSig([kWasmExternRef, kWasmExternRef, kWasmI32], [kWasmI32]);
 let kSig_i_riii = makeSig([kWasmExternRef, kWasmI32, kWasmI32, kWasmI32],
                           [kWasmI32]);
 let kSig_ii_riii = makeSig([kWasmExternRef, kWasmI32, kWasmI32, kWasmI32],
@@ -53,6 +54,14 @@ function HasIsolatedSurrogate(str) {
   }
   return false;
 }
+function ReplaceIsolatedSurrogates(str, replacement='\ufffd') {
+  let replaced = '';
+  for (let codepoint of str) {
+    replaced +=
+      IsSurrogate(codepoint.codePointAt(0)) ? replacement : codepoint;
+  }
+  return replaced;
+}
 
 function encodeWtf16LE(str) {
   // String iterator coalesces surrogate pairs.
@@ -67,9 +76,13 @@ function encodeWtf16LE(str) {
 
 let kArrayI16;
 let kArrayI8;
+let kStringCast;
+let kStringTest;
 let kStringFromWtf16Array;
-let kStringFromWtf8Array;
+let kStringFromUtf8Array;
+let kStringIntoUtf8Array;
 let kStringToWtf16Array;
+let kStringMeasureUtf8;
 let kStringFromCharCode;
 let kStringFromCodePoint;
 let kStringCharCodeAt;
@@ -79,40 +92,194 @@ let kStringConcat;
 let kStringSubstring;
 let kStringEquals;
 let kStringCompare;
+let kStringIndexOfImported;
+let kStringToLowerCaseImported;
 
 function MakeBuilder() {
   let builder = new WasmModuleBuilder();
-  kArrayI16 = builder.addArray(kWasmI16, true);
-  kArrayI8 = builder.addArray(kWasmI8, true);
+  builder.startRecGroup();
+  kArrayI16 = builder.addArray(kWasmI16, true, kNoSuperType, true);
+  builder.endRecGroup();
+  builder.startRecGroup();
+  kArrayI8 = builder.addArray(kWasmI8, true, kNoSuperType, true);
+  builder.endRecGroup();
   let arrayref = wasmRefNullType(kArrayI16);
   let array8ref = wasmRefNullType(kArrayI8);
 
+  kStringCast = builder.addImport('wasm:js-string', 'cast', kSig_e_r);
+  kStringTest = builder.addImport('wasm:js-string', 'test', kSig_i_r);
   kStringFromWtf16Array = builder.addImport(
-      'String', 'fromWtf16Array',
+      'wasm:js-string', 'fromCharCodeArray',
       makeSig([arrayref, kWasmI32, kWasmI32], [kRefExtern]));
-  kStringFromWtf8Array = builder.addImport(
-      'String', 'fromWtf8Array',
+  kStringFromUtf8Array = builder.addImport(
+      'wasm:text-decoder', 'decodeStringFromUTF8Array',
       makeSig([array8ref, kWasmI32, kWasmI32], [kRefExtern]));
   kStringToWtf16Array = builder.addImport(
-      'String', 'toWtf16Array',
+      'wasm:js-string', 'intoCharCodeArray',
       makeSig([kWasmExternRef, arrayref, kWasmI32], [kWasmI32]));
-  kStringFromCharCode = builder.addImport('String', 'fromCharCode', kSig_e_i);
-  kStringFromCodePoint = builder.addImport('String', 'fromCodePoint', kSig_e_i);
-  kStringCharCodeAt = builder.addImport('String', 'charCodeAt', kSig_i_ri);
-  kStringCodePointAt = builder.addImport('String', 'codePointAt', kSig_i_ri);
-  kStringLength = builder.addImport('String', 'length', kSig_i_r);
-  kStringConcat = builder.addImport('String', 'concat', kSig_e_rr);
-  kStringSubstring = builder.addImport('String', 'substring', kSig_e_rii);
-  kStringEquals = builder.addImport('String', 'equals', kSig_i_rr);
-  kStringCompare = builder.addImport('String', 'compare', kSig_i_rr);
+  kStringMeasureUtf8 =
+      builder.addImport('wasm:text-encoder', 'measureStringAsUTF8', kSig_i_r);
+  kStringIntoUtf8Array = builder.addImport(
+      'wasm:text-encoder', 'encodeStringIntoUTF8Array',
+      makeSig([kWasmExternRef, array8ref, kWasmI32], [kWasmI32]));
+  kStringFromCharCode =
+      builder.addImport('wasm:js-string', 'fromCharCode', kSig_e_i);
+  kStringFromCodePoint =
+      builder.addImport('wasm:js-string', 'fromCodePoint', kSig_e_i);
+  kStringCharCodeAt =
+      builder.addImport('wasm:js-string', 'charCodeAt', kSig_i_ri);
+  kStringCodePointAt =
+      builder.addImport('wasm:js-string', 'codePointAt', kSig_i_ri);
+  kStringLength = builder.addImport('wasm:js-string', 'length', kSig_i_r);
+  kStringConcat = builder.addImport('wasm:js-string', 'concat', kSig_e_rr);
+  kStringSubstring =
+      builder.addImport('wasm:js-string', 'substring', kSig_e_rii);
+  kStringEquals = builder.addImport('wasm:js-string', 'equals', kSig_i_rr);
+  kStringCompare = builder.addImport('wasm:js-string', 'compare', kSig_i_rr);
+  kStringIndexOfImported = builder.addImport('m', 'indexOf', kSig_i_rri);
+  kStringToLowerCaseImported = builder.addImport('m', 'toLowerCase', kSig_r_r);
 
   return builder;
 }
 
 let kImports = {
-  String: WebAssembly.String,
   strings: interestingStrings,
+  m: {
+    indexOf: Function.prototype.call.bind(String.prototype.indexOf),
+    toLowerCase: Function.prototype.call.bind(String.prototype.toLowerCase),
+  },
 };
+let kBuiltins = { builtins: ["js-string", "text-decoder", "text-encoder"] };
+
+(function TestStringCast() {
+  print(arguments.callee.name);
+  let builder = MakeBuilder();
+
+  builder.addFunction("cast", kSig_e_r)
+    .exportFunc()
+    .addBody([
+      kExprLocalGet, 0,
+      kExprCallFunction, kStringCast,
+    ]);
+
+  builder.addFunction("cast_null", kSig_e_v)
+    .exportFunc()
+    .addBody([
+      kExprRefNull, kExternRefCode,
+      kExprCallFunction, kStringCast,
+    ]);
+
+  let instance = builder.instantiate(kImports, kBuiltins);
+
+  assertEquals('foo', instance.exports.cast('foo'));
+  assertThrows(
+      () => instance.exports.cast(123), WebAssembly.RuntimeError,
+      'illegal cast');
+  assertThrows(
+      () => instance.exports.cast(undefined), WebAssembly.RuntimeError,
+      'illegal cast');
+  assertThrows(
+      () => instance.exports.cast(true), WebAssembly.RuntimeError,
+      'illegal cast');
+  assertThrows(
+      () => instance.exports.cast(null), WebAssembly.RuntimeError,
+      'illegal cast');
+  assertThrows(
+      () => instance.exports.cast_null(), WebAssembly.RuntimeError,
+      'illegal cast');
+})();
+
+(function TestStringTest() {
+  print(arguments.callee.name);
+  let builder = MakeBuilder();
+
+  builder.addFunction("test", kSig_i_r)
+    .exportFunc()
+    .addBody([
+      kExprLocalGet, 0,
+      kExprCallFunction, kStringTest,
+    ]);
+
+  builder.addFunction("test_null", kSig_i_v)
+    .exportFunc()
+    .addBody([
+      kExprRefNull, kExternRefCode,
+      kExprCallFunction, kStringTest,
+    ]);
+
+  let instance = builder.instantiate(kImports, kBuiltins);
+
+  assertEquals(1, instance.exports.test("foo"));
+  assertEquals(0, instance.exports.test(123));
+  assertEquals(0, instance.exports.test(undefined));
+  assertEquals(0, instance.exports.test(true));
+  assertEquals(0, instance.exports.test(null));
+  assertEquals(0, instance.exports.test_null());
+})();
+
+(function TestIndexOfImportedStrings() {
+  print(arguments.callee.name);
+  let builder = new MakeBuilder();
+
+  builder.addFunction('indexOf', kSig_i_rri).exportFunc().addBody([
+    kExprLocalGet, 0,
+    kExprCallFunction, kStringCast,
+    kExprLocalGet, 1,
+    kExprCallFunction, kStringCast,
+    kExprLocalGet, 2,
+    kExprCallFunction, kStringIndexOfImported,
+  ]);
+  let instance = builder.instantiate(kImports, kBuiltins);
+
+  assertEquals(2, instance.exports.indexOf('xxfooxx', 'foo', 0));
+  assertEquals(2, instance.exports.indexOf('xxfooxx', 'foo', -2));
+  assertEquals(-1, instance.exports.indexOf('xxfooxx', 'foo', 100));
+  // Make sure we don't lose bits when Smi-tagging of the start position.
+  assertEquals(-1, instance.exports.indexOf('xxfooxx', 'foo', 0x4000_0000));
+  assertEquals(-1, instance.exports.indexOf('xxfooxx', 'foo', 0x2000_0000));
+  assertEquals(
+      2,
+      instance.exports.indexOf(
+          'xxfooxx', 'foo', 0x8000_0000));  // Negative i32.
+
+  // Both first and second args should be non-null strings.
+  assertThrows(
+      () => instance.exports.indexOf('xxnullxx', null, 0),
+      WebAssembly.RuntimeError, 'illegal cast');
+  assertThrows(
+      () => instance.exports.indexOf(12345, 234, 0), WebAssembly.RuntimeError,
+      'illegal cast');
+  assertThrows(
+      () => instance.exports.indexOf(null, 'foo', 0), WebAssembly.RuntimeError,
+      'illegal cast');
+  assertThrows(
+      () => instance.exports.indexOf(null, 'null', 0), WebAssembly.RuntimeError,
+      'illegal cast');
+})();
+
+(function TestStringToLowerCaseImported() {
+  print(arguments.callee.name);
+  let builder = new MakeBuilder();
+
+  builder.addFunction('toLowerCase', kSig_r_r).exportFunc().addBody([
+    kExprLocalGet, 0,
+    kExprCallFunction, kStringCast,
+    kExprCallFunction, kStringToLowerCaseImported,
+  ]);
+  let instance = builder.instantiate(kImports, kBuiltins);
+
+  assertEquals(
+      'make this lowercase!',
+      instance.exports.toLowerCase('MAKE THIS LOWERCASE!'));
+
+  // The argument should be a non-null string.
+  assertThrows(
+      () => instance.exports.toLowerCase(null), WebAssembly.RuntimeError,
+      'illegal cast');
+  assertThrows(
+      () => instance.exports.toLowerCase(123), WebAssembly.RuntimeError,
+      'illegal cast');
+})();
 
 (function TestStringConst() {
   print(arguments.callee.name);
@@ -126,7 +293,7 @@ let kImports = {
     builder.addGlobal(kRefExtern, false, [kExprGlobalGet, i])
       .exportAs("global" + i);
   }
-  let instance = builder.instantiate(kImports);
+  let instance = builder.instantiate(kImports, kBuiltins);
   for (let [i, str] of interestingStrings.entries()) {
     assertEquals(str, instance.exports["string_const" + i]());
     assertEquals(str, instance.exports["global" + i].value);
@@ -151,7 +318,7 @@ let kImports = {
       kExprCallFunction, kStringLength,
     ]);
 
-  let instance = builder.instantiate(kImports);
+  let instance = builder.instantiate(kImports, kBuiltins);
   for (let str of interestingStrings) {
     assertEquals(str.length, instance.exports.string_length(str));
   }
@@ -190,7 +357,7 @@ let kImports = {
       kExprCallFunction, kStringConcat,
     ]);
 
-  let instance = builder.instantiate(kImports);
+  let instance = builder.instantiate(kImports, kBuiltins);
 
   for (let head of interestingStrings) {
     for (let tail of interestingStrings) {
@@ -244,7 +411,7 @@ let kImports = {
       kExprCallFunction, kStringEquals,
     ]);
 
-  let instance = builder.instantiate(kImports);
+  let instance = builder.instantiate(kImports, kBuiltins);
 
   for (let head of interestingStrings) {
     for (let tail of interestingStrings) {
@@ -315,7 +482,7 @@ let kImports = {
       kExprCallFunction, kStringSubstring,
     ]);
 
-  let instance = builder.instantiate(kImports);
+  let instance = builder.instantiate(kImports, kBuiltins);
   for (let str of interestingStrings) {
     for (let i = 0; i < str.length; i++) {
       assertEquals(str.charCodeAt(i),
@@ -416,7 +583,7 @@ let kImports = {
       kExprCallFunction, kStringCompare,
     ]);
 
-  let instance = builder.instantiate(kImports);
+  let instance = builder.instantiate(kImports, kBuiltins);
   for (let lhs of interestingStrings) {
     for (let rhs of interestingStrings) {
       const expected = lhs < rhs ? -1 : lhs > rhs ? 1 : 0;
@@ -459,7 +626,7 @@ let kImports = {
       kExprCallFunction, kStringCompare,
     ]);
 
-  let instance = builder.instantiate(kImports);
+  let instance = builder.instantiate(kImports, kBuiltins);
   assertThrows(() => instance.exports.compareLhsNullable(null, "abc"),
                WebAssembly.RuntimeError, "dereferencing a null pointer");
   assertThrows(() => instance.exports.compareLhsNullable("abc", null),
@@ -480,7 +647,7 @@ let kImports = {
       kExprCallFunction, kStringFromCodePoint,
     ]);
 
-  let instance = builder.instantiate(kImports);
+  let instance = builder.instantiate(kImports, kBuiltins);
   for (let char of "Az1#\n\ucccc\ud800\udc00") {
     assertEquals(char, instance.exports.asString(char.codePointAt(0)));
   }
@@ -500,7 +667,7 @@ let kImports = {
       kExprCallFunction, kStringFromCharCode,
     ]);
 
-  let instance = builder.instantiate(kImports);
+  let instance = builder.instantiate(kImports, kBuiltins);
   let inputs = "Az1#\n\ucccc\ud800\udc00";
   for (let i = 0; i < inputs.length; i++) {
     assertEquals(inputs.charAt(i),
@@ -546,12 +713,27 @@ function makeWtf8TestDataSegment() {
       data.push(byte);
     }
   }
-  for (let bytes of ['trailing high byte \xa9',
-                     'interstitial high \xa9 byte',
-                     'invalid \xc0 byte',
-                     'invalid three-byte \xed\xd0\x80',
-                     'surrogate \xed\xa0\x80\xed\xb0\x80 pair']) {
-    invalid[bytes] = { offset: data.length, length: bytes.length };
+  let invalid_inputs = [
+    'trailing high byte \xa9',
+    'interstitial high \xa9 byte',
+    'invalid \xc0 byte',
+    'invalid three-byte \xed\xd0\x80',
+    'surrogate \xed\xa0\x80\xed\xb0\x80 pair'
+  ];
+  let invalid_replaced = [
+    'trailing high byte \uFFFD',
+    'interstitial high \uFFFD byte',
+    'invalid \uFFFD byte',
+    'invalid three-byte \uFFFD\u0400',
+    'surrogate \uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD pair'
+  ];
+  for (let i = 0; i < invalid_inputs.length; i++) {
+    let bytes = invalid_inputs[i];
+    invalid[bytes] = {
+      offset: data.length,
+      length: bytes.length,
+      replaced: invalid_replaced[i]
+    };
     for (let i = 0; i < bytes.length; i++) {
       data.push(bytes.charCodeAt(i));
     }
@@ -559,7 +741,7 @@ function makeWtf8TestDataSegment() {
   return { valid, invalid, data: Uint8Array.from(data) };
 };
 
-(function TestStringNewWtf8Array() {
+(function TestStringNewUtf8Array() {
   print(arguments.callee.name);
   let builder = MakeBuilder();
   let data = makeWtf8TestDataSegment();
@@ -576,12 +758,12 @@ function makeWtf8TestDataSegment() {
       kGCPrefix, kExprArrayNewData, kArrayI8, data_index
     ]).index;
 
-  builder.addFunction("new_wtf8", kSig_e_ii)
+  builder.addFunction("new_utf8", kSig_e_ii)
     .exportFunc()
     .addBody([
       kExprCallFunction, make_i8_array,
       kExprLocalGet, 0, kExprLocalGet, 1,
-      kExprCallFunction, kStringFromWtf8Array,
+      kExprCallFunction, kStringFromUtf8Array,
     ]);
 
   builder.addFunction("bounds_check", kSig_e_ii)
@@ -591,27 +773,38 @@ function makeWtf8TestDataSegment() {
       ...wasmI32Const("ascii".length),
       kGCPrefix, kExprArrayNewData, kArrayI8, ascii_data_index,
       kExprLocalGet, 0, kExprLocalGet, 1,
-      kExprCallFunction, kStringFromWtf8Array,
+      kExprCallFunction, kStringFromUtf8Array,
     ]);
 
   builder.addFunction("null_array", kSig_e_v).exportFunc()
     .addBody([
       kExprRefNull, kArrayI8,
       kExprI32Const, 0, kExprI32Const, 0,
-      kExprCallFunction, kStringFromWtf8Array,
+      kExprCallFunction, kStringFromUtf8Array,
     ]);
 
-  let instance = builder.instantiate(kImports);
+  let instance = builder.instantiate(kImports, kBuiltins);
   for (let [str, {offset, length}] of Object.entries(data.valid)) {
     let start = offset;
     let end = offset + length;
-    assertEquals(str, instance.exports.new_wtf8(start, end));
+    if (HasIsolatedSurrogate(str)) {
+      // Isolated surrogates have the three-byte pattern ED [A0,BF] [80,BF].
+      // When the sloppy decoder gets to the second byte, it will reject
+      // the sequence, and then retry parsing at the second byte.
+      // Seeing the second byte can't start a sequence, it replaces the
+      // second byte and continues with the next, which also can't start
+      // a sequence.  The result is that one isolated surrogate is replaced
+      // by three U+FFFD codepoints.
+      assertEquals(ReplaceIsolatedSurrogates(str, '\ufffd\ufffd\ufffd'),
+                   instance.exports.new_utf8(start, end));
+    } else {
+      assertEquals(str, instance.exports.new_utf8(start, end));
+    }
   }
-  for (let [str, {offset, length}] of Object.entries(data.invalid)) {
+  for (let [str, {offset, length, replaced}] of Object.entries(data.invalid)) {
     let start = offset;
     let end = offset + length;
-    assertThrows(() => instance.exports.new_wtf8(start, end),
-                 WebAssembly.RuntimeError, "invalid WTF-8 string");
+    assertEquals(replaced, instance.exports.new_utf8(start, end));
   }
 
   assertEquals("ascii", instance.exports.bounds_check(0, "ascii".length));
@@ -707,7 +900,7 @@ function makeWtf16TestDataSegment(strings) {
       kExprCallFunction, kStringFromWtf16Array,
     ]);
 
-  let instance = builder.instantiate(kImports);
+  let instance = builder.instantiate(kImports, kBuiltins);
   for (let [str, {offset, length}] of Object.entries(data.valid)) {
     let start = offset / 2;
     let end = start + length;
@@ -778,7 +971,7 @@ function makeWtf16TestDataSegment(strings) {
       kExprCallFunction, kStringToWtf16Array,
     ]);
 
-  let instance = builder.instantiate(kImports);
+  let instance = builder.instantiate(kImports, kBuiltins);
   for (let str of interestingStrings) {
     assertEquals(str, instance.exports.encode(str, str.length, 0));
     assertEquals(str, instance.exports.encode(str, str.length + 20, 10));
@@ -797,3 +990,107 @@ function makeWtf16TestDataSegment(strings) {
                  WebAssembly.RuntimeError, message);
   }
 })();
+
+(function TestStringMeasureUtf8() {
+  print(arguments.callee.name);
+  let builder = MakeBuilder();
+
+  builder.addFunction("string_measure_utf8", kSig_i_r)
+    .exportFunc()
+    .addBody([
+      kExprLocalGet, 0,
+      kExprCallFunction, kStringMeasureUtf8,
+    ]);
+
+  builder.addFunction("string_measure_utf8_null", kSig_i_v)
+    .exportFunc()
+    .addBody([
+      kExprRefNull, kExternRefCode,
+      kExprCallFunction, kStringMeasureUtf8,
+    ]);
+
+  let instance = builder.instantiate(kImports, kBuiltins);
+  for (let str of interestingStrings) {
+    let wtf8 = encodeWtf8(str);
+    assertEquals(wtf8.length, instance.exports.string_measure_utf8(str));
+  }
+
+  assertThrows(() => instance.exports.string_measure_utf8_null(),
+               WebAssembly.RuntimeError, "illegal cast");
+})();
+
+(function TestStringEncodeUtf8Array() {
+  print(arguments.callee.name);
+  let builder = MakeBuilder();
+
+  // Allocate an array that's exactly the expected size, and encode
+  // into it.  Then decode it.
+  // (str, length, offset=0) -> str
+  builder.addFunction("encode_utf8", kSig_e_rii)
+    .exportFunc()
+    .addLocals(wasmRefNullType(kArrayI8), 1)
+    .addLocals(kWasmI32, 1)
+    .addBody([
+      // Allocate buffer.
+      kExprLocalGet, 1,
+      kGCPrefix, kExprArrayNewDefault, kArrayI8,
+      kExprLocalSet, 3,
+
+      // Write buffer, store number of bytes written.
+      kExprLocalGet, 0,
+      kExprLocalGet, 3,
+      kExprLocalGet, 2,
+      kExprCallFunction, kStringIntoUtf8Array,
+      kExprLocalSet, 4,
+
+      // Read buffer.
+      kExprLocalGet, 3,
+      kExprLocalGet, 2,
+      kExprLocalGet, 2, kExprLocalGet, 4, kExprI32Add,
+      kExprCallFunction, kStringFromUtf8Array,
+    ]);
+
+
+  builder.addFunction("encode_null_string", kSig_i_v)
+    .exportFunc()
+    .addBody([
+        kExprRefNull, kExternRefCode,
+        kExprI32Const, 0, kGCPrefix, kExprArrayNewDefault, kArrayI8,
+        kExprI32Const, 0,
+        kExprCallFunction, kStringIntoUtf8Array,
+      ]);
+  builder.addFunction("encode_null_array", kSig_i_v)
+    .exportFunc()
+    .addBody([
+        kExprI32Const, 0, kGCPrefix, kExprArrayNewDefault, kArrayI8,
+        kExprI32Const, 0, kExprI32Const, 0,
+        kExprCallFunction, kStringFromUtf8Array,
+        kExprRefNull, kArrayI8,
+        kExprI32Const, 0,
+        kExprCallFunction, kStringIntoUtf8Array,
+      ]);
+
+  let instance = builder.instantiate(kImports, kBuiltins);
+
+  for (let str of interestingStrings) {
+    let replaced = ReplaceIsolatedSurrogates(str);
+    if (!HasIsolatedSurrogate(str)) assertEquals(str, replaced);
+    let wtf8 = encodeWtf8(replaced);
+    assertEquals(replaced,
+                 instance.exports.encode_utf8(str, wtf8.length, 0));
+    assertEquals(replaced,
+                 instance.exports.encode_utf8(str, wtf8.length + 20, 10));
+  }
+
+  assertThrows(() => instance.exports.encode_null_array(),
+               WebAssembly.RuntimeError, "dereferencing a null pointer");
+  assertThrows(() => instance.exports.encode_null_string(),
+               WebAssembly.RuntimeError, "illegal cast");
+
+  for (let str of interestingStrings) {
+    let wtf8 = encodeWtf8(str);
+    let message = "array element access out of bounds";
+    assertThrows(() => instance.exports.encode_utf8(str, wtf8.length, 1),
+                 WebAssembly.RuntimeError, message);
+  }
+})();
