@@ -11,18 +11,9 @@
 
 d8.file.execute("test/mjsunit/wasm/wasm-module-builder.js");
 
-function ToPromising(wasm_export) {
-  let sig = WebAssembly.Function.type(wasm_export);
-  assertTrue(sig.parameters.length > 0);
-  assertEquals('externref', sig.parameters[0]);
-  assertEquals(1, sig.results.length);
-  let wrapper_sig = {
-    parameters: sig.parameters.slice(1),
-    results: ['externref']
-  };
-  return new WebAssembly.Function(
-      wrapper_sig, wasm_export, {promising: 'first'});
-
+function GC() {
+  %CheckIsOnCentralStack();
+  gc();
 }
 
 (function testGenericWrapper0Param() {
@@ -38,13 +29,13 @@ function ToPromising(wasm_export) {
 
   let x = 12;
   function import_func() {
-    gc();
+    GC();
     x = 20;
   }
 
   let instance = builder.instantiate({ mod: { func: import_func } });
   let main = ToPromising(instance.exports.main);
-  assertEquals(undefined, main());
+  assertPromiseResult(main(), v => assertEquals(undefined, v));
   assertEquals(20, x);
 })();
 
@@ -60,7 +51,7 @@ function ToPromising(wasm_export) {
 
   let instance = builder.instantiate();
   let main = ToPromising(instance.exports.main);
-  assertTraps(kTrapUnreachable, main);
+  assertThrowsAsync(main(), WebAssembly.RuntimeError);
 })();
 
 (function testGenericWrapper1ParamTrap() {
@@ -76,7 +67,7 @@ function ToPromising(wasm_export) {
 
   let instance = builder.instantiate();
   let main = ToPromising(instance.exports.main);
-  assertTraps(kTrapUnreachable, main);
+  assertThrowsAsync(main(), WebAssembly.RuntimeError);
 })();
 
 (function testGenericWrapper1ParamGeneral() {
@@ -95,14 +86,14 @@ function ToPromising(wasm_export) {
 
   let x = 12;
   function import_func(suspender, param) {
-    gc();
+    GC();
     assertInstanceof(suspender, WebAssembly.Suspender);
     x += param;
   }
 
   let instance = builder.instantiate({ mod: { func: import_func } });
   let main = ToPromising(instance.exports.main);
-  assertEquals(undefined, main(5));
+  assertPromiseResult(main(5), v => { assertEquals(undefined, v); });
   assertEquals(17, x);
 })();
 
@@ -122,15 +113,15 @@ function ToPromising(wasm_export) {
 
   let x = 12;
   function import_func(suspender, param) {
-    gc();
+    GC();
     assertInstanceof(suspender, WebAssembly.Suspender);
     x += param;
   }
 
-  let y = { valueOf: () => { print("Hello!"); gc(); return 24; } };
+  let y = { valueOf: () => { print("Hello!"); GC(); return 24; } };
   let instance = builder.instantiate({ mod: { func: import_func } });
   let main = ToPromising(instance.exports.main);
-  assertEquals(undefined, main(y));
+  assertPromiseResult(main(y), v => { assertEquals(undefined, v); });
   assertEquals(36, x);
 })();
 
@@ -154,16 +145,18 @@ function ToPromising(wasm_export) {
 
   let x = 12;
   function import_func(suspender, param1, param2, param3, param4) {
-    gc();
+    GC();
     assertInstanceof(suspender, WebAssembly.Suspender);
     x += 2 * param1 + 3 * param2 + 4 * param3 + 5 * param4;
   }
 
-  let param2 = { valueOf: () => { gc(); return 6; } };
-  let param3 = { valueOf: () => { gc(); return 3; } };
+  let param2 = { valueOf: () => { GC(); return 6; } };
+  let param3 = { valueOf: () => { GC(); return 3; } };
   let instance = builder.instantiate({ mod: { func: import_func } });
   let main = ToPromising(instance.exports.main);
-  assertEquals(undefined, main(9, param2, param3, 0));
+  assertPromiseResult(
+      main(9, param2, param3, 0),
+      v => { assertEquals(undefined, v); });
   assertEquals(60, x);
   // Now we test if the evaluation order of the parameters is correct.
   x = 12;
@@ -175,7 +168,7 @@ function ToPromising(wasm_export) {
       return 3;
     }
   };
-  assertEquals(undefined, main(9, param2, param3, 0));
+  assertPromiseResult(main(9, param2, param3, 0), undefined);
   assertEquals(60, x);
 })();
 
@@ -205,19 +198,21 @@ let kSig_r_riiiiiiii = makeSig([kWasmExternRef, kWasmI32, kWasmI32, kWasmI32,
   let x = 12;
   function import_func(suspender, param1, param2, param3, param4, param5,
       param6, param7, param8) {
-    gc();
+    GC();
     assertInstanceof(suspender, WebAssembly.Suspender);
     x += param1 + 2 * param2 + 3 * param3 + 4 * param4 + 5 * param5
       + 6 * param6 + 7 * param7 + 8 * param8;
   }
 
-  let param1 = { valueOf: () => { gc(); return 5; } };
-  let param4 = { valueOf: () => { gc(); return 8; } };
-  let param6 = { valueOf: () => { gc(); return 10; } };
-  let param8 = { valueOf: () => { gc(); return 12; } };
+  let param1 = { valueOf: () => { GC(); return 5; } };
+  let param4 = { valueOf: () => { GC(); return 8; } };
+  let param6 = { valueOf: () => { GC(); return 10; } };
+  let param8 = { valueOf: () => { GC(); return 12; } };
   let instance = builder.instantiate({ mod: { func: import_func } });
   let main = ToPromising(instance.exports.main);
-  assertEquals(undefined, main(param1, 6, 7, param4, 9, param6, 11, param8));
+  assertPromiseResult(
+      main(param1, 6, 7, param4, 9, param6, 11, param8),
+      v => assertEquals(undefined, v));
   assertEquals(360, x);
 })();
 
@@ -242,15 +237,15 @@ let kSig_r_riiiiiiii = makeSig([kWasmExternRef, kWasmI32, kWasmI32, kWasmI32,
 
   let x = 12;
   function import_func(suspender, param1, param2, param3, param4) {
-    gc();
+    GC();
     assertInstanceof(suspender, WebAssembly.Suspender);
     x += param1 + param2 + param3 + param4;
   }
 
-  let param2 = { valueOf: () => { gc(); return 3; } };
+  let param2 = { valueOf: () => { GC(); return 3; } };
   let instance = builder.instantiate({ mod: { func: import_func } });
   let main = ToPromising(instance.exports.main);
-  assertEquals(undefined, main(5, param2));
+  assertPromiseResult(main(5, param2), v => assertEquals(undefined, v));
   assertEquals(20, x);
 })();
 
@@ -275,16 +270,18 @@ let kSig_r_riiiiiiii = makeSig([kWasmExternRef, kWasmI32, kWasmI32, kWasmI32,
 
   let x = 12;
   function import_func(suspender, param1, param2, param3, param4) {
-    gc();
+    GC();
     assertInstanceof(suspender, WebAssembly.Suspender);
     x += param1 + param2 + param3 + param4;
   }
 
-  let param2 = { valueOf: () => { gc(); return 3; } };
-  let param3 = { valueOf: () => { gc(); return 6; } };
+  let param2 = { valueOf: () => { GC(); return 3; } };
+  let param3 = { valueOf: () => { GC(); return 6; } };
   let instance = builder.instantiate({ mod: { func: import_func } });
   let main = ToPromising(instance.exports.main);
-  assertEquals(undefined, main(5, param2, param3, 7, 200, 300, 400));
+  assertPromiseResult(
+      main(5, param2, param3, 7, 200, 300, 400),
+      v => assertEquals(undefined, v));
   assertEquals(33, x);
 })();
 
@@ -304,14 +301,14 @@ let kSig_r_riiiiiiii = makeSig([kWasmExternRef, kWasmI32, kWasmI32, kWasmI32,
 
   let x = 12;
   function import_func(suspender, param) {
-    gc();
+    GC();
     assertInstanceof(suspender, WebAssembly.Suspender);
     return x + param;
   }
 
   let instance = builder.instantiate({ mod: { func: import_func } });
   let main = ToPromising(instance.exports.main);
-  assertEquals(17, main(5));
+  assertPromiseResult(main(5), v => assertEquals(17, v));
 })();
 
 (function testGenericWrapper1I32ReturnHeapNumber() {
@@ -338,7 +335,7 @@ let kSig_r_riiiiiiii = makeSig([kWasmExternRef, kWasmI32, kWasmI32, kWasmI32,
 
   let instance = builder.instantiate({ mod: { func: import_func } });
   let main = ToPromising(instance.exports.main);
-  assertEquals(2147483645, main(5));
+  assertPromiseResult(main(5), v => assertEquals(2147483645, v));
 })();
 
 let kSig_i_rlili = makeSig([kWasmExternRef, kWasmI64, kWasmI32, kWasmI64, kWasmI32],
@@ -367,11 +364,11 @@ let kSig_i_rlili = makeSig([kWasmExternRef, kWasmI64, kWasmI32, kWasmI64, kWasmI
     return Number(x);
   }
 
-  let param2 = { valueOf: () => { gc(); return 6; } };
-  let param3 = { valueOf: () => { gc(); return 3n; } };
+  let param2 = { valueOf: () => { GC(); return 6; } };
+  let param3 = { valueOf: () => { GC(); return 3n; } };
   let instance = builder.instantiate({ mod: { func: import_func } });
   let main = ToPromising(instance.exports.main);
-  assertEquals(60, main(9n, param2, param3, 0));
+  assertPromiseResult(main(9n, param2, param3, 0), v => assertEquals(60, v));
 })();
 
 let kSig_r_riiili = makeSig([kWasmExternRef, kWasmI32, kWasmI32, kWasmI32, kWasmI64,
@@ -396,16 +393,18 @@ let kSig_r_riiili = makeSig([kWasmExternRef, kWasmI32, kWasmI32, kWasmI32, kWasm
 
   let x = 12;
   function import_func(suspender, param1, param2, param3, param4, param5) {
-    gc();
+    GC();
     assertInstanceof(suspender, WebAssembly.Suspender);
     x += 2 * param1 + 3 * param2 + 4 * param3 + 5 * Number(param4) + 6 * param5;
   }
 
-  let param2 = { valueOf: () => { gc(); return 6; } };
-  let param3 = { valueOf: () => { gc(); return 3; } };
+  let param2 = { valueOf: () => { GC(); return 6; } };
+  let param3 = { valueOf: () => { GC(); return 3; } };
   let instance = builder.instantiate({ mod: { func: import_func } });
   let main = ToPromising(instance.exports.main);
-  assertEquals(undefined, main(9, param2, param3, 0n, 2));
+  assertPromiseResult(
+      main(9, param2, param3, 0n, 2),
+      v => assertEquals(undefined, v));
   assertEquals(72, x);
 })();
 
@@ -438,11 +437,13 @@ let kSig_r_riiilii = makeSig([kWasmExternRef, kWasmI32, kWasmI32, kWasmI32,
     x += 2 * param1 + 3 * param2 + 4 * param3 + 5 * Number(param4) + 6 * param5 + 7 * param6;
   }
 
-  let param2 = { valueOf: () => { gc(); return 6; } };
-  let param3 = { valueOf: () => { gc(); return 3; } };
+  let param2 = { valueOf: () => { GC(); return 6; } };
+  let param3 = { valueOf: () => { GC(); return 3; } };
   let instance = builder.instantiate({ mod: { func: import_func } });
   let main = ToPromising(instance.exports.main);
-  assertEquals(undefined, main(9, param2, param3, 0n, 2, 3));
+  assertPromiseResult(
+      main(9, param2, param3, 0n, 2, 3),
+      v => assertEquals(undefined, v));
   assertEquals(93, x);
 })();
 
@@ -479,13 +480,15 @@ let kSig_r_rliilliiil = makeSig([kWasmExternRef, kWasmI64, kWasmI32, kWasmI32,
     return x;
   }
 
-  let param1 = { valueOf: () => { gc(); return 5n; } };
-  let param4 = { valueOf: () => { gc(); return 8n; } };
-  let param6 = { valueOf: () => { gc(); return 10; } };
-  let param8 = { valueOf: () => { gc(); return 12; } };
+  let param1 = { valueOf: () => { GC(); return 5n; } };
+  let param4 = { valueOf: () => { GC(); return 8n; } };
+  let param6 = { valueOf: () => { GC(); return 10; } };
+  let param8 = { valueOf: () => { GC(); return 12; } };
   let instance = builder.instantiate({ mod: { func: import_func } });
   let main = ToPromising(instance.exports.main);
-  assertEquals(360, main(param1, 6, 7, param4, 9n, param6, 11, param8, 0n));
+  assertPromiseResult(
+      main(param1, 6, 7, param4, 9n, param6, 11, param8, 0n),
+      v => assertEquals(360, v));
 })();
 
 // The function expects BigInt, but gets Number.
@@ -511,7 +514,7 @@ let kSig_r_rliilliiil = makeSig([kWasmExternRef, kWasmI64, kWasmI32, kWasmI32,
 
   let instance = builder.instantiate({ mod: { func: import_func } });
   let main = ToPromising(instance.exports.main);
-  assertThrows(() => { main(17) }, TypeError);
+  assertThrows(() => main(17), TypeError);
 })();
 
 (function testGenericWrapper1I64Return() {
@@ -528,14 +531,14 @@ let kSig_r_rliilliiil = makeSig([kWasmExternRef, kWasmI64, kWasmI32, kWasmI32,
     .exportFunc();
 
   function import_func(suspender) {
-    gc();
+    GC();
     assertInstanceof(suspender, WebAssembly.Suspender);
     return 10000000000n;
   }
 
   let instance = builder.instantiate({ mod: { func: import_func } });
   let main = ToPromising(instance.exports.main);
-  assertEquals(10000000000n, main());
+  assertPromiseResult(main(), v => assertEquals(10000000000n, v));
 })();
 
 (function testGenericWrapper1F32Return() {
@@ -552,14 +555,14 @@ let kSig_r_rliilliiil = makeSig([kWasmExternRef, kWasmI64, kWasmI32, kWasmI32,
     .exportFunc();
 
   function import_func(suspender) {
-    gc();
+    GC();
     assertInstanceof(suspender, WebAssembly.Suspender);
     return 0.5;
   }
 
   let instance = builder.instantiate({ mod: { func: import_func } });
   let main = ToPromising(instance.exports.main);
-  assertEquals(0.5, main());
+  assertPromiseResult(main(), v => assertEquals(0.5, v));
 })();
 
 (function testGenericWrapper1F64Return() {
@@ -576,14 +579,14 @@ let kSig_r_rliilliiil = makeSig([kWasmExternRef, kWasmI64, kWasmI32, kWasmI32,
     .exportFunc();
 
   function import_func(suspender) {
-    gc();
+    GC();
     assertInstanceof(suspender, WebAssembly.Suspender);
     return 0.25;
   }
 
   let instance = builder.instantiate({ mod: { func: import_func } });
   let main = ToPromising(instance.exports.main);
-  assertEquals(0.25, main());
+  assertPromiseResult(main(), v => assertEquals(0.25, v));
 })();
 
 (function testGenericWrapper1Float32() {
@@ -602,14 +605,14 @@ let kSig_r_rliilliiil = makeSig([kWasmExternRef, kWasmI64, kWasmI32, kWasmI32,
 
   let x = 12.5;
   function import_func(suspender, param) {
-    gc();
+    GC();
     assertInstanceof(suspender, WebAssembly.Suspender);
     x += param;
   }
 
   let instance = builder.instantiate({ mod: { func: import_func } });
   let main = ToPromising(instance.exports.main);
-  assertEquals(undefined, main(12.5));
+  assertPromiseResult(main(12.5), v => assertEquals(undefined, v));
   assertEquals(25, x);
 })();
 
@@ -629,14 +632,14 @@ let kSig_r_rliilliiil = makeSig([kWasmExternRef, kWasmI64, kWasmI32, kWasmI32,
 
   let x = 12.5;
   function import_func(suspender, param) {
-    gc();
+    GC();
     assertInstanceof(suspender, WebAssembly.Suspender);
     x += param;
   }
 
   let instance = builder.instantiate({ mod: { func: import_func } });
   let main = ToPromising(instance.exports.main);
-  assertEquals(undefined, main(12.5));
+  assertPromiseResult(main(12.5), v => assertEquals(undefined, v));
   assertEquals(25, x);
 })();
 
@@ -666,20 +669,21 @@ let kSig_r_rffddddff = makeSig([kWasmExternRef, kWasmF32, kWasmF32, kWasmF64,
   let x = 12;
   function import_func(suspender, param1, param2, param3, param4, param5,
       param6, param7, param8) {
-    gc();
+    GC();
     assertInstanceof(suspender, WebAssembly.Suspender);
     x += param1 + 2 * param2 + 3 * param3 + 4 * param4 + 5 * param5
       + 6 * param6 + 7 * param7 + 8 * param8;
   }
 
-  let param1 = { valueOf: () => { gc(); return 1.5; } };
-  let param4 = { valueOf: () => { gc(); return 4.5; } };
-  let param6 = { valueOf: () => { gc(); return 6.5; } };
-  let param8 = { valueOf: () => { gc(); return 8.5; } };
+  let param1 = { valueOf: () => { GC(); return 1.5; } };
+  let param4 = { valueOf: () => { GC(); return 4.5; } };
+  let param6 = { valueOf: () => { GC(); return 6.5; } };
+  let param8 = { valueOf: () => { GC(); return 8.5; } };
   let instance = builder.instantiate({ mod: { func: import_func } });
   let main = ToPromising(instance.exports.main);
-  assertEquals(undefined, main(param1, 2.5, 3.5, param4, 5.5, param6, 7.5,
-        param8));
+  assertPromiseResult(
+      main(param1, 2.5, 3.5, param4, 5.5, param6, 7.5, param8),
+      v => assertEquals(undefined, v));
   assertEquals(234, x);
 })();
 
@@ -716,7 +720,7 @@ let kSig_r_riiliffddlfdff = makeSig([kWasmExternRef, kWasmI32, kWasmI32, kWasmI6
   let y = 1.0;
   function import_func(suspender, parami1, parami2, paraml1, parami3, paramf1,
       paramf2, paramd1, paramd2, paraml2, paramf3, paramd3, paramf4, paramf5) {
-    gc();
+    GC();
     assertInstanceof(suspender, WebAssembly.Suspender);
     x += parami1 + 2 * parami2 + 3 * Number(paraml1) + 4 * parami3
       + 5 * Number(paraml2);
@@ -726,8 +730,9 @@ let kSig_r_riiliffddlfdff = makeSig([kWasmExternRef, kWasmI32, kWasmI32, kWasmI6
 
   let instance = builder.instantiate({ mod: { func: import_func } });
   let main = ToPromising(instance.exports.main);
-  assertEquals(undefined, main(5, 6, 7n, 8, 1.5, 2.5, 3.5, 4.5, 11n, 5.5, 6.5,
-        7.5, 8.5));
+  assertPromiseResult(
+      main(5, 6, 7n, 8, 1.5, 2.5, 3.5, 4.5, 11n, 5.5, 6.5, 7.5, 8.5),
+      v => assertEquals(undefined, v));
   assertEquals(137, x);
   assertEquals(223, y);
 })();
@@ -764,21 +769,22 @@ let kSig_r_riiliiiffddli = makeSig([kWasmExternRef, kWasmI32, kWasmI32, kWasmI64
   let y = 1.0;
   function import_func(suspender, param1, param2, param3, param4, param5,
       param6, paramf1, paramf2, paramd1, paramd2, param7, param8) {
-    gc();
+    GC();
     assertInstanceof(suspender, WebAssembly.Suspender);
     x += param1 + 2 * param2 + 3 * Number(param3) + 4 * param4 + 5 * param5
       + 6 * param6 + 7 * Number(param7) + 8 * param8;
     y += paramf1 + paramf2 + paramd1 + paramd2;
   }
 
-  let param1 = { valueOf: () => { gc(); return 5; } };
-  let param4 = { valueOf: () => { gc(); return 8; } };
-  let param6 = { valueOf: () => { gc(); return 10; } };
-  let param8 = { valueOf: () => { gc(); return 12; } };
+  let param1 = { valueOf: () => { GC(); return 5; } };
+  let param4 = { valueOf: () => { GC(); return 8; } };
+  let param6 = { valueOf: () => { GC(); return 10; } };
+  let param8 = { valueOf: () => { GC(); return 12; } };
   let instance = builder.instantiate({ mod: { func: import_func } });
   let main = ToPromising(instance.exports.main);
-  assertEquals(undefined, main(param1, 6, 7n, param4, 9, param6,
-    1.5, 2.5, 3.6, 4.4, 11n, param8));
+  assertPromiseResult(
+      main(param1, 6, 7n, param4, 9, param6, 1.5, 2.5, 3.6, 4.4, 11n, param8),
+      v => assertEquals(undefined, v));
   assertEquals(360, x);
   assertEquals(13, y);
 })();
@@ -821,7 +827,7 @@ let kSig_f_riiliiiffddlifffdi = makeSig([kWasmExternRef, kWasmI32, kWasmI32,
   function import_func(suspender, param1, param2, param3, param4, param5,
       param6, paramf1, paramf2, paramd1, paramd2, param7, param8, paramf3,
       paramf4, paramf5, paramd3, param9) {
-    gc();
+    GC();
     assertInstanceof(suspender, WebAssembly.Suspender);
     x += param1 + 2 * param2 + 3 * Number(param3) + 4 * param4 + 5 * param5
       + 6 * param6 + 7 * Number(param7) + 8 * param8 + 9 * param9;
@@ -840,8 +846,10 @@ let kSig_f_riiliiiffddlifffdi = makeSig([kWasmExternRef, kWasmI32, kWasmI32,
   let param9 = { valueOf: () => { gc(); return 0; } };
   let instance = builder.instantiate({ mod: { func: import_func } });
   let main = ToPromising(instance.exports.main);
-  assertEquals(223, main(param1, 6, 7n, param4, 9, param6, 1.5, 2.5, paramd1,
-        4.5, 11n, param8, paramf3, 6.5, 7.5, 8.5, param9));
+  assertPromiseResult(
+      main(param1, 6, 7n, param4, 9, param6, 1.5, 2.5, paramd1,
+        4.5, 11n, param8, paramf3, 6.5, 7.5, 8.5, param9),
+      v => assertEquals(223, v));
   assertEquals(360, x);
 })();
 
@@ -876,7 +884,7 @@ let kSig_f_riiliiiffddlifffdi = makeSig([kWasmExternRef, kWasmI32, kWasmI32,
 
   let instance = builder.instantiate();
   let f1 = ToPromising(instance.exports.f1);
-  assertEquals(15, f1());
+  assertPromiseResult(f1(), v => assertEquals(15, v));
 })();
 
 (function testDeoptWithIncorrectNumberOfParams() {
@@ -930,5 +938,5 @@ let kSig_f_riiliiiffddlifffdi = makeSig([kWasmExternRef, kWasmI32, kWasmI32,
   let instance = new WebAssembly.Instance(module);
   let func0 = ToPromising(instance.exports.func0);
   let res = func0(1, 2, "3", "4", "5", "6", "7", "8", 9, 10, 11, 12, 13);
-  assertEquals("8", res);
+  assertPromiseResult(res, v => assertEquals("8", v));
 })();
