@@ -2,8 +2,6 @@
 # Use of this source code is governed by a BSD-style license that can be
 # found in the LICENSE file.
 
-import os
-
 from testrunner.local import testsuite
 from testrunner.objects import testcase
 from testrunner.outproc import mkgrokdump
@@ -22,9 +20,6 @@ class TestSuite(testsuite.TestSuite):
   def __init__(self, ctx, *args, **kwargs):
     super(TestSuite, self).__init__(ctx, *args, **kwargs)
 
-    v8_path = os.path.dirname(os.path.dirname(os.path.abspath(self.root)))
-    self.expected_path = os.path.join(v8_path, 'tools', 'v8heapconst.py')
-
   def _test_loader_class(self):
     return TestLoader
 
@@ -47,4 +42,4 @@ class TestCase(testcase.TestCase):
 
   @property
   def output_proc(self):
-    return mkgrokdump.OutProc(self.expected_outcomes, self.suite.expected_path)
+    return mkgrokdump.OutProc(self.expected_outcomes)
