@@ -273,9 +273,9 @@ TEST_F(AccessorTest, RedeclareAccessor) {
   v8::Local<v8::FunctionTemplate> templ = v8::FunctionTemplate::New(isolate());
 
   v8::Local<v8::ObjectTemplate> object_template = templ->InstanceTemplate();
-  object_template->SetAccessor(
-      NewString("foo"), nullptr, Setter, v8::Local<v8::Value>(),
-      v8::AccessControl::DEFAULT, v8::PropertyAttribute::DontDelete);
+  object_template->SetNativeDataProperty(NewString("foo"), nullptr, Setter,
+                                         v8::Local<v8::Value>(),
+                                         v8::PropertyAttribute::DontDelete);
 
   v8::Local<v8::Context> ctx =
       v8::Context::New(isolate(), nullptr, object_template);
@@ -304,8 +304,8 @@ static void CheckSideEffectFreeAccesses(v8::Isolate* isolate,
   // Check getter. Run enough number of times to ensure IC creates data handler.
   for (int i = 0; i < kIterationsCountForICProgression; i++) {
     v8::TryCatch try_catch(isolate);
-    CHECK(EvaluateGlobalForTesting(
-              isolate, func,
+    CHECK(EvaluateGlobal(
+              isolate, call_getter,
               v8::debug::EvaluateGlobalMode::kDisableBreaksAndThrowOnSideEffect,
               true)
               .IsEmpty());
@@ -320,8 +320,8 @@ static void CheckSideEffectFreeAccesses(v8::Isolate* isolate,
   // Check setter. Run enough number of times to ensure IC creates data handler.
   for (int i = 0; i < kIterationsCountForICProgression; i++) {
     v8::TryCatch try_catch(isolate);
-    CHECK(EvaluateGlobalForTesting(
-              isolate, func,
+    CHECK(EvaluateGlobal(
+              isolate, call_setter,
               v8::debug::EvaluateGlobalMode::kDisableBreaksAndThrowOnSideEffect,
               true)
               .IsEmpty());
@@ -359,16 +359,19 @@ TEST_F(AccessorTest, AccessorsWithSideEffects) {
   obj->SetAccessorProperty(
       NewString("get"),
       templ_with_sideffect->GetFunction(context()).ToLocalChecked(), {},
-      v8::PropertyAttribute::None, v8::AccessControl::DEFAULT);
+      v8::PropertyAttribute::None);
 
   obj->SetAccessorProperty(
       NewString("set"),
       templ_no_sideffect->GetFunction(context()).ToLocalChecked(),
       templ_with_sideffect->GetFunction(context()).ToLocalChecked(),
-      v8::PropertyAttribute::None, v8::AccessControl::DEFAULT);
+      v8::PropertyAttribute::None);
 
-  CheckSideEffectFreeAccesses(isolate(), NewString("obj.get"),
-                              NewString("obj.set = 123;"));
+  RunJS(
+      "function callGetter() { obj.get; }"
+      "function callSetter() { obj.set = 123; }");
+  CheckSideEffectFreeAccesses(isolate(), NewString("callGetter()"),
+                              NewString("callSetter()"));
 }
 
 TEST_F(AccessorTest, TemplateAccessorsWithSideEffects) {
@@ -396,8 +399,11 @@ TEST_F(AccessorTest, TemplateAccessorsWithSideEffects) {
   v8::Local<v8::Object> obj = templ->NewInstance(context()).ToLocalChecked();
   CHECK(context()->Global()->Set(context(), NewString("obj"), obj).FromJust());
 
-  CheckSideEffectFreeAccesses(isolate(), NewString("obj.get"),
-                              NewString("obj.set = 123;"));
+  RunJS(
+      "function callGetter() { obj.get; }"
+      "function callSetter() { obj.set = 123; }");
+  CheckSideEffectFreeAccesses(isolate(), NewString("callGetter()"),
+                              NewString("callSetter()"));
 }
 
 TEST_F(AccessorTest, NativeTemplateAccessorWithSideEffects) {
@@ -408,19 +414,22 @@ TEST_F(AccessorTest, NativeTemplateAccessorWithSideEffects) {
   i_isolate->debug()->SetDebugDelegate(&delegate);
 
   v8::Local<v8::ObjectTemplate> templ = v8::ObjectTemplate::New(isolate());
-  templ->SetAccessor(NewString("get"), Getter, nullptr, v8::Local<v8::Value>(),
-                     v8::AccessControl::DEFAULT, v8::PropertyAttribute::None,
-                     v8::SideEffectType::kHasSideEffect);
-  templ->SetAccessor(NewString("set"), Getter, Setter, v8::Local<v8::Value>(),
-                     v8::AccessControl::DEFAULT, v8::PropertyAttribute::None,
-                     v8::SideEffectType::kHasNoSideEffect,
-                     v8::SideEffectType::kHasSideEffect);
+  templ->SetNativeDataProperty(
+      NewString("get"), Getter, nullptr, v8::Local<v8::Value>(),
+      v8::PropertyAttribute::None, v8::SideEffectType::kHasSideEffect);
+  templ->SetNativeDataProperty(
+      NewString("set"), Getter, Setter, v8::Local<v8::Value>(),
+      v8::PropertyAttribute::None, v8::SideEffectType::kHasNoSideEffect,
+      v8::SideEffectType::kHasSideEffect);
 
   v8::Local<v8::Object> obj = templ->NewInstance(context()).ToLocalChecked();
   CHECK(context()->Global()->Set(context(), NewString("obj"), obj).FromJust());
 
-  CheckSideEffectFreeAccesses(isolate(), NewString("obj.get"),
-                              NewString("obj.set = 123;"));
+  RunJS(
+      "function callGetter() { obj.get; }"
+      "function callSetter() { obj.set = 123; }");
+  CheckSideEffectFreeAccesses(isolate(), NewString("callGetter()"),
+                              NewString("callSetter()"));
 }
 
 TEST_F(AccessorTest, NativeAccessorsWithSideEffects) {
@@ -431,23 +440,24 @@ TEST_F(AccessorTest, NativeAccessorsWithSideEffects) {
   CHECK(context()->Global()->Set(context(), NewString("obj"), obj).FromJust());
 
   // Install native data property with side effects.
-  obj->SetAccessor(context(), NewString("get"), Getter, nullptr,
-                   v8::MaybeLocal<v8::Value>(), v8::AccessControl::DEFAULT,
-                   v8::PropertyAttribute::None,
-                   v8::SideEffectType::kHasSideEffect)
+  obj->SetNativeDataProperty(context(), NewString("get"), Getter, nullptr, {},
+                             v8::PropertyAttribute::None,
+                             v8::SideEffectType::kHasSideEffect)
       .ToChecked();
-  obj->SetAccessor(context(), NewString("set"), Getter, Setter,
-                   v8::MaybeLocal<v8::Value>(), v8::AccessControl::DEFAULT,
-                   v8::PropertyAttribute::None,
-                   v8::SideEffectType::kHasNoSideEffect,
-                   v8::SideEffectType::kHasSideEffect)
+  obj->SetNativeDataProperty(context(), NewString("set"), Getter, Setter, {},
+                             v8::PropertyAttribute::None,
+                             v8::SideEffectType::kHasNoSideEffect,
+                             v8::SideEffectType::kHasSideEffect)
       .ToChecked();
 
-  CheckSideEffectFreeAccesses(isolate(), NewString("obj.get"),
-                              NewString("obj.set = 123;"));
+  RunJS(
+      "function callGetter() { obj.get; }"
+      "function callSetter() { obj.set = 123; }");
+  CheckSideEffectFreeAccesses(isolate(), NewString("callGetter()"),
+                              NewString("callSetter()"));
 }
 
-// Accessors can be allowlisted as side-effect-free via SetAccessor.
+// Accessors can be allowlisted as side-effect-free via SetNativeDataProperty.
 TEST_F(AccessorTest, AccessorSetHasNoSideEffect) {
   v8::HandleScope scope(isolate());
 
@@ -458,16 +468,15 @@ TEST_F(AccessorTest, AccessorSetHasNoSideEffect) {
   v8::Local<v8::ObjectTemplate> templ = v8::ObjectTemplate::New(isolate());
   v8::Local<v8::Object> obj = templ->NewInstance(context()).ToLocalChecked();
   CHECK(context()->Global()->Set(context(), NewString("obj"), obj).FromJust());
-  obj->SetAccessor(context(), NewString("foo"), Getter).ToChecked();
+  obj->SetNativeDataProperty(context(), NewString("foo"), Getter).ToChecked();
   CHECK(v8::debug::EvaluateGlobal(
             isolate(), NewString("obj.foo"),
             v8::debug::EvaluateGlobalMode::kDisableBreaksAndThrowOnSideEffect)
             .IsEmpty());
 
-  obj->SetAccessor(context(), NewString("foo"), Getter, nullptr,
-                   v8::MaybeLocal<v8::Value>(), v8::AccessControl::DEFAULT,
-                   v8::PropertyAttribute::None,
-                   v8::SideEffectType::kHasNoSideEffect)
+  obj->SetNativeDataProperty(context(), NewString("foo"), Getter, nullptr, {},
+                             v8::PropertyAttribute::None,
+                             v8::SideEffectType::kHasNoSideEffect)
       .ToChecked();
   v8::debug::EvaluateGlobal(
       isolate(), NewString("obj.foo"),
@@ -489,8 +498,9 @@ TEST_F(AccessorTest, AccessorSetHasNoSideEffect) {
   CHECK_EQ(0, set_accessor_call_count);
 }
 
-// Set accessors can be allowlisted as side-effect-free via SetAccessor.
-TEST_F(AccessorTest, SetAccessorSetSideEffectReceiverCheck1) {
+// Set accessors can be allowlisted as side-effect-free via
+// SetNativeDataProperty.
+TEST_F(AccessorTest, SetNativeDataPropertySetSideEffectReceiverCheck1) {
   v8::HandleScope scope(isolate());
 
   i::Isolate* i_isolate = reinterpret_cast<i::Isolate*>(isolate());
@@ -500,11 +510,10 @@ TEST_F(AccessorTest, SetAccessorSetSideEffectReceiverCheck1) {
   v8::Local<v8::ObjectTemplate> templ = v8::ObjectTemplate::New(isolate());
   v8::Local<v8::Object> obj = templ->NewInstance(context()).ToLocalChecked();
   CHECK(context()->Global()->Set(context(), NewString("obj"), obj).FromJust());
-  obj->SetAccessor(context(), NewString("foo"), Getter, Setter,
-                   v8::MaybeLocal<v8::Value>(), v8::AccessControl::DEFAULT,
-                   v8::PropertyAttribute::None,
-                   v8::SideEffectType::kHasNoSideEffect,
-                   v8::SideEffectType::kHasSideEffectToReceiver)
+  obj->SetNativeDataProperty(context(), NewString("foo"), Getter, Setter, {},
+                             v8::PropertyAttribute::None,
+                             v8::SideEffectType::kHasNoSideEffect,
+                             v8::SideEffectType::kHasSideEffectToReceiver)
       .ToChecked();
   CHECK(v8::debug::EvaluateGlobal(
             isolate(), NewString("obj.foo"),
@@ -524,7 +533,7 @@ TEST_F(AccessorTest, SetAccessorSetSideEffectReceiverCheck1) {
 static void ConstructCallback(const v8::FunctionCallbackInfo<v8::Value>& info) {
 }
 
-TEST_F(AccessorTest, SetAccessorSetSideEffectReceiverCheck2) {
+TEST_F(AccessorTest, SetNativeDataPropertySetSideEffectReceiverCheck2) {
   v8::HandleScope scope(isolate());
 
   i::Isolate* i_isolate = reinterpret_cast<i::Isolate*>(isolate());
@@ -535,10 +544,9 @@ TEST_F(AccessorTest, SetAccessorSetSideEffectReceiverCheck2) {
       isolate(), ConstructCallback, v8::Local<v8::Value>(),
       v8::Local<v8::Signature>(), 0, v8::ConstructorBehavior::kAllow,
       v8::SideEffectType::kHasNoSideEffect);
-  templ->InstanceTemplate()->SetAccessor(
+  templ->InstanceTemplate()->SetNativeDataProperty(
       NewString("bar"), Getter, Setter, v8::Local<v8::Value>(),
-      v8::AccessControl::DEFAULT, v8::PropertyAttribute::None,
-      v8::SideEffectType::kHasSideEffectToReceiver,
+      v8::PropertyAttribute::None, v8::SideEffectType::kHasSideEffectToReceiver,
       v8::SideEffectType::kHasSideEffectToReceiver);
   CHECK(context()
             ->Global()
@@ -638,7 +646,7 @@ TEST_F(AccessorTest, AccessorSetLazyDataPropertyHasNoSideEffect) {
                   .FromJust());
 }
 
-TEST_F(AccessorTest, ObjectTemplateSetAccessorHasNoSideEffect) {
+TEST_F(AccessorTest, ObjectTemplateSetNativeDataPropertyHasNoSideEffect) {
   v8::HandleScope scope(isolate());
 
   i::Isolate* i_isolate = reinterpret_cast<i::Isolate*>(isolate());
@@ -646,11 +654,10 @@ TEST_F(AccessorTest, ObjectTemplateSetAccessorHasNoSideEffect) {
   i_isolate->debug()->SetDebugDelegate(&delegate);
 
   v8::Local<v8::ObjectTemplate> templ = v8::ObjectTemplate::New(isolate());
-  templ->SetAccessor(NewString("foo"), StringGetter);
-  templ->SetAccessor(NewString("foo2"), StringGetter, nullptr,
-                     v8::Local<v8::Value>(), v8::AccessControl::DEFAULT,
-                     v8::PropertyAttribute::None,
-                     v8::SideEffectType::kHasNoSideEffect);
+  templ->SetNativeDataProperty(NewString("foo"), StringGetter);
+  templ->SetNativeDataProperty(
+      NewString("foo2"), StringGetter, nullptr, v8::Local<v8::Value>(),
+      v8::PropertyAttribute::None, v8::SideEffectType::kHasNoSideEffect);
   v8::Local<v8::Object> obj = templ->NewInstance(context()).ToLocalChecked();
   CHECK(context()->Global()->Set(context(), NewString("obj"), obj).FromJust());
 
@@ -686,10 +693,9 @@ TEST_F(AccessorTest, ObjectTemplateSetNativePropertyHasNoSideEffect) {
 
   v8::Local<v8::ObjectTemplate> templ = v8::ObjectTemplate::New(isolate());
   templ->SetNativeDataProperty(NewString("foo"), Getter);
-  templ->SetNativeDataProperty(
-      NewString("foo2"), Getter, nullptr, v8::Local<v8::Value>(),
-      v8::PropertyAttribute::None, v8::AccessControl::DEFAULT,
-      v8::SideEffectType::kHasNoSideEffect);
+  templ->SetNativeDataProperty(NewString("foo2"), Getter, nullptr, {},
+                               v8::PropertyAttribute::None,
+                               v8::SideEffectType::kHasNoSideEffect);
   v8::Local<v8::Object> obj = templ->NewInstance(context()).ToLocalChecked();
   CHECK(context()->Global()->Set(context(), NewString("obj"), obj).FromJust());
 
