@@ -40,11 +40,11 @@
 namespace v8 {
 namespace internal {
 
-using DisasmRiscv64Test = TestWithIsolate;
+using DisasmRiscvTest = TestWithIsolate;
 
 bool prev_instr_compact_branch = false;
 
-bool DisassembleAndCompare(byte* pc, const char* compare_string) {
+bool DisassembleAndCompare(uint8_t* pc, const char* compare_string) {
   disasm::NameConverter converter;
   disasm::Disassembler disasm(converter);
   base::EmbeddedVector<char, 128> disasm_buffer;
@@ -71,11 +71,11 @@ bool DisassembleAndCompare(byte* pc, const char* compare_string) {
 // Set up V8 to a state where we can at least run the assembler and
 // disassembler. Declare the variables and allocate the data structures used
 // in the rest of the macros.
-#define SET_UP()                                             \
-  HandleScope scope(isolate());                              \
-  byte* buffer = reinterpret_cast<byte*>(malloc(4 * 1024));  \
-  Assembler assm(AssemblerOptions{},                         \
-                 ExternalAssemblerBuffer(buffer, 4 * 1024)); \
+#define SET_UP()                                                  \
+  HandleScope scope(isolate());                                   \
+  uint8_t* buffer = reinterpret_cast<uint8_t*>(malloc(4 * 1024)); \
+  Assembler assm(AssemblerOptions{},                              \
+                 ExternalAssemblerBuffer(buffer, 4 * 1024));      \
   bool failure = false;
 
 // This macro assembles one instruction using the preallocated assembler and
@@ -85,7 +85,7 @@ bool DisassembleAndCompare(byte* pc, const char* compare_string) {
 #define COMPARE(asm_, compare_string)                                        \
   {                                                                          \
     int pc_offset = assm.pc_offset();                                        \
-    byte* progcounter = &buffer[pc_offset];                                  \
+    uint8_t* progcounter = &buffer[pc_offset];                               \
     assm.asm_;                                                               \
     if (!DisassembleAndCompare(progcounter, compare_string)) failure = true; \
   }
@@ -93,7 +93,7 @@ bool DisassembleAndCompare(byte* pc, const char* compare_string) {
 #define COMPARE_PC_REL(asm_, compare_string, offset)                           \
   {                                                                            \
     int pc_offset = assm.pc_offset();                                          \
-    byte* progcounter = &buffer[pc_offset];                                    \
+    uint8_t* progcounter = &buffer[pc_offset];                                 \
     char str_with_address[100];                                                \
     snprintf(str_with_address, sizeof(str_with_address), "%s -> %p",           \
              compare_string, static_cast<void*>(progcounter + (offset)));      \
@@ -108,7 +108,7 @@ bool DisassembleAndCompare(byte* pc, const char* compare_string) {
     FATAL("RISCV Disassembler tests failed.\n"); \
   }
 
-TEST_F(DisasmRiscv64Test, Arith) {
+TEST_F(DisasmRiscvTest, Arith) {
   SET_UP();
 
   // Arithmetic with immediate
@@ -136,11 +136,68 @@ TEST_F(DisasmRiscv64Test, Arith) {
   COMPARE(sra(a0, s3, s4), "4149d533       sra       a0, s3, s4");
   COMPARE(or_(a0, s3, s4), "0149e533       or        a0, s3, s4");
   COMPARE(and_(a0, s3, s4), "0149f533       and       a0, s3, s4");
-
   VERIFY_RUN();
 }
 
-TEST_F(DisasmRiscv64Test, LD_ST) {
+TEST_F(DisasmRiscvTest, RVB) {
+  SET_UP();
+
+#ifdef CAN_USE_ZBS_INSTRUCTIONS
+  COMPARE(sh1add(s6, t1, t5), "21e32b33       sh1add    s6, t1, t5");
+  COMPARE(sh2add(s7, t1, t2), "20734bb3       sh2add    s7, t1, t2");
+  COMPARE(sh3add(s4, t1, t3), "21c36a33       sh3add    s4, t1, t3");
+
+#ifdef V8_TARGET_ARCH_RISCV64
+  COMPARE(adduw(s6, t1, t5), "09e30b3b       add.uw    s6, t1, t5");
+  COMPARE(zextw(s6, t1), "08030b3b       zext.w    s6, t1");
+  COMPARE(sh1adduw(s6, t1, t5), "21e32b3b       sh1add.uw s6, t1, t5");
+  COMPARE(sh2adduw(s7, t1, t2), "20734bbb       sh2add.uw s7, t1, t2");
+  COMPARE(sh3adduw(s4, t1, t3), "21c36a3b       sh3add.uw s4, t1, t3");
+  COMPARE(slliuw(s6, t1, 10), "08a31b1b       slli.uw   s6, t1, 10");
+#endif
+#endif
+
+#ifdef CAN_USE_ZBB_INSTRUCTIONS
+  COMPARE(andn(a0, s3, s4), "4149f533       andn      a0, s3, s4");
+  COMPARE(orn(a0, s3, s4), "4149e533       orn       a0, s3, s4");
+  COMPARE(xnor(a0, s3, s4), "4149c533       xnor      a0, s3, s4");
+
+  COMPARE(clz(a0, s3), "60099513       clz       a0, s3");
+  COMPARE(ctz(a0, s3), "60199513       ctz       a0, s3");
+#ifdef V8_TARGET_ARCH_RISCV64
+  COMPARE(clzw(a1, s3), "6009959b       clzw      a1, s3");
+  COMPARE(ctzw(a0, s2), "6019151b       ctzw      a0, s2");
+#endif
+  COMPARE(max(a1, s3, t0), "0a59e5b3       max       a1, s3, t0");
+  COMPARE(maxu(a0, s2, a1), "0ab97533       maxu      a0, s2, a1");
+  COMPARE(min(a1, s3, fp), "0a89c5b3       min       a1, s3, fp");
+  COMPARE(minu(a0, s2, sp), "0a295533       minu      a0, s2, sp");
+
+  COMPARE(sextb(a0, s2), "60491513       sext.b    a0, s2");
+  COMPARE(sexth(a1, s3), "60599593       sext.h    a1, s3");
+#ifdef V8_TARGET_ARCH_RISCV64
+  COMPARE(zexth(a0, s2), "0809453b       zext.h    a0, s2");
+  COMPARE(rev8(a0, s2), "6b895513       rev8      a0, s2");
+#else
+  COMPARE(zexth(a0, s2), "08094533       zext.h    a0, s2");
+  COMPARE(rev8(a0, s2), "69895513       rev8      a0, s2");
+#endif
+#endif
+
+#ifdef CAN_USE_ZBS_INSTRUCTIONS
+  COMPARE(bclr(a0, s2, s1), "48991533       bclr      a0, s2, s1");
+  COMPARE(bclri(a0, s1, 3), "48349513       bclri     a0, s1, 3");
+  COMPARE(bext(a0, s2, s1), "48995533       bext      a0, s2, s1");
+  COMPARE(bexti(a0, s1, 3), "4834d513       bexti     a0, s1, 3");
+  COMPARE(binv(a0, s2, s1), "68991533       binv      a0, s2, s1");
+  COMPARE(binvi(a0, s1, 3), "68349513       binvi     a0, s1, 3");
+  COMPARE(bset(a0, s2, s1), "28991533       bset      a0, s2, s1");
+  COMPARE(bseti(a0, s1, 3), "28349513       bseti     a0, s1, 3");
+#endif
+  VERIFY_RUN();
+}
+
+TEST_F(DisasmRiscvTest, LD_ST) {
   SET_UP();
   // Loads
   COMPARE(lb(t0, a0, 0), "00050283       lb        t0, 0(a0)");
@@ -157,7 +214,7 @@ TEST_F(DisasmRiscv64Test, LD_ST) {
   VERIFY_RUN();
 }
 
-TEST_F(DisasmRiscv64Test, MISC) {
+TEST_F(DisasmRiscvTest, MISC) {
   SET_UP();
 
   COMPARE(lui(sp, 0x64), "00064137       lui       sp, 0x64");
@@ -193,7 +250,7 @@ TEST_F(DisasmRiscv64Test, MISC) {
   VERIFY_RUN();
 }
 
-TEST_F(DisasmRiscv64Test, CSR) {
+TEST_F(DisasmRiscvTest, CSR) {
   SET_UP();
 
   COMPARE(csrrw(a0, csr_fflags, t3), "001e1573       fsflags   a0, t3");
@@ -209,7 +266,7 @@ TEST_F(DisasmRiscv64Test, CSR) {
   VERIFY_RUN();
 }
 #ifdef V8_TARGET_ARCH_RISCV64
-TEST_F(DisasmRiscv64Test, RV64I) {
+TEST_F(DisasmRiscvTest, RV64I) {
   SET_UP();
 
   COMPARE(lwu(a0, s3, -268), "ef49e503       lwu       a0, -268(s3)");
@@ -228,7 +285,7 @@ TEST_F(DisasmRiscv64Test, RV64I) {
   VERIFY_RUN();
 }
 #endif
-TEST_F(DisasmRiscv64Test, RV32M) {
+TEST_F(DisasmRiscvTest, RV32M) {
   SET_UP();
 
   COMPARE(mul(a0, s3, t4), "03d98533       mul       a0, s3, t4");
@@ -243,7 +300,7 @@ TEST_F(DisasmRiscv64Test, RV32M) {
   VERIFY_RUN();
 }
 #ifdef V8_TARGET_ARCH_RISCV64
-TEST_F(DisasmRiscv64Test, RV64M) {
+TEST_F(DisasmRiscvTest, RV64M) {
   SET_UP();
 
   COMPARE(mulw(a0, s3, s4), "0349853b       mulw      a0, s3, s4");
@@ -255,7 +312,7 @@ TEST_F(DisasmRiscv64Test, RV64M) {
   VERIFY_RUN();
 }
 #endif
-TEST_F(DisasmRiscv64Test, RV32A) {
+TEST_F(DisasmRiscvTest, RV32A) {
   SET_UP();
   // RV32A Standard Extension
   COMPARE(lr_w(true, false, a0, s3), "1409a52f       lr.w.aq    a0, (s3)");
@@ -282,7 +339,7 @@ TEST_F(DisasmRiscv64Test, RV32A) {
   VERIFY_RUN();
 }
 #ifdef V8_TARGET_ARCH_RISCV64
-TEST_F(DisasmRiscv64Test, RV64A) {
+TEST_F(DisasmRiscvTest, RV64A) {
   SET_UP();
 
   COMPARE(lr_d(true, true, a0, s3), "1609b52f       lr.d.aqrl a0, (s3)");
@@ -309,7 +366,7 @@ TEST_F(DisasmRiscv64Test, RV64A) {
   VERIFY_RUN();
 }
 #endif
-TEST_F(DisasmRiscv64Test, RV32F) {
+TEST_F(DisasmRiscvTest, RV32F) {
   SET_UP();
   // RV32F Standard Extension
   COMPARE(flw(fa0, s3, -268), "ef49a507       flw       fa0, -268(s3)");
@@ -345,7 +402,7 @@ TEST_F(DisasmRiscv64Test, RV32F) {
   VERIFY_RUN();
 }
 #ifdef V8_TARGET_ARCH_RISCV64
-TEST_F(DisasmRiscv64Test, RV64F) {
+TEST_F(DisasmRiscvTest, RV64F) {
   SET_UP();
   // RV64F Standard Extension (in addition to RV32F)
   COMPARE(fcvt_l_s(a0, ft8, RNE), "c02e0553       fcvt.l.s  [RNE] a0, ft8");
@@ -355,7 +412,7 @@ TEST_F(DisasmRiscv64Test, RV64F) {
   VERIFY_RUN();
 }
 #endif
-TEST_F(DisasmRiscv64Test, RV32D) {
+TEST_F(DisasmRiscvTest, RV32D) {
   SET_UP();
   // RV32D Standard Extension
   COMPARE(fld(ft0, s3, -268), "ef49b007       fld       ft0, -268(s3)");
@@ -392,7 +449,7 @@ TEST_F(DisasmRiscv64Test, RV32D) {
   VERIFY_RUN();
 }
 #ifdef V8_TARGET_ARCH_RISCV64
-TEST_F(DisasmRiscv64Test, RV64D) {
+TEST_F(DisasmRiscvTest, RV64D) {
   SET_UP();
   // RV64D Standard Extension (in addition to RV32D)
   COMPARE(fcvt_l_d(a0, ft8, RMM), "c22e4553       fcvt.l.d  [RMM] a0, ft8");
@@ -405,7 +462,7 @@ TEST_F(DisasmRiscv64Test, RV64D) {
 }
 #endif
 
-TEST_F(DisasmRiscv64Test, PSEUDO) {
+TEST_F(DisasmRiscvTest, PSEUDO) {
   SET_UP();
   // pseodu instructions according to rISCV assembly programmer's handbook
   COMPARE(nop(), "00000013       nop");
@@ -468,7 +525,7 @@ TEST_F(DisasmRiscv64Test, PSEUDO) {
   VERIFY_RUN();
 }
 #ifdef V8_TARGET_ARCH_RISCV64
-TEST_F(DisasmRiscv64Test, RV64C) {
+TEST_F(DisasmRiscvTest, RV64C) {
   i::v8_flags.riscv_c_extension = true;
   SET_UP();
 
@@ -520,7 +577,7 @@ TEST_F(DisasmRiscv64Test, RV64C) {
 }
 #endif
 /*
-TEST_F(DisasmRiscv64Test,  Previleged) {
+TEST_F(DisasmRiscvTest,  Previleged) {
   SET_UP();
   // Privileged
   COMPARE(uret(), "");
@@ -532,15 +589,15 @@ TEST_F(DisasmRiscv64Test,  Previleged) {
 }
 */
 
-TEST_F(DisasmRiscv64Test, RVV) {
+TEST_F(DisasmRiscvTest, RVV) {
   if (!CpuFeatures::IsSupported(RISCV_SIMD)) return;
   SET_UP();
   COMPARE(VU.set(kScratchReg, E64, m1),
           "018079d7       vsetvli   s3, zero_reg, E64, m1");
-  COMPARE(vl(v2, a0, 0, VSew::E8), "02050107       vle8.v       v2, (a0)");
-  COMPARE(vl(v2, a0, 0, VSew::E8), "02050107       vle8.v       v2, (a0)");
-  COMPARE(vl(v2, a0, 0, VSew::E16), "02055107       vle16.v       v2, (a0)");
-  COMPARE(vl(v2, a0, 0, VSew::E32), "02056107       vle32.v       v2, (a0)");
+  COMPARE(vl(v2, a0, 0, VSew::E8), "02050107       vle8.v    v2, (a0)");
+  COMPARE(vl(v2, a0, 0, VSew::E16), "02055107       vle16.v    v2, (a0)");
+  COMPARE(vl(v2, a0, 0, VSew::E32), "02056107       vle32.v    v2, (a0)");
+  COMPARE(vl(v2, a0, 0, VSew::E64), "02057107       vle64.v    v2, (a0)");
 
   COMPARE(vadd_vv(v0, v0, v1), "02008057       vadd.vv   v0, v0, v1");
   COMPARE(vadd_vx(v0, v1, t0), "0212c057       vadd.vx   v0, v1, t0");
@@ -646,9 +703,9 @@ TEST_F(DisasmRiscv64Test, RVV) {
   COMPARE(vfwsub_wf(v17, v28, fa5), "dbc7d8d7       vfwsub.wf v17, v28, fa5");
 
   // Vector Widening Floating-Point Reduction Instructions
-  COMPARE(vfwredusum_vv(v17, v14, v28),
+  COMPARE(vfwredusum_vs(v17, v14, v28),
           "c6ee18d7       vfwredusum.vs v17, v14, v28");
-  COMPARE(vfwredosum_vv(v17, v14, v28),
+  COMPARE(vfwredosum_vs(v17, v14, v28),
           "ceee18d7       vfwredosum.vs v17, v14, v28");
 
   // Vector Widening Floating-Point Multiply
