@@ -407,7 +407,7 @@ TEST_F(DisasmX64Test, DisasmX64CheckOutput) {
   COMPARE("4883448d0c0c         REX.W addq [rbp+rcx*4+0xc],0xc",
           addq(Operand(rbp, rcx, times_4, 12), Immediate(12)));
 
-  COMPARE("400fc8               bswapl rax", bswapl(rax));
+  COMPARE("0fc8                 bswapl rax", bswapl(rax));
   COMPARE("480fcf               REX.W bswapq rdi", bswapq(rdi));
   COMPARE("410fbdc7             bsrl rax,r15", bsrl(rax, r15));
   COMPARE("440fbd0ccd0f670100   bsrl r9,[rcx*8+0x1670f]",
@@ -636,7 +636,7 @@ TEST_F(DisasmX64Test, DisasmX64CheckOutput) {
   COMPARE("4883fb0c             REX.W cmpq rbx,0xc", cmpq(rbx, Immediate(12)));
   COMPARE("4883bc8a102700000c   REX.W cmpq [rdx+rcx*4+0x2710],0xc",
           cmpq(Operand(rdx, rcx, times_4, 10000), Immediate(12)));
-  COMPARE("80f864               cmpb al,0x64", cmpb(rax, Immediate(100)));
+  COMPARE("3c64                 cmpb al,0x64", cmpb(rax, Immediate(100)));
 
   COMPARE("4881cb39300000       REX.W orq rbx,0x3039",
           orq(rbx, Immediate(12345)));
