@@ -34,12 +34,12 @@ static Handle<Object> GetGlobalProperty(const char* name) {
       .ToHandleChecked();
 }
 
-static void SetGlobalProperty(const char* name, Object value) {
+static void SetGlobalProperty(const char* name, Tagged<Object> value) {
   Isolate* isolate = reinterpret_cast<i::Isolate*>(v8::Isolate::GetCurrent());
   Handle<Object> object(value, isolate);
   Handle<String> internalized_name =
       isolate->factory()->InternalizeUtf8String(name);
-  Handle<JSObject> global(isolate->context().global_object(), isolate);
+  Handle<JSObject> global(isolate->context()->global_object(), isolate);
   Runtime::SetObjectProperty(isolate, global, internalized_name, object,
                              StoreOrigin::kMaybeKeyed, Just(kDontThrow))
       .Check();
@@ -50,11 +50,13 @@ static Handle<JSFunction> Compile(const char* source) {
   Handle<String> source_code = isolate->factory()
                                    ->NewStringFromUtf8(base::CStrVector(source))
                                    .ToHandleChecked();
+  ScriptCompiler::CompilationDetails compilation_details;
   Handle<SharedFunctionInfo> shared =
       Compiler::GetSharedFunctionInfoForScript(
           isolate, source_code, ScriptDetails(),
           v8::ScriptCompiler::kNoCompileOptions,
-          ScriptCompiler::kNoCacheNoReason, NOT_NATIVES_CODE)
+          ScriptCompiler::kNoCacheNoReason, NOT_NATIVES_CODE,
+          &compilation_details)
           .ToHandleChecked();
   return Factory::JSFunctionBuilder{isolate, shared, isolate->native_context()}
       .Build();
@@ -68,11 +70,11 @@ static double Inc(Isolate* isolate, int x) {
   Handle<JSFunction> fun = Compile(buffer.begin());
   if (fun.is_null()) return -1;
 
-  Handle<JSObject> global(isolate->context().global_object(), isolate);
+  Handle<JSObject> global(isolate->context()->global_object(), isolate);
   Execution::CallScript(isolate, fun, global,
                         isolate->factory()->empty_fixed_array())
       .Check();
-  return GetGlobalProperty("result")->Number();
+  return Object::Number(*GetGlobalProperty("result"));
 }
 
 TEST_F(CompilerTest, Inc) {
@@ -86,11 +88,11 @@ static double Add(Isolate* isolate, int x, int y) {
 
   SetGlobalProperty("x", Smi::FromInt(x));
   SetGlobalProperty("y", Smi::FromInt(y));
-  Handle<JSObject> global(isolate->context().global_object(), isolate);
+  Handle<JSObject> global(isolate->context()->global_object(), isolate);
   Execution::CallScript(isolate, fun, global,
                         isolate->factory()->empty_fixed_array())
       .Check();
-  return GetGlobalProperty("result")->Number();
+  return Object::Number(*GetGlobalProperty("result"));
 }
 
 TEST_F(CompilerTest, Add) {
@@ -103,11 +105,11 @@ static double Abs(Isolate* isolate, int x) {
   if (fun.is_null()) return -1;
 
   SetGlobalProperty("x", Smi::FromInt(x));
-  Handle<JSObject> global(isolate->context().global_object(), isolate);
+  Handle<JSObject> global(isolate->context()->global_object(), isolate);
   Execution::CallScript(isolate, fun, global,
                         isolate->factory()->empty_fixed_array())
       .Check();
-  return GetGlobalProperty("result")->Number();
+  return Object::Number(*GetGlobalProperty("result"));
 }
 
 TEST_F(CompilerTest, Abs) {
@@ -121,11 +123,11 @@ static double Sum(Isolate* isolate, int n) {
   if (fun.is_null()) return -1;
 
   SetGlobalProperty("n", Smi::FromInt(n));
-  Handle<JSObject> global(isolate->context().global_object(), isolate);
+  Handle<JSObject> global(isolate->context()->global_object(), isolate);
   Execution::CallScript(isolate, fun, global,
                         isolate->factory()->empty_fixed_array())
       .Check();
-  return GetGlobalProperty("result")->Number();
+  return Object::Number(*GetGlobalProperty("result"));
 }
 
 TEST_F(CompilerTest, Sum) {
@@ -145,7 +147,7 @@ TEST_F(CompilerPrintTest, Print) {
   const char* source = "for (n = 0; n < 100; ++n) print(n, 1, 2);";
   Handle<JSFunction> fun = Compile(source);
   if (fun.is_null()) return;
-  Handle<JSObject> global(i_isolate()->context().global_object(), i_isolate());
+  Handle<JSObject> global(i_isolate()->context()->global_object(), i_isolate());
   Execution::CallScript(i_isolate(), fun, global,
                         i_isolate()->factory()->empty_fixed_array())
       .Check();
@@ -176,11 +178,11 @@ TEST_F(CompilerTest, Stuff) {
 
   Handle<JSFunction> fun = Compile(source);
   EXPECT_TRUE(!fun.is_null());
-  Handle<JSObject> global(i_isolate()->context().global_object(), i_isolate());
+  Handle<JSObject> global(i_isolate()->context()->global_object(), i_isolate());
   Execution::CallScript(i_isolate(), fun, global,
                         i_isolate()->factory()->empty_fixed_array())
       .Check();
-  EXPECT_EQ(511.0, GetGlobalProperty("r")->Number());
+  EXPECT_EQ(511.0, Object::Number(*GetGlobalProperty("r")));
 }
 
 TEST_F(CompilerTest, UncaughtThrow) {
@@ -190,11 +192,11 @@ TEST_F(CompilerTest, UncaughtThrow) {
   Handle<JSFunction> fun = Compile(source);
   EXPECT_TRUE(!fun.is_null());
   Isolate* isolate = fun->GetIsolate();
-  Handle<JSObject> global(isolate->context().global_object(), isolate);
+  Handle<JSObject> global(isolate->context()->global_object(), isolate);
   EXPECT_TRUE(Execution::CallScript(isolate, fun, global,
                                     isolate->factory()->empty_fixed_array())
                   .is_null());
-  EXPECT_EQ(42.0, isolate->pending_exception().Number());
+  EXPECT_EQ(42.0, Object::Number(isolate->exception()));
 }
 
 using CompilerC2JSFramesTest = WithPrintExtensionMixin<v8::TestWithIsolate>;
@@ -221,7 +223,7 @@ TEST_F(CompilerC2JSFramesTest, C2JSFrames) {
   Isolate* isolate = fun0->GetIsolate();
 
   // Run the generated code to populate the global object with 'foo'.
-  Handle<JSObject> global(isolate->context().global_object(), isolate);
+  Handle<JSObject> global(isolate->context()->global_object(), isolate);
   Execution::CallScript(isolate, fun0, global,
                         isolate->factory()->empty_fixed_array())
       .Check();
@@ -229,7 +231,7 @@ TEST_F(CompilerC2JSFramesTest, C2JSFrames) {
   Handle<Object> fun1 =
       JSReceiver::GetProperty(isolate, isolate->global_object(), "foo")
           .ToHandleChecked();
-  EXPECT_TRUE(fun1->IsJSFunction());
+  EXPECT_TRUE(IsJSFunction(*fun1));
 
   Handle<Object> argv[] = {
       isolate->factory()->InternalizeString(base::StaticCharVector("hello"))};
@@ -252,7 +254,7 @@ TEST_F(CompilerTest, Regression236) {
 
 TEST_F(CompilerTest, GetScriptLineNumber) {
   v8::HandleScope scope(isolate());
-  v8::ScriptOrigin origin = v8::ScriptOrigin(isolate(), NewString("test"));
+  v8::ScriptOrigin origin = v8::ScriptOrigin(NewString("test"));
   const char function_f[] = "function f() {}";
   const int max_rows = 1000;
   const int buffer_size = max_rows + sizeof(function_f);
@@ -299,21 +301,21 @@ TEST_F(CompilerTest, FeedbackVectorPreservedAcrossRecompiles) {
   FeedbackSlot slot_for_a(0);
   MaybeObject object = feedback_vector->Get(slot_for_a);
   {
-    HeapObject heap_object;
-    EXPECT_TRUE(object->GetHeapObjectIfWeak(&heap_object));
-    EXPECT_TRUE(heap_object.IsJSFunction());
+    Tagged<HeapObject> heap_object;
+    EXPECT_TRUE(object.GetHeapObjectIfWeak(&heap_object));
+    EXPECT_TRUE(IsJSFunction(heap_object));
   }
 
   RunJS("%OptimizeFunctionOnNextCall(f); f(fun1);");
 
   // Verify that the feedback is still "gathered" despite a recompilation
   // of the full code.
-  EXPECT_TRUE(f->HasAttachedOptimizedCode());
-  object = f->feedback_vector().Get(slot_for_a);
+  EXPECT_TRUE(f->HasAttachedOptimizedCode(i_isolate()));
+  object = f->feedback_vector()->Get(slot_for_a);
   {
-    HeapObject heap_object;
-    EXPECT_TRUE(object->GetHeapObjectIfWeak(&heap_object));
-    EXPECT_TRUE(heap_object.IsJSFunction());
+    Tagged<HeapObject> heap_object;
+    EXPECT_TRUE(object.GetHeapObjectIfWeak(&heap_object));
+    EXPECT_TRUE(IsJSFunction(heap_object));
   }
 }
 
@@ -346,12 +348,12 @@ TEST_F(CompilerTest, FeedbackVectorUnaffectedByScopeChanges) {
 
   // If we are compiling lazily then it should not be compiled, and so no
   // feedback vector allocated yet.
-  EXPECT_TRUE(!f->shared().is_compiled());
+  EXPECT_TRUE(!f->shared()->is_compiled());
 
   RunJS("morphing_call();");
 
   // Now a feedback vector / closure feedback cell array is allocated.
-  EXPECT_TRUE(f->shared().is_compiled());
+  EXPECT_TRUE(f->shared()->is_compiled());
   EXPECT_TRUE(f->has_feedback_vector() || f->has_closure_feedback_cell_array());
 }
 
@@ -390,11 +392,11 @@ TEST_F(CompilerTest, OptimizedCodeSharing1) {
                 ->Global()
                 ->Get(context(), NewString("closure2"))
                 .ToLocalChecked())));
-    EXPECT_TRUE(fun1->HasAttachedOptimizedCode() ||
+    EXPECT_TRUE(fun1->HasAttachedOptimizedCode(i_isolate()) ||
                 !i_isolate()->use_optimizer());
-    EXPECT_TRUE(fun2->HasAttachedOptimizedCode() ||
+    EXPECT_TRUE(fun2->HasAttachedOptimizedCode(i_isolate()) ||
                 !i_isolate()->use_optimizer());
-    EXPECT_EQ(fun1->code(), fun2->code());
+    EXPECT_EQ(fun1->code(i_isolate()), fun2->code(i_isolate()));
   }
 }
 
@@ -605,7 +607,7 @@ TEST_F(CompilerTest, CompileFunctionQuirks) {
 
 TEST_F(CompilerTest, CompileFunctionScriptOrigin) {
   v8::HandleScope scope(isolate());
-  v8::ScriptOrigin origin(isolate(), NewString("test"), 22, 41);
+  v8::ScriptOrigin origin(NewString("test"), 22, 41);
   v8::ScriptCompiler::Source script_source(NewString("throw new Error()"),
                                            origin);
   v8::Local<v8::Function> fun =
@@ -613,11 +615,11 @@ TEST_F(CompilerTest, CompileFunctionScriptOrigin) {
           .ToLocalChecked();
   EXPECT_TRUE(!fun.IsEmpty());
   auto fun_i = i::Handle<i::JSFunction>::cast(Utils::OpenHandle(*fun));
-  EXPECT_TRUE(fun_i->shared().IsSharedFunctionInfo());
-  EXPECT_TRUE(
-      Utils::ToLocal(i::handle(i::Script::cast(fun_i->shared().script()).name(),
-                               i_isolate()))
-          ->StrictEquals(NewString("test")));
+  EXPECT_TRUE(IsSharedFunctionInfo(fun_i->shared()));
+  EXPECT_TRUE(Utils::ToLocal(
+                  i::handle(i::Script::cast(fun_i->shared()->script())->name(),
+                            i_isolate()))
+                  ->StrictEquals(NewString("test")));
   v8::TryCatch try_catch(isolate());
   isolate()->SetCaptureStackTraceForUncaughtExceptions(true);
   EXPECT_TRUE(fun->Call(context(), context()->Global(), 0, nullptr).IsEmpty());
@@ -649,7 +651,7 @@ TEST_F(CompilerTest, CompileFunctionFunctionToString) {
 
     // Regression test for v8:6190
     {
-      v8::ScriptOrigin origin(isolate(), NewString("test"), 22, 41);
+      v8::ScriptOrigin origin(NewString("test"), 22, 41);
       v8::ScriptCompiler::Source script_source(NewString("return event"),
                                                origin);
 
@@ -675,7 +677,7 @@ TEST_F(CompilerTest, CompileFunctionFunctionToString) {
 
     // With no parameters:
     {
-      v8::ScriptOrigin origin(isolate(), NewString("test"), 17, 31);
+      v8::ScriptOrigin origin(NewString("test"), 17, 31);
       v8::ScriptCompiler::Source script_source(NewString("return 0"), origin);
 
       v8::TryCatch try_catch(isolate());
@@ -698,7 +700,7 @@ TEST_F(CompilerTest, CompileFunctionFunctionToString) {
 
     // With a name:
     {
-      v8::ScriptOrigin origin(isolate(), NewString("test"), 17, 31);
+      v8::ScriptOrigin origin(NewString("test"), 17, 31);
       v8::ScriptCompiler::Source script_source(NewString("return 0"), origin);
 
       v8::TryCatch try_catch(isolate());
@@ -738,13 +740,13 @@ TEST_F(CompilerTest, InvocationCount) {
       "%EnsureFeedbackVectorForFunction(foo);"
       "foo();");
   Handle<JSFunction> foo = Handle<JSFunction>::cast(GetGlobalProperty("foo"));
-  EXPECT_EQ(1, foo->feedback_vector().invocation_count());
+  EXPECT_EQ(1, foo->feedback_vector()->invocation_count());
   RunJS("foo()");
-  EXPECT_EQ(2, foo->feedback_vector().invocation_count());
+  EXPECT_EQ(2, foo->feedback_vector()->invocation_count());
   RunJS("bar()");
-  EXPECT_EQ(2, foo->feedback_vector().invocation_count());
+  EXPECT_EQ(2, foo->feedback_vector()->invocation_count());
   RunJS("foo(); foo()");
-  EXPECT_EQ(4, foo->feedback_vector().invocation_count());
+  EXPECT_EQ(4, foo->feedback_vector()->invocation_count());
 }
 
 TEST_F(CompilerTest, ShallowEagerCompilation) {
@@ -915,11 +917,11 @@ TEST_F(CompilerTest, ProfilerEnabledDuringBackgroundCompile) {
   v8::Local<v8::Script> script =
       v8::ScriptCompiler::Compile(isolate()->GetCurrentContext(),
                                   &streamed_source, NewString(source),
-                                  v8::ScriptOrigin(isolate(), NewString("foo")))
+                                  v8::ScriptOrigin(NewString("foo")))
           .ToLocalChecked();
 
   i::Handle<i::Object> obj = Utils::OpenHandle(*script);
-  EXPECT_TRUE(i::JSFunction::cast(*obj).shared().AreSourcePositionsAvailable(
+  EXPECT_TRUE(i::JSFunction::cast(*obj)->shared()->AreSourcePositionsAvailable(
       i_isolate()));
 
   cpu_profiler->StopProfiling(profile);
