@@ -7,10 +7,10 @@
 int main(int argc, char** argv) {
   // Don't catch SEH exceptions and continue as the following tests might hang
   // in an broken environment on windows.
-  testing::GTEST_FLAG(catch_exceptions) = false;
+  GTEST_FLAG_SET(catch_exceptions, false);
 
   // Most unit-tests are multi-threaded, so enable thread-safe death-tests.
-  testing::FLAGS_gtest_death_test_style = "threadsafe";
+  GTEST_FLAG_SET(death_test_style, "threadsafe");
 
   testing::InitGoogleMock(&argc, argv);
   return RUN_ALL_TESTS();
