@@ -5,12 +5,12 @@
 #include <cstring>
 
 #include "include/cppgc/allocation.h"
+#include "include/cppgc/common.h"
 #include "include/cppgc/cross-thread-persistent.h"
 #include "include/cppgc/custom-space.h"
 #include "include/cppgc/garbage-collected.h"
 #include "include/cppgc/name-provider.h"
 #include "include/cppgc/persistent.h"
-#include "include/cppgc/platform.h"
 #include "include/v8-cppgc.h"
 #include "include/v8-profiler.h"
 #include "src/api/api-inl.h"
@@ -80,10 +80,19 @@ class UnifiedHeapSnapshotTest : public UnifiedHeapTest {
   explicit UnifiedHeapSnapshotTest(
       std::vector<std::unique_ptr<cppgc::CustomSpaceBase>> custom_spaces)
       : UnifiedHeapTest(std::move(custom_spaces)) {}
-  const v8::HeapSnapshot* TakeHeapSnapshot() {
+  const v8::HeapSnapshot* TakeHeapSnapshot(
+      cppgc::EmbedderStackState stack_state =
+          cppgc::EmbedderStackState::kMayContainHeapPointers) {
     v8::HeapProfiler* heap_profiler = v8_isolate()->GetHeapProfiler();
-    return heap_profiler->TakeHeapSnapshot(nullptr, nullptr,
-                                           false /* hide internals */);
+
+    v8::HeapProfiler::HeapSnapshotOptions options;
+    options.control = nullptr;
+    options.global_object_name_resolver = nullptr;
+    options.snapshot_mode =
+        v8::HeapProfiler::HeapSnapshotMode::kExposeInternals;
+    options.numerics_mode = v8::HeapProfiler::NumericsMode::kHideNumericValues;
+    options.stack_state = stack_state;
+    return heap_profiler->TakeHeapSnapshot(options);
   }
 };
 
@@ -178,9 +187,11 @@ class GCed final : public BaseWithoutName, public cppgc::NameProvider {
 // static
 constexpr const char GCed::kExpectedName[];
 
-constexpr const char kExpectedCppRootsName[] = "C++ roots";
-constexpr const char kExpectedCppCrossThreadRootsName[] =
-    "C++ cross-thread roots";
+static constexpr const char kExpectedCppRootsName[] = "C++ Persistent roots";
+static constexpr const char kExpectedCppCrossThreadRootsName[] =
+    "C++ CrossThreadPersistent roots";
+static constexpr const char kExpectedCppStackRootsName[] =
+    "C++ native stack roots";
 
 template <typename T>
 constexpr const char* GetExpectedName() {
@@ -251,34 +262,29 @@ TEST_F(UnifiedHeapWithCustomSpaceSnapshotTest, ConsistentIdAfterCompaction) {
       cppgc::MakeGarbageCollected<CompactableHolder>(allocation_handle(),
                                                      allocation_handle());
 
-  CompactableGCed* original_pointer = gced->object;
-
   // Release the persistent reference to the other object.
   trash.Release();
 
+  void* original_pointer = gced->object;
+
   // This first snapshot should not trigger compaction of the cppgc heap because
   // the heap is still very small.
-  const v8::HeapSnapshot* snapshot1 = TakeHeapSnapshot();
+  const v8::HeapSnapshot* snapshot1 =
+      TakeHeapSnapshot(cppgc::EmbedderStackState::kNoHeapPointers);
   EXPECT_TRUE(IsValidSnapshot(snapshot1));
   EXPECT_EQ(original_pointer, gced->object);
 
-  // PerformGarbageCollection, called during TakeHeapSnapshot above, reset the
-  // heap's embedder_stack_state_, so we need to do this again so it has an
-  // effect on the next GC.
-  DisableConservativeStackScanningScopeForTesting no_css_again(
-      isolate()->heap());
-
   // Manually run a GC with compaction. The GCed object should move.
   CppHeap::From(isolate()->heap()->cpp_heap())
       ->compactor()
       .EnableForNextGCForTesting();
-  isolate()->heap()->CollectAllGarbage(i::Heap::kReduceMemoryFootprintMask,
-                                       i::GarbageCollectionReason::kTesting);
+  i::InvokeMajorGC(isolate(), i::GCFlag::kReduceMemoryFootprint);
   EXPECT_NE(original_pointer, gced->object);
 
   // In the second heap snapshot, the moved object should still have the same
   // ID.
-  const v8::HeapSnapshot* snapshot2 = TakeHeapSnapshot();
+  const v8::HeapSnapshot* snapshot2 =
+      TakeHeapSnapshot(cppgc::EmbedderStackState::kNoHeapPointers);
   EXPECT_TRUE(IsValidSnapshot(snapshot2));
   std::vector<SnapshotObjectId> ids1 =
       GetIds(*snapshot1, GetExpectedName<CompactableGCed>());
@@ -302,6 +308,16 @@ TEST_F(UnifiedHeapSnapshotTest, RetainedByCppCrossThreadRoot) {
       *snapshot, {kExpectedCppCrossThreadRootsName, GetExpectedName<GCed>()}));
 }
 
+TEST_F(UnifiedHeapSnapshotTest, RetainedByStackRoots) {
+  auto* volatile gced = cppgc::MakeGarbageCollected<GCed>(allocation_handle());
+  const v8::HeapSnapshot* snapshot =
+      TakeHeapSnapshot(cppgc::EmbedderStackState::kMayContainHeapPointers);
+  EXPECT_TRUE(IsValidSnapshot(snapshot));
+  EXPECT_TRUE(ContainsRetainingPath(
+      *snapshot, {kExpectedCppStackRootsName, GetExpectedName<GCed>()}));
+  EXPECT_STREQ(gced->GetHumanReadableName(), GetExpectedName<GCed>());
+}
+
 TEST_F(UnifiedHeapSnapshotTest, RetainingUnnamedType) {
   cppgc::Persistent<BaseWithoutName> base_without_name =
       cppgc::MakeGarbageCollected<BaseWithoutName>(allocation_handle());
@@ -406,16 +422,19 @@ class GCedWithJSRef : public cppgc::GarbageCollected<GCedWithJSRef> {
     v8_object_.Reset(isolate, object);
   }
 
-  void SetWrapperClassId(uint16_t class_id) {
-    v8_object_.SetWrapperClassId(class_id);
-  }
-
-  uint16_t WrapperClassId() const { return v8_object_.WrapperClassId(); }
-
   TracedReference<v8::Object>& wrapper() { return v8_object_; }
 
+  void set_detachedness(v8::EmbedderGraph::Node::Detachedness detachedness) {
+    detachedness_ = detachedness;
+  }
+  v8::EmbedderGraph::Node::Detachedness detachedness() const {
+    return detachedness_;
+  }
+
  private:
   TracedReference<v8::Object> v8_object_;
+  v8::EmbedderGraph::Node::Detachedness detachedness_ =
+      v8::EmbedderGraph::Node::Detachedness ::kUnknown;
 };
 
 constexpr const char GCedWithJSRef::kExpectedName[];
@@ -443,13 +462,16 @@ class V8_NODISCARD JsTestingScope {
 
 cppgc::Persistent<GCedWithJSRef> SetupWrapperWrappablePair(
     JsTestingScope& testing_scope, cppgc::AllocationHandle& allocation_handle,
-    const char* name) {
+    const char* name,
+    v8::EmbedderGraph::Node::Detachedness detachedness =
+        v8::EmbedderGraph::Node::Detachedness::kUnknown) {
   cppgc::Persistent<GCedWithJSRef> gc_w_js_ref =
       cppgc::MakeGarbageCollected<GCedWithJSRef>(allocation_handle);
   v8::Local<v8::Object> wrapper_object = WrapperHelper::CreateWrapper(
       testing_scope.context(), &GCedWithJSRef::kWrappableType,
       gc_w_js_ref.Get(), name);
   gc_w_js_ref->SetV8Object(testing_scope.isolate(), wrapper_object);
+  gc_w_js_ref->set_detachedness(detachedness);
   return gc_w_js_ref;
 }
 
@@ -473,21 +495,24 @@ TEST_F(UnifiedHeapSnapshotTest, JSReferenceForcesVisibleObject) {
   JsTestingScope testing_scope(v8_isolate());
   cppgc::Persistent<GCedWithJSRef> gc_w_js_ref = SetupWrapperWrappablePair(
       testing_scope, allocation_handle(), "LeafJSObject");
+  // Reset the JS->C++ ref or otherwise the nodes would be merged.
+  WrapperHelper::ResetWrappableConnection(
+      gc_w_js_ref->wrapper().Get(v8_isolate()));
   const v8::HeapSnapshot* snapshot = TakeHeapSnapshot();
   EXPECT_TRUE(IsValidSnapshot(snapshot));
   EXPECT_TRUE(ContainsRetainingPath(
-      *snapshot, {kExpectedCppRootsName, GetExpectedName<GCedWithJSRef>(),
-                  "LeafJSObject"}));
+      *snapshot,
+      {kExpectedCppRootsName, GetExpectedName<GCedWithJSRef>(), "LeafJSObject"},
+      true));
 }
 
 TEST_F(UnifiedHeapSnapshotTest, MergedWrapperNode) {
   // Test ensures that the snapshot sets a wrapper node for C++->JS references
-  // that have a class id set and that object nodes are merged. In practice, the
-  // C++ node is merged into the existing JS node.
+  // that have a valid back reference and that object nodes are merged. In
+  // practice, the C++ node is merged into the existing JS node.
   JsTestingScope testing_scope(v8_isolate());
   cppgc::Persistent<GCedWithJSRef> gc_w_js_ref = SetupWrapperWrappablePair(
       testing_scope, allocation_handle(), "MergedObject");
-  gc_w_js_ref->SetWrapperClassId(1);  // Any class id will do.
   v8::Local<v8::Object> next_object = WrapperHelper::CreateWrapper(
       testing_scope.context(), nullptr, nullptr, "NextObject");
   v8::Local<v8::Object> wrapper_object =
@@ -527,19 +552,16 @@ TEST_F(UnifiedHeapSnapshotTest, MergedWrapperNode) {
 
 namespace {
 
-constexpr uint16_t kClassIdForAttachedState = 0xAAAA;
-
 class DetachednessHandler {
  public:
   static size_t callback_count;
 
   static v8::EmbedderGraph::Node::Detachedness GetDetachedness(
-      v8::Isolate* isolate, const v8::Local<v8::Value>& v8_value,
-      uint16_t class_id, void* data) {
+      v8::Isolate* isolate, const v8::Local<v8::Value>& v8_value, uint16_t,
+      void*) {
     callback_count++;
-    return class_id == kClassIdForAttachedState
-               ? v8::EmbedderGraph::Node::Detachedness::kAttached
-               : v8::EmbedderGraph::Node::Detachedness::kDetached;
+    return WrapperHelper::UnwrapAs<GCedWithJSRef>(v8_value.As<v8::Object>())
+        ->detachedness();
   }
 
   static void Reset() { callback_count = 0; }
@@ -556,17 +578,20 @@ constexpr uint8_t kExpectedDetachedValueForDetached =
 
 }  // namespace
 
-TEST_F(UnifiedHeapSnapshotTest, NoTriggerForClassIdZero) {
-  // Test ensures that objects with JS references that have no class id set do
-  // not have their detachedness state queried.
+TEST_F(UnifiedHeapSnapshotTest, NoTriggerForStandAloneTracedReference) {
+  // Test ensures that C++ objects with TracedReference have their V8 objects
+  // not merged and queried for detachedness if the backreference is invalid.
   JsTestingScope testing_scope(v8_isolate());
+  // Marking the object as attached. The check below queries for unknown, making
+  // sure that the state is not propagated.
   cppgc::Persistent<GCedWithJSRef> gc_w_js_ref = SetupWrapperWrappablePair(
-      testing_scope, allocation_handle(), "MergedObject");
+      testing_scope, allocation_handle(), "MergedObject",
+      v8::EmbedderGraph::Node::Detachedness::kAttached);
   DetachednessHandler::Reset();
   v8_isolate()->GetHeapProfiler()->SetGetDetachednessCallback(
       DetachednessHandler::GetDetachedness, nullptr);
-  gc_w_js_ref->SetWrapperClassId(0);
-  EXPECT_EQ(0u, gc_w_js_ref->WrapperClassId());
+  WrapperHelper::ResetWrappableConnection(
+      gc_w_js_ref->wrapper().Get(v8_isolate()));
   const v8::HeapSnapshot* snapshot = TakeHeapSnapshot();
   EXPECT_EQ(0u, DetachednessHandler::callback_count);
   EXPECT_TRUE(IsValidSnapshot(snapshot));
@@ -582,16 +607,16 @@ TEST_F(UnifiedHeapSnapshotTest, NoTriggerForClassIdZero) {
 }
 
 TEST_F(UnifiedHeapSnapshotTest, TriggerDetachednessCallbackSettingAttached) {
-  // Test ensures that objects with JS references that have a non-zero class id
-  // set do  have their detachedness state queried and set (attached version).
+  // Test ensures that objects with JS references that have a valid back
+  // reference set do have their detachedness state queried and set (attached
+  // version).
   JsTestingScope testing_scope(v8_isolate());
   cppgc::Persistent<GCedWithJSRef> gc_w_js_ref = SetupWrapperWrappablePair(
-      testing_scope, allocation_handle(), "MergedObject");
+      testing_scope, allocation_handle(), "MergedObject",
+      v8::EmbedderGraph::Node::Detachedness::kAttached);
   DetachednessHandler::Reset();
   v8_isolate()->GetHeapProfiler()->SetGetDetachednessCallback(
       DetachednessHandler::GetDetachedness, nullptr);
-  gc_w_js_ref->SetWrapperClassId(kClassIdForAttachedState);
-  EXPECT_NE(0u, gc_w_js_ref->WrapperClassId());
   const v8::HeapSnapshot* snapshot = TakeHeapSnapshot();
   EXPECT_EQ(1u, DetachednessHandler::callback_count);
   EXPECT_TRUE(IsValidSnapshot(snapshot));
@@ -607,16 +632,16 @@ TEST_F(UnifiedHeapSnapshotTest, TriggerDetachednessCallbackSettingAttached) {
 }
 
 TEST_F(UnifiedHeapSnapshotTest, TriggerDetachednessCallbackSettingDetached) {
-  // Test ensures that objects with JS references that have a non-zero class id
-  // set do  have their detachedness state queried and set (detached version).
+  // Test ensures that objects with JS references that have a valid back
+  // reference set do have their detachedness state queried and set (detached
+  // version).
   JsTestingScope testing_scope(v8_isolate());
   cppgc::Persistent<GCedWithJSRef> gc_w_js_ref = SetupWrapperWrappablePair(
-      testing_scope, allocation_handle(), "MergedObject");
+      testing_scope, allocation_handle(), "MergedObject",
+      v8::EmbedderGraph::Node::Detachedness ::kDetached);
   DetachednessHandler::Reset();
   v8_isolate()->GetHeapProfiler()->SetGetDetachednessCallback(
       DetachednessHandler::GetDetachedness, nullptr);
-  gc_w_js_ref->SetWrapperClassId(kClassIdForAttachedState - 1);
-  EXPECT_NE(0u, gc_w_js_ref->WrapperClassId());
   const v8::HeapSnapshot* snapshot = TakeHeapSnapshot();
   EXPECT_EQ(1u, DetachednessHandler::callback_count);
   EXPECT_TRUE(IsValidSnapshot(snapshot));
