@@ -18,6 +18,7 @@
 #include "src/heap/remembered-set.h"
 #include "src/heap/safepoint.h"
 #include "src/heap/spaces-inl.h"
+#include "src/heap/trusted-range.h"
 #include "src/objects/objects-inl.h"
 #include "test/unittests/heap/heap-utils.h"
 #include "test/unittests/test-utils.h"
@@ -192,21 +193,29 @@ TEST_F(HeapTest, HeapLayout) {
     EXPECT_TRUE(IsAligned(code_cage_base, size_t{4} * GB));
   }
 
+#if V8_ENABLE_SANDBOX
+  Address trusted_space_base =
+      TrustedRange::GetProcessWideTrustedRange()->base();
+  EXPECT_TRUE(IsAligned(trusted_space_base, size_t{4} * GB));
+  base::AddressRegion trusted_reservation(trusted_space_base, size_t{4} * GB);
+#endif
+
   // Check that all memory chunks belong this region.
   base::AddressRegion heap_reservation(cage_base, size_t{4} * GB);
   base::AddressRegion code_reservation(code_cage_base, size_t{4} * GB);
 
   IsolateSafepointScope scope(i_isolate()->heap());
   OldGenerationMemoryChunkIterator iter(i_isolate()->heap());
-  for (;;) {
-    MemoryChunk* chunk = iter.next();
-    if (chunk == nullptr) break;
-
+  while (MemoryChunk* chunk = iter.next()) {
     Address address = chunk->address();
     size_t size = chunk->area_end() - address;
     AllocationSpace owner_id = chunk->owner_identity();
     if (V8_EXTERNAL_CODE_SPACE_BOOL && IsAnyCodeSpace(owner_id)) {
       EXPECT_TRUE(code_reservation.contains(address, size));
+#if V8_ENABLE_SANDBOX
+    } else if (IsAnyTrustedSpace(owner_id)) {
+      EXPECT_TRUE(trusted_reservation.contains(address, size));
+#endif
     } else {
       EXPECT_TRUE(heap_reservation.contains(address, size));
     }
@@ -391,7 +400,7 @@ TEST_F(HeapTest, OptimizedAllocationAlwaysInNewSpace) {
 
 namespace {
 template <RememberedSetType direction>
-static size_t GetRememberedSetSize(HeapObject obj) {
+static size_t GetRememberedSetSize(Tagged<HeapObject> obj) {
   size_t count = 0;
   auto chunk = MemoryChunk::FromHeapObject(obj);
   RememberedSet<direction>::Iterate(
@@ -466,9 +475,9 @@ TEST_F(HeapTest, Regress978156) {
   // 3. Trim the last array by one word thus creating a one-word filler.
   Handle<FixedArray> last = arrays.back();
   CHECK_GT(last->length(), 0);
-  heap->RightTrimFixedArray(*last, 1);
+  heap->RightTrimArray(*last, last->length() - 1, last->length());
   // 4. Get the last filler on the page.
-  HeapObject filler = HeapObject::FromAddress(
+  Tagged<HeapObject> filler = HeapObject::FromAddress(
       MemoryChunk::FromHeapObject(*last)->area_end() - kTaggedSize);
   HeapObject::FromAddress(last->address() + last->Size());
   CHECK(IsFiller(filler));
