@@ -56,9 +56,7 @@ void FillPageInPagedSpace(Page* page,
   ManualGCScope manual_gc_scope(heap->isolate());
   DCHECK(page->SweepingDone());
   PagedSpaceBase* paged_space = static_cast<PagedSpaceBase*>(page->owner());
-  // Make sure the LAB is empty to guarantee that all free space is accounted
-  // for in the freelist.
-  DCHECK_EQ(paged_space->limit(), paged_space->top());
+  heap->FreeLinearAllocationAreas();
 
   PauseAllocationObserversScope no_observers_scope(heap);
 
@@ -105,7 +103,7 @@ void FillPageInPagedSpace(Page* page,
     }
   }
 
-  paged_space->FreeLinearAllocationArea();
+  heap->FreeLinearAllocationAreas();
 
   // Allocate FixedArrays in remaining free list blocks, from largest
   // category to smallest.
@@ -146,6 +144,7 @@ void FillPageInPagedSpace(Page* page,
                            GCTracer::Scope::ScopeId::MARK_COMPACTOR));
   CHECK_EQ(young_epoch, heap->tracer()->CurrentEpoch(
                             GCTracer::Scope::ScopeId::MINOR_MARK_SWEEPER));
+  heap->FreeLinearAllocationAreas();
 }
 
 }  // namespace
@@ -153,20 +152,24 @@ void FillPageInPagedSpace(Page* page,
 void HeapInternalsBase::SimulateFullSpace(
     v8::internal::NewSpace* space,
     std::vector<Handle<FixedArray>>* out_handles) {
+  Heap* heap = space->heap();
+  IsolateSafepointScope safepoint_scope(heap);
+  heap->FreeLinearAllocationAreas();
   // If you see this check failing, disable the flag at the start of your test:
   // v8_flags.stress_concurrent_allocation = false;
   // Background thread allocating concurrently interferes with this function.
   CHECK(!v8_flags.stress_concurrent_allocation);
   space->heap()->EnsureSweepingCompleted(
       Heap::SweepingForcedFinalizationMode::kV8Only);
-  space->FreeLinearAllocationArea();
   if (v8_flags.minor_ms) {
-    while (space->AddFreshPage()) {}
+    auto* space = heap->paged_new_space()->paged_space();
+    space->AllocatePageUpToCapacityForTesting();
     for (Page* page : *space) {
       FillPageInPagedSpace(page, out_handles);
     }
     DCHECK_IMPLIES(space->free_list(), space->free_list()->Available() == 0);
   } else {
+    SemiSpaceNewSpace* space = SemiSpaceNewSpace::From(heap->new_space());
     do {
       FillCurrentPage(space, out_handles);
     } while (space->AddFreshPage());
@@ -174,30 +177,21 @@ void HeapInternalsBase::SimulateFullSpace(
 }
 
 void HeapInternalsBase::SimulateFullSpace(v8::internal::PagedSpace* space) {
+  Heap* heap = space->heap();
+  IsolateSafepointScope safepoint_scope(heap);
+  heap->FreeLinearAllocationAreas();
   // If you see this check failing, disable the flag at the start of your test:
   // v8_flags.stress_concurrent_allocation = false;
   // Background thread allocating concurrently interferes with this function.
   CHECK(!v8_flags.stress_concurrent_allocation);
-  Heap* heap = space->heap();
   if (heap->sweeping_in_progress()) {
     heap->EnsureSweepingCompleted(
         Heap::SweepingForcedFinalizationMode::kV8Only);
   }
-  space->FreeLinearAllocationArea();
   space->ResetFreeList();
 }
 
 namespace {
-int GetSpaceRemainingOnCurrentSemiSpacePage(v8::internal::NewSpace* space) {
-  Address top = space->top();
-  if ((top & kPageAlignmentMask) == 0) {
-    // `top` points to the start of a page signifies that there is not room in
-    // the current page.
-    return 0;
-  }
-  return static_cast<int>(Page::FromAddress(space->top())->area_end() - top);
-}
-
 std::vector<Handle<FixedArray>> CreatePadding(Heap* heap, int padding_size,
                                               AllocationType allocation) {
   std::vector<Handle<FixedArray>> handles;
@@ -205,8 +199,8 @@ std::vector<Handle<FixedArray>> CreatePadding(Heap* heap, int padding_size,
   int allocate_memory;
   int length;
   int free_memory = padding_size;
+  heap->FreeMainThreadLinearAllocationAreas();
   if (allocation == i::AllocationType::kOld) {
-    heap->old_space()->FreeLinearAllocationArea();
     int overall_free_memory = static_cast<int>(heap->old_space()->Available());
     CHECK(padding_size <= overall_free_memory || overall_free_memory == 0);
   } else {
@@ -223,11 +217,11 @@ std::vector<Handle<FixedArray>> CreatePadding(Heap* heap, int padding_size,
       if (length <= 0) {
         // Not enough room to create another FixedArray, so create a filler.
         if (allocation == i::AllocationType::kOld) {
-          heap->CreateFillerObjectAt(
-              *heap->old_space()->allocation_top_address(), free_memory);
+          heap->CreateFillerObjectAt(*heap->OldSpaceAllocationTopAddress(),
+                                     free_memory);
         } else {
-          heap->CreateFillerObjectAt(
-              *heap->new_space()->allocation_top_address(), free_memory);
+          heap->CreateFillerObjectAt(*heap->NewSpaceAllocationTopAddress(),
+                                     free_memory);
         }
         break;
       }
@@ -243,14 +237,16 @@ std::vector<Handle<FixedArray>> CreatePadding(Heap* heap, int padding_size,
   return handles;
 }
 
-void FillCurrentSemiSpacePage(v8::internal::NewSpace* space,
+void FillCurrentSemiSpacePage(v8::internal::SemiSpaceNewSpace* space,
                               std::vector<Handle<FixedArray>>* out_handles) {
   // We cannot rely on `space->limit()` to point to the end of the current page
   // in the case where inline allocations are disabled, it actually points to
   // the current allocation pointer.
-  DCHECK_IMPLIES(!space->heap()->IsInlineAllocationEnabled(),
-                 space->limit() == space->top());
-  int space_remaining = GetSpaceRemainingOnCurrentSemiSpacePage(space);
+  DCHECK_IMPLIES(
+      !space->heap()->IsInlineAllocationEnabled(),
+      space->heap()->NewSpaceTop() == space->heap()->NewSpaceLimit());
+
+  int space_remaining = space->GetSpaceRemainingOnCurrentPageForTesting();
   if (space_remaining == 0) return;
   std::vector<Handle<FixedArray>> handles =
       CreatePadding(space->heap(), space_remaining, i::AllocationType::kYoung);
@@ -259,13 +255,13 @@ void FillCurrentSemiSpacePage(v8::internal::NewSpace* space,
   }
 }
 
-void FillCurrenPagedSpacePage(v8::internal::NewSpace* space,
-                              std::vector<Handle<FixedArray>>* out_handles) {
-  if (space->top() == kNullAddress) return;
-  Page* page = Page::FromAllocationAreaAddress(space->top());
+void FillCurrentPagedSpacePage(v8::internal::NewSpace* space,
+                               std::vector<Handle<FixedArray>>* out_handles) {
+  const Address top = space->heap()->NewSpaceTop();
+  if (top == kNullAddress) return;
+  Page* page = Page::FromAllocationAreaAddress(top);
   space->heap()->EnsureSweepingCompleted(
       Heap::SweepingForcedFinalizationMode::kV8Only);
-  space->FreeLinearAllocationArea();
   FillPageInPagedSpace(page, out_handles);
 }
 
@@ -275,10 +271,14 @@ void HeapInternalsBase::FillCurrentPage(
     v8::internal::NewSpace* space,
     std::vector<Handle<FixedArray>>* out_handles) {
   PauseAllocationObserversScope pause_observers(space->heap());
-  if (v8_flags.minor_ms)
-    FillCurrenPagedSpacePage(space, out_handles);
-  else
-    FillCurrentSemiSpacePage(space, out_handles);
+  MainAllocator* allocator = space->heap()->allocator()->new_space_allocator();
+  allocator->FreeLinearAllocationArea();
+  if (v8_flags.minor_ms) {
+    FillCurrentPagedSpacePage(space, out_handles);
+  } else {
+    FillCurrentSemiSpacePage(SemiSpaceNewSpace::From(space), out_handles);
+  }
+  allocator->FreeLinearAllocationArea();
 }
 
 bool IsNewObjectInCorrectGeneration(Tagged<HeapObject> object) {
