@@ -12,6 +12,7 @@
 #include "src/heap/heap-write-barrier-inl.h"
 #include "src/heap/heap.h"
 #include "src/heap/large-spaces.h"
+#include "src/heap/main-allocator.h"
 #include "src/heap/memory-chunk.h"
 #include "src/heap/spaces-inl.h"
 #include "test/unittests/test-utils.h"
@@ -19,6 +20,30 @@
 namespace v8 {
 namespace internal {
 
+static Tagged<HeapObject> AllocateUnaligned(MainAllocator* allocator,
+                                            SpaceWithLinearArea* space,
+                                            int size) {
+  AllocationResult allocation =
+      allocator->AllocateRaw(size, kTaggedAligned, AllocationOrigin::kRuntime);
+  CHECK(!allocation.IsFailure());
+  Tagged<HeapObject> filler;
+  CHECK(allocation.To(&filler));
+  space->heap()->CreateFillerObjectAt(filler.address(), size);
+  return filler;
+}
+
+static Tagged<HeapObject> AllocateUnaligned(OldLargeObjectSpace* allocator,
+                                            OldLargeObjectSpace* space,
+                                            int size) {
+  AllocationResult allocation =
+      allocator->AllocateRaw(space->heap()->main_thread_local_heap(), size);
+  CHECK(!allocation.IsFailure());
+  Tagged<HeapObject> filler;
+  CHECK(allocation.To(&filler));
+  space->heap()->CreateFillerObjectAt(filler.address(), size);
+  return filler;
+}
+
 using SpacesTest = TestWithIsolate;
 
 TEST_F(SpacesTest, CompactionSpaceMerge) {
@@ -26,9 +51,12 @@ TEST_F(SpacesTest, CompactionSpaceMerge) {
   OldSpace* old_space = heap->old_space();
   EXPECT_TRUE(old_space != nullptr);
 
+  heap->SetGCState(Heap::MARK_COMPACT);
+
   CompactionSpace* compaction_space =
       new CompactionSpace(heap, OLD_SPACE, NOT_EXECUTABLE,
                           CompactionSpaceKind::kCompactionSpaceForMarkCompact);
+  MainAllocator allocator(heap, compaction_space, MainAllocator::kInGC);
   EXPECT_TRUE(compaction_space != nullptr);
 
   for (Page* p : *old_space) {
@@ -46,29 +74,36 @@ TEST_F(SpacesTest, CompactionSpaceMerge) {
       (kNumObjects + kNumObjectsPerPage - 1) / kNumObjectsPerPage;
   for (int i = 0; i < kNumObjects; i++) {
     Tagged<HeapObject> object =
-        compaction_space->AllocateRawUnaligned(kMaxRegularHeapObjectSize)
+        allocator
+            .AllocateRaw(kMaxRegularHeapObjectSize, kTaggedAligned,
+                         AllocationOrigin::kGC)
             .ToObjectChecked();
     heap->CreateFillerObjectAt(object.address(), kMaxRegularHeapObjectSize);
   }
   int pages_in_old_space = old_space->CountTotalPages();
   int pages_in_compaction_space = compaction_space->CountTotalPages();
   EXPECT_EQ(kExpectedPages, pages_in_compaction_space);
+  allocator.FreeLinearAllocationArea();
   old_space->MergeCompactionSpace(compaction_space);
   EXPECT_EQ(pages_in_old_space + pages_in_compaction_space,
             old_space->CountTotalPages());
 
   delete compaction_space;
+
+  heap->SetGCState(Heap::NOT_IN_GC);
 }
 
 TEST_F(SpacesTest, WriteBarrierFromHeapObject) {
   constexpr Address address1 = Page::kPageSize;
-  Tagged<HeapObject> object1 = HeapObject::unchecked_cast(Object(address1));
+  Tagged<HeapObject> object1 =
+      HeapObject::unchecked_cast(Tagged<Object>(address1));
   BasicMemoryChunk* chunk1 = BasicMemoryChunk::FromHeapObject(object1);
   heap_internals::MemoryChunk* slim_chunk1 =
       heap_internals::MemoryChunk::FromHeapObject(object1);
   EXPECT_EQ(static_cast<void*>(chunk1), static_cast<void*>(slim_chunk1));
   constexpr Address address2 = 2 * Page::kPageSize - 1;
-  Tagged<HeapObject> object2 = HeapObject::unchecked_cast(Object(address2));
+  Tagged<HeapObject> object2 =
+      HeapObject::unchecked_cast(Tagged<Object>(address2));
   BasicMemoryChunk* chunk2 = BasicMemoryChunk::FromHeapObject(object2);
   heap_internals::MemoryChunk* slim_chunk2 =
       heap_internals::MemoryChunk::FromHeapObject(object2);
@@ -207,48 +242,6 @@ TEST_F(SpacesTest, FreeListManySelectFreeListCategoryType) {
   }
 }
 
-// Tests that FreeListMany::GuaranteedAllocatable returns what it should.
-TEST_F(SpacesTest, FreeListManyGuaranteedAllocatable) {
-  FreeListMany free_list;
-
-  for (int cat = kFirstCategory; cat < free_list.last_category_; cat++) {
-    std::vector<size_t> sizes;
-    // Adding size less than this category's minimum
-    sizes.push_back(free_list.categories_min[cat] - 8);
-    // Adding size equal to this category's minimum
-    sizes.push_back(free_list.categories_min[cat]);
-    // Adding size greater than this category's minimum
-    sizes.push_back(free_list.categories_min[cat] + 8);
-    if (cat != free_list.last_category_) {
-      // Adding size between this category's minimum and the next category
-      sizes.push_back(
-          (free_list.categories_min[cat] + free_list.categories_min[cat + 1]) /
-          2);
-    }
-
-    for (size_t size : sizes) {
-      FreeListCategoryType cat_free =
-          free_list.SelectFreeListCategoryType(size);
-      size_t guaranteed_allocatable = free_list.GuaranteedAllocatable(size);
-      if (cat_free == free_list.last_category_) {
-        // If |cat_free| == last_category, then guaranteed_allocatable must
-        // return the last category, because when allocating, the last category
-        // is searched entirely.
-        EXPECT_EQ(free_list.SelectFreeListCategoryType(guaranteed_allocatable),
-                  free_list.last_category_);
-      } else if (size < free_list.categories_min[0]) {
-        // If size < free_list.categories_min[0], then the bytes are wasted, and
-        // guaranteed_allocatable should return 0.
-        EXPECT_EQ(guaranteed_allocatable, 0ul);
-      } else {
-        // Otherwise, |guaranteed_allocatable| is equal to the minimum of
-        // |size|'s category (|cat_free|);
-        EXPECT_EQ(free_list.categories_min[cat_free], guaranteed_allocatable);
-      }
-    }
-  }
-}
-
 // Tests that
 // FreeListManyCachedFastPath::SelectFastAllocationFreeListCategoryType returns
 // what it should.
@@ -298,5 +291,140 @@ TEST_F(SpacesTest,
   }
 }
 
+class Observer : public AllocationObserver {
+ public:
+  explicit Observer(intptr_t step_size)
+      : AllocationObserver(step_size), count_(0) {}
+
+  void Step(int bytes_allocated, Address addr, size_t) override { count_++; }
+
+  int count() const { return count_; }
+
+ private:
+  int count_;
+};
+
+template <typename T, typename A>
+void testAllocationObserver(Isolate* i_isolate, T* space, A* allocator) {
+  Observer observer1(128);
+  i_isolate->heap()->FreeMainThreadLinearAllocationAreas();
+  allocator->AddAllocationObserver(&observer1);
+
+  // The observer should not get notified if we have only allocated less than
+  // 128 bytes.
+  AllocateUnaligned(allocator, space, 64);
+  CHECK_EQ(observer1.count(), 0);
+
+  // The observer should get called when we have allocated exactly 128 bytes.
+  AllocateUnaligned(allocator, space, 64);
+  CHECK_EQ(observer1.count(), 1);
+
+  // Another >128 bytes should get another notification.
+  AllocateUnaligned(allocator, space, 136);
+  CHECK_EQ(observer1.count(), 2);
+
+  // Allocating a large object should get only one notification.
+  AllocateUnaligned(allocator, space, 1024);
+  CHECK_EQ(observer1.count(), 3);
+
+  // Allocating another 2048 bytes in small objects should get 16
+  // notifications.
+  for (int i = 0; i < 64; ++i) {
+    AllocateUnaligned(allocator, space, 32);
+  }
+  CHECK_EQ(observer1.count(), 19);
+
+  // Multiple observers should work.
+  Observer observer2(96);
+  i_isolate->heap()->FreeMainThreadLinearAllocationAreas();
+  allocator->AddAllocationObserver(&observer2);
+
+  AllocateUnaligned(allocator, space, 2048);
+  CHECK_EQ(observer1.count(), 20);
+  CHECK_EQ(observer2.count(), 1);
+
+  AllocateUnaligned(allocator, space, 104);
+  CHECK_EQ(observer1.count(), 20);
+  CHECK_EQ(observer2.count(), 2);
+
+  // Callback should stop getting called after an observer is removed.
+  allocator->RemoveAllocationObserver(&observer1);
+
+  AllocateUnaligned(allocator, space, 384);
+  CHECK_EQ(observer1.count(), 20);  // no more notifications.
+  CHECK_EQ(observer2.count(), 3);   // this one is still active.
+
+  // Ensure that PauseInlineAllocationObserversScope work correctly.
+  AllocateUnaligned(allocator, space, 48);
+  CHECK_EQ(observer2.count(), 3);
+  {
+    i_isolate->heap()->FreeMainThreadLinearAllocationAreas();
+    PauseAllocationObserversScope pause_observers(i_isolate->heap());
+    CHECK_EQ(observer2.count(), 3);
+    AllocateUnaligned(allocator, space, 384);
+    CHECK_EQ(observer2.count(), 3);
+    i_isolate->heap()->FreeMainThreadLinearAllocationAreas();
+  }
+  CHECK_EQ(observer2.count(), 3);
+  // Coupled with the 48 bytes allocated before the pause, another 48 bytes
+  // allocated here should trigger a notification.
+  AllocateUnaligned(allocator, space, 48);
+  CHECK_EQ(observer2.count(), 4);
+
+  allocator->RemoveAllocationObserver(&observer2);
+  AllocateUnaligned(allocator, space, 384);
+  CHECK_EQ(observer1.count(), 20);
+  CHECK_EQ(observer2.count(), 4);
+}
+
+TEST_F(SpacesTest, AllocationObserver) {
+  if (v8_flags.single_generation) return;
+  v8::Isolate::Scope isolate_scope(v8_isolate());
+  v8::HandleScope handle_scope(v8_isolate());
+  v8::Context::New(v8_isolate())->Enter();
+
+  testAllocationObserver<NewSpace>(
+      i_isolate(), i_isolate()->heap()->new_space(),
+      i_isolate()->heap()->allocator()->new_space_allocator());
+  // Old space is used but the code path is shared for all
+  // classes inheriting from PagedSpace.
+  testAllocationObserver<PagedSpace>(
+      i_isolate(), i_isolate()->heap()->old_space(),
+      i_isolate()->heap()->allocator()->old_space_allocator());
+  testAllocationObserver<OldLargeObjectSpace>(i_isolate(),
+                                              i_isolate()->heap()->lo_space(),
+                                              i_isolate()->heap()->lo_space());
+}
+
+TEST_F(SpacesTest, InlineAllocationObserverCadence) {
+  if (v8_flags.single_generation) return;
+  v8::Isolate::Scope isolate_scope(v8_isolate());
+  v8::HandleScope handle_scope(v8_isolate());
+  v8::Context::New(v8_isolate())->Enter();
+
+  // Clear out any pre-existing garbage to make the test consistent
+  // across snapshot/no-snapshot builds.
+  InvokeMajorGC(i_isolate());
+
+  MainAllocator* new_space_allocator =
+      i_isolate()->heap()->allocator()->new_space_allocator();
+
+  Observer observer1(512);
+  new_space_allocator->AddAllocationObserver(&observer1);
+  Observer observer2(576);
+  new_space_allocator->AddAllocationObserver(&observer2);
+
+  for (int i = 0; i < 512; ++i) {
+    AllocateUnaligned(new_space_allocator, i_isolate()->heap()->new_space(),
+                      32);
+  }
+
+  new_space_allocator->RemoveAllocationObserver(&observer1);
+  new_space_allocator->RemoveAllocationObserver(&observer2);
+
+  CHECK_EQ(observer1.count(), 32);
+  CHECK_EQ(observer2.count(), 28);
+}
+
 }  // namespace internal
 }  // namespace v8
