@@ -116,8 +116,8 @@ TEST_F(ElementsKindTest, JSObjectAddingProperties) {
   JSObject::DefinePropertyOrElementIgnoreAttributes(object, name, value, NONE)
       .Check();
   CHECK_NE(object->map(), *previous_map);
-  CHECK_EQ(HOLEY_ELEMENTS, object->map().elements_kind());
-  CHECK_LE(1, object->property_array().length());
+  CHECK_EQ(HOLEY_ELEMENTS, object->map()->elements_kind());
+  CHECK_LE(1, object->property_array()->length());
   CHECK(EQUALS(i_isolate(), object->elements(), empty_fixed_array));
 }
 
@@ -132,7 +132,7 @@ TEST_F(ElementsKindTest, JSObjectInObjectAddingProperties) {
   int nof_inobject_properties = 10;
   // force in object properties by changing the expected_nof_properties
   // (we always reserve 8 inobject properties slack on top).
-  function->shared().set_expected_nof_properties(nof_inobject_properties - 8);
+  function->shared()->set_expected_nof_properties(nof_inobject_properties - 8);
   Handle<Object> value(Smi::FromInt(42), i_isolate());
 
   Handle<JSObject> object = factory->NewJSObject(function);
@@ -149,7 +149,7 @@ TEST_F(ElementsKindTest, JSObjectInObjectAddingProperties) {
         .Check();
   }
   CHECK_NE(object->map(), *previous_map);
-  CHECK_EQ(HOLEY_ELEMENTS, object->map().elements_kind());
+  CHECK_EQ(HOLEY_ELEMENTS, object->map()->elements_kind());
   CHECK(EQUALS(i_isolate(), object->property_array(), empty_property_array));
   CHECK(EQUALS(i_isolate(), object->elements(), empty_fixed_array));
 
@@ -160,9 +160,9 @@ TEST_F(ElementsKindTest, JSObjectInObjectAddingProperties) {
   JSObject::DefinePropertyOrElementIgnoreAttributes(object, name, value, NONE)
       .Check();
   CHECK_NE(object->map(), *previous_map);
-  CHECK_EQ(HOLEY_ELEMENTS, object->map().elements_kind());
+  CHECK_EQ(HOLEY_ELEMENTS, object->map()->elements_kind());
   // there must be at least 1 element in the properies store
-  CHECK_LE(1, object->property_array().length());
+  CHECK_LE(1, object->property_array()->length());
   CHECK(EQUALS(i_isolate(), object->elements(), empty_fixed_array));
 }
 
@@ -189,9 +189,9 @@ TEST_F(ElementsKindTest, JSObjectAddingElements) {
       .Check();
   // no change in elements_kind => no map transition
   CHECK_EQ(object->map(), *previous_map);
-  CHECK_EQ(HOLEY_ELEMENTS, object->map().elements_kind());
+  CHECK_EQ(HOLEY_ELEMENTS, object->map()->elements_kind());
   CHECK(EQUALS(i_isolate(), object->property_array(), empty_property_array));
-  CHECK_LE(1, object->elements().length());
+  CHECK_LE(1, object->elements()->length());
 
   // Adding more consecutive elements without a change in the backing store
   int non_dict_backing_store_limit = 100;
@@ -202,9 +202,9 @@ TEST_F(ElementsKindTest, JSObjectAddingElements) {
   }
   // no change in elements_kind => no map transition
   CHECK_EQ(object->map(), *previous_map);
-  CHECK_EQ(HOLEY_ELEMENTS, object->map().elements_kind());
+  CHECK_EQ(HOLEY_ELEMENTS, object->map()->elements_kind());
   CHECK(EQUALS(i_isolate(), object->property_array(), empty_property_array));
-  CHECK_LE(non_dict_backing_store_limit, object->elements().length());
+  CHECK_LE(non_dict_backing_store_limit, object->elements()->length());
 
   // Adding an element at an very large index causes a change to
   // DICTIONARY_ELEMENTS
@@ -213,9 +213,9 @@ TEST_F(ElementsKindTest, JSObjectAddingElements) {
       .Check();
   // change in elements_kind => map transition
   CHECK_NE(object->map(), *previous_map);
-  CHECK_EQ(DICTIONARY_ELEMENTS, object->map().elements_kind());
+  CHECK_EQ(DICTIONARY_ELEMENTS, object->map()->elements_kind());
   CHECK(EQUALS(i_isolate(), object->property_array(), empty_property_array));
-  CHECK_LE(non_dict_backing_store_limit, object->elements().length());
+  CHECK_LE(non_dict_backing_store_limit, object->elements()->length());
 }
 
 TEST_F(ElementsKindTest, JSArrayAddingProperties) {
@@ -241,8 +241,8 @@ TEST_F(ElementsKindTest, JSArrayAddingProperties) {
       .Check();
   // No change in elements_kind but added property => new map
   CHECK_NE(array->map(), *previous_map);
-  CHECK_EQ(PACKED_SMI_ELEMENTS, array->map().elements_kind());
-  CHECK_LE(1, array->property_array().length());
+  CHECK_EQ(PACKED_SMI_ELEMENTS, array->map()->elements_kind());
+  CHECK_LE(1, array->property_array()->length());
   CHECK(EQUALS(i_isolate(), array->elements(), empty_fixed_array));
   CHECK_EQ(0, Smi::ToInt(array->length()));
 }
@@ -270,9 +270,9 @@ TEST_F(ElementsKindTest, JSArrayAddingElements) {
       .Check();
   // no change in elements_kind => no map transition
   CHECK_EQ(array->map(), *previous_map);
-  CHECK_EQ(PACKED_SMI_ELEMENTS, array->map().elements_kind());
+  CHECK_EQ(PACKED_SMI_ELEMENTS, array->map()->elements_kind());
   CHECK(EQUALS(i_isolate(), array->property_array(), empty_property_array));
-  CHECK_LE(1, array->elements().length());
+  CHECK_LE(1, array->elements()->length());
   CHECK_EQ(1, Smi::ToInt(array->length()));
 
   // Adding more consecutive elements without a change in the backing store
@@ -284,9 +284,9 @@ TEST_F(ElementsKindTest, JSArrayAddingElements) {
   }
   // no change in elements_kind => no map transition
   CHECK_EQ(array->map(), *previous_map);
-  CHECK_EQ(PACKED_SMI_ELEMENTS, array->map().elements_kind());
+  CHECK_EQ(PACKED_SMI_ELEMENTS, array->map()->elements_kind());
   CHECK(EQUALS(i_isolate(), array->property_array(), empty_property_array));
-  CHECK_LE(non_dict_backing_store_limit, array->elements().length());
+  CHECK_LE(non_dict_backing_store_limit, array->elements()->length());
   CHECK_EQ(non_dict_backing_store_limit, Smi::ToInt(array->length()));
 
   // Adding an element at an very large index causes a change to
@@ -297,10 +297,10 @@ TEST_F(ElementsKindTest, JSArrayAddingElements) {
       .Check();
   // change in elements_kind => map transition
   CHECK_NE(array->map(), *previous_map);
-  CHECK_EQ(DICTIONARY_ELEMENTS, array->map().elements_kind());
+  CHECK_EQ(DICTIONARY_ELEMENTS, array->map()->elements_kind());
   CHECK(EQUALS(i_isolate(), array->property_array(), empty_property_array));
-  CHECK_LE(non_dict_backing_store_limit, array->elements().length());
-  CHECK_LE(array->elements().length(), index);
+  CHECK_LE(non_dict_backing_store_limit, array->elements()->length());
+  CHECK_LE(array->elements()->length(), index);
   CHECK_EQ(index + 1, Smi::ToInt(array->length()));
 }
 
@@ -326,14 +326,14 @@ TEST_F(ElementsKindTest, JSArrayAddingElementsGeneralizingiFastSmiElements) {
       .Check();
   // no change in elements_kind => no map transition
   CHECK_EQ(array->map(), *previous_map);
-  CHECK_EQ(PACKED_SMI_ELEMENTS, array->map().elements_kind());
+  CHECK_EQ(PACKED_SMI_ELEMENTS, array->map()->elements_kind());
   CHECK_EQ(1, Smi::ToInt(array->length()));
 
   // `delete array[0]` does not alter length, but changes the elments_kind
   name = MakeString("0");
   CHECK(JSReceiver::DeletePropertyOrElement(array, name).FromMaybe(false));
   CHECK_NE(array->map(), *previous_map);
-  CHECK_EQ(HOLEY_SMI_ELEMENTS, array->map().elements_kind());
+  CHECK_EQ(HOLEY_SMI_ELEMENTS, array->map()->elements_kind());
   CHECK_EQ(1, Smi::ToInt(array->length()));
   previous_map = handle(array->map(), i_isolate());
 
@@ -347,7 +347,7 @@ TEST_F(ElementsKindTest, JSArrayAddingElementsGeneralizingiFastSmiElements) {
                                                     NONE)
       .Check();
   CHECK_EQ(array->map(), *previous_map);
-  CHECK_EQ(HOLEY_SMI_ELEMENTS, array->map().elements_kind());
+  CHECK_EQ(HOLEY_SMI_ELEMENTS, array->map()->elements_kind());
   CHECK_EQ(2, Smi::ToInt(array->length()));
 
   // Adding a string to the array changes from FAST_HOLEY_SMI to FAST_HOLEY
@@ -356,7 +356,7 @@ TEST_F(ElementsKindTest, JSArrayAddingElementsGeneralizingiFastSmiElements) {
                                                     NONE)
       .Check();
   CHECK_NE(array->map(), *previous_map);
-  CHECK_EQ(HOLEY_ELEMENTS, array->map().elements_kind());
+  CHECK_EQ(HOLEY_ELEMENTS, array->map()->elements_kind());
   CHECK_EQ(2, Smi::ToInt(array->length()));
   previous_map = handle(array->map(), i_isolate());
 
@@ -396,14 +396,14 @@ TEST_F(ElementsKindTest, JSArrayAddingElementsGeneralizingFastElements) {
       .Check();
   // no change in elements_kind => no map transition
   CHECK_EQ(array->map(), *previous_map);
-  CHECK_EQ(PACKED_ELEMENTS, array->map().elements_kind());
+  CHECK_EQ(PACKED_ELEMENTS, array->map()->elements_kind());
   CHECK_EQ(1, Smi::ToInt(array->length()));
 
   // `delete array[0]` does not alter length, but changes the elments_kind
   name = MakeString("0");
   CHECK(JSReceiver::DeletePropertyOrElement(array, name).FromMaybe(false));
   CHECK_NE(array->map(), *previous_map);
-  CHECK_EQ(HOLEY_ELEMENTS, array->map().elements_kind());
+  CHECK_EQ(HOLEY_ELEMENTS, array->map()->elements_kind());
   CHECK_EQ(1, Smi::ToInt(array->length()));
   previous_map = handle(array->map(), i_isolate());
 
@@ -417,7 +417,7 @@ TEST_F(ElementsKindTest, JSArrayAddingElementsGeneralizingFastElements) {
                                                     NONE)
       .Check();
   CHECK_EQ(array->map(), *previous_map);
-  CHECK_EQ(HOLEY_ELEMENTS, array->map().elements_kind());
+  CHECK_EQ(HOLEY_ELEMENTS, array->map()->elements_kind());
   CHECK_EQ(2, Smi::ToInt(array->length()));
 }
 
@@ -440,7 +440,7 @@ TEST_F(ElementsKindTest, JSArrayAddingElementsGeneralizingiFastDoubleElements) {
                                                     NONE)
       .Check();
   CHECK_NE(array->map(), *previous_map);
-  CHECK_EQ(PACKED_DOUBLE_ELEMENTS, array->map().elements_kind());
+  CHECK_EQ(PACKED_DOUBLE_ELEMENTS, array->map()->elements_kind());
   CHECK_EQ(1, Smi::ToInt(array->length()));
   previous_map = handle(array->map(), i_isolate());
 
@@ -450,14 +450,14 @@ TEST_F(ElementsKindTest, JSArrayAddingElementsGeneralizingiFastDoubleElements) {
                                                     NONE)
       .Check();
   CHECK_EQ(array->map(), *previous_map);
-  CHECK_EQ(PACKED_DOUBLE_ELEMENTS, array->map().elements_kind());
+  CHECK_EQ(PACKED_DOUBLE_ELEMENTS, array->map()->elements_kind());
   CHECK_EQ(2, Smi::ToInt(array->length()));
 
   // `delete array[0]` does not alter length, but changes the elments_kind
   name = MakeString("0");
   CHECK(JSReceiver::DeletePropertyOrElement(array, name).FromMaybe(false));
   CHECK_NE(array->map(), *previous_map);
-  CHECK_EQ(HOLEY_DOUBLE_ELEMENTS, array->map().elements_kind());
+  CHECK_EQ(HOLEY_DOUBLE_ELEMENTS, array->map()->elements_kind());
   CHECK_EQ(2, Smi::ToInt(array->length()));
   previous_map = handle(array->map(), i_isolate());
 
@@ -467,7 +467,7 @@ TEST_F(ElementsKindTest, JSArrayAddingElementsGeneralizingiFastDoubleElements) {
                                                     NONE)
       .Check();
   CHECK_EQ(array->map(), *previous_map);
-  CHECK_EQ(HOLEY_DOUBLE_ELEMENTS, array->map().elements_kind());
+  CHECK_EQ(HOLEY_DOUBLE_ELEMENTS, array->map()->elements_kind());
   CHECK_EQ(2, Smi::ToInt(array->length()));
 
   // Adding a string to the array changes to elements_kind PACKED_ELEMENTS
@@ -476,7 +476,7 @@ TEST_F(ElementsKindTest, JSArrayAddingElementsGeneralizingiFastDoubleElements) {
                                                     NONE)
       .Check();
   CHECK_NE(array->map(), *previous_map);
-  CHECK_EQ(HOLEY_ELEMENTS, array->map().elements_kind());
+  CHECK_EQ(HOLEY_ELEMENTS, array->map()->elements_kind());
   CHECK_EQ(2, Smi::ToInt(array->length()));
   previous_map = handle(array->map(), i_isolate());
 
