@@ -23,7 +23,7 @@ The suite json format is expected to be:
   "timeout_XXX": <how long test is allowed run run for arch XXX>,
   "retry_count": <how many times to retry failures (in addition to first try)",
   "retry_count_XXX": <how many times to retry failures for arch XXX>
-  "resources": [<js file to be moved to android device>, ...]
+  "resources": [<js file to be moved to android device or "*">, ...]
   "variants": [
     {
       "name": <name of the variant>,
@@ -119,16 +119,18 @@ Path pieces are concatenated. D8 is always run with the suite's path as cwd.
 The test flags are passed to the js test file after '--'.
 """
 
-from collections import OrderedDict
+from abc import ABC, abstractmethod
 from math import sqrt
+from pathlib import Path
 from statistics import mean, stdev
+
+import argparse
 import copy
 import json
 import logging
 import math
-import argparse
-import pathlib
 import os
+import psutil
 import re
 import subprocess
 import sys
@@ -154,6 +156,8 @@ TOOLS_BASE = os.path.abspath(os.path.dirname(__file__))
 INFRA_FAILURE_RETCODE = 87
 MIN_RUNS_FOR_CONFIDENCE = 10
 
+WARMUP_CACHE_FILE = Path.cwd() / 'cache' / 'v8_perf' / 'warmup_cache.json'
+
 
 def GeometricMean(values):
   """Returns the geometric mean of a list of values.
@@ -612,7 +616,6 @@ class RunnableLeafTraceConfig(LeafTraceConfig, RunnableConfig):
       logging.error("No owners provided for %s" % self.name)
 
   def ProcessOutput(self, output, result_tracker, count):
-    result_tracker.AddRunnableDuration(self, output.duration)
     self.ConsumeOutput(output, result_tracker)
 
 
@@ -702,6 +705,9 @@ def BuildGraphConfigs(suite, parent, arch):
       graph.AppendChild(variant_graph)
       for subsuite in suite.get('tests', []):
         BuildGraphConfigs(subsuite, variant_graph, arch)
+      # Add variant specific tests.
+      for subsuite in variant_suite.get('tests', []):
+        BuildGraphConfigs(subsuite, variant_graph, arch)
   parent.AppendChild(graph)
   return graph
 
@@ -749,6 +755,73 @@ def find_build_directory(base_path, arch):
   return actual_paths[0]
 
 
+class CacheHandler:
+  def __init__(self, cache_file):
+    self.cache_file = cache_file
+
+  def read_cache(self):
+    try:
+      with open(self.cache_file) as f:
+        return json.load(f)
+    except FileNotFoundError:
+      logging.info(f"{self.cache_file} doesn't exist yet. Creating new.")
+    return {}
+
+  def write_cache(self, cache):
+    with open(self.cache_file, 'w') as f:
+      return json.dump(cache, f)
+
+
+class WarmupManager(ABC):
+  @abstractmethod
+  def maybe_warm_up(self, name, warmup_fun):
+    """Run the warmup_fun if needed, e.g. after a system reboot."""
+
+
+class NullWarmupManager(WarmupManager):
+  """Null-object place-holder used when warm-up isn't activated or for
+  platforms where it isn't implemented.
+  """
+  def maybe_warm_up(self, name, warmup_fun):
+    pass
+
+
+class CachedWarmupManager(WarmupManager):
+  """On-demand warm-up based on system reboot.
+
+  The warm-up function is run once after reboot for every benchmark key.
+  The keys are cached in a file in cache/v8_perf. This relies on the caller
+  creating this directory.
+  """
+  def __init__(self):
+    self.cache_handler = CacheHandler(WARMUP_CACHE_FILE)
+    self.cache = self.cache_handler.read_cache()
+    self.last_reboot = psutil.boot_time()
+    self.trim_cache()
+    # Ensure the trimmed version is on disk.
+    self.cache_handler.write_cache(self.cache)
+
+  def is_warmed_up(self, timestamp):
+    return timestamp > self.last_reboot
+
+  def trim_cache(self):
+    """Prevent obsolete entries occupying the cache file."""
+    self.cache = dict(
+        (k, v) for k, v in self.cache.items() if self.is_warmed_up(v))
+
+  def maybe_warm_up(self, name, warmup_fun):
+    if self.is_warmed_up(self.cache.get(name, 0)):
+      return
+
+    logging.info(f'Warm-up run of {name} - disregarding output.')
+    try:
+      warmup_fun()
+    finally:
+      self.cache[name] = time.time()
+      self.cache_handler.write_cache(self.cache)
+      logging.info(f'Warm-up done.')
+
+
 class Platform(object):
   def __init__(self, args):
     self.shell_dir = args.shell_dir
@@ -756,6 +829,7 @@ class Platform(object):
     self.is_dry_run = args.dry_run
     self.extra_flags = args.extra_flags.split()
     self.args = args
+    self.warmup_manager = NullWarmupManager()
 
   @staticmethod
   def ReadBuildConfig(args):
@@ -772,14 +846,14 @@ class Platform(object):
     else:
       return DesktopPlatform(args)
 
-  def _Run(self, runnable, count, secondary=False):
+  def _Run(self, runnable, count, secondary=False, post_process=True):
     raise NotImplementedError()  # pragma: no cover
 
-  def _LoggedRun(self, runnable, count, secondary=False):
+  def _LoggedRun(self, runnable, count, secondary=False, post_process=True):
     suffix = ' - secondary' if secondary else ''
     title = '>>> %%s (#%d)%s:' % ((count + 1), suffix)
     try:
-      output = self._Run(runnable, count, secondary)
+      output = self._Run(runnable, count, secondary, post_process)
     except OSError:
       logging.exception(title % 'OSError')
       raise
@@ -806,6 +880,10 @@ class Platform(object):
       A tuple with the two benchmark outputs. The latter will be NULL_OUTPUT if
       secondary is False.
     """
+    self.warmup_manager.maybe_warm_up(
+        runnable.name,
+        lambda: self._LoggedRun(
+            runnable, 0, secondary=False, post_process=False))
     output = self._LoggedRun(runnable, count, secondary=False)
     if secondary:
       return output, self._LoggedRun(runnable, count, secondary=True)
@@ -821,11 +899,11 @@ class DesktopPlatform(Platform):
     # Setup command class to OS specific version.
     command.setup(utils.GuessOS(), args.device)
 
-    if args.prioritize or args.affinitize != None:
+    if args.prioritize or args.affinitize is not None:
       self.command_prefix = ['schedtool']
       if args.prioritize:
         self.command_prefix += ['-n', '-20']
-      if args.affinitize != None:
+      if args.affinitize is not None:
         # schedtool expects a bit pattern when setting affinity, where each
         # bit set to '1' corresponds to a core where the process may run on.
         # First bit corresponds to CPU 0. Since the 'affinitize' parameter is
@@ -835,6 +913,9 @@ class DesktopPlatform(Platform):
         self.command_prefix += ['-a', ('0x%x' % core)]
       self.command_prefix += ['-e']
 
+    if args.checked_warmup:
+      self.warmup_manager = CachedWarmupManager()
+
   def PreExecution(self):
     pass
 
@@ -845,18 +926,20 @@ class DesktopPlatform(Platform):
     if isinstance(node, RunnableConfig):
       node.ChangeCWD(path)
 
-  def _Run(self, runnable, count, secondary=False):
+  def _Run(self, runnable, count, secondary=False, post_process=True):
     shell_dir = self.shell_dir_secondary if secondary else self.shell_dir
     cmd = runnable.GetCommand(self.command_prefix, shell_dir, self.extra_flags)
     logging.debug('Running command: %s' % cmd)
     output = Output() if self.is_dry_run else cmd.execute()
 
-    if output.IsSuccess() and '--prof' in self.extra_flags:
+    if (not self.is_dry_run and
+        post_process and
+        output.IsSuccess() and
+        '--prof' in self.extra_flags):
       os_prefix = {'linux': 'linux', 'macos': 'mac'}.get(utils.GuessOS())
       if os_prefix:
-        if not self.is_dry_run:
-          tick_tools = os.path.join(TOOLS_BASE, '%s-tick-processor' % os_prefix)
-          subprocess.check_call(tick_tools + ' --only-summary', shell=True)
+        tick_tools = os.path.join(TOOLS_BASE, '%s-tick-processor' % os_prefix)
+        subprocess.check_call(tick_tools + ' --only-summary', shell=True)
       else:  # pragma: no cover
         logging.warning(
             'Profiler option currently supported on Linux and Mac OS.')
@@ -899,9 +982,12 @@ class AndroidPlatform(Platform):  # pragma: no cover
     if isinstance(node, RunnableConfig):
       self.driver.push_file(bench_abs, node.main, bench_rel)
     for resource in node.resources:
-      self.driver.push_file(bench_abs, resource, bench_rel)
+      if resource == '*':
+        self.driver.push_files_rec(bench_abs, bench_rel)
+      else:
+        self.driver.push_file(bench_abs, resource, bench_rel)
 
-  def _Run(self, runnable, count, secondary=False):
+  def _Run(self, runnable, count, secondary=False, post_process=True):
     target_dir = 'bin_secondary' if secondary else 'bin'
     self.driver.drop_ram_caches()
 
@@ -920,7 +1006,7 @@ class AndroidPlatform(Platform):  # pragma: no cover
       logging.debug('Dumping logcat into %s', logcat_file)
 
     output = Output()
-    start = time.time()
+    output.start_time = time.time()
     try:
       if not self.is_dry_run:
         output.stdout = self.driver.run(
@@ -939,7 +1025,7 @@ class AndroidPlatform(Platform):  # pragma: no cover
       output.timed_out = True
     if runnable.process_size:
       output.stdout += 'MaxMemory: Unsupported'
-    output.duration = time.time() - start
+    output.end_time = time.time()
     return output
 
 
@@ -954,15 +1040,15 @@ class CustomMachineConfiguration:
     if self.disable_aslr:
       self.aslr_backup = CustomMachineConfiguration.GetASLR()
       CustomMachineConfiguration.SetASLR(0)
-    if self.governor != None:
+    if self.governor is not None:
       self.governor_backup = CustomMachineConfiguration.GetCPUGovernor()
       CustomMachineConfiguration.SetCPUGovernor(self.governor)
     return self
 
   def __exit__(self, type, value, traceback):
-    if self.aslr_backup != None:
+    if self.aslr_backup is not None:
       CustomMachineConfiguration.SetASLR(self.aslr_backup)
-    if self.governor_backup != None:
+    if self.governor_backup is not None:
       CustomMachineConfiguration.SetCPUGovernor(self.governor_backup)
 
   @staticmethod
@@ -1018,7 +1104,7 @@ class CustomMachineConfiguration:
         with open(cpu_device, 'r') as f:
           # We assume the governors of all CPUs are set to the same value
           val = f.readline().strip()
-          if ret == None:
+          if ret is None:
             ret = val
           elif ret != val:
             raise Exception('CPU cores have differing governor settings')
@@ -1141,6 +1227,8 @@ def Main(argv):
       help='Do not run any actual tests.')
   parser.add_argument('-v', '--verbose', default=False, action='store_true',
                       help='Be verbose and print debug output.')
+  parser.add_argument('--checked-warmup', default=False, action='store_true',
+                      help='Warm up benchmarks not run since last reboot.')
   parser.add_argument('suite', nargs='+', help='Path to the suite config file.')
 
   try:
@@ -1167,12 +1255,12 @@ def Main(argv):
 
   workspace = os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))
 
-  if args.binary_override_path == None:
+  if args.binary_override_path is None:
     args.shell_dir = find_build_directory(
         os.path.join(workspace, args.outdir), args.arch)
     default_binary_name = 'd8'
   else:
-    path = pathlib.Path(args.binary_override_path).expanduser().resolve()
+    path = Path(args.binary_override_path).expanduser().resolve()
     if not path.is_file():
       logging.error(f'binary-override-path "{path}" must be a file name')
       return INFRA_FAILURE_RETCODE
@@ -1206,8 +1294,6 @@ def Main(argv):
   # directory.
   args.suite = list(map(os.path.abspath, args.suite))
 
-  prev_aslr = None
-  prev_cpu_gov = None
   platform = Platform.GetPlatform(args)
 
   result_tracker = ResultTracker()
